#!/bin/sh
#
TOP_DIR=`pwd`
if [ $1 ] && [ -d $1 ]; then
    RELEASE_DIR=`cd $1; pwd`
else
    echo "Use default directory."
    RELEASE_DIR=~/Desktop/graphics_repos/release
fi

SourceDir_DFB=$TOP_DIR/'DirectFB-1.4.3'

SourceDir_drivers=$TOP_DIR/drivers


############################################
echo "----------Building DFB singlecore---------"
#single core build
cd $SourceDir_DFB
./autogen.sh
./singlecore.sh.general
make -j2
if [ $? != 0 ]; then 
     echo "(###) failing building DFB libs"
     exit -1
fi
make -j2 install-strip
if [ $? != 0 ]; then
     echo "(###) failing building DFB libs"
     exit -1
fi

cd gfxdrivers
cd xcode4
make -j2
if [ $? != 0 ]; then 
     echo "(###) failing building gfxdrivers"
     exit -1
fi
make -j2 install-strip
if [ $? != 0 ]; then
     echo "(###) failing building gfxdrivers"
     exit -1
fi

#back up the DFB from last release
echo "-----------Backing up DirectFB folder---------------"
cd $RELEASE_DIR
if [ ! -e DirectFB_bk ]; then
    mkdir DirectFB_bk
fi
rm DirectFB_bk/* -r
cp DirectFB/* DirectFB_bk/ -r
cd DirectFB
rm single_core/* -r
rm multicore/* -r

#copy the libs to the release folder
cd $RELEASE_DIR/DirectFB/single_core
cp /usr/local/mips/include . -r
cp /usr/local/mips/lib . -r


#multicore build
echo "-----------Building DFB multicore--------"
cd $SourceDir_DFB
./multicore.sh.general
make -j2
if [ $? != 0 ]; then 
     echo "(###) failing building DFB libs"
     exit -1
fi
make -j2 install-strip
if [ $? != 0 ]; then
     echo "(###) failing building DFB libs"
     exit -1
fi

cd gfxdrivers
cd xcode4
make -j2
if [ $? != 0 ]; then 
     echo "(###) failing building gfxdrivers"
     exit -1
fi
make -j2 install-strip
if [ $? != 0 ]; then
     echo "(###) failing building gfxdrivers"
     exit -1
fi

cd $RELEASE_DIR/DirectFB/multicore
cp /usr/local/mips/include . -r
cp /usr/local/mips/lib . -r




################################################
echo "-----------Building the linux kernel----------"
cd $TOP_DIR/mips-linux-2.6.29
cp ./arch/mips/configs/xcode_defconfig .config
make



################################################
echo "-----------Building the vixsfb.ko----------"
cd $SourceDir_drivers/fb/xcode4_disp

make -f Makefile.general
if [ $? != 0 ]; then
     echo "(###) failing building vixsfb.ko"
     exit -1
fi

cp vixsfb.ko $RELEASE_DIR/DirectFB/VixsFB/



################################################
echo "-----------Building fusion.ko----------"
cd $SourceDir_drivers/linux_fusion_8_1_2

make -f Makefile.general
if [ $? != 0 ]; then
     echo "(###) failing building fusion.ko"
     exit -1
fi

cp fusion.ko $RELEASE_DIR/DirectFB/VixsFB/



###############################################
echo "-----------Building srm.ko-----------"
cd $SourceDir_drivers/srm

make -f Makefile.general
if [ $? != 0 ]; then
      echo "(###) failing building srm.ko"
      exit -1
fi

cp srm.ko $RELEASE_DIR/DirectFB/VixsFB/

################################################
echo "------------Backing up the G3D folder----------"
cd $RELEASE_DIR
if [ ! -e G3D_bk ]; then
     mkdir G3D_bk
fi
rm G3D_bk/* -r
cp G3D/* G3D_bk/ -r
cd G3D
rm sdk_shared/* -r

echo "------------Building the G3D drivers------------"
cd $TOP_DIR/xcode4
./driver_build_vdk.sh.general
if [ $? != 0 ]; then
     echo "(###) failing building G3D drivers"
     exit -1
fi
cp -r ./build/sdk/* $RELEASE_DIR/G3D/sdk_shared/
cp ./build/sdk/drivers/galcore.ko $RELEASE_DIR/DirectFB/VixsFB/

./driver_build_dfb.sh.general
if [ $? != 0 ]; then
     echo "(###) failing building G3D drivers"
     exit -1
fi
cp ./build/sdk/drivers/libEGL_DFB.so $RELEASE_DIR/G3D/sdk_shared/drivers/


cd $RELEASE_DIR
rm VixsFB/ -r
cp DirectFB/VixsFB . -r

tar -czvf DirectFB.tar.gz DirectFB/
tar -czvf G3D.tar.gz G3D/
