/*
 * Copyright (C) 2008 ViXS Systems Inc
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


/** include files **/
#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>
#include <core/core.h>


#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/system.h>



#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include <fbdev/fb.h>
#include <fbdev/fbdev.h>

#include "xcdfb.h"
#include "xcioctl.h"
#include "xcmmio.h"
 
/** public functions **/

int xc_ovl_datasize(void);

DFBResult   xc_ovl_init_layer( CoreLayer                   *layer,
                               void                        *driver_data,
                               void                        *layer_data,
                               DFBDisplayLayerDescription  *description,
                               DFBDisplayLayerConfig       *config,
                               DFBColorAdjustment          *adjustment );

DFBResult    xc_ovl_set_region( CoreLayer                  *layer,
                                void                       *driver_data,
                                void                       *layer_data,
                                void                       *region_data,
                                CoreLayerRegionConfig      *config,
                                CoreLayerRegionConfigFlags  updated,
                                CoreSurface                *surface,
                                CorePalette                *palette,
			        CoreSurfaceBufferLock      *lock );

DFBResult    xc_ovl_remove_region ( CoreLayer                  *layer,
                                    void                       *driver_data,
                                    void                       *layer_data,
                                    void                       *region_data );

DFBResult    xc_ovl_test_region   ( CoreLayer                  *layer,
                                    void                       *driver_data,
                                    void                       *layer_data,
                                    CoreLayerRegionConfig      *config,
                                    CoreLayerRegionConfigFlags *failed );

DFBResult    xc_ovl_flip_region   ( CoreLayer                  *layer,
                                    void                       *driver_data,
                                    void                       *layer_data,
                                    void                       *region_data,
                                    CoreSurface                *surface,
                                    DFBSurfaceFlipFlags         flags,
				    CoreSurfaceBufferLock      *lock  );
DFBResult xc_ovl_update_region ( CoreLayer                  *layer,
                                 void                       *driver_data,
                                 void                       *layer_data,
                                 void                       *region_data,
                                 CoreSurface                *surface,
                                 const DFBRegion            *update,
			         CoreSurfaceBufferLock	    *lock );

DFBResult xc_ovl_get_current_output( CoreLayer              *layer,
                                    void                   *driver_data,
                                    void                   *layer_data,
                                    int                    *field );



/** private functions **/
static __inline void submit_ovl_frame(pxc_dfb_overlay_data movl);
static void reset_ovl_data_structure(pxc_dfb_overlay_data movl);
static void idle_dve(volatile u8 *mmio);



/** public data **/
DisplayLayerFuncs xcOverlayFuncs = {
    LayerDataSize:        xc_ovl_datasize,
    InitLayer:            xc_ovl_init_layer,
    SetRegion:            xc_ovl_set_region,
    RemoveRegion:         xc_ovl_remove_region,
    TestRegion:           xc_ovl_test_region,
//    FlipRegion:           xc_ovl_flip_region,
    UpdateRegion:         xc_ovl_update_region,
    GetCurrentOutputField: xc_ovl_get_current_output
};


/** private data **/

/** external functions **/
extern int adjust_phys_offset(int offset);
extern int adjust_surface_phys_offset(int offset);

/** external data **/

/** local definitions **/

//#define ENABLE_DVE_TILED_MODE 


#define HANDLE_UNTILE_DSCL
#define XC_OVL_CAPS (DLCAPS_SURFACE)
#define MAX_QUEUE_OVL_FRAMES 5

#define OVL_INITIAL_WIDTH     64
#define OVL_INITIAL_HEIGHT    64
#define OVL_SUPPORTED_OPTIONS   (DLOP_ALPHACHANNEL)
  
/** default settings  **/
 

/**************************************************************************************************/


#ifdef    HANDLE_UNTILE_DSCL

/*
 *
 *Function Name:reset_ovl_software_fifo
 *
 *Parameters: 
 *             pSWFifo -> Pointer to fifo structure
 *             
 *
 *Description:
 *            Re-inits the fifo data structure
 *
 *Returns:
 *        NA
 *
*/
static void reset_ovl_software_fifo(psw_dve_fifo pSWFifo)
{
     memset(pSWFifo,0x0, sizeof(sw_dve_fifo));

     pSWFifo->uDSCLBufferIndex=1;
     pSWFifo->uDVEBufferIndex=1;

}


#endif //HANDLE_UNTILE_DSCL

/*
 *
 *Function Name:enable_repeat_last_frame
 *
 *Parameters: 
 *             mmio -> Pointer to registers
 *
 *Description:
 *            Reads DVE CTRL register and sets repeat mode
 *
 *Returns:
 *
*/
static u32 set_dve_repeat_mode(volatile u8 *mmio, u32 empty_fifo_mode)
{
     u32  tmp;

     D_DEBUG("enable_repeat_last_frame using %x\n", empty_fifo_mode);

     //
     //Enable repeat mode
     // 
     tmp = xc_in32(mmio,XC_DVE_CTRL);
     tmp &= ~XC_DVE_CTRL_EMPTY_FIFO_MODE_MASK;

     D_DEBUG("enable_repeat_last_frame tmp =  %x\n", tmp);

     tmp|= (empty_fifo_mode << XC_DVE_CTRL_EMPTY_FIFO_MODE_SHIFT);
     xc_out32(mmio,XC_DVE_CTRL,tmp);

     D_DEBUG("enable_repeat_last_frame tmp =  %x\n", tmp);


}


/*
 *
 *Function Name:read_dve_frm_cnt
 *
 *Parameters: 
 *             mmio -> Pointer to registers
 *
 *Description:
 *            Reads DVE_DUMMY_REG to query frame count
 *
 *Returns:
 *             Frame count
 *
*/
static u32 read_dve_frm_cnt(volatile u8 *mmio)
{
     u32 frame_count;

     D_DEBUG(" %s enter:\n",__FUNCTION__);

     frame_count = xc_in32(mmio,XC_DVE_DUMMY_REG);

     D_DEBUG(" %s exit:\n",__FUNCTION__);

     return frame_count;

}

/*
 *
 *Function Name:idle_dve
 *
 *Parameters: 
 *             mmio -> Pointer to registers of device to idel
 *
 *Description:
 *            Idle DVE
 *
 *Returns:
 *
*/
static void idle_dve(volatile u8 *mmio)
{
     u32 tmp;

     D_DEBUG(" %s enter:\n",__FUNCTION__);

#ifdef USE_ISR_COUNT
     //
     //Wait for beginning of new frame before updating
     //
     tmp = xc_in32(mmio,XC_DVE_DUMMY_REG);
     

     //
     //Read frame count, then wait until at least 16 frames 
     // 
     while (xc_in32(mmio,XC_DVE_DUMMY_REG) <= tmp+0x10);
#else
     while( (xc_in32(mmio,XC_DVE_FIFO_STATUS) & XC_DVE_FIFO_STATUS_FREE_ENTRIES_MASK)  !=0x10);
#endif

     D_DEBUG(" %s exit:\n",__FUNCTION__);

}


/*
 *
 *Function Name:submit_ovl_frame
 *
 *Parameters: 
 *             movl -> Pointer to overlay data
 *
 *Description:
 *            Adds a frame to the SW overlay FIFO
 *
 *Returns:
 *
*/
static __inline void submit_ovl_frame(pxc_dfb_overlay_data movl)
{
     movl->ufifo_submitted_frames++;
}

/*
 *
 *Function Name:reset_ovl_data_structure
 *
 *Parameters: 
 *             movl -> Pointer to overlay data
 *             
 *
 *Description:
 *            Re-inits the overlay data structure
 *
 *Returns:
 *        NA
 *
*/
static void reset_ovl_data_structure(pxc_dfb_overlay_data movl)
{
     FBDev *dfb_fbdev = dfb_system_data();
     app_ioctl_command dfb_ioctl_cmd;

     memset(movl, 0 , sizeof(xc_dfb_overlay_data));

     movl->size = sizeof(xc_dfb_overlay_data);

     //
     //Retrieve tiling buffer locations from FB 
     // 
     dfb_ioctl_cmd.uSize = sizeof(app_ioctl_command);
     dfb_ioctl_cmd.uIOCTLCommand = CMD_DFB_IOCTL_GET_TILED_OFFSETS;

     if (ioctl( dfb_fbdev->fd,CMD_APPLICATION_IOCTL , &dfb_ioctl_cmd ) )
     {
          D_ERROR("failed to retrieve tiled offsets\n");
     }
     else
     {
          movl->uOverlayTiledBase0 =  dfb_ioctl_cmd.dfb_tiled_offsets.uTiledOffset0;
          movl->uOverlayTiledBase1 =  dfb_ioctl_cmd.dfb_tiled_offsets.uTiledOffset1; 
     }


}


/*
 *
 *Function Name:xc_ovl_datasize
 *
 *Parameters: NA
 *
 *Description:
 *             Used to query the size of xcode shared data streucture
 *
 *Returns:
 *             Returns the size of the vixs overlay structure
 *
*/
int xc_ovl_datasize(void)
{

     D_DEBUG(" %s enter:\n",__FUNCTION__);

     D_DEBUG( "size of xc_dfb_overlay_data is %d bytes\n", sizeof(xc_dfb_overlay_data) );

     D_DEBUG(" %s exit:\n",__FUNCTION__);

     return sizeof(xc_dfb_overlay_data);


}

/*
 *
 *Function Name:xc_ovl_init_layer
 *
 *Parameters: 
 *
 *Description:
 *
 *Returns:
 *
*/
DFBResult   xc_ovl_init_layer( CoreLayer                   *layer,
                               void                        *driver_data,
                               void                        *layer_data,
                               DFBDisplayLayerDescription  *description,
                               DFBDisplayLayerConfig       *config,
                               DFBColorAdjustment          *adjustment )
{
     D_DEBUG(" %s enter:\n",__FUNCTION__);

     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_overlay_data movl = (pxc_dfb_overlay_data) layer_data;


     //
     // Save a pointer to the layer data for access by the primary
     // This is needed to properly support levels and the primary alpha channel
     //
     mdrv->poverlay_data = movl;


     //
     // Set layer type, capabilities and name
     //

     description->caps = XC_OVL_CAPS;
     description->type = DLTF_VIDEO;

     snprintf(description->name,
         DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, XC_DFB_LAYER_OVERLAY_NAME);

     adjustment->flags = DCAF_NONE;

     //
     // Fill out the default configuration
     //

     config->flags  = DLCONF_WIDTH | DLCONF_HEIGHT |
                      DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS;

     config->width  = OVL_INITIAL_WIDTH; 
     config->height = OVL_INITIAL_HEIGHT; 
                      
     config->pixelformat = DSPF_I420;
     config->buffermode  = DLBM_FRONTONLY;
     config->options     = DLOP_NONE;
     
     D_DEBUG(" %s exit:\n",__FUNCTION__);

     return DFB_OK;
}

/*
 *
 *Function Name:xc_ovl_update_region
 *
 *Parameters: 
 *            
 *            
 *
 *Description:
 *
 *             Updates a region of the overlya surface. If the surface type is SEPARATED     
 *             then two calls are required to perfrom 1 update. If the surface is INTERLACED,
 *             then only one call is required.                                               
 *
 *Returns:
 *        DFB_BUSY if engine can't take command or DFB_OK   
 *
*/

DFBResult xc_ovl_update_region ( CoreLayer                  *layer,
                                 void                       *driver_data,
                                 void                       *layer_data,
                                 void                       *region_data,
                                 CoreSurface                *surface,
                                 const DFBRegion            *update,
			         CoreSurfaceBufferLock	    *lock  )
{
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_overlay_data movl = (pxc_dfb_overlay_data) layer_data;
     volatile u8 *mmio     = ((pxc_dfb_driver_data) driver_data)->mmio_base;
     int  ovl_y_res = 0,ovl_dve_index =0,ovl_dve_bot_index =0, dve_cmd_fifo_0=0, dve_cmd_fifo_entries_avail=0;
     int tiledVideoFrame=0;
     u32 uTileYOffset,uTileCOffset,index; 
     DFBResult dfb_result = DFB_BUSY;



     D_DEBUG("%s enter:\n",__FUNCTION__);

     D_DEBUG( "Region data = %x %x %x %x \n",update->x1,update->y1,update->x2,update->y2);


     //
     //Skip initial set(called by DirectFB
     // 
     if ( (update->y2 > update->y1) || (update->x2 == OVL_INITIAL_WIDTH -1))
     {
          if ((update->y2 - update->y1) > (YUV_1088_HEIGHT*2 ) ||
              (update->x2 == OVL_INITIAL_WIDTH -1 ) )
          {
               D_DEBUG( "update region called by DirectFB not application \n");
               return DFB_OK;
          }
     }

     //
     //Ensure there is room iside the FIFO
     // 
     dve_cmd_fifo_entries_avail = ( (xc_in32(mmio,XC_DVE_FIFO_STATUS) & XC_DVE_FIFO_STATUS_FREE_ENTRIES_MASK) );


     if ( (movl->ufifo_submitted_frames + MAX_QUEUE_OVL_FRAMES) <= 16) 
     {
          if ( (movl->uOverlaySurfaceFlags & DSCAPS_TILED8x8) || 
               (movl->uOverlaySurfaceFlags & DSCAPS_TILED16x16)
              )
          {
               tiledVideoFrame=1;
               D_DEBUG( "SURFACE IS TILED!!!!\n");

          }


          ovl_y_res = movl->uOverlayYPlaneHeight;

          if (update->y1 >= ovl_y_res)
          {
               ovl_dve_index = update->y1 / ovl_y_res;
          }
          //
          //To support 3:2 pulldown and make the interface more flexible 
          //the current algorithm needs two calls per frame, one indicating the 
          // top field index and the other the bottom field index. However, when
          // we have a progressive output mode, only one call is needed.
          // 
          movl->uovl_field_index[movl->ufield_update] = ovl_dve_index;
          movl->ufield_update ^=1;

          D_DEBUG(" ovl_dve_index %x \n",movl->uovl_field_index[0]);
          D_DEBUG(" ovl_dve_bot_index %x \n",movl->uovl_field_index[1]);

          //
          //Only write the DVE every two field updates(for interlaced display)
          //
          if (!movl->ufield_update)
          {
               if (tiledVideoFrame)
               {

                    //
                    //Get Tile Base and then Add index*size to it
                    // 
                    uTileYOffset = ((psw_dve_fifo) mdrv->shared->pSWFifoAddress)->uTiledOriginalYOffset; 
                    uTileYOffset+= ( movl->uovl_field_index[0] * (surface->config.size.w * ovl_y_res));

                    uTileCOffset = ((psw_dve_fifo) mdrv->shared->pSWFifoAddress)->uTiledOriginalCOffset; 
                    uTileCOffset+= ( movl->uovl_field_index[0] * ((surface->config.size.h * ovl_y_res) /2) );

                    index = ((psw_dve_fifo) mdrv->shared->pSWFifoAddress)->uWrite;

                    SET_FIFO_ELEMENT( ((psw_dve_fifo) mdrv->shared->pSWFifoAddress), 
                                      index, uTileYOffset,uTileCOffset);       

                    ADVANCE_FIFO_WRITE_PTR(((psw_dve_fifo) mdrv->shared->pSWFifoAddress));

                    //
                    //Create 2-comp for mask
                    // 
                    submit_ovl_frame(movl);
                    
                    //
                    //Only cahce frame count when it has been cleared
                    // NOTE: Gets cleared inside get_current_output)
                    // 
                    if (!movl->udve_hw_frm_cnt)
                    {
                         movl->udve_hw_frm_cnt = xc_in32(mmio, XC_DVE_DUMMY_REG);
                    }
               }
               else
               {

                    D_DEBUG(" ovl_dve_index %x \n",movl->uovl_field_index[0]);
                    D_DEBUG(" ovl_dve_bot_index %x \n",movl->uovl_field_index[1]);


                    ovl_dve_index =  movl->uovl_field_index[0]; //top index
                    ovl_dve_bot_index =  movl->uovl_field_index[1]; //bottom index

                    dve_cmd_fifo_0 = xc_in32(mmio,XC_DVE_CMD_FIFO);
               
                    //
                    //Mask off index
                    // 
                    dve_cmd_fifo_0&=~(XC_DVE_CMD_FIFO_TOP_IDX_MASK | XC_DVE_CMD_FIFO_BOT_IDX_MASK);
               
                    //
                    //Add new index
                    // 
                    dve_cmd_fifo_0|=( (ovl_dve_index) | ( (ovl_dve_bot_index ) << XC_DVE_CMD_FIFO_BOT_IDX_SHIFT));
               
                    dve_cmd_fifo_0&=~XC_DVE_CMD_FIFO_UPDATE_MASK;     
                    //
                    //Write cmd fifo's
                    // 
                    xc_out32(mmio,XC_DVE1_CMD_FIFO, 
                             (xc_in32(mmio,XC_DVE1_CMD_FIFO) & ~XC_DVE_CMD_FIFO_UPDATE_MASK) );
               
                    xc_out32(mmio,XC_DVE_CMD_FIFO, dve_cmd_fifo_0);
               
                    //
                    //Create 2-comp for mask
                    // 
                    submit_ovl_frame(movl);
               
                    //
                    //Only cahce frame count when it has been cleared
                    // NOTE: Gets cleared inside get_current_output)
                    // 
                    if (!movl->udve_hw_frm_cnt)
                    {
                         movl->udve_hw_frm_cnt = xc_in32(mmio, XC_DVE_DUMMY_REG);
                    }
               }
               
          }
          dfb_result = DFB_OK;
     
     }

     D_DEBUG(" %s exit:\n",__FUNCTION__);

     return dfb_result;

}

/*
 *
 *Function Name:xc_ovl_set_region
 *
 *Parameters: 
 *
 *Description:
 *
 *Returns:
 *
*/
DFBResult    xc_ovl_set_region( CoreLayer                  *layer,
                                void                       *driver_data,
                                void                       *layer_data,
                                void                       *region_data,
                                CoreLayerRegionConfig      *config,
                                CoreLayerRegionConfigFlags  updated,
                                CoreSurface                *surface,
                                CorePalette                *palette,
			        CoreSurfaceBufferLock      *lock )
{
     pxc_dfb_overlay_data movl = (pxc_dfb_overlay_data) layer_data;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     volatile u8 *mmio     = ((pxc_dfb_driver_data) driver_data)->mmio_base;
     u32 y_offset=0, v_offset = 0, u_offset = 0; 
     u32 ovl_height=0, ovl_width=0, dve_and_mask=0xffffffff, dve_or_mask=XC_DVE_CMD_FIFO_UPDATE_MASK;
     u32 y_plane_size;
     u32  tile_mode,tmp,enable_prog=0;
     u32  tiled_y_size,tiled_uv_size;


     D_DEBUG(" %s enter:\n",__FUNCTION__);

     D_DEBUG("xc_ovl_set_region flags = %x \n",updated );

     D_DEBUG("xc_ovl_set_region surafce caps  = %x \n",surface->config.caps );

     D_DEBUG("xc_ovl_set_region source x1=%x y1=%x \n",config->source.x,config->source.y  );

     D_DEBUG("xc_ovl_set_region surface height = %x \n", surface->config.size.h);


     //
     //Wait for IDLE on display
     // 
     idle_dve(mmio);


     ovl_height = surface->config.size.h;
     ovl_width  = surface->config.size.w;


     //
     // Set DVE YUV buffer pointers 
     // 
     //y_offset = adjust_phys_offset(front_buffer->video.offset);
     y_offset   = lock->phys;
      
     switch (surface->config.format) 
     {
          case DSPF_YV12:
          case DSPF_I420:

               D_DEBUG("xc_ovl_set_region format = DSP_I420/YV12 and pitch is %x \n", lock->pitch);

               y_plane_size = movl->uOverlayYPlaneSize;

               y_plane_size/=256;
               u_offset = y_offset + ovl_height * lock->pitch;
               v_offset = u_offset + ((ovl_height >> 1) * (lock->pitch >> 1));

               break;
     default:
               return DFB_UNSUPPORTED;

     }
     D_DEBUG("Done setting DVE YUV pointers \n");
     //
     //Determine if we have a progressive output
     //if yes, do not change the FIELD select(this code needs to be fixed)
     // 
     if ( (xc_in32(mmio,XC_DVE_CMD_FIFO) & XC_DVE_CMD_FIFO_FLD_SEL_MASK) != 0x40)
     {
          //
          //Configure VIDEO interlacing
          // 
          if (surface->config.caps & DSCAPS_SEPARATED)
          {
               D_DEBUG("xc_ovl_set_region caps is separated \n");
     
               //
               //Set DVE control upto handle data stored in field basis. 
               // 
               dve_and_mask = ~XC_DVE_CMD_FIFO_FLD_SEL_MASK;
     
               //
               //Set size of frame for separated memory layout
               // 
               y_plane_size/=2; 
          }
          else
          {
               D_DEBUG("xc_ovl_set_region caps is progressive memory layout (DFB) def. \n");
               dve_or_mask |= 2 << XC_DVE_CMD_FIFO_FLD_SEL_SHIFT;  
          }
     }
     else
     {
          dve_or_mask |= 1 << XC_DVE_CMD_FIFO_FLD_SEL_SHIFT;  

     }

     D_DEBUG("y = %x, u = %x, v=%x planar=%x \n", y_offset, u_offset, v_offset, y_plane_size);
     
     D_DEBUG("Setting DVE memory mode (tiled/linear). \n");

     //
     //Configure TILED mode
     //
     if ( (movl->uOverlaySurfaceFlags & DSCAPS_TILED8x8) || 
          (movl->uOverlaySurfaceFlags & DSCAPS_TILED16x16)
          )
     {
#ifdef ENABLE_DVE_TILED_MODE
          if (movl->uOverlaySurfaceFlags & DSCAPS_TILED8x8)          
          {
               D_DEBUG("Tiled mode is 8x8 \n");
               tile_mode=DVE_MEM_FORMAT_TILED_8x8;
          }
          else
          {
               D_DEBUG("Tiled mode is 16x16 \n");
               tile_mode=DVE_MEM_FORMAT_TILED_16x16;
          }

          tmp = xc_in32(mmio,XC_DVE_CTRL);
          tmp &= ~XC_DVE_CTRL_MEM_FORMAT_MASK;
          tmp |=tile_mode;
          xc_out32(mmio,XC_DVE_CTRL,tmp);


          //
          //The TILED BUG registre requires the size of the Y,U and V planes
          //in terms of 8192 bytes. Since the Y plane size is already in multiples of 256
          //we just have to further divide by 32
          // 
          tiled_y_size = y_plane_size / 32;
          tiled_uv_size = tiled_y_size / 2;


          xc_out32(mmio,XC_DVE_TILED_BUF, ( surface->config.size.w | 
                                            (tiled_y_size << XC_DVE_TILED_BUF_SIZE_Y_SHIFT) |
                                            (tiled_uv_size << XC_DVE_TILED_BUF_SIZE_C_SHIFT) 
                                           ) );


          //
          //TILED_SAMPLES is the same as ACTIVE_WIDTH
          // 
          xc_out32(mmio,XC_DVE_TILED_SAMPLES, (xc_in32(mmio,XC_DVE_PLANAR_SAMPLES)) );
          xc_out32(mmio,XC_DVE_C_ADR, u_offset);
#else

          //
          //Store OLD offsets
          // 
          ((psw_dve_fifo) mdrv->shared->pSWFifoAddress)->uSize = sizeof(sw_dve_fifo);
          ((psw_dve_fifo) mdrv->shared->pSWFifoAddress)->uTiledOriginalYOffset = y_offset;
          ((psw_dve_fifo) mdrv->shared->pSWFifoAddress)->uTiledOriginalCOffset = u_offset;

          //
          //Set offsets to point to buffer 0 of new ping-pong architecture
          // 
          y_offset = movl->uOverlayTiledBase1;  
          u_offset = y_offset + YUV_2048x1088_PLANE_SIZE;
          v_offset = u_offset + ((YUV_1088_HEIGHT >> 1) * (YUV_2048_WIDTH >> 1));


          //
          //Configure linear mode, should be default
          // 
          tmp = xc_in32(mmio,XC_DVE_CTRL);
          tmp &= ~XC_DVE_CTRL_MEM_FORMAT_MASK;
          tmp |=DVE_MEM_FORMAT_LINEAR;
          xc_out32(mmio,XC_DVE_CTRL,tmp);

          xc_out32(mmio,XC_DVE_PLANAR_BUF, ( (y_plane_size << XC_DVE_PLANAR_BUF_SIZE_SHIFT) | YUV_2048_WIDTH) );
          
#endif // ENABLE_DVE_TILED_MODE

     }
     else
     {
          //
          //Configure linear mode, should be default
          // 
          tmp = xc_in32(mmio,XC_DVE_CTRL);
          tmp &= ~XC_DVE_CTRL_MEM_FORMAT_MASK;
          tmp |=DVE_MEM_FORMAT_LINEAR;
          xc_out32(mmio,XC_DVE_CTRL,tmp);

          xc_out32(mmio,XC_DVE_PLANAR_BUF, ( (y_plane_size << XC_DVE_PLANAR_BUF_SIZE_SHIFT) | surface->config.size.w) );
     }

     xc_out32(mmio,XC_DVE_Y_ADR, y_offset);
     xc_out32(mmio,XC_DVE_CB_ADR, u_offset);
     xc_out32(mmio,XC_DVE_CR_ADR, v_offset);

     //
     //Configure empty fifo mode
     // 
     D_DEBUG("Configure empty fifo mode.\n");
     if (surface->config.size.h == OVL_INITIAL_HEIGHT)
     {
          set_dve_repeat_mode(mmio,2);

     }
     else
     {
          set_dve_repeat_mode(mmio,0);
     }
     

     //
     //Write cmd fifo's
     //
     D_DEBUG("Write cmd fifo's\n"); 
     xc_out32(mmio,XC_DVE1_CMD_FIFO, 
              (xc_in32(mmio,XC_DVE1_CMD_FIFO) | (dve_or_mask)) );
     
     
     xc_out32(mmio,XC_DVE_CMD_FIFO, 
              ((xc_in32(mmio,XC_DVE_CMD_FIFO) | dve_or_mask) ) &
                dve_and_mask );
     
     
     //
     //Reset data structure.
     // WARNING!!! Will wipe out saved flags, cant do this here
     // 
     //reset_ovl_data_structure(movl);


     //
     //Wait for IDLE on display
     // 
     idle_dve(mmio);
      
     D_DEBUG(" %s exit:\n",__FUNCTION__);

     return DFB_OK;

}

/*
 *
 *Function Name:xc_ovl_remove_region
 *
 *Parameters: 
 *
 *Description:
 *
 *Returns:
 *
*/
DFBResult    xc_ovl_remove_region ( CoreLayer                  *layer,
                                    void                       *driver_data,
                                    void                       *layer_data,
                                    void                       *region_data )
{
     volatile u8 *mmio     = ((pxc_dfb_driver_data) driver_data)->mmio_base;

     D_DEBUG(" %s enter:\n",__FUNCTION__);

     set_dve_repeat_mode(mmio, 2);

     D_DEBUG(" %s exit:\n",__FUNCTION__);

     return DFB_OK;

}

/*
 *
 *Function Name:xc_ovl_test_region
 *
 *Parameters: 
 *
 *Description:
 *
 *Returns:
 *
*/
DFBResult    xc_ovl_test_region   ( CoreLayer                  *layer,
                                    void                       *driver_data,
                                    void                       *layer_data,
                                    CoreLayerRegionConfig      *config,
                                    CoreLayerRegionConfigFlags *failed )
{
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_overlay_data movl = (pxc_dfb_overlay_data) layer_data;
     FBDev *dfb_fbdev = dfb_system_data();
     app_ioctl_command dfb_ioctl_cmd;
     CoreLayerRegionConfigFlags fail = 0;
     DFBResult retVal=DFB_OK;
     int  i;

     D_DEBUG(" %s enter:\n",__FUNCTION__);

     D_DEBUG("region buffer mode = %x \n", config->buffermode );

     D_DEBUG("region width height = %d %d \n", config->width, config->height );

     D_DEBUG("surface caps =%x \n", config->surface_caps );

     //
     //Check for default ovl setup, if yes, just return
     // 
     if ((config->width != OVL_INITIAL_WIDTH) &&
         (config->height != OVL_INITIAL_HEIGHT)
        )
     {
     
          //
          // Check unsupported options 
          // 
          if (config->options & ~OVL_SUPPORTED_OPTIONS )
          {
               fail |= CLRCF_OPTIONS;
	       D_DEBUG("Test Region Fail: Overlay Options\n");
	  }
          else
          {
               
               //
               //Check video mode to ensure that Alpha is enabled and overlay is correct size
               // 
               if (!dfb_fbdev->shared->current_var.transp.length)
               {
                    fail |= CLRCF_DEST; //video mode is not ARGB
		    D_DEBUG("Test Region Fail: Destination (overlay) \n"); 
               }
               else
               {
               
                    //
                    //Check format
                    // 
                    if ( (config->format != DSPF_I420) && (config->format != DSPF_YV12) )
                    {
                         fail |= CLRCF_FORMAT;
			 D_DEBUG("Test Region Fail: Pixel Format (overlay) \n");
                    }
                    else
                    {
                         //
                         //Ensure width matches current video mode
                         // 
                         if (
                             (dfb_fbdev->shared->current_mode.xres == 720 && config->width != 768) &&
                             (dfb_fbdev->shared->current_mode.xres == 1280 && config->width != 1280) && 
                             (dfb_fbdev->shared->current_mode.xres == 1920 && config->width != 2048)  &&
                             (config->width != OVL_INITIAL_WIDTH)
     
                            )
                         {
                              fail |= CLRCF_WIDTH;
			      D_DEBUG("Test Region Fail: Mode and Overlay width mismatch (overlay) \n");
                         }
                         else
                         {
                              //
                              //Reset data structure
                              // 
                              reset_ovl_data_structure(movl);
                         
               #ifdef    HANDLE_UNTILE_DSCL
                              reset_ovl_software_fifo(((psw_dve_fifo) mdrv->shared->pSWFifoAddress));
               #endif
                         
                         
     
     
                              movl->uOverlaySurfaceFlags = config->surface_caps;
     
                              //
                              //Configure overlay parameters
                              // 
                              if (config->width == 768)
                              {
                                   movl->uOverlayYPlaneHeight = 768;
                                   movl->uOverlayYPlaneSize = 768*768;
                              }
                              else if (config->width == 1280)
                              {
                                   movl->uOverlayYPlaneHeight = 768;
                                   movl->uOverlayYPlaneSize = 1280*768;
                              }
                              else
                              {
     
     
                                   if ( (movl->uOverlaySurfaceFlags & DSCAPS_TILED8x8) || 
                                       (movl->uOverlaySurfaceFlags & DSCAPS_TILED16x16)
                                      )
                                  {
                                        movl->uOverlayYPlaneHeight = 1152;
                                  }
                                  else
                                  {
                                       movl->uOverlayYPlaneHeight = 1088;
     
                                  }
     
                                  movl->uOverlayYPlaneSize = 2048*1088;
     
                              }
     
                              if (movl->uOverlaySurfaceFlags & DSCAPS_SEPARATED)
                              {
                                   movl->uOverlayYPlaneHeight/=2;
                              }
     
                              //
                              //Inform FB driver of whether or not we have a tiled mode
                              // 
                              dfb_ioctl_cmd.uSize = sizeof(app_ioctl_command);
                              dfb_ioctl_cmd.uIOCTLCommand = CMD_DFB_IOCTL_SET_TILED_DISPLAY_MODE;
                              dfb_ioctl_cmd.dfb_tiled_display_mode.uTiledMode = (config->surface_caps & (DSCAPS_TILED8x8 | DSCAPS_TILED16x16));
                         
                         
                              if (ioctl( dfb_fbdev->fd,CMD_APPLICATION_IOCTL , &dfb_ioctl_cmd ) )
                              {
                                   D_ERROR("failed to inform FB driver of tiled mode\n");
                              }
                         
                         }
                    }
               }
          }
     }

     D_DEBUG(" %s exit:\n",__FUNCTION__);

     //
     // propogate the failing fields 
     // 
     //Isaac: The following code is being changed since it
     //looks to be the part responsible for segment faulting
     //on the XCodeIII
     //if (fail)
     //{
     //     *failed = fail;
     //     retVal = DFB_UNSUPPORTED;
     //}
     if (failed)    
	  *failed   =	 fail;
     
     if (fail)
	  retVal    =	 DFB_UNSUPPORTED;
     
     return retVal;
}

/*
 *
 *Function Name:xc_ovl_flip_region
 *
 *Parameters: 
 *
 *Description:
 *
 *Returns:
 *
*/
DFBResult    xc_ovl_flip_region   ( CoreLayer                  *layer,
                                    void                       *driver_data,
                                    void                       *layer_data,
                                    void                       *region_data,
                                    CoreSurface                *surface,
                                    DFBSurfaceFlipFlags         flags,
				    CoreSurfaceBufferLock      *lock   )
{
     D_DEBUG(" %s enter:\n",__FUNCTION__);


     dfb_surface_flip( surface, false );

     D_DEBUG(" %s exit:\n",__FUNCTION__);

     return DFB_OK;

}

/*
 *
 *Function Name:xc_ovl_get_current_output
 *
 *Parameters: 
 *
 *Description:
 *
 *Returns:
 *
*/
DFBResult xc_ovl_get_current_output( CoreLayer              *layer,
                                    void                   *driver_data,
                                    void                   *layer_data,
                                    int                    *field )
{
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_overlay_data movl = (pxc_dfb_overlay_data) layer_data;
     volatile u8 *mmio     = ((pxc_dfb_driver_data) driver_data)->mmio_base;
     u32 current_dve_frm_cnt=0;
     u32 frames_displayed=0;
     u32 frames_to_free=0;
     u32 current_fifo_level=0;

     D_DEBUG(" %s enter:\n",__FUNCTION__);

     current_dve_frm_cnt = xc_in32(mmio, XC_DVE_DUMMY_REG);
     D_DEBUG("xc_ovl_get_current_output frames_displayed = %x \n",current_dve_frm_cnt );

     //
     //Calculate how many frames displayed sice last submit
     // 
     frames_displayed  = current_dve_frm_cnt - movl->udve_hw_frm_cnt;
     D_DEBUG("xc_ovl dve frames displayed = %x \n",frames_displayed );


#ifdef HANDLE_UNTILE_DSCL
     if ( (movl->uOverlaySurfaceFlags & DSCAPS_TILED8x8) || 
          (movl->uOverlaySurfaceFlags & DSCAPS_TILED16x16)
         )
     {
          current_fifo_level = GET_FIFO_FREE_ENTRIES(((psw_dve_fifo) mdrv->shared->pSWFifoAddress));
     }
     else
     {
#endif //HANDLE_UNTILE_DSCL

          current_fifo_level = xc_in32(mmio, XC_DVE_FIFO_STATUS);
          current_fifo_level&=XC_DVE_FIFO_STATUS_FREE_ENTRIES_MASK;
#ifdef HANDLE_UNTILE_DSCL
     }
#endif //HANDLE_UNTILE_DSCL 

     D_DEBUG("current fifo level = %x \n", current_fifo_level);

     current_fifo_level = 16 - current_fifo_level;
     D_DEBUG("current fifo level after mod= %x \n", current_fifo_level);

     D_DEBUG("current submitted frames %x \n", movl->ufifo_submitted_frames);

     frames_to_free = movl->ufifo_submitted_frames - current_fifo_level;

     D_DEBUG("frames to free = %x \n", frames_to_free);

     if (frames_to_free > 1)
     {
          //
          //Hold one frame
          // 
          frames_to_free--;
     }
     else
     {
          frames_to_free=0;
     }

     movl->ufifo_submitted_frames-=frames_to_free;


     *field = frames_to_free | (current_fifo_level << 24) | (movl->ufifo_submitted_frames << 16) ;
     D_DEBUG("field = %x \n", *field);

     //
     //Clear cached frame count
     // 
     movl->udve_hw_frm_cnt=current_dve_frm_cnt;

     D_DEBUG(" %s exit:\n",__FUNCTION__);

     return DFB_OK;

}







