/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xc4_vir_gra0.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/


#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>


#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"
#include "xcioctl.h"

extern int XcodeVidLayerDataSize();

extern void Virtual_Layer_Combine_To_Physical_Hd_Gra0(pxc_dfb_driver_data    mdrv,
                                               void                  *layer_data,
                                               CoreSurfaceBufferLock *lock,
                                               int layer_id,
                                               DFBSurfaceFlipFlags flags);  

/**********************************************************************************************************************/

static DFBResult
Xc4BgGraInitLayer( CoreLayer                *layer,
              void                       *driver_data,
              void                       *layer_data,
              DFBDisplayLayerDescription *description,
              DFBDisplayLayerConfig      *config,
              DFBColorAdjustment         *adjustment )
{
     u32     tmp;
     D_ASSERT( layer_data != NULL );
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     D_DEBUG( "xcode DFB driver Xc4BgGraInitLayer enter:\n" );

     /* set capabilities and type */
     description->caps = DLCAPS_SURFACE | DLCAPS_ALPHACHANNEL | DLCAPS_OPACITY | 
                         DLCAPS_SCREEN_POSITION | DLCAPS_SCREEN_LOCATION | DLCAPS_LEVELS;

     description->type = DLTF_GRAPHICS;

     /* set name */
     snprintf( description->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "ViXS Xcode4 Background Graphics Plane " );

     /* fill out the default configuration */
     config->flags       = DLCONF_WIDTH       | DLCONF_HEIGHT |
                           DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS | DLCONF_SURFACE_CAPS ;
     
     if (dfb_config->preallocate_graphic_layer & (0x1 << BACKGROUND_GRA_LAYER_ID))
     {
         config->width       = MAX_GRAPHIC_BUFFER_WIDTH;
         config->height      = MAX_GRAPHIC_BUFFER_HEIGHT;
     }
     else
     {
         config->width       = XCODE_DEFAULT_WIDTH;
         config->height      = XCODE_DEFAULT_HEIGHT;
     }
     
     config->pixelformat = DSPF_RGB16;
     config->buffermode  = DLBM_FRONTONLY;
     config->surface_caps |= DSCAPS_STATIC_ALLOC ;

     m_layer_data->flip_height = 0;
     m_layer_data->alpha       = 0;
     D_DEBUG( "xcode DFB driver Xc4BgGraInitLayer exit:\n" );

     return DFB_OK;
}

static DFBResult
Xc4BgGraTestRegion( CoreLayer                  *layer,
               void                       *driver_data,
               void                       *layer_data,
               CoreLayerRegionConfig      *config,
               CoreLayerRegionConfigFlags *failed )
{
     CoreLayerRegionConfigFlags fail = 0;

     D_DEBUG( "xcode DFB driver Xc4BgGraTestRegion enter:\n" );

     if (config->options & ~XCODE4_OSD_SUPPORTED_OPTIONS)
          fail |= CLRCF_OPTIONS;

     switch (config->format) {
          case DSPF_I420:
          case DSPF_LUT8:
          case DSPF_RGB16:
          case DSPF_RGB32:
          case DSPF_ARGB1555:
          case DSPF_RGB555:
          case DSPF_ARGB:
               break;

          default:
               fail |= CLRCF_FORMAT;
     }

     if (failed)
          *failed = fail;

     if (fail)
     {
         return DFB_UNSUPPORTED;
     }

     D_DEBUG( "xcode DFB driver Xc4BgGraTestRegion exit:\n" );

     return DFB_OK;
}


static DFBResult
Xc4BgGraSetRegion( CoreLayer                  *layer,
              void                       *driver_data,
              void                       *layer_data,
              void                       *region_data,
              CoreLayerRegionConfig      *config,
              CoreLayerRegionConfigFlags  updated,
              CoreSurface                *surface,
              CorePalette                *palette,
              CoreSurfaceBufferLock      *lock )
{  
    dfb_ioctl_layer_config * ioctl_layer_config_ptr;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    D_ASSERT( driver_data != NULL );
    D_ASSERT( layer_data != NULL );
    app_ioctl_command dfb_ioctl_cmd;
 
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_device_data pmdev = mdrv->pdevice_data;

    D_DEBUG( "xcode DFB driver Xc4VirtualGra0SetRegion enter \n" );

    D_DEBUG ("surface width %d \n", config->width);
    D_DEBUG ("surface height %d \n", config->height);
    D_DEBUG ("destination x %d \n", config->dest.x);
    D_DEBUG ("destination y %d \n", config->dest.y);
    D_DEBUG ("destination width %d \n", config->dest.w);
    D_DEBUG ("destination height %d \n", config->dest.h);
    D_DEBUG ("surface caps %d \n", config->surface_caps);
    D_DEBUG ("surface pitch %d \n", lock->pitch);
    D_DEBUG ("surface phys 0x%x \n", lock->phys);

    if(updated == CLRCF_Z_DEPTH)
    {
        m_layer_data->z_depth = config->z_depth;
        Virtual_Layer_Combine_To_Physical_Hd_Gra0(mdrv, layer_data, NULL, BACKGROUND_GRA_LAYER_ID, DSFLIP_WAITFORSYNC);
        return DFB_OK;
    }
  
    if ((m_layer_data->y_start_addr != lock->phys) ||
        (m_layer_data->buf_width != config->width) ||
        (m_layer_data->buf_height != config->height) ||
        (m_layer_data->format != config->format))
    {
        memset (lock->addr, 0, config->width * config->height * DFB_BYTES_PER_PIXEL(config->format));
    }

    m_layer_data->buf_width = config->width;
    m_layer_data->buf_height = config->height;
    m_layer_data->x = config->dest.x;
    m_layer_data->y = config->dest.y;
    m_layer_data->dest_width = config->dest.w;
    m_layer_data->dest_height = config->dest.h;
    m_layer_data->source_x = config->source.x;
    m_layer_data->source_y = config->source.y;
    m_layer_data->source_width = config->source.w;
    m_layer_data->source_height = config->source.h;
    m_layer_data->flip_height = config->source.h;
    m_layer_data->y_start_addr = lock->phys;
    m_layer_data->y_pitch = lock->pitch;
    m_layer_data->format = config->format;
    m_layer_data->alpha = config->opacity & 0xff;
    m_layer_data->y_frame_size = m_layer_data->y_pitch * m_layer_data->flip_height;
    m_layer_data->z_depth = config->z_depth;
    m_layer_data->surface_caps = config->surface_caps;

    mdrv->shared->vir_gra_gic_ctl.gic_opt_status |= GIC_BG_READY; 

    D_DEBUG( "xcode DFB driver Xc4BgGraSetRegion exit \n" );

    return DFB_OK;
}

/****************************************************************************************************************************/

static DFBResult
XcodeBgFlipRegion( CoreLayer             *layer,
                 void                  *driver_data,
                 void                  *layer_data,
                 void                  *region_data,
                 CoreSurface           *surface,
                 DFBSurfaceFlipFlags    flags,
                 CoreSurfaceBufferLock *lock )
{
     DFBResult              ret;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "XcodeBgFlipRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( mdrv != NULL );

     D_DEBUG( "( 0x%08lx [%d] 0x%04x)\n", 
                 lock->phys, lock->pitch, flags);

     dfb_surface_flip( surface, false );

     D_DEBUG( "XcodeBgFlipRegion exit \n" );
     return DFB_OK;
}

/**********************************************************************************************************************/
static DFBResult
XcodeBgSetLevel( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int        level )
{
    D_DEBUG( "XcodeBgSetLevel enter \n" );

    D_DEBUG( "level %d \n", level ); 

    D_DEBUG( "XcodeBgSetLevel exit \n" );
 
    return DFB_OK;
}

/**********************************************************************************************************************/
static DFBResult
XcodeBgUpdateRegion( CoreLayer             *layer,
                   void                  *driver_data,
                   void                  *layer_data,
                   void                  *region_data,
                   CoreSurface           *surface,
                   const DFBRegion       *update,
                   CoreSurfaceBufferLock *lock )
{
    DFBResult              ret;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    D_DEBUG( "XcodeBgUpdateRegion enter \n" );

    D_ASSERT( surface != NULL );
    D_ASSERT( lock != NULL );
    D_ASSERT( mdrv != NULL );

    D_DEBUG( "( 0x%08lx [%d] )\n", 
             lock->phys, lock->pitch);

    Virtual_Layer_Combine_To_Physical_Hd_Gra0(mdrv, layer_data, lock, BACKGROUND_GRA_LAYER_ID, DSFLIP_WAITFORSYNC); 

    D_DEBUG( "XcodeBgUpdateRegion exit \n" );

}

/**********************************************************************************************************************/
static DFBResult
XcodeBgReallocateSurface( CoreLayer             *layer,
                          void                  *driver_data,
                          void                  *layer_data,
                          void                  *region_data,
                          CoreLayerRegionConfig *config,
                          CoreSurface           *surface )
{
     DFBResult ret;
     CoreSurfaceConfig conf;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;

     D_DEBUG( "XcodeBgReallocateSurface enter \n" );

     //layer id 
     if ((surface) &&
         (dfb_config->preallocate_graphic_layer & (0x1 << BACKGROUND_GRA_LAYER_ID)))
     {

         conf.flags  = CSCONF_SIZE | CSCONF_FORMAT | CSCONF_CAPS;
         conf.size.w = config->width;
         conf.size.h = config->height;
         conf.format = config->format;
         conf.caps   = DSCAPS_VIDEOONLY;

         ret = dfb_surface_arrange_again( surface, CSAID_LAYER4, CSAF_WRITE|CSAF_READ, &conf );

     }
     else
     {
         ret = DFB_FAILURE;
     }

     D_DEBUG( "XcodeBgReallocateSurface exit \n" );

     return ret;
}

/****************************************************************************************************************************/

const DisplayLayerFuncs XcodeBackLayerFuncs = {
     LayerDataSize:      XcodeVidLayerDataSize,
     InitLayer:          Xc4BgGraInitLayer,
     TestRegion:         Xc4BgGraTestRegion,
     SetRegion:          Xc4BgGraSetRegion,
     FlipRegion:         XcodeBgFlipRegion,
     UpdateRegion:       XcodeBgUpdateRegion,
     SetLevel:          XcodeBgSetLevel,
     ReallocateSurface:  XcodeBgReallocateSurface,
};

