/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_video_vid0.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/


#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include <pthread.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>


#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers_internal.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"
#include "xcioctl.h"


extern int XcodeVidLayerDataSize();

#define TOP_GRA1_SETTING_REGISTER_NUMBER 6


extern DFBResult Set_Layer_Level( void *driver_data,
                 int level,
                 u32 reg_left_shift,
                 u32 reg_mask);

extern DFBResult
Xc4GraReallocateSurface( CoreLayer             *layer,
                          void                  *driver_data,
                          void                  *layer_data,
                          void                  *region_data,
                          CoreLayerRegionConfig *config,
                          CoreSurface           *surface );
 
extern void Virtual_Layer_Combine_To_Physical_Hd_Gra0(pxc_dfb_driver_data    mdrv,
                                               void                  *layer_data,
                                               CoreSurfaceBufferLock *lock,
                                               int layer_id,
                                               DFBSurfaceFlipFlags flags);

/**********************************************************************************************************************/

static DFBResult
XcodeTopGra1InitLayer( CoreLayer                *layer,
                    void                       *driver_data,
                    void                       *layer_data,
                    DFBDisplayLayerDescription *description,
                    DFBDisplayLayerConfig      *config,
                    DFBColorAdjustment         *adjustment )
{
     u32     tmp;
     D_ASSERT( layer_data != NULL );
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     D_DEBUG( "xcode DFB driver XcodeTopGra1InitLayer enter:\n" );


     /* set capabilities and type */
     description->caps = DLCAPS_SURFACE | DLCAPS_ALPHACHANNEL | DLCAPS_OPACITY | 
                         DLCAPS_SCREEN_POSITION | DLCAPS_SCREEN_LOCATION | DLCAPS_LEVELS;

     description->type = DLTF_GRAPHICS;

     /* set name */
     snprintf( description->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "ViXS Xcode4 Top Graphics1 Plane " );

     /* fill out the default configuration */
     config->flags       = DLCONF_WIDTH       | DLCONF_HEIGHT |
                           DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS | DLCONF_SURFACE_CAPS ;

     if (dfb_config->preallocate_graphic_layer & (0x1 << TOP_GRA1_LAYER_ID) )
     {
         config->width       = 1920;
         config->height      = 1080;
     }
     else
     {
         config->width       = XCODE_DEFAULT_WIDTH;
         config->height      = XCODE_DEFAULT_HEIGHT;
     }

     config->pixelformat = DSPF_ARGB;
     config->buffermode  = DLBM_BACKVIDEO;
     config->surface_caps |= DSCAPS_VIDEOONLY | DSCAPS_STATIC_ALLOC | DSCAPS_DOUBLE ;

     m_layer_data->flip_height = 0;
     m_layer_data->z_depth = 0;
     D_DEBUG( "xcode DFB driver XcodeTopGra1InitLayer exit:\n" );

     return DFB_OK;
}

static DFBResult
XcodeTopGra1TestRegion( CoreLayer                  *layer,
                     void                       *driver_data,
                     void                       *layer_data,
                     CoreLayerRegionConfig      *config,
                     CoreLayerRegionConfigFlags *failed )
{
     CoreLayerRegionConfigFlags fail = 0;

     D_DEBUG( "xcode DFB driver XcodeTopGra1TestRegion enter:\n" );

     D_DEBUG ("surface width %d \n", config->width);
     D_DEBUG ("surface height %d \n", config->height);
     D_DEBUG ("destination x %d \n", config->dest.x);
     D_DEBUG ("destination y %d \n", config->dest.y);
     D_DEBUG ("destination weight %d \n", config->dest.w);
     D_DEBUG ("destination height %d \n", config->dest.h);
     D_DEBUG ("surface caps %d \n", config->surface_caps);
     D_DEBUG ("surface options %d \n", config->options);

     if (config->options & ~XCODE4_OSD_SUPPORTED_OPTIONS)
          fail |= CLRCF_OPTIONS;

     /* check buffermode */
     switch (config->buffermode) {
          case DLBM_BACKVIDEO:
               break;

          default:
               fail |= CLRCF_BUFFERMODE;
               break;
     }

     switch (config->format) {
          case DSPF_RGB32:
          case DSPF_ARGB:
               break;

          default:
               fail |= CLRCF_FORMAT;
     }


     if (failed)
          *failed = fail;

     if (fail)
	 {
          return DFB_UNSUPPORTED;
	 }


     D_DEBUG( "xcode DFB driver XcodeTopGra1TestRegion exit:\n" );

     return DFB_OK;
}

static DFBResult
XcodeTopGra1SetRegion( CoreLayer                  *layer,
                    void                       *driver_data,
                    void                       *layer_data,
                    void                       *region_data,
                    CoreLayerRegionConfig      *config,
                    CoreLayerRegionConfigFlags  updated,
                    CoreSurface                *surface,
                    CorePalette                *palette,
                    CoreSurfaceBufferLock      *lock )
{
    int                  ret;
    u32     tmp;
	u32     fifo_entries;
	dfb_ioctl_layer_config * ioctl_layer_config_ptr;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    D_ASSERT( driver_data != NULL );
    D_ASSERT( layer_data != NULL );
    app_ioctl_command dfb_ioctl_cmd;
	u32 gic_gra_format = 0;
	float float_coef;

    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_device_data pmdev = mdrv->pdevice_data;

    D_DEBUG( "xcode DFB driver XcodeTopGra1SetRegion enter \n" );

    D_DEBUG ("surface width %d \n", config->width);
    D_DEBUG ("surface height %d \n", config->height);
    D_DEBUG ("destination x %d \n", config->dest.x);
    D_DEBUG ("destination y %d \n", config->dest.y);
    D_DEBUG ("destination width %d \n", config->dest.w);
    D_DEBUG ("destination height %d \n", config->dest.h);
    D_DEBUG ("surface caps %d \n", config->surface_caps);
    D_DEBUG ("surface pitch %d \n", lock->pitch);
    D_DEBUG ("surface phys 0x%x \n", lock->phys);

    if(updated == CLRCF_Z_DEPTH)
    {
        m_layer_data->z_depth = config->z_depth;
        Virtual_Layer_Combine_To_Physical_Hd_Gra0(mdrv, layer_data, NULL, TOP_GRA1_LAYER_ID, DSFLIP_WAITFORSYNC);
        return DFB_OK;
    }

    if ((m_layer_data->y_start_addr != lock->phys) ||
        (m_layer_data->buf_width != config->width) ||
        (m_layer_data->buf_height != config->height) ||
        (m_layer_data->format != config->format))
    {
        memset (lock->addr, 0, config->width * config->height * DFB_BYTES_PER_PIXEL(config->format));
    }

    m_layer_data->buf_width = config->width;
    m_layer_data->buf_height = config->height;
    m_layer_data->x = config->dest.x;
    m_layer_data->y = config->dest.y;
    m_layer_data->dest_width = config->dest.w;
    m_layer_data->dest_height = config->dest.h;
    m_layer_data->source_x = config->source.x;
    m_layer_data->source_y = config->source.y;
    m_layer_data->source_width = config->source.w;
    m_layer_data->source_height = config->source.h;
    m_layer_data->flip_height = config->source.h;
    m_layer_data->y_start_addr = lock->phys;
    m_layer_data->y_pitch = lock->pitch;
    m_layer_data->format = config->format;
    m_layer_data->alpha = config->opacity & 0xff;
    m_layer_data->y_frame_size = m_layer_data->y_pitch * m_layer_data->flip_height;
    m_layer_data->z_depth = config->z_depth;
    m_layer_data->surface_caps = config->surface_caps;

    //set the alpha mode mask
    if (config->options & DLOP_ALPHACHANNEL)
    {
        mdrv->shared->vir_gra_gic_ctl.gic_alpha_mode_src1 = GIC_PER_PIXEL_ALPHA | (m_layer_data->alpha << GIC1_SRC1_ALPHA_C_SHIFT);
    }
    else
    {
        mdrv->shared->vir_gra_gic_ctl.gic_alpha_mode_src1 = GIC_CONSTANT_ALPHA | (m_layer_data->alpha << GIC1_SRC1_ALPHA_C_SHIFT);
    }

    mdrv->shared->vir_gra_gic_ctl.gic_opt_status |= GIC_SRC1_READY;

    D_DEBUG( "XcodeTopGra1SetRegion exit \n" );

    return ret;
}

/****************************************************************************************************************************/

static DFBResult
XcodeTopGra1FlipRegion( CoreLayer             *layer,
                 void                  *driver_data,
                 void                  *layer_data,
                 void                  *region_data,
                 CoreSurface           *surface,
                 DFBSurfaceFlipFlags    flags,
                 CoreSurfaceBufferLock *lock )
{
     DFBResult              ret;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "XcodeTopGra1FlipRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( mdrv != NULL );

     D_DEBUG( "( 0x%08lx [%d] 0x%04x)\n", 
                 lock->phys, lock->pitch, flags);


     dfb_surface_flip( surface, false );

     Virtual_Layer_Combine_To_Physical_Hd_Gra0(mdrv, layer_data, lock, TOP_GRA1_LAYER_ID, flags);

     if(flags & DSFLIP_WAIT)
     {
        int base_frm_cnt;
        WAIT_FOP_NEXT_HD_FRAME;
     }
    
     D_DEBUG( "XcodeTopGra1FlipRegion exit \n" );
     return DFB_OK;
}

/****************************************************************************************************************************/
static DFBResult
Xc4TopGra1UpdateRegion( CoreLayer             *layer,
                 void                  *driver_data,
                 void                  *layer_data,
                 void                  *region_data,
                 CoreSurface           *surface,
                 DFBSurfaceFlipFlags    flags,
                 CoreSurfaceBufferLock *lock )
{
     DFBResult              ret;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     CoreLayer * bg_layer;
     pxc_dfb_layer_data bg_layer_data;

     D_DEBUG( "Xc4TopGra0UpdateRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( mdrv != NULL );

     D_DEBUG( "( 0x%08lx [%d] 0x%04x)\n", 
                 lock->phys, lock->pitch, flags);

     Virtual_Layer_Combine_To_Physical_Hd_Gra0(mdrv, layer_data, lock, TOP_GRA1_LAYER_ID, DSFLIP_WAITFORSYNC);

     D_DEBUG( "Xc4TopGra0UpdateRegion exit \n" );
     return DFB_OK;
}

/**********************************************************************************************************************/
static DFBResult
XcodeTopGra1SetLevel( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int        level )
{
    DFBResult ret;

    D_DEBUG( "XcodeTopGra1SetLevel enter \n" );
    D_DEBUG( "level %d \n", level );

	ret = Set_Layer_Level(driver_data, level, GRA0_MUX_SHIFT, GRA0_MUX_MASK);

    D_DEBUG( "XcodeTopGra1SetLevel exit \n" );
	
     
    return ret;
}

/**********************************************************************************************************************/

const DisplayLayerFuncs Xc4TopGra1LayerFuncs = {
     LayerDataSize:      XcodeVidLayerDataSize,
     InitLayer:          XcodeTopGra1InitLayer,
     TestRegion:         XcodeTopGra1TestRegion,
     SetRegion:          XcodeTopGra1SetRegion,
     FlipRegion:         XcodeTopGra1FlipRegion,
     UpdateRegion:       (void *)Xc4TopGra1UpdateRegion,
     SetLevel:           XcodeTopGra1SetLevel,
     ReallocateSurface:  Xc4GraReallocateSurface
};
