/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xc3dfb.h  
*
*  DESCRIPTION: Conatins XC3 specific information  
*               
*
*  AUTHOR:
*
*  HISTORY
*
* $Log: not supported by cvs2svn $
* Revision 1.16  2010/11/19 16:00:02  ibosompem
* Fixed up the GIC DSPD_SRC_OVER not working with the new changes.
*
* Revision 1.15  2010/11/14 23:48:50  lichen
* no message
*
* Revision 1.14  2010/11/10 20:19:46  lichen
* change the definition of ufifo_submitted_frames
*
* Revision 1.13  2010/10/26 19:53:34  lichen
* make back ground layer can work
*
* Revision 1.12  2010/10/11 19:12:08  lichen
* fix the bug when the position is negative number
*
* Revision 1.11  2010/09/25 20:41:10  lichen
* add gic0_dst_index in the driver data
*
* Revision 1.10  2010/09/22 21:49:03  lichen
* add top gra0 and gra1 layer
*
* Revision 1.9  2010/08/25 19:20:52  lichen
* check porter duff
*
* Revision 1.8  2010/08/24 15:54:47  lichen
* enable the disp hd line interrupt on vblank start line
*
* Revision 1.7  2010/08/20 21:38:53  lichen
* switch lut for graphic layer in vblank
*
* Revision 1.6  2010/08/10 19:28:55  lichen
* add master field in driver structure
*
* Revision 1.5  2010/08/06 15:13:19  ibosompem
* -Added HD TO SD mirroring toggle control in DFB driver.
* -Added fixes for sync polarities in HDMI 3D frame packing output.
*
* Revision 1.4  2010/07/19 23:28:10  lichen
* virtual layer can scale
*
* Revision 1.3  2010/07/09 20:25:27  lichen
* add two field buf_width and buf_height in the layer data
*
* Revision 1.2  2010/07/08 14:49:54  lichen
* add init field in the layer data
*
* Revision 1.1.1.1  2010/02/25 00:13:08  lichen
* no message
*
* Revision 1.13  2010/02/18 15:12:27  lichen
* add fes0_opt in driver data for doing blit using FES
*
* Revision 1.12  2010/01/18 20:07:24  lichen
* add fes blit feature for yuv to yuv
*
* Revision 1.11  2009/12/23 15:09:42  lichen
* adjust virtual trigger mode
*
* Revision 1.10  2009/12/14 19:21:02  lichen
* adding virtual layer
*
* Revision 1.9  2009/11/26 20:31:03  lichen
* move xc3fb_if.h to xc4fb_if.h
*
* Revision 1.8  2009/10/26 14:33:42  lichen
* for set the screen position
*
* Revision 1.7  2009/09/22 15:40:05  lichen
* C_PITCH size align as 16 bytes on video layer
*
* Revision 1.6  2009/08/26 21:54:37  lichen
* add field select for video update
*
* Revision 1.5  2009/08/25 19:11:16  lichen
* make gra0 layer work as dfb setting
*
* Revision 1.4  2009/08/21 21:38:29  lichen
* add UpdateRegion to video layer
*
* Revision 1.3  2009/08/12 22:07:36  lichen
* sd graphic pipe can work
*
* Revision 1.2  2009/08/04 14:10:37  lichen
* gra0 and vid0 work for 480i, 480p, 720p, 1080i
*
* Revision 1.1  2009/04/03 22:11:43  lichen
* port Directfb 1.3 to arc, add xcode4 driver
*
* Revision 1.9  2009/04/03 16:09:56  lichen
* save for moving to 1.3.0
*
* Revision 1.4  2008/10/22 22:29:59  lichen
* adding color key feature
*
* Revision 1.3  2008/10/21 17:56:34  lichen
* adding color alpha for XCODE4 Blit
*
* Revision 1.1  2008/10/10 14:39:39  lichen
* for adding gfxdriver vixs_xcode4
*
* Revision 1.1  2008/10/08 21:05:45  jdoyle
* Initial version
*
*
***************************************************/

/* */
#ifndef __XCDFB_H__

#define __XCDFB_H__ 

#include "xc4fb_if.h"
#include "xcioctl.h"                   
#define   XC_DFB_DRIVER_NAME "XCODE 4 DirectFB Driver"

#define   XC_FOUR_DFB_DEVICE_NAME "XCODE IV (SDK)"


#define   XC_DFB_DRIVER_VENDOR "ViXS Systems Inc"
#define   XC_DFB_DRV_VERSION_MAJ   6
#define   XC_DFB_DRV_VERSION_MIN   46

#define   XC_DFB_LAYER_OVERLAY_NAME     "Vixs XCODE IV Overlay" 
#define   XC_DFB_LAYER_PRIMARY_NAME     "Vixs XCODE IV Primary" 

#define XCODE_VALIDATE_STATE(flag)      (mdev->chip_state |= (flag))
#define XCODE_INVALIDATE_STATE(flag)    (mdev->chip_state &= ~(flag))

#define XCODE_CHECK_VALID_STATE(flag)   if (! (mdev->chip_state & flag))                 \
                                                xcode_validate_##flag( mdev, state );   \

#define XCODE_CHECK_VALID_STATE_NO_LOCK(flag)   if (! (mdev->chip_state & flag))                 \
                                                xcode_validate_##flag( mdev, state);   \

#define	FB0_DEV	"/dev/fb0"
#define	HDTOSD_DEV	"/dev/fb1"

#define MAX_OUTPUTS         8

#define HDCP_LIB_NAME   "libhdcp.so"


/* 
*   Upsampling factor is fixed at 64 
*   There are 10 horizontal taps for both VID0/VID1
*   6 vertical taps for VID0, VID1 has 4 taps
*   
*   Worst case is: 64 * 10 taps   
*   VID0_V = 64 phase * 6
*   VID0_H = 64 phase * 10
*
*   VID1_V = 64 phase * 4
*   VID1_H = 64 phase * 10
*/
#define VIDEO_SCL_MAX_TAPS      640


/* Function to load HDCP keys into hardware ... */
typedef int (*load_HDCP_key)(void);


/*
 * State validation flags.
 *
 * There's no prefix because of the above macros.
 */

typedef enum
{
     DESTINATION  = 0x00000001,    
     COLOR        = 0x00000002,

     SOURCE       = 0x00000010,

     ALL          = 0x00010013

}VIXS_XC_STATE;

/*
typedef struct _xc_dfb_overlay_data
{
     u32  size;
     u32  ufifo_submitted_frames;
     u32  ufifo_index_mask;
     u32  udve_hw_frm_cnt;
     u32  ufifo_read_ptr;
     u32  ufifo_write_ptr;
     u32  ufifo[16];
     u32  ufield_update;
     u32  uovl_field_index[2];
     u32  uReserved1;  
     u32  uReserved2;  
     u32  uReserved3;  

}xc_dfb_overlay_data, *pxc_dfb_overlay_data;
*/


typedef enum {
    /* Disable power to all display subsystems ... */
    DISPLAY_POWER_DISABLE_ALL   =   0x00000001,

} display_pwr_flags_t;



typedef struct _xc_dfb_device_data
{
     u32  size;
     
     /* Display clock enables (do not allow display calls) ... */
     u8   disp_clock;
     
     /* GFX clock (GIC/G2D), fail all calls if these are hard disabled ... */
     u8   gfx_clock;

     /* for fifo/performance monitoring */
     unsigned int fifo_space;
     unsigned int waitfifo_sum;
     unsigned int waitfifo_calls;
     unsigned int fifo_waitcycles;
     unsigned int idle_waitcycles;
     unsigned int fifo_cache_hits;
     DFBSurfaceDrawingFlags   drawingflags;  /* drawing flags */
     DFBSurfaceBlittingFlags  blittingflags; /* blitting flags */

     DFBRegion                 clip;

     //
     //Used to determine chip type
     // 
     unsigned int    chip_type;
     //VIXS_XC_CHIP_TYPE   chip_type;

     //
     //Controls what states are currently valid
     // 

     VIXS_XC_STATE       chip_state;

     //
     //destination parameters for rendering
     // 
     u32  dst_addr;
     u32  dst_phys;
     u32  dst_cb_phys;
     u32  dst_cr_phys;
     u32  dst_pitch;
     u32  dst_c_pitch;
     u32  dst_width;
     u32  dst_height;
     u32  dst_flip_height;
     u32  dst_format;
	 DFBSurfaceCapabilities dst_caps;
     u32  dst_bpp;
	 DFBColor dst_color;
     u32  dst_colorkey;  /* colorkey for destination */
     DFBSurfaceBlendFunction  dst_blend;     /* blend function for destination */

     //
     //source parameters for blt/rendering
     // 
     u32  src_addr;
     u32  src_phys;                 
     u32  src_cb_phys;                 
     u32  src_cr_phys;                 
     u32  src_pitch;
     u32  src_c_pitch;
     u32  src_width;
     u32  src_height;
     u32  src_flip_height;
     u32  src_format;
	 DFBSurfaceCapabilities src_caps;
     u32  src_bpp;
     u32  src_colorkey;  /* colorkey for source */
     DFBSurfaceBlendFunction  src_blend;     /* blend function for destination */


     //
     //color used for color fills
     // 
     u32  fill_color;

	 u8  src_color_alpha;
	 u8  src_color_r;
	 u8  src_color_g;
	 u8  src_color_b;
	 
	 u8	 src_color_update;
	 
	 
     //
     //used for selecting dma scratch area(we use two)
     // 
     u32  dma_list_index;

     u32  uReserved1;
     u32  uReserved2;
     u32  uReserved3;
     
     int  target_dx;
     int  target_dy;
     
     u32  rectsrc_x;
     u32  rectsrc_y;
     
     u32  rectsrc_w;
     u32  rectsrc_h;
     
     /* 
     *  These are the REAL source widths and heights,
     *  these are used for scale blits that will need to be scaled
     *  by the G2D before they can be used by the GIC
     */
     u32  rect_scalesrc_w;
     u32  rect_scalesrc_h;
     
     DFBAccelerationMask    current_op;
     
     
     
     /* Adjusted source physical address, used for two pass operations.. */
     /* This is also used incase we need to make conversions on the source ... */
     u32  adj_src_phys;
     u32  adj_src_format;
     u32  adj_src_pitch;
     /* Used to tell original routine where to start grabbing data from */
     u32  adj_offset;
     
     /* Flag to tell us whether to use adj_phys address or not ... */
     u32  use_adj_phys;
     
     int temp_render_offset;
     u32 temp_render_physical;
     u32 temp_render_logical;
     u32 temp_render_extra_physical;
     u32 temp_render_line_physical;
     
     /* Framebuffer base address ... */
     u32 fb_base;
     u32 fb_size;
     u32 fb_phys;
     
     /* Adjustment done to pmdev->target_dx and pmdev->rectsrc_w*/
     u32 align_adjust;
     
     /* 
     *  Used to determine whether we should refresh the text color palette
     *  when we are asked to draw text (palette could change based on the 
     *  blitting/drawing flags state)
     */
     u32 last_text_flags;
     u8  last_text_r;
     u8  last_text_g;
     u8  last_text_b;
     u32 last_dest_caps;
     u32 last_text_format;
     u32 text_palette[256];
     u32 text_entries_num;
     u32 text_entries_num_g2d;
     
     /* 
     *  A primitive buffer allocation scheme:
     *  last_temp_physical holds the last physical address
     *  used for a G2D -> GIC -> final operation
     *
     *  last_temp_size holds the total size of this buffer
     *  this is so we can cycle around
     *  If for whatever reason, we are too big or the next op
     *  will require a write into currently "owned" memory, we will have to wait 
     *  for the GIC to idle.
     */
     u32 last_temp_physical;
     u32 last_temp_size;
     u32 last_temp_position;
     u32 last_temp_target;
     u32 last_temp_rect;
     u32 last_temp_pitch;
     DFBRegion last_temp_clip;
          
     StateModificationFlags modified;
}xc_device_data, *pxc_dfb_device_data;

typedef struct _gic_opt_struct
{
	u32 gic_opt_status;

	u32 gic_src1_adr0;
	u32 gic_src1_adr1;
	u32 gic_src1_adr2;
	u32 gic_src1_h_w;
	u32 gic_src1_pitch;
	u32 gic_src1_y_x;
	u32 gic_src1_mode;
	u32 gic_alpha_mode_src1;

	u32 gic_src2_adr0;
	u32 gic_src2_adr1;
	u32 gic_src2_adr2;
	u32 gic_src2_h_w;
	u32 gic_src2_pitch;
	u32 gic_src2_y_x;
	u32 gic_src2_mode;
	u32 gic_alpha_mode_src2;

	u32 gic_dst_adr0;
	u32 gic_dst_adr1;
	u32 gic_dst_adr2;
	u32 gic_dst_h_w;
	u32 gic_dst_pitch;
	u32 gic_dst_y_x;
	u32 gic_dst_mode;

	u32 gic1_src_csc_parm0;
	u32 gic1_src_csc_parm1;
	u32 gic1_src_csc_parm2;
	u32 gic1_src_csc_parm3;
	u32 gic1_src_csc_parm4;
	u32 gic1_src_csc_parm5;
	u32 gic1_src_csc_parm6;
	u32 gic1_src_csc_parm7;

	u32 gic_clip_top_left;
	u32 gic_clip_bot_right;
	u32 gic_backgrd_color;
	u32 gic_trigger_command;
	u32 gic_dst_subfld;

	u32 dst_buffer1;
	u32 dst_buffer2;
	u32 dst_pitch;

	u32 src2_dest_width;
	u32 src2_dest_height;
	u32 src2_source_width;
	u32 src2_source_height;

}gic_opt_struct;

typedef struct sd_frame_buffer_s
{
    int sd_buffer_addr[8];
    int sd_buffer_num;
    int sd_buffer_idx;
    int sd_buffer_max;
    DFBRectangle sd_buffer_rect[8];
    DFBRectangle sd_buffer_src[8];
    int sd_buffer_update;
    int sd_buffer_first;
    DFBSurfacePixelFormat sd_buffer_fmt;
    DFBColorSpace sd_buffer_csp;
}sd_frame_buffer_t;

typedef struct sd_frame_entries_s
{
    int base;
    int start;
    int terminated;
    int double_frame;
    pthread_t thread;
    pthread_cond_t cond;
    pthread_mutex_t mutex;
    sd_frame_buffer_t buffer[4];
    DFBRectangle curr_vid_src;
    DFBRectangle curr_vid_dst;
    int palette_update;
    DFBColorSpace color_space;
}sd_frame_entries_t;

typedef struct _XC4Shared
{
     gic_opt_struct vir_gra_gic_ctl;
     DFBScreenOutputResolution current_hd_resolution;
     DFBScreenEncoderScanMode  current_hd_scanmode;

     int mode_index;
     int last_mode_index;
     DFBScreenOutputSignals last_out_signals[MAX_OUTPUTS];
     int last_mode_index_out[MAX_OUTPUTS];
     int last_frame_counter;
     int last_frame_update;
     u32 last_frame_address;
     u32 last_frame_pitch;
     DFBScreenOutputResolution current_sd_resolution;
     DFBScreenPowerMode current_sd_power_mode;
     DFBColorSpace current_hd_color_space;
     int last_output_init;

     /* re-entrancy mutex to prevent simultaneous access to shared HW blocks */
     pthread_mutex_t reentrancy_mutex;

     /* g2d mutex */
     pthread_mutex_t g2d_mutex;

     /* g2d mutex */
     pthread_mutex_t virtual_layer_mutex;

     /* the lock to ensure only one thread of all DFB application processes to get HW access for image decoding */  
     FusionSkirmish img_hw_lock;

     sd_frame_entries_t sd_frame_entries;
}XC4Shared;

#define GIC_SRC1_READY 0x01
#define GIC_SRC2_READY 0x02
#define GIC_DST_READY 0x04
#define SRC2_NEED_SCALE 0x08

#define GIC_DST_SUBFLD	0x10
#define GIC_BACKGRD_COLOR 0x20
#define DST_NEED_CLEAR 0x40
#define NEED_COMBINE_3D_BACKGROUND_LAYER 0x80
#define NEED_SHOW_3D_VIDEO 0x100
#define BACKGROUND_LAYER_IS_REAY_FOR_3D 0x200

#define GIC_BG_READY 0x200

#define NO_COLOR_CONVERT 0
#define FROM_RGB_TO_YUV 1
#define FROM_YUV_TO_RGB 2

/* 
*   Define the render size (of a temporary buffer) 
*   Let us have it at 1920x1080 ARGB for now....
*/

/* Size of temporary rendering */
#define TEMP_RENDER_W       1920
#define TEMP_RENDER_H       1080
#define TEMP_RENDER_SIZE    TEMP_RENDER_W*4*TEMP_RENDER_H

/* Size of extra temporary rendering */
#define TEMP_RENDER_EXTRA_SIZE 0x10000
#define TEMP_RENDER_LINE_SIZE TEMP_RENDER_W * 4

/* Flag defines for the asic_flags in driver_data */
#define VIPER_ASIC_A11	0x1 
#define VIPER_ASIC_A21  0x2

typedef struct _xc_dfb_driver_data
{
    u32  accelerator;
    XC4Shared *shared;
    pxc_dfb_device_data pdevice_data;
    CoreScreen * screen;
    CoreScreen * sdscreen;
    int sd_mode_index;
    int hdtosd_fd;
    int fb0_handle;
    int master;
    u32 asic_flags;
    volatile u8 *mmio_base;
    volatile u8 *mmaper_base;
    CoreDFB * dfb_core;
    display_pwr_flags_t pwr_flags;
    u32 hd_vid0_pending_cmd;
    void * hd_vid0_pending_ptr;
    int output;
}xc_driver_data, *pxc_dfb_driver_data;


typedef struct _scaleCoefParam
{
    double  lpf;
    double  beta;
    double  lpf0;   
    double  rho;  
    u32     type;
} scaleCoefParam;


/* Scalar re-calculation flags */
#define RECALC_SHARPEN      0x00000001
#define RECALC_PREMULT      0x00000002
#define RECALC_FULLRECALC   0x00000004

typedef struct _colorModeParms
{
    /* Luma/chroma scaling (can be used for gain control) */
    double  premulty;
    double  premultc;
    
    /* 
    *   Soften/Sharpen variable is only used for storage (v/h are passed directly 
    *   to co-efficient calculator)
    */
    double  csoften;
    double  ysoften;
    
    double  vcsoften;
    double  hcsoften;
    
    double  vysoften;
    double  hysoften;
    
    double  csharpen;
    double  ysharpen;
    
    double  hcsharpen;
    double  vcsharpen;
    
    double  hysharpen;
    double  vysharpen;
    
    /* For now, we will only control the Horizontal scaler taps */
    u32     ntaps_up_c;
    u32     ntaps_up_y;
} colorModeParameters;


typedef struct _xc_dfb_layer_data
{
    u32 x;
    u32 y;
    u32 buf_width;
    u32 buf_height;
    u32 dest_width;
    u32 dest_height;
    u32 source_width;
    u32 source_height;
    u32 flip_height;
    u32 uovl_field_index[2];
    u32 ufield_update;
    u32 field_select;
    u32 y_pitch;
    u32 c_pitch;
    u32 y_start_addr;
    u32 u_start_addr;
    u32 v_start_addr;
    u32 back_addr;
    DFBSurfacePixelFormat format;
    u32 alpha;
    u32 y_frame_size;
    u32 c_frame_size;
    int last_frame_counter;
    int total_submitted_frames;
    int total_return_frame_to_free;
    u32 current_active_lut_page;
    DFBDisplayLayerBufferMode buffermode;
    DFBSurfaceCapabilities surface_caps;
    u32 lut_require_switch;
    u32 palette_need_switch;
    u32 layer_is_set;
    u32 status;
    u32 source_x;
    u32 source_y;
    int z_depth;
    u32 layer_set_return_fail_counter;
    pthread_mutex_t layer_mutex ;
    int last_frame_index;
    int last_drop_flag;
    u32 palette[256];
    u32 palette_num;
    int overlay_rotation;
    DFBColorSpace color_space;
    
    /* Scaling calculations .. */
    double g[VIDEO_SCL_MAX_TAPS];
    double h[VIDEO_SCL_MAX_TAPS];
    double dcg;
    double dch;
    double mulfg;
    double oneoverdcg;
    double oneoverdch;
    scaleCoefParam  vY;
    scaleCoefParam  vC;
    scaleCoefParam  hY;
    scaleCoefParam  hC;
    colorModeParameters colorParams;
    u32 passthru;
    
    CoreLayerRegionConfig      config;
}xc_layer_data, *pxc_dfb_layer_data;

#define LAYER_CHECK_BIT 1
#define LAYER_SETTING_CHANGE 2
#define WAIT_FOR_LAYER_UPDATE_FINISH 4
#define WAIT_FOR_LAYER_CLEAR 8

#define UPDATE_REGION_TOP_INDEX 0
#define UPDATE_REGION_BOT_INDEX 1

/* copied from generic.c*/
/* whether pixelformat is YUV or not */
static bool is_ycbcr[DFB_NUM_PIXELFORMATS] = {
     false,    /* DSPF_ARGB1555 */
     false,    /* DSPF_RGB16 */
     false,    /* DSPF_RGB24 */
     false,    /* DSPF_RGB32 */
     false,    /* DSPF_ARGB */
     false,    /* DSPF_A8 */
     true,     /* DSPF_YUY2 */
     false,    /* DSPF_RGB332 */
     true,     /* DSPF_UYVY */
     true,     /* DSPF_I420 */
     true,     /* DSPF_YV12 */
     false,    /* DSPF_LUT8 */
     false,    /* DSPF_ALUT44 */
     false,    /* DSPF_AiRGB */
     false,    /* DSPF_A1 */
     true,     /* DSPF_NV12 */
     true,     /* DSPF_NV16 */
     false,    /* DSPF_ARGB2554 */
     false,    /* DSPF_ARGB4444 */
     false,    /* DSPF_RGBA4444 */
     true,     /* DSPF_NV21 */
     true,     /* DSPF_AYUV */
     false,    /* DSPF_A4 */
     false,    /* DSPF_ARGB1666 */
     false,    /* DSPF_ARGB6666 */
     false,    /* DSPF_RGB18 */
     false,    /* DSPF_LUT2 */
     false,    /* DSPF_RGB444 */
     false,    /* DSPF_RGB555 */
     false,    /* DSPF_BGR555 */
     false,    /* DSPF_RGBA5551 */
     true,     /* DSPF_YUV444P */
     false,    /* DSPF_ARGB8565 */
     true,     /* DSPF_AVYU */
     true,     /* DSPF_VYU */
     false,    /* DSPF_A1_LSB */
     true,     /* DSPF_YV16 */
};



#endif //__XCDFB_H__ 

