/*  Copyright (c) 2006-2010 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_adv7511.h
*
*  DESCRIPTION: ADV7511 DirectFB driver extension for XCode4 
*
*  AUTHOR:
*
*  HISTORY
*
*/

#ifndef __VIXS_ADV7511_H__
#define __VIXS_ADV7511_H__

#include "xcode4_osd.h"
#include "xcodeapi.h"

#define IOCTL_TIMEOUTVALUE  2000
#define XC4_I2C_CHANNEL     0
#define MASTER_I2C_CLOCK            27000000
#define I2C_BUS_CLOCK               100000
#define MASTER_I2C_ATOMIC_PERIOD    (MASTER_I2C_CLOCK / I2C_BUS_CLOCK / 4)
#define MASTER_I2C_TIMEOUT          300
#define ADV7511_I2C_ADDR        0x72
#define	ADV7511_PACKET_I2C_ADDR 0x70

#define	XCODE_DEVICE_NAME	"/dev/vixs/xcodedrv"
#define	I2C_SLEEP_US	4000



#define	FPGA_I2C_ADDR	 0x30
#define	FPGA_I2C_MUX_REG 0x0D

#define	FPGA_I2C_SYNC_TO_ADV7511 (0x1 << 1)
/*
	Defines the base addresses for the packets in the 
	packet memory space.	
*/
#define	ADV7511_SPD_PACKET_ADDR	  0x00
#define ADV7511_MPEG_PACKET_ADDR  0x20
#define ADV7511_ACP_PACKET_ADDR   0x40
#define ADV7511_ISRC1_PACKET_ADDR 0x60
#define ADV7511_ISRC2_PACKET_ADDR 0x80
#define ADV7511_GM_PACKET_ADDR    0xA0
#define ADV7511_VSI_PACKET_ADDR   0xC0 
/* 
	The above to the ADV7511 is a spare packet, but we will use it for the vendor
	specific info frame.
*/
#define ADV7511_SPAREPACKET1_ADDR 0xE0


/* Packet identifiers (for manipulating packet information) */
#define ADV7511_SPD_PACKET		  0x00
#define ADV7511_MPEG_PACKET		  0x01
#define ADV7511_ACP_PACKET		  0x02
#define ADV7511_ISRC1_PACKET	  0x03
#define ADV7511_ISRC2_PACKET	  0x04
#define ADV7511_GM_PACKET		  0x05
#define ADV7511_VSI_PACKET		  0x06
#define ADV7511_SPARE1_PACKET	  0x07

/* Spare packet masks for ADV7511_PACKET_CTRL_REG */
#define ADV7511_GC_PACKET_MASK	   0x80
#define ADV7511_SPD_PACKET_MASK    0x40
#define ADV7511_MPEG_PACKET_MASK   0x20
#define ADV7511_ACP_PACKET_MASK	   0x10
#define ADV7511_ISRC_PACKET_MASK   0x08
#define ADV7511_GM_PACKET_MASK	   0x04
#define ADV7511_SPARE1_PACKET_MASK 0x02
#define ADV7511_VSI_PACKET_MASK	   0x01

/* ADV7511 Packet Control Register */
#define ADV7511_PACKET_CTRL_REG	   0x40

/* ADV7511 packet update flags */
#define	ADV7511_PACKET_UPDATE	  0x80
#define	ADV7511_PACKET_NOUPDATE	  0x00


/*
	Functions which are private to the ADV7511 module
*/
static int xc_i2c_read(int I2CChannel, int Address, void *data, size_t n, int start, int stop);
static int xc_i2c_write(int I2CChannel, int Address, const void *data, size_t n, int start,int stop);
static int xc_i2c_write_then_read(int I2CChannel, int Address, const void *wdata, void *rdata, size_t wn, size_t rn);
static int xcode_i2c_read(xcode_handle_t XCodeHandle, int I2CChannel, int Address, void *data, size_t n, int atomic_period, int timeout, int start, int stop);
static int xcode_i2c_write(xcode_handle_t XCodeHandle, int I2CChannel, int Address, const void *data, size_t n, int atomic_period, int timeout, int start, int stop);

static int i2c_read(unsigned char dev_addr, unsigned char *byte, unsigned int addr);
static int i2c_write(unsigned char dev_addr, unsigned char byte, unsigned int addr);






/* All functions will return 0 if OK, any other number if failure */


/*
    Initializes the AV driver/I2C interface

*/
int	ADV7511_Init();

/*
    Runs a script (reg val, data val)

*/
int	ADV7511_RunScript(unsigned char *script);

/*
	Grabs a display mode struct and tries to set the appropriate mode
*/
int	ADV7511_SetMode(struct display_mode_struct *mode);


/*
	Updates the specified packet in the ADV7511 packet memory
	len = length of PACKET body
	So packet_data points to 3 + len number of bytes
*/
int ADV7511_UpdatePacket(int packet_id, unsigned char *packet_data, unsigned int len);

/*
	Enables the sending a particular packet over the HDMI Tx
*/
int ADV7511_EnablePacket(int packet_id);

/*
	Disables the sending of a particular packet over the HDMI Tx
*/
int ADV7511_DisablePacket(int packet_id);

/* #ifndef ... */
#endif
