#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>


#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <asm/cachectl.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/system.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>
#include <gfx/clip.h>

#include <misc/conf.h>

#include <math.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"
#include "xcioctl.h"

D_DEBUG_DOMAIN( Xcode2D, "Xcode2D", "Xcode2D" );

D_DEBUG_DOMAIN( XcodeBlit, "XcodeBlit", "XcodeBlit" );

#define DPRINTF(...) D_DEBUG_AT( Xcode2D, __VA_ARGS__)

#define BLITPRINTF(...) D_DEBUG_AT( XcodeBlit, __VA_ARGS__)

#define DPRINTF_VALUE(x) DPRINTF( #x"=0x%x\n", x)

#define G2D_NEGATIVE_SIGN_MASK 0x4000
#define G2D_VALUE_MASK 0x3fff

#define TRANSFER_G2D_POSITION_NUMBER(output,input) \
     if (input >= 0) \
     { \
         output = input;\
     }\
     else \
     { \
         output = G2D_NEGATIVE_SIGN_MASK | ((u32)input & G2D_VALUE_MASK ); \
     }







   







/*
 *
 *Function Name:xcodeEngineReset
 *
 *Parameters: 
 *             drv-> Pointer to driver data
 *             dev-> Pointer to device data
 *
 *Description:
 *             Reset engine
 *Returns:
 *             NA
*/
void xcodeDummyEngineReset( void *drv, void *dev )
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) drv;
    pxc_dfb_device_data mdev = (pxc_dfb_device_data) dev;
    app_ioctl_command dfb_ioctl_cmd;


    u32 tmp;

    DPRINTF( "xcode DFB driver xcodeEngineReset enter:\n" );
    
    DPRINTF( "xcode DFB driver xcodeEngineReset exit:\n" );

}


/*
 *
 *Function Name:xcodeEngineSync
 *
 *Parameters: 
 *             drv-> Pointer to driver data
 *             dev-> Pointer to device data
 *
 *Description:
 *             Sync the G2D engine
 *Returns:
 *             DFB_OK when engine is IDLE
*/

DFBResult xcodeDummyEngineSync( void *drv, void *dev )
{

     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) drv;
     pxc_dfb_device_data mdev = (pxc_dfb_device_data) dev;

     return DFB_OK;
}



/*
 *
 *Function Name:xcodeCheckState
 *
 *Parameters: 
 *           drv-> Xcode driver context
 *           dev-> Current device context(only 1 supported for now)
 *           *state-> Indicates current state of card(clr keys etc)
 *           accel-> Indicates what type of operation is being performed
 *
 *Description:
 *           Parses the accel parameters to ensure current ASIC can carry out intended  
 *           operation, if it can't, it doesnt set any bits within the state->accel variable
 *
 *Returns:
 *           N/A
*/
void xcodeDummyCheckState( void *drv, void *dev,
                      CardState *state, DFBAccelerationMask accel )
{

     int supported_accel=0;   

     DPRINTF( "xcodeCheckState enter:\n" );

     DPRINTF( "driver accel= %08x \n", accel );

     state->accel |= supported_accel;

     DPRINTF( "xcode DFB driver state->accel %08x \n", state->accel );

     DPRINTF( "xcodeCheckState exit:\n" );

}







bool xcodeDummyFillRectangle( void *drv, void *dev, DFBRectangle *rect )
{
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) drv;
     pxc_dfb_device_data pmdev = (pxc_dfb_device_data) dev;
     int dst_pixel_format_color;
     u32 g2d_op_ctl_value;
     u32 size_of_operation = BASE_SIZE_OF_FILLRECTANGLE;
     u32 fes_tmp, fes_val;

     DPRINTF( "xcode DFB driver xcodeFillRectangle = %x %x %x %x\n",rect->x,rect->y,rect->w,rect->h );
     
    
     /* H/W clock not turned on... do nothing ... */
     return false;
     
  

    
}

bool xcodeDummyBlit( void *drv, void *dev, DFBRectangle *rect, int dx, int dy)
{
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) drv;
     pxc_dfb_device_data pmdev = (pxc_dfb_device_data) dev;
   
     
     return false;
}

/*
 *
 *Function Name:xcodeG2dStretchBlit
 *
 *Parameters: 
 *           drv-> Xcode driver context
 *           dev-> Current device context(only 1 supported for now)
 *           *src_rec-> Rectangle describing source size
 *           *dst_rec-> Rectangle describing destination size   
 *
 *Description:
 *           Performs a stretch SRC copy.
 *
 *Returns:
 *           true or false
*/
bool xcodeDummyStretchBlit(void* drv, void* dev, DFBRectangle* src_rec, DFBRectangle* dst_rec)
{
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) drv;
     pxc_dfb_device_data pmdev = (pxc_dfb_device_data) dev;
       

     return false;

}


  

void xcodeDummySetState( void *drv, void *dev,
                    GraphicsDeviceFuncs *funcs,
                    CardState *state, DFBAccelerationMask accel)
{

     pxc_dfb_device_data mdev = (pxc_dfb_device_data) dev;
     StateModificationFlags modified = state->modified;

     DPRINTF( "xcode DFB driver xcodeSetState enter:\n" );

    return;

}