/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_osd_gra0.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/

#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"



extern DFBResult convert_dfb_format_to_disp_format(const CoreLayerRegionConfig *config, 
                                            int * disp_format, int *ignorePixelAlpha);

extern struct display_mode_struct * xc4_get_mode(int mode_index);

extern int xc_layer_search_mode_index(CoreLayerRegionConfig * config);

extern u32 Calculate_gfx_hsca_inv_ratio(u32 input_h_size, u32 output_h_size);

extern void Calculate_gfx_vsca_inv_ratio(DFBScreenEncoderScanMode  scanmode,
                                         u32 input_v_size, 
                                         u32 output_v_size,
                                         u32 * gra_vsca_inv_ratio,
                                         u32 * gra_vsca_init,
                                         u32 * gra_vsca_start);

static u32 disp_gra0_register_table[SIZE_OF_DISP_GRA_REGISTER_TABLE];

extern int XcodeVidLayerDataSize();

void xcodeGraFlipRegion(u32 *disp_reg_tab, 
                  void *driver_data, 
                  void * layer_data,
                  CoreSurface *surface, 
                  DFBSurfaceFlipFlags flags, 
                  CoreSurfaceBufferLock *lock, 
                  int compositor);

void	  xcodeAlignBufferSize(u32 *disp_reg_tab, void *driver_data);

DFBResult Xcode4_Set_Graphic_Layers(void                       *driver_data,
                                    void                       *layer_data,
                                    CoreLayerRegionConfig      *config,
                                    CoreLayerRegionConfigFlags  updated,
                                    CorePalette                *palette,
                                    CoreSurfaceBufferLock      *lock,
                                    u32 * disp_gra_register_table);

extern DFBResult
Xc4GraReallocateSurface( CoreLayer             *layer,
                          void                  *driver_data,
                          void                  *layer_data,
                          void                  *region_data,
                          CoreLayerRegionConfig *config,
                          CoreSurface           *surface );

#define GIC_ARGB_SURFACE 13

/**********************************************************************************************************************/

static DFBResult
XcodeDummyInitLayer( CoreLayer                *layer,
              void                       *driver_data,
              void                       *layer_data,
              DFBDisplayLayerDescription *description,
              DFBDisplayLayerConfig      *config,
              DFBColorAdjustment         *adjustment )
{
    u32     tmp;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    
    description->caps = DLCAPS_SURFACE ;

    description->type = DLTF_GRAPHICS;

    /* set name */
    snprintf( description->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "ViXS Xcode4 Graphics Plane 0" );

    /* fill out the default configuration */
    config->flags       = DLCONF_PIXELFORMAT | DLCONF_OPTIONS;


     config->width       = XCODE_DEFAULT_WIDTH;
     config->height      = XCODE_DEFAULT_HEIGHT;



     config->pixelformat = DSPF_ARGB;


     config->options     = DLOP_ALPHACHANNEL;
     adjustment->flags   = DCAF_BRIGHTNESS;

     memset (layer_data, 0, sizeof (xc_layer_data));

     //Set the gra0 register table


    return DFB_OK;
}

/**********************************************************************************************************************/

static DFBResult
XcodeDummyTestRegion( CoreLayer                  *layer,
               void                       *driver_data,
               void                       *layer_data,
               CoreLayerRegionConfig      *config,
               CoreLayerRegionConfigFlags *failed )
{
    return DFB_OK;
}



/**********************************************************************************************************************/

static DFBResult
XcodeDummySetRegion( CoreLayer                  *layer,
                    void                       *driver_data,
                    void                       *layer_data,
                    void                       *region_data,
                    CoreLayerRegionConfig      *config,
                    CoreLayerRegionConfigFlags  updated,
                    CoreSurface                *surface,
                    CorePalette                *palette,
                    CoreSurfaceBufferLock      *lock )
{
    DFBResult ret;
    D_ASSERT( driver_data != NULL );
    D_ASSERT( layer_data != NULL );
    app_ioctl_command dfb_ioctl_cmd;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32 gic_fifo_entries;
    u32 temp_height;

    return DFB_TRUE;
}


/****************************************************************************************************************************/

static DFBResult
XcodeDummyFlipRegion( CoreLayer             *layer,
                 void                  *driver_data,
                 void                  *layer_data,
                 void                  *region_data,
                 CoreSurface           *surface,
                 DFBSurfaceFlipFlags    flags,
                 CoreSurfaceBufferLock *lock )
{
    return DFB_OK;
}


/****************************************************************************************************************************/

static DFBResult
XcodeDummyUpdateRegion( CoreLayer             *layer,
                   void                  *driver_data,
                   void                  *layer_data,
                   void                  *region_data,
                   CoreSurface           *surface,
                   const DFBRegion       *update,
                   CoreSurfaceBufferLock *lock )
{
    return DFB_OK;
}

/****************************************************************************************************************************/

DFBResult XcodeDummySetColorAdjustment(CoreLayer            *layer,
                                void                 *driver_data,
                                void                 *layer_data,
                                DFBColorAdjustment   *adj)
{
   return DFB_OK;
}



/**********************************************************************************************************************/
static DFBResult
XcodeDummyGetLevel( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int       *level )
{
     
    D_DEBUG( "XcodeGra0GetLevel enter \n" );

    *level = 0;

    D_DEBUG( "XcodeGra0GetLevel exit \n" );

    return DFB_OK;
}




/**********************************************************************************************************************/
static DFBResult
XcodeDummySetLevel( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int        level )
{
    return DFB_OK;
}

/**********************************************************************************************************************/


/**********************************************************************************************************************/

const DisplayLayerFuncs XcodeDummyLayerFuncs = {
     LayerDataSize:      XcodeVidLayerDataSize,
     InitLayer:          XcodeDummyInitLayer,
     TestRegion:         XcodeDummyTestRegion,
     SetRegion:          XcodeDummySetRegion,
     FlipRegion:         XcodeDummyFlipRegion,
     UpdateRegion:       XcodeDummyUpdateRegion,
     SetColorAdjustment: XcodeDummySetColorAdjustment,
     GetLevel:           XcodeDummyGetLevel,
     SetLevel:           XcodeDummySetLevel,
};

