#include <sys/ioctl.h>

/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_dummy_screen.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               of the dummy screen. Used in non-display related
*               board ID's.
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/
#include <config.h>

#include <asm/types.h>

#include <stdio.h>
#include <sys/mman.h>
#include <sys/ioctl.h>

#include <directfb.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>

#include <math.h>

#include <misc/conf.h>

#include <gfx/convert.h>

#include "viper.h"
#include "xcmmio.h"
#include "xcode4_osd.h"
#include "xcioctl.h"
#include "xcode4_adv7511.h"
#include "xcode4_hd_comp_venc.h"

/////////////////////////////////////////////////////

typedef u8 byte;
/* byte must be 8 bits */

/* int must be at least 16 bits */

/* long must be at least 32 bits */

#define UPPER_NIBBLE( x ) \
        (((128|64|32|16) & (x)) >> 4)

#define LOWER_NIBBLE( x ) \
        ((1|2|4|8) & (x))

#define COMBINE_HI_8LO( hi, lo ) \
        ( (((unsigned)hi) << 8) | (unsigned)lo )

#define COMBINE_HI_4LO( hi, lo ) \
        ( (((unsigned)hi) << 4) | (unsigned)lo )

#define EDID_LENGTH                             0x80

#define EDID_HEADER                             0x00
#define EDID_HEADER_END                         0x07

#define ID_MANUFACTURER_NAME                    0x08
#define ID_MANUFACTURER_NAME_END                0x09
#define ID_MODEL				0x0a

#define ID_SERIAL_NUMBER			0x0c

#define MANUFACTURE_WEEK			0x10
#define MANUFACTURE_YEAR			0x11

#define EDID_STRUCT_VERSION                     0x12
#define EDID_STRUCT_REVISION                    0x13

#define DPMS_FLAGS				0x18
#define ESTABLISHED_TIMING_1                    0x23
#define ESTABLISHED_TIMING_2                    0x24
#define MANUFACTURERS_TIMINGS                   0x25

/* Standard Timing Identification */
#define STANDARD_TIMING_ID_START		0x26
#define STANDARD_TIMING_ID_END			0x35

#define DETAILED_TIMING_DESCRIPTIONS_START      0x36
#define DETAILED_TIMING_DESCRIPTION_SIZE        18
#define NO_DETAILED_TIMING_DESCRIPTIONS         4

#define DETAILED_TIMING_DESCRIPTION_1           0x36
#define DETAILED_TIMING_DESCRIPTION_2           0x48
#define DETAILED_TIMING_DESCRIPTION_3           0x5a
#define DETAILED_TIMING_DESCRIPTION_4           0x6c

#define EXTENSION_FLAG				0x7e

#define PIXEL_CLOCK_LO     (unsigned)dtd[ 0 ]
#define PIXEL_CLOCK_HI     (unsigned)dtd[ 1 ]
#define PIXEL_CLOCK        (COMBINE_HI_8LO( PIXEL_CLOCK_HI,PIXEL_CLOCK_LO )*10000)

#define H_ACTIVE_LO        (unsigned)dtd[ 2 ]

#define H_BLANKING_LO      (unsigned)dtd[ 3 ]

#define H_ACTIVE_HI        UPPER_NIBBLE( (unsigned)dtd[ 4 ] )

#define H_ACTIVE           COMBINE_HI_8LO( H_ACTIVE_HI, H_ACTIVE_LO )

#define H_BLANKING_HI      LOWER_NIBBLE( (unsigned)dtd[ 4 ] )

#define H_BLANKING         COMBINE_HI_8LO( H_BLANKING_HI, H_BLANKING_LO )


#define V_ACTIVE_LO        (unsigned)dtd[ 5 ]

#define V_BLANKING_LO      (unsigned)dtd[ 6 ]

#define V_ACTIVE_HI        UPPER_NIBBLE( (unsigned)dtd[ 7 ] )

#define V_ACTIVE           COMBINE_HI_8LO( V_ACTIVE_HI, V_ACTIVE_LO )

#define V_BLANKING_HI      LOWER_NIBBLE( (unsigned)dtd[ 7 ] )

#define V_BLANKING         COMBINE_HI_8LO( V_BLANKING_HI, V_BLANKING_LO )


#define H_SYNC_OFFSET_LO   (unsigned)dtd[ 8 ]
#define H_SYNC_WIDTH_LO    (unsigned)dtd[ 9 ]

#define V_SYNC_OFFSET_LO   UPPER_NIBBLE( (unsigned)dtd[ 10 ] )
#define V_SYNC_WIDTH_LO    LOWER_NIBBLE( (unsigned)dtd[ 10 ] )

#define V_SYNC_WIDTH_HI    ((unsigned)dtd[ 11 ] & (1|2))
#define V_SYNC_OFFSET_HI   (((unsigned)dtd[ 11 ] & (4|8)) >> 2)

#define H_SYNC_WIDTH_HI    (((unsigned)dtd[ 11 ] & (16|32)) >> 4)
#define H_SYNC_OFFSET_HI   (((unsigned)dtd[ 11 ] & (64|128)) >> 6)


#define V_SYNC_WIDTH       COMBINE_HI_4LO( V_SYNC_WIDTH_HI, V_SYNC_WIDTH_LO )
#define V_SYNC_OFFSET      COMBINE_HI_4LO( V_SYNC_OFFSET_HI, V_SYNC_OFFSET_LO )

#define H_SYNC_WIDTH       COMBINE_HI_4LO( H_SYNC_WIDTH_HI, H_SYNC_WIDTH_LO )
#define H_SYNC_OFFSET      COMBINE_HI_4LO( H_SYNC_OFFSET_HI, H_SYNC_OFFSET_LO )

#define H_SIZE_LO          (unsigned)dtd[ 12 ]
#define V_SIZE_LO          (unsigned)dtd[ 13 ]

#define H_SIZE_HI          UPPER_NIBBLE( (unsigned)dtd[ 14 ] )
#define V_SIZE_HI          LOWER_NIBBLE( (unsigned)dtd[ 14 ] )

#define H_SIZE             COMBINE_HI_8LO( H_SIZE_HI, H_SIZE_LO )
#define V_SIZE             COMBINE_HI_8LO( V_SIZE_HI, V_SIZE_LO )

#define H_BORDER           (unsigned)dtd[ 15 ]
#define V_BORDER           (unsigned)dtd[ 16 ]

#define FLAGS              (unsigned)dtd[ 17 ]

#define INTERLACED         (FLAGS&128)
#define SYNC_TYPE	   (FLAGS&3<<3)  /* bits 4,3 */
#define SYNC_SEPARATE	   (3<<3)
#define HSYNC_POSITIVE	   (FLAGS & 4)
#define VSYNC_POSITIVE     (FLAGS & 2)

#define MONITOR_NAME            0xfc
#define MONITOR_LIMITS          0xfd
#define UNKNOWN_DESCRIPTOR      -1
#define DETAILED_TIMING_BLOCK   -2


#define DESCRIPTOR_DATA         5
#define V_MIN_RATE              block[ 5 ]
#define V_MAX_RATE              block[ 6 ]
#define H_MIN_RATE              block[ 7 ]
#define H_MAX_RATE              block[ 8 ]

#define MAX_PIXEL_CLOCK         (((int)block[ 9 ]) * 10)
#define GTF_SUPPORT		block[10]

#define DPMS_ACTIVE_OFF		(1 << 5)
#define DPMS_SUSPEND		(1 << 6)
#define DPMS_STANDBY		(1 << 7)

#define DISP_EDID

#ifdef DISP_EDID
#define DISP_EDID_DATA printf
#endif

/////////////////////////////////////////////////////

void Set_HD_Component_Hard_Code_720p(pxc_dfb_driver_data mdrv);

void Set_HD_Component_Hard_Code(pxc_dfb_driver_data mdrv);

void Set_HD_Component_Hard_Code_1080i(pxc_dfb_driver_data mdrv);

void ViperHDDACPowerUp(pxc_dfb_driver_data mdrv);

static u32 disp_hd_comp_register_table[SIZE_OF_DISP_COMP_REGISTER_TABLE];

extern void RESET_TOGGLE(pxc_dfb_driver_data mdrv, u32 reg_addr, u32 mask);

void Xcode4_Set_DVE_656_Output(pxc_dfb_driver_data mdrv,
                               struct display_mode_struct * mode,
                               struct dve_656_reg_struct * dve_656_reg_ptr);

void Xcode4_Set_HD_DVE_656_2_Output_Order(pxc_dfb_driver_data mdrv,
                               const DFBScreenOutputConfig *config);

int get_edid_data(pxc_dfb_driver_data mdrv, DFBScreenOutputEdidConfig *config);

extern void Set_Sd_Comp_As_HardCode(pxc_dfb_driver_data mdrv );

#define PRESET_LINES 1



//
//Timings based of CEA-861-D, includes borders with blanking interval
// 




/***********************************************************************************************/
/***********************************************************************************************/
/* 
*   If we have DSOR_1920_1080_3D_SIDE_BY_SIDE, DSOR_1920_1080 can also suffice, this 
*   way any mode that is implemented 1920x1080 (whether it is progressive or interlaced)
*   can be used as a side-by-side mode..
*/




/**********************************************************************************************************************/

static DFBResult
XcodeDummyInitScreen( CoreScreen           *screen,
                 CoreGraphicsDevice   *device,
                 void                 *driver_data,
                 void                 *screen_data,
                 DFBScreenDescription *description )
{
     u32 tmp;

     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    
     

     D_DEBUG( "XcodeInitScreen enter \n");

     /* Set the screen capabilities. */
     description->caps = DSCCAPS_VSYNC | DSCCAPS_ENCODERS | DSCCAPS_OUTPUTS;

     /* Set the screen name. */
     snprintf( description->name, DFB_SCREEN_DESC_NAME_LENGTH, "ViXS XCODE4 Screen" );

     /* Set number of encoders and outputs. */
     description->encoders = 1;
#ifndef  OUTPUT_INTHDMI_ONLY     
     description->outputs   = DISP_HD_OUTPUTS;
#else
     description->outputs   = DISP_HD_LIMIT_OUTPUTS;    
#endif     
     mdrv->shared->mode_index = INDEX_UNKNOWN;
     mdrv->shared->last_mode_index = INDEX_UNKNOWN;
     mdrv->shared->last_out_signals[DISP_HD_DVE656_2_OUTPUT]    = DSOS_NONE ;
     mdrv->shared->last_out_signals[DISP_HD_HDMI_OUTPUT]        = DSOS_NONE ;
     mdrv->shared->last_out_signals[DISP_HD_COMPONENT_OUTPUT]   = DSOS_NONE ;
     
     D_DEBUG( "XcodeInitScreen exit \n");
     return DFB_OK;
}

/**************************************************************************************************/

static DFBResult
Xcode4DummyTestEncoderConfig( CoreScreen                   *screen,
                        void                         *driver_data,
                        void                         *screen_data,
                        int                           encoder,
                        const DFBScreenEncoderConfig *config,
                        DFBScreenEncoderConfigFlags  *failed )
{

    DFBResult result=DFB_UNSUPPORTED;

    D_DEBUG( "Xcode4TestEncoderConfig enter \n");
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;


    return DFB_OK;
}

static DFBResult Xcode4DummySetEncoderConfig ( CoreScreen                   *screen,
                                          void                         *driver_data,
                                          void                         *screen_data,
                                          int                           encoder,
                                          const DFBScreenEncoderConfig *config )
{
    DFBResult result = DFB_UNSUPPORTED;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    u32 tmp;
    u32 bg_color_y, bg_color_cb, bg_color_cr;
    DFBConfigLayer *conf  = &dfb_config->layers[0];
    struct display_mode_struct * mode = NULL;

    D_DEBUG( "Xcode4SetEncoderConfig enter \n");
    D_DEBUG( "encoder tv_standard 0x%x\n", config->tv_standard);
    D_DEBUG( "encoder out_signals  0x%x \n", config->out_signals );
    D_DEBUG( "encoder out_connectors 0x%x  \n", config->out_connectors );
    D_DEBUG( "encoder slow_blanking  0x%x  \n", config->slow_blanking  );
    D_DEBUG( "encoder scanmode  0x%x \n", config->scanmode   );
    D_DEBUG( "encoder frequency  0x%x \n", config->frequency    );
    D_DEBUG( "encoder resolution 0x%x \n", config->resolution  );


    
    D_DEBUG( "Xcode4SetEncoderConfig exit \n");
    return DFB_OK;

}

/**************************************************************************************************/

static DFBResult
Xcode4DummyTestOutputConfig( CoreScreen                  *screen,
                        void                        *driver_data,
                        void                        *screen_data,
                        int                          output,
                        const DFBScreenOutputConfig *config,
                        DFBScreenOutputConfigFlags  *failed )
{

    DFBResult result = DFB_UNSUPPORTED;
    struct display_mode_struct * mode = NULL;

    D_DEBUG( "Xcode4TestOutputConfig enter \n");
    D_DEBUG( "config->resolution 0x%x  \n", config->resolution);
    D_DEBUG( "config->slow_blanking 0x%x  \n", config->slow_blanking);

    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    result = DFB_OK;
    
    D_DEBUG( "Xcode4TestOutputConfig exit \n");
    return result;
}

/**************************************************************************************************/

static DFBResult 
Xcode4DummySetOutputConfig ( CoreScreen                  *screen,
                        void                        *driver_data,
                        void                        *screen_data,
                        int                          output,
                        const DFBScreenOutputConfig *config )
{

    D_DEBUG( "Xcode4SetOutputConfig enter \n");

    DFBResult result = DFB_UNSUPPORTED;
    struct display_mode_struct * mode = NULL;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    struct dve_656_reg_struct dve_656_reg;
    u32 tmp, timeout, tmp_status;
    DFBScreenOutputConfig interlaced_config;
    int ret;
    
    D_DEBUG( "Xcode4SetOutputConfig enter \n");
    D_DEBUG( "output flags 0x%x\n", config->flags);
    D_DEBUG( "output encoder  0x%x \n", config->encoder );
    D_DEBUG( "output out_signals 0x%x  \n", config->out_signals );
    
    D_DEBUG( "output connectors  0x%x \n", config->out_connectors );
    D_DEBUG( "output resolution 0x%x \n", config->resolution );
    
    
    interlaced_config.flags                   = config->flags;
    interlaced_config.out_signals         = config->out_signals;
    interlaced_config.out_connectors  = config->out_connectors;
    interlaced_config.slow_blanking     = config->slow_blanking;  
    interlaced_config.resolution           = config->resolution;
    interlaced_config.output_order       = config->output_order;
    interlaced_config.bus_type             = config->bus_type;

    return DFB_OK;

}


/**************************************************************************************************/

static DFBResult
Xcode4DummyInitEncoder( CoreScreen                  *screen,
                  void                        *driver_data,
                  void                        *screen_data,
                  int                          encoder,
                  DFBScreenEncoderDescription *description,
                  DFBScreenEncoderConfig      *config )
{
    DFBResult result = DFB_UNSUPPORTED;
    app_ioctl_command dfb_ioctl_cmd;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    DFBConfigLayer *conf;
    int i;

    D_DEBUG( "Xcode4InitEncoder enter \n");

    //set the initialize display mode as CEA-861-D 
    config->flags = DSECONF_TV_STANDARD |
                    DSECONF_OUT_SIGNALS |
                    DSECONF_SCANMODE |
                    DSECONF_ADJUSTMENT |
                    DSECONF_FREQUENCY |
                    DSECONF_CONNECTORS  |
                    DSECONF_SLOW_BLANKING |
                    DSECONF_RESOLUTION ;

    config->tv_standard = DSETV_NTSC;
    config->out_signals = DSOS_YCBCR;
    config->out_connectors = DSOC_COMPONENT;

    if ((dfb_config->mode.width == 0) &&
        (dfb_config->mode.height == 0) &&
        (dfb_config->layers[XCODE4_VID1_LAYER_ID].init == true))
    {
        dfb_config->mode.width = 1920;
        dfb_config->mode.height = 1080;

        conf = &dfb_config->layers[XCODE4_VID1_LAYER_ID];
        conf->config.width  = DEFAULT_VID1_BUFFER_WIDTH;
        conf->config.height = DEFAULT_VID1_BUFFER_HEIGHT;
    }

    for (i = 0; i < D_ARRAY_SIZE(dfb_config->layers); i ++)
    {
        dfb_config->layers[i].background.color.a     = 0;
        dfb_config->layers[i].background.color.r     = 0;
        dfb_config->layers[i].background.color.g     = 0;
        dfb_config->layers[i].background.color.b     = 0;
        dfb_config->layers[i].background.color_index = -1;
        dfb_config->layers[i].background.mode        = DLBM_COLOR;
    }

    if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 480))
    {
        config->slow_blanking = DSOSB_4x3;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_720_480;
    }
    if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 481))
    {
        config->slow_blanking = DSOSB_16x9;
        config->scanmode = DSESM_PROGRESSIVE;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_720_480;
    }

    else if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 576))
    {
        config->slow_blanking = DSOSB_4x3;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_50HZ;
        config->resolution = DSOR_720_576;
    }
    else if ((dfb_config->mode.width == 1280) && (dfb_config->mode.height == 720))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_PROGRESSIVE;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_1280_720;
    }
    else if ((dfb_config->mode.width == 1280) && (dfb_config->mode.height == 721))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_PROGRESSIVE;
        config->frequency = DSEF_50HZ;
        config->resolution = DSOR_1280_720;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1080))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1081))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_50HZ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1082))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_PROGRESSIVE ;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1083))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_PROGRESSIVE ;
        config->frequency = DSEF_50HZ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 2205))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_PROGRESSIVE ;
        config->frequency = DSEF_24HZ;
        config->resolution = DSOR_1920_2205;
    }
    else
    {
        D_DEBUG( "Xcode4InitEncoder unsupport display mode \n");
    }
    
    
    
    
    /* Disable this part of code ... */
#if 0
    if (mdrv->fb0_handle != -1)
	{
        //
        //Inform FB driver to  layer region reset
        // 
        dfb_ioctl_cmd.uSize = sizeof(app_ioctl_command);
        dfb_ioctl_cmd.uIOCTLCommand = CMD_DFB_IOCTL_SET_DFB_VIDEO_START_PHYS_ADDR;
        if (dfb_config->extra_video_phys)
        {
            dfb_ioctl_cmd.video_buffer_info.dfb_start_phys_addr = dfb_config->extra_video_phys;
        }
        else
        {
            dfb_ioctl_cmd.video_buffer_info.dfb_start_phys_addr = dfb_config->video_phys;
        }
	    
        if (ioctl( mdrv->fb0_handle,CMD_APPLICATION_IOCTL , &dfb_ioctl_cmd ) )
        {
             D_ERROR("failed to inform FB driver of CMD_DFB_IOCTL_SET_DFB_VIDEO_START_PHYS_ADDR \n");
        }
	}
#endif

    D_DEBUG( "Xcode4InitEncoder exit \n");
    return DFB_OK;
}

/**************************************************************************************************/

static DFBResult
Xcode4DummyInitOutput( CoreScreen                 *screen,
                 void                       *driver_data,
                 void                       *screen_data,
                 int                         output,
                 DFBScreenOutputDescription *description,
                 DFBScreenOutputConfig      *config )
{
    D_DEBUG( "Xcode4InitOutput enter \n");

    DFBResult result = DFB_OK;
    /* Set the output capabilities. */
    description->caps = DSOCAPS_ALL;

    /* Set supported output connectors and signals. */
    description->all_connectors = DSOC_VGA | 
                                  DSOC_SCART |   
                                  DSOC_YC |  
                                  DSOC_CVBS |
                                  DSOC_SCART2 |   
                                  DSOC_COMPONENT |
                                  DSOC_HDMI;

    description->all_signals  = DSOS_YC |
    							DSOS_CVBS |
    							DSOS_YCBCR |
    							DSOS_HDMI |
                                DSOS_656;

    description->all_resolutions = DSOR_ALL;



    /* Set default configuration. */
    config->flags = DSOCONF_ALL;

    config->out_signals = DSOS_YCBCR;
    config->out_connectors = DSOC_COMPONENT;

    if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 480))
    {
        config->slow_blanking = DSOSB_4x3;
        config->resolution = DSOR_720_480;
    }
    else if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 576))
    {
        config->slow_blanking = DSOSB_4x3;
        config->resolution = DSOR_720_576;
    }
    else if ((dfb_config->mode.width == 1280) && (dfb_config->mode.height == 720))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1280_720;
    }
    else if ((dfb_config->mode.width == 1280) && (dfb_config->mode.height == 721))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1280_720;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1080))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1081))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1082))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1083))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1920_1080;
    }
    else
    {
        D_DEBUG( "Xcode4InitOutput unsupport output mode \n");
        result = DFB_UNSUPPORTED;
    }

    config->bus_type = DSBT_TRIPLE;
    config->output_order = DSSO_CBYCR;

    D_DEBUG( "Xcode4InitOutput exit \n");

    return result;
}

static DFBResult
Xcode4DummyGetScreenSize( CoreScreen *screen,
                     void       *driver_data,
                     void       *screen_data,
                     int        *ret_width,
                     int        *ret_height )
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    struct display_mode_struct * mode = NULL;
    DFBConfigLayer *conf;

    D_DEBUG( "Xcode4GetScreenSize enter \n");

    D_ASSERT( ret_width != NULL );
    D_ASSERT( ret_height != NULL );

    //if (mdrv->shared->mode_index != INDEX_UNKNOWN)
    //{
    //    mode = xc4_get_mode(mdrv->shared->mode_index);
    //    *ret_width  = mode->xres;
    //    *ret_height = mode->v_active;
    //}
    //else
   // {
    //    conf = dfb_config->config_layer;
        *ret_width  = 1920;
        *ret_height = 1080;
    //}

    D_DEBUG( "Xcode4GetScreenSize exit \n");

    return DFB_OK;
}


static DFBResult
Xcode4DummySetPowerMode( CoreScreen         *screen,
                   void               *driver_data,
                   void               *screen_data,
                   DFBScreenPowerMode  mode )
{
     
    return DFB_OK;
}






static DFBResult 
Xcode4DummyGetEdidOutputConfig ( CoreScreen                  *screen,
                    void                        *driver_data,
                    void                        *screen_data,
                    int                          output,
                    DFBScreenOutputEdidConfig *config )
{
    DFBResult result = DFB_UNSUPPORTED;

    return DFB_OK;
}


static DFBResult Xcode4DummySetBackgroundColor ( CoreScreen                  *screen,
                                            void                        *driver_data,
                                            void                        *screen_data,
									   u8                        red,
									   u8                        green,
                                       u8                        blue)
{
  
    return DFB_OK;
}

const ScreenFuncs XcodeDummyScreenFuncs = {
     InitScreen:    XcodeDummyInitScreen,
     TestEncoderConfig:    Xcode4DummyTestEncoderConfig,
     TestOutputConfig:    Xcode4DummyTestOutputConfig,
     InitEncoder:   Xcode4DummyInitEncoder,
     InitOutput:    Xcode4DummyInitOutput,
     SetEncoderConfig:   Xcode4DummySetEncoderConfig,
     SetOutputConfig:    Xcode4DummySetOutputConfig,
     GetScreenSize: Xcode4DummyGetScreenSize,
     SetPowerMode:  Xcode4DummySetPowerMode,
     GetEdidOutputConfig: Xcode4DummyGetEdidOutputConfig,
     SetBackgroundColor: Xcode4DummySetBackgroundColor
};

