
/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_sd_screen.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/
#include <config.h>

#include <asm/types.h>

#include <stdio.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>


#include <directfb.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>
#include <linux/fb.h>

#include <math.h>

#include <gfx/convert.h>

#include "viper.h"

#include "xcmmio.h"
#include "xcioctl.h"

#include "xcode4_osd.h"

extern int xc_find_mode_index(const DFBScreenEncoderConfig * config);

extern struct display_mode_struct * xc4_get_mode(int mode_index);


/**********************************************************************************************************************/

/**********************************************************************************************************************/
static DFBResult
XcodeDummySdInitScreen( CoreScreen           *screen,
                 CoreGraphicsDevice   *device,
                 void                 *driver_data,
                 void                 *screen_data,
                 DFBScreenDescription *description )
{
     u32 tmp;

     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "XcodeSdInitScreen enter \n");

     /* Set the screen capabilities. */
     description->caps = DSCCAPS_VSYNC | DSCCAPS_ENCODERS | DSCCAPS_OUTPUTS | DSCCAPS_POWER_MANAGEMENT;

     /* Set the screen name. */
     snprintf( description->name, DFB_SCREEN_DESC_NAME_LENGTH, "ViXS XCODE4 Sd Screen" );

     /* Set number of encoders and outputs. */
     description->encoders = 1;
     description->outputs  = 3;	    /* 3 outputs */
     mdrv->sd_mode_index = INDEX_UNKNOWN;
	 mdrv->shared->current_sd_power_mode = DSPM_OFF;

     D_DEBUG( "XcodeSdInitScreen exit \n");
     return DFB_OK;
}

/**************************************************************************************************/

static DFBResult
Xcode4DummySdTestEncoderConfig( CoreScreen                   *screen,
                        void                         *driver_data,
                        void                         *screen_data,
                        int                           encoder,
                        const DFBScreenEncoderConfig *config,
                        DFBScreenEncoderConfigFlags  *failed )
{

    DFBResult result=DFB_UNSUPPORTED;

    D_DEBUG( "Xcode4SdTestEncoderConfig enter \n");
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    if ((config->flags & DSECONF_SCANMODE ) &&
        (config->flags & DSECONF_OUT_SIGNALS) &&
        (config->flags & DSECONF_RESOLUTION) &&
        (config->flags & DSECONF_FREQUENCY))
    {
	     if ((config->resolution == DSOR_720_480 ) ||
		     (config->resolution == DSOR_720_576 ) ||
			 (config->resolution == DSOR_1440_480))
		 {
             //mdrv->sd_mode_index = xc_find_mode_index(config);

			 //if (mdrv->sd_mode_index != INDEX_UNKNOWN)
			 //{
			 //    result = DFB_OK;
			 //}

			 result = DFB_OK;

		 }
    }

    D_DEBUG( "Xcode4SdTestEncoderConfig exit \n");

    return result;
}

/**************************************************************************************************/

static DFBResult Xcode4DummySdSetEncoderConfig ( CoreScreen                   *screen,
                                          void                         *driver_data,
                                          void                         *screen_data,
                                          int                           encoder,
                                          const DFBScreenEncoderConfig *config )
{
    DFBResult result = DFB_UNSUPPORTED;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
	u32 bg_color_y, bg_color_cb, bg_color_cr;
    DFBConfigLayer *conf  = &dfb_config->layers[0];
	u32 tmp;
	app_ioctl_command	app_cmd;
	struct fb_var_screeninfo	var_info;


    D_DEBUG( "Xcode4SdSetEncoderConfig enter \n");
    D_DEBUG( "encoder tv_standard 0x%x\n", config->tv_standard);
    D_DEBUG( "encoder out_signals  0x%x \n", config->out_signals );
    D_DEBUG( "encoder out_connectors 0x%x  \n", config->out_connectors );
    D_DEBUG( "encoder slow_blanking  0x%x  \n", config->slow_blanking  );
    D_DEBUG( "encoder scanmode  0x%x \n", config->scanmode   );
    D_DEBUG( "encoder frequency  0x%x \n", config->frequency    );
    D_DEBUG( "encoder resolution 0x%x \n", config->resolution  );

    printf( "encoder slow_blanking  0x%x  \n", config->slow_blanking  );
    printf( "encoder scanmode  0x%x \n", config->scanmode   );
    printf( "encoder frequency  0x%x \n", config->frequency    );
    printf( "encoder resolution 0x%x \n", config->resolution  );
    
	//Enable the HD to SD compositing
	

    result = DFB_OK;
    
    D_DEBUG( "Xcode4SdSetEncoderConfig exit \n");
    return result;

}

/**************************************************************************************************/

static DFBResult
Xcode4DummySdTestOutputConfig( CoreScreen                  *screen,
                        void                        *driver_data,
                        void                        *screen_data,
                        int                          output,
                        const DFBScreenOutputConfig *config,
                        DFBScreenOutputConfigFlags  *failed )
{

     return DFB_OK;
}

/**************************************************************************************************/

static DFBResult 
Xcode4DummySdSetOutputConfig ( CoreScreen                  *screen,
                        void                        *driver_data,
                        void                        *screen_data,
                        int                          output,
                        const DFBScreenOutputConfig *config )
{

   D_DEBUG( "Xcode4SdSetOutputConfig enter \n");

   D_DEBUG( "Xcode4SdSetOutputConfig exit \n");
   return DFB_OK;

}


/**************************************************************************************************/

static DFBResult
Xcode4DummySdInitEncoder( CoreScreen                  *screen,
                  void                        *driver_data,
                  void                        *screen_data,
                  int                          encoder,
                  DFBScreenEncoderDescription *description,
                  DFBScreenEncoderConfig      *config )
{
    DFBResult result = DFB_UNSUPPORTED;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    D_DEBUG( "Xcode4SdInitEncoder enter \n");

    //set the initialize display mode as CEA-861-D 
    config->flags = DSECONF_TV_STANDARD |
                    DSECONF_OUT_SIGNALS |
                    DSECONF_SCANMODE |
                    DSECONF_ADJUSTMENT |
                    DSECONF_FREQUENCY |
                    DSECONF_CONNECTORS  |
                    DSECONF_SLOW_BLANKING |
                    DSECONF_RESOLUTION ;

    config->tv_standard = DSETV_NTSC;
    config->out_signals = DSOS_YCBCR;
    config->out_connectors = DSOC_COMPONENT;
    
 //   printf("dfb_config->mode.width = %d\n", dfb_config->mode.width);
 //   printf("dfb_config->mode.height = %d\n", dfb_config->mode.height);

    if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 480))
    {
        config->slow_blanking = DSOSB_4x3;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_720_480;
    }
    else if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 576))
    {
        config->slow_blanking = DSOSB_4x3;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_50HZ;
        config->resolution = DSOR_720_576;
    }
    else
    {
        D_DEBUG( "Xcode4InitEncoder unsupport display mode \n");
    }

//    printf( "encoder slow_blanking 0x%x \n", config->slow_blanking);
//    printf( "encoder scanmode  0x%x \n", config->scanmode   );
 //   printf( "encoder frequency  0x%x \n", config->frequency    );
 //   printf( "encoder resolution 0x%x \n", config->resolution  );
    
    mdrv->sd_mode_index = xc_find_mode_index(config);
    
  //  mdrv->sd_mode_index = 22;
//    printf("^^^^^^^^^^^ mdrv->sd_mode_index = %d\n", mdrv->sd_mode_index);

    if (mdrv->sd_mode_index != INDEX_UNKNOWN)
    {
        result = DFB_OK;
    }

	//Init_Disp_Sd_Comp_Register();

    D_DEBUG( "Xcode4SdInitEncoder exit \n");
    return result;
}

/**************************************************************************************************/

static DFBResult
Xcode4DummySdInitOutput( CoreScreen                 *screen,
                 void                       *driver_data,
                 void                       *screen_data,
                 int                         output,
                 DFBScreenOutputDescription *description,
                 DFBScreenOutputConfig      *config )
{
     DFBResult result = DFB_UNSUPPORTED;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     
     D_DEBUG( "Xcode4SdInitOutput enter \n");
     /* Initialize defaults for the output */
     config->flags	=	DSOCONF_SIGNALS		|
				DSOCONF_CONNECTORS	|
				DSOCONF_SLOW_BLANKING	|   
				DSOCONF_RESOLUTION	;

     config->out_signals=	DSOS_CVBS | DSOS_YCBCR; 
     config->out_connectors =	DSOC_CVBS;  
     if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 480))
     {
         config->slow_blanking = DSOSB_4x3;
         config->resolution = DSOR_720_480;
     }
     else if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 576))
     {
         config->slow_blanking = DSOSB_4x3;
         config->resolution = DSOR_720_576;
     }
     else
     {
         D_DEBUG( "Xcode4InitOutput unsupported display mode on output!! \n");
     }
     

 



     D_DEBUG( "Xcode4SdInitOutput exit \n");

     return DFB_OK;
}

static DFBResult
Xcode4DummySdGetScreenSize( CoreScreen *screen,
                     void       *driver_data,
                     void       *screen_data,
                     int        *ret_width,
                     int        *ret_height )
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    struct display_mode_struct * mode = NULL;
    DFBConfigLayer *conf;

    D_DEBUG( "Xcode4SdGetScreenSize enter \n");

    D_ASSERT( ret_width != NULL );
    D_ASSERT( ret_height != NULL );

    if (mdrv->sd_mode_index != INDEX_UNKNOWN)
    {
        mode = xc4_get_mode(mdrv->sd_mode_index);
        *ret_width  = mode->xres;
        *ret_height = mode->v_active;
    }
    else
    {
        conf = dfb_config->config_layer;
        *ret_width  = conf->config.width;
        *ret_height = conf->config.height;
    }

    D_DEBUG( "Xcode4SdGetScreenSize exit \n");

    return DFB_OK;
}


/**************************************************************************************************/

static DFBResult
Xc4DummySdSetPowerMode( CoreScreen         *screen,
                   void               *driver_data,
                   void               *screen_data,
                   DFBScreenPowerMode  mode )
{
     
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
	u32 tmp;
	app_ioctl_command	app_cmd;
    
    return DFB_OK;
}

static DFBResult Xcode4DummySdSetBackgroundColor ( CoreScreen                  *screen,
                                            void                        *driver_data,
                                            void                        *screen_data,
									   u8                        red,
									   u8                        green,
                                       u8                        blue)
{
    u32 bg_color_y, bg_color_cb, bg_color_cr;
    u32 tmp;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    D_DEBUG( "Xcode4SdSetBackgroundColor enter \n");
    D_DEBUG( "Xcode4SdSetBackgroundColor exit \n");

    return DFB_OK;
}

const ScreenFuncs XcodeDummySdScreenFuncs = {
     InitScreen:    XcodeDummySdInitScreen,
     TestEncoderConfig:    Xcode4DummySdTestEncoderConfig,
     TestOutputConfig:    Xcode4DummySdTestOutputConfig,
     InitEncoder:   Xcode4DummySdInitEncoder,
     InitOutput:    Xcode4DummySdInitOutput,
     SetEncoderConfig:   Xcode4DummySdSetEncoderConfig,
     SetOutputConfig:    Xcode4DummySdSetOutputConfig,
     GetScreenSize: Xcode4DummySdGetScreenSize,
     SetPowerMode:  Xc4DummySdSetPowerMode,
     SetBackgroundColor: Xcode4DummySdSetBackgroundColor
};



































