/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_video_vid0.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/


#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include <pthread.h>

#include <sys/mman.h>
#include <fcntl.h>
 #include <sys/ioctl.h>


#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"

extern void Wait_For_Fifo_Entries(pxc_dfb_driver_data mdrv,
                           u32 status_reg, 
                           u32 free_ent_mask, 
                           u32 free_ent_shift,
                           u32 requested_fifo_space);

extern double my_round(double x, int N, int I);

/*************************************************************************************************/
u32 Convert_DFB_Format_To_Fes_mode(DFBSurfacePixelFormat dfb_format, 
                                   DFBSurfaceCapabilities dfb_surface_caps,
                                   u32 src_mode)
{
    u32 ret_mode = 0;

    if (src_mode)
    {
        ret_mode = FES1_ARGB_ORDER;
    }

    switch(dfb_format)
    {
        case DSPF_I420:
        case DSPF_YV12:
            if (dfb_surface_caps & DSCAPS_VIXS_INTERLACED)
            {
                ret_mode |= (FES_YUV_420 << FES1_SRC1_PLANAR_MODE_SHIFT) | FES_YUV_8_BIT_INTERLACED ;
            }
            else
            {
                ret_mode |= (FES_YUV_420 << FES1_SRC1_PLANAR_MODE_SHIFT) | FES_YUV_8_BIT_PROGRESSIVE ;
            }
            break;

        case DSPF_YV16: //replaced DSPF_UYVY
        case DSPF_YUY2: //TODO: not consistent with DirectFB pixel format.
            if (dfb_surface_caps & DSCAPS_VIXS_INTERLACED)
            {
                ret_mode |= (FES_YUV_422 << FES1_SRC1_PLANAR_MODE_SHIFT) | FES_YUV_8_BIT_INTERLACED ;
            }
            else
            {
                ret_mode |= (FES_YUV_422 << FES1_SRC1_PLANAR_MODE_SHIFT) | FES_YUV_8_BIT_PROGRESSIVE ;
            }
            break;

        case DSPF_RGB555:
            ret_mode |= FES_RGB555;
            break;

        case DSPF_ARGB1555:
            ret_mode |= FES_ARGB1555;
            break;

        case DSPF_ARGB4444:
            ret_mode |= FES_RGB4444;
            break;

        case DSPF_ARGB:
        case DSPF_RGB32:
            ret_mode |= FES_ARGB8888;
            break;

        default:
            break;
    }

    return ret_mode;
    
}


/*****************************************************************************************************************
Function Name: set_fes1_coefficient_table

*****************************************************************************************************************/
void set_fes1_coefficient_table(u32 base_address, double htable[][FES1_HORIZONAL_SCALER_TAPS], int upscaling_fractor_L, int ntaps, pxc_dfb_driver_data mdrv)
{
    u32 coef[10];
    int Lmirrored = (upscaling_fractor_L / 2) + 1;
    int i;
    int j;
    int h;
	u32 word0;
	u32 word1;
	u32 word2;
  
    Wait_For_Fes1_Fifo(1);
    REG_WRITE(FES1_CONFIG_CTRL, base_address);

    for(i = 0; i < Lmirrored; i++)
    {
        for(j = 0; j < ntaps; j++)
        {

            if(ntaps == 10)
            {
                if(j<=1 || j>=8)
                {
                    h = (int)(my_round(htable[i][j], 12, 1)*pow(2.,8+3-1));
                }
                else if(j<=2 || j>=7)
                {
                    h = (int)(my_round(htable[i][j], 12, 1)*pow(2.,9+2-1));
                }
                else if(j<=3 || j>=6)
                {
                    h = (int)(my_round(htable[i][j], 12, 1)*pow(2.,10+1-1));
                }
                else
                {
                    h = (int)(my_round(htable[i][j],12, 1)*pow(2.,12-1-1));
                }
            }
            else
            {
                if(j<=0 || j>=5)
                {
                    h = (int)(my_round(htable[i][j], 12, 1)*pow(2.,9+2-1));
                }
                else if(j<=1 || j>=4)
                {
                    h = (int)(my_round(htable[i][j], 12, 1)*pow(2.,10+1-1));
                }
                else
                {
                    h = (int)(my_round(htable[i][j],12, 1)*pow(2.,12-1-1));
                }
            }
            coef[j] = h;
        }
  
        if(ntaps == 6)
        {
            word0 = ((coef[2]&0x1)<<31) + ((coef[3]&0xFFF)<<19) + ((coef[4]&0x3FF)<<9) + (coef[5]&0x1FF);
            word1 = ((coef[0]&0x1FF)<<21) + ((coef[1]&0x3FF)<<11) + ((coef[2]&0xFFF)>>1);
            Wait_For_Fes1_Fifo(2);
            REG_WRITE (FES1_LUT_PROGM, word0);
            REG_WRITE (FES1_LUT_PROGM, word1);
        }
        else if(ntaps == 10)
        {
            word0 = ((coef[6]&0x7f)<<25) + ((coef[7]&0x1FF)<<16) + ((coef[8]&0xFF)<<8)  + (coef[9]&0xFF);       
            word1 = ((coef[3]&0x1f)<<27) + ((coef[4]&0xFFF)<<15) + ((coef[5]&0xFFF)<<3) + ((coef[6]&0x380)>>7); 
            word2 = ((coef[0]&0xff)<<22) + ((coef[1]&0xFF) <<14) + ((coef[2]&0x1FF)<<5) + ((coef[3]&0x3E0)>>5); 
            Wait_For_Fes1_Fifo(3);
            REG_WRITE (FES1_LUT_PROGM, word0);
            REG_WRITE (FES1_LUT_PROGM, word1);
            REG_WRITE (FES1_LUT_PROGM, word2);
        }
    }
}

/*************************************************************************************************/

