

/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_gfxdriver.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
* $Log: not supported by cvs2svn $
* Revision 1.9  2010/10/24 23:11:40  lichen
* add bg layer support
*
* Revision 1.8  2010/10/23 01:39:58  lichen
* no message
*
* Revision 1.7  2010/09/22 21:49:25  lichen
* add top gra0 and gra1 layer
*
* Revision 1.6  2010/08/20 21:39:21  lichen
* switch lut for graphic layer in vblank
*
* Revision 1.5  2010/08/18 15:34:24  ibosompem
* -Disables SD mirroring on startup.
*
* Revision 1.4  2010/08/11 15:25:20  ibosompem
* -Fixed minor bug in SVideo/Component co-existance.
* -Fixed minor bug in SD mirroring handling.
*
* Revision 1.3  2010/08/10 19:29:35  lichen
* add register table initialization for dfb mulit core
*
* Revision 1.2  2010/08/06 15:13:19  ibosompem
* -Added HD TO SD mirroring toggle control in DFB driver.
* -Added fixes for sync polarities in HDMI 3D frame packing output.
*
* Revision 1.1.1.1  2010/02/25 00:13:08  lichen
* no message
*
* Revision 1.10  2010/01/18 20:07:43  lichen
* add fes blit feature for yuv to yuv
*
* Revision 1.9  2009/12/23 15:08:41  lichen
* set the surface byteoffset alignment as 0x2000
*
* Revision 1.8  2009/12/14 19:21:24  lichen
* adding virtual layer
*
* Revision 1.7  2009/11/26 20:31:03  lichen
* move xc3fb_if.h to xc4fb_if.h
*
* Revision 1.6  2009/10/21 13:55:14  lichen
* generate coefficients for vid1
*
* Revision 1.5  2009/09/22 15:40:05  lichen
* C_PITCH size align as 16 bytes on video layer
*
* Revision 1.4  2009/09/11 15:11:29  lichen
* support new graphic and video fifo
*
* Revision 1.3  2009/08/12 22:07:36  lichen
* sd graphic pipe can work
*
* Revision 1.2  2009/08/04 14:12:00  lichen
* gra0 and vid0 work for 480i, 480p, 720p, 1080i
*
* Revision 1.1  2009/04/03 22:11:43  lichen
* port Directfb 1.3 to arc, add xcode4 driver
*
* Revision 1.18  2009/04/03 16:10:24  lichen
* save for moving to 1.3.0
*
* Revision 1.9  2008/10/23 20:33:48  lichen
* adjust the blit function
*
* Revision 1.6  2008/10/21 17:56:18  lichen
* adding color alpha for XCODE4 Blit
*
* Revision 1.1  2008/10/10 14:39:39  lichen
* for adding gfxdriver vixs_xcode4
*
* Revision 1.1  2008/10/08 21:05:45  jdoyle
* Initial version
*
***************************************************/

/** include files **/
#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <dlfcn.h>


#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/fb.h>
#include <fusion/arena.h>
#include <fusion/fusion.h>
#include <fusion/reactor.h>
#include <fusion/shmalloc.h>

#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/core.h>
#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include <core/graphics_driver.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"
#include "xcioctl.h"

extern void Init_Disp_Hd_Comp_Register();
extern void Init_Disp_Gra0_Register_Table();
extern void Init_Disp_Vid0_Register_Table();
extern void Init_Disp_Gra1_Register_Table();
extern void Init_Disp_Vid1_Register_Table();
extern void Init_Disp_Sd_Comp_Register();

extern ScreenFuncs XcodeScreenFuncs;
extern ScreenFuncs XcodeDummyScreenFuncs;
extern ScreenFuncs XcodeDummySdScreenFuncs;
extern DisplayLayerFuncs XcodeGra0LayerFuncs;
extern DisplayLayerFuncs XcodeGra1LayerFuncs;
extern DisplayLayerFuncs XcodeVid0LayerFuncs;
extern DisplayLayerFuncs XcodeVid1LayerFuncs;
extern DisplayLayerFuncs XcodeBackLayerFuncs;
extern DisplayLayerFuncs Xc4BglGraLayerFuncs;
extern DisplayLayerFuncs XcodeVirVidLayerFuncs;
extern DisplayLayerFuncs Xc4TopGra0LayerFuncs;
extern DisplayLayerFuncs Xc4TopGra1LayerFuncs;
extern DisplayLayerFuncs XcodeDummyLayerFuncs;

extern DisplayLayerFuncs XcodeHdVbi0LayerFuncs;
extern DisplayLayerFuncs XcodeSdVbi0LayerFuncs;

extern ScreenFuncs XcodeSdScreenFuncs;
extern DisplayLayerFuncs XcodeSdGra0LayerFuncs;

static int
driver_probe( CoreGraphicsDevice *device );


/** public data **/

/** local definitions **/
DFB_GRAPHICS_DRIVER( xcode )

/*
 *
 *Function Name:driver_probe
 *
 *Parameters: 
 *             CoreGraphicsDevice->Framebuffer device
 *
 *Description:
 *             Check the framebuffer device to determine if its ours
 *             
 *Returns:
 *             Returns TRUE if our device, otherwise FALSE
 *
*/
static int
driver_probe( CoreGraphicsDevice *device )
{
     int  iRetVal=0;

     D_DEBUG( "xcode DFB driver_probe enter:\n" );

     switch (dfb_gfxcard_get_accelerator( device ))
     {
          case FB_ACCEL_VIXS_XC4:
          {
               iRetVal=1;
          }
     }

     D_DEBUG( "xcode DFB driver_probe exit %x:\n",iRetVal );

     return iRetVal;
}


/*
 *
 *Function Name:driver_get_info
 *
 *Parameters: 
 *             CoreGraphicsDevice -> Framebuffer device
 *             GraphicsDriverInfo -> DirectFB driver info structure
 *Description:
 *             Gets the driver specific information(name, vendor etc)
 *Returns:
 *             Driver must fill out the relevant fields of the info structure
 *
*/

static void
driver_get_info( CoreGraphicsDevice     *device,
                 GraphicsDriverInfo *info )
{
     D_DEBUG( "xcode DFB driver_get_info enter:\n");

     /* fill driver info structure */
     snprintf( info->name,
               DFB_GRAPHICS_DRIVER_INFO_NAME_LENGTH,
               XC_DFB_DRIVER_NAME );

     snprintf( info->vendor,
               DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH,
               XC_DFB_DRIVER_VENDOR);

     info->version.major = XC_DFB_DRV_VERSION_MAJ;
     info->version.minor = XC_DFB_DRV_VERSION_MIN;

     info->driver_data_size = sizeof (xc_driver_data);
     info->device_data_size = sizeof (xc_device_data);

     D_DEBUG( "xcode DFB driver_get_info exit:\n");

}


/*
 *
 *Function Name:driver_init_driver
 *
 *Parameters: 
 *             CoreGraphicsDevice -> Framebuffer device
 *             GraphicsDeviceFuncs -> DirectFB driver functions structure
 *             driver_data -> driver specific structure
 *             device_data -> device specific structure
 *             core -> DirectFB core object
 *Description:
 *             Fills out the driver specific functions and maps in the MMIO
 *
 *Returns:
 *             DFB_OK
*/

static DFBResult
driver_init_driver( CoreGraphicsDevice      *device,
                    GraphicsDeviceFuncs *funcs,
                    void                *driver_data,
                    void                *device_data,
                    CoreDFB             *core )
{

     DFBResult      result = DFB_OK;
     XC4Shared     *shared = NULL;
     FusionSHMPoolShared *pool;
     FusionWorld   *world;
     app_ioctl_command     app_cmd;
     u32 rbm_id, board_id;
     void   *hdcp_handle;
     
     struct fb_var_screeninfo vinfo;
     struct fb_fix_screeninfo finfo;
     
     void *mmap_ret;
     
     /* Function pointer prototype ... */
     load_HDCP_key  load_HDCP;
     int hdcp_ret;
     
     u32 *r_array, j;
     double accumulator;

     D_DEBUG( "xcode DFB driver_init_driver enter:\n");
    
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_device_data mdev = (pxc_dfb_device_data) device_data;
     
     Xcode4InitAuxOSDFunctions();
     //check if the core is master
     if (dfb_core_is_master(core))
     {
          pthread_mutexattr_t mutex_attr;

          //set master owner
          mdrv->master = 1;

          pool = dfb_core_shmpool( core );

          //allocate the xcode4 shared region
          shared = (XC4Shared*) SHCALLOC( pool, 1, sizeof(XC4Shared) );

          memset (&(shared->vir_gra_gic_ctl), 0, sizeof (gic_opt_struct));

          /* allocate mutex to prevent simultaneous access to shared HW blocks */
          pthread_mutexattr_init( &mutex_attr );
          pthread_mutexattr_setpshared( &mutex_attr, PTHREAD_PROCESS_SHARED );
          if ( pthread_mutex_init( &shared->reentrancy_mutex, &mutex_attr ) )
          {
               D_ERROR( "could not allocate re-entrancy mutex\n" );
               SHFREE( pool, shared );
               return DFB_INIT;
          }
          if ( pthread_mutex_init( &shared->g2d_mutex, &mutex_attr) )
          {
               D_ERROR( "could not allocate g2d mutex\n" );
               SHFREE( pool, shared );
               return DFB_INIT;
          }
          if ( pthread_mutex_init( &shared->virtual_layer_mutex, &mutex_attr) )
          {
               D_ERROR( "could not allocate virtual layer mutex\n" );
               SHFREE( pool, shared );
               return DFB_INIT;
          }

          world = dfb_core_world( core );
          if ( fusion_skirmish_init(&shared->img_hw_lock, "xcode4 image hw decoder lock",world) )
          {
               D_ERROR( "could not allocate hw image decoder lock\n" );
               pthread_mutex_destroy( &shared->reentrancy_mutex );
               SHFREE( pool, shared );
               return DFB_INIT;
          }

          //add the shared filed 
          fusion_arena_add_shared_field( dfb_core_arena( core ), "xcode4", shared );

          /* Initialize ... */
          shared->last_output_init     =   0xFFFFFFFE;
     }
     else
     {
          //get the shared field 
          fusion_arena_get_shared_field( dfb_core_arena( core ),
                         "xcode4", (void **)&shared );

          mdrv->master = 0;

          Init_Disp_Hd_Comp_Register();
          Init_Disp_Gra0_Register_Table();
          Init_Disp_Vid0_Register_Table();
          Init_Disp_Gra1_Register_Table();
          Init_Disp_Vid1_Register_Table();
          Init_Disp_Sd_Comp_Register();
     }

     mdrv->shared = shared;
     mdrv->hdtosd_fd     =   -1;
     mdrv->dfb_core = core;
     mdrv->hd_vid0_pending_ptr = NULL;

     //map the xcode4 register address
     mdrv->mmio_base = dfb_gfxcard_map_mmio( device, 0, -1 ); 

     /* Map the aperture */
     mdrv->mmaper_base = dfb_gfxcard_memory_virtual(device,0);

     D_DEBUG( "xcode DFB driver mmio = %08x \n", mdrv->mmio_base );
     
     mdrv->hdtosd_fd = open(HDTOSD_DEV, O_RDWR);
     if (mdrv->hdtosd_fd == -1)
     {
          D_DEBUG("Could not open HDTOSD FB driver. SD mirroring disabled.\n");
     }

     mdrv->fb0_handle     =   -1;
     mdrv->fb0_handle     =   open(FB0_DEV, O_RDWR);
     if (mdrv->fb0_handle == -1)
     {
         D_DEBUG("Could not open fb0 FB driver.\n");
     }
     else
     {
        /* Obtain framebuffer information (in the case we use it) */
        ioctl(mdrv->fb0_handle, FBIOGET_VSCREENINFO, &vinfo);
        ioctl(mdrv->fb0_handle, FBIOGET_FSCREENINFO, &finfo);
        
        mdev->fb_phys   =   finfo.smem_start;
        D_DEBUG("fb_phys = 0x%X\n", mdev->fb_phys);
        mdev->fb_size   =   finfo.smem_len;
        D_DEBUG("fb_size = 0x%X\n", mdev->fb_size);
     
     }

     if(mdrv->master)
     {
          mdrv->shared->current_sd_power_mode = DSPM_OFF;
     }
     
     if (mdrv->mmio_base)
     {
          D_DEBUG( "Board ID = %x\n", REG_READ(RBM_BOARD_ID)  );
          
          board_id  =   REG_READ(CG_DUMMY_REG);
          
          if (board_id == 0x1A01 || board_id == 0x1A03 || board_id == 0x1A05)
          {
            /* Profile 1 - Enable video output ... */
            mdrv->pwr_flags =   0;
          }
          else
          {
            mdrv->pwr_flags =   0;
          }
          
          mdrv->pdevice_data = (pxc_dfb_device_data) device_data;
     
          mdrv->accelerator = dfb_gfxcard_get_accelerator( device );
          
          if (mdrv->pwr_flags == 0)
          {
              /* Normal function register ... */
              funcs->EngineReset   = xcodeEngineReset;
              funcs->EngineSync    = xcodeEngineSync;
              funcs->CheckState    = xcodeCheckState;
              funcs->SetState      = xcodeSetState;
              funcs->FillRectangle = xcodeFillRectangle;
              funcs->Blit          = xcodeBlit;
              funcs->StretchBlit   = xcodeStretchBlit;
              
              mdrv->screen = dfb_screens_register( device, driver_data, &XcodeScreenFuncs );
              mdrv->sdscreen = dfb_screens_register( device, driver_data, &XcodeSdScreenFuncs );

              dfb_layers_register( mdrv->screen, driver_data, &XcodeGra0LayerFuncs );
              dfb_layers_register( mdrv->screen, driver_data, &XcodeVid0LayerFuncs );
              dfb_layers_register( mdrv->screen, driver_data, &XcodeGra1LayerFuncs );
              dfb_layers_register( mdrv->screen, driver_data, &XcodeVid1LayerFuncs );
              dfb_layers_register( mdrv->screen, driver_data, &XcodeBackLayerFuncs );
              dfb_layers_register( mdrv->screen, driver_data, &XcodeDummyLayerFuncs );
              dfb_layers_register( mdrv->screen, driver_data, &Xc4TopGra0LayerFuncs );
              dfb_layers_register( mdrv->screen, driver_data, &Xc4TopGra1LayerFuncs );
              dfb_layers_register( mdrv->sdscreen, driver_data, &XcodeSdGra0LayerFuncs );
              dfb_layers_register( mdrv->screen, driver_data, &XcodeHdVbi0LayerFuncs );
              dfb_layers_register( mdrv->sdscreen, driver_data, &XcodeSdVbi0LayerFuncs );

          }
          else
          {
            /* Register our dummy functions in the case we are completely shut off ... */
            funcs->EngineReset   = xcodeDummyEngineReset;
            funcs->EngineSync    = xcodeDummyEngineSync;
            funcs->CheckState    = xcodeDummyCheckState;
            funcs->SetState      = xcodeDummySetState;
            funcs->FillRectangle = xcodeDummyFillRectangle;
            funcs->Blit          = xcodeDummyBlit;
            funcs->StretchBlit   = xcodeDummyStretchBlit;
            
            mdrv->screen = dfb_screens_register( device, driver_data, &XcodeDummyScreenFuncs );
            mdrv->sdscreen = dfb_screens_register( device, driver_data, &XcodeDummySdScreenFuncs );
            
            dfb_layers_register( mdrv->sdscreen, driver_data, &XcodeBackLayerFuncs );
            
            
            dfb_layers_register( mdrv->screen, driver_data, &XcodeBackLayerFuncs );

          }

          mdrv->asic_flags = VIPER_ASIC_A21;
     }
     else 
     {
          result = DFB_IO;
          D_BUG( "no mmio found\n");
     }
 
     mdev->src_color_r          =     0;
     mdev->src_color_g          =     0;
     mdev->src_color_b          =     0;
     mdev->src_color_alpha     =     0;
     mdev->src_color_update     =     0;

#ifdef HDCP_KEY_AUTOLOAD     
     if (dfb_config->hdcp_mode != 1)
     {
          hdcp_handle = dlopen("libhdcp.so", RTLD_NOW);
          hdcp_ret   =   1;
          if (!hdcp_handle)
          {
               D_INFO("libhdcp.so NOT FOUND! HDCP keys not loaded!!\n");
          }
          else
          {
               /* Loaded, attempt to get the symbol.. */
               load_HDCP = (load_HDCP_key)dlsym(hdcp_handle, "load_HDCP_key");
               if (!load_HDCP)
               {
                    D_INFO("Failed to load HDCP keys...\n");
               }
               else
               {
                    hdcp_ret = load_HDCP();
                    dlclose(hdcp_handle);
               }
          }

          if (hdcp_ret == 0)
          {
               D_INFO("HDCP keys loaded successfully.\n");
          }
          else
          {
               D_INFO("Failed to load HDCP keys...\n");
         }
     }
#endif
    
     D_DEBUG( "xcode DFB driver_init_driver exit:\n");

     return result;
}


/*
 *
 *Function Name:driver_init_device
 *
 *Parameters: 
 *             CoreGraphicsDevice -> Framebuffer device
 *             CoreGraphicsDeviceInfo -> DirectFB device capabilities
 *             driver_data -> driver specific structure
 *             device_data -> device specific structure
 *Description:
 *             Fills out the device specific functions(BLT etc) and capabilities(colorkey etc)
 *
 *Returns:
 *             DFB_OK
*/

static DFBResult
driver_init_device( CoreGraphicsDevice     *device,
                    GraphicsDeviceInfo *device_info,
                    void               *driver_data,
                    void               *device_data )
{
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_device_data mdev = (pxc_dfb_device_data) device_data;
     u32 tmp;

     D_DEBUG( "xcode DFB driver_init_device enter:\n");



     snprintf( device_info->name,
               DFB_GRAPHICS_DRIVER_INFO_NAME_LENGTH,
               XC_FOUR_DFB_DEVICE_NAME );


     snprintf( device_info->vendor,
               DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH,
               XC_DFB_DRIVER_VENDOR);

     /* fill device info */
     device_info->caps.flags = CCF_CLIPPING | CCF_READSYSMEM;

     device_info->caps.drawing  = XC4_SUPPORTED_DRAWINGFLAGS;

     device_info->caps.blitting = XC4_SUPPORTED_BLITTINGFLAGS;

     device_info->caps.accel    = XC4_SUPPORTED_DRAWINGFUNCTIONS |
                                  XC4_SUPPORTED_BLITTINGFUNCTIONS;

     device_info->limits.surface_byteoffset_alignment = 0x4000;
     device_info->limits.surface_bytepitch_alignment  = Y_BYTE_ALIGNMENT;
     device_info->limits.surface_pixelpitch_alignment = 1;

     DISP_SET_POWER_START;

     G2D_SET_POWER_START;

     GIC_SET_POWER_START;
    
    
     //u32 last_temp_physical;
     //u32 last_temp_size;
     /* Set both to 0 to initialize ... */
     mdev->last_temp_physical    = 0;
     mdev->last_temp_size        = 0;
     mdev->last_temp_position    = 0;

     /* Allocate enough memory for a full HD temporary render surface... */
     mdev->temp_render_offset   =   dfb_gfxcard_reserve_memory(device, TEMP_RENDER_SIZE + TEMP_RENDER_EXTRA_SIZE + TEMP_RENDER_LINE_SIZE);

     /* Set to 0 */
     mdev->temp_render_physical = 0;
     mdev->last_text_flags   =   0;
     mdev->last_text_r       =   0;
     mdev->last_text_g       =   0;
     mdev->last_text_b       =   0;
     mdev->last_dest_caps    =   0;

     if (mdev->temp_render_offset == -1)
     {
        /* ERROR, not fatal though, shouldnt happen at all... */
        D_DEBUG ( "Failed to reserve temporary rendering buffer of size %u\n", TEMP_RENDER_SIZE);
     }
     else
     {
        /* Convert the offset into a physical address */
        mdev->temp_render_physical = dfb_gfxcard_memory_physical(device, mdev->temp_render_offset);
        mdev->temp_render_logical  = (u32)(mdrv->mmaper_base + mdev->temp_render_offset);
        mdev->temp_render_extra_physical = mdev->temp_render_physical + TEMP_RENDER_SIZE;
        mdev->temp_render_line_physical = mdev->temp_render_extra_physical + TEMP_RENDER_EXTRA_SIZE;
        
        D_DEBUG ( "Successfully reserved temporary rendering buffer of size %u at offset 0x%X (phys: 0x%X, log: 0x%X)\n",
                    TEMP_RENDER_SIZE, mdev->temp_render_offset, mdev->temp_render_physical,
                    mdev->temp_render_logical);
     }

     D_DEBUG( "xcode DFB driver_init_device exit:\n");

     return DFB_OK;
}


/*
 *
 *Function Name:driver_close_device
 *
 *Parameters: 
 *             CoreGraphicsDevice -> Framebuffer device
 *             driver_data -> driver specific structure
 *             device_data -> device specific structure
 *Description:
 *             Perform any cleanup or statistic loggging before device is shutdown.
 *
 *Returns:
 *             NA
*/

static void
driver_close_device( CoreGraphicsDevice *device,
                     void           *driver_data,
                     void           *device_data )
{
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_device_data mdev = (pxc_dfb_device_data) device_data;
     app_ioctl_command     app_cmd;
     u32 tmp;
     D_DEBUG( "xcode DFB driver_close_device enter:\n");
     
     if (mdrv->hdtosd_fd != -1)
     {
          app_cmd.uIOCTLCommand  =   CMD_DFB_IOCTL_DISABLE_HD2SD;
          ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd);

          /* Close HDTOSD FD in case it is open */
          close(mdrv->hdtosd_fd);
     }

     if (mdrv->fb0_handle != -1)
     {
         /* Close HDTOSD FD in case it is open */
         close(mdrv->fb0_handle);
     }     

     D_DEBUG( "xcode DFB driver_close_device exit:\n");
}

/*
 *
 *Function Name:driver_close_driver
 *
 *Parameters: 
 *             CoreGraphicsDevice -> Framebuffer device
 *             driver_data -> driver specific structure
 *Description:
 *             Perform any cleanup or statistic loggging before driver is unloaded.
 *
 *Returns:
 *             NA
*/

static void
driver_close_driver( CoreGraphicsDevice *device,
                     void           *driver_data )
{
     D_DEBUG( "xcode DFB driver_close_driver enter:\n");

     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     /* free re-entrancy mutex */
     if ( mdrv->master )
     {
         fusion_skirmish_destroy (&mdrv->shared->img_hw_lock);
         pthread_mutex_destroy( &mdrv->shared->reentrancy_mutex );
     }

     /*unmap MMIO*/
     dfb_gfxcard_unmap_mmio( device, mdrv->mmio_base, -1 );

     D_DEBUG( "xcode DFB driver_close_driver exit:\n");

}

