/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_screen.c  
*
*  DESCRIPTION: Contains Xcode-IIII Component output VENC driver  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/

#include "xcode4_hd_comp_venc.h"


static venc_table_entry_t venc_script_table[] = {
    {DSESM_PROGRESSIVE, DSEF_60HZ,      DSOR_1920_1080,  (const comp_reg_entry_t *)&comp1080p60},          /* 1080p60 */
    {DSESM_PROGRESSIVE, DSEF_59_94HZ,   DSOR_1920_1080,  (const comp_reg_entry_t *)&comp1080p60},          /* 1080p59.94 */
    {DSESM_PROGRESSIVE, DSEF_50HZ,      DSOR_1920_1080,  (const comp_reg_entry_t *)&comp1080p50},          /* 1080p50 */
    {DSESM_PROGRESSIVE, DSEF_25HZ,      DSOR_1920_1080,  (const comp_reg_entry_t *)&comp1080p25},          /* 1080p25 */
    {DSESM_PROGRESSIVE, DSEF_24HZ,      DSOR_1920_1080,  (const comp_reg_entry_t *)&comp1080p24},          /* 1080p24 */
    {DSESM_PROGRESSIVE, DSEF_23_976HZ,  DSOR_1920_1080,  (const comp_reg_entry_t *)&comp1080p24},          /* 1080p23.976 */
    {DSESM_INTERLACED,  DSEF_60HZ,      DSOR_1920_1080,  (const comp_reg_entry_t *)&comp1080i30},          /* 1080i60 */
    {DSESM_INTERLACED,  DSEF_59_94HZ,   DSOR_1920_1080,  (const comp_reg_entry_t *)&comp1080i30},          /* 1080i59.94 */
    {DSESM_INTERLACED,  DSEF_50HZ,      DSOR_1920_1080,  (const comp_reg_entry_t *)&comp1080i25},          /* 1080i50 */
    {DSESM_PROGRESSIVE, DSEF_60HZ,      DSOR_1280_720,   (const comp_reg_entry_t *) &comp720p60},           /*  720p60 */
    {DSESM_PROGRESSIVE, DSEF_59_94HZ,   DSOR_1280_720,   (const comp_reg_entry_t *) &comp720p60},           /*  720p59.94 */
    {DSESM_PROGRESSIVE, DSEF_50HZ,      DSOR_1280_720,   (const comp_reg_entry_t *) &comp720p50},           /*  720p50 */
    {DSESM_PROGRESSIVE, DSEF_30HZ,      DSOR_1280_720,   (const comp_reg_entry_t *) &comp720p30},           /*  720p30 */
    {DSESM_PROGRESSIVE, DSEF_29_97HZ,   DSOR_1280_720,   (const comp_reg_entry_t *) &comp720p30},           /*  720p29.97 */
    {DSESM_PROGRESSIVE, DSEF_25HZ,      DSOR_1280_720,   (const comp_reg_entry_t *) &comp720p25},           /*  720p25 */
    {DSESM_PROGRESSIVE, DSEF_24HZ,      DSOR_1280_720,   (const comp_reg_entry_t *) &comp720p24},           /*  720p24 */
    {DSESM_PROGRESSIVE, DSEF_23_976HZ,  DSOR_1280_720,   (const comp_reg_entry_t *) &comp720p24},           /*  720p23.976 */
    {DSESM_PROGRESSIVE, DSEF_50HZ,      DSOR_720_576,    (const comp_reg_entry_t *) &comp576p50},           /*  576p50 */
    {DSESM_PROGRESSIVE, DSEF_60HZ,      DSOR_720_480,    (const comp_reg_entry_t *) &comp480p60},           /*  480p60 */
    {DSESM_PROGRESSIVE, DSEF_59_94HZ,   DSOR_720_480,    (const comp_reg_entry_t *) &comp480p60},           /*  480p59.94 */
    {DSESM_INTERLACED,  DSEF_50HZ,      DSOR_720_576,    (const comp_reg_entry_t *) &comp576i25},           /*  576i50 */
    {DSESM_INTERLACED,  DSEF_60HZ,      DSOR_720_480,    (const comp_reg_entry_t *) &comp480i30},           /*  480i60 */
    {DSESM_INTERLACED,  DSEF_59_94HZ,   DSOR_720_480,    (const comp_reg_entry_t *) &comp480i30},           /*  480i59.94 */
    {DSESM_INTERLACED,  DSEF_24HZ,      DSOR_640_480,    (const comp_reg_entry_t *) 0xFFFFFFFF},            /* End of Table Marker */
};

int    Xcode4_SetUpComponentVenc(pxc_dfb_driver_data mdrv, struct display_mode_struct *pdisplay_mode, const DFBScreenOutputConfig *config)
{
    /* 
    *   Programs the video encoder based on the settings contained within the pdisplay_mode struct 
    *   If a corresponding VENC script is not found for the mode indicated in pdisplay_mode struct
    *   a -1 will be returned, otherwise the VENC will be programmed and a 0 will be returned.
    *   
    *   1. Scan the venc_script_table for the correct entry, if not found return failure
    *   2. Run the script, reading/masking/writing as necessary
    */
    DFBScreenEncoderScanMode            sc_mode;
    DFBScreenEncoderFrequency           freq;
    DFBScreenOutputResolution           res;
    const comp_reg_entry_t              *venc_script;
    
    u32 reg, mask, value;
    u32 temp, tmp_sft_reset;
    
    int i   =   0;
    
    while (venc_script_table[i].venc_script != (const comp_reg_entry_t *)0xFFFFFFFF) 
    {
        
        sc_mode =   venc_script_table[i].scan_mode;
        freq    =   venc_script_table[i].frequency;
        res     =   venc_script_table[i].resolution;
        venc_script =   venc_script_table[i].venc_script;
        
        
        if (venc_script_table[i].scan_mode  ==  pdisplay_mode->scanmode     &&
            venc_script_table[i].frequency  ==  pdisplay_mode->frequency    &&
            venc_script_table[i].resolution ==  pdisplay_mode->resolution
            )
        {
            //Target has been found break
            break;
        }
        else
        {
            i++;
        
        }
    
        
    }
    
    if (venc_script == (const comp_reg_entry_t *)0xFFFFFFFF)
    {
        /* Could not find the corresponding mode in the table, fail the call */
        D_DEBUG("Could not find a VENC script for the requested mode!!\n");
        return -1;
    }
    else
    {
        D_DEBUG("Found a VENC script for the requested mode!\n");
    }
    
    /* Now program the hardware ... */
    i = 0;
    reg     =   venc_script[i].reg;
    mask    =   venc_script[i].mask;
    value   =   venc_script[i].value;
    D_DEBUG("reg    =   0x%X\n", reg);
    D_DEBUG("mask    =   0x%X\n", mask);
    D_DEBUG("value    =   0x%X\n", value);
    
    
    tmp_sft_reset   =   REG_READ(DISP_SFT_RESET);
    /* Clear out the HD VENC reset and the OUTPUT_MUX reset ... */
    tmp_sft_reset&= (~(HD_VENC_SFT_RSTN_MASK | OUTPUT_MUX_SFT_RSTN_MASK));
    REG_WRITE(DISP_SFT_RESET, tmp_sft_reset);
    
    while (1)
    {
        /* 
        *   It is a good idea to put safeguards here to limit where in the register space
        *   we touch ...
        */
        
        if (reg != 0xFFFFFFFF)
        {
            temp    =   REG_READ(reg);
            D_DEBUG("REG[0x%X] = 0x%X\n", reg, temp);
            D_DEBUG("MASK = 0x%X\n", mask);
            temp    &=  (~mask);
            temp|=value;
            D_DEBUG("REG[0x%X]_NEW = 0x%X\n", reg, temp);
            REG_WRITE(reg, temp);
            i++;
            reg     =   venc_script[i].reg;
            mask    =   venc_script[i].mask;
            value   =   venc_script[i].value;
            D_DEBUG("reg    =   0x%X\n", reg);
            D_DEBUG("mask    =   0x%X\n", mask);
            D_DEBUG("value    =   0x%X\n", value);
        }
        else
        {
            break;
        
        }
        
        
        
    }
    
    /* De-assert the reset and the VENC and OUTPUT_MUX */
    tmp_sft_reset|= (HD_VENC_SFT_RSTN_MASK | OUTPUT_MUX_SFT_RSTN_MASK);
    
    REG_WRITE(DISP_SFT_RESET, tmp_sft_reset);
    
    /* Check for degredation flag */
    if (config->flags & DSOCONF_SIGNAL_DEGRADE)
    {
        D_DEBUG("signal_degrade = 0x%X\n", config->signal_degrade);
        switch (config->signal_degrade)
        {
            case DSD_FULLRESOLUTION:
                temp    =   DISP_HD_VENC0_FULL_RESOLUTION;
                D_DEBUG("Component output is at full resolution - DSOCONF_SIGNAL_DEGRADE set \n");
                break;
                
            case DSD_HALFRESOLUTION:
                temp    =   DISP_HD_VENC0_HALF_RESOLUTION;
                D_DEBUG("Component output is at half resolution \n");
                break;
                
            case DSD_QUARTERRESOLUTION:
                temp    =   DISP_HD_VENC0_QUARTER_RESOLUTION;
                D_DEBUG("Component output is at quarter resolution \n");
                break;
            
            default:
                temp    =   DISP_HD_VENC0_FULL_RESOLUTION;
                D_DEBUG("Component output is at full resolution \n");
                break;
        }
    
    }
    else
    {
        /* Default to no signal degredation */
        temp    =   DISP_HD_VENC0_FULL_RESOLUTION;
        D_DEBUG("Component output is at full resolution - DSOCONF_SIGNAL_DEGRADE not set  \n");
    }
    
    REG_WRITE(DISP_HD_VENC0_COMPDEG, temp);
    return 0;
}
