/************************************************** 
*
*  Copyright (c) 2006-2011 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_hd_vbi.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR: 
*
*  HISTORY
*
*
***************************************************/


#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include <pthread.h>

#include <sys/mman.h>
#include <fcntl.h>
 #include <sys/ioctl.h>


#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/layers_internal.h>
#include <core/surface.h>
#include <core/palette.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"
#include "xcioctl.h"


#define MAX_VBI_REGION_INDEX 16


DFBResult Xcode_Hd_Vbi_Plane_Update_Region(void *driver_data,
                                        void *layer_data,
                                        const DFBRegion *update);

static void enable_hd_vbi_output(void *driver_data);
static void disable_hd_vbi_output(void *driver_data);


/*******************************************************************************************************/

static void enable_hd_vbi_output(void *driver_data)
{
     u32 tmp;
     u32 temp = 0;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     // set DISP_HD_VENC0_DVE_CNTRL to enable hd VBI output
     tmp = REG_READ(DISP_HD_VENC0_DVE_CNTRL);

     D_DEBUG( "REG_READ(DISP_HD_VENC0_DVE_CNTRL) = %x\n", tmp );

     tmp |= (DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_VENC_MASK | DISP_HD_VENC0_DVE_CNTRL_VBI_ON_Y_MASK);  //  0xC0000000;
     REG_WRITE(DISP_HD_VENC0_DVE_CNTRL, tmp);
     
     temp = REG_READ(DISP_HD_VENC0_DVE_CNTRL);
     D_DEBUG( "REG_WRITE(DISP_HD_VENC0_DVE_CNTRL) = %x\n", temp );

     // clear DISP_HD_VBI0_HANC0_FORMAT.VBI_PIPE_DISABLE bit
     tmp = REG_READ(DISP_HD_VBI0_HANC0_FORMAT);
     D_DEBUG( "REG_READ(DISP_HD_VBI0_HANC0_FORMAT) = %x\n", tmp);
     tmp &= ~DISP_HD_VBI0_HANC0_FORMAT_VBI_PIPE_DISABLE_MASK;   // 0x7FFFFFFF;
     REG_WRITE(DISP_HD_VBI0_HANC0_FORMAT, tmp);
     D_DEBUG( "DISP_HD_VBI0_HANC0_FORMAT = %x\n", tmp);
}


static void disable_hd_vbi_output(void *driver_data)
{
     u32 tmp;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     // set DISP_HD_VENC0_DVE_CNTRL to disable VBI output
     tmp = REG_READ(DISP_HD_VENC0_DVE_CNTRL);
  
     D_DEBUG( "REG_READ(DISP_HD_VENC0_DVE_CNTRL) = %x\n", tmp );

     tmp &= ~(DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_VENC_MASK | DISP_HD_VENC0_DVE_CNTRL_VBI_ON_Y_MASK);      // 0x3FFFFFFF;
     REG_WRITE(DISP_HD_VENC0_DVE_CNTRL, tmp);

  //   tmp = REG_READ(DISP_HD_VENC0_DVE_CNTRL);

     D_DEBUG( "REG_WRITE(DISP_HD_VENC0_DVE_CNTRL) = %x\n", tmp );


     // set DISP_HD_VBI0_HANC0_FORMAT.VBI_PIPE_DISABLE bit
     tmp = REG_READ(DISP_HD_VBI0_HANC0_FORMAT);
     D_DEBUG( "REG_READ(DISP_HD_VBI0_HANC0_FORMAT) = %x\n", tmp);
     tmp |= DISP_HD_VBI0_HANC0_FORMAT_VBI_PIPE_DISABLE_MASK;
     REG_WRITE(DISP_HD_VBI0_HANC0_FORMAT, tmp);
     D_DEBUG( "REG_WRITE(DISP_HD_VBI0_HANC0_FORMAT) = %x\n", tmp);

}


/****************************************************************************************************/

static DFBResult
XcodeHdVbi0InitLayer( CoreLayer                *layer,
                    void                       *driver_data,
                    void                       *layer_data,
                    DFBDisplayLayerDescription *description,
                    DFBDisplayLayerConfig      *config,
                    DFBColorAdjustment         *adjustment )
{
     u32     tmp;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "xcode DFB driver XcodeHdVbi0InitLayer enter:\n" );

     /* set capabilities and type */
     description->caps = DLCAPS_SURFACE | DLCAPS_SCREEN_POSITION | DLCAPS_SCREEN_LOCATION | DLCAPS_LEVELS;

     description->type = DLTF_NONE;

     /* set name */
     snprintf( description->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "ViXS Xcode4 HD VBI Plane 0 over HD output" );

     /* fill out the default configuration */
     config->flags       = DLCONF_WIDTH       | DLCONF_HEIGHT | DLCONF_FLIP_HEIGHT |
                           DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS | DLCONF_SURFACE_CAPS;

     config->width       = 2048;  //XCODE_DEFAULT_WIDTH;
     config->height      = 16 * 4;    //XCODE_DEFAULT_HEIGHT;
     config->flip_height = 16;

     config->pixelformat = DSPF_LUT8;   //DSPF_I420;
     config->buffermode  = DLBM_FRONTONLY;
     config->surface_caps |= DSCAPS_STATIC_ALLOC ;
     config->options     = DLOP_ALPHACHANNEL;

     adjustment->flags   = DCAF_ALL;
     adjustment->brightness = LAYER_DEFAULT_BRIGHT;
     adjustment->contrast = LAYER_DEFAULT_CONTRAST;
     adjustment->saturation = LAYER_DEFAULT_SATURATION;
     adjustment->hue = LAYER_DEFAULT_HUE;
     
     m_layer_data->last_frame_counter = 0;
     m_layer_data->total_submitted_frames = 0;
     m_layer_data->total_return_frame_to_free = 0;
     m_layer_data->status = 0;
     m_layer_data->flip_height = 16;
     m_layer_data->dest_width = 1920;
     m_layer_data->dest_height = 16;

     D_DEBUG( "before enable hd vbi0 output \n");

     enable_hd_vbi_output(mdrv);  // enable hd vbi output
     D_DEBUG( "enable hd vbi0 output \n" );

     D_DEBUG( "xcode DFB driver XcodeHdVbi0InitLayer exit:\n" );

     return DFB_OK;
}



static DFBResult
XcodeHdVbi0SetRegion( CoreLayer                  *layer,
                    void                       *driver_data,
                    void                       *layer_data,
                    void                       *region_data,
                    CoreLayerRegionConfig      *config,
                    CoreLayerRegionConfigFlags  updated,
                    CoreSurface                *surface,
                    CorePalette                *palette,
                    CoreSurfaceBufferLock      *lock )
{
     DFBResult ret;
     u32 low_key;
     u32 temp;
     u32 tmp;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     D_ASSERT( driver_data != NULL );
     D_ASSERT( layer_data != NULL );

     D_DEBUG( "XcodeHdVbi0SetRegion enter \n" );
     D_DEBUG ("surface width %d \n", config->width);
     D_DEBUG ("surface height %d \n", config->height);
     D_DEBUG ("destination x %d \n", config->dest.x);
     D_DEBUG ("destination y %d \n", config->dest.y);
     D_DEBUG ("destination width %d \n", config->dest.w);
     D_DEBUG ("destination height %d \n", config->dest.h);
     D_DEBUG ("surface caps %d \n", config->surface_caps);
     D_DEBUG ("surface pitch %d \n", lock->pitch);
     D_DEBUG ("surface phys 0x%x \n", lock->phys);

     
     // set HD VBI0 buffer size to register: DISP_HD_VBI0_HANC0_SIZE
     tmp = REG_READ(DISP_HD_VBI0_HANC0_SIZE);
     tmp &= ~DISP_HD_VBI0_HANC0_SIZE_VBI_SIZE_MASK;   //  0xFFFFF000;
     D_DEBUG( "config->dest.w = %x\n", config->dest.w);
     tmp |= config->dest.w;
     REG_WRITE(DISP_HD_VBI0_HANC0_SIZE, tmp);
     D_DEBUG( "REG_WRITE(DISP_HD_VBI0_HANC0_SIZE) = %x\n", tmp );     

     // set the pitch width of the VBI buffer to register: DISP_HD_VBI0_BUF
     //tmp = (u32)config->width & 0x0FFFF; 
     tmp = (u32)config->width & DISP_HD_VBI0_BUF_PITCH_MASK;    
     temp = ((config->width * config->flip_height) / 16) & ~DISP_HD_VBI0_BUF_SIZE_MASK;   // 0xFFFF;
     temp <<= 16;
     tmp |= temp;
     REG_WRITE(DISP_HD_VBI0_BUF, tmp);
     D_DEBUG( "REG_WRITE(DISP_HD_VBI0_BUF) = %x\n", tmp );

     // set VBI buffer start address to register: DISP_HD_VBI0_ADR
     REG_WRITE(DISP_HD_VBI0_ADR, lock->phys);
     D_DEBUG( "REG_WRITE(DISP_HD_VBI0_ADR) = %x\n", lock->phys);

     // set register:  DISP_HD_COMP_START_VBI_POS
     tmp = config->dest.y;
     REG_WRITE(DISP_HD_COMP_START_VBI_POS, tmp);
     D_DEBUG( "REG_WRITE(DISP_HD_COMP_START_VBI_POS) = %x\n", tmp );
 
     // set register:  DISP_HD_COMP_STOP_VBI_POS
     tmp = config->dest.h;
     REG_WRITE(DISP_HD_COMP_STOP_VBI_POS, tmp);
     D_DEBUG( "REG_WRITE(DISP_HD_COMP_STOP_VBI_POS) = %x\n", tmp );

     // set register:  DISP_HD_VBI0_SAMPLES
     tmp = REG_READ(DISP_HD_VBI0_SAMPLES) & ~DISP_HD_VBI0_SAMPLES_WIDTH_MASK;   // 0xFFFF0000;
     temp = config->dest.w;
     tmp |= temp;
     REG_WRITE(DISP_HD_VBI0_SAMPLES, tmp);
           
     D_DEBUG( "XcodeHdVbi0SetRegion exit \n" );

     return DFB_OK;
}


static DFBResult
XcodeHdVbi0TestRegion( CoreLayer                  *layer,
                     void                       *driver_data,
                     void                       *layer_data,
                     CoreLayerRegionConfig      *config,
                     CoreLayerRegionConfigFlags *failed )
{
     CoreLayerRegionConfigFlags fail = 0;

     D_DEBUG( "xcode DFB driver XcodeHdVbi0TestRegion enter:\n" );

     D_DEBUG ("surface width %d \n", config->width);
     D_DEBUG ("surface height %d \n", config->height);
     D_DEBUG ("destination x %d \n", config->dest.x);
     D_DEBUG ("destination y %d \n", config->dest.y);
     D_DEBUG ("destination width %d \n", config->dest.w);
     D_DEBUG ("destination height %d \n", config->dest.h);
     D_DEBUG ("surface caps %d \n", config->surface_caps);
     D_DEBUG ("surface options %d \n", config->options);

     if (config->options & ~XCODE4_OSD_SUPPORTED_OPTIONS)
          fail |= CLRCF_OPTIONS;

     switch (config->format) {
          case DSPF_I420:
          case DSPF_YV12:
          case DSPF_YV16: //replaced DSPF_UYVY
          case DSPF_LUT8:
          case DSPF_YUY2: ////TODO: not consistent with DirectFB pixel format.
               break;

          default:
               fail |= CLRCF_FORMAT;
     }

     if (failed)
          *failed = fail;

     if (fail)
     {
          return DFB_UNSUPPORTED;
     }

     D_DEBUG( "xcode DFB driver XcodeHdVbi0TestRegion exit:\n" );

     return DFB_OK;
}


/****************************************************************************************************************************/

static DFBResult
XcodeHdVbi0FlipRegion( CoreLayer             *layer,
                 void                  *driver_data,
                 void                  *layer_data,
                 void                  *region_data,
                 CoreSurface           *surface,
                 DFBSurfaceFlipFlags    flags,
                 CoreSurfaceBufferLock *lock )
{
     DFBResult              ret;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "XcodeHdVbi0FlipRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( mdrv != NULL );

     D_DEBUG( "( 0x%08lx [%d] 0x%04x)\n", 
                 lock->phys, lock->pitch, flags);

     dfb_surface_flip( surface, false );

     D_DEBUG( "XcodeHdVbi0FlipRegion exit \n" );
     return DFB_OK;
}


/****************************************************************************************************************************/

static DFBResult
XcodeHdVbi0UpdateRegion( CoreLayer             *layer,
                   void                  *driver_data,
                   void                  *layer_data,
                   void                  *region_data,
                   CoreSurface           *surface,
                   const DFBRegion       *update,
                   CoreSurfaceBufferLock *lock )
{

     D_DEBUG( "XcodeHdVbi0UpdateRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( driver_data != NULL );
     D_ASSERT( layer_data != NULL );
     D_ASSERT( update != NULL );
     DFBResult ret;


     ret = Xcode_Hd_Vbi_Plane_Update_Region(driver_data, layer_data, update);
  
     D_DEBUG( "XcodeHdVbi0UpdateRegion exit \n" );
     
     return ret;
}

/**********************************************************************************************************************/

DFBResult Xcode_Hd_Vbi_Plane_Update_Region(void *driver_data,
                                        void *layer_data,
                                        const DFBRegion *update)
{
    u32 free_entries;
    u32 region_index = 0;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32 tmp;
    DFBResult ret; 

    D_DEBUG("region calculate\n");

    D_DEBUG("update->y1 = %d\n", update->y1);
    D_DEBUG("update->y2 = %d\n", update->y2);
    D_DEBUG("update->x1 = %d\n", update->x1);
    D_DEBUG("update->x2 = %d\n", update->x2);

    D_DEBUG("m_layer_data->flip_height = %d\n", m_layer_data->flip_height);

    region_index = update->y1 / m_layer_data->flip_height;

    D_DEBUG("region_index = %d\n", region_index);

    if (region_index < MAX_VBI_REGION_INDEX )
    {

        tmp = VID0_CMD1_FIFO_WE_MASK |
              DISP_HD_VID0_CMD1_FIFO_UPDATE_COMP_REGS_MASK |
              UPDATE_VBI0_REGS_MASK |
              HD_VBI0_FLD_SEL_MASK |
              (region_index << HD_VBI0_TOP_IDX_SHIFT) |
              (region_index << HD_VBI0_BOT_IDX_SHIFT);

        REG_WRITE(DISP_HD_VID0_CMD1_FIFO, tmp);
        D_DEBUG( "REG_WRITE(DISP_HD_VID0_CMD1_FIFO) = %x\n", tmp );
        
        return DFB_OK;

    }else{
        return DFB_INVAREA;   
    }

}



/**********************************************************************************************************************/
/*
static DFBResult XcodeHdVbi0RemoveRegion( CoreLayer *layer,
                                        void      *driver_data,
                                        void      *layer_data,
					void      *region_data )
{
    DFBResult ret;
     
    D_DEBUG( "XcodeHdVbi0RemoveRegion enter \n" );
 
    D_DEBUG( "XcodeHdVbi0RemoveRegion exit \n" );

    return 0;

}
*/

static DFBResult XcodeHdVbi0ShutdownLayer( CoreLayer *layer,
                                        void       *driver_data,
                                        void       *layer_data )
{
    DFBResult ret;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
 
    D_DEBUG( "XcodeHdVbi0ShutdownLayer enter \n" );
    
    disable_hd_vbi_output(mdrv);
    D_DEBUG( "disable hd vbi0 output \n" );
 
    D_DEBUG( "XcodeHdVbi0ShutdownLayer exit \n" );

    return DFB_OK;

}


int XcodeHdVbiLayerDataSize()
{
     return sizeof(xc_layer_data);
}


const DisplayLayerFuncs XcodeHdVbi0LayerFuncs = {
     LayerDataSize:      XcodeHdVbiLayerDataSize,
     InitLayer:          XcodeHdVbi0InitLayer,
     TestRegion:         XcodeHdVbi0TestRegion,
     SetRegion:          XcodeHdVbi0SetRegion,
     FlipRegion:         XcodeHdVbi0FlipRegion,
     UpdateRegion:       XcodeHdVbi0UpdateRegion,
     ShutdownLayer:      XcodeHdVbi0ShutdownLayer
};

