#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>
#include <core/layers_internal.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>
#include <time.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"
#include "xcioctl.h"
#include "xcode4_fes1x.h"

/* This is done for the automatic build. */
#define FES_SHIFT

#define SD_FLIP_DELAY

D_DEBUG_DOMAIN( XcodeHDtoSD, "XcodeHDtoSD", "XcodeHDtoSD" );

D_DEBUG_DOMAIN( XcodeHDtoSD2, "XcodeHDtoSD2", "xcode4_osd_merge" );

#define DPRINTF(...) D_DEBUG_AT( XcodeHDtoSD, __VA_ARGS__)

#define DPRINTF2(...) D_DEBUG_AT( XcodeHDtoSD2, __VA_ARGS__)

#if 0
GIC1_BACKGRD_COLOR (MM:0x4EE4) RW {
@  ALPHA             7:0     def=0x0;   #Background color to be used - alpha
    GIC1_RED                 15:8;               #Background color to be used - red
    GIC1_GREEN               23:16;              #Background color to be used - green
    GIC1_BLUE                31:24;              #Background color to be used - blue
};
#endif

#define GIC_BACKGROUND_COLOR 0xFFFF0000

#define SD_FRAME_AVAILABLE        0x80000000

#define SD_OUTPUT_FRAME_NUM        4

#ifdef SD_OUTPUT_HIRES
#define SD_FRAME_WIDTH 1440
#else
#define SD_FRAME_WIDTH 720
#endif

#define SD_FRAME_HEIGHT 576
#define SD_FRAME_PITCH (SD_FRAME_WIDTH * 4)
#define SD_OUTPUT_PITCH (SD_FRAME_PITCH * 2)
#define SD_FRAME_SIZE (SD_FRAME_HEIGHT * SD_FRAME_PITCH)
#define SD_OUTPUT_SIZE (SD_FRAME_HEIGHT * SD_OUTPUT_PITCH)

extern void Wait_For_Fifo_Entries(pxc_dfb_driver_data mdrv,
                           u32 status_reg, 
                           u32 free_ent_mask, 
                           u32 free_ent_shift,
                           u32 requested_fifo_space);

extern int xcode4_g2d_scale_and_rotate ( xc_driver_data *mdrv,
                                         int srcPhys, int dfbSrcFmt, int srcX, int srcY, int srcW, int srcH, int srcPitch,
                                         int dstPhys, int dfbDstFmt, int dstX, int dstY, int dstW, int dstH, int dstPitch,
                                         int rotation );

extern u32 Calculate_gfx_hsca_inv_ratio(u32 input_h_size, u32 output_h_size);

extern DFBResult SetFes1ScalerCoef(pxc_dfb_driver_data mdrv, 
                                   int in_width, 
                                   int in_height,
                                   int in_format,
                                   int out_width,
                                   int out_height,
                                   int out_format);

void Set_Fes1_Parameter(pxc_dfb_driver_data mdrv,
                        int vsize_in, 
                        int vsize_out, 
                        int hsize_in, 
                        int hsize_out, 
                        int interlaced_in, 
                        int interlaced_out, 
                        int chroma_in, 
                        int chroma_out, 
                        int genLh, 
                        int genLv);

static int update_fes(pxc_dfb_driver_data mdrv, int src_w, int src_h, int dst_w, int dst_h)
{
    u64 ratio;
    u32 r;
        
    REG_WRITE(FES_CFIFO_CTRL, 0x2);         //Accept from both FES1 and FES2 CMD FIFOs

    SetFes1ScalerCoef(mdrv, src_w, src_h, 420, dst_w, dst_h, 444);

    Set_Fes1_Parameter(mdrv, 
                       src_h, 
                       dst_h, 
                       src_w, 
                       dst_w,
                       0, 0, 420, 444, 0, 0);
        
    FES1_REG_WRITE(FES1_CSC_PARM4,0x00100000);
    FES1_REG_WRITE(FES1_CSC_PARM5,0x00800080);
    FES1_REG_WRITE(FES1_CSC_PARM6,0xFF00FF00);
    FES1_REG_WRITE(FES1_CSC_PARM7,0x0000FF00);

    return 0;
}

static int get_pixel_format_color(DFBSurfacePixelFormat dsf_color)
{
    int pixel_format_color;

    switch (dsf_color)
    {
        case DSPF_RGB32:
        case DSPF_ARGB:
            pixel_format_color = PIX_FORMAT_RGB32;
            break;

        case DSPF_RGB16:
            pixel_format_color = PIX_FORMAT_RGB565;
            break;

        case DSPF_ARGB1555:
            pixel_format_color = PIX_FORMAT_RGB1555;
            break;

        case DSPF_ARGB4444:
            pixel_format_color = PIX_FORMAT_RGB4444;
            break;

        case DSPF_LUT8:
        case DSPF_A8:
            pixel_format_color = PIX_FORMAT_LUT8;
            break;

        case DSPF_ALUT44:
            pixel_format_color = PIX_FORMAT_LUT4;
            break;

        case DSPF_LUT2:
            pixel_format_color = PIX_FORMAT_LUT2;
            break;

        case DSPF_A1:
            pixel_format_color = PIX_FORMAT_MONOCHROME;
            break;

        default:
            pixel_format_color = PIX_FORMAT_UNKNOWN;
            break;
    }

    return pixel_format_color;
}

static int xcode4_gic_clear2(
                    pxc_dfb_driver_data mdrv,
                    int dst_addr,
                    int dst_fmt,
                    int dst_w,
                    int dst_h,
                    int dst_pitch)
{
    gic_opt_struct current_gic_ctl;
    u32 fifo_entries;

    if(dst_fmt != DSPF_ARGB)
        return -1;

    if(!dst_w || !dst_h)
        return -3;

    memset(&current_gic_ctl, 0, sizeof(current_gic_ctl));

    current_gic_ctl.gic_src1_adr0 = mdrv->shared->sd_frame_entries.base;
    current_gic_ctl.gic_src1_h_w = 0x10001;
    current_gic_ctl.gic_src1_pitch = dst_pitch;
    current_gic_ctl.gic_src1_y_x = 0;
    current_gic_ctl.gic_alpha_mode_src1 = GIC_ALPHA_MODE_CONSTANT | GIC0_SRC2_ALPHA_PREMUL_MASK;

    current_gic_ctl.gic_src1_mode = GIC_ARGB8888 | 
                             (ALPHA_24_TO_31 << GIC0_SRC1_A_PACK_ORDER_SHIFT) |
                             (RED_16_TO_23 << GIC0_SRC1_YR_PACK_ORDER_SHIFT) |
                             (GREEN_8_TO_15 << GIC0_SRC1_UG_PACK_ORDER_SHIFT) |
                             (BLUE_0_TO_7 << GIC0_SRC1_VB_PACK_ORDER_SHIFT);

    current_gic_ctl.gic_src2_adr0 = mdrv->shared->sd_frame_entries.base;
    current_gic_ctl.gic_src2_h_w = 0x10001;
    current_gic_ctl.gic_src2_pitch = dst_pitch;
    current_gic_ctl.gic_src2_y_x = 0;
    current_gic_ctl.gic_alpha_mode_src2 = GIC_ALPHA_MODE_CONSTANT | GIC0_SRC2_ALPHA_PREMUL_MASK;
    
    current_gic_ctl.gic_src2_mode = GIC_ARGB8888 | 
                             (ALPHA_24_TO_31 << GIC0_SRC2_A_PACK_ORDER_SHIFT) |
                             (RED_16_TO_23 << GIC0_SRC2_YR_PACK_ORDER_SHIFT) |
                             (GREEN_8_TO_15 << GIC0_SRC2_UG_PACK_ORDER_SHIFT) |
                             (BLUE_0_TO_7 << GIC0_SRC2_VB_PACK_ORDER_SHIFT);

    current_gic_ctl.gic_dst_adr0 = dst_addr;
    current_gic_ctl.gic_dst_h_w = dst_w | (dst_h << GIC0_DST_H_SHIFT);
    current_gic_ctl.gic_dst_pitch = dst_pitch;
    current_gic_ctl.gic_dst_y_x = 0;
    
    current_gic_ctl.gic_dst_mode = GIC_ARGB8888;
    
    current_gic_ctl.gic_trigger_command = GIC0_OP_KICKOFF_MASK | GIC0_BLEND_EN_MASK;

    gic_opt_struct * gic_ctl_ptr = &current_gic_ctl;

    //make sure GIC0 command fifo is enough
    WAIT_FOR_GIC0_FIFO_ENTRIES(fifo_entries, 8);
    
    GIC0_REG_WRITE(GIC0_SRC1_ADR0, gic_ctl_ptr->gic_src1_adr0);
    GIC0_REG_WRITE(GIC0_SRC1_ADR1, gic_ctl_ptr->gic_src1_adr1);
    GIC0_REG_WRITE(GIC0_SRC1_ADR2, gic_ctl_ptr->gic_src1_adr2);
    GIC0_REG_WRITE(GIC0_SRC1_H_W, gic_ctl_ptr->gic_src1_h_w);  
    GIC0_REG_WRITE(GIC0_SRC1_PITCH, gic_ctl_ptr->gic_src1_pitch);
    GIC0_REG_WRITE(GIC0_SRC1_Y_X, gic_ctl_ptr->gic_src1_y_x);
    GIC0_REG_WRITE(GIC0_SRC1_MODE, gic_ctl_ptr->gic_src1_mode); 
    GIC0_REG_WRITE(GIC0_ALPHA_MODE_SRC1, gic_ctl_ptr->gic_alpha_mode_src1);

    //make sure GIC0 command fifo is enough
    WAIT_FOR_GIC0_FIFO_ENTRIES(fifo_entries, 8);

    //configure src2 as graphic background, src2 will be overwritten by src1
    GIC0_REG_WRITE(GIC0_SRC2_ADR0, gic_ctl_ptr->gic_src2_adr0);
    GIC0_REG_WRITE(GIC0_SRC2_ADR1, gic_ctl_ptr->gic_src2_adr1);
    GIC0_REG_WRITE(GIC0_SRC2_ADR2, gic_ctl_ptr->gic_src2_adr2);
    GIC0_REG_WRITE(GIC0_SRC2_H_W, gic_ctl_ptr->gic_src2_h_w);
    GIC0_REG_WRITE(GIC0_SRC2_Y_X, gic_ctl_ptr->gic_src2_y_x);
    GIC0_REG_WRITE(GIC0_SRC2_PITCH, gic_ctl_ptr->gic_src2_pitch);
    GIC0_REG_WRITE(GIC0_SRC2_MODE, gic_ctl_ptr->gic_src2_mode); 
    GIC0_REG_WRITE(GIC0_ALPHA_MODE_SRC2, gic_ctl_ptr->gic_alpha_mode_src2);

    //make sure GIC0 command fifo is enough
    WAIT_FOR_GIC0_FIFO_ENTRIES(fifo_entries, 7);

    GIC0_REG_WRITE(GIC0_DST_ADR0, gic_ctl_ptr->gic_dst_adr0);
    GIC0_REG_WRITE(GIC0_DST_ADR1, gic_ctl_ptr->gic_dst_adr1);
    GIC0_REG_WRITE(GIC0_DST_ADR2, gic_ctl_ptr->gic_dst_adr2);
    GIC0_REG_WRITE(GIC0_DST_H_W, gic_ctl_ptr->gic_dst_h_w);
    GIC0_REG_WRITE(GIC0_DST_PITCH, gic_ctl_ptr->gic_dst_pitch);
    GIC0_REG_WRITE(GIC0_DST_Y_X, gic_ctl_ptr->gic_dst_y_x);
    GIC0_REG_WRITE(GIC0_DST_MODE, gic_ctl_ptr->gic_dst_mode);

    //make sure GIC0 command fifo is enough
    WAIT_FOR_GIC0_FIFO_ENTRIES(fifo_entries, 1);

    //start to kick off gic0 to do the combine
    GIC0_REG_WRITE(GIC0_CONFIG_CTRL, gic_ctl_ptr->gic_trigger_command);

    return 0;
}

static int xcode4_gic_write(
                    pxc_dfb_driver_data mdrv,
                    int src_addr,
                    int src_fmt,
                    int src_x,
                    int src_y,
                    int src_w,
                    int src_h,
                    int src_pitch,
                    int dst_addr,
                    int dst_fmt,
                    int dst_x,
                    int dst_y,
                    int dst_w,
                    int dst_h,
                    int dst_pitch
                    )
{
    gic_opt_struct current_gic_ctl;
    u32 fifo_entries;
    sd_frame_entries_t * entries = &mdrv->shared->sd_frame_entries;

    DPRINTF2("%s\n", __FUNCTION__);

    DPRINTF2("src_addr = %x, src_fmt = %x, (%d, %d, %d, %d), src_pitch = %d\n",
        src_addr, src_fmt, src_x, src_y, src_w, src_h, src_pitch);

    DPRINTF2("dst_addr = %x, dst_fmt = %x, (%d, %d, %d, %d), dst_pitch = %d\n",
        dst_addr, dst_fmt, dst_x, dst_y, dst_w, dst_h, dst_pitch);

    if(!(src_addr && dst_addr))
        return -2;

    if(!dst_w || !dst_h)
        return -3;

    memset(&current_gic_ctl, 0, sizeof(current_gic_ctl));

    src_addr = src_addr + src_y * src_pitch + src_x / 8 * 32;

    current_gic_ctl.gic_src1_adr0 = src_addr;
    current_gic_ctl.gic_src1_h_w = src_w | (src_h << GIC1_SRC1_H_SHIFT);
    current_gic_ctl.gic_src1_pitch = src_pitch;
    current_gic_ctl.gic_src1_y_x = dst_x | (dst_y << GIC1_SRC1_Y_SHIFT);

    current_gic_ctl.gic_src1_mode = GIC_AYUV444 | 
                             (ALPHA_24_TO_31 << GIC1_SRC1_A_PACK_ORDER_SHIFT) |
                             (RED_16_TO_23 << GIC1_SRC1_YR_PACK_ORDER_SHIFT) |
                             (GREEN_8_TO_15 << GIC1_SRC1_UG_PACK_ORDER_SHIFT) |
                             (BLUE_0_TO_7 << GIC1_SRC1_VB_PACK_ORDER_SHIFT);

    if(REG_READ(DISP_HD_COMP_ALPHA_MODE) & ALPHA_MODE_G0_MASK)
    {
        current_gic_ctl.gic_alpha_mode_src1 = GIC_ALPHA_MODE_PERPIXEL | GIC_ALPHA_MODE_NOPREMULTIPLY;
    }
    else
    {
        current_gic_ctl.gic_alpha_mode_src1 = GIC_ALPHA_MODE_PERPIXEL;
    }

    current_gic_ctl.gic_backgrd_color = GIC_BACKGROUND_COLOR;

    current_gic_ctl.gic_dst_adr0 = dst_addr;
    current_gic_ctl.gic_dst_h_w = dst_w | (dst_h << GIC1_DST_H_SHIFT);
    current_gic_ctl.gic_dst_pitch = dst_pitch;
    current_gic_ctl.gic_dst_y_x = 0;
    
    current_gic_ctl.gic_dst_mode = GIC_AYUV444;
    
    current_gic_ctl.gic_trigger_command = GIC1_OP_KICKOFF_MASK | 
                                             (SRC1_OVER_BACKGROUND << GIC1_BLEND_LEVELS_SHIFT)  | 
                                             GIC1_CMD_RBM_DIRECT_EN_MASK |
                                             (GIC1_BLEND_DOMAIN_YCBCR << GIC1_BLEND_DOMAIN_SHIFT);

    if(src_fmt == DSPF_ARGB)
    {
        current_gic_ctl.gic_trigger_command |= (GIC1_CSC_SEL_SRC1 << GIC1_CSC_SRC_SEL_SHIFT);
    }

    gic_opt_struct * gic_ctl_ptr = &current_gic_ctl;

    //make sure GIC1 command fifo is enough
    WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 16);
    
    GIC1_REG_WRITE(GIC1_SRC1_ADR0, gic_ctl_ptr->gic_src1_adr0);
    GIC1_REG_WRITE(GIC1_SRC1_ADR1, gic_ctl_ptr->gic_src1_adr1);
    GIC1_REG_WRITE(GIC1_SRC1_ADR2, gic_ctl_ptr->gic_src1_adr2);
    GIC1_REG_WRITE(GIC1_SRC1_H_W, gic_ctl_ptr->gic_src1_h_w);  
    GIC1_REG_WRITE(GIC1_SRC1_PITCH, gic_ctl_ptr->gic_src1_pitch);
    GIC1_REG_WRITE(GIC1_SRC1_Y_X, gic_ctl_ptr->gic_src1_y_x);
    GIC1_REG_WRITE(GIC1_SRC1_MODE, gic_ctl_ptr->gic_src1_mode); 
    GIC1_REG_WRITE(GIC1_ALPHA_MODE_SRC1, gic_ctl_ptr->gic_alpha_mode_src1);


    if(entries->color_space == DSCS_709)
    {
        /* Set CSC for RGB->YCbCr 709:          */
        /*  [  0.182586   0.614231   0.062007   */
        /*    -0.100644  -0.338572   0.439216   */
        /*     0.439216  -0.398942  -0.040274 ] */
        /*                                      */
        /*  Hardware Settings:                  */
        /*    [ 05D8 13A8 01FC                  */
        /*      FCC8 F52A 0E0E                  */
        /*      0E0E F33C FEB6 ]                */
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM0, 0x13a805d8);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM1, 0xfcc801fc);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM2, 0x0e0ef52a);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM3, 0xf33c0e0e);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM4, 0x0000feb6);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM5, 0x00800080);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM6, 0xff00ff00);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM7, 0x0000ff00);
    }
    else
    {
        /* Set CSC for RGB->YCbCr 601:          */
        /*   [ 0.256788   0.504129   0.097906 */
        /* -0.148223  -0.290993   0.439216 */
        /*  0.439216  -0.367788  -0.071427] */
        /*                                      */
        /*  Hardware Settings:                  */
        /* [0838 1022 0322                      */
        /* FB42 F6B0 0E0E                       */
        /* 0E0E F43B FDB7]                      */
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM0, 0x10220838);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM1, 0xfb420322);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM2, 0x0e0ef6b0);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM3, 0xf43b0e0e);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM4, 0x0000fdb7);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM5, 0x00800080);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM6, 0xff00ff00);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM7, 0x0000ff00);
    }

    //make sure GIC0 command fifo is enough
    WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 8);

    GIC1_REG_WRITE(GIC1_BACKGRD_COLOR, gic_ctl_ptr->gic_backgrd_color);
    GIC1_REG_WRITE(GIC1_DST_ADR0, gic_ctl_ptr->gic_dst_adr0);
    GIC1_REG_WRITE(GIC1_DST_ADR1, gic_ctl_ptr->gic_dst_adr1);
    GIC1_REG_WRITE(GIC1_DST_ADR2, gic_ctl_ptr->gic_dst_adr2);
    GIC1_REG_WRITE(GIC1_DST_H_W, gic_ctl_ptr->gic_dst_h_w);
    GIC1_REG_WRITE(GIC1_DST_PITCH, gic_ctl_ptr->gic_dst_pitch);
    GIC1_REG_WRITE(GIC1_DST_Y_X, gic_ctl_ptr->gic_dst_y_x);
    GIC1_REG_WRITE(GIC1_DST_MODE, gic_ctl_ptr->gic_dst_mode);

    //make sure GIC0 command fifo is enough
    WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 1);

    //start to kick off gic1 to do the combine
    GIC1_REG_WRITE(GIC1_CONFIG_CTRL, gic_ctl_ptr->gic_trigger_command);

    return 0;
}

static int xcode4_gic_merge(
                    pxc_dfb_driver_data mdrv,
                    int src0_addr,
                    int src0_fmt,
                    int src0_x,
                    int src0_y,
                    int src0_w,
                    int src0_h,
                    int src0_pitch,
                    int src1_addr,
                    int src1_fmt,
                    int src1_x,
                    int src1_y,
                    int src1_w,
                    int src1_h,
                    int src1_pitch,
                    int dst_addr,
                    int dst_fmt,                    
                    int dst0_x,
                    int dst0_y,
                    int dst1_x,
                    int dst1_y,
                    int dst_w,
                    int dst_h,
                    int dst_pitch
                    )
{
    gic_opt_struct current_gic_ctl;
    u32 fifo_entries;
    sd_frame_entries_t * entries = &mdrv->shared->sd_frame_entries;

    DPRINTF2("%s\n", __FUNCTION__);

    DPRINTF2("src0_addr = %x, src0_fmt = %x, (%d, %d, %d, %d), src0_pitch = %d\n",
        src0_addr, src0_fmt, src0_x, src0_y, src0_w, src0_h, src0_pitch);

    DPRINTF2("src1_addr = %x, src1_fmt = %x, (%d, %d, %d, %d), src1_pitch = %d\n",
        src1_addr, src1_fmt, src1_x, src1_y, src1_w, src1_h, src1_pitch);

    DPRINTF2("dst_addr = %x, dst_fmt = %x, (%d, %d), (%d, %d), (%d, %d), dst_pitch = %d\n",
        dst_addr, dst_fmt, dst0_x, dst0_y, dst1_x, dst1_y, dst_w, dst_h, dst_pitch);

    if(src0_fmt != DSPF_ARGB || src1_fmt != DSPF_ARGB || dst_fmt != DSPF_AYUV )
        return -1;
    
    if(!(src0_addr && src1_addr && dst_addr))
        return -2;

    if(!dst_w || !dst_h)
        return -3;

    memset(&current_gic_ctl, 0, sizeof(current_gic_ctl));

    //alighment is 8    
    src0_addr = src0_addr + src0_y * src0_pitch + src0_x / 8 * 32;
    src1_addr = src1_addr + src1_y * src1_pitch + src1_x / 8 * 32;

    current_gic_ctl.gic_src1_adr0 = src0_addr;
    current_gic_ctl.gic_src1_h_w = src0_w | (src0_h << GIC1_SRC1_H_SHIFT);
    current_gic_ctl.gic_src1_pitch = src0_pitch;
    current_gic_ctl.gic_src1_y_x = dst0_x | (dst0_y << GIC1_SRC1_Y_SHIFT);

    current_gic_ctl.gic_src1_mode = GIC_AYUV444 | 
                             (ALPHA_24_TO_31 << GIC1_SRC1_A_PACK_ORDER_SHIFT) |
                             (RED_16_TO_23 << GIC1_SRC1_YR_PACK_ORDER_SHIFT) |
                             (GREEN_8_TO_15 << GIC1_SRC1_UG_PACK_ORDER_SHIFT) |
                             (BLUE_0_TO_7 << GIC1_SRC1_VB_PACK_ORDER_SHIFT);

    if(REG_READ(DISP_HD_COMP_ALPHA_MODE) & ALPHA_MODE_G0_MASK)
    {
        current_gic_ctl.gic_alpha_mode_src1 = GIC_ALPHA_MODE_PERPIXEL | GIC_ALPHA_MODE_NOPREMULTIPLY;
    }
    else
    {
        current_gic_ctl.gic_alpha_mode_src1 = GIC_ALPHA_MODE_PERPIXEL;
    }

    current_gic_ctl.gic_src2_adr0 = src1_addr;
    current_gic_ctl.gic_src2_h_w = src1_w | (src1_h << GIC1_SRC2_H_SHIFT);
    current_gic_ctl.gic_src2_pitch = src1_pitch;
    current_gic_ctl.gic_src2_y_x = dst1_x | (dst1_y << GIC1_SRC2_Y_SHIFT);
    
    current_gic_ctl.gic_src2_mode = GIC_AYUV444 | 
                             (ALPHA_24_TO_31 << GIC1_SRC2_A_PACK_ORDER_SHIFT) |
                             (RED_16_TO_23 << GIC1_SRC2_YR_PACK_ORDER_SHIFT) |
                             (GREEN_8_TO_15 << GIC1_SRC2_UG_PACK_ORDER_SHIFT) |
                             (BLUE_0_TO_7 << GIC1_SRC2_VB_PACK_ORDER_SHIFT);

    current_gic_ctl.gic_alpha_mode_src2 = GIC_ALPHA_MODE_PERPIXEL | GIC1_SRC2_ALPHA_PREMUL_MASK;

    current_gic_ctl.gic_backgrd_color = GIC_BACKGROUND_COLOR;

    current_gic_ctl.gic_dst_adr0 = dst_addr;
    current_gic_ctl.gic_dst_h_w = dst_w | (dst_h << GIC1_DST_H_SHIFT);
    current_gic_ctl.gic_dst_pitch = dst_pitch;
    current_gic_ctl.gic_dst_y_x = 0;
    
    current_gic_ctl.gic_dst_mode = GIC_AYUV444;
    
    current_gic_ctl.gic_trigger_command = GIC1_OP_KICKOFF_MASK | 
                                             (SRC1_OVER_SRC2_OVER_BACKGROUND << GIC1_BLEND_LEVELS_SHIFT)  | 
                                             GIC1_BLEND_EN_MASK |
                                             GIC1_CMD_RBM_DIRECT_EN_MASK |
                                             (GIC1_BLEND_DOMAIN_YCBCR << GIC1_BLEND_DOMAIN_SHIFT) |
                                             (GIC1_CSC_SEL_SRC1 << GIC1_CSC_SRC_SEL_SHIFT);

    gic_opt_struct * gic_ctl_ptr = &current_gic_ctl;

    //make sure GIC1 command fifo is enough
    WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 8);
    
    GIC1_REG_WRITE(GIC1_SRC1_ADR0, gic_ctl_ptr->gic_src1_adr0);
    GIC1_REG_WRITE(GIC1_SRC1_ADR1, gic_ctl_ptr->gic_src1_adr1);
    GIC1_REG_WRITE(GIC1_SRC1_ADR2, gic_ctl_ptr->gic_src1_adr2);
    GIC1_REG_WRITE(GIC1_SRC1_H_W, gic_ctl_ptr->gic_src1_h_w);  
    GIC1_REG_WRITE(GIC1_SRC1_PITCH, gic_ctl_ptr->gic_src1_pitch);
    GIC1_REG_WRITE(GIC1_SRC1_Y_X, gic_ctl_ptr->gic_src1_y_x);
    GIC1_REG_WRITE(GIC1_SRC1_MODE, gic_ctl_ptr->gic_src1_mode); 
    GIC1_REG_WRITE(GIC1_ALPHA_MODE_SRC1, gic_ctl_ptr->gic_alpha_mode_src1);

    //make sure GIC1 command fifo is enough
    WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 16);

    //configure src2 as graphic background, src2 will be overwritten by src1
    GIC1_REG_WRITE(GIC1_SRC2_ADR0, gic_ctl_ptr->gic_src2_adr0);
    GIC1_REG_WRITE(GIC1_SRC2_ADR1, gic_ctl_ptr->gic_src2_adr1);
    GIC1_REG_WRITE(GIC1_SRC2_ADR2, gic_ctl_ptr->gic_src2_adr2);
    GIC1_REG_WRITE(GIC1_SRC2_H_W, gic_ctl_ptr->gic_src2_h_w);
    GIC1_REG_WRITE(GIC1_SRC2_Y_X, gic_ctl_ptr->gic_src2_y_x);
    GIC1_REG_WRITE(GIC1_SRC2_PITCH, gic_ctl_ptr->gic_src2_pitch);
    GIC1_REG_WRITE(GIC1_SRC2_MODE, gic_ctl_ptr->gic_src2_mode); 
    GIC1_REG_WRITE(GIC1_ALPHA_MODE_SRC2, gic_ctl_ptr->gic_alpha_mode_src2);

    if(entries->color_space == DSCS_709)
    {
        /* Set CSC for RGB->YCbCr 709:          */
        /*  [  0.182586   0.614231   0.062007   */
        /*    -0.100644  -0.338572   0.439216   */
        /*     0.439216  -0.398942  -0.040274 ] */
        /*                                      */
        /*  Hardware Settings:                  */
        /*    [ 05D8 13A8 01FC                  */
        /*      FCC8 F52A 0E0E                  */
        /*      0E0E F33C FEB6 ]                */
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM0, 0x13a805d8);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM1, 0xfcc801fc);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM2, 0x0e0ef52a);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM3, 0xf33c0e0e);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM4, 0x0000feb6);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM5, 0x00800080);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM6, 0xff00ff00);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM7, 0x0000ff00);
    }
    else
    {
        /* Set CSC for RGB->YCbCr 601:          */
        /*   [ 0.256788   0.504129   0.097906 */
        /* -0.148223  -0.290993   0.439216 */
        /*  0.439216  -0.367788  -0.071427] */
        /*                                      */
        /*  Hardware Settings:                  */
        /* [0838 1022 0322                      */
        /* FB42 F6B0 0E0E                       */
        /* 0E0E F43B FDB7]                      */
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM0, 0x10220838);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM1, 0xfb420322);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM2, 0x0e0ef6b0);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM3, 0xf43b0e0e);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM4, 0x0000fdb7);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM5, 0x00800080);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM6, 0xff00ff00);
        GIC1_REG_WRITE(GIC1_SRC_CSC_PARM7, 0x0000ff00);
    }

    //make sure GIC0 command fifo is enough
    WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 8);
    GIC1_REG_WRITE(GIC1_BACKGRD_COLOR, gic_ctl_ptr->gic_backgrd_color);
    GIC1_REG_WRITE(GIC1_DST_ADR0, gic_ctl_ptr->gic_dst_adr0);
    GIC1_REG_WRITE(GIC1_DST_ADR1, gic_ctl_ptr->gic_dst_adr1);
    GIC1_REG_WRITE(GIC1_DST_ADR2, gic_ctl_ptr->gic_dst_adr2);
    GIC1_REG_WRITE(GIC1_DST_H_W, gic_ctl_ptr->gic_dst_h_w);
    GIC1_REG_WRITE(GIC1_DST_PITCH, gic_ctl_ptr->gic_dst_pitch);
    GIC1_REG_WRITE(GIC1_DST_Y_X, gic_ctl_ptr->gic_dst_y_x);
    GIC1_REG_WRITE(GIC1_DST_MODE, gic_ctl_ptr->gic_dst_mode);

    //make sure GIC0 command fifo is enough
    WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 1);

    //start to kick off gic1 to do the combine
    GIC1_REG_WRITE(GIC1_CONFIG_CTRL, gic_ctl_ptr->gic_trigger_command);

    return 0;
}

static int xcode4_gic_over(
                    pxc_dfb_driver_data mdrv,
                    int src0_addr,
                    int src0_fmt,
                    int src0_x,
                    int src0_y,
                    int src0_w,
                    int src0_h,
                    int src0_pitch,
                    int src1_addr,
                    int src1_fmt,
                    int src1_x,
                    int src1_y,
                    int src1_w,
                    int src1_h,
                    int src1_pitch,
                    int dst_addr,
                    int dst_fmt,                    
                    int dst0_x,
                    int dst0_y,
                    int dst1_x,
                    int dst1_y,
                    int dst_w,
                    int dst_h,
                    int dst_pitch,
                    u32 * palette,
                    int palette_num
                    )
{
    gic_opt_struct current_gic_ctl;
    u32 fifo_entries;

    DPRINTF2("%s\n", __FUNCTION__);

    DPRINTF2("src0_addr = %x, src0_fmt = %x, (%d, %d, %d, %d), src0_pitch = %d\n",
        src0_addr, src0_fmt, src0_x, src0_y, src0_w, src0_h, src0_pitch);

    DPRINTF2("src1_addr = %x, src1_fmt = %x, (%d, %d, %d, %d), src1_pitch = %d\n",
        src1_addr, src1_fmt, src1_x, src1_y, src1_w, src1_h, src1_pitch);

    DPRINTF2("dst_addr = %x, dst_fmt = %x, (%d, %d), (%d, %d), (%d, %d), dst_pitch = %d\n",
        dst_addr, dst_fmt, dst0_x, dst0_y, dst1_x, dst1_y, dst_w, dst_h, dst_pitch);
    
    if(!(src0_addr && src1_addr && dst_addr))
        return -2;

    if(!dst_w || !dst_h)
        return -3;

    memset(&current_gic_ctl, 0, sizeof(current_gic_ctl));

    //alighment is 8    
    src0_addr = src0_addr + src0_y * src0_pitch + src0_x / 8 * 32;
    src1_addr = src1_addr + src1_y * src1_pitch + src1_x / 8 * 32;

    current_gic_ctl.gic_src1_adr0 = src0_addr;
    current_gic_ctl.gic_src1_h_w = src0_w | (src0_h << GIC1_SRC1_H_SHIFT);
    current_gic_ctl.gic_src1_pitch = src0_pitch;
    current_gic_ctl.gic_src1_y_x = dst0_x | (dst0_y << GIC1_SRC1_Y_SHIFT);

    current_gic_ctl.gic_src1_mode = GIC_ARGB8888 | 
                             (ALPHA_24_TO_31 << GIC1_SRC1_A_PACK_ORDER_SHIFT) |
                             (RED_16_TO_23 << GIC1_SRC1_YR_PACK_ORDER_SHIFT) |
                             (GREEN_8_TO_15 << GIC1_SRC1_UG_PACK_ORDER_SHIFT) |
                             (BLUE_0_TO_7 << GIC1_SRC1_VB_PACK_ORDER_SHIFT);

    if(current_gic_ctl.gic_src1_h_w == 0)
    {
        current_gic_ctl.gic_src1_h_w = 1;
        current_gic_ctl.gic_alpha_mode_src1 = GIC_ALPHA_MODE_CONSTANT;
    }
    else
    {
        current_gic_ctl.gic_alpha_mode_src1 = GIC_ALPHA_MODE_PERPIXEL;
    }

    if(REG_READ(DISP_HD_COMP_ALPHA_MODE) & ALPHA_MODE_G0_MASK)
    {
        current_gic_ctl.gic_alpha_mode_src1 |= GIC_ALPHA_MODE_NOPREMULTIPLY;
    }

    current_gic_ctl.gic_src2_adr0 = src1_addr;
    current_gic_ctl.gic_src2_h_w = src1_w | (src1_h << GIC1_SRC2_H_SHIFT);
    current_gic_ctl.gic_src2_pitch = src1_pitch;
    current_gic_ctl.gic_src2_y_x = dst1_x | (dst1_y << GIC1_SRC2_Y_SHIFT);
    
    current_gic_ctl.gic_src2_mode = ((src1_fmt == DSPF_LUT8) ? GIC_LUT8 : GIC_ARGB8888) |
                             (ALPHA_24_TO_31 << GIC1_SRC2_A_PACK_ORDER_SHIFT) |
                             (RED_16_TO_23 << GIC1_SRC2_YR_PACK_ORDER_SHIFT) |
                             (GREEN_8_TO_15 << GIC1_SRC2_UG_PACK_ORDER_SHIFT) |
                             (BLUE_0_TO_7 << GIC1_SRC2_VB_PACK_ORDER_SHIFT);

    current_gic_ctl.gic_alpha_mode_src2 = GIC_ALPHA_MODE_PERPIXEL;

    current_gic_ctl.gic_dst_adr0 = dst_addr;
    current_gic_ctl.gic_dst_h_w = dst_w | (dst_h << GIC1_DST_H_SHIFT);
    current_gic_ctl.gic_dst_pitch = dst_pitch;
    current_gic_ctl.gic_dst_y_x = 0;
    
    current_gic_ctl.gic_dst_mode = GIC_ARGB8888;
    
    current_gic_ctl.gic_trigger_command = GIC1_OP_KICKOFF_MASK | 
                                             GIC1_BLEND_EN_MASK |
                                             GIC1_CMD_RBM_DIRECT_EN_MASK |
                                             (GIC1_BLEND_DOMAIN_RGB << GIC1_BLEND_DOMAIN_SHIFT);

    gic_opt_struct * gic_ctl_ptr = &current_gic_ctl;

    if(palette_num)
    {
        int i;

        WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 1);
        
        GIC1_REG_WRITE(GIC1_CONFIG_CTRL,
            GIC_PROGRAM_ALL_CLUT << GIC1_SRC_CLUT_PROG_EN_SHIFT
        );
        
        for (i = 0 ; i < palette_num; i ++)
        {
            WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 1);
            GIC1_REG_WRITE(GIC1_LUT_PROGM, palette[i]);
        }
    }

    //make sure GIC1 command fifo is enough
    WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 8);
    
    GIC1_REG_WRITE(GIC1_SRC1_ADR0, gic_ctl_ptr->gic_src1_adr0);
    GIC1_REG_WRITE(GIC1_SRC1_ADR1, gic_ctl_ptr->gic_src1_adr1);
    GIC1_REG_WRITE(GIC1_SRC1_ADR2, gic_ctl_ptr->gic_src1_adr2);
    GIC1_REG_WRITE(GIC1_SRC1_H_W, gic_ctl_ptr->gic_src1_h_w);  
    GIC1_REG_WRITE(GIC1_SRC1_PITCH, gic_ctl_ptr->gic_src1_pitch);
    GIC1_REG_WRITE(GIC1_SRC1_Y_X, gic_ctl_ptr->gic_src1_y_x);
    GIC1_REG_WRITE(GIC1_SRC1_MODE, gic_ctl_ptr->gic_src1_mode); 
    GIC1_REG_WRITE(GIC1_ALPHA_MODE_SRC1, gic_ctl_ptr->gic_alpha_mode_src1);

    //make sure GIC1 command fifo is enough
    WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 8);

    //configure src2 as graphic background, src2 will be overwritten by src1
    GIC1_REG_WRITE(GIC1_SRC2_ADR0, gic_ctl_ptr->gic_src2_adr0);
    GIC1_REG_WRITE(GIC1_SRC2_ADR1, gic_ctl_ptr->gic_src2_adr1);
    GIC1_REG_WRITE(GIC1_SRC2_ADR2, gic_ctl_ptr->gic_src2_adr2);
    GIC1_REG_WRITE(GIC1_SRC2_H_W, gic_ctl_ptr->gic_src2_h_w);
    GIC1_REG_WRITE(GIC1_SRC2_Y_X, gic_ctl_ptr->gic_src2_y_x);
    GIC1_REG_WRITE(GIC1_SRC2_PITCH, gic_ctl_ptr->gic_src2_pitch);
    GIC1_REG_WRITE(GIC1_SRC2_MODE, gic_ctl_ptr->gic_src2_mode); 
    GIC1_REG_WRITE(GIC1_ALPHA_MODE_SRC2, gic_ctl_ptr->gic_alpha_mode_src2);

    //make sure GIC0 command fifo is enough
    WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 8);
    GIC1_REG_WRITE(GIC1_BACKGRD_COLOR, gic_ctl_ptr->gic_backgrd_color);
    GIC1_REG_WRITE(GIC1_DST_ADR0, gic_ctl_ptr->gic_dst_adr0);
    GIC1_REG_WRITE(GIC1_DST_ADR1, gic_ctl_ptr->gic_dst_adr1);
    GIC1_REG_WRITE(GIC1_DST_ADR2, gic_ctl_ptr->gic_dst_adr2);
    GIC1_REG_WRITE(GIC1_DST_H_W, gic_ctl_ptr->gic_dst_h_w);
    GIC1_REG_WRITE(GIC1_DST_PITCH, gic_ctl_ptr->gic_dst_pitch);
    GIC1_REG_WRITE(GIC1_DST_Y_X, gic_ctl_ptr->gic_dst_y_x);
    GIC1_REG_WRITE(GIC1_DST_MODE, gic_ctl_ptr->gic_dst_mode);

    //make sure GIC0 command fifo is enough
    WAIT_FOR_GIC1_FIFO_ENTRIES(fifo_entries, 1);

    //start to kick off gic1 to do the combine
    GIC1_REG_WRITE(GIC1_CONFIG_CTRL, gic_ctl_ptr->gic_trigger_command);

    return 0;
}

static int xcode4_fes_scale(   
                    pxc_dfb_driver_data mdrv,
                    int src_y_addr,
                    int src_cb_addr,
                    int src_cr_addr,
                    int src_fmt,
                    int src_x,
                    int src_y,
                    int src_w,
                    int src_h,
                    int src_pitch,
                    int dst_addr,
                    int dst_fmt,
                    int dst_x,
                    int dst_y,
                    int dst_w,
                    int dst_h,
                    int dst_pitch,
                    int src_buf_width
                    )
{
    u64 ratio_result;
	u32	dst_subfld;
    
    DPRINTF2("%s\n", __FUNCTION__);

    DPRINTF2("src_addr = %x | %x | %x, src_fmt = %x, (%d, %d, %d, %d), src_pitch = %d, src_buf_width = %d\n",
        src_y_addr, src_cb_addr, src_cr_addr, src_fmt, src_x, src_y, src_w, src_h, src_pitch, src_buf_width);
    
    DPRINTF2("dst_addr = %x, dst_fmt = %x, (%d, %d, %d, %d), dst_pitch = %d\n",
        dst_addr, dst_fmt, dst_x, dst_y, dst_w, dst_h, dst_pitch);

    if(src_x)
    {
#ifdef FES_SHIFT
        int src_x64 = (src_x) >> 6 << 6;
#else
        int src_x64 = (src_x + 32) >> 6 << 6;
#endif
        src_y_addr += src_x64;
        src_cb_addr += (src_x64 >> 1);
        src_cr_addr += (src_x64 >> 1);
    }
    
    if(src_y)
    {
        int src_y64 = (src_y + 1) >> 2 << 2;
        src_y_addr += src_y64 * src_pitch;
        src_cb_addr += src_y64 * src_pitch / 4;
        src_cr_addr += src_y64 * src_pitch / 4;
    }

#ifdef FES_SHIFT
    int offset = src_x - (src_x >> 6 << 6);
    //int delta = src_buf_width - src_w - src_x;
    //int delta   =   0;
    
    u32 tmp = REG_READ(FES1_NUMTAPS_HELD);
    tmp &= ~(FES1_NUMTAPS_HELD_FES1_HTAPS_MASK | FES1_NUMTAPS_HELD_FES1_HTAPS_UV_MASK);
    tmp |= ((5 + offset) << FES1_NUMTAPS_HELD_FES1_HTAPS_SHIFT) | ((5 + offset / 2) << FES1_NUMTAPS_HELD_FES1_HTAPS_UV_SHIFT);
    Wait_For_Fes1_Fifo(1);
    FES1_REG_WRITE(FES1_NUMTAPS_HELD, tmp) ;
    
    /* Let us try and see if adding offset to src_w will fix our issue */
    /* 
    *   It might be a good idea to bound src_w, but since offset will never be 64 or greater,
    *   it may not be worth it, we just want the FES to read in extra source data for each line.
    */
    src_w+=offset;
    
    if (src_w & 0x1)
    {
        src_w++;
    }
#endif

    Wait_For_Fes1_Fifo(16);
    FES1_REG_WRITE(FES1_DST_PITCH, dst_pitch) ;
    FES1_REG_WRITE(FES1_DST_H_W, dst_w | (dst_h << FES1_DST_H_SHIFT)) ; 
    FES1_REG_WRITE(FES1_DST_MODE, 0x00000E);
    FES1_REG_WRITE(FES1_DST_UV_H_W, 0x00);
    FES1_REG_WRITE(FES1_DST_ADR0, dst_addr); 

    //  DST_SUBFLD_W = even(floor(238*(destination_width/source_width)))
    //if ($fes_numtaps_held_hz > 12) 
    //{
    //    $subfld_base = 238 - $fes_numtaps_held_hz ;
    //} else {
    //    $subfld_base = 238;
    //}
    //
    //$dst_subfld = floor($subfld_base / $inv_sratio_fld / 2) * 2 - 2;
    //numtaps_held = 5 + offset

    ratio_result    =       (1 << FES_RATIO_SHIFT) * dst_w;            //46.18
    ratio_result    =       ratio_result / src_w;
#ifdef FES_SHIFT
    if ( (5+offset) > 12)
    {
        ratio_result    =       ratio_result * (238 - (5 + offset));
    }
    else
#endif
    {
        ratio_result    =   ratio_result * 238;
    }
    ratio_result    >>=     FES_RATIO_SHIFT;
    ratio_result &= ~0x1;
    dst_subfld	=	(u32)ratio_result;

    if ( (dst_subfld % (dst_subfld - 2) <= 4))
    {
        dst_subfld-=2;
        /* This is a restriction on the DST_SUBFLD to prevent corruption on output and hanging ... */
    }

    FES1_REG_WRITE(FES1_DST_SUBFLD, dst_subfld);
    //FES1_REG_WRITE(FES1_DST_SUBFLD, 0x46);

    FES1_REG_WRITE(FES1_SRC1_Y_MINMAX, (1 << FES1_SRC1_Y_MINMAX_MIN_Y_SHIFT ) | (0xff << FES1_SRC1_Y_MINMAX_MAX_Y_SHIFT)  );
    FES1_REG_WRITE(FES1_SRC1_UV_H_W, ((src_w / 2) | (( src_h / 2) <<  FES1_SRC1_UV_H_SHIFT)));
    FES1_REG_WRITE(FES1_SRC1_PITCH,  (src_pitch)  | ((src_pitch / 2 / FES_ALIGN_DIV) << FES1_SRC1_UV_PITCH_SHIFT));
    FES1_REG_WRITE(FES1_SRC1_H_W, src_w | (src_h << FES1_SRC1_H_SHIFT)) ; //
    FES1_REG_WRITE(FES1_SRC1_MODE, 0xE4000001);
    FES1_REG_WRITE(FES1_SRC1_ADR0, src_y_addr ); 
    FES1_REG_WRITE(FES1_SRC1_ADR1, src_cb_addr ); 
    FES1_REG_WRITE(FES1_SRC1_ADR2, src_cr_addr ); 
    FES1_REG_WRITE(FES1_ALPHA_MODE_SRC1, 0);

    FES1_REG_WRITE(FES1_CONFIG_CTRL, 0x80000000);
}

static int XcodeFESScale(pxc_dfb_driver_data mdrv, int src_y_addr, int src_cb_addr, int src_cr_addr,
                                        int src_pitch, int src_x, int src_y, int src_w, int src_h,
                                        int dst_addr, int dst_pitch, int dst_x, int dst_y, int dst_w, int dst_h, int src_buf_width)
{
    if(src_w != mdrv->shared->sd_frame_entries.curr_vid_src.w ||
        src_h != mdrv->shared->sd_frame_entries.curr_vid_src.h ||
        dst_w != mdrv->shared->sd_frame_entries.curr_vid_dst.w ||
        dst_h != mdrv->shared->sd_frame_entries.curr_vid_dst.h)
    {
        update_fes(mdrv, src_w, src_h, dst_w, dst_h);
            
        mdrv->shared->sd_frame_entries.curr_vid_src.w = src_w;
        mdrv->shared->sd_frame_entries.curr_vid_src.h = src_h;
        mdrv->shared->sd_frame_entries.curr_vid_dst.w = dst_w;
        mdrv->shared->sd_frame_entries.curr_vid_dst.h = dst_h;
    }

    return xcode4_fes_scale(mdrv, src_y_addr, src_cb_addr, src_cr_addr, DSPF_I420, src_x, src_y, src_w, src_h, src_pitch,
        dst_addr, DSPF_ARGB, dst_x, dst_y, dst_w, dst_h, dst_pitch, src_buf_width);
}

static int XcodeG2DScale(pxc_dfb_driver_data mdrv, int src_phy, int src_format, int src_pitch, int src_x, int src_y, int src_w, int src_h,
                                        int dst_phy, int dst_format, int dst_pitch, int dst_x, int dst_y, int dst_w, int dst_h)
{
    DPRINTF2("%s\n", __FUNCTION__);

    DPRINTF2("src_addr = %x, src_fmt = %x, (%d, %d, %d, %d), src_pitch = %d, src_buf_width = %d\n",
        src_phy, src_format, src_x, src_y, src_w, src_h, src_pitch, src_pitch);
        
    DPRINTF2("dst_addr = %x, dst_fmt = %x, (%d, %d, %d, %d), dst_pitch = %d\n",
        dst_phy, dst_format, dst_x, dst_y, dst_w, dst_h, dst_pitch);

    pthread_mutex_lock( &mdrv->shared->g2d_mutex );
    
    xcode4_g2d_scale_and_rotate(mdrv, src_phy, src_format, src_x, src_y, src_w, src_h, src_pitch, dst_phy, dst_format, dst_x, dst_y, dst_w, dst_h, dst_pitch, 0);

    pthread_mutex_unlock( &mdrv->shared->g2d_mutex );

    return 0;
}

static int XcodeGICCopy2(pxc_dfb_driver_data mdrv, int src_base, int src_fmt, int dst_base, DFBRectangle * src_rect, DFBRectangle * dst_rect, int screen_width, int screen_height)
{
    CoreSurfaceBufferLock dst_lock, src_lock;

    int src_phy = src_base;
    int src_x = src_rect->x;
    int src_y = src_rect->y;
    int src_w = src_rect->w;
    int src_h = src_rect->h;
    int src_pitch = SD_OUTPUT_PITCH * 2;

    int dst_phy = dst_base;
    int dst_x = dst_rect->x;
    int dst_y = dst_rect->y;
    int dst_w = dst_rect->w;
    int dst_h = dst_rect->h;
    int dst_pitch = SD_OUTPUT_PITCH;

    xcode4_gic_write(mdrv, src_phy, src_fmt, src_x, src_y, src_w, src_h, src_pitch,
        dst_phy, DSPF_AYUV, dst_x, dst_y, screen_width, screen_height, dst_pitch);

    return 0;
}

static int XcodeGICMerge2(pxc_dfb_driver_data mdrv, int src1_base, int src2_base, int dst_base, DFBRectangle * src1_rect, DFBRectangle * dst1_rect, DFBRectangle * src2_rect, DFBRectangle * dst2_rect, int screen_width, int screen_height)
{
    CoreSurfaceBufferLock dst_lock, src_lock;

    int src1_phy = src1_base;
    int src1_x = src1_rect->x;
    int src1_y = src1_rect->y;
    int src1_w = src1_rect->w;
    int src1_h = src1_rect->h;
    int src1_pitch = SD_OUTPUT_PITCH * 2;

    int src2_phy = src2_base;
    int src2_x = src2_rect->x;
    int src2_y = src2_rect->y;
    int src2_w = src2_rect->w;
    int src2_h = src2_rect->h;
    int src2_pitch = SD_OUTPUT_PITCH * 2;

    int dst_phy = dst_base;
    int dst1_x = dst1_rect->x;
    int dst1_y = dst1_rect->y;
    int dst2_x = dst2_rect->x;
    int dst2_y = dst2_rect->y;

    xcode4_gic_merge(mdrv, 
        src1_phy, DSPF_ARGB, src1_x, src1_y, src1_w, src1_h, src1_pitch, 
        src2_phy, DSPF_ARGB, src2_x, src2_y, src2_w, src2_h, src2_pitch, 
        dst_phy, DSPF_AYUV,
        dst1_x, dst1_y,
        dst2_x, dst2_y,
        screen_width,
        screen_height,
        SD_OUTPUT_PITCH);

    return 0;
}

static int XcodeConvHDtoSD(CoreLayer *layer,
                                         const DFBRegion *update,
                                         CoreSurfaceBufferLock *src_lock)
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) layer->driver_data;

    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer->layer_data;

    sd_frame_buffer_t * buffer = &mdrv->shared->sd_frame_entries.buffer[layer->shared->layer_id];

    //check if layer is enabled
    if((dfb_config->soft_mirror_sd & (1 << layer->shared->layer_id)) == 0)
    {
        return -1;
    }

    switch(layer->shared->layer_id)
    {
        case XCODE4_GRA0_LAYER_ID:
            if(m_layer_data->format != DSPF_ARGB || m_layer_data->source_height > 1920)
            {
                return -1;
            }
            break;
        case XCODE4_GRA1_LAYER_ID:
            if(m_layer_data->format != DSPF_LUT8 && m_layer_data->format != DSPF_ARGB  || m_layer_data->source_height > 1920)
            {
                return -1;
            }
            break;
        case XCODE4_VID0_LAYER_ID:
            if(m_layer_data->format != DSPF_I420)
            {
                return -1;
            }
            break;
        default:
            return -1;
    }

    if(buffer->sd_buffer_num < buffer->sd_buffer_max)
    {
        int src_phy, src_pitch, src_x, src_y, src_w, src_h, src_cb, src_cr;
        int dst_phy, dst_pitch, dst_x, dst_y, dst_w, dst_h;
        int hd_width, hd_height, sd_width, sd_height;

        //assume input is always progressive
        switch(mdrv->shared->current_hd_resolution)
        {
            case  DSOR_640_480:
                hd_width = 640;
                hd_height = 480;
                break;
            case  DSOR_720_480:
            case  DSOR_720_480_3D_SIDE_BY_SIDE:
            case  DSOR_720_480_3D_TOP_AND_BOT:
                hd_width = 720;
                hd_height = 480;
                break;
            case  DSOR_720_576:
            case  DSOR_720_576_3D_SIDE_BY_SIDE:
            case  DSOR_720_576_3D_TOP_AND_BOT:
                hd_width = 720;
                hd_height = 576;
                break;
            case  DSOR_1280_720:
            case  DSOR_1280_720_3D_SIDE_BY_SIDE:
            case  DSOR_1280_720_3D_TOP_AND_BOT:
                hd_width = 1280;
                hd_height = 720;
                break;
            case  DSOR_1920_1080:
            case  DSOR_1920_1080_3D_SIDE_BY_SIDE:
            case  DSOR_1920_1080_3D_TOP_AND_BOT:
                hd_width = 1920;
                hd_height = 1080;
                break;
            default:
                DPRINTF("Unsupported HD resolution %x\n", mdrv->shared->current_hd_resolution);
                return -1;
        }

        switch(mdrv->shared->current_sd_resolution)
        {
            case DSOR_720_480:
                sd_width = SD_FRAME_WIDTH;
                sd_height = 480;
                break;
            case DSOR_720_576:
                sd_width = SD_FRAME_WIDTH;
                sd_height = 576;
                break;
            default:
                DPRINTF("Unsupported SD resolution %x\n", mdrv->shared->current_sd_resolution);
                return -1;
        }

        int idx = (buffer->sd_buffer_idx + buffer->sd_buffer_num) % buffer->sd_buffer_max;

        if(src_lock)
        {
            if(layer->shared->layer_id == XCODE4_VID0_LAYER_ID)
            {
                src_phy = m_layer_data->y_start_addr + update->y1 * src_lock->pitch;
                src_cb = m_layer_data->u_start_addr + update->y1 * src_lock->pitch / 4;
                src_cr = m_layer_data->v_start_addr + update->y1 * src_lock->pitch / 4;
                src_pitch = src_lock->pitch;
            }
            else
            {
                src_phy = src_lock->phys;
                src_pitch = src_lock->pitch;
                src_cb = 0;
                src_cr = 0;
            }
        }
        else
        {
            src_phy = REG_READ(DISP_HD_GRA0_ADR);
            src_pitch = (REG_READ(DISP_HD_GRA0_BUF) & 0xFFFF);
            src_cb = 0;
            src_cr = 0;
        }
        
        src_x = m_layer_data->source_x;
        src_y = m_layer_data->source_y;
        src_w = m_layer_data->source_width;
        src_h = m_layer_data->source_height;

        //Check source
        if(src_w <= 64 || src_h <= 64 || src_w > 1920 || src_h > 1080)
        {
            DPRINTF("Invalid SRC %dx%d\n", src_w, src_h);
            return -1;
        }

        //Scaled output
        dst_phy = buffer->sd_buffer_addr[idx];
        dst_pitch = SD_OUTPUT_PITCH;
        dst_x = 0;
        dst_y = 0;
        dst_w = m_layer_data->dest_width * sd_width / hd_width;
        dst_h = m_layer_data->dest_height * sd_height / hd_height;

        if( dst_w > sd_width || dst_h > sd_height )
        {
            dst_w = sd_width;
            dst_h = sd_height;
        }

        //The actual box in source
        buffer->sd_buffer_src[idx].x = 0;
        buffer->sd_buffer_src[idx].y = 0;
        buffer->sd_buffer_src[idx].w = dst_w;
        buffer->sd_buffer_src[idx].h = dst_h;

        //The actual box in destination
        buffer->sd_buffer_rect[idx].x = m_layer_data->x * sd_width / hd_width;
        buffer->sd_buffer_rect[idx].y = m_layer_data->y * sd_height / hd_height;
        buffer->sd_buffer_rect[idx].w = dst_w;
        buffer->sd_buffer_rect[idx].h = dst_h;

        DPRINTF("Convert layer %d to %d/%d, (%d,%d %dx%d) -> (%d,%d %dx%d)\n", layer->shared->layer_id, idx, buffer->sd_buffer_num, src_x, src_y, src_w, src_h, 
            buffer->sd_buffer_rect[idx].x, buffer->sd_buffer_rect[idx].y, buffer->sd_buffer_rect[idx].w, buffer->sd_buffer_rect[idx].h);

        if(m_layer_data->format == DSPF_ARGB)
        {
            XcodeG2DScale(mdrv, src_phy, DSPF_ARGB, src_pitch, src_x, src_y, src_w, src_h, dst_phy, DSPF_ARGB, dst_pitch, dst_x, dst_y, dst_w, dst_h);
            Wait_For_G2d_Idle;
        }
        else if(m_layer_data->format == DSPF_LUT8)
        {
            XcodeG2DScale(mdrv, src_phy, DSPF_LUT8, src_pitch, src_x, src_y, src_w, src_h, dst_phy, DSPF_LUT8, dst_pitch, dst_x, dst_y, dst_w, dst_h);
            Wait_For_G2d_Idle;
        }
        else
        {
            XcodeFESScale(mdrv, src_phy, src_cb, src_cr, src_pitch, src_x, src_y, src_w, src_h, dst_phy, dst_pitch, dst_x, dst_y, dst_w, dst_h, m_layer_data->buf_width);
            Wait_For_Fes1_Idle;

            //Set color space
            buffer->sd_buffer_csp = (src_w <= 720 ? DSCS_601 : DSCS_709);
        }

        buffer->sd_buffer_fmt = m_layer_data->format;
        buffer->sd_buffer_num++;
        buffer->sd_buffer_update = 3;

        return 0;
        
    }
    else
    {
        DPRINTF("DROP %d UPDATE\n", layer->shared->layer_id);
    }
    
    return 1;
};

static int XcodeUpdateBufferIndex(sd_frame_buffer_t * buffer)
{
    if(buffer->sd_buffer_first)
    {
        while(buffer->sd_buffer_num > 1)
        {
            buffer->sd_buffer_idx = (buffer->sd_buffer_idx + 1) % buffer->sd_buffer_max;
            buffer->sd_buffer_num--;
        }
        
        buffer->sd_buffer_first = 0;
    }
    else
    {
        if(buffer->sd_buffer_num > 1)
        {
            buffer->sd_buffer_idx = (buffer->sd_buffer_idx + 1) % buffer->sd_buffer_max;
            buffer->sd_buffer_num--;
        }
    }

    return buffer->sd_buffer_idx; 
}

static int XcodeGetFieldRect(const DFBRectangle * i_rect, DFBRectangle * o_rect, int field)
{
    o_rect->x = i_rect->x;
    o_rect->w = i_rect->w;
    o_rect->y = i_rect->y / 2;
    o_rect->h = i_rect->h / 2;
    return 0;
}

static int XcodeUpdateField(pxc_dfb_driver_data mdrv, 
                                                                    sd_frame_buffer_t * buffer1, 
                                                                    sd_frame_buffer_t * buffer2, 
                                                                    int update_field,
                                                                    int sd_width,
                                                                    int sd_height,
                                                                    int output_index)
{
    DFBRectangle * srect1, *drect1, * srect2, *drect2;
    DFBRectangle new_srect1, new_srect2, new_drect1, new_drect2;
    int input_index1;
    int input_index2;

    if(buffer1)
    {
        input_index1 = XcodeUpdateBufferIndex(buffer1);
        srect1 = &buffer1->sd_buffer_src[input_index1];
        drect1 = &buffer1->sd_buffer_rect[input_index1];
    }

    if(buffer2)
    {
        input_index2 = XcodeUpdateBufferIndex(buffer2);
        srect2 = &buffer2->sd_buffer_src[input_index2];
        drect2 = &buffer2->sd_buffer_rect[input_index2];
    }

    //Update field only
    sd_height >>= 1;

    if(buffer1 && buffer2)
    {
        if(update_field & 0x1)
        {
            XcodeGetFieldRect(srect1, &new_srect1, 0);
            XcodeGetFieldRect(drect1, &new_drect1, 0);
            XcodeGetFieldRect(srect2, &new_srect2, 0);
            XcodeGetFieldRect(drect2, &new_drect2, 0);

            XcodeGICMerge2(mdrv, 
                buffer1->sd_buffer_addr[input_index1],
                buffer2->sd_buffer_addr[input_index2],
                mdrv->shared->sd_frame_entries.base + output_index * SD_OUTPUT_SIZE,
                &new_srect1, 
                &new_drect1,
                &new_srect2, 
                &new_drect2,
                sd_width,
                sd_height);
        }

        if(update_field & 0x2)
        {
            XcodeGetFieldRect(srect1, &new_srect1, 1);
            XcodeGetFieldRect(drect1, &new_drect1, 1);
            XcodeGetFieldRect(srect2, &new_srect2, 1);
            XcodeGetFieldRect(drect2, &new_drect2, 1);

            XcodeGICMerge2(mdrv, 
                buffer1->sd_buffer_addr[input_index1] + SD_OUTPUT_PITCH,
                buffer2->sd_buffer_addr[input_index2] + SD_OUTPUT_PITCH,
                mdrv->shared->sd_frame_entries.base + output_index * SD_OUTPUT_SIZE + SD_FRAME_SIZE,
                &new_srect1, 
                &new_drect1,
                &new_srect2, 
                &new_drect2,
                sd_width,
                sd_height);
        }
    }
    else
    {
        sd_frame_buffer_t * buffer;
        DFBRectangle * srect, * drect;
        int input_index;
        int src_fmt;
        
        if(buffer1)
        {
            buffer = buffer1;
            srect = srect1;
            drect = drect1;
            input_index = input_index1;
            src_fmt = DSPF_ARGB;
        }
        else if(buffer2)
        {
            buffer = buffer2;
            srect = srect2;
            drect = drect2;
            input_index = input_index2;
            src_fmt = DSPF_AYUV;
        }
        else
        {
            return -1;
        }

        if(update_field & 0x1)
        {
            XcodeGetFieldRect(srect, &new_srect1, 0);
            XcodeGetFieldRect(drect, &new_drect1, 0);

            XcodeGICCopy2(mdrv,
                buffer->sd_buffer_addr[input_index],
                src_fmt,
                mdrv->shared->sd_frame_entries.base + output_index * SD_OUTPUT_SIZE,
                &new_srect1,
                &new_drect1,
                sd_width,
                sd_height);
        }

        if(update_field & 0x2)
        {
            XcodeGetFieldRect(srect, &new_srect1, 1);
            XcodeGetFieldRect(drect, &new_drect1, 1);

            XcodeGICCopy2(mdrv,
                buffer->sd_buffer_addr[input_index] + SD_OUTPUT_PITCH,
                src_fmt,
                mdrv->shared->sd_frame_entries.base + output_index * SD_OUTPUT_SIZE + SD_FRAME_SIZE,
                &new_srect1,
                &new_drect1,
                sd_width,
                sd_height);
        }
    }

    return 0;
}

static int XcodeUpdateGraphic(pxc_dfb_driver_data mdrv, 
                                                                    sd_frame_buffer_t * buffer1, 
                                                                    sd_frame_buffer_t * buffer2, 
                                                                    sd_frame_buffer_t * dst,
                                                                    int sd_width,
                                                                    int sd_height)
{    
    DFBRectangle * srect1, *drect1, * srect2, *drect2;
    DFBRectangle zero_rect = (DFBRectangle){0, 0, 0, 0};
    DFBRectangle full_rect = (DFBRectangle){0, 0, sd_width, sd_height};
    int input_index1;
    int input_index2;
    int palette_num = 0;
    u32 * palette = NULL;

    if(buffer1)
    {
        input_index1 = XcodeUpdateBufferIndex(buffer1);
        srect1 = &buffer1->sd_buffer_src[input_index1];
        drect1 = &buffer1->sd_buffer_rect[input_index1];
    }
    else
    {
        buffer1 = buffer2;
        input_index1 = 0;
        srect1 = &zero_rect;
        drect1 = &zero_rect;
    }

    if(buffer2)
    {
        input_index2 = XcodeUpdateBufferIndex(buffer2);
        srect2 = &buffer2->sd_buffer_src[input_index2];
        drect2 = &buffer2->sd_buffer_rect[input_index2];
    }
    else
    {
        return -1;
    }

    if(dst)
    {
        dst->sd_buffer_src[0] = full_rect;
        dst->sd_buffer_rect[0] = full_rect;
    }
    else
    {
        return -1;
    }

    if(buffer2->sd_buffer_fmt == DSPF_LUT8 && mdrv->shared->sd_frame_entries.palette_update)
    {
        pxc_dfb_layer_data layer_data;
        CoreLayer * layer;

        mdrv->shared->sd_frame_entries.palette_update = 0;
        
        layer = dfb_layer_at (XCODE4_GRA1_LAYER_ID);
        if(layer)
        {
            layer_data = (pxc_dfb_layer_data) layer->layer_data;
            palette_num = layer_data->palette_num;
            palette = layer_data->palette;
        }
    }
    
    xcode4_gic_over(mdrv, 
        buffer1->sd_buffer_addr[input_index1], 
        buffer1->sd_buffer_fmt, 
        srect1->x, 
        srect1->y, 
        srect1->w, 
        srect1->h, 
        SD_OUTPUT_PITCH, 
        buffer2->sd_buffer_addr[input_index2], 
        buffer2->sd_buffer_fmt, 
        srect2->x, 
        srect2->y, 
        srect2->w, 
        srect2->h, 
        SD_OUTPUT_PITCH, 
        dst->sd_buffer_addr[0], 
        DSPF_ARGB,
        drect1->x, 
        drect1->y,
        drect2->x, 
        drect2->y,
        sd_width,
        sd_height,
        SD_OUTPUT_PITCH,
        palette,
        palette_num);

    return 0;
}


static int XcodeMergeAll(pxc_dfb_driver_data mdrv, int output_index)
{
    pxc_dfb_layer_data layer_data;

    DFBRectangle rect;

    CoreLayer * layer;

    sd_frame_buffer_t * vid_buffer = NULL, * gra_buffer = NULL, * sub_buffer = NULL;

    int sd_width, sd_height;

    int input_index;

    int i;

    switch(mdrv->shared->current_sd_resolution)
    {
        case DSOR_720_480:
            sd_width = SD_FRAME_WIDTH;
            sd_height = 480;
            break;
        case DSOR_720_576:
            sd_width = SD_FRAME_WIDTH;
            sd_height = 576;
            break;
        default:
            return -1;
    }

    //Check video layer
    layer = (dfb_layer_num() > XCODE4_VID0_LAYER_ID) ? dfb_layer_at (XCODE4_VID0_LAYER_ID) : NULL;
    if(layer)
    {
        layer_data = (pxc_dfb_layer_data) layer->layer_data;
        if(layer_data->layer_is_set && layer_data->alpha)
        {
            vid_buffer = &mdrv->shared->sd_frame_entries.buffer[XCODE4_VID0_LAYER_ID];
            if(!vid_buffer->sd_buffer_num)
            {
                vid_buffer = NULL;
            }
            else if(vid_buffer->sd_buffer_num == 1 && !vid_buffer->sd_buffer_first)
            {
                if(vid_buffer->sd_buffer_update)
                {
                    vid_buffer->sd_buffer_update--;
                }
            }
        }
        else
        {
            vid_buffer = &mdrv->shared->sd_frame_entries.buffer[XCODE4_VID0_LAYER_ID];
            if(vid_buffer->sd_buffer_num > 1)
            {
                if(mdrv->shared->sd_frame_entries.double_frame)
                {
                    XcodeUpdateBufferIndex(vid_buffer);
                }
                else
                {
                    XcodeUpdateBufferIndex(vid_buffer);
                    XcodeUpdateBufferIndex(vid_buffer);
                }
            }
            vid_buffer->sd_buffer_first = 1;
            vid_buffer = NULL;
        }
    }

    //Check graphic layer
    layer = (dfb_layer_num() > XCODE4_GRA0_LAYER_ID) ? dfb_layer_at (XCODE4_GRA0_LAYER_ID) : NULL;
    if(layer)
    {
        layer_data = (pxc_dfb_layer_data) layer->layer_data;
        if(!(REG_READ(DISP_HD_COMP_ALPHA_MODE) & ALPHA_MODE_G0_MASK)  || (REG_READ(DISP_HD_COMP_ALPHA) & ALPHA_G0_MASK))
        {
            gra_buffer = &mdrv->shared->sd_frame_entries.buffer[XCODE4_GRA0_LAYER_ID];
            if(!gra_buffer->sd_buffer_num)
            {
                gra_buffer = NULL;
            }
            else if(gra_buffer->sd_buffer_num == 1 && !gra_buffer->sd_buffer_first)
            {
                if(gra_buffer->sd_buffer_update)
                {
                    gra_buffer->sd_buffer_update--;
                }
            }
        }
        else
        {
            gra_buffer = &mdrv->shared->sd_frame_entries.buffer[XCODE4_GRA0_LAYER_ID];
            if(gra_buffer->sd_buffer_num > 1)
            {
                if(mdrv->shared->sd_frame_entries.double_frame)
                {
                    XcodeUpdateBufferIndex(gra_buffer);
                }
                else
                {
                    XcodeUpdateBufferIndex(gra_buffer);
                    XcodeUpdateBufferIndex(gra_buffer);
                }
            }
            gra_buffer->sd_buffer_first = 1;
            gra_buffer = NULL;
        }
    }

    //Check subtitle layer
    layer = (dfb_layer_num() > XCODE4_GRA1_LAYER_ID) ? dfb_layer_at (XCODE4_GRA1_LAYER_ID) : NULL;
    if(layer)
    {
        layer_data = (pxc_dfb_layer_data) layer->layer_data;
        if(!(REG_READ(DISP_HD_COMP_ALPHA_MODE) & ALPHA_MODE_G1_MASK)  || (REG_READ(DISP_HD_COMP_ALPHA) & ALPHA_G1_MASK))
        {
            sub_buffer = &mdrv->shared->sd_frame_entries.buffer[XCODE4_GRA1_LAYER_ID];
            if(!sub_buffer->sd_buffer_num)
            {
                sub_buffer = NULL;
            }
            else if(sub_buffer->sd_buffer_num == 1 && !sub_buffer->sd_buffer_first)
            {
                if(sub_buffer->sd_buffer_update)
                {
                    sub_buffer->sd_buffer_update--;
                }
            }
        }
        else
        {
            sub_buffer = &mdrv->shared->sd_frame_entries.buffer[XCODE4_GRA1_LAYER_ID];
            if(sub_buffer->sd_buffer_num > 1)
            {
                if(mdrv->shared->sd_frame_entries.double_frame)
                {
                    XcodeUpdateBufferIndex(sub_buffer);
                }
                else
                {
                    XcodeUpdateBufferIndex(sub_buffer);
                    XcodeUpdateBufferIndex(sub_buffer);
                }
            }
            sub_buffer->sd_buffer_first = 1;
            sub_buffer = NULL;
        }
    }

    if((vid_buffer && vid_buffer->sd_buffer_update) || (gra_buffer && gra_buffer->sd_buffer_update) || (sub_buffer && sub_buffer->sd_buffer_update))
    {

        DPRINTF("Merge G(%d) S(%d) V(%d) layers to %d\n", 
            gra_buffer ? gra_buffer->sd_buffer_num : -1, 
            sub_buffer ? sub_buffer->sd_buffer_num : -1,
            vid_buffer ? vid_buffer->sd_buffer_num : -1, 
            output_index);
    }
    else
    {
        return -1;
    }

    //Merge graphic and subtitle
    if(sub_buffer)
    {
        sd_frame_buffer_t * m_buffer = &mdrv->shared->sd_frame_entries.buffer[3];
        
        if(m_buffer->sd_buffer_first || sub_buffer->sd_buffer_first || sub_buffer->sd_buffer_num > 1 ||
            (gra_buffer && (gra_buffer->sd_buffer_first || gra_buffer->sd_buffer_num > 1)))
        {
            XcodeUpdateGraphic(mdrv, gra_buffer, sub_buffer, m_buffer, sd_width, sd_height);

            m_buffer->sd_buffer_first = 0;
        }

        gra_buffer = m_buffer;
    }
    else
    {
        mdrv->shared->sd_frame_entries.buffer[3].sd_buffer_first = 1;
    }

    if(vid_buffer)
    {
        if(mdrv->shared->sd_frame_entries.color_space != vid_buffer->sd_buffer_csp)
        {
            app_ioctl_command app_cmd;
            app_cmd.uIOCTLCommand = CMD_APP_IOCTL_GET_SD_SCREEN_MODE;
            if(!ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd))
            {
               app_cmd.uIOCTLCommand = CMD_APP_IOCTL_SET_SD_SCREEN_MODE;
               app_cmd.dfb_sd_screen_mode.color_space = (vid_buffer->sd_buffer_csp == DSCS_601 ? COLOR_SPACE_YUV601 : COLOR_SPACE_YUV709);
               ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
            }
            mdrv->shared->sd_frame_entries.color_space = vid_buffer->sd_buffer_csp;
        }
    }

    if(mdrv->shared->sd_frame_entries.double_frame)
    {
        XcodeUpdateField(mdrv, gra_buffer, vid_buffer, 3, sd_width, sd_height, output_index);
    }
    else
    {
        XcodeUpdateField(mdrv, gra_buffer, vid_buffer, 1, sd_width, sd_height, output_index);
            
        XcodeUpdateField(mdrv, gra_buffer, vid_buffer, 2, sd_width, sd_height, output_index);
    }

    return 0;
}

static int XcodeFrameUpdated(pxc_dfb_driver_data mdrv)
{
    sd_frame_buffer_t * buffer;

    buffer = &mdrv->shared->sd_frame_entries.buffer[XCODE4_VID0_LAYER_ID];
    if(buffer->sd_buffer_update || buffer->sd_buffer_num > 1)
    {
        return 1;
    }

    buffer = &mdrv->shared->sd_frame_entries.buffer[XCODE4_GRA0_LAYER_ID];
    if(buffer->sd_buffer_update || buffer->sd_buffer_num > 1)
    {
        return 1;
    }

    buffer = &mdrv->shared->sd_frame_entries.buffer[XCODE4_GRA1_LAYER_ID];
    if(buffer->sd_buffer_update || buffer->sd_buffer_num > 1)
    {
        return 1;
    }

    return 0;
}

static int XcodeInitLayerBuffer(pxc_dfb_driver_data mdrv)
{
    sd_frame_entries_t * entries = &mdrv->shared->sd_frame_entries;
    sd_frame_buffer_t * buffer;

    /********************/
    /*    O0      |       G0    */
    /*    O1      |       G1    */
    /*    O2      |       S0    */
    /*    O3      |       S1    */
    /*    V0      |       V3    */
    /*    V1      |       V4    */
    /*    V2      |       M0    */
    /********************/

    buffer = &entries->buffer[XCODE4_GRA0_LAYER_ID];
    buffer->sd_buffer_addr[0] = entries->base + SD_FRAME_PITCH;
    buffer->sd_buffer_addr[1] = buffer->sd_buffer_addr[0] + SD_OUTPUT_SIZE;
    buffer->sd_buffer_first = 1;
    buffer->sd_buffer_max = 2;
    buffer->sd_buffer_update = 0;
    buffer->sd_buffer_idx = 0;
    buffer->sd_buffer_num = 0;
    buffer->sd_buffer_csp = DSCS_NONE;
    memset(buffer->sd_buffer_rect, 0, sizeof(DFBRectangle) * buffer->sd_buffer_max);
    memset(buffer->sd_buffer_src, 0, sizeof(DFBRectangle) * buffer->sd_buffer_max);

    buffer = &entries->buffer[XCODE4_GRA1_LAYER_ID];
    buffer->sd_buffer_addr[0] = entries->base + SD_FRAME_PITCH + SD_OUTPUT_SIZE * 2;
    buffer->sd_buffer_addr[1] = buffer->sd_buffer_addr[0] + SD_OUTPUT_SIZE;
    buffer->sd_buffer_first = 1;
    buffer->sd_buffer_max = 2;
    buffer->sd_buffer_update = 0;
    buffer->sd_buffer_idx = 0;
    buffer->sd_buffer_num = 0;
    buffer->sd_buffer_csp = DSCS_NONE;
    memset(buffer->sd_buffer_rect, 0, sizeof(DFBRectangle) * buffer->sd_buffer_max);
    memset(buffer->sd_buffer_src, 0, sizeof(DFBRectangle) * buffer->sd_buffer_max);

    buffer = &entries->buffer[XCODE4_VID0_LAYER_ID];
    buffer->sd_buffer_addr[0] = entries->base + SD_OUTPUT_SIZE * 4;
    buffer->sd_buffer_addr[1] = buffer->sd_buffer_addr[0] + SD_OUTPUT_SIZE;
    buffer->sd_buffer_addr[2] = buffer->sd_buffer_addr[1] + SD_OUTPUT_SIZE;
    buffer->sd_buffer_addr[3] = entries->base + SD_OUTPUT_SIZE * 4 + SD_FRAME_PITCH;
    buffer->sd_buffer_addr[4] = buffer->sd_buffer_addr[3] + SD_OUTPUT_SIZE;
    buffer->sd_buffer_first = 1;
    buffer->sd_buffer_max = 5;
    buffer->sd_buffer_update = 0;
    buffer->sd_buffer_idx = 0;
    buffer->sd_buffer_num = 0;
    buffer->sd_buffer_csp = DSCS_NONE;
    memset(buffer->sd_buffer_rect, 0, sizeof(DFBRectangle) * buffer->sd_buffer_max);
    memset(buffer->sd_buffer_src, 0, sizeof(DFBRectangle) * buffer->sd_buffer_max);
    
    buffer = &entries->buffer[XCODE4_VID1_LAYER_ID];
    buffer->sd_buffer_addr[0] = entries->base + SD_FRAME_PITCH + SD_OUTPUT_SIZE * 6;
    buffer->sd_buffer_first = 1;
    buffer->sd_buffer_max = 1;
    buffer->sd_buffer_update = 0;
    buffer->sd_buffer_idx = 0;
    buffer->sd_buffer_num = 0;
    buffer->sd_buffer_csp = DSCS_NONE;
    memset(buffer->sd_buffer_rect, 0, sizeof(DFBRectangle) * buffer->sd_buffer_max);
    memset(buffer->sd_buffer_src, 0, sizeof(DFBRectangle) * buffer->sd_buffer_max);

    return 0;
}



/********************************************************************/

static DFBResult XcodeStartHDtoSD(pxc_dfb_driver_data mdrv)
{
    CoreLayer * layer;

    if(!mdrv->shared->sd_frame_entries.start)
    {
        DPRINTF("Start HD to SD\n");

        //Wait for one frame
        usleep(50000);

        //Update buffer address
        XcodeInitLayerBuffer(mdrv);

        //Reset FES
        memset(&mdrv->shared->sd_frame_entries.curr_vid_src, 0, sizeof(DFBRectangle));
        memset(&mdrv->shared->sd_frame_entries.curr_vid_dst, 0, sizeof(DFBRectangle));

        //Reset color space
        switch(REG_READ(DISP_SD_CC))
        {
            case 0x7:
                mdrv->shared->sd_frame_entries.color_space = DSCS_601;
                break;
            case 0x8:
                mdrv->shared->sd_frame_entries.color_space = DSCS_709;
                break;
            default:
                mdrv->shared->sd_frame_entries.color_space = DSCS_NONE;
                break;
        }

        //Enable HD to SD
        mdrv->shared->sd_frame_entries.start = 1;

        //Convert for the fisrt graphic picture
        layer = (dfb_layer_num() > XCODE4_GRA0_LAYER_ID) ? dfb_layer_at (XCODE4_GRA0_LAYER_ID) : NULL;
        if(layer)
        {
            XcodeConvHDtoSD(layer, NULL, NULL);
        }
    }  

    return DFB_OK;
}

static DFBResult XcodeStopHDtoSD(pxc_dfb_driver_data mdrv)
{
    mdrv->shared->sd_frame_entries.start = 0;
        
    DPRINTF("Stop HD to SD\n");
    
    return DFB_OK;
}

DFBResult XcodeFlipSDFrame(CoreLayer *layer, 
                                         const DFBRegion *update,
                                         CoreSurfaceBufferLock *lock)
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) layer->driver_data;

    sd_frame_buffer_t * buffer;

    if(!mdrv->shared->sd_frame_entries.start)
    {
        return 0;
    }

    pthread_mutex_lock(&mdrv->shared->sd_frame_entries.mutex);

#ifdef SD_FLIP_DELAY
    if(layer->shared->layer_id != XCODE4_VID0_LAYER_ID)
    {
        buffer = &mdrv->shared->sd_frame_entries.buffer[XCODE4_VID0_LAYER_ID];
        if(buffer->sd_buffer_num)
        {
            pthread_mutex_unlock(&mdrv->shared->sd_frame_entries.mutex);
            usleep(buffer->sd_buffer_num * 5000);
            pthread_mutex_lock(&mdrv->shared->sd_frame_entries.mutex);
        }
    }
#endif

    buffer = &mdrv->shared->sd_frame_entries.buffer[layer->shared->layer_id];
    while(buffer->sd_buffer_num >= buffer->sd_buffer_max)
    {
        buffer->sd_buffer_idx = (buffer->sd_buffer_idx + 1) % buffer->sd_buffer_max;
        buffer->sd_buffer_num--;
    }

    if(!XcodeConvHDtoSD(layer, update, lock))
    {
        pthread_cond_signal(&mdrv->shared->sd_frame_entries.cond);
    }

    pthread_mutex_unlock(&mdrv->shared->sd_frame_entries.mutex);

    return 0;
}

static void * XcodeMergeThread(void * param)
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data)param;

    app_ioctl_command app_cmd;

    int ret;

    while(!mdrv->shared->sd_frame_entries.terminated)
    {
        app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_GET_SD_FRAME_FLIP_INDEX;

        ret = ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd);

        if(ret >= 0)
        {
            pthread_mutex_lock(&mdrv->shared->sd_frame_entries.mutex);
            
            if(!mdrv->shared->sd_frame_entries.start)
            {
                XcodeStartHDtoSD(mdrv);
            }
            
            if(XcodeFrameUpdated(mdrv))
            {
                int update = 0;

                if(app_cmd.dfb_sd_index.index >= 0)
                {
                    //Update index
                    int output_index = app_cmd.dfb_sd_index.index;
                        
                    //Merge layers
                    if(!XcodeMergeAll(mdrv, output_index))
                    {
                        //Wait for GIC done
                        Wait_For_Gic1_Idle;

                        //Flip
                        app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_SET_SD_FRAME_FLIP_INDEX;
                        app_cmd.dfb_sd_index.index = output_index;
                        ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 

                        update = 1;
                    }
                }

                if(!update)
                {
                    pthread_mutex_unlock(&mdrv->shared->sd_frame_entries.mutex);

                    //Check every 5ms
                    usleep(5000);
                    continue;
                }
            }
            else
            {
                pthread_cond_wait(&mdrv->shared->sd_frame_entries.cond, &mdrv->shared->sd_frame_entries.mutex);
            }

            pthread_mutex_unlock(&mdrv->shared->sd_frame_entries.mutex);
        }
        else
        {
            if(mdrv->shared->sd_frame_entries.start)
            {
                XcodeStopHDtoSD(mdrv);
            }

            //Check every 100ms
            usleep(100000);
        }
    }

    mdrv->shared->sd_frame_entries.terminated = 2;

    return NULL;
}

DFBResult XcodeListenHDtoSD(pxc_dfb_driver_data mdrv)
{
    pthread_mutexattr_t mutex_attr;
    pthread_condattr_t cond_attr;
    struct sched_param priority;

    app_ioctl_command app_cmd;
    app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_GET_SD_BUFFER_ADDR;
    ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
    mdrv->shared->sd_frame_entries.base = app_cmd.dfb_sd_buffer_addr.dfb_start_phys_addr;

    if(!mdrv->shared->sd_frame_entries.base)
    {
        return DFB_FAILURE;
    }

    //Initialize worker thread
    pthread_mutexattr_init( &mutex_attr );
    pthread_mutexattr_setpshared( &mutex_attr, PTHREAD_PROCESS_SHARED );
    pthread_mutex_init(&mdrv->shared->sd_frame_entries.mutex, &mutex_attr);

    pthread_condattr_init( &cond_attr );
    pthread_condattr_setpshared( &cond_attr, PTHREAD_PROCESS_SHARED );
    pthread_cond_init(&mdrv->shared->sd_frame_entries.cond, &cond_attr);

    mdrv->shared->sd_frame_entries.start = 0;
    mdrv->shared->sd_frame_entries.terminated = 0;
    memset(&mdrv->shared->sd_frame_entries.curr_vid_src, 0, sizeof(DFBRectangle));
    memset(&mdrv->shared->sd_frame_entries.curr_vid_dst, 0, sizeof(DFBRectangle));

    DPRINTF("Start listen\n");

    //Start working thread
    pthread_create(&mdrv->shared->sd_frame_entries.thread, NULL, XcodeMergeThread, mdrv);
    priority.sched_priority = sched_get_priority_max(SCHED_RR);
    pthread_setschedparam(mdrv->shared->sd_frame_entries.thread, SCHED_RR, &priority);
    pthread_setname_np(mdrv->shared->sd_frame_entries.thread, __FUNCTION__);

    return DFB_OK;
}

int XcodeSDFullness(CoreLayer *layer)
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) layer->driver_data;

    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer->layer_data;

    sd_frame_buffer_t * buffer = &mdrv->shared->sd_frame_entries.buffer[layer->shared->layer_id];

    int video_level = 16 - (REG_READ(DISP_HD_FIFO_STATUS0) & 0x1F);

    if(!mdrv->shared->sd_frame_entries.start)
    {
        return 0;
    }

    if(buffer->sd_buffer_num < buffer->sd_buffer_max)
    {
        return 0;
    }

    if(video_level < buffer->sd_buffer_max)
    {
        return 0;
    }

    return 1;
}

