/*
   ViXS driver - Graphics Layer

*/

#ifndef __XCODE4_OSD_H__
#define __XCODE4_OSD_H__

#include <core/layers.h>

#define XCODE4_OSD_SUPPORTED_OPTIONS  (DLOP_ALPHACHANNEL | DLOP_OPACITY | DLOP_SRC_COLORKEY)
#define XCODE4_VID_SUPPORTED_OPTIONS  (DLOP_ALPHACHANNEL | DLOP_OPACITY | DLOP_SRC_COLORKEY | DLOP_ROTATE90 | DLOP_ROTATE180 | DLOP_ROTATE270)

#define XCODE_DEFAULT_WIDTH  64
#define XCODE_DEFAULT_HEIGHT 64

#define DEFAULT_BYTES_PER_PIXEL 4 

#define CALC_GRA_SIZE(width,height,BPP) ((width*height*BPP) / 256)

#define CALC_GRA_SIZE_PITCH(width,height,BPP) (( width*BPP) | (((width*height*BPP) / 256) << DISP_HD_GRA0_BUF_SIZE_SHIFT))

#define SCREEN_WIDTH_720X480i 720
#define SCREEN_HEIGHT_720X480i 480

#define SCREEN_WIDTH_720X576i 720
#define SCREEN_HEIGHT_720X576i 576

#define SCREEN_WIDTH_720X480p 720
#define SCREEN_HEIGHT_720X480p 480
							
#define SCREEN_WIDTH_1280X720p 1280
#define SCREEN_HEIGHT_1280X720p 720

#define SCREEN_WIDTH_1920X1080i 1920
#define SCREEN_HEIGHT_1920X1080i 1080

#define SCREEN_WIDTH_1920X1080p 1920
#define SCREEN_HEIGHT_1920X1080p 1080

/* 1 second wait for PHY lock (should be enough...)
*   This is based on the CG_SYSTEM_CNT register in Viper
*/
#define PHY_TIMEOUT     1687500


#define OUTPUT_MODE_480i 0
#define OUTPUT_MODE_480p 1
#define OUTPUT_MODE_576i 2
#define OUTPUT_MODE_720p 3
#define OUTPUT_MODE_1080i 4
#define OUTPUT_MODE_1080p 5

#define HD_GRA0_PLANE 0
#define HD_GRA1_PLANE 1
#define SD_GRA0_PLANE 2
#define SD_GRA1_PLANE 4

#define HD_COMPOSITOR 0
#define SD_COMPOSITOR 1

#define DFB_ALPHA_POS 0x03
#define DFB_RED_POS   0x02
#define DFB_GREEN_POS 0x01
#define DFB_BLUE_POS  0x00

#define DISP_HD_GRA0_FORMAT_MONO 0	
#define DISP_HD_GRA0_FORMAT_CLUT2 1
#define DISP_HD_GRA0_FORMAT_CLUT4 2
#define DISP_HD_GRA0_FORMAT_CLUT8 3
#define DISP_HD_GRA0_FORMAT_RGB1555 4
#define DISP_HD_GRA0_FORMAT_RGB565 5
#define DISP_HD_GRA0_FORMAT_RGB24 6
#define DISP_HD_GRA0_FORMAT_ARGB 7

#define DISP_HD_COMP_SIZE_INTERLACED 1

#define ALPHA_MODE_G0_USE_REGISTER 1

#define ALPHA_MODE_G1_USE_REGISTER 1

#define DISP_HD_COMP_SIZE_OSR 1

#define DISP_HD_COMP_CTRL_COMP_EN 1
#define DISP_SD_COMP_CTRL_COMP_EN 1

#define DISP_HD_COMP_ALPHA_ALPHA_G0_SHIFT 16

#define DEFAULT_FIXED_ALPHA 0x80

#define PPC_RESET 1

#define DISP_RESET 1

#define GIC_RESET 1

#define HDCP_SPIN_WAIT_LIMIT    30

#define SD_TIMING_IN_PICOSECONDS   1000000/13     //not exact, but doesnt have to be(only used for ID)
#define SD_27MHZ_TIMING_IN_PICOSECONDS   1000000/27
#define HD_TIMING_IN_PICOSECONDS   1000000/74

#define INDEX_UNKNOWN 0xFF

#define MAX_OVERLAY_WIDTH (DISP_HD_VID0_SIZE_H_SIZE_MASK >> DISP_HD_VID0_SIZE_H_SIZE_SHIFT)

#define MAX_OVERLAY_HEIGHT (DISP_HD_VID0_SIZE_V_SIZE_MASK >> DISP_HD_VID0_SIZE_V_SIZE_SHIFT)

#define DEFAULT_GRAPHIC_VSCA_START 0x10001

#define DEFAULT_GRAPHIC_VSCA_INIT 0x0

#define DEFAULT_VIDEO_VSCA_INIT0 0xC000C000

#define DEFAULT_VIDEO_VSCA_INIT1 0x0

#define DEFAULT_VIDEO_VSCA_START 0x33343334

#define VID0_VERTICAL_SCALER_TAPS 6
#define VID1_VERTICAL_SCALER_TAPS 4

#define HORIZONAL_SCALER_TAPS 10

#define UPSAMPLING_FACTOR 64


#define DISP_HD_VID0_VSCALER_Y_BASE_ADDRESS 0x10000

#define DISP_HD_VID0_VSCALER_CBCR_BASE_ADDRESS 0x20000

#define DISP_HD_VID0_HSCALER_Y_BASE_ADDRESS 0x30000

#define DISP_HD_VID0_HSCALER_CBCR_BASE_ADDRESS 0x40000

#define DISP_HD_VID1_VSCALER_Y_BASE_ADDRESS 0x90000

#define DISP_HD_VID1_VSCALER_CBCR_BASE_ADDRESS 0xa0000

#define DISP_HD_VID1_HSCALER_Y_BASE_ADDRESS 0xb0000

#define DISP_HD_VID1_HSCALER_CBCR_BASE_ADDRESS 0xc0000

#define FES_UPSAMPLING_FACTOR 64
#define FES0_VERTICAL_SCALER_TAPS 6
#define FES1_VERTICAL_SCALER_TAPS 6
#define FES0_HORIZONAL_SCALER_TAPS 10
#define FES1_HORIZONAL_SCALER_TAPS 10

#define FES0_COEF_VERTICAL_Y_BASE_ADDR 0x00002000
#define FES0_COEF_VERTICAL_CBCR_BASE_ADDR 0x00003000
#define FES0_COEF_HOREIZONAL_Y_BASE_ADDR 0x00000800
#define FES0_COEF_HOREIZONAL_CBCR_BASE_ADDR 0x00000c00

#define FES1_COEF_VERTICAL_Y_BASE_ADDR 0x00002000
#define FES1_COEF_VERTICAL_CBCR_BASE_ADDR 0x00003000
#define FES1_COEF_HOREIZONAL_Y_BASE_ADDR 0x00000800
#define FES1_COEF_HOREIZONAL_CBCR_BASE_ADDR 0x00000C00


#define DISP_HD_GRA_CLUT_DATA_INDEX 0
#define DISP_HD_GRA_CLUT_INDEX 1
#define DISP_HD_GRA_FORMAT_INDEX 2
#define DISP_HD_GRA_FORMAT1_INDEX 3
#define DISP_HD_GRA_BUF_INDEX 4
#define DISP_HD_GRA_SAMPLES_INDEX 5
#define DISP_HD_GRA_ADR_INDEX 6
#define DISP_HD_GRA_SIZE_INDEX 7
#define DISP_HD_GRA_VSCA_INV_RATIO_INDEX 8
#define DISP_HD_GRA_VSCA_INIT_INDEX 9
#define DISP_HD_GRA_VSCA_START_INDEX 10
#define DISP_HD_GRA_HSCA_INV_RATIO_INDEX 11
#define DISP_HD_COMP_GRA_POS_INDEX 12
#define DISP_HD_COMP_ALPHA_GRA_SHIFT_INDEX 13
#define DISP_HD_GRA_SCALED_SIZE_INDEX 14
#define DISP_HD_COMP_ALPHA_MODE_GRA_SHIFT_INDEX 15
#define DISP_HD_CC_GRA_MODE_SHIFT_INDEX 16
#define DISP_HD_CC_GRA_MODE_MASK_INDEX 17
#define DISP_HD_GRA_DEFAULT_TAG_INDEX 18
#define DISP_HD_GRA_IDX_MASK_INDEX 19
#define DISP_HD_GRA_IDX_SHIFT_INDEX 20
#define ALPHA_GRAPHIC_MASK_INDEX 21
#define DISP_HD_GRA_FREE_ENTRIES_MASK_INDEX 22
#define DISP_HD_GRA_FREE_ENTRIES_SHIFT_INDEX 23
#define DISP_HD_GRA_BUF_SIZE_MASK_INDEX	24
#define DISP_HD_GRA_BUF_SIZE_SHIFT_INDEX 25

#define DISP_HD_GRA_CMD_FIFO_INDEX 26

#define DISP_HD_COMP_ALPHA_MODE_GRA_MASK_INDEX 27

#define GRA_EMPTY_MASK_INDEX 28
#define GRA_EMPTY_SHIFT_INDEX 29


#define SIZE_OF_DISP_GRA_REGISTER_TABLE GRA_EMPTY_SHIFT_INDEX + 1




#define DISP_HD_VID_FORMAT_INDEX 0
#define DISP_HD_VID_BUF_PITCH_INDEX 1
#define DISP_HD_VID_BUF_SIZE_INDEX 2
#define DISP_HD_VID_SIZE_INDEX 3
#define DISP_HD_VID_SAMPLES_INDEX 4
#define DISP_HD_VID_SCALED_SIZE_INDEX 5
#define DISP_HD_COMP_VID_POS_INDEX 6
#define DISP_HD_VID_VSCA_INV_RATIO_INDEX 7
#define DISP_HD_VID_HSCA_INV_RATIO_INDEX 8
#define DISP_HD_VID_VSCA_INIT0_INDEX 9
#define DISP_HD_VID_VSCA_INIT1_INDEX 10
#define DISP_HD_VID_VSCA_START_INDEX 11
#define DISP_HD_VID_VSCALER_Y_BASE_ADDRESS_INDEX 12
#define DISP_HD_VID_VSCALER_CBCR_BASE_ADDRESS_INDEX 13
#define DISP_HD_VID_HSCALER_Y_BASE_ADDRESS_INDEX 14
#define DISP_HD_VID_HSCALER_CBCR_BASE_ADDRESS_INDEX 15
#define DISP_HD_COMP_ALPHA_VID_SHIFT_INDEX 16
#define DISP_HD_VID_HSBC0_INDEX 17
#define DISP_HD_VID_HSBC1_INDEX 18
#define DISP_HD_VID_HSBC2_INDEX 19
#define DISP_HD_VID_Y_ADR_INDEX 20
#define DISP_HD_VID_CB_ADR_INDEX 21
#define DISP_HD_VID_CR_ADR_INDEX 22
#define DISP_HD_VID_HSCA_INIT_INDEX 23
#define DISP_HD_VID_HSCA_START_INDEX 24
#define DISP_HD_CMD_FIFO_INDEX 25
#define DISP_HD_CURR_CMD_INDEX 26
#define DISP_HD_VID_FREE_ENTRIES_MASK_INDEX 27
#define DISP_HD_VID_FREE_ENTRIES_SHIFT_INDEX 28
#define DISP_HD_VID_FORMAT_MAX_REQ_LINE_INDEX 29
#define DISP_HD_VID_VERTICAL_SCALER_TAP_INDEX 30
#define DISP_HD_COMP_LUMA_KEY_INDEX 31
#define VID_EMPTY_MASK_INDEX 32
#define ALPHA_VID_MASK_INDEX 33
#define GIC_DUMMY_REG_INDEX 34
#define HD_CC_VID_MODE_SHIFT_INDEX 35
#define HD_CC_VID_MODE_MASK_INDEX 36
#define DISP_HD_VID_INPUT_Y_LIMIT_INDEX 37
#define DISP_HD_VID_INPUT_Y_PLUS_MASK_INDEX 38
#define DISP_HD_VID_INPUT_Y_MINUS_MASK_INDEX 39
#define DISP_HD_VID_INPUT_Y_PLUS_SHIFT_INDEX 40
#define DISP_HD_VID_INPUT_Y_MINUS_SHIFT_INDEX 41

#define DISP_HD_VID_INPUT_CBCR_LIMIT_INDEX    42
#define DISP_HD_VID_INPUT_CBCR_PLUS_MASK_INDEX 43
#define DISP_HD_VID_INPUT_CBCR_MINUS_MASK_INDEX 44
#define DISP_HD_VID_INPUT_CBCR_PLUS_SHIFT_INDEX 45
#define DISP_HD_VID_INPUT_CBCR_MINUS_SHIFT_INDEX 46

#define SIZE_OF_DISP_VID_REGISTER_TABLE DISP_HD_VID_INPUT_CBCR_MINUS_SHIFT_INDEX + 1


#define DISP_COMP_SIZE_INDEX 0
#define DISP_COMP_GLOBAL_SIZE_INDEX 1
#define DISP_COMP_START_PIX_POS_INDEX 2
#define DISP_COMP_START_FLD_POS_INDEX 3
#define DISP_COMP_START_VBI_POS_INDEX 4
#define DISP_COMP_STOP_VBI_POS_INDEX 5
#define DISP_COMP_START_HANC_POS_INDEX 6
#define DISP_COMP_STOP_HANC_POS_INDEX 7
#define DISP_VBI0_HANC0_FORMAT_INDEX 8
#define DISP_VENC0_DVE_FIELD_BNDRY_INDEX 9
#define DISP_VENC0_DVE_VBLANK_START_BNDRY_INDEX 10
#define DISP_VENC0_DVE_VBLANK_END_BNDRY_INDEX 11
#define DISP_VENC0_DVE_CNTRL_INDEX 12
#define DISP_GENLOCK_PCR_INDEX 13
#define DISP_GENLOCK_NUM_INDEX 14
#define DISP_GENLOCK_DEN_INDEX 15
#define DISP_VENC0_DVE_VSYNC_START_INDEX 16
#define DISP_VENC0_DVE_VSYNC_END_INDEX 17
#define DISP_VENC0_DVE_HSYNC_INDEX 18
#define DISP_OUTPUT_LIMIT_INDEX    19

#define SIZE_OF_DISP_COMP_REGISTER_TABLE DISP_OUTPUT_LIMIT_INDEX + 1

#define DISP_HD_CMD_FIFO_UPDATE 1

#define PROGRESSIVE_FIELD_SEL 7
#define INTERLACE_FIELD_STACK_TOP_MODE 0
#define INTERWOVEN_TOP_FIELD 2
#define INTERWOVEN_BOT_FIELD 3
#define INTERLACED_INTERWOVEN_BOTH_FIELD 4

//#define INTERLEAVE_MASK 0x8
#define INTERLACED_INTERWOVEN_BOTH_FIELD_INTERLEAVE 0x84
#define INTERWOVEN_FIELD_SWITCH 10

#define STACKED_FORMAT 1
#define INTERWOVEN_FORMAT 0

#define NTSC_SUBCARRIER_FREQ 0x21F07C1F
#define NTSC_SUBCARRIER_PHASE 0

#define DISP_HD_VENC0_FULL_RESOLUTION           0
#define DISP_HD_VENC0_HALF_RESOLUTION           1
#define DISP_HD_VENC0_QUARTER_RESOLUTION        2

#define DISP_HD_VENC0_CMPST_GAIN_CMPST_Y_GAIN 194
#define DISP_HD_VENC0_CMPST_GAIN_CMPST_V_GAIN 122
#define DISP_HD_VENC0_CMPST_GAIN_CMPST_U_GAIN 88
#define DISP_HD_VENC0_CMPST_GAIN_CMPST_CB_GAIN 49

#define DISP_HD_VENC0_YCMPN_YCMPNGAIN 192
#define YCMPN_YCMPNOFF_FOR_COMPOSITE 0xd4
#define YCMPN_YCMPNOFF_FOR_COMPONENT 0x3c0

#define DISP_HD_VENC0_CB2GAIN_CB2PBGAIN 96
#define DISP_HD_VENC0_CB2GAIN_CB2PRGAIN 96

#define DISP_HD_Y_PLUS_LIMIT        255
#define DISP_HD_Y_MINUS_LIMIT       0

#define DISP_HD_CBCR_PLUS_LIMIT     255
#define DISP_HD_CBCR_MINUS_LIMIT    0


#define	DISP_HD_VENC_POLARITY_NEG   1
#define	DISP_HD_VENC_POLARITY_POS   0

/* These are for PHY value overrides ... */
#define PHY_VALUE_GAIN_HI               0x1
#define PHY_VALUE_PREEMPHASIS_HI        0x2

#define PHY_VALUE_GAIN_LO               0x3
#define PHY_VALUE_PREEMPHASIS_LO        0x4


#define PHY_MASK_ALL                0xF0000000
#define PHY_MASK_SHIFT              28
struct display_mode_struct
{
    DFBScreenOutputSlowBlankingSignals     slow_blanking;
    DFBScreenEncoderScanMode     scanmode;
    DFBScreenEncoderFrequency     frequency;
    DFBScreenOutputResolution     resolution;

    // for digital signal output (HDMI/656),
    u32 video_id_code;	 //format id of CEA-861-D
    u32 xres;
    u32 h_active;
    u32 v_active;
    u32 h_total;
    u32 v_total;
    u32 hsync_active_edge_delay;
    u32 hsync_active_pulse_width;
    u32 vsync_active_edge_delay;
    u32 vsync_active_pulse_width;
    u32 field1_vblank_start;
    u32 field2_vblank_start;
    u32 field1_vsync_start;
    u32 field2_vsync_start;

	u32 denominator;
    
    //for analog signal output 
    double subcarrier_freq; //Mhz unit
    double sample_freq;
    u32 subcarrier_phase;

    u32 field1_post_equalization;
    u32 field2_single_equalization;

    u32 black_level;
    u32 blank_level;
    u32 sync1_level;
    u32 sync2_level;
    
    //pol_<x>sync = 1 for negative polarity (falling), 0 for positive polarity (rising)
    u32 pol_hsync;
    u32	pol_vsync;
	
	/*
	* Incoming/Outgoing factors
	* These factors tell us how much repetition is present on the
	* DISP->HDMI (incoming) and HDMI Tx->TV Rx (outgoing)
	* incoming = h_active / xres
	* outgoing = outgoing_factor
	* Ex:
	* 
	* Currently for 480i
	*/
	u32 incoming_factor;
	u32 outgoing_factor;
    
    /* 
    *   Used to implant hardcoded f2off values, since we do not want to change the code that gave us 
    *   HDMI certification, we need to hardcode these values in order to give us what we want ... 
    */
    u32 f2off;
    
    /*
    *   This field contains the length of a single field of video in a video mode.
    *   It is only used for interlaced frame packing video, not used
    *   for other group of modes.
    */
    u32 field_length;


};


typedef struct 
{
    DFBScreenOutputResolution   listResolution;     /* Resolution to be equated to in our mode array ... */
    DFBScreenOutputResolution   configResolution;   /* Resolution which is in the config structure ... */
} screen_mode_alias_t;


typedef enum  
{
    ALIAS_NONE              =       0,              /* No aliasing found .. */
    ALIAS_3D_TOP_AND_BOT,                           /* Top and Bottom 3D .. */
    ALIAS_3D_SIDE_BY_SIDE                           
} screen_mode_alias_type_t;


struct dve_656_reg_struct
{
    u32 dve_field_bndry_reg;
    u32 dve_vblank_start_bndry_reg;
    u32 dve_vblank_end_bndry_reg;
	u32 dve_vsync_start_reg;
	u32 dve_vsync_end_reg;
	u32 dve_hsync_reg;
    u32 dve_cntrl_reg;
};

#define HDMI_MC_PHYRSTZ 0x04005
    #define HDMI_PHYSICAL_RESET 01

#define HDMI_FC_INVIDCONF 0x1000
    #define IN_I_P_MASK 0x01
    #define IN_I_P_SHIFT 0
        #define HDMI_INTERLACE_INPUT 1
        #define HDMI_PROGRESSIVE_INPUT 0
    #define R_V_BLANK_IN_OSC_Mask 0x02
    #define R_V_BLANK_IN_OSC_SHIFT 1
        #define MODE_WITH_FRACTIONAL_VBLANK 1
        #define MODE_WITHOUT_FRACTIONAL_VBLANK 0
    #define DVI_MODEZ_MASK 0x04
    #define DVI_MODEZ_SHIFT 3
        #define NONE_DVI_MODE 1
        #define DVI_MODE 0
    #define DE_IN_POLARITY_MASK 0x10
    #define DE_IN_POLARITY_SHIFT 4
        #define DATA_ENABLE_INPUT 1
        #define NO_DATA_ENABLE_INPUT 0
    #define HSYNC_IN_POLARITY_MASK 0x20
    #define HSYNC_IN_POLARITY_SHIFT 5
    #define VSYNC_IN_POLARITY_MASK 0x40
    #define VSYNC_IN_POLARITY_SHIFT 6
	#define HDCP_KEEPOUT_MASK 0x80

#define HDMI_FC_INHACTIV0 0x1001
    #define FC_INHACTIV0_MASK 0x0ff

#define HDMI_FC_INHACTIV1 0x1002
    #define FC_INHACTIV1_MASK 0x0f00
    #define FC_INHACTIV1_SHIFT 8 

#define HDMI_FC_INHBLANK0 0x1003
    #define FC_INIHBLANK0_MASK 0x0ff

#define HDMI_FC_INHBLANK1 0x1004
    #define FC_INIHBLANK1_MASK 0x0300
    #define FC_INIHBLANK1_SHIFT 8

#define HDMI_FC_INVACTIV0 0x1005
    #define FC_INVACTIV0_MASK 0x0ff

/* 
*	The mask below differs for A21 and A11,
*	the shift count is the same
*/
#define HDMI_FC_INVACTIV1 0x1006
    #define FC_INVACTIV1_MASK 0x0700
	#define FC_INVACTIV1_MASK_A21 0xF00
    #define FC_INVACTIV1_SHIFT 8
	
	

#define HDMI_FC_INVBLANK 0x1007

#define HDMI_FC_HSYNCINDELAY0 0x1008
    #define FC_HSYNCINDELAY0_MASK 0x0ff;

#define HDMI_FC_HSYNCINDELAY1 0x1009
    #define FC_HSYNCINDELAY1_MASK 0x0700
    #define FC_HSYNCINDELAY1_SHIFT 8

#define HDMI_FC_HSYNCINWIDTH0 0x100a
    #define FC_HSYNCINWIDTH0_MASK 0x0ff

#define HDMI_FC_HSYNCINWIDTH1 0x100b
    #define FC_HSYNCINWIDTH1_MASK 0x0300
    #define FC_HSYNCINWIDTH1_SHIFT 8

#define HDMI_FC_VSYNCINDELAY 0x100c

#define HDMI_FC_VSYNCINWIDTH 0x100d
    #define FC_VSYNCINWIDTH_MASK 0x0f

#define HDMI_FC_INFREQ0 0x100e
    #define FC_INFREQ0_MASK 0x0ff

#define HDMI_FC_INFREQ1 0x100f
    #define FC_INFREQ1_MASK 0x0ff00
    #define FC_INFREQ1_SHIFT 8

#define HDMI_FC_INFREQ2 0x1010
    #define FC_INFREQ2_MASK 0x0f0000
    #define FC_INFREQ2_SHIFT 16

#define HDMI_FC_CTRLDUR 0x1011
    #define  HDMI_CONTROL_PERIOD_MINIMUM_DURATION 12

#define HDMI_FC_EXCTRLDUR 0x1012
    #define HDMI_EXTENDED_CONTROL_PERIOD_MINIMUM_DURATION 32

#define HDMI_FC_EXCTRLSPAC 0x1013
    #define HDMI_EXT_CTRL_PERIOD_SPACE 1

#define HDMI_FC_CH0PREAM 0x1014
    #define HDMI_CH0_PREAMBLE_FILTER 0x0b

#define HDMI_FC_CH1PREAM 0x1015
    #define HDMI_CH1_PREAMBLE_FILTER 0x16

#define HDMI_FC_CH2PREAM 0x1016
    #define HDMI_CH2_PREAMBLE_FILTER 0x21

#define HDMI_FC_GCP 0x1018

#define HDMI_FC_AVICONF0 0x1019
    #define AVICONF_YCBCR_444 0x02
	#define AVICONF_YCBCR_422 0x01
	#define AVICONF_RGB		  0x00	

#define HDMI_FC_AVICONF1 0x101a
    #define COLORIMETRY_SHIFT 6
        #define SMPTE_170M              0x01
        #define COLORIMETRY_ITU601      0x01
        #define COLORIMETRY_ITU709      0x02
        #define COLORIMETRY_XVYCC       0x03
        
    #define PICTURE_ASPECT_RATIO_SHIFT 4
        #define PICTURE_ASPECT_RATIO_4X3 0x01
        #define PICTURE_ASPECT_RATIO_16X9 0x02
    #define ACTIVE_ASPECT_RATIO_SHIFT 0
        #define SAME_AS_PICTURE_ASPECT_RATIO 0x8
        #define ACTIVE_ASPECT_RATIO_4X3 0x09
        #define ACTIVE_ASPECT_RATIO_16X9 0x0a

#define HDMI_FC_AVICONF2 0x101b
    #define EXTENDED_COLORIMETRY_SHIFT  4
        #define COLORIMETRY_XVYCC601    0
        #define COLORIMETRY_XVYCC709    1
    #define QUANTIZATION_RANGE_SHIFT 2
        #define QUANTIZATION_RANGE_DEFAULT 0
        #define QUANTIZATION_RANGE_LIMITED 1
        #define QUANTIZATION_RANGE_FULL 2

#define HDMI_FC_AVIVID 0x101c

#define HDMI_FC_AVIETB0 0x101d

#define HDMI_FC_AVIETB1 0x101e

#define HDMI_FC_AVISBB0 0x101f

#define HDMI_FC_AVISBB1 0x1020

#define HDMI_FC_AVIELB0 0x1021

#define HDMI_FC_AVIELB1 0x1022

#define HDMI_FC_AVISRB0 0x1023

#define HDMI_FC_AVISRB1 0x1024




#define HDMI_FC_VSDIEEEID0 0x1029
    #define VIXS_ID 0xaa

#define HDMI_FC_VSDIEEEID1 0x1030

#define HDMI_FC_VSDIEEEID2 0x1031


#define HDMI_TX_INVID0 0x200
    #define YCBCR_444_8 	9
	#define YCBCR_444_10   11
	#define YCBCR_444_12   13	

#define HDMI_MC_FLOWCTRL 0x4004
    #define COLOR_CONVERT_BYPASS 0
	#define COLOR_CONVERT_ENABLE 1

#define HDMI_VP_PR_CD 0x801
    //#define DEPTH_24_BIT_8_PER_COMPONENT 0x4 
	//The above should be 0 according to JimK
	#define DEPTH_24_BIT_8_PER_COMPONENT 0x0
	#define DEPTH_30_BIT_10_PER_COMPONENT 0x5
    #define DEPTH_36_BIT_12_PER_COMPONENT 0x6
    #define COLOR_DEPTH_SHIFT 4
    #define NO_PIXEL_REPETITION 0
    #define PIXEL_REPEAT_ONCE   1
	#define PIXEL_REPEAT_TWICE 	2
	#define	PIXEL_REPEAT_THREE  3

#define HDMI_FC_PRCONF 0x10e0
    #define INPUT_NO_REPEAT 0x01
    #define INPUT_REPEAT_ONCE 0x02
    #define INCOMING_PR_FACTOR_SHIFT 4
    #define OUTPUT_NO_REPEAT  0x00
    #define OUTPUT_REPEAT_ONCE  0x01
	#define OUTPUT_REPEAT_TWICE 0x02
	#define OUTPUT_REPEAT_THREE 0x03

#define HDMI_VP_STUFF 0x0802
    #define IDEFAULT_PHASE_MASK 0x20
    #define DEFAULT_VP_STUFF 0
    #define VP_STUFF_DEEP_COLOR 0x20
    /* bit5 needs to be set to 0 if htotal for mode is 
    *  not divisible by 4
    */
    #define VP_STUFF_DEEP_COLOR_NON_DIV_BY_4 0x00

#define HDMI_VP_REMAP 0x0803
    #define YCC422_16BIT 0	//8 bits per component
	#define YCC422_20BIT 1	//10 bits per component
	#define YCC422_24BIT 2	//12 bits per component	
#define HDMI_VP_CONF 0x0804
		
		/* 
		*	Data select is in the lowest 2-bits of this register and there is no  
		*	need for a mask or a shift count	
		*/
        #define DATA_FROM_8_BIT_BYPASS_BLOCK  0x02	
		#define	DATA_FROM_YCC422_REMAP_BLOCK  0x01		
        #define DATA_FROM_PIXEL_PACKING_BLOCK 0x00
    #define BYPASS_SELECT_SHIFT 2
        #define DATA_FROM_PIXEL_REPEATER 0
        #define DATA_FROM_VIDEO_PACKETIZER 1
    #define YUV_422_ENABLE_SHIFT 3
	    #define YUV_422_ENABLE 1
	#define PIXEL_PACKET_ENABLE_SHIFT 5
	    #define PIXEL_PACKET_ENABLE 1
	#define BYPASS_ENABLE_SHIFT 6
	    #define BYPASS_ENABLE 1
    

#define HDMI_FC_SPDDEVICEINF 0x1062
    #define BLU_RAY_DISC 0x0a

#define HDMI_FC_DATAUTO0 0x10b3
    #define DEFAULT_RDRB_MODE 0x010
    #define VSDB_RDRB_MODE    (DEFAULT_RDRB_MODE | 0x08)

#define HDMI_FC_DATAUTO1 0x10b4

#define HDMI_FC_DATAUTO2 0x10b5

#define HDMI_FC_DATAUTO3 0x10b7

#define HDMI_FC_RDRB0 0x10b8

#define HDMI_FC_RDRB1 0x10b9

#define HDMI_FC_RDRB2 0x10ba

#define HDMI_FC_RDRB3 0x10bb

#define HDMI_FC_RDRB4 0x10bc

#define HDMI_FC_RDRB5 0x10bd

#define HDMI_FC_RDRB6 0x10be

#define HDMI_FC_RDRB7 0x10bf

#define HDMI_FC_GMD_STAT    0x1100

#define HDMI_FC_GMD_EN 0x1101
    #define GMD_ENABLE_PACKET_MASK  0x1
    
#define HDMI_FC_GMD_UP 0x1102
    #define GMD_UPDATE_PACKET_MASK  0x1
    
#define HDMI_FC_GMD_CONF 0x1103
    #define HDMI_FC_GMD_CONF_DEFAULT    0x10

#define HDMI_FC_GMD_HB   0x1104
    #define GMD_HB_PROFILE_MASK     0x7
    #define GMD_HB_PROFILE_SHIFT    0x4
    
    #define GMD_HB_PROFILE_P0       0x0
    #define GMD_HB_PROFILE_P1       0x1
    #define GMD_HB_PROFILE_P2       0x2
    #define GMD_HB_PROFILE_P3       0x3
    
    #define GMD_HB_AFFECTED_MASK    0xF
    #define GMD_HB_AFFECTED_SHIFT   0x0

/* 
*   Define the base address of the packet body
*   All bytes are in contiguous sequence...
*/
#define HDMI_FC_GMD_PB0  0x1105


#define HDMI_CSC_CFG 0x4100

#define HDMI_CSC_SCALE 0x4101
    #define COLOR_SPACE_CONV_COLOR_DEPTH_SHIFT 4
	  #define COLOR_DEPTH_24_BIT_PER_PIXEL 4
	  #define COLOR_DEPTH_30_BIT_PER_PIXEL 5
	  #define COLOR_DEPTH_36_BIT_PER_PIXEL 6

#define HDMI_CSC_COEFA1_MSB 0x4102

#define HDMI_CSC_COEFB2_MSB 0x410c

#define HDMI_CSC_COEFC3_MSB 0x4116


#define HDMI_CSC_COEFC4_LSB 0x4119


#define HDMI_TX_INSTUFFING 0x201

#define HDMI_TX_BCBDATA1 0x207

#define HDMI_VSD_PACKET_ID 0x81
		#define HDMI_VSD_VERSION_ID 0x01


#define HDMI_FC_VSDPAYLOAD0 0x1032
        #define STRUCTURE_3D 0x40
#define HDMI_FC_VSDPAYLOAD1 0x1033

    #define FRAME_PACKING_3D 0x0
    #define SIDE_BY_SIDE_3D 0x8
	#define TOP_AND_BOTTOM_3D 0x6
    #define MODE_MASK_3D 0x0f0
    #define MODE_SHIFT_3D 4

#define HDMI_FC_VSDPAYLOAD23 0x1049

    #define LENGTH_OF_VENDOR_SPECIFIC 24

#define HDMI_FC_SPDVENDORNAME0 0x104a

#define HDMI_FC_SPDVENDORNAME7 0x1051

    #define LENGTH_OF_SPD_VENDOR_NAME 8

#define HDMI_FC_SPDPRODUCTNAME0 0x1052

#define HDMI_FC_SPDPRODUCTNAME15 0x1061

    #define LENGTH_OF_SPD_PRODUCT_NAME 16

#define HDMI_FC_CTRLQHIGH 0x1073

#define HDMI_FC_CTRLQLOW 0x1074

#define HDMI_FC_ACP0 0x1075




#define HDMI_FC_ISCR1_0 0x1092
    #define ISRC_CONT 0x01
    #define ISRC_VALID 0x02

#define HDMI_FC_ISCR1_1 0x1093

#define HDMI_FC_ISCR1_16 0x10a2
  
    #define LENGTH_OF_ISCR1 16

#define HDMI_FC_ISCR2_0 0x10a3

#define HDMI_FC_ISCR2_15 0x10b2
  
    #define LENGTH_OF_ISCR2 16

#define FC_GMD_PB0 0x1105

#define FC_GMD_PB27 0x1120
    #define LENGTH_OF_GMD 28

#define HDMI_FC_DATMAN 0x10b6 
    #define SPD_TX 0x10
    #define VSD_TX 0x80
    #define ISCR2_TX 0x40
    #define ISCR1_TX 0x20


#define HDMI_MC_SFRDIV 0x4000

#define HDMI_PHY_CONF0 0x3000

#define HDMI_PHY_TST0 0x3001

#define HDMI_PHY_TST1 0x3002


#define HDMI_PHY_STAT0 0x3004
    #define HDMI_TX_PHY_LOCK_MASK  0x1
    #define HDMI_HPD_STATUS_LOCK   0x2
    

#define HDMI_PHY_MASK0 0x3006
	#define HDMI_HPD_MASK 0x02

#define HDMI_PHY_POL0  0x3007
	#define	HPD_POL_MASK  0x02
    
#define HDMI_MC_CLKGAT 0x4001
    #define CLKGATE_HDCP_MASK    0x40
    #define CLKGATE_CEC_MASK     0x20
    #define CLKGATE_CSC_MASK     0x10
    #define CLKGATE_AUD_MASK     0x08
    #define CLKGATE_PREP_MASK    0x04
    #define CLKGATE_TMDS_MASK    0x02
    #define CLKGATE_PCLK_MASK    0x01

/* Base address in HDMI for CSC matrix */	
#define HDMI_CSC_COEF_BASE 0x4102	
	

#define HDMI_A_HDCPCFG0 0x5000
    #define HDMI_DVI_MASK 0x01
    #define RX_DETECT_MASK 0x04
	#define BYPASSES_DATA_ENCRYPTION_MASK 0x20
	#define ENHANCED_LINK_VERIFICATION_MASK 0x80
	
#define	HDMI_A_HDCPPOLCFG 	        0x5009
	#define	VSYNC_POS_MASK	        0x8
	#define	HSYNC_POS_MASK	        0x2
	#define DE_POS_MASK		        0x10
    #define UNENCRYPT_COLOR_MASK    0x60
    #define UNENCRYPT_COLOR_SHIFT   5
    #define UNENCRYPT_COLOR_BLUE    0x2
    #define UNENCRYPT_COLOR_RED     0x0

#define HDMI_A_TIMER2SETUP0 0x500d
#define HDMI_A_TIMER2SETUP1 0x500e


#define HDMI_A_PRESETUP 0x501a
    #define PRE_CLK_ENABLE_MASK 0x80
    #define PRE_CLK_DIV_FULL_HD 4
    #define PRE_CLK_DIV_HD 2
    #define PRE_CLK_DIV_SD 1


#define FIELD_RATE_50_ENABLE 1

#define SD_COMPOSITE 1

#define COMPOSITE_Y_GAIN 0xb2

#define COMPOSITE_V_GAIN 0x68

#define COMPOSITE_U_GAIN 0x48

#define COMPOSITE_CB_GAIN 0x39

#define COMPOSITE_DAC_BLACK_LEVEL 51

#define COMPOSITE_DAC_BLANK_LEVEL 293

#define COMPOSITE_DAC_SYNC_STEP 146

#define COMPOSITE_DAC_SYNC1_LEVEL 0

#define COMPOSITE_DAC_CNST1_LEVEL 0

#define COMPOSITE_DAC_SYNC2_LEVEL 528

#define COMPOSITE_DAC_CNST2_LEVEL 0



#define SD_COMPONENT_I 1

#define SD_COMPONENT_P 1

#define HD_720P 1

#define HD_1080I 1

#define HD_1080P 1

#define COMPONENT_DAC_BLACK_LEVEL 61

#define COMPONENT_DAC_BLANK_LEVEL 293

#define COMPONENT_DAC_SYNC_STEP 146

#define COMPONENT_DAC_SYNC1_LEVEL 16

#define COMPONENT_DAC_CNST1_LEVEL 0

#define COMPONENT_DAC_SYNC2_LEVEL 528

#define COMPONENT_DAC_CNST2_LEVEL 0

//define the horizontal state for venc 
#define H_STATE_ACTIVE_VIDEO 0

#define H_STATE_VBI 1

#define H_STATE_BLANK 2

#define H_STATE_SYNC1 3

#define H_STATE_SYNC2 4

#define H_STATE_CNST1 5

#define H_STATE_CNST2 6

#define H_STATE_MACRO_PS 7

#define V_STATE_EQ_EQ 0 //two narrow negative equalization pulse
#define V_STATE_BR_BR 1 //two broad negative pulse
#define V_STATE_NP 2 //one normal negative sync pulse
#define V_STATE_ACTIVE_VIDEO 3
#define V_STATE_NP_EQ 4 //one normal negative sync pulse followed by one narrow negative pulse
#define V_STATE_EQ_BR 5 //one narrow negative pulse followed by one broad negative pulse
#define V_STATE_BR_EQ 6 //one broad negative pulse followed by one narrow negative pulse
#define V_STATE_EQ 7 //only one narrow negative 

#define V_STATE_NP_HP 8 //for 576i, line 23, one normal pulse folllowed by half pixture of samples
#define V_STATE_HP_EQ 9 //for 576i, line 623, half pixture of samples followed by one narrow negative pulse
#define V_STATE_BR 10 //one broad negative pulse 

#define V_STATE_HD_SYNC_PROG 11//for hd, tri-level, the broad pulse is asserted, for vertical sync
#define V_STATE_HD_VBI 12 //tri_level
#define V_STATE_HD_ACTIVE 13 //for the line tri_level and  active video data

#define V_STATE_HD_SYNC_INT 14
#define V_STATE_HD_FIELD1_SYNC_TO_BLANK_INT 15 //for hd, two tri-level pulse, for interlace

#define V_STATE_HD_FIELD2_BLANK_TO_SYNC_INT 0 








#define VERTICAL_TIMING_LINE_NUMBER_SHIFT 5

#define VENC_VERTICAL_STATE_START_ADDR 0x100


struct analog_encoder_vertical_timing
{
    u32 line_number;
    u32 state;
};

#define MAX_VERTICAL_TIME_EVENTS 16


#define HORIZONTAL_TIMING_SAMPLE_NUMBER_SHIFT 15
#define HOTIZONTAL_TIMING_STATE_SHIFT 28

struct analog_encoder_horizontal_timing
{
    u32 current_state;
    u32 transition_sample;
    u32 control_bits;
};


#define ENABLE_MACROVISION_CS 6

#define ENABLE_COLOR_BURST 1

#define INVERT_PHASE_FOR_MACRIPHASE_CS 2

#define ADD_BLACK_TO_COMPOSITE 8

#define MAX_EVENTS_PER_LINE 8

#define MAX_VENC_HORIZONTAL_ADDR 128

#define VENC_HORIZONTAL_STATE_START_ADDR 0

#define DEFAULT_Y_COMPOSITE0_COEF 0x40a03fb

#define DEFAULT_Y_COMPOSITE1_COEF 0x4e13eef2

#define DEFAULT_Y_COMPOSITE2_COEF 0x6b

#define DEFAULT_UV_COMPOSITE0_COEF 0x2fef8f6

#define DEFAULT_UV_COMPOSITE1_COEF 0x302c2618

#define DEFAULT_UV_COMPOSITE2_COEF 0x30

#define DEFAULT_MACROV_CS_STRTF 0x12d001e

#define DEFAULT_MACROV_CS_PARAM 0x20011

#define DEFAULT_MACROV_T1_EVEN 0x8f0118

#define DEFAULT_MACROV_T2_EVEN 0xf0003d

#define DEFAULT_MACROV_T1_ODD 0x8f0118

#define DEFAULT_MACROV_T2_ODD 0xf0003d

#define DEFAULT_MACROV_LVL 0x4040380

#define FORMAL_DFB_DRIVER

//#define USE_HARD_CODE_COEF

//define the color conversion

//graphic pipe
#define GRA_PASS_THRU 0
#define GRA_RGB255_TO_601 1
#define GRA_RGB255_TO_709 2
#define GRA_RGB235_TO_601 5
#define GRA_RGB235_TO_709 6
#define GRA_YUV_PASS_THRU 7
#define GRA_YUV709_TO_601 8
#define GRA_YUV601_TO_709 9

#define BOB_DEINTERLACE 1
#define HYBRID_DEINTERLACE 3


#define COEF_AUTO_GENERATE 1

#define USE_PER_PIXEL_ALPHA 0
#define USE_VALUE_IN_REGISTER 1

#define DISP_HD_INV_RATIO_SHIFT 28

#define NFACT    12
#define VALFACT  (22<<NFACT)

//#define SINGLE_BUS 1

#define DUAL_BUS 1

#define DISP_HD_GRA0_DEFAULT_TAG 0x12
#define DISP_HD_GRA1_DEFAULT_TAG 0x13

#define MAX_FRAME_INDEX  (HD_VID1_TOP_IDX_MASK >> HD_VID1_TOP_IDX_SHIFT)


#define DISP_DVE_OUTPUT_CONFIG 0x1e4



/* This was changed from 256 to 32 to test */
#define Y_BYTE_ALIGNMENT 32
//#define C_BYTE_ALIGNMENT 256

#define VIDEO_HEIGHT_ALIGNMENT 64

#define DISP_VID_INPUT_422 1

#define DEFAULT_VID0_MAX_REQ_LINE 6
#define DOWNSCALE_MAX_REQ_LINE 1
#define DEFAULT_VID1_MAX_REQ_LINE 4

#define MAX_LAYER_LEVEL 3

#define COMP_MODE_CMD_REPEAT 0
#define COMP_MODE_DISP_BACKGROUND 1
#define COMP_MODE_TURN_OFF 2
#define COMP_MODE_VIDEO_TEST 3

#define LAYER_DEFAULT_BRIGHT 512
#define LAYER_DEFAULT_CONTRAST 0x80
#define LAYER_DEFAULT_SATURATION 0x80
#define LAYER_DEFAULT_HUE 0

#define LAYER_DEFAULT_SHARPNESS_VALUE   0x0
#define LAYER_DEFAULT_SOFTEN_VALUE      0xFFFF
//1.0 in 4.12 fixed point is 4096
#define LAYER_DEFAULT_GAIN_VALUE        0x1000       

#define HUE_LEFT_SHIFT 15
#define BRIGHTNESS_SIGN 0x200

#define PI 3.1415926535

#define DISP_HD_GRA0_CLUT_ADDR 0x1100000
#define DISP_HD_GRA1_CLUT_ADDR 0x1110000

#define GIC_8_BIT_YUV_SURFACE 0

#define GIC_LUT8 7
#define GIC_RGB565 9
#define GIC_ARGB8888 13
#define GIC_AYUV444  14
#define GIC_ARGB1555 10

#define GIC_ALPHA_MODE_NONE	0x00
#define GIC_ALPHA_MODE_CONSTANT 0x01
#define GIC_ALPHA_MODE_PERPIXEL 0x02
#define GIC_ALPHA_MODE_NOPREMULTIPLY   GIC1_SRC1_ALPHA_PREMUL_MASK
#define GIC_ALPHA_MODE_PREMULTIPLY     0x00


#define GIC_DEFAULT_PACK_ORDER  0xE4000000
#define GIC_ALL_APHA_PACK_ORDER 0xFF000000
#define GIC_ALPHA_FROM_R_PACK_ORDER 0xBF000000
#define GIC_ALPHA_R_SWAP_PACK_ORDER 0xB4000000
 
#define GIC_LUT8_PACK_ORDER	   0x40000000
#define GIC_PLANAR_MODE 0xe4000000
#define GIC_420 0
#define GIC_422 1


/* Defines CSCValueFull/256 for Alpha to CSC conversion */

#define GIC_CSC_ALPHASCALE 0x20

#define GIC_TRIGGER_SRC1_OVER_SRC2 GIC0_OP_KICKOFF_MASK
#define SRC1_OVER_SRC2_OVER_BACKGROUND 4
#define SRC1_OVER_BACKGROUND 2

#define GIC_TRIGGER_SRC1_OVER_SRC2_OVER_BACKGROUND (SRC1_OVER_SRC2_OVER_BACKGROUND << GIC0_BLEND_LEVELS_SHIFT) | GIC0_OP_KICKOFF_MASK | GIC0_INT_EN_MASK

#define ALPHA_24_TO_31 3
#define RED_16_TO_23 2
#define GREEN_8_TO_15 1
#define BLUE_0_TO_7 0

#define GIC_CONSTANT_ALPHA 1
#define GIC_PER_PIXEL_ALPHA 2

#define GIC1_BLEND_DOMAIN_RGB 0

#define GIC0_BLEND_DOMAIN_YCBCR 1


#define GIC0_CSC_SEL_SRC1  1
#define GIC1_CSC_SEL_SRC1  1
#define GIC0_CSC_SEL_SRC2  2

#define SRC2_CLUT_PROG_EN 3

#define GIC1_BLEND_DOMAIN_YCBCR 1


#define SET_NUMBER_ALIGNMENT(output_value,input_value,byte_alignment) \
    output_value = input_value +  byte_alignment - 1;\
    output_value = output_value - (output_value % byte_alignment);

#define GIC_ALIGN_DIV 32

#define TRIGGER_FES0_FEATURE 

#define FES_ALIGN_DIV 32
#define FES0_DIRECT_ACCESS 0

#define FES_YUV_420 0
#define FES_YUV_422 1

#define FES0_ARGB_ORDER 0xe4000000
#define FES1_ARGB_ORDER 0xe4000000


#define FES_YUV_8_BIT_INTERLACED 0
#define FES_YUV_8_BIT_PROGRESSIVE 1
#define FES_RGB555 8
#define FES_RGB565 9
#define FES_ARGB1555 10
#define FES_RGB4444 11
#define FES_ARGB8888 13


#define FES_RATIO_SHIFT 18

#define KICK_OFF_FES0 0x81000000
#define KICK_OFF_FES1 0x80000000

#define VIDEO_BUFFER_SIZE_ALIGNMENT 256

#define Y_BUF_SIZE_ALIGNMENT 256
#define C_BUF_SIZE_ALIGNMENT 64

#define WEAVE_DEINTERLACED 1
#define DIAG_DEINTERLACED 2
#define HYBRID_DEINTERLACED 3

#define FES_FIFO_CTRL_MODE 5


#define GET_HD_GRA_FREE_FIFO_ENTRIES(reg_tab) \
	(((REG_READ(DISP_HD_FIFO_STATUS1)) & reg_tab[DISP_HD_GRA_FREE_ENTRIES_MASK_INDEX]) >> \
                   		reg_tab[DISP_HD_GRA_FREE_ENTRIES_SHIFT_INDEX]) 

#define GET_SD_GRA_FREE_FIFO_ENTRIES(reg_tab) \
	(((REG_READ(DISP_SD_FIFO_STATUS)) & reg_tab[DISP_HD_GRA_FREE_ENTRIES_MASK_INDEX]) >> \
                   		reg_tab[DISP_HD_GRA_FREE_ENTRIES_SHIFT_INDEX]) 

#define GET_HD_FRM_COUNT		((REG_READ(DISP_HD_FIFO_STATUS1) & DISP_HD_FIFO_STATUS1_FRM_CNT_MASK) >> DISP_HD_FIFO_STATUS1_FRM_CNT_SHIFT)

#define GET_SD_FRM_COUNT		((REG_READ(DISP_SD_FIFO_STATUS) & DISP_SD_FIFO_STATUS_FRM_CNT_MASK) >> DISP_SD_FIFO_STATUS_FRM_CNT_SHIFT)

#define WAIT_FOP_NEXT_HD_FRAME \
base_frm_cnt = GET_HD_FRM_COUNT; \
while ((base_frm_cnt == GET_HD_FRM_COUNT) && \
       (REG_READ(DISP_HD_COMP_CTRL) & DISP_HD_COMP_CTRL_COMP_EN_MASK))\
{\
    usleep(10);\
}


#define GET_GRA_BUF_SIZE(reg_tab) ((REG_READ(reg_tab[DISP_HD_GRA_BUF_INDEX]) & reg_tab[DISP_HD_GRA_BUF_SIZE_MASK_INDEX]) >> \
					 reg_tab[DISP_HD_GRA_BUF_SIZE_SHIFT_INDEX] )



#define Wait_For_Fes0_Fifo(requested_fifo_space) \
    Wait_For_Fifo_Entries(mdrv, \
                          FES_STATUS, \
                          FES0_CF_FREE_ENT_MASK, \
                          FES0_CF_FREE_ENT_SHIFT, \
                          requested_fifo_space) 

#define Wait_For_Fes1_Fifo(requested_fifo_space) \
    Wait_For_Fifo_Entries(mdrv, \
                          FES_STATUS, \
                          FES1_CF_FREE_ENT_MASK, \
                          FES1_CF_FREE_ENT_SHIFT, \
                          requested_fifo_space) 
#define Wait_For_Fes0_Idle \
    while(REG_READ(FES_STATUS) & FES0_CF_BUSY_MASK) \
	{ \
	    usleep(1); \
	}

#define Wait_For_Fes1_Idle \
    while(REG_READ(FES_STATUS) & FES1_CF_BUSY_MASK) \
	{ \
	    usleep(1); \
	}

#define Wait_For_Gic_Idle \
    while (REG_READ(GIC_STATUS) & GIC_STATUS_BUSY_MASK) \
	{ \
	    usleep(1); \
	}

#define Wait_For_Gic1_Idle \
	while(!(REG_READ(GIC_STATUS) & GIC_CF1_EMPTY_MASK)) \
	{\
		usleep(1);\
	}\
	while (REG_READ(GIC_STATUS) & GIC1_CF_BUSY_MASK) \
	{ \
	    usleep(1); \
	}\

#define Wait_For_Gic0_Idle \
	while(!(REG_READ(GIC_STATUS) & GIC_CF0_EMPTY_MASK)) \
	{\
		usleep(1);\
	}\
	while (REG_READ(GIC_STATUS) & GIC0_CF_BUSY_MASK) \
	{ \
	    usleep(1); \
	}\
    

#define Wait_For_G2d_Idle \
	while(!(REG_READ(G2D_GUI_STAT) & STAT_CMD_BUFFER_EMPTY_MASK)) \
	{\
		usleep(1);\
	}\
	while (REG_READ(G2D_GUI_STAT) & STAT_GFX_BUSY_MASK)\
	{ \
	    usleep(1); \
	}\


#define fes_format_fract(tmp,fract) \
    tmp = (int) (fract*65536+0.5); \
    if(tmp > 65535) tmp = 65535;

#define FES_CONSTANT_ALPHA 1
#define FES_PER_PIXEL_ALPHA 2

#define Dfb_Format_420(format) \
((format == DSPF_I420) || (format == DSPF_YV12))

#define Dfb_Format_422(format) \
((format == DSPF_YV16) || (format == DSPF_YUY2))  //DSPF_YV16 replaced DSPF_UYVY; TODO: not consistent with DirectFB pixel format.

#define Dfb_Format_Graphic(format) \
((format == DSPF_RGB555) || \
 (format == DSPF_ARGB1555) || \
 (format == FES_RGB4444) || \
 (format == DSPF_ARGB) || \
 (format == DSPF_RGB32))

#define format_fes_ratio(tmp,in,out,mult) \
  tmp = (int)((1. / mult * in * pow(2., 18.) / out) + 0.5);\
  if (tmp > (pow(2.,22)-1)) \
  {						   \
    tmp = pow(2,22)-1;\
  }

#define KAISER_256_TABLE_SIZE 256
#define KAISER_384_TABLE_SIZE 384

#define VIDEO_BUFFER_SIZE_ALIGNMENT_SHIFT 8

#define DISABLE_RANDOM 2

#define DISP_HD_DVE656_2_OUTPUT 0
#define DISP_HD_HDMI_OUTPUT 1
#define DISP_HD_COMPONENT_OUTPUT 2

#define DISP_HD_OUTPUTS 3
#define DISP_HD_LIMIT_OUTPUTS 1

#define CG1_PLL4_DIV_5 00 << VSCLK_SRC_SEL_SHIFT
#define VO_SELECT 1 << PADU_CTRL_HDMI_VO_SHIFT
#define INT_HDMI_SELECT 0 << PADU_CTRL_HDMI_VO_SHIFT

#define FRAC_MASK 0x80000000

#define FULL_HD_DENOMINATOR_NO_FRAC  0x00000672 
#define FULL_HD_DENOMINATOR_WITH_FRAC  0x80000672 

#define HD_DENOMINATOR_NO_FRAC  0x339
#define HD_DENOMINATOR_WITH_FRAC  0x80000339

#define SD_DENOMINATOR_NO_FRAC  0x012c
#define SD_DENOMINATOR_WITH_FRAC  0x8000012c

#define	VGA_DENOMINATOR_25P175MHZ	0x042C
#define	VGA_DENOMINATOR_25P2MHZ		0x8000042C

/* Defines for 148.5MHz, 148.35, 74.25 and 74.176MHz modes */
#define HD_CG1_PLL0_FREQ_NO_FRAC (0x16 << CG1_PLL0_FREQ_FB_DIV_INT_SHIFT)
#define HD_CG1_PLL0_FREQ_WITH_FRAC ((0x15 << CG1_PLL0_FREQ_FB_DIV_INT_SHIFT) | 0xFA5FA)

/* Defines for 27MHz and 27.027MHz modes */
#define SD_CG1_PLL1_FREQ_NO_FRAC (0x10 << CG1_PLL1_FREQ_FB_DIV_INT_SHIFT)
#define SD_CG1_PLL1_FREQ_WITH_FRAC ((0x10 << CG1_PLL1_FREQ_FB_DIV_INT_SHIFT) | 0x4189)

/* Defines for 25.2MHz and 25.175MHz modes */
#define VGA_CG1_PLL1_FREQ_25_2MHZ (0xE << CG1_PLL1_FREQ_FB_DIV_INT_SHIFT | 0xEEEEF)
#define VGA_CG1_PLL1_FREQ_25_175MHZ (0xE << CG1_PLL1_FREQ_FB_DIV_INT_SHIFT | 0xEB1D4)

/* Full HD clocks ~148.5MHz */
#define FULL_HD_NO_FRAC_GENLOCK_DEN             1650000
#define FULL_HD_NO_FRAC_GENLOCK_NUM         1000

#define FULL_HD_PCR_NO_FRAC_DEN                 5500
#define FULL_HD_PCR_NO_FRAC_NUM                 1000

#define FULL_HD_WITH_FRAC_GENLOCK_DEN   1650000
#define FULL_HD_WITH_FRAC_GENLOCK_NUM   1001

#define FULL_HD_PCR_WITH_FRAC_DEN               5500
#define FULL_HD_PCR_WITH_FRAC_NUM               1001

/* Normal HD clocks ~74.25MHz */
#define HD_NO_FRAC_GENLOCK_DEN                  825000
#define HD_NO_FRAC_GENLOCK_NUM                  1000

#define HD_PCR_NO_FRAC_DEN                              2750
#define HD_PCR_NO_FRAC_NUM                              1000

#define HD_WITH_FRAC_GENLOCK_DEN                825000
#define HD_WITH_FRAC_GENLOCK_NUM                1001

#define HD_PCR_WITH_FRAC_DEN                    2750
#define HD_PCR_WITH_FRAC_NUM                    1001

/* Regular SD clocks ~27MHz */
#define SD_NO_FRAC_GENLOCK_DEN                  300000
#define SD_NO_FRAC_GENLOCK_NUM                  1000

#define SD_PCR_NO_FRAC_DEN                              1000
#define SD_PCR_NO_FRAC_NUM                              1000

#define SD_WITH_FRAC_GENLOCK_DEN                300300
#define SD_WITH_FRAC_GENLOCK_NUM                1000

#define SD_PCR_WITH_FRAC_DEN                    1001    
#define SD_PCR_WITH_FRAC_NUM                    1000

/* VGA clocks ~25.2MHz */
#define VGA_25P2_GENLOCK_DEN                    280000
#define VGA_25P2_GENLOCK_NUM                    1000

#define VGA_25P2_PCR_DEN                        2800
#define VGA_25P2_PCR_NUM                        3000

#define VGA_25P175_GENLOCK_DEN                  280000
#define VGA_25P175_GENLOCK_NUM                  1001

#define VGA_25P175_PCR_DEN                      2800
#define VGA_25P175_PCR_NUM                      3003

#define CLEAR_FRAME_COUNTER_FLAG 0x080000000

#define DEFAULT_VID0_LEVEL 0
#define DEFAULT_VID1_LEVEL 1
#define DEFAULT_GRA0_LEVEL 3
#define DEFAULT_GRA1_LEVEL 2

#define FES_BOTH_USE_FIFO 2	

#define DEFAULT_VID1_BUFFER_WIDTH 2048
#define DEFAULT_VID1_BUFFER_HEIGHT 1088 * 10

#define DEFAULT_VIR_VIDEO_SCALE_POS 9

#define GIC_BOTH_CMD_FIF0 0x02

#define DEFAULT_VIDEO_BUFFER_WIDTH 1920
#define DEFAULT_VIDEO_BUFFER_HEIGHT 1088
#define DEFAULT_VIDEO_BUFFER_NUMBER 18

#define DEFALUT_BACK_LAYER_HEIGHT 2205

#define MAX_GRAPHIC_BUFFER_WIDTH 1920
#define MAX_GRAPHIC_BUFFER_HEIGHT 2205

//229 * 1024 * 1024
#define MAX_VIDEO_AREA_SIZE 0x0e500000 

#define YUV_PASS_THROUGH    0
#define YUV709_TO_601       1
#define YUV601_TO_709       2

#define CURRENT_MODE_IS_3D_OLD  \
    ((mdrv->shared->current_hd_resolution == DSOR_1920_2205) ||\
     (mdrv->shared->current_hd_resolution == DSOR_1920_1080_3D_SIDE_BY_SIDE) ||\
     (mdrv->shared->current_hd_resolution == DSOR_1280_1470))

     
/* 
*   The code that depends on this define should only be executed when in a frame packing mode,
*   SbS and TaB should behave the exact same as 2D modes.
*/
#define CURRENT_MODE_IS_3D  \
    ((mdrv->shared->current_hd_resolution == DSOR_1920_2205) ||\
     (mdrv->shared->current_hd_resolution == DSOR_1280_1470))     
     
     
#define CURRENT_MODE_IS_INTERLACED_3D \
    ((mdrv->shared->current_hd_resolution == DSOR_1920_2205) &&\
     (mdrv->shared->current_hd_scanmode == DSESM_INTERLACED))
     
     
#define TOUCH_3D_BACKGROUND_LAYER \
 ((CURRENT_MODE_IS_3D_OLD) &&\
  (dfb_layer_id (layer) == BACKGROUND_GRA_LAYER_ID))

#define FRAME_PACKING_1080P_VIDEO_LAYER_NOT_FULL_SCREEN \
((mdrv->shared->current_hd_resolution == DSOR_1920_2205) &&\
 (( video0_screen_width < 1920) || (video0_screen_height < 1080) || (video1_screen_width < 1920) || (video1_screen_height < 1080)))

#define FRAME_PACKING_720P_VIDEO_LAYER_NOT_FULL_SCREEN \
((mdrv->shared->current_hd_resolution == DSOR_1280_1470) &&\
 (( video0_screen_width < 1280) || (video0_screen_height < 720) || (video1_screen_width < 1280) || (video1_screen_height < 720)))

#define SIDE_BY_SIDE_1080P_VIDEO_LAYER_NOT_FULL_SCREEN \
((mdrv->shared->current_hd_resolution == DSOR_1920_1080_3D_SIDE_BY_SIDE) &&\
 (( video0_screen_width < 960) || (video0_screen_height < 1080) || (video1_screen_width < 960) || (video1_screen_height < 1080)))

#define VIDEO_CAN_BE_SEEN \
(REG_READ(DISP_HD_COMP_ALPHA) && (ALPHA_V0_MASK | ALPHA_V1_MASK))

#define CURRENT_VIDEO_LAYER_IS_NOT_FULL_SCREEN \
((FRAME_PACKING_1080P_VIDEO_LAYER_NOT_FULL_SCREEN ||\
 FRAME_PACKING_720P_VIDEO_LAYER_NOT_FULL_SCREEN || \
 SIDE_BY_SIDE_1080P_VIDEO_LAYER_NOT_FULL_SCREEN) && \
VIDEO_CAN_BE_SEEN)

#define VIDEO_LAYER_LUMA_ADDR_ALIGNMENT 64
#define DISP_INIT_POS 5

#define START_POS_OUT_OF_RANGE \
((config->source.x - x_round_down) + DISP_INIT_POS) >= VIDEO_LAYER_LUMA_ADDR_ALIGNMENT

#define G2D_IS_LOCKED_BY_KERNEL \
((REG_READ(MIPS_INTERLOCK5) & MIPS_INTERLOCK5_ID0_MASK) == KERNEL_G2D_HW_SEMAPHORE_ID)


#define G2D_MAX_PITCH_SIZE (G2D_SRC_PITCH_FORMAT_SRC_PITCH_MASK >> G2D_SRC_PITCH_FORMAT_SRC_PITCH_SHIFT)
#define MAX_GIC_SRC_PITCH GIC1_SRC1_PITCH_MASK

#define REG_WRITE_AND_POLL(reg,value)\
    REG_WRITE (reg, value);\
    while (REG_READ(reg) != value) \
    {\
		;\
	}

typedef struct _CoefParams
{

  u32 type;
  double lpf;
  double beta;
  double lpf0;   // not setable from command line 
  double rho;  // not setable from command line 
}CoefParams;

#define RANDOM_FILTER 1
#define SINC_FILTER 2
#define ZOH_FILTER 3
#define LINEAR_FILTER 4
#define LANCZOS2_FILTER 5
#define LANCZOS3_FILTER 6

#define DISP_HDMI_POWER_DOWN 0x80



DFBResult
Xcode4InitAuxOSDFunctions(void);


#endif


