/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_osd_gra0.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/

#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <directfb.h>
#include <pthread.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"

extern DFBResult XcodeFlipSDFrame(CoreLayer *layer, 
                                         const DFBRegion *update,
                                         CoreSurfaceBufferLock *lock);

extern DFBResult convert_dfb_format_to_disp_format(const CoreLayerRegionConfig *config, 
                                            int * disp_format, int *ignorePixelAlpha);

extern struct display_mode_struct * xc4_get_mode(int mode_index);

extern int xc_layer_search_mode_index(CoreLayerRegionConfig * config);

extern u32 Calculate_gfx_hsca_inv_ratio(u32 input_h_size, u32 output_h_size);

extern void Calculate_gfx_vsca_inv_ratio(DFBScreenEncoderScanMode  scanmode,
                                         u32 input_v_size, 
                                         u32 output_v_size,
                                         u32 * gra_vsca_inv_ratio,
                                         u32 * gra_vsca_init,
                                         u32 * gra_vsca_start);

static u32 disp_gra0_register_table[SIZE_OF_DISP_GRA_REGISTER_TABLE];

extern int XcodeVidLayerDataSize();

void xcodeGraFlipRegion(u32 *disp_reg_tab, 
                  void *driver_data, 
                  void * layer_data,
                  CoreSurface *surface, 
                  DFBSurfaceFlipFlags flags, 
                  CoreSurfaceBufferLock *lock, 
                  int compositor);

void	  xcodeAlignBufferSize(u32 *disp_reg_tab, void *driver_data);

DFBResult Xcode4_Set_Graphic_Layers(void                       *driver_data,
                                    void                       *layer_data,
                                    CoreLayerRegionConfig      *config,
                                    CoreLayerRegionConfigFlags  updated,
                                    CorePalette                *palette,
                                    CoreSurfaceBufferLock      *lock,
                                    u32 * disp_gra_register_table);

extern DFBResult
Xc4GraReallocateSurface( CoreLayer             *layer,
                          void                  *driver_data,
                          void                  *layer_data,
                          void                  *region_data,
                          CoreLayerRegionConfig *config,
                          CoreSurface           *surface );

extern void CSC_ProgAdjustMatrix(pxc_dfb_driver_data mdrv, int layer_id);

#define GIC_ARGB_SURFACE 13

/**********************************************************************************************************************/
void Init_Disp_Gra0_Register_Table()
{
     disp_gra0_register_table[DISP_HD_GRA_CLUT_DATA_INDEX] = DISP_HD_RAM_WDATA;
     disp_gra0_register_table[DISP_HD_GRA_CLUT_INDEX] = DISP_HD_GRA0_CLUT_ADDR;
     disp_gra0_register_table[DISP_HD_GRA_FORMAT_INDEX] = DISP_HD_GRA0_FORMAT;
     disp_gra0_register_table[DISP_HD_GRA_FORMAT1_INDEX] = DISP_HD_GRA0_FORMAT1;
     disp_gra0_register_table[DISP_HD_GRA_BUF_INDEX] = DISP_HD_GRA0_BUF;
     disp_gra0_register_table[DISP_HD_GRA_SAMPLES_INDEX] = DISP_HD_GRA0_SAMPLES;
     disp_gra0_register_table[DISP_HD_GRA_ADR_INDEX] = DISP_HD_GRA0_ADR;
     disp_gra0_register_table[DISP_HD_GRA_SIZE_INDEX] = DISP_HD_GRA0_SIZE;
     disp_gra0_register_table[DISP_HD_GRA_VSCA_INV_RATIO_INDEX] = DISP_HD_GRA0_VSCA_INV_RATIO;
     disp_gra0_register_table[DISP_HD_GRA_VSCA_INIT_INDEX] = DISP_HD_GRA0_VSCA_INIT;
     disp_gra0_register_table[DISP_HD_GRA_VSCA_START_INDEX] = DISP_HD_GRA0_VSCA_START;
     disp_gra0_register_table[DISP_HD_GRA_HSCA_INV_RATIO_INDEX] = DISP_HD_GRA0_HSCA_INV_RATIO;
     disp_gra0_register_table[DISP_HD_COMP_GRA_POS_INDEX] = DISP_HD_COMP_GRA0_POS;
     disp_gra0_register_table[DISP_HD_COMP_ALPHA_GRA_SHIFT_INDEX] = ALPHA_G0_SHIFT;
     disp_gra0_register_table[DISP_HD_GRA_SCALED_SIZE_INDEX] = DISP_HD_GRA0_SCALED_SIZE;
     disp_gra0_register_table[DISP_HD_COMP_ALPHA_MODE_GRA_SHIFT_INDEX] = ALPHA_MODE_G0_SHIFT;
     disp_gra0_register_table[DISP_HD_CC_GRA_MODE_SHIFT_INDEX] = DISP_HD_CC_GRA0_MODE_SHIFT;
     disp_gra0_register_table[DISP_HD_CC_GRA_MODE_MASK_INDEX] = DISP_HD_CC_GRA0_MODE_MASK;
     disp_gra0_register_table[DISP_HD_GRA_DEFAULT_TAG_INDEX] = DISP_HD_GRA0_DEFAULT_TAG;
     disp_gra0_register_table[ALPHA_GRAPHIC_MASK_INDEX] = ALPHA_G0_MASK;
     disp_gra0_register_table[DISP_HD_GRA_IDX_MASK_INDEX] = HD_GRA0_IDX_MASK;
     disp_gra0_register_table[DISP_HD_GRA_IDX_SHIFT_INDEX] = HD_GRA0_IDX_SHIFT;
     disp_gra0_register_table[DISP_HD_GRA_FREE_ENTRIES_MASK_INDEX] = GRA0_FREE_ENTRIES_MASK;
     disp_gra0_register_table[DISP_HD_GRA_FREE_ENTRIES_SHIFT_INDEX] = GRA0_FREE_ENTRIES_SHIFT;
     disp_gra0_register_table[DISP_HD_GRA_CMD_FIFO_INDEX] = DISP_HD_GRA0_CMD0_FIFO;
     disp_gra0_register_table[DISP_HD_GRA_BUF_SIZE_MASK_INDEX]	 =    DISP_HD_GRA0_BUF_SIZE_MASK;
     disp_gra0_register_table[DISP_HD_GRA_BUF_SIZE_SHIFT_INDEX]	 =    DISP_HD_GRA0_BUF_SIZE_SHIFT;
     disp_gra0_register_table[DISP_HD_COMP_ALPHA_MODE_GRA_MASK_INDEX]	 =    ALPHA_MODE_G0_MASK;
     disp_gra0_register_table[GRA_EMPTY_MASK_INDEX] = GRA0_EMPTY_MASK;
     disp_gra0_register_table[GRA_EMPTY_SHIFT_INDEX] = GRA0_EMPTY_SHIFT;

}

/**********************************************************************************************************************/

static DFBResult
XcodeGra0InitLayer( CoreLayer                *layer,
              void                       *driver_data,
              void                       *layer_data,
              DFBDisplayLayerDescription *description,
              DFBDisplayLayerConfig      *config,
              DFBColorAdjustment         *adjustment )
{
     u32     tmp;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;

     D_DEBUG( "xcode DFB driver XcodeGra0InitLayer enter:\n" );


     /* set capabilities and type */
     description->caps = DLCAPS_SURFACE | DLCAPS_ALPHACHANNEL | DLCAPS_OPACITY | 
                         DLCAPS_SCREEN_POSITION | DLCAPS_SCREEN_LOCATION | DLCAPS_LEVELS;

     description->type = DLTF_GRAPHICS;

     /* set name */
     snprintf( description->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "ViXS Xcode4 Graphics Plane 0" );

     /* fill out the default configuration */
     config->flags       = DLCONF_WIDTH       | DLCONF_HEIGHT |
                           DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS | DLCONF_SURFACE_CAPS ;

     if (dfb_config->preallocate_graphic_layer & (0x1 << XCODE4_GRA0_LAYER_ID))
     {
         config->width       = MAX_GRAPHIC_BUFFER_WIDTH;
         config->height      = MAX_GRAPHIC_BUFFER_HEIGHT;
     }
     else
     {
         config->width       = XCODE_DEFAULT_WIDTH;
         config->height      = XCODE_DEFAULT_HEIGHT;
     }

     config->pixelformat = DSPF_ARGB;

     config->surface_caps |= DSCAPS_VIDEOONLY | DSCAPS_STATIC_ALLOC | DSCAPS_DOUBLE ;

     if (dfb_config->preallocate_graphic_layer & (0x1 << XCODE4_GRA0_LAYER_ID))
     {
         config->buffermode  = DLBM_BACKVIDEO;
         config->options     = DLOP_ALPHACHANNEL;
     }
     else
     {
         config->buffermode  = DLBM_FRONTONLY;
         config->options     = DLOP_ALPHACHANNEL;
     }

     adjustment->flags   = DCAF_BRIGHTNESS;

     memset (layer_data, 0, sizeof (xc_layer_data));

     //Set the gra0 register table
     Init_Disp_Gra0_Register_Table();

     //Initailize destination buffer
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     mdrv->shared->vir_gra_gic_ctl.dst_buffer1 = 0;
     mdrv->shared->vir_gra_gic_ctl.dst_buffer2 = 0;

     D_DEBUG( "xcode DFB driver XcodeGra0InitLayer exit:\n" );

     return DFB_OK;
}

/**********************************************************************************************************************/

static DFBResult
XcodeGra0TestRegion( CoreLayer                  *layer,
               void                       *driver_data,
               void                       *layer_data,
               CoreLayerRegionConfig      *config,
               CoreLayerRegionConfigFlags *failed )
{
     CoreLayerRegionConfigFlags fail = 0;

     D_DEBUG( "xcode DFB driver Xcode4Gra0TestRegion enter:\n" );

     if (config->options & ~XCODE4_OSD_SUPPORTED_OPTIONS)
          fail |= CLRCF_OPTIONS;

     /* check buffermode */
     switch (config->buffermode) {
          case DLBM_FRONTONLY:
          case DLBM_BACKSYSTEM:
          case DLBM_BACKVIDEO:
          case DLBM_TRIPLE:
               break;

          default:
               fail |= CLRCF_BUFFERMODE;
               break;
     }

     /* check graphic format */
     switch (config->format) {
          case DSPF_LUT8:
          case DSPF_RGB24:
          case DSPF_RGB16:
          case DSPF_RGB32:
          case DSPF_ARGB1555:
          case DSPF_RGB555:
          case DSPF_ARGB:
               break;

          default:
               fail |= CLRCF_FORMAT;
     }


     if (failed)
          *failed = fail;

     if (fail)
     {
          return DFB_UNSUPPORTED;
     }

     D_DEBUG( "xcode DFB driver XcodeGra0TestRegion exit:\n" );

     return DFB_OK;
}

/**********************************************************************************************************************/

DFBResult
xcode4_set_disp_gra_palette( void * driver_data, 
                             void *layer_data,
                             CorePalette *palette, 
                             u32 * disp_gra_register_table )
{
     int i;
     int num_entries;  
     u32 color;
     u32 current_active_lut_page;

     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;

     D_ASSERT( palette != NULL );

     m_layer_data->lut_require_switch = 1;

     if (mdrv->asic_flags == VIPER_ASIC_A21)
     {
         current_active_lut_page = (REG_READ(disp_gra_register_table[DISP_HD_GRA_FORMAT1_INDEX]) & DISP_HD_GRA1_FORMAT1_CLUT_PAGE_MASK) >> DISP_HD_GRA1_FORMAT1_CLUT_PAGE_SHIFT;
     }
     else
     {
         current_active_lut_page = m_layer_data->current_active_lut_page;
     }
     //get number of palette entries
     num_entries = palette->num_entries <= 256 ? palette->num_entries : 256;

     //set the start value of CLUT index
     if (m_layer_data->buffermode == DLBM_FRONTONLY )
     {
         if (current_active_lut_page == 0)
         {
             REG_WRITE( DISP_HD_RAM_ADDR, disp_gra_register_table[DISP_HD_GRA_CLUT_INDEX]);
         }
         else
         {
             REG_WRITE( DISP_HD_RAM_ADDR, disp_gra_register_table[DISP_HD_GRA_CLUT_INDEX] + 256);
         }
     }
     else
     {
         if (current_active_lut_page == 0)
         {

             REG_WRITE( DISP_HD_RAM_ADDR, disp_gra_register_table[DISP_HD_GRA_CLUT_INDEX] + 256);
         }
         else
         {
             REG_WRITE( DISP_HD_RAM_ADDR, disp_gra_register_table[DISP_HD_GRA_CLUT_INDEX]);
         }
     }
    
     //fill the color into the CLUT
     for (i = 0; i < num_entries; i++) 
     {
          color = palette->entries[i].b | (palette->entries[i].g << 8) | (palette->entries[i].r << 16) | (palette->entries[i].a << 24);
    
          REG_WRITE( 
                   disp_gra_register_table[DISP_HD_GRA_CLUT_DATA_INDEX], 
                   color);

          m_layer_data->palette[i] = color;

          usleep(1);
     }

     m_layer_data->palette_num = num_entries;

     m_layer_data->palette_need_switch = 1;

     mdrv->shared->sd_frame_entries.palette_update = 1;

     return DFB_OK;
}

/**********************************************************************************************************************/

struct display_mode_struct * xc4_find_dve_mode(pxc_dfb_driver_data mdrv)
{
    struct display_mode_struct * mode = NULL;

    if (mdrv->shared->mode_index != INDEX_UNKNOWN)
    {
        mode = xc4_get_mode(mdrv->shared->mode_index);
    }

    return mode;
}

/**********************************************************************************************************************/

void set_graphic_plane_gra(void * driver_data, 
                           void                       *layer_data,
                           CoreLayerRegionConfig *config,
                           CoreLayerRegionConfigFlags  updated,
                           CoreSurfaceBufferLock *lock,
                           u32 * disp_gra_register_table,
                           int disp_format,
			   int ignorePixelAlpha)
{
     u32 tmp;
     u32 gra_vsca_inv_ratio;
     u32 gra_vsca_init;
     u32 gra_vsca_start;
     u32 alpha_mode;
     struct display_mode_struct * mode = NULL;

     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     u32 max_req_size;
     u32 gra_adr;

     mode = xc4_find_dve_mode(mdrv);

     if (mode == NULL )
     {
         
         D_DEBUG ("Failed to get the display mode \n");
         return;
     }

     m_layer_data->buf_width = config->width;
     m_layer_data->buf_height = config->height;
     m_layer_data->dest_width = config->dest.w;
     m_layer_data->dest_height = config->dest.h;
     m_layer_data->source_width = config->source.w;
     m_layer_data->source_height = config->source.h;
     m_layer_data->flip_height = config->source.h;
     m_layer_data->source_x = config->source.x;
     m_layer_data->source_y = config->source.y;
     m_layer_data->y_start_addr = lock->phys;
     m_layer_data->y_pitch = lock->pitch;
     m_layer_data->format = config->format;
     m_layer_data->x = config->dest.x;
     m_layer_data->y = config->dest.y;

     m_layer_data->buffermode = config->buffermode;

     //setup the memory interface


     if ((config->format == DSPF_LUT8) &&
         (config->source.h <= 720))
     {
         max_req_size = 0x20;
     }
     else
     {
         max_req_size = 0x28;
     }

     tmp = disp_format | (max_req_size << DISP_HD_GRA0_FORMAT_MAX_REQ_SIZE_SHIFT) | 
           (disp_gra_register_table[DISP_HD_GRA_DEFAULT_TAG_INDEX] << DISP_HD_GRA0_FORMAT_TAG_SHIFT) |
           (DISABLE_RANDOM << DISP_HD_GRA0_FORMAT_DITHER_SHIFT) |
           DISP_HD_GRA0_FORMAT_UNDERFLOW_RESET_OFF_MASK;

     //Set TAG value
     if(disp_gra_register_table[DISP_HD_GRA_FORMAT_INDEX] == DISP_HD_GRA0_FORMAT)
     {
          tmp = (tmp & ~DISP_HD_GRA0_FORMAT_TAG_MASK) | (0x8 << DISP_HD_GRA0_FORMAT_TAG_SHIFT);
     }
     else
     {
          tmp = (tmp & ~DISP_HD_GRA1_FORMAT_TAG_MASK) | (0x9 << DISP_HD_GRA1_FORMAT_TAG_SHIFT);
     }

     REG_WRITE(disp_gra_register_table[DISP_HD_GRA_FORMAT_INDEX], 
               tmp);

     tmp = REG_READ( disp_gra_register_table[DISP_HD_GRA_FORMAT1_INDEX]);

     if (mdrv->asic_flags == VIPER_ASIC_A21)
     {
         tmp = tmp & (~0x0ff);
         tmp = tmp | DFB_ALPHA_POS 
                   | (DFB_RED_POS << DISP_HD_GRA0_FORMAT1_PEL_POSITION_R_SHIFT)
                   | (DFB_GREEN_POS << DISP_HD_GRA0_FORMAT1_PEL_POSITION_G_SHIFT)
                   | (DFB_BLUE_POS << DISP_HD_GRA0_FORMAT1_PEL_POSITION_B_SHIFT);
     }
     else
     {
         tmp = tmp & (~0x0ff) & (~ DISP_HD_GRA0_FORMAT1_CLUT_PAGE_MASK);
     tmp = tmp | DFB_ALPHA_POS 
               | (DFB_RED_POS << DISP_HD_GRA0_FORMAT1_PEL_POSITION_R_SHIFT)
     	       | (DFB_GREEN_POS << DISP_HD_GRA0_FORMAT1_PEL_POSITION_G_SHIFT)
     	       | (DFB_BLUE_POS << DISP_HD_GRA0_FORMAT1_PEL_POSITION_B_SHIFT)
               | (m_layer_data->current_active_lut_page << DISP_HD_GRA0_FORMAT1_CLUT_PAGE_SHIFT);
     }

     REG_WRITE( disp_gra_register_table[DISP_HD_GRA_FORMAT1_INDEX], tmp);

     tmp = (config->source.h * lock->pitch);
     
     D_DEBUG("Old size: %08X\n", tmp);
//   SET_NUMBER_ALIGNMENT(tmp, tmp, 0x2000)
     
//   D_DEBUG("New size: %08X\n", tmp);
     if (updated != CLRCF_DEST)
	 {
         tmp>>= VIDEO_BUFFER_SIZE_ALIGNMENT_SHIFT;
         tmp = (tmp <<  DISP_HD_GRA0_BUF_SIZE_SHIFT) | ((lock->pitch >> 4) << DISP_HD_GRA0_BUF_PITCH_SHIFT);
         REG_WRITE( disp_gra_register_table[DISP_HD_GRA_BUF_INDEX], tmp);
	 }

     REG_WRITE( disp_gra_register_table[DISP_HD_GRA_SAMPLES_INDEX], config->source.w * DFB_BYTES_PER_PIXEL(config->format));

     gra_adr = lock->phys + (config->source.y * lock->pitch) + (config->source.x * DFB_BYTES_PER_PIXEL(config->format));
     REG_WRITE( disp_gra_register_table[DISP_HD_GRA_ADR_INDEX], gra_adr);

     /**********************     setup h scaler and v scaler for gra0 ****************************/
     REG_WRITE( disp_gra_register_table[DISP_HD_GRA_SIZE_INDEX], (config->source.w | (config->source.h << DISP_HD_GRA0_SIZE_V_SIZE_SHIFT)));

     REG_WRITE( disp_gra_register_table[DISP_HD_GRA_SCALED_SIZE_INDEX], (config->dest.w | (config->dest.h << DISP_HD_GRA0_SIZE_V_SIZE_SHIFT) ));

     Calculate_gfx_vsca_inv_ratio(mode->scanmode, 
                                  config->source.h, 
                                  config->dest.h,
                                  &gra_vsca_inv_ratio,
                                  &gra_vsca_init,
                                  &gra_vsca_start);

     REG_WRITE( disp_gra_register_table[DISP_HD_GRA_VSCA_INV_RATIO_INDEX], gra_vsca_inv_ratio);

     if (mode->scanmode == DSESM_INTERLACED)
     {
         REG_WRITE( disp_gra_register_table[DISP_HD_GRA_VSCA_INIT_INDEX], gra_vsca_init);
         REG_WRITE( disp_gra_register_table[DISP_HD_GRA_VSCA_START_INDEX], gra_vsca_start);
     }
    
     tmp = Calculate_gfx_hsca_inv_ratio(config->source.w, config->dest.w);
     REG_WRITE( disp_gra_register_table[DISP_HD_GRA_HSCA_INV_RATIO_INDEX], tmp);

     //set the screen position 
     REG_WRITE( 
              disp_gra_register_table[DISP_HD_COMP_GRA_POS_INDEX], 
              config->dest.x | (config->dest.y << DISP_HD_COMP_GRA0_POS_POS_Y_SHIFT));


     //set the layer global alpha
     tmp = REG_READ( DISP_HD_COMP_ALPHA);
     tmp = tmp & (~disp_gra_register_table[ALPHA_GRAPHIC_MASK_INDEX]);
     if (ignorePixelAlpha == 0) 
     {	
	//Leave as previous driver 
	tmp = tmp | ((config->opacity & 0x0ff) << disp_gra_register_table[DISP_HD_COMP_ALPHA_GRA_SHIFT_INDEX]);
	
     }
     else
     {
	//Set plane to be fully opaque in the case that ignorePixelAlpha is 1
	tmp = tmp | ((0xff) << disp_gra_register_table[DISP_HD_COMP_ALPHA_GRA_SHIFT_INDEX]);
	
     }			
     REG_WRITE( DISP_HD_COMP_ALPHA, tmp);
     
     //set the alpha mode mask
     //Use the PER_PIXEL_ALPHA if either DLOP alpha channel or ignorePixelAlpha is clear.
     if (config->options & DLOP_ALPHACHANNEL && ignorePixelAlpha == 0)
     {
        alpha_mode = USE_PER_PIXEL_ALPHA;
     }
     else
     {
        alpha_mode = USE_VALUE_IN_REGISTER;
     }
     tmp = REG_READ(DISP_HD_COMP_ALPHA_MODE);
     tmp = tmp & (~disp_gra_register_table[DISP_HD_COMP_ALPHA_MODE_GRA_MASK_INDEX]);
     tmp = tmp | (alpha_mode << disp_gra_register_table[DISP_HD_COMP_ALPHA_MODE_GRA_SHIFT_INDEX]);
     REG_WRITE( DISP_HD_COMP_ALPHA_MODE, tmp);

     if(disp_gra_register_table[DISP_HD_GRA_FORMAT_INDEX] == DISP_HD_GRA0_FORMAT)
     {
        CSC_ProgAdjustMatrix(mdrv, XCODE4_GRA0_LAYER_ID);
     }
     else
     {
        CSC_ProgAdjustMatrix(mdrv, XCODE4_GRA1_LAYER_ID);
     }

     tmp = GRA0_CMD0_FIFO_WE_MASK | DISP_HD_GRA0_CMD0_FIFO_UPDATE_COMP_REGS_MASK | UPDATE_GRA0_REGS_MASK;
     REG_WRITE(disp_gra_register_table[DISP_HD_GRA_CMD_FIFO_INDEX], tmp);

     REG_WRITE(disp_gra_register_table[DISP_HD_GRA_CMD_FIFO_INDEX], UPDATE_GRA0_REGS_MASK);

}

/**********************************************************************************************************************/

static DFBResult
XcodeGra0SetRegion( CoreLayer                  *layer,
                    void                       *driver_data,
                    void                       *layer_data,
                    void                       *region_data,
                    CoreLayerRegionConfig      *config,
                    CoreLayerRegionConfigFlags  updated,
                    CoreSurface                *surface,
                    CorePalette                *palette,
                    CoreSurfaceBufferLock      *lock )
{
    DFBResult ret;
    D_ASSERT( driver_data != NULL );
    D_ASSERT( layer_data != NULL );
    app_ioctl_command dfb_ioctl_cmd;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32 gic_fifo_entries;
    u32 temp_height;

    D_DEBUG( "xcode DFB driver XcodeGra0SetRegion enter \n" );
    D_DEBUG ("surface width %d \n", config->width);
    D_DEBUG ("surface buffer height %d \n", config->height);
    D_DEBUG ("destination x %d \n", config->dest.x);
    D_DEBUG ("destination y %d \n", config->dest.y);
    D_DEBUG ("destination weight %d \n", config->dest.w);
    D_DEBUG ("destination height %d \n", config->dest.h);
    D_DEBUG ("source x %d \n", config->source.x);
    D_DEBUG ("source y %d \n", config->source.y);
    D_DEBUG ("source width %d \n", config->source.w);
    D_DEBUG ("source height %d \n", config->source.h);
    D_DEBUG ("surface caps %d \n", config->surface_caps);
    D_DEBUG ("surface pitch %d \n", lock->pitch);
    D_DEBUG ("surface phys 0x%x \n", lock->phys);
    D_DEBUG ("surface opacity  0x%x \n", config->opacity);

    pthread_mutex_lock( &mdrv->shared->reentrancy_mutex );
    ret = Xcode4_Set_Graphic_Layers(driver_data,
                                    layer_data,
                                    config,
                                    updated,
                                    palette,
                                    lock,
                                    disp_gra0_register_table);
    pthread_mutex_unlock( &mdrv->shared->reentrancy_mutex );

    temp_height =  config->source.h;

    //adjut the dst height to cut down the data traffic for gic combine
    switch (mdrv->shared->current_hd_resolution)
    {         
        case DSOR_1920_1080:
            temp_height = 1080;
            break;

        case DSOR_1920_2205:
            temp_height = 2205;
            break;

        case DSOR_1280_720:
            temp_height = 720;
            break;

        case DSOR_1280_1470:
            temp_height = 1470;
            break;

        case DSOR_720_480:
        case DSOR_1440_480:
        case DSOR_640_480:
            temp_height = 480;
            break;

        case DSOR_720_576:
            temp_height = 576;
            break;

        default:
            break;
    }

    //Initialize parameters
    mdrv->shared->vir_gra_gic_ctl.dst_buffer1 = m_layer_data->y_start_addr;
    mdrv->shared->vir_gra_gic_ctl.dst_buffer2 = m_layer_data->y_start_addr;
    mdrv->shared->vir_gra_gic_ctl.dst_pitch = m_layer_data->y_pitch  | ((m_layer_data->c_pitch/GIC_ALIGN_DIV)  << GIC0_DST_UV_PITCH_SHIFT);
    mdrv->shared->vir_gra_gic_ctl.gic_dst_adr0 = mdrv->shared->vir_gra_gic_ctl.dst_buffer1;
    mdrv->shared->vir_gra_gic_ctl.gic_dst_pitch = mdrv->shared->vir_gra_gic_ctl.dst_pitch;

    if (config->buffermode != DLBM_FRONTONLY)
    {
        if( surface )
        {
            CoreSurfaceBufferLock tmp;
            ret = dfb_surface_lock_buffer( surface, CSBR_FRONT, CSAID_GPU, CSAF_READ, &tmp );
            if(!ret)
            {
                mdrv->shared->vir_gra_gic_ctl.dst_buffer1 = tmp.phys;
                dfb_surface_unlock_buffer( surface, &tmp);
            }
            ret = dfb_surface_lock_buffer( surface, CSBR_BACK, CSAID_GPU, CSAF_READ, &tmp );
            if(!ret)
            {
                mdrv->shared->vir_gra_gic_ctl.dst_buffer2 = tmp.phys;
                mdrv->shared->vir_gra_gic_ctl.gic_dst_adr0 = tmp.phys;
                dfb_surface_unlock_buffer( surface, &tmp);
            }
        }
    }

    mdrv->shared->vir_gra_gic_ctl.gic_opt_status |= GIC_DST_READY;

    D_DEBUG( "xcode DFB driver XcodeGra0SetRegion exit \n" );

    return ret;
}

void xcodeGraFlipRegion(u32 *disp_reg_tab, 
                  void *driver_data, 
                  void * layer_data,
                  CoreSurface *surface, 
                  DFBSurfaceFlipFlags flags, 
                  CoreSurfaceBufferLock *lock, 
                  int compositor ) 
{ 

    u32       new_index, fifo_word[2];
    u32       current_frm_cnt, base_frm_cnt;     
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32       cnt;
	u32 tmp;
	u32 free_entries = 0;
    u32 buff_size;
    app_ioctl_command	app_cmd;
    u32 current_active_lut_page;
    u32 gra_adr;
    /* Code needs to be put here to fail the call if the queued frame has not been displayed */

    D_DEBUG("Xcode4 Base Address Register change (new: %08X, current: %08X)\n", lock->phys, REG_READ(disp_reg_tab[DISP_HD_GRA_ADR_INDEX]));

    if(surface->config.caps & DSCAPS_TRIPLE)
    {
        while ((m_layer_data->last_frame_counter == GET_HD_FRM_COUNT) &&
          (REG_READ(DISP_HD_COMP_CTRL) & DISP_HD_COMP_CTRL_COMP_EN_MASK))
        {
            usleep(1);
        }
    }

    //check if switch lut 
    if ((m_layer_data->lut_require_switch ) && 
        (m_layer_data->buffermode != DLBM_FRONTONLY ) &&
        (DFB_PIXELFORMAT_IS_INDEXED(m_layer_data->format)))
    {

        if (mdrv->asic_flags == VIPER_ASIC_A21)
        {

            gra_adr = lock->phys + (m_layer_data->source_y * lock->pitch) + (m_layer_data->source_x * DFB_BYTES_PER_PIXEL(m_layer_data->format));

            REG_WRITE(disp_reg_tab[DISP_HD_GRA_ADR_INDEX], gra_adr);

            if (m_layer_data->lut_require_switch)
            {
                m_layer_data->lut_require_switch = 0;

                current_active_lut_page = (REG_READ(disp_reg_tab[DISP_HD_GRA_FORMAT1_INDEX]) & DISP_HD_GRA1_FORMAT1_CLUT_PAGE_MASK) >> DISP_HD_GRA1_FORMAT1_CLUT_PAGE_SHIFT;

                current_active_lut_page = current_active_lut_page ^ 1;

                tmp = (REG_READ(disp_reg_tab[DISP_HD_GRA_FORMAT1_INDEX]) & (~DISP_HD_GRA1_FORMAT1_CLUT_PAGE_MASK)) | (current_active_lut_page << DISP_HD_GRA1_FORMAT1_CLUT_PAGE_SHIFT);
                REG_WRITE(disp_reg_tab[DISP_HD_GRA_FORMAT1_INDEX], tmp);
            }

            tmp = GRA0_CMD0_FIFO_WE_MASK | UPDATE_GRA0_REGS_MASK | DISP_HD_GRA0_CMD0_FIFO_UPDATE_COMP_REGS_MASK;
            REG_WRITE(disp_reg_tab[DISP_HD_GRA_CMD_FIFO_INDEX], tmp);

        }
        else if (mdrv->asic_flags == VIPER_ASIC_A11)
        {
            while ((free_entries != 0x10)
 &&
                   (REG_READ(DISP_HD_COMP_CTRL) & DISP_HD_COMP_CTRL_COMP_EN_MASK))
            {
                usleep(1);
                free_entries = ((REG_READ(DISP_HD_FIFO_STATUS1)) & disp_reg_tab[DISP_HD_GRA_FREE_ENTRIES_MASK_INDEX]) >>
                               disp_reg_tab[DISP_HD_GRA_FREE_ENTRIES_SHIFT_INDEX] ;
            }

            while ((REG_READ(GIC_DUMMY_DIR_REG)) &&
                   (REG_READ(DISP_HD_COMP_CTRL) & DISP_HD_COMP_CTRL_COMP_EN_MASK))
            {
                usleep(1);
            }

            if (m_layer_data->palette_need_switch)
            {
                m_layer_data->current_active_lut_page = m_layer_data->current_active_lut_page ^ 0x01;
                REG_WRITE(GIC_DUMMY_DIR_REG, lock->phys);
            }
            else
            {
                REG_WRITE(disp_reg_tab[DISP_HD_GRA_ADR_INDEX], lock->phys);
                REG_WRITE(disp_reg_tab[DISP_HD_GRA_CMD_FIFO_INDEX], UPDATE_GRA0_REGS_MASK);
            }

            m_layer_data->palette_need_switch = 0;

            if (flags & DSFLIP_ONSYNC)
            {
                while (REG_READ(GIC_DUMMY_DIR_REG))
                {
                    usleep(1);
                }
            }
        }
    }
    else
    {
        gra_adr = lock->phys + (m_layer_data->source_y * lock->pitch) + (m_layer_data->source_x * DFB_BYTES_PER_PIXEL(m_layer_data->format));

        REG_WRITE(disp_reg_tab[DISP_HD_GRA_ADR_INDEX], gra_adr);
/*
        while (free_entries < 2)
	    {
	        usleep(1);
            free_entries = ((REG_READ(DISP_HD_FIFO_STATUS1)) & disp_reg_tab[DISP_HD_GRA_FREE_ENTRIES_MASK_INDEX]) >>
                           disp_reg_tab[DISP_HD_GRA_FREE_ENTRIES_SHIFT_INDEX] ;
	    }

*/       
        tmp = GRA0_CMD0_FIFO_WE_MASK | UPDATE_GRA0_REGS_MASK | DISP_HD_GRA0_CMD0_FIFO_UPDATE_COMP_REGS_MASK;
        REG_WRITE(disp_reg_tab[DISP_HD_GRA_CMD_FIFO_INDEX], tmp);

       
  //      REG_WRITE(disp_reg_tab[DISP_HD_GRA_CMD_FIFO_INDEX], GRA0_CMD0_FIFO_WE_MASK);

        m_layer_data->back_addr = m_layer_data->y_start_addr;
        m_layer_data->y_start_addr = lock->phys;
    }

    /* If the flag is specified the thing will block until the next frame (change). */
    if (flags & DSFLIP_WAIT) 
    {
        if (compositor == SD_COMPOSITOR) 
        {
            /* SD: A single frame is composed of two fields, so do this twice */
            for (cnt = 0 ; cnt < 2 ; cnt++) 
            {		    
                base_frm_cnt	 =    GET_SD_FRM_COUNT;
                
                while ((base_frm_cnt == GET_SD_FRM_COUNT) && 
                       (REG_READ(DISP_SD_COMP_CTRL) & DISP_SD_COMP_CTRL_COMP_EN_MASK))
                {
                    usleep(10);
                }
            }
        }
        else 
        {
            if(!(surface->config.caps & DSCAPS_TRIPLE))
            {
                 WAIT_FOP_NEXT_HD_FRAME;
            }
        } 
    }
    else 
    {
        D_DEBUG("<WARNING>: DSFLIP_WAIT not specified!\n");

    }

    m_layer_data->last_frame_counter = GET_HD_FRM_COUNT;

    return;
}

/****************************************************************************************************************************/

static DFBResult
XcodeGra0FlipRegion( CoreLayer             *layer,
                 void                  *driver_data,
                 void                  *layer_data,
                 void                  *region_data,
                 CoreSurface           *surface,
                 DFBSurfaceFlipFlags    flags,
                 CoreSurfaceBufferLock *lock )
{
     DFBResult              ret;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "XcodeGra0FlipRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( mdrv != NULL );

     D_DEBUG( "( 0x%08lx [%d] 0x%04x)\n", 
                 lock->phys, lock->pitch, flags);

    xcodeGraFlipRegion((u32 *)&disp_gra0_register_table,
        driver_data,
        layer_data,
        surface,
        flags,
        lock,
        HD_COMPOSITOR);

     dfb_surface_flip( surface, false );

     if(dfb_config->soft_mirror_sd)
     {
         XcodeFlipSDFrame(layer, NULL, lock);
     }

     D_DEBUG( "XcodeGra0FlipRegion exit \n" );
     return DFB_OK;
}


/***************************************************************************************/


/***************************************************************************************/

DFBResult Xcode_Gra_Plane_Update_Region(void                       *driver_data,
                                        void                       *layer_data,
                                        const DFBRegion       *update,
                                        u32 * disp_gra_register_table,
                                        CoreSurfaceBufferLock *lock)
{
    u32 free_entries;
    u32 frame_index = 0;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32 tmp;
    DFBResult ret; 

    if (lock)
    {
        REG_WRITE(disp_gra_register_table[DISP_HD_GRA_ADR_INDEX], lock->phys);
    }

    free_entries = ((REG_READ(DISP_HD_FIFO_STATUS1)) & disp_gra_register_table[DISP_HD_GRA_FREE_ENTRIES_MASK_INDEX]) >>
                   disp_gra_register_table[DISP_HD_GRA_FREE_ENTRIES_SHIFT_INDEX] ;

    if (free_entries > 0)
    {
        frame_index =  update->y1 / m_layer_data->flip_height;

        if (frame_index <= MAX_FRAME_INDEX)
        {

            tmp = GRA1_CMD0_FIFO_WE_MASK | UPDATE_GRA0_REGS_MASK | DISP_HD_GRA0_CMD0_FIFO_UPDATE_COMP_REGS_MASK |
                  (frame_index & HD_GRA1_IDX_MASK);

            REG_WRITE(disp_gra_register_table[DISP_HD_GRA_CMD_FIFO_INDEX], tmp);
        }

        ret = DFB_OK;
    }
    else
    {
        ret = DFB_FAILURE;
    }

    return ret ;

}

/****************************************************************************************************************************/

static DFBResult
XcodeGra0UpdateRegion( CoreLayer             *layer,
                   void                  *driver_data,
                   void                  *layer_data,
                   void                  *region_data,
                   CoreSurface           *surface,
                   const DFBRegion       *update,
                   CoreSurfaceBufferLock *lock )
{

     D_DEBUG( "XcodeGra0UpdateRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( driver_data != NULL );
     D_ASSERT( layer_data != NULL );
     D_ASSERT( update != NULL );
     DFBResult ret;

     ret = Xcode_Gra_Plane_Update_Region (driver_data,
                                          layer_data,
                                          update,
                                          disp_gra0_register_table,
                                          lock);

     if(dfb_config->soft_mirror_sd)
     {
         XcodeFlipSDFrame(layer, update, lock);
     }

     D_DEBUG( "XcodeGra0UpdateRegion  exit \n" );
     return ret;
}

/***************************************************************************************/

DFBResult
Gra_Set_Color_Visibility( void               *driver_data,
                          DFBColorAdjustment *adj,
                          u32  * disp_hd_gra_register_table)
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    u32 tmp;

    tmp = REG_READ(disp_hd_gra_register_table[DISP_HD_GRA_FORMAT_INDEX]);

    if (adj->brightness == 0)
    {
        tmp = tmp | DISP_HD_GRA0_FORMAT_PIPE_DISABLE_MASK;
    }
    else
    {
        tmp = tmp & (~ DISP_HD_GRA0_FORMAT_PIPE_DISABLE_MASK);
    }

    REG_WRITE(disp_hd_gra_register_table[DISP_HD_GRA_FORMAT_INDEX], tmp);

    return DFB_OK;
}

/****************************************************************************************************************************/

DFBResult XcodeGra0SetColorAdjustment(CoreLayer            *layer,
                                void                 *driver_data,
                                void                 *layer_data,
                                DFBColorAdjustment   *adj)
{
    D_DEBUG( "XcodeGra0SetColorAdjustment enter \n" );

    D_DEBUG ("brightness %d \n", adj->brightness);
    D_DEBUG ("contrast  %d \n", adj->contrast );
    D_DEBUG ("hue  %d \n", adj->hue );
    D_DEBUG ("saturation  %d \n", adj->saturation );

    //set hue , bright, contrast, sat
    Gra_Set_Color_Visibility(driver_data, adj, disp_gra0_register_table);

    D_DEBUG( "XcodeGra0SetColorAdjustment exit \n" );

    return DFB_OK;
}

/***************************************************************************************/

inline int
Get_Layer_Level( void *driver_data,
                 u32 reg_right_shift,
                 u32 reg_mask)
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    u32 level;

    level = REG_READ(DISP_HD_COMP_CTRL);
    level = (level & reg_mask) >> reg_right_shift;

    return level;
}


/**********************************************************************************************************************/
static DFBResult
XcodeGra0GetLevel( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int       *level )
{
     
    D_DEBUG( "XcodeGra0GetLevel enter \n" );

    * level = Get_Layer_Level(driver_data, GRA0_MUX_SHIFT, GRA0_MUX_MASK);

    D_DEBUG( "XcodeGra0GetLevel exit \n" );

    return DFB_OK;
}

/***************************************************************************************/

DFBResult
Set_Layer_Level( void *driver_data,
                 int level,
                 u32 reg_left_shift,
                 u32 reg_mask)
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    u32 ret = DFB_OK;

    if ((level >= 0) && (level <= MAX_LAYER_LEVEL) && (reg_left_shift < 16))
    {
        u32 tmp = REG_READ(DISP_HD_COMP_CTRL);
        int layer_level[4];
        int layer_shift[4];
        int tmp_level, tmp_shift;
        int i, j;

        for(i = 0; i < 4; i++)
        {
            layer_shift[i] = i * 4;
            layer_level[i] = (tmp >> layer_shift[i]) & 0xF;
        }

        //Reorder layer level
        if(level != layer_level[reg_left_shift /4])
        {
            layer_level[reg_left_shift /4] = 0xF;

            for(i = 0; i < 4; i++)
            {
                for(j = i + 1; j < 4; j++)
                {
                    if(layer_level[i] > layer_level[j])
                    {
                        tmp_level = layer_level[i];
                        tmp_shift = layer_shift[i];
                        layer_level[i] = layer_level[j];
                        layer_shift[i] = layer_shift[j];
                        layer_level[j] = tmp_level;
                        layer_shift[j] = tmp_shift;
                    }
                }
            }

            for(i = 0; i < 4; i++)
            {
                if(i == level)
                {
                    for(j = 3; j > i; j--)
                    {
                        layer_shift[j] = layer_shift[j - 1];
                    }
                    layer_shift[i] = reg_left_shift;
                    break;
                }
            }

            tmp &= ~0xFFFF;
            for(i = 0; i < 4; i++)
            {
                tmp |= (i << layer_shift[i]);
            }

            REG_WRITE(DISP_HD_COMP_CTRL, tmp);
        }
    }
    else
    {
        ret = DFB_UNSUPPORTED;
    }

    return ret;
}


/**********************************************************************************************************************/
static DFBResult
XcodeGra0SetLevel( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int        level )
{
    DFBResult ret;

    D_DEBUG( "XcodeGra0SetLevel enter \n" );
    D_DEBUG( "level %d \n", level );

    ret = Set_Layer_Level(driver_data, level, GRA0_MUX_SHIFT, GRA0_MUX_MASK);

    D_DEBUG( "XcodeGra0SetLevel exit \n" );
    
     
    return ret;
}

/**********************************************************************************************************************/

DFBResult Xcode4_Set_Graphic_Layers(void                       *driver_data,
                                    void                       *layer_data,
                                    CoreLayerRegionConfig      *config,
                                    CoreLayerRegionConfigFlags  updated,
                                    CorePalette                *palette,
                                    CoreSurfaceBufferLock      *lock,
                                    u32 * disp_gra_register_table)
{
    int     disp_format;
    int ignorePixelAlpha;
    app_ioctl_command	app_cmd;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    u32 sd_buffer_size;
    int dev_mem_fd;
    char *dev_mem="/dev/mem";
    DFBResult ret;

    if (convert_dfb_format_to_disp_format(config, &disp_format, &ignorePixelAlpha) != DFB_OK)
    {
         return DFB_UNSUPPORTED;
    }

    if ((updated == CLRCF_PALETTE) && palette)
    {
        xcode4_set_disp_gra_palette( driver_data, layer_data, palette, disp_gra_register_table );
    }
    else
    {
        if ((updated & CLRCF_PALETTE) && palette)
        {
             xcode4_set_disp_gra_palette( driver_data, layer_data, palette, disp_gra_register_table );
        }

        D_DEBUG("ignorePixelAlpha: %u\n", ignorePixelAlpha);
        //set the related grahic plane
        set_graphic_plane_gra(driver_data, layer_data, config, updated, lock, disp_gra_register_table, disp_format, ignorePixelAlpha);

    }

    if(dfb_config->soft_mirror_sd)
    {
        XcodeFlipSDFrame(dfb_layer_at(XCODE4_GRA0_LAYER_ID), NULL, NULL);
    }

    return DFB_OK;
}

/**********************************************************************************************************************/

const DisplayLayerFuncs XcodeGra0LayerFuncs = {
     LayerDataSize:      XcodeVidLayerDataSize,
     InitLayer:          XcodeGra0InitLayer,
     TestRegion:         XcodeGra0TestRegion,
     SetRegion:          XcodeGra0SetRegion,
     FlipRegion:         XcodeGra0FlipRegion,
     UpdateRegion:       XcodeGra0UpdateRegion,
     SetColorAdjustment: XcodeGra0SetColorAdjustment,
     GetLevel:           XcodeGra0GetLevel,
     SetLevel:           XcodeGra0SetLevel,
     ReallocateSurface:  Xc4GraReallocateSurface
};

