/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_osd_gra1.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/


#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <pthread.h>

#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"

extern DFBResult XcodeFlipSDFrame(CoreLayer *layer, 
                                         const DFBRegion *update,
                                         CoreSurfaceBufferLock *lock);

extern DFBResult convert_dfb_format_to_disp_format(const CoreLayerRegionConfig *config, 
                                            int * disp_format, int *ignorePixelAlpha);

extern DFBResult xcode4_set_disp_gra_palette( void * driver_data, void * layer_data, CorePalette *palette, u32 * disp_gra_register_table );

extern void set_graphic_plane_gra(void * driver_data, 
                           void *            layer_data,
                           CoreLayerRegionConfig *config,
                           CoreLayerRegionConfigFlags  updated,
                           CoreSurfaceBufferLock *lock,
					       u32 * disp_gra_register_table,
                           int disp_format,
			   int ignorePixelAlpha);

extern DFBResult Xcode_Gra_Plane_Update_Region(void                       *driver_data,
                                        void                       *layer_data,
                                        const DFBRegion       *update,
                                        u32 * disp_gra_register_table,
                                        CoreSurfaceBufferLock *lock);

extern int XcodeVidLayerDataSize();

extern DFBResult
Gra_Set_Color_Visibility( void               *driver_data,
                          DFBColorAdjustment *adj,
                          u32  * disp_hd_gra_register_table);

extern DFBResult Set_Layer_Level( void *driver_data,
                 int level,
                 u32 reg_left_shift,
                 u32 reg_mask);

extern inline int Get_Layer_Level( void *driver_data,
                  u32 reg_right_shift,
                  u32 reg_mask);

extern void		xcodeGraFlipRegion(u32 					*disp_gra_register_table, 
						void					*driver_data,
                              void * layer_data,
						CoreSurface			*surface,
						DFBSurfaceFlipFlags		flags,
						CoreSurfaceBufferLock	*lock,
						int					compositor
);


extern DFBResult Xcode4_Set_Graphic_Layers(void                       *driver_data,
                                    void                       *layer_data,
                                    CoreLayerRegionConfig      *config,
                                    CoreLayerRegionConfigFlags  updated,
                                    CorePalette                *palette,
                                    CoreSurfaceBufferLock      *lock,
                                    u32 * disp_gra_register_table);

extern DFBResult
Xc4GraReallocateSurface( CoreLayer             *layer,
                          void                  *driver_data,
                          void                  *layer_data,
                          void                  *region_data,
                          CoreLayerRegionConfig *config,
                          CoreSurface           *surface );

static u32 disp_gra1_register_table[SIZE_OF_DISP_GRA_REGISTER_TABLE];

/**********************************************************************************************************************/
void Init_Disp_Gra1_Register_Table()
{
     disp_gra1_register_table[DISP_HD_GRA_CLUT_DATA_INDEX] = DISP_HD_RAM_WDATA;
	disp_gra1_register_table[DISP_HD_GRA_CLUT_INDEX] = DISP_HD_GRA1_CLUT_ADDR;
     disp_gra1_register_table[DISP_HD_GRA_FORMAT_INDEX] = DISP_HD_GRA1_FORMAT;
     disp_gra1_register_table[DISP_HD_GRA_FORMAT1_INDEX] = DISP_HD_GRA1_FORMAT1;
     disp_gra1_register_table[DISP_HD_GRA_BUF_INDEX] = DISP_HD_GRA1_BUF;
     disp_gra1_register_table[DISP_HD_GRA_SAMPLES_INDEX] = DISP_HD_GRA1_SAMPLES;
     disp_gra1_register_table[DISP_HD_GRA_ADR_INDEX] = DISP_HD_GRA1_ADR;
     disp_gra1_register_table[DISP_HD_GRA_SIZE_INDEX] = DISP_HD_GRA1_SIZE;
     disp_gra1_register_table[DISP_HD_GRA_VSCA_INV_RATIO_INDEX] = DISP_HD_GRA1_VSCA_INV_RATIO;
     disp_gra1_register_table[DISP_HD_GRA_VSCA_INIT_INDEX] = DISP_HD_GRA1_VSCA_INIT;
     disp_gra1_register_table[DISP_HD_GRA_VSCA_START_INDEX] = DISP_HD_GRA1_VSCA_START;
     disp_gra1_register_table[DISP_HD_GRA_HSCA_INV_RATIO_INDEX] = DISP_HD_GRA1_HSCA_INV_RATIO;
     disp_gra1_register_table[DISP_HD_COMP_GRA_POS_INDEX] = DISP_HD_COMP_GRA1_POS;
     disp_gra1_register_table[DISP_HD_COMP_ALPHA_GRA_SHIFT_INDEX] = ALPHA_G1_SHIFT;
     disp_gra1_register_table[DISP_HD_GRA_SCALED_SIZE_INDEX] = DISP_HD_GRA1_SCALED_SIZE;
     disp_gra1_register_table[DISP_HD_COMP_ALPHA_MODE_GRA_SHIFT_INDEX] = ALPHA_MODE_G1_SHIFT;
     disp_gra1_register_table[DISP_HD_CC_GRA_MODE_SHIFT_INDEX] = GRA1_MODE_SHIFT;
     disp_gra1_register_table[DISP_HD_CC_GRA_MODE_MASK_INDEX] = GRA1_MODE_MASK;
     disp_gra1_register_table[ALPHA_GRAPHIC_MASK_INDEX] = ALPHA_G1_MASK;
     disp_gra1_register_table[DISP_HD_GRA_DEFAULT_TAG_INDEX] = DISP_HD_GRA1_DEFAULT_TAG;
     disp_gra1_register_table[DISP_HD_GRA_IDX_MASK_INDEX] = HD_GRA1_IDX_MASK;
     disp_gra1_register_table[DISP_HD_GRA_IDX_SHIFT_INDEX] = HD_GRA1_IDX_SHIFT;
     disp_gra1_register_table[DISP_HD_GRA_FREE_ENTRIES_MASK_INDEX] = GRA1_FREE_ENTRIES_MASK;
     disp_gra1_register_table[DISP_HD_GRA_FREE_ENTRIES_SHIFT_INDEX] = GRA1_FREE_ENTRIES_SHIFT;
     disp_gra1_register_table[DISP_HD_GRA_CMD_FIFO_INDEX] = DISP_HD_GRA1_CMD0_FIFO;
     disp_gra1_register_table[DISP_HD_GRA_BUF_SIZE_MASK_INDEX]  =	DISP_HD_GRA1_BUF_SIZE_MASK;
     disp_gra1_register_table[DISP_HD_GRA_BUF_SIZE_SHIFT_INDEX] = DISP_HD_GRA1_BUF_SIZE_SHIFT;
     disp_gra1_register_table[DISP_HD_COMP_ALPHA_MODE_GRA_MASK_INDEX]	 =    ALPHA_MODE_G1_MASK;
     disp_gra1_register_table[GRA_EMPTY_MASK_INDEX] = GRA1_EMPTY_MASK;
     disp_gra1_register_table[GRA_EMPTY_SHIFT_INDEX] = GRA1_EMPTY_SHIFT;

}

/**********************************************************************************************************************/

static DFBResult
XcodeGra1InitLayer( CoreLayer                *layer,
              void                       *driver_data,
              void                       *layer_data,
              DFBDisplayLayerDescription *description,
              DFBDisplayLayerConfig      *config,
              DFBColorAdjustment         *adjustment )
{
     u32     tmp;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;

     D_DEBUG( "xcode DFB driver XcodeGra1InitLayer enter:\n" );

     /* set capabilities and type */
     description->caps = DLCAPS_SURFACE | DLCAPS_ALPHACHANNEL | DLCAPS_OPACITY | 
                         DLCAPS_SCREEN_POSITION | DLCAPS_SCREEN_LOCATION | DLCAPS_LEVELS;

     description->type = DLTF_GRAPHICS;

     /* set name */
     snprintf( description->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "ViXS Xcode4 Graphics Plane 1" );

     /* fill out the default configuration */
     config->flags       = DLCONF_WIDTH       | DLCONF_HEIGHT |
                           DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS | DLCONF_SURFACE_CAPS ;

     if (dfb_config->preallocate_graphic_layer & (0x1 << XCODE4_GRA1_LAYER_ID))
     {
         config->width       = MAX_GRAPHIC_BUFFER_WIDTH;
         config->height      = MAX_GRAPHIC_BUFFER_HEIGHT;
         config->buffermode  = DLBM_BACKVIDEO;
     }
     else
     {
         config->width       = XCODE_DEFAULT_WIDTH;
         config->height      = XCODE_DEFAULT_HEIGHT;
         config->buffermode  = DLBM_FRONTONLY;
     }

     config->pixelformat = DSPF_LUT8;
     config->surface_caps |= DSCAPS_VIDEOONLY | DSCAPS_STATIC_ALLOC | DSCAPS_DOUBLE ;
     config->options     = DLOP_OPACITY;
	adjustment->flags   = DCAF_BRIGHTNESS;

     memset (layer_data, 0, sizeof (xc_layer_data));

     //Set the GRA1 register table
     Init_Disp_Gra1_Register_Table();

     D_DEBUG( "xcode DFB driver XcodeGra1InitLayer exit:\n" );

     return DFB_OK;
}

static DFBResult
XcodeGra1TestRegion( CoreLayer                  *layer,
               void                       *driver_data,
               void                       *layer_data,
               CoreLayerRegionConfig      *config,
               CoreLayerRegionConfigFlags *failed )
{
     CoreLayerRegionConfigFlags fail = 0;

     D_DEBUG( "xcode DFB driver Xcode4Gra1TestRegion enter:\n" );

     if (config->options & ~XCODE4_OSD_SUPPORTED_OPTIONS)
          fail |= CLRCF_OPTIONS;

     /* check buffermode */
     switch (config->buffermode) {
          case DLBM_FRONTONLY:
          case DLBM_BACKSYSTEM:
          case DLBM_BACKVIDEO:
          case DLBM_TRIPLE:
               break;

          default:
               fail |= CLRCF_BUFFERMODE;
               break;
     }

     switch (config->format) {
          case DSPF_LUT8:
          case DSPF_RGB24:
          case DSPF_RGB16:
          case DSPF_RGB32:
          case DSPF_ARGB1555:
		  case DSPF_RGB555:
          case DSPF_ARGB:
               break;

          default:
               fail |= CLRCF_FORMAT;
     }


     if (failed)
          *failed = fail;

     if (fail)
	 {
          return DFB_UNSUPPORTED;
	 }

     D_DEBUG( "xcode DFB driver XcodeGra1TestRegion exit:\n" );

     return DFB_OK;
}


static DFBResult
XcodeGra1SetRegion( CoreLayer                  *layer,
              void                       *driver_data,
              void                       *layer_data,
              void                       *region_data,
              CoreLayerRegionConfig      *config,
              CoreLayerRegionConfigFlags  updated,
              CoreSurface                *surface,
              CorePalette                *palette,
              CoreSurfaceBufferLock      *lock )
{
     DFBResult ret;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_ASSERT( driver_data != NULL );
     D_ASSERT( layer_data != NULL );

     D_DEBUG( "xcode DFB driver XcodeGra1SetRegion enter \n" );

     D_DEBUG ("surface width %d \n", config->width);
     D_DEBUG ("surface height %d \n", config->height);
     D_DEBUG ("destination x %d \n", config->dest.x);
     D_DEBUG ("destination y %d \n", config->dest.y);
     D_DEBUG ("destination weight %d \n", config->dest.w);
     D_DEBUG ("destination height %d \n", config->dest.h);

     D_DEBUG ("source x %d \n", config->source.x);
     D_DEBUG ("source y %d \n", config->source.y);
     D_DEBUG ("source weight %d \n", config->source.w);
     D_DEBUG ("source height %d \n", config->source.h);

     D_DEBUG ("surface caps %d \n", config->surface_caps);
     D_DEBUG ("surface pitch %d \n", lock->pitch);
     D_DEBUG ("surface phys 0x%x \n", lock->phys);

     pthread_mutex_lock( &mdrv->shared->reentrancy_mutex );
     ret = Xcode4_Set_Graphic_Layers(driver_data,
                                     layer_data,
                                     config,
                                     updated,
                                     palette,
                                     lock,
                                     disp_gra1_register_table);
     pthread_mutex_unlock( &mdrv->shared->reentrancy_mutex );

     D_DEBUG( "xcode DFB driver XcodeGra1SetRegion exit \n" );

     return ret;
}


/****************************************************************************************************************************/

static DFBResult
XcodeGra1UpdateRegion( CoreLayer             *layer,
                   void                  *driver_data,
                   void                  *layer_data,
                   void                  *region_data,
                   CoreSurface           *surface,
                   const DFBRegion       *update,
                   CoreSurfaceBufferLock *lock )
{

     D_DEBUG( "XcodeGra1UpdateRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( driver_data != NULL );
     D_ASSERT( layer_data != NULL );
     D_ASSERT( update != NULL );
     DFBResult ret;

	ret = Xcode_Gra_Plane_Update_Region (driver_data,
	                                     layer_data,
	                                     update,
	                                     disp_gra1_register_table,
                                          lock);

     if(dfb_config->soft_mirror_sd)
     {
         XcodeFlipSDFrame(layer, update, lock);
     }

     D_DEBUG( "XcodeGra1UpdateRegion  exit \n" );
     return ret;
}

/****************************************************************************************************************************/

static DFBResult
XcodeGra1FlipRegion( CoreLayer             *layer,
                 void                  *driver_data,
                 void                  *layer_data,
                 void                  *region_data,
                 CoreSurface           *surface,
                 DFBSurfaceFlipFlags    flags,
                 CoreSurfaceBufferLock *lock )
{
     DFBResult              ret;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;

     D_DEBUG( "XcodeGra1FlipRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( mdrv != NULL );

     D_DEBUG( "( 0x%08lx [%d] 0x%04x)\n", 
                 lock->phys, lock->pitch, flags);

	 dfb_surface_flip( surface, false );

     xcodeGraFlipRegion((u32 *)&disp_gra1_register_table,
     			driver_data,
                layer_data,
     			surface,
     			flags,
     			lock,
     			HD_COMPOSITOR);	

     if(dfb_config->soft_mirror_sd)
     {
         XcodeFlipSDFrame(layer, NULL, lock);
     }

     D_DEBUG( "XcodeGra1FlipRegion exit \n" );
     return DFB_OK;
}

/****************************************************************************************************************************/

DFBResult XcodeGra1SetColorAdjustment(CoreLayer            *layer,
                                void                 *driver_data,
                                void                 *layer_data,
                                DFBColorAdjustment   *adj)
{
    D_DEBUG( "XcodeGra1SetColorAdjustment enter \n" );

    D_DEBUG ("brightness %d \n", adj->brightness);
    D_DEBUG ("contrast  %d \n", adj->contrast );
    D_DEBUG ("hue  %d \n", adj->hue );
    D_DEBUG ("saturation  %d \n", adj->saturation );

    //set hue , bright, contrast, sat
    Gra_Set_Color_Visibility(driver_data, adj, disp_gra1_register_table);

    D_DEBUG( "XcodeGra1SetColorAdjustment exit \n" );

    return DFB_OK;
}

/**********************************************************************************************************************/
static DFBResult
XcodeGra1GetLevel( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int       *level )
{
     
    D_DEBUG( "XcodeGra1GetLevel enter \n" );

	* level = Get_Layer_Level(driver_data, GRA1_MUX_SHIFT, GRA1_MUX_MASK);

    D_DEBUG( "XcodeGra1GetLevel exit \n" );

    return DFB_OK;
}

/**********************************************************************************************************************/
static DFBResult
XcodeGra1SetLevel( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int        level )
{
    DFBResult ret;

    D_DEBUG( "XcodeGra1SetLevel enter \n" );
    D_DEBUG( "level %d \n", level );

	ret = Set_Layer_Level(driver_data, level, GRA1_MUX_SHIFT, GRA1_MUX_MASK);

    D_DEBUG( "XcodeGra1SetLevel exit \n" );
	
     
    return ret;
}

/****************************************************************************************************************************/

const DisplayLayerFuncs XcodeGra1LayerFuncs = {
     LayerDataSize:      XcodeVidLayerDataSize,
     InitLayer:          XcodeGra1InitLayer,
     TestRegion:         XcodeGra1TestRegion,
     SetRegion:          XcodeGra1SetRegion,
     FlipRegion:         XcodeGra1FlipRegion,
     UpdateRegion:       XcodeGra1UpdateRegion,
     SetColorAdjustment: XcodeGra1SetColorAdjustment,
     GetLevel:           XcodeGra1GetLevel,
     SetLevel:           XcodeGra1SetLevel,
     ReallocateSurface:  Xc4GraReallocateSurface
};

