/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_osd_sd_gra0.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/

#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"
#include "xcioctl.h"

#define SD_FRAME_AVAILABLE        0x80000000

extern int XcodeVidLayerDataSize();

extern u32 Calculate_gfx_hsca_inv_ratio(u32 input_h_size, u32 output_h_size);

/* This register table is only used to pass on register locations to routines shared by all graphic planes (flip, etc.) */
u32		disp_sd_gra0_register_table[SIZE_OF_DISP_GRA_REGISTER_TABLE];


/**********************************************************************************************************************/

static DFBResult
XcodeSdGra0InitLayer( CoreLayer                *layer,
              void                       *driver_data,
              void                       *layer_data,
              DFBDisplayLayerDescription *description,
              DFBDisplayLayerConfig      *config,
              DFBColorAdjustment         *adjustment )
{
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     u32     tmp;
     D_DEBUG( "xcode DFB driver XcodeSdGra0InitLayer enter:\n" );


     /* set capabilities and type */
     description->caps = DLCAPS_SURFACE | DLCAPS_ALPHACHANNEL | DLCAPS_OPACITY | DLCAPS_SCREEN_POSITION | DLCAPS_SCREEN_LOCATION;

     description->type = DLTF_GRAPHICS;

     /* set name */
     snprintf( description->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "ViXS Xcode4 SD Graphics Plane 0" );

     /* fill out the default configuration */
     config->flags       = DLCONF_WIDTH       | DLCONF_HEIGHT |
                           DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS;
     config->width       = 768;
     config->height      = 480;
     config->pixelformat = DSPF_ARGB;
     config->buffermode  = DLBM_FRONTONLY;
     config->options     = DLOP_ALPHACHANNEL;
     config->buffermode = DLBM_INTERNAL;
     
     /* Fill out register table */
     disp_sd_gra0_register_table[DISP_HD_GRA_SIZE_INDEX]			=		DISP_SD_GRA0_SIZE;
     disp_sd_gra0_register_table[DISP_HD_GRA_ADR_INDEX]				=		DISP_SD_GRA0_ADR;
     disp_sd_gra0_register_table[DISP_HD_GRA_BUF_INDEX]				=		DISP_SD_GRA0_BUF;
     disp_sd_gra0_register_table[DISP_HD_GRA_FORMAT_INDEX]			=		DISP_SD_GRA0_FORMAT;
     disp_sd_gra0_register_table[DISP_HD_GRA_FORMAT1_INDEX]			=		DISP_SD_GRA0_FORMAT1;
     disp_sd_gra0_register_table[DISP_HD_GRA_SAMPLES_INDEX]			=		DISP_SD_GRA0_SAMPLES;
     disp_sd_gra0_register_table[DISP_HD_GRA_SCALED_SIZE_INDEX]			=		DISP_SD_GRA0_SCALED_SIZE;

     disp_sd_gra0_register_table[DISP_HD_GRA_FREE_ENTRIES_MASK_INDEX]		=		DISP_SD_FIFO_STATUS_FREE_ENTRIES_MASK;
     disp_sd_gra0_register_table[DISP_HD_GRA_FREE_ENTRIES_SHIFT_INDEX]		=		DISP_SD_FIFO_STATUS_FREE_ENTRIES_SHIFT;

     disp_sd_gra0_register_table[DISP_HD_GRA_IDX_MASK_INDEX]			=		SD_GRA0_IDX_MASK;
     disp_sd_gra0_register_table[DISP_HD_GRA_IDX_SHIFT_INDEX]			=		SD_GRA0_IDX_SHIFT;
     disp_sd_gra0_register_table[DISP_HD_COMP_GRA_POS_INDEX]			=		DISP_SD_COMP_GRA0_POS;
     disp_sd_gra0_register_table[DISP_HD_GRA_CMD_FIFO_INDEX]			=		DISP_SD_CMD1_FIFO;

     disp_sd_gra0_register_table[DISP_HD_GRA_BUF_SIZE_MASK_INDEX]		=		DISP_SD_GRA0_BUF_SIZE_MASK;
     disp_sd_gra0_register_table[DISP_HD_GRA_BUF_SIZE_SHIFT_INDEX]		=		DISP_SD_GRA0_BUF_SIZE_SHIFT;

     D_DEBUG( "xcode DFB driver XcodeSdGra0InitLayer exit:\n" );

     return DFB_OK;
}

/**********************************************************************************************************************/

static DFBResult
XcodeSdGra0TestRegion( CoreLayer                  *layer,
               void                       *driver_data,
               void                       *layer_data,
               CoreLayerRegionConfig      *config,
               CoreLayerRegionConfigFlags *failed )
{
     CoreLayerRegionConfigFlags fail = 0;

     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     app_ioctl_command app_cmd;

     D_DEBUG( "xcode DFB driver XcodeSdGra0TestRegion enter:\n" );

     /* Check if internal loop is used */
     if(config->buffermode == DLBM_INTERNAL)
     {
          app_cmd.uIOCTLCommand = CMD_APP_IOCTL_GET_SD_SCREEN_MODE;
          if(!ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd))
          {
               app_cmd.uIOCTLCommand = CMD_APP_IOCTL_SET_SD_SCREEN_MODE;
               app_cmd.dfb_sd_screen_mode.color_space = COLOR_SPACE_YUV601;
               ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
          }
          else
          {
               D_ERROR( "sd_buffer_base is not set when loading vixsfb.ko\n" );
               return DFB_UNSUPPORTED;
          }
     }
     else
     {
          app_cmd.uIOCTLCommand = CMD_APP_IOCTL_GET_SD_SCREEN_MODE;
          if(!ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd))
          {
               app_cmd.uIOCTLCommand = CMD_APP_IOCTL_SET_SD_SCREEN_MODE;
               app_cmd.dfb_sd_screen_mode.color_space = COLOR_SPACE_RGB_FULL;
               ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
          }

          /* Width must be set to 768 to meet DISP requirement */
          config->width = 768;
     }

     if (config->options & ~XCODE4_OSD_SUPPORTED_OPTIONS)
          fail |= CLRCF_OPTIONS;

     switch (config->format) {
          case DSPF_RGB32:
          case DSPF_ARGB:
               break;
          default:
               fail |= CLRCF_FORMAT;
     }

     if (failed)
          *failed = fail;

     if (fail)
     {
          return DFB_UNSUPPORTED;
     }

     D_DEBUG( "xcode DFB driver XcodeSdGra0TestRegion exit:\n" );

     return DFB_OK;
}

/**********************************************************************************************************************/

static DFBResult
XcodeSdGra0SetRegion( CoreLayer                  *layer,
                    void                       *driver_data,
                    void                       *layer_data,
                    void                       *region_data,
                    CoreLayerRegionConfig      *config,
                    CoreLayerRegionConfigFlags  updated,
                    CoreSurface                *surface,
                    CorePalette                *palette,
                    CoreSurfaceBufferLock      *lock )
{
     int     ret;
     u32     tmp;
     int     disp_format;
     unsigned int num_entries;
     int output_mode;
     app_ioctl_command dfb_ioctl_cmd;


     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     pxc_dfb_device_data pmdev = mdrv->pdevice_data;
     struct display_mode_struct * mode = NULL;
     u32 flip_height;

     D_DEBUG( "xcode DFB driver XcodeSdGra0SetRegion enter \n" );

     D_DEBUG ("surface width %d \n", config->width);
     D_DEBUG ("surface height %d \n", config->height);
     D_DEBUG ("destination x %d \n", config->dest.x);
     D_DEBUG ("destination y %d \n", config->dest.y);
     D_DEBUG ("destination weight %d \n", config->dest.w);
     D_DEBUG ("destination height %d \n", config->dest.h);
     D_DEBUG ("surface caps %d \n", config->surface_caps);
     D_DEBUG ("surface pitch %d \n", lock->pitch);
     D_DEBUG ("surface phys 0x%x \n", lock->phys);
     D_DEBUG ("surface opacity  0x%x \n", config->opacity);

     SET_NUMBER_ALIGNMENT(flip_height, config->source.h, VIDEO_HEIGHT_ALIGNMENT);

     m_layer_data->dest_width = config->dest.w;
     m_layer_data->dest_height = config->dest.h;
     m_layer_data->source_width = config->source.w;
     m_layer_data->source_height = config->source.h;
     m_layer_data->y_start_addr = lock->phys;
     m_layer_data->y_pitch = lock->pitch;
     m_layer_data->format = config->format;
     m_layer_data->buf_width = config->width;
     m_layer_data->buf_height = config->height;

     //Not need to setup SD screen
     if(config->buffermode == DLBM_INTERNAL)
     {
          return DFB_OK;
     }

     tmp = REG_READ( DISP_SD_GRA0_FORMAT1);
     tmp = tmp & (~0x0ff);
     tmp = tmp | DFB_ALPHA_POS 
          | (DFB_RED_POS << DISP_SD_GRA0_FORMAT1_PEL_POSITION_R_SHIFT)
          | (DFB_GREEN_POS << DISP_SD_GRA0_FORMAT1_PEL_POSITION_G_SHIFT)
          | (DFB_BLUE_POS << DISP_SD_GRA0_FORMAT1_PEL_POSITION_B_SHIFT);
     REG_WRITE( DISP_SD_GRA0_FORMAT1, tmp);

     tmp = config->width * DFB_BYTES_PER_PIXEL(config->format);
     
     tmp  >>=   VIDEO_BUFFER_SIZE_ALIGNMENT_SHIFT;
     
     tmp = (tmp <<  DISP_SD_GRA0_BUF_SIZE_SHIFT) | (config->width * DFB_BYTES_PER_PIXEL(config->format) * 2);

     REG_WRITE(DISP_SD_GRA0_BUF, tmp);

     REG_WRITE( DISP_SD_GRA0_SAMPLES, config->source.w * DFB_BYTES_PER_PIXEL(config->format));

     REG_WRITE( DISP_SD_GRA0_SIZE, (config->source.w | (config->source.h << DISP_HD_GRA0_SIZE_V_SIZE_SHIFT)));

     REG_WRITE( DISP_SD_GRA0_SCALED_SIZE, (config->dest.w | (config->dest.h << DISP_HD_GRA0_SIZE_V_SIZE_SHIFT) ));

     tmp = Calculate_gfx_hsca_inv_ratio(config->source.w, config->dest.w);

     REG_WRITE( DISP_SD_GRA0_HSCA_INV_RATIO, tmp);

     //set the screen position 
     REG_WRITE(DISP_SD_COMP_GRA0_POS, 
               config->dest.x | (config->dest.y << DISP_HD_COMP_GRA0_POS_POS_Y_SHIFT));

     D_DEBUG( "xcode DFB driver XcodeSdGra0SetRegion exit \n" );

     return DFB_OK;
}

/****************************************************************************************************************************/

static DFBResult
XcodeSdGra0FlipRegion( CoreLayer             *layer,
                 void                  *driver_data,
                 void                  *layer_data,
                 void                  *region_data,
                 CoreSurface           *surface,
                 DFBSurfaceFlipFlags    flags,
                 CoreSurfaceBufferLock *lock )
{
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "XcodeSdGra0FlipRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( mdrv != NULL );

     D_DEBUG( "( 0x%08lx [%d] 0x%04x)\n", 
                 lock->phys, lock->pitch, flags);

     dfb_surface_flip( surface, false );

     REG_WRITE(DISP_SD_GRA0_ADR, lock->phys);
     REG_WRITE(FES1_DUMMY_REG, SD_FRAME_AVAILABLE);

     D_DEBUG( "XcodeSdGra0FlipRegion exit \n" );
     return DFB_OK;
}

/****************************************************************************************************************************/

static DFBResult
XcodeSdGra0UpdateRegion( CoreLayer             *layer,
                   void                  *driver_data,
                   void                  *layer_data,
                   void                  *region_data,
                   CoreSurface           *surface,
                   const DFBRegion       *update,
                   CoreSurfaceBufferLock *lock )
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    D_DEBUG( "XcodeSdGra0UpdateRegion enter \n" );

    D_ASSERT( surface != NULL );
    D_ASSERT( lock != NULL );
    D_ASSERT( driver_data != NULL );
    D_ASSERT( layer_data != NULL );
    D_ASSERT( update != NULL );

    REG_WRITE(DISP_SD_GRA0_ADR, lock->phys);
    REG_WRITE(FES1_DUMMY_REG, SD_FRAME_AVAILABLE);

    D_DEBUG( "XcodeSdGra0UpdateRegion  exit \n" );
    return DFB_OK;
}

/**********************************************************************************************************************/

const DisplayLayerFuncs XcodeSdGra0LayerFuncs = {
     LayerDataSize:      XcodeVidLayerDataSize,
     InitLayer:          XcodeSdGra0InitLayer,
     TestRegion:         XcodeSdGra0TestRegion,
     SetRegion:          XcodeSdGra0SetRegion,
     FlipRegion:         XcodeSdGra0FlipRegion,
     UpdateRegion:       XcodeSdGra0UpdateRegion,
};

