
/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_sd_screen.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/
#include <config.h>

#include <asm/types.h>

#include <stdio.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>


#include <directfb.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>
#include <linux/fb.h>
#include <core/screens_internal.h>

#include <math.h>

#include <gfx/convert.h>

#include "viper.h"

#include "xcmmio.h"
#include "xcioctl.h"

#include "xcode4_osd.h"

extern DFBResult XcodeListenHDtoSD(pxc_dfb_driver_data mdrv);

extern int xc_find_mode_index(const DFBScreenEncoderConfig * config);
                            
void ViperSDDACPowerUp(pxc_dfb_driver_data mdrv);
void ViperSDDACPowerDown(pxc_dfb_driver_data mdrv);

extern struct display_mode_struct * xc4_get_mode(int mode_index);

static u32 disp_sd_comp_register_table[SIZE_OF_DISP_COMP_REGISTER_TABLE];

static void SD_COMP_480i(pxc_dfb_driver_data mdrv, const DFBScreenEncoderConfig *config );
static void SD_COMP_576i(pxc_dfb_driver_data mdrv, const DFBScreenEncoderConfig *config);


/**********************************************************************************************************************/
void Init_Disp_Sd_Comp_Register()
{
    disp_sd_comp_register_table[DISP_COMP_SIZE_INDEX] = DISP_SD_COMP_SIZE;
    disp_sd_comp_register_table[DISP_COMP_GLOBAL_SIZE_INDEX] = DISP_SD_COMP_GLOBAL_SIZE;
    disp_sd_comp_register_table[DISP_COMP_START_PIX_POS_INDEX] = DISP_SD_COMP_START_PIX_POS;
    disp_sd_comp_register_table[DISP_COMP_START_FLD_POS_INDEX] = DISP_SD_COMP_START_FLD_POS;
    disp_sd_comp_register_table[DISP_COMP_START_VBI_POS_INDEX] = DISP_SD_COMP_START_VBI_POS;
    disp_sd_comp_register_table[DISP_COMP_STOP_VBI_POS_INDEX] = DISP_SD_COMP_STOP_VBI_POS;
    disp_sd_comp_register_table[DISP_COMP_START_HANC_POS_INDEX] = DISP_SD_COMP_START_HANC_POS;
    disp_sd_comp_register_table[DISP_COMP_STOP_HANC_POS_INDEX] = DISP_SD_COMP_STOP_HANC_POS;
    disp_sd_comp_register_table[DISP_VBI0_HANC0_FORMAT_INDEX] = DISP_SD_VBI0_HANC0_FORMAT;
    disp_sd_comp_register_table[DISP_VENC0_DVE_FIELD_BNDRY_INDEX] = DISP_SD_VENC0_DVE_FIELD_BNDRY;
    disp_sd_comp_register_table[DISP_VENC0_DVE_VBLANK_START_BNDRY_INDEX] = DISP_SD_VENC0_DVE_VBLANK_START_BNDRY;
    disp_sd_comp_register_table[DISP_VENC0_DVE_VBLANK_END_BNDRY_INDEX] = DISP_SD_VENC0_DVE_VBLANK_END_BNDRY;
    disp_sd_comp_register_table[DISP_VENC0_DVE_CNTRL_INDEX] = DISP_SD_VENC0_DVE_CNTRL;

    disp_sd_comp_register_table[DISP_GENLOCK_PCR_INDEX] = DISP_SD_GENLOCK_PCR;
    disp_sd_comp_register_table[DISP_GENLOCK_NUM_INDEX] = DISP_SD_GENLOCK_NUM;
    disp_sd_comp_register_table[DISP_GENLOCK_DEN_INDEX] = DISP_SD_GENLOCK_DEN;
}

/**********************************************************************************************************************/
static DFBResult
XcodeSdInitScreen( CoreScreen           *screen,
                 CoreGraphicsDevice   *device,
                 void                 *driver_data,
                 void                 *screen_data,
                 DFBScreenDescription *description )
{
     u32 tmp;

     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "XcodeSdInitScreen enter \n");

     /* Set the screen capabilities. */
     description->caps = DSCCAPS_VSYNC | DSCCAPS_ENCODERS | DSCCAPS_OUTPUTS | DSCCAPS_POWER_MANAGEMENT;

     /* Set the screen name. */
     snprintf( description->name, DFB_SCREEN_DESC_NAME_LENGTH, "ViXS XCODE4 Sd Screen" );

     /* Set number of encoders and outputs. */
     description->encoders = 1;
     description->outputs  = 3;        /* 3 outputs */
     mdrv->sd_mode_index = INDEX_UNKNOWN;
     mdrv->shared->current_sd_power_mode = DSPM_OFF;
     screen->shared->mode = DSPM_OFF;

     if(dfb_config->soft_mirror_sd)
     {
         /* Create Daemon Thread */
         XcodeListenHDtoSD(mdrv);
     }

     D_DEBUG( "XcodeSdInitScreen exit \n");
     return DFB_OK;
}

/**************************************************************************************************/

static DFBResult
Xcode4SdTestEncoderConfig( CoreScreen                   *screen,
                        void                         *driver_data,
                        void                         *screen_data,
                        int                           encoder,
                        const DFBScreenEncoderConfig *config,
                        DFBScreenEncoderConfigFlags  *failed )
{

    DFBResult result=DFB_UNSUPPORTED;

    D_DEBUG( "Xcode4SdTestEncoderConfig enter \n");
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    if ((config->flags & DSECONF_SCANMODE ) &&
        (config->flags & DSECONF_OUT_SIGNALS) &&
        (config->flags & DSECONF_RESOLUTION) &&
        (config->flags & DSECONF_FREQUENCY))
    {
        if((config->resolution == DSOR_720_480 && config->frequency == DSEF_59_94HZ) ||
            (config->resolution == DSOR_720_576 && config->frequency == DSEF_50HZ))
         {
            result = DFB_OK;
         }
    }

    D_DEBUG( "Xcode4SdTestEncoderConfig exit \n");

    return result;
}

/**************************************************************************************************/

static DFBResult Xcode4SdSetEncoderConfig ( CoreScreen                   *screen,
                                          void                         *driver_data,
                                          void                         *screen_data,
                                          int                           encoder,
                                          const DFBScreenEncoderConfig *config )
{
    DFBResult result = DFB_UNSUPPORTED;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    u32 bg_color_y, bg_color_cb, bg_color_cr;
    DFBConfigLayer *conf  = &dfb_config->layers[0];
    u32 tmp;
    app_ioctl_command    app_cmd;
    struct fb_var_screeninfo    var_info;
    DFBScreenPowerMode power_mode;

    D_DEBUG( "Xcode4SdSetEncoderConfig enter \n");
    D_DEBUG( "encoder tv_standard 0x%x\n", config->tv_standard);
    D_DEBUG( "encoder out_signals  0x%x \n", config->out_signals );
    D_DEBUG( "encoder out_connectors 0x%x  \n", config->out_connectors );
    D_DEBUG( "encoder slow_blanking  0x%x  \n", config->slow_blanking  );
    D_DEBUG( "encoder scanmode  0x%x \n", config->scanmode   );
    D_DEBUG( "encoder frequency  0x%x \n", config->frequency    );
    D_DEBUG( "encoder resolution 0x%x \n", config->resolution  );
    
    //Enable the HD to SD compositing
    if (config->resolution != mdrv->shared->current_sd_resolution )
    {
        if (config->resolution == DSOR_720_480) 
        {
            //720x480
            var_info.xres    =    var_info.xres_virtual    =    720;
            var_info.yres    =    var_info.yres_virtual    =    480;
            var_info.bits_per_pixel        =    32;
            var_info.activate = FB_ACTIVATE_FORCE;
        }
        else if (config->resolution == DSOR_720_576)
        {
            //720x576
            var_info.xres    =    var_info.xres_virtual    =    720;
            var_info.yres    =    var_info.yres_virtual    =    576;
            var_info.bits_per_pixel        =    32;
            var_info.activate = FB_ACTIVATE_FORCE;
        }

        mdrv->shared->current_sd_resolution = config->resolution;

        power_mode = mdrv->shared->current_sd_power_mode;

        if(REG_READ(DISP_SD_COMP_CTRL) & DISP_SD_COMP_CTRL_COMP_EN_MASK)
        {
            tmp = REG_READ(DISP_SD_COMP_CTRL);
            tmp = tmp & (~DISP_SD_COMP_CTRL_COMP_EN_MASK);
            tmp = tmp & (~DISP_SD_COMP_CTRL_COMP_MODE_MASK);
            tmp = tmp | (0x2 << DISP_SD_COMP_CTRL_COMP_MODE_SHIFT);
            REG_WRITE(DISP_SD_COMP_CTRL, tmp);
            D_INFO("Before SD %x %x\n", REG_READ(DISP_INT_STATUS), tmp);
            REG_WRITE(DISP_SD_CMD0_FIFO, DISP_SD_CMD0_FIFO_SD_UPDATE_CMD_MASK | CMD0_FIFO_WE_MASK);
            usleep(20000);
            ViperSDDACPowerDown(mdrv);
        }

        if (mdrv->hdtosd_fd >= 0)
        {
            /* Set our resolution */
            ioctl(mdrv->hdtosd_fd, FBIOPUT_VSCREENINFO, &var_info);
        }

        if (REG_READ(DISP_SD_GRA0_BUF) == 0)
        {
            u32 output_xres;

#ifdef SD_OUTPUT_HIRES
            output_xres = var_info.xres * 2;
#else
            output_xres = var_info.xres;
#endif

            REG_WRITE( DISP_SD_GRA0_BUF, output_xres * 4);

            REG_WRITE( DISP_SD_GRA0_SAMPLES, output_xres * 4);

            REG_WRITE( DISP_SD_GRA0_SIZE, (output_xres | (var_info.yres << DISP_SD_GRA0_SIZE_V_SIZE_SHIFT)));

            REG_WRITE( DISP_SD_GRA0_SCALED_SIZE, (output_xres | (var_info.xres << DISP_SD_GRA0_SIZE_V_SIZE_SHIFT) ));
        }

        if (config->resolution == DSOR_720_480) 
        {
            SD_COMP_480i(mdrv, config);
        }
        else if (config->resolution == DSOR_720_576)
        {
            SD_COMP_576i(mdrv, config);
        }

        if(power_mode == DSPM_ON)
        {
            ViperSDDACPowerUp(mdrv);
            tmp = REG_READ(DISP_SD_COMP_CTRL);
            tmp = tmp | (DISP_SD_COMP_CTRL_COMP_EN_MASK);
            tmp = tmp & (~DISP_SD_COMP_CTRL_COMP_MODE_MASK);
            REG_WRITE(DISP_SD_COMP_CTRL, tmp);
            REG_WRITE(DISP_SD_CMD0_FIFO, DISP_SD_CMD0_FIFO_SD_UPDATE_CMD_MASK | CMD0_FIFO_WE_MASK);
            D_INFO("After SD %x %x\n", REG_READ(DISP_INT_STATUS), tmp);
        }

    }
    result = DFB_OK;
    
    D_DEBUG( "Xcode4SdSetEncoderConfig exit \n");
    return result;

}

/**************************************************************************************************/

static DFBResult
Xcode4SdTestOutputConfig( CoreScreen                  *screen,
                        void                        *driver_data,
                        void                        *screen_data,
                        int                          output,
                        const DFBScreenOutputConfig *config,
                        DFBScreenOutputConfigFlags  *failed )
{
    return DFB_OK;
}

/**************************************************************************************************/

static DFBResult 
Xcode4SdSetOutputConfig ( CoreScreen                  *screen,
                        void                        *driver_data,
                        void                        *screen_data,
                        int                          output,
                        const DFBScreenOutputConfig *config )
{
    D_DEBUG( "Xcode4SdSetOutputConfig enter \n");

    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
  
    
    D_DEBUG( "Xcode4SdSetOutputConfig exit \n");
    
    return DFB_OK;
}


/**************************************************************************************************/

static DFBResult
Xcode4SdInitEncoder( CoreScreen                  *screen,
                  void                        *driver_data,
                  void                        *screen_data,
                  int                          encoder,
                  DFBScreenEncoderDescription *description,
                  DFBScreenEncoderConfig      *config )
{
    DFBResult result = DFB_UNSUPPORTED;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    D_DEBUG( "Xcode4SdInitEncoder enter \n");

    //set the initialize display mode as CEA-861-D 
    config->flags = DSECONF_TV_STANDARD |
                    DSECONF_OUT_SIGNALS |
                    DSECONF_SCANMODE |
                    DSECONF_ADJUSTMENT |
                    DSECONF_FREQUENCY |
                    DSECONF_CONNECTORS  |
                    DSECONF_SLOW_BLANKING |
                    DSECONF_RESOLUTION ;

    config->tv_standard = DSETV_NTSC;
    config->out_signals = DSOS_CVBS;
    config->out_connectors = DSOC_CVBS;
    
    if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 480))
    {
        config->slow_blanking = DSOSB_4x3;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_720_480;
    }
    else if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 576))
    {
        config->slow_blanking = DSOSB_4x3;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_50HZ;
        config->resolution = DSOR_720_576;
    }
    else
    {
        D_DEBUG( "Xcode4InitEncoder unsupport display mode \n");
    }
    
    
    result = DFB_OK;
    Init_Disp_Sd_Comp_Register();

    D_DEBUG( "Xcode4SdInitEncoder exit \n");
    return result;
}

/**************************************************************************************************/

static DFBResult
Xcode4SdInitOutput( CoreScreen                 *screen,
                 void                       *driver_data,
                 void                       *screen_data,
                 int                         output,
                 DFBScreenOutputDescription *description,
                 DFBScreenOutputConfig      *config )
{
     DFBResult result = DFB_UNSUPPORTED;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     
     D_DEBUG( "Xcode4SdInitOutput enter \n");
     /* Initialize defaults for the output */
     config->flags    =    DSOCONF_SIGNALS        |
                DSOCONF_CONNECTORS    |
                DSOCONF_SLOW_BLANKING    |   
                DSOCONF_RESOLUTION    ;

     config->out_signals=    DSOS_CVBS | DSOS_YCBCR; 
      
     if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 480))
     {
         config->slow_blanking = DSOSB_4x3;
         config->resolution = DSOR_720_480;
     }
     else if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 576))
     {
         config->slow_blanking = DSOSB_4x3;
         config->resolution = DSOR_720_576;
     }
     else
     {
         D_DEBUG( "Xcode4InitOutput unsupported display mode on output!! \n");
     }
     
     /* Keep CVBS output off by default ... */
     config->out_signals    = DSOS_NONE;
     config->out_connectors = DSOC_UNKNOWN; 
     D_DEBUG( "Xcode4SdInitOutput exit \n");

     return DFB_OK;
}

static DFBResult
Xcode4SdGetScreenSize( CoreScreen *screen,
                     void       *driver_data,
                     void       *screen_data,
                     int        *ret_width,
                     int        *ret_height )
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    struct display_mode_struct * mode = NULL;
    DFBConfigLayer *conf;

    D_DEBUG( "Xcode4SdGetScreenSize enter \n");

    D_ASSERT( ret_width != NULL );
    D_ASSERT( ret_height != NULL );

    if (mdrv->sd_mode_index != INDEX_UNKNOWN)
    {
        mode = xc4_get_mode(mdrv->sd_mode_index);
        *ret_width  = mode->xres;
        *ret_height = mode->v_active;
    }
    else
    {
        conf = dfb_config->config_layer;
        *ret_width  = conf->config.width;
        *ret_height = conf->config.height;
    }

    D_DEBUG( "Xcode4SdGetScreenSize exit \n");

    return DFB_OK;
}


/**************************************************************************************************/

static DFBResult
Xc4SdSetPowerMode( CoreScreen         *screen,
                   void               *driver_data,
                   void               *screen_data,
                   DFBScreenPowerMode  mode )
{
     
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    u32 tmp;
    app_ioctl_command    app_cmd;
    
    if (mdrv->shared->current_sd_power_mode != mode)
    {
        mdrv->shared->current_sd_power_mode = mode;
        tmp = REG_READ(DISP_SD_COMP_CTRL);
        tmp = tmp & (~DISP_SD_COMP_CTRL_COMP_MODE_MASK);

        //tmp = tmp | DISP_SD_CMD_FIFO_DISABLE_MASK;
        
        /* This is to simply test the CVBS blanking procedure ... */
        //if (mode == DSPM_OFF) mode = DSPM_SUSPEND;
        switch (mode) 
        {
            case DSPM_OFF:
                tmp =  tmp & (~DISP_SD_COMP_CTRL_COMP_EN_MASK);
                /* Disable the HD to SD */
                if (mdrv->hdtosd_fd >= 0)
                {
                    app_cmd.uIOCTLCommand    =    CMD_DFB_IOCTL_DISABLE_HD2SD;
                    /* Disable the output */
                    ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd);
                }
                
                ViperSDDACPowerDown(mdrv);
                break;

            case DSPM_SUSPEND:
            case DSPM_STANDBY:
                //tmp =  tmp | (COMP_MODE_DISP_BACKGROUND << DISP_SD_COMP_CTRL_COMP_MODE_SHIFT);
                /* Disable the HD to SD */
                
                if (mdrv->hdtosd_fd >= 0)
                {
                    app_cmd.uIOCTLCommand    =    CMD_DFB_IOCTL_BLANK_CVBS;
                    /* Enable the output */
                    ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd);

                }
                break;

            case DSPM_ON:
                tmp =  tmp | 
                       (COMP_MODE_CMD_REPEAT << DISP_SD_COMP_CTRL_COMP_MODE_SHIFT) |
                       (DISP_SD_COMP_CTRL_COMP_EN << DISP_SD_COMP_CTRL_COMP_EN_SHIFT);
                /* Enable the HD to SD */
                if (mdrv->hdtosd_fd >= 0)
                {
                    app_cmd.uIOCTLCommand    =    CMD_DFB_IOCTL_ENABLE_HD2SD;
                    /* Enable the output */
                    ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd);
                    
                  
                    app_cmd.uIOCTLCommand    =    CMD_DFB_IOCTL_UNBLANK_CVBS;
                    /* Enable the output */
                    ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd);
                }
                /* 
                *    For our testing purposes this mode will use our default settings for the following registers:
                *    DISP_VDAC01_CNTL
                *    DISP_VDAC23_CNTL
                *    DISP_VDAC45_CNTL     
                 */
                ViperSDDACPowerUp(mdrv);
                break;

            default:
                 D_DEBUG( "unknown power mode" );
                 return DFB_INVARG;
         }
         
         /* Allow master to do everything, do not allow slave to turn off ... */
         if ( (mdrv->master != 0)  ||
              (mode != DSPM_OFF && mdrv->master == 0)
            )
         {
            REG_WRITE( DISP_SD_COMP_CTRL, tmp );
            D_INFO("PowerMode SD %x %x\n", REG_READ(DISP_INT_STATUS), tmp);
         }
    }

    return DFB_OK;
}

static DFBResult Xcode4SdSetBackgroundColor ( CoreScreen                  *screen,
                                            void                        *driver_data,
                                            void                        *screen_data,
                                       u8                        red,
                                       u8                        green,
                                       u8                        blue)
{
    u32 bg_color_y, bg_color_cb, bg_color_cr;
    u32 tmp;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    D_DEBUG( "Xcode4SdSetBackgroundColor enter \n");

    //set screen background color
    tmp = REG_READ(DISP_SD_COMP_ALPHA_MODE);

    RGB_TO_YCBCR(red, 
                 green,
                 blue,
                 bg_color_y,
                 bg_color_cb,
                 bg_color_cr);


    bg_color_y = bg_color_y & 0x0ff;
    bg_color_cb = bg_color_cb & 0x0ff;
    bg_color_cr = bg_color_cr & 0x0ff;

    tmp = tmp & (~DISP_SD_COMP_ALPHA_MODE_BACKGROUND_Y_MASK);
    tmp = tmp & (~DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CB_MASK);
    tmp = tmp & (~DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CR_MASK);

    tmp = tmp | 
          (bg_color_y << DISP_SD_COMP_ALPHA_MODE_BACKGROUND_Y_SHIFT) |
          (bg_color_cb << DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CB_SHIFT) |
          (bg_color_cr << DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CR_SHIFT);

    REG_WRITE(DISP_SD_COMP_ALPHA_MODE, tmp);

    D_DEBUG( "Xcode4SdSetBackgroundColor exit \n");

    return DFB_OK;
}

const ScreenFuncs XcodeSdScreenFuncs = {
     InitScreen:    XcodeSdInitScreen,
     TestEncoderConfig:    Xcode4SdTestEncoderConfig,
     TestOutputConfig:    Xcode4SdTestOutputConfig,
     InitEncoder:   Xcode4SdInitEncoder,
     InitOutput:    Xcode4SdInitOutput,
     SetEncoderConfig:   Xcode4SdSetEncoderConfig,
     SetOutputConfig:    Xcode4SdSetOutputConfig,
     GetScreenSize: Xcode4SdGetScreenSize,
     SetPowerMode:  Xc4SdSetPowerMode,
     SetBackgroundColor: Xcode4SdSetBackgroundColor
};


/**************************************************************************************************/
void ViperSDDACPowerUp(pxc_dfb_driver_data mdrv)
{
    /* 
    *    Power up ONLY the SD analog DAC's 
    *    any other DAC's should remain untouched
    *    HD Y, Pb, Pr
    *    It is important to note 
    */
    u32 tmp;
    
    /* 
    *   The following two register settings are used only
    *   when 576i is set as the output on the SD path.
    */
    //REG_WRITE(DISP_VDAC23_CNTL , 0xa610a600);
    //REG_WRITE(DISP_VDAC23_CNTL , 0xa610a610);
    
    if (mdrv->asic_flags & VIPER_ASIC_A21)
    {
        //REG_WRITE(DISP_VDAC_SET0_CNTL, 0x80000015);
        REG_WRITE(DISP_VDAC_SET1_CNTL, 0x80000080);
        //REG_WRITE(DISP_VDAC01_CNTL, 0xA600A600);
        if (mdrv->shared->current_sd_resolution == DSOR_720_576)
        {
            tmp = REG_READ(DISP_VDAC23_CNTL);
            tmp|= 0xA6100000; /* Enable the SD CV DAC */
            REG_WRITE(DISP_VDAC23_CNTL, tmp);
            REG_WRITE(DISP_VDAC45_CNTL, 0xA610A610);
        }
        else
        {
            tmp = REG_READ(DISP_VDAC23_CNTL);
            tmp|= 0xA6000000; /* Enable the SD CV DAC */
            REG_WRITE(DISP_VDAC23_CNTL, tmp);
            REG_WRITE(DISP_VDAC45_CNTL, 0xA600A600);
        }
        
        /*
        *    The SVideo DAC's should probably be enabled as well
        *
        */
        
        REG_WRITE(DISP_VDAC_BIAS_CNTL, 0x10);
    }
    else
    {
        //REG_WRITE(DISP_VDAC_SET0_CNTL,0x04440015);
        REG_WRITE(DISP_VDAC_SET1_CNTL,0x04444080);
        //  REG_WRITE(DISP_VDAC01_CNTL,0xB600B600);
        tmp = REG_READ(DISP_VDAC23_CNTL);
        tmp|= 0xB6000000;
        REG_WRITE(DISP_VDAC23_CNTL,tmp);  
        /* Enabling composite DAC on A11 */
        
        REG_WRITE(DISP_VDAC45_CNTL,0xB600B600);
        /* Enabling SVideo Y/C DAC's on A11 */
        //  REG_WRITE(DISP_VDAC_CABLE_CONNECT,0xB6000000);
        REG_WRITE(DISP_VDAC_BIAS_CNTL, 0x10);
    }

    //Enable SD output
    RegMaskWrite(DISP_OUTPUT_MUX, 0x00, 0x20);

    
    /* 
    *    This will powerup the analog DAC's 
    *    It will also be used the SD parts as well
    */
    D_INFO("**** SD DAC on...\n");
    return;
}

void ViperSDDACPowerDown(pxc_dfb_driver_data mdrv)
{
    /* 
    *    Power down ONLY the SD analog DAC's 
    *    any other DAC's should remain untouched
    *    HD Y, Pb, Pr
    *     
    */
    u32 tmp;
    
    if (mdrv->asic_flags & VIPER_ASIC_A21)
    {
        //REG_WRITE(DISP_VDAC_SET0_CNTL, 0x0);
        REG_WRITE(DISP_VDAC_SET1_CNTL, 0x0);
        
        /* Disable the SD CV DAC */
        tmp = REG_READ(DISP_VDAC23_CNTL);
        tmp&= (~0xFF000000); 
        REG_WRITE(DISP_VDAC23_CNTL, tmp);
    
        /* SVideo DAC's are disabled */
        REG_WRITE(DISP_VDAC45_CNTL, 0x00000000);
        //REG_WRITE(DISP_VDAC_BIAS_CNTL, 0x00);
    }
    else
    {
        //REG_WRITE(DISP_VDAC_SET0_CNTL,0x0);
        REG_WRITE(DISP_VDAC_SET1_CNTL,0x0);
        //  REG_WRITE(DISP_VDAC01_CNTL,0xB600B600);
        tmp = REG_READ(DISP_VDAC23_CNTL);
        tmp&= (~0xFF000000);
        REG_WRITE(DISP_VDAC23_CNTL,tmp);  
        /* Disabling composite DAC on A11 */
        
        REG_WRITE(DISP_VDAC45_CNTL,0x0);
        /* Disabling SVideo Y/C DAC's on A11 */
        //  REG_WRITE(DISP_VDAC_CABLE_CONNECT,0xB6000000);
        //REG_WRITE(DISP_VDAC_BIAS_CNTL, 0x0);
    }
    D_INFO("**** SD DAC off...\n");
    //Disable SD output
    return;
}



/**************************************************************************************************/
static void SD_COMP_480i(pxc_dfb_driver_data mdrv, const DFBScreenEncoderConfig *config )
{
    u32        reg_value, tmp;
    static u32         first = 1;    
    D_INFO("Setup 480i composite... \n");

#ifdef SD_OUTPUT_HIRES
    REG_WRITE(DISP_SD_COMP_SIZE, 0x801e05a0);
    REG_WRITE(DISP_SD_COMP_GLOBAL_SIZE, 0x20d06b4);
    REG_WRITE(DISP_SD_COMP_START_PIX_POS, 0x120);
#else
    REG_WRITE(DISP_SD_COMP_SIZE, 0x811e02d0);
    REG_WRITE(DISP_SD_COMP_GLOBAL_SIZE, 0x20d035a);
    REG_WRITE(DISP_SD_COMP_START_PIX_POS, 0x8D);
#endif

    REG_WRITE(DISP_SD_COMP_START_VBI_POS, 0xffffff);
    REG_WRITE(DISP_SD_COMP_STOP_VBI_POS, 0xffffff);
    REG_WRITE(DISP_SD_COMP_START_FLD_POS, 0x11e017);
    REG_WRITE(DISP_SD_COMP_START_HANC_POS, 0x0);
    REG_WRITE(DISP_SD_COMP_STOP_HANC_POS, 0x0);
    REG_WRITE(DISP_SD_VENC0_DVE_FIELD_BNDRY, 0x10a0004);
    REG_WRITE(DISP_SD_VENC0_DVE_VBLANK_START_BNDRY, 0x1080001);
    REG_WRITE(DISP_SD_VENC0_DVE_VBLANK_END_BNDRY, 0x11c0015);
    
 
    if (first)
    {
        REG_WRITE(DISP_SD_VENC0_DVE_CNTRL, 0x20);
        REG_WRITE(DISP_SD_VBI0_HANC0_FORMAT, 0x80808000);
        first = 0;
    }else{
        reg_value = REG_READ(DISP_SD_VENC0_DVE_CNTRL);
        reg_value &= (DISP_SD_VENC0_DVE_CNTRL_VBI_OUT_VENC_MASK | DISP_SD_VENC0_DVE_CNTRL_VBI_ON_Y_MASK); 
        reg_value |= 0x20;    
        REG_WRITE(DISP_SD_VENC0_DVE_CNTRL, reg_value);

        reg_value = REG_READ(DISP_SD_VBI0_HANC0_FORMAT) & DISP_SD_VBI0_HANC0_FORMAT_VBI_PIPE_DISABLE_MASK;
        reg_value |= 0x808000;
        REG_WRITE(DISP_SD_VBI0_HANC0_FORMAT, reg_value);
    }

    REG_WRITE(DISP_SD_VENC0_CONT_STATUS, 0x0);

////////////////////////////////////

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118001);  
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30518001);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x01);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20908001);    /* jk0529 from 0x20928001 to 0x20908001 */
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x02);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000008);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x03);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x04);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x05);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x06);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x07);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118002);   //center of vsync   -____-____
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x08);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31808006);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x09);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x332e8002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0F);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118002);  // equalizing pulses without bpp's  -_----_----      
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x10);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30318002 );
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x11);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x12);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x13);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x14);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x15);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x16);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x17);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118002);  // Transition from equ to vsync in odd field wtih bpp  -_--_____-    XX
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x18);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30318002); 
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x19);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x204e8002);   // This is where the bpp's are in the alt state 
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08002);   //   xxxx  sition is at 900, should be 1716/2 ?
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x332b8002);   //   til 1626
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1f);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108003);  //  blank lines with color burst to end v blank without ps has blank redux -__vvvvvvvv  XX
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x20);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30518003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x21);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20908003);    /* jk0529, change from 0x20928003 to 0x20908003 */
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x22);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000008);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x23);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x24);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x25);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x26);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x27);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118006);  // Transition from vsync to equ in odd field  -____-_-----
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x28);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31808002);   // til 772
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x29);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08002);   // til 900  (seems like it should be 890)
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E08002);   // til 932
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2F);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118000);  //  half line at end of first field    -__-----_------              XX
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x30);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30518002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x31);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x306c8003);           // This is where the bpp's go in the alt event  (SHOULD be state 2)  <<<<<<
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x32);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20928000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x33);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08008);  // til 900
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x34);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x35);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x36);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x37);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118002);  // first non-equ pulse after vsync in odd field, no color burst  -_pppppppp
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x38);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30338002);  //  This is screwy
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x39);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x74000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3F);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118003);  //   blank lines with color burst to end v blank with ps and blank redux -__-pppppppp
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x40);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30518003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x41);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20828003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x42);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x74000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x43);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x44);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x45);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x46);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x47);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118002);  // equalizing pulses with bpp's   -_b-----_------
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x48);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30310002);   
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x49);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x404b8002);    // change from  504e8002 to 404e8002 jk0525
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4F);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118001);   //normal active video with bpp's  -__b-vvvvvvvvv
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x50);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30518001);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x51);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x606c8001);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x52);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20908001);    /* jk0529, change from 0x20928001 to 0x20908001  */
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x53);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000008);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x54);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x55);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x56);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x57);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118002);  // Transition from equ to vsync in odd field with bpp alt3 -_b---_____-
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x58);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30310002);   
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x59);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x404b8002);   // change from 504e8002 t0 404e8002    jk0525
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x332b8002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5f);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118002);  // first non-equ pulse after vsync in odd field, no color burst  -_bpppppppp    XX
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x60);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30320002);    // change from 30350002 to 30338002    jk0525
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x61);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x404b8003);    // change from 50520003 to 404d0003    jk0525
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x62);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x74000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x63);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x64);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x65);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x66);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x67);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20118000);  //  half line at end of first field  Alt 6   -__b----_----
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x68);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30518002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x69);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x606c8003);           // This is where the bpp's go in the alt event
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20928000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08008);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6f);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20128003);  //   blank lines with color burst to end v blank with ps and blank redux  -__b-ppppppp   ((XX))
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x70);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30338002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x71);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x406b8003);    // change from 506d0003 to 406d0003  jk0525
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x72);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20828003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x73);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x74000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x74);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x75);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x76);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x77);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x80);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30530003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x81);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20930003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x82);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30b30002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x83);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00dd0002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x84);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21030002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x85);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31230002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x86);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x014d0002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x87);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21730002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x88);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21e30002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x89);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x22530002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x32730002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x029d0002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x22c30002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x23330002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x23a30002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8f);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x241300);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x90);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x69);    //equ with bpp's
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x100);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0xc1);    //vsync
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x101);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x122);   //equ wo bpps
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x102);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x268);   // blank lines with color burst, allow ps, blank redux
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x103);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x2080);  // active video to line 260
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x104);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20ca);  // active video to line 262 with bpp's
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x105);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20ed);  // half line with bpp allowed 
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x106);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x2129);  //equ with bpp's
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x107);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x214b);  //xstion from equ to vsync with bpp
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x108);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x2181);  // vsync
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x109);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21a5);  // xsition from vsync to equ in field 2
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x10a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21e2);  // equ without bpps
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x10b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x2207);  // blank line wo color burst
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x10c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x2348);  // blank lines with color burst. allow ps. blank redux until line 283
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x10d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x4140);  // normal video until line 522
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x10e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x3ffea);  // normal video with bpps
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x10f);

////////////////////////
        
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130001);   //normal active video  0
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30530001);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x01);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20940001);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x02);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000009);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x03);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x04);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x05);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x06);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x07);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130002);   //center of vsync
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x08);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31820006);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x09);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C20002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x332F0002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0F);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130002);  // equalizing pulses without bpp's
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x10);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30330002 );
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x11);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C20002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x12);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E20002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x13);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x14);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x15);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x16);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x17);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130002);  // Transition from equ to vsync in odd field wtih bpp
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x18);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30330002 );
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x19);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20500002 );  // This is where the bpp's are in the alt state 
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C20002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x332D0002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x1f);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130003);  //  blank lines with color burst to end v blank without ps has blank redux
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x20);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30530003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x21);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20940003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x22);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000008);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x23);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x24);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x25);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x26);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x27);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130006);  // Transition from vsync to equ in odd field
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x28);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31820002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x29);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C20002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E20002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x2F);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130000);  //  half line at end of first field
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x30);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30530002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x31);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x306e0003);           // This is where the bpp's go in the alt event
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x32);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20940000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x33);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C20008);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x34);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E20002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x35);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x36);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x37);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130002);  // first non-equ pulse after vsync in odd field, no color burst
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x38);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30350002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x39);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x74000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x3F);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130003);  //   blank lines with color burst to end v blank with ps and blank redux
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x40);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30530003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x41);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20840003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x42);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x74000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x43);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x44);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x45);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x46);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x47);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130002);  // equalizing pulses with bpp's
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x48);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30300002 );
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x49);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x404e0002 );   // change to 0x4yyyyyyy
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C20002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E20002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x4F);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130001);   //normal active video with bpp's
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x50);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30530001);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x51);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x606e0001);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x52);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20940001);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x53);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000008);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x54);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x55);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x56);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x57);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130002);  // Transition from equ to vsync in odd field with bpp alt3
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x58);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30300002 );
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x59);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x404e0002 );  // change to 0x4yyyyyyy 
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C20002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x332D0002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x5f);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130002);  // first non-equ pulse after vsync in odd field, no color burst
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x60);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30350002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x61);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x40520003 );   // change to 0x4yyyyyyy
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x62);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x74000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x63);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x64);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x65);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x66);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x67);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130000);  //  half line at end of first field
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x68);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30530002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x69);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x606e0003);           // This is where the bpp's go in the alt event
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20940000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C20008);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E20002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x6f);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130003);  //   blank lines with color burst to end v blank with ps and blank redux
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x70);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30500003 );
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x71);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x406d0003 );   // change to 0x4yyyyyyy
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x72);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20840003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x73);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x74000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x74);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x75);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x76);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x77);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20130003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x80);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30530003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x81);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20930003);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x82);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30b30002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x83);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00dd0002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x84);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21030002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x85);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31230002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x86);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x014d0002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x87);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21730002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x88);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21e30002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x89);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x22530002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x32730002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x029d0002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x22c30002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x23330002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x23a30002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x8f);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x241300);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x90);

    REG_WRITE(DISP_SD_VENC0_CB_FREQ, 0x21f07c1f);
    REG_WRITE(DISP_SD_VENC0_CB_PHASE, 0x0fd0bedc);
    REG_WRITE(DISP_SD_VENC0_CMPST_GAIN, 0x3A587cff);
    REG_WRITE(DISP_SD_VENC0_BLACK, 0x00);
    REG_WRITE(DISP_SD_VENC0_BLANK, 0x3200ec);
    REG_WRITE(DISP_SD_VENC0_SYNC1, 0xb4000b);
    REG_WRITE(DISP_SD_VENC0_SYNC2, 0x2d000ec);
    REG_WRITE(DISP_SD_VENC0_YCMPN, 0x3c000a0);
    REG_WRITE(DISP_SD_VENC0_CB2GAIN, 0x80660066);
    
    REG_WRITE(DISP_SD_VENC0_FIELDBNDRY, 0x106);
    REG_WRITE(DISP_SD_VENC0_COEF_FYCMPS0, 0x0);
    REG_WRITE(DISP_SD_VENC0_COEF_FYCMPS1, 0x0);
    REG_WRITE(DISP_SD_VENC0_COEF_FYCMPS2, 0x20000000);
    REG_WRITE(DISP_SD_VENC0_COEF_FYCMPS3, 0xc0);
    REG_WRITE(DISP_SD_VENC0_COEF_FUVCMPS0, 0x0);
    REG_WRITE(DISP_SD_VENC0_COEF_FUVCMPS1, 0x0802fef4);
    REG_WRITE(DISP_SD_VENC0_COEF_FUVCMPS2, 0x26211910);
    REG_WRITE(DISP_SD_VENC0_COEF_FUVCMPS3, 0x00000028);

    REG_WRITE(DISP_SD_VENC0_CB_PHASE, 0x1020bedc);
    RegMaskWrite(DISP_OUTPUT_MUX, 0xc, 0x2c);

    REG_WRITE(DISP_SD_VENC0_DCS_CONFIG, 0x4);
    REG_WRITE(DISP_SD_VENC0_DCS_CONFIG, 0x80210104);
    REG_WRITE(DISP_SD_VENC0_DCS_CMPNT, 0x3e0002c);
    REG_WRITE(DISP_SD_VENC0_DCS_CMPNT_ICLK227K, 0x450081);
    REG_WRITE(DISP_SD_VENC0_DCS_CMPNT_ICLK397K, 0x3f00e3);
    REG_WRITE(DISP_SD_VENC0_DCS_CONFIG_CMST, 0x2);
    REG_WRITE(DISP_SD_VENC0_DCS_CONFIG_CMST, 0x80210102);
    REG_WRITE(DISP_SD_VENC0_DCS_CMST, 0x3ad0028);
    REG_WRITE(DISP_SD_VENC0_DCS_CMST_ICLK227K, 0x450081);
    REG_WRITE(DISP_SD_VENC0_DCS_CMST_ICLK397K, 0x3f00e3);
    REG_WRITE(DISP_SD_VENC0_DCS_START_LINE, 0x4);
    REG_WRITE(DISP_SD_VENC0_DCS_FAKE_FRAME_START, 0x0);
    REG_WRITE(DISP_SD_VENC0_DCS_KEY_HIGH, 0x9c926ccb);
    REG_WRITE(DISP_SD_VENC0_DCS_KEY_LOW, 0xe51cdf98);
    REG_WRITE(DISP_SD_VENC0_DCS_KEY_HIGH, 0xd993f817);
    REG_WRITE(DISP_SD_VENC0_DCS_KEY_LOW, 0x9bae50d5);

#ifdef LG_BDR_4210
    REG_WRITE(DISP_SD_VENC0_BLACK, 0x24);
    REG_WRITE(DISP_SD_VENC0_CB2GAIN, 0x80600060);
#endif

    REG_WRITE(DISP_SD_VENC0_CONT_STATUS, 0x3);
}

static void SD_COMP_576i(pxc_dfb_driver_data mdrv, const DFBScreenEncoderConfig *config)
{
    u32 reg_value, tmp;
    static u32 first    =   1;
    D_INFO("Setup 576i composite... \n");

#ifdef SD_OUTPUT_HIRES
    REG_WRITE(DISP_SD_COMP_SIZE, 0x802405a0);
    REG_WRITE(DISP_SD_COMP_GLOBAL_SIZE, 0x027106C0);
    REG_WRITE(DISP_SD_COMP_START_PIX_POS, 0x130);
#else
    REG_WRITE(DISP_SD_COMP_SIZE, 0x812402d0);
    REG_WRITE(DISP_SD_COMP_GLOBAL_SIZE, 0x02710360);
    REG_WRITE(DISP_SD_COMP_START_PIX_POS, 0x98);
#endif

    REG_WRITE(DISP_SD_COMP_START_FLD_POS, 0x150017); 
    REG_WRITE(DISP_SD_COMP_START_VBI_POS, 0xffffff);
    REG_WRITE(DISP_SD_COMP_STOP_VBI_POS, 0xffffff);
 
    if (first)
    {
        REG_WRITE(DISP_SD_VENC0_DVE_CNTRL, 0x20);
        REG_WRITE(DISP_SD_VBI0_HANC0_FORMAT, 0x80808000);
        first = 0;
    }else{
        reg_value = REG_READ(DISP_SD_VENC0_DVE_CNTRL);
        reg_value &= (DISP_SD_VENC0_DVE_CNTRL_VBI_OUT_VENC_MASK | DISP_SD_VENC0_DVE_CNTRL_VBI_ON_Y_MASK); 
        reg_value |= 0x20;    
        REG_WRITE(DISP_SD_VENC0_DVE_CNTRL, reg_value);

        reg_value = REG_READ(DISP_SD_VBI0_HANC0_FORMAT) & DISP_SD_VBI0_HANC0_FORMAT_VBI_PIPE_DISABLE_MASK;
        reg_value |= 0x808000;
        REG_WRITE(DISP_SD_VBI0_HANC0_FORMAT, reg_value);
    }
    
    REG_WRITE(DISP_SD_VENC0_CONT_STATUS, 0x0);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30500011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000001);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x205D8011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000002);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x207C0011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000003);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20A00011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000004);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000005);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000006);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000007);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000008);     
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31810002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000009);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000000A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x33310002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000000B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000000C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000000D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000000E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000000F);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108002);  
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000010);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30300002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000011);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000012);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E00002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000013);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000014);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000015);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000016);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000017);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000018);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31810002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000019);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000001A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E00002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000001B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000001C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000001D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000001E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000001F);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108014);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000020);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30500010);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000021);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20A00010);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000022);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000010);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000023);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000024);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000025);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000026);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000027);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000028); 
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30300002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000029);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000002A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x33310002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000002B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000002C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000002D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000002E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000002F);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108016);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000030);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30500012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000031);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20A00012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000032);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000033);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000034);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000035);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000036);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000037);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108016);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000038);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30500012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000039);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x606b0010);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000003a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20A00012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000003b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000003c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000003D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000003E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000003F);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108013);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000040);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30500013);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000041);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20800013);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000042);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x74000013);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000043);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000044);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000045);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000046);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000047);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108006);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000048);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30300002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000049);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000004A);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000004B);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000004C);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000004D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000004E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000004F);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000050);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30500011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000051);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x406b0011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000052);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x207C0011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000053);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20A00011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000054);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000011);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000055);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000056);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000057);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000058);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30300002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000059);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x604b0002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000005a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x21C08002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000005b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x31E00002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000005c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x24000002);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000005d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000005e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000005f);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108013);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000060);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30500013);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000061);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x606b0013);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000062);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20A00013);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000063);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x74000013);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000064);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000065);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000066);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000067);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108014);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000068);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30500010);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000069);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x406b0010);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000006a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20A00010);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000006b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000010);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000006c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000006d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000006e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000006f);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108002);

    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108012);        //blank line with pal alternate color burst and no ps/agc activation
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000070);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30500012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000071);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20A00012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000072);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000073);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000074);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000075);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000076);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000077);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20108012);  //same as previous with bpp's
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000078);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x30500012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000079);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x606b0010);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000007a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x20A00012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000007b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x04000012);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000007c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000007D);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000007E);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000007F);
    
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000041);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000100);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000063);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000101);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x000000A2);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000102);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x000000C6);   // 6 for normal, 0xc for bpp
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000103);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x000000e8);   // 8 for normal, 0xc for bpp
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000104);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x000002C8);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000105);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00002680);    // Set end point to last line wo bpp
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000106);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x000026a0);    // Set to 0xa for bpp's, 0x0 for no bpp's
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000107);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x000026c4);    // Set to 4 if no bpp, 0xd for bpp
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000108);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x000026e2);    // 0xb for bpp, 0x2 for no bpp
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000109);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00002702);    // 0xb for bpp, 0x2 for no bpp
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000010a);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00002725);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000010b);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00002761);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000010c);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x000027a2);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000010d);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x000027c9);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000010e);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x000027ee);    // e for normal, 0xf for bpp, inc end address to add pulses after vertical blank
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x0000010f);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00002808);    // 8 for normal, 0xc for bpp, inc end address to add pulses after vertical blank
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000110);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x000029e8);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000111);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00004d80);    // dec end address to add bpp's b4 vbi
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000112);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00004da0);    // 0x0 for no bpp's, 0xa for bpp's
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000113);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00004dc4);    // 0xd for bpp b4 vbi, 4 for no bpp b4 f1 vbi
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000114);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00004e02);    // 2 for no bpp, 0xb for bpp.. change end point to change number of bpps b4 f1 vb
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000115);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00004e42);    // 2 for no bpp, 0xb for bpp
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000116);
    REG_WRITE(DISP_SD_VENC0_WRDATA, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_WRADDR, 0x00000116);
    REG_WRITE(DISP_SD_VENC0_FIELDBNDRY , 0x139);
    
    REG_WRITE(DISP_SD_VENC0_COEF_FYCMPS0, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_COEF_FYCMPS1, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_COEF_FYCMPS2, 0x20000000);
    REG_WRITE(DISP_SD_VENC0_COEF_FYCMPS3, 0x000000c0);
    REG_WRITE(DISP_SD_VENC0_COEF_FUVCMPS0, 0x00000000);
    REG_WRITE(DISP_SD_VENC0_COEF_FUVCMPS1, 0x0802fef4);
    REG_WRITE(DISP_SD_VENC0_COEF_FUVCMPS2, 0x26211910);
    REG_WRITE(DISP_SD_VENC0_COEF_FUVCMPS3, 0x00000028);

    //new
    REG_WRITE(DISP_SD_VENC0_CB_FREQ, 0x2A098ACB);
    REG_WRITE(DISP_SD_VENC0_CB_PHASE, 0xF80BC09);
    REG_WRITE(DISP_SD_VENC0_CMPST_GAIN, 0x254d6cdc);
    REG_WRITE(DISP_SD_VENC0_BLACK, 0x0);
    REG_WRITE(DISP_SD_VENC0_BLANK, 0x280170);        // without bpps
    REG_WRITE(DISP_SD_VENC0_BLANK, 0x6f0170);        // with bpps
    REG_WRITE(DISP_SD_VENC0_SYNC1, 0x01420088);
    REG_WRITE(DISP_SD_VENC0_SYNC2, 0x3900362);       // with sync suppression
    REG_WRITE(DISP_SD_VENC0_SYNC2, 0x3900390);       // without sync suppression
    REG_WRITE(DISP_SD_VENC0_YCMPN, 0x03C000a0);
    REG_WRITE(DISP_SD_VENC0_CB2GAIN, 0x80660066);

    REG_WRITE(DISP_SD_VENC0_CONT_STATUS, 0x1);
}

