
/************************************************** 
*
*  Copyright (c) 2006-2011 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_sd_vbi.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/


#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include <pthread.h>

#include <sys/mman.h>
#include <fcntl.h>
 #include <sys/ioctl.h>


#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/layers_internal.h>
#include <core/surface.h>
#include <core/palette.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"
#include "xcioctl.h"



#define MAX_VBI_REGION_INDEX 16


DFBResult Xcode_Sd_Vbi_Plane_Update_Region(void *driver_data,
                                        void *layer_data,
                                        CoreSurface *surface,
                                        const DFBRegion *update);

static void enable_sd_vbi_output(void *driver_data);
static void disable_sd_vbi_output(void *driver_data);

/*******************************************************************************************************/

int
XcodeSdVbiLayerDataSize()
{
     return sizeof(xc_layer_data);
}


/*******************************************************************************************************/

static void enable_sd_vbi_output(void *driver_data)
{
     u32 tmp;
     u32 temp = 0;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     // set DISP_SD_VENC0_DVE_CNTRL to enable sd vbi output
     tmp = REG_READ(DISP_SD_VENC0_DVE_CNTRL);

     D_DEBUG( "REG_READ(DISP_SD_VENC0_DVE_CNTRL) = %x\n", tmp );

     tmp |= (DISP_SD_VENC0_DVE_CNTRL_VBI_OUT_VENC_MASK | DISP_SD_VENC0_DVE_CNTRL_VBI_ON_Y_MASK);
     REG_WRITE(DISP_SD_VENC0_DVE_CNTRL, tmp);

     // clear DISP_SD_VBI0_HANC0_FORMAT.VBI_PIPE_DISABLE bit
     tmp = REG_READ(DISP_SD_VBI0_HANC0_FORMAT);
     D_DEBUG( "REG_READ(DISP_SD_VBI0_HANC0_FORMAT) = %x\n", tmp);
     tmp &= ~DISP_SD_VBI0_HANC0_FORMAT_VBI_PIPE_DISABLE_MASK;           //0x7FFFFFFF;
     REG_WRITE(DISP_SD_VBI0_HANC0_FORMAT, tmp);

}


static void disable_sd_vbi_output(void *driver_data)
{
     u32 tmp;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     // set DISP_SD_VENC0_DVE_CNTRL to disable sd vbi output
     tmp = REG_READ(DISP_SD_VENC0_DVE_CNTRL);
  
     D_DEBUG( "REG_READ(DISP_SD_VENC0_DVE_CNTRL) = %x\n", tmp );

     tmp &= ~(DISP_SD_VENC0_DVE_CNTRL_VBI_OUT_VENC_MASK | DISP_SD_VENC0_DVE_CNTRL_VBI_ON_Y_MASK);  // 0x3FFFFFFF;
     REG_WRITE(DISP_SD_VENC0_DVE_CNTRL, tmp);

     // set DISP_SD_VBI0_HANC0_FORMAT.VBI_PIPE_DISABLE bit
     tmp = REG_READ(DISP_SD_VBI0_HANC0_FORMAT);
     D_DEBUG( "REG_READ(DISP_SD_VBI0_HANC0_FORMAT) = %x\n", tmp);
     tmp |= DISP_SD_VBI0_HANC0_FORMAT_VBI_PIPE_DISABLE_MASK; 
     REG_WRITE(DISP_SD_VBI0_HANC0_FORMAT, tmp);
     D_DEBUG( "REG_WRITE(DISP_SD_VBI0_HANC0_FORMAT) = %x\n", tmp);

}


/****************************************************************************************************/

static DFBResult
XcodeSdVbi0InitLayer( CoreLayer                *layer,
                    void                       *driver_data,
                    void                       *layer_data,
                    DFBDisplayLayerDescription *description,
                    DFBDisplayLayerConfig      *config,
                    DFBColorAdjustment         *adjustment )
{
     u32     tmp;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "xcode DFB driver XcodeSdVbi0InitLayer enter:\n" );

     /* set capabilities and type */
     description->caps = DLCAPS_SURFACE | DLCAPS_SCREEN_POSITION | DLCAPS_SCREEN_LOCATION | DLCAPS_LEVELS;

     description->type = DLTF_NONE;

     /* set name */
     snprintf( description->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "ViXS Xcode4 SD VBI Plane 0 over SD output" );

     /* fill out the default configuration */
     config->flags       = DLCONF_WIDTH       | DLCONF_HEIGHT | DLCONF_FLIP_HEIGHT |
                           DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS;

     config->width       = 2048;  //XCODE_DEFAULT_WIDTH;
     config->height      = 16 * 4;    //XCODE_DEFAULT_HEIGHT;
     config->flip_height = 16;

     config->pixelformat = DSPF_LUT8; //DSPF_I420;
     config->buffermode  = DLBM_FRONTONLY;
     config->options     = DLOP_ALPHACHANNEL;

     adjustment->flags   = DCAF_ALL;
     adjustment->brightness = LAYER_DEFAULT_BRIGHT;
     adjustment->contrast = LAYER_DEFAULT_CONTRAST;
     adjustment->saturation = LAYER_DEFAULT_SATURATION;
     adjustment->hue = LAYER_DEFAULT_HUE;

     m_layer_data->last_frame_index = -1;
     m_layer_data->last_frame_counter = 0;
     m_layer_data->total_submitted_frames = 0;
     m_layer_data->total_return_frame_to_free = 0;
     m_layer_data->status = 0;
     m_layer_data->flip_height = 16;
     m_layer_data->dest_width = 1920;
     m_layer_data->dest_height = 16;

     enable_sd_vbi_output(mdrv);  // enable sd vbi output
     D_DEBUG( "enable sd vbi0 output \n" );

     D_DEBUG( "xcode DFB driver XcodeSdVbi0InitLayer exit:\n" );

     return DFB_OK;
}



static DFBResult
XcodeSdVbi0SetRegion( CoreLayer                  *layer,
                    void                       *driver_data,
                    void                       *layer_data,
                    void                       *region_data,
                    CoreLayerRegionConfig      *config,
                    CoreLayerRegionConfigFlags  updated,
                    CoreSurface                *surface,
                    CorePalette                *palette,
                    CoreSurfaceBufferLock      *lock )
{
     DFBResult ret;
     u32 low_key;
     u32 temp;
     u32 tmp;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     D_ASSERT( driver_data != NULL );
     D_ASSERT( layer_data != NULL );

     D_DEBUG( "XcodeSdVbi0SetRegion enter \n" );
     D_DEBUG ("surface width %d \n", config->width);
     D_DEBUG ("surface height %d \n", config->height);
     D_DEBUG ("destination x %d \n", config->dest.x);
     D_DEBUG ("destination y %d \n", config->dest.y);
     D_DEBUG ("destination width %d \n", config->dest.w);
     D_DEBUG ("destination height %d \n", config->dest.h);
     D_DEBUG ("surface caps %d \n", config->surface_caps);
     D_DEBUG ("surface pitch %d \n", lock->pitch);
     D_DEBUG ("surface phys 0x%x \n", lock->phys);

     
     // set sd VBI0 buffer size to register: DISP_SD_VBI0_HANC0_SIZE
     tmp = REG_READ(DISP_SD_VBI0_HANC0_SIZE);
     D_DEBUG( "REG_READ(DISP_SD_VBI0_HANC0_SIZE) = %x\n", tmp );
     tmp &= ~DISP_SD_VBI0_HANC0_SIZE_VBI_SIZE_MASK;          //0xFFFFF000;
     D_DEBUG( "config->dest.w = %x\n", config->dest.w);
     tmp |= config->dest.w;
     REG_WRITE(DISP_SD_VBI0_HANC0_SIZE, tmp);
     D_DEBUG( "REG_WRITE(DISP_SD_VBI0_HANC0_SIZE) = %x\n", tmp );

     // set the pitch width of the sd VBI buffer to register: DISP_SD_VBI0_BUF
     //tmp = (u32)config->width & 0x0FFFF; 
     tmp = (u32)config->width & DISP_SD_VBI0_BUF_PITCH_MASK;   
     //temp = ((config->width * config->flip_height) / 16) & 0xFFFF;
     temp = ((config->width * config->flip_height) / 16) & ~DISP_SD_VBI0_BUF_SIZE_MASK;
     temp <<= 16;
     tmp |= temp;
     REG_WRITE(DISP_SD_VBI0_BUF, tmp);
     D_DEBUG( "REG_WRITE(DISP_SD_VBI0_BUF) = %x\n", tmp );

     // set sd VBI buffer start address to register: DISP_SD_VBI0_ADR
     REG_WRITE(DISP_SD_VBI0_ADR, lock->phys);
     D_DEBUG( "REG_WRITE(DISP_SD_VBI0_ADR) = %x\n", lock->phys);

     // set register:  DISP_SD_COMP_START_VBI_POS
     tmp = config->dest.y;
     REG_WRITE(DISP_SD_COMP_START_VBI_POS, tmp);
     D_DEBUG( "REG_WRITE(DISP_SD_COMP_START_VBI_POS) = %x\n", tmp );
 
     // set register:  DISP_SD_COMP_STOP_VBI_POS
     tmp = config->dest.h;
     REG_WRITE(DISP_SD_COMP_STOP_VBI_POS, tmp);
     D_DEBUG( "REG_WRITE(DISP_SD_COMP_STOP_VBI_POS) = %x\n", tmp );

     // set register:  DISP_SD_VBI0_SAMPLES
     tmp = REG_READ(DISP_SD_VBI0_SAMPLES) & ~DISP_SD_VBI0_SAMPLES_WIDTH_MASK;  
     temp = config->dest.w;
     tmp |= temp;
     REG_WRITE(DISP_SD_VBI0_SAMPLES, tmp);
     D_DEBUG( "REG_WRITE(DISP_SD_VBI0_SAMPLES) = %x\n", tmp );
     
     enable_sd_vbi_output(mdrv);  // enable sd vbi output
     
     D_DEBUG( "XcodeSdVbi0SetRegion exit \n" );

     return DFB_OK;
}


static DFBResult
XcodeSdVbi0TestRegion( CoreLayer                  *layer,
                     void                       *driver_data,
                     void                       *layer_data,
                     CoreLayerRegionConfig      *config,
                     CoreLayerRegionConfigFlags *failed )
{
     CoreLayerRegionConfigFlags fail = 0;

     D_DEBUG( "xcode DFB driver XcodeSdVbi0TestRegion enter:\n" );

     D_DEBUG ("surface width %d \n", config->width);
     D_DEBUG ("surface height %d \n", config->height);
     D_DEBUG ("destination x %d \n", config->dest.x);
     D_DEBUG ("destination y %d \n", config->dest.y);
     D_DEBUG ("destination width %d \n", config->dest.w);
     D_DEBUG ("destination height %d \n", config->dest.h);
     D_DEBUG ("surface caps %d \n", config->surface_caps);
     D_DEBUG ("surface options %d \n", config->options);

     if (config->options & ~XCODE4_OSD_SUPPORTED_OPTIONS)
          fail |= CLRCF_OPTIONS;

     switch (config->format) {
          case DSPF_I420:
          case DSPF_YV12:
          case DSPF_YV16:  //replaced DSPF_UYVY
          case DSPF_LUT8:
          case DSPF_YUY2://TODO: not consistent with DirectFB pixel format.
               break;

          default:
               fail |= CLRCF_FORMAT;
     }

     if (failed)
          *failed = fail;

     if (fail)
     {
          return DFB_UNSUPPORTED;
     }

     D_DEBUG( "xcode DFB driver XcodeSdVbi0TestRegion exit:\n" );

     return DFB_OK;
}

/***************************************************************************************/


/****************************************************************************************************************************/

static DFBResult
XcodeSdVbi0FlipRegion( CoreLayer             *layer,
                 void                  *driver_data,
                 void                  *layer_data,
                 void                  *region_data,
                 CoreSurface           *surface,
                 DFBSurfaceFlipFlags    flags,
                 CoreSurfaceBufferLock *lock )
{
     DFBResult              ret;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "XcodeSdVbi0FlipRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( mdrv != NULL );

     D_DEBUG( "( 0x%08lx [%d] 0x%04x)\n", 
                 lock->phys, lock->pitch, flags);

     dfb_surface_flip( surface, false );

     D_DEBUG( "XcodeSdVbi0FlipRegion exit \n" );
     return DFB_OK;
}

/****************************************************************************************************************************/


/****************************************************************************************************************************/

static DFBResult
XcodeSdVbi0UpdateRegion( CoreLayer             *layer,
                   void                  *driver_data,
                   void                  *layer_data,
                   void                  *region_data,
                   CoreSurface           *surface,
                   const DFBRegion       *update,
                   CoreSurfaceBufferLock *lock )
{

     D_DEBUG( "XcodeSdVbi0UpdateRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( driver_data != NULL );
     D_ASSERT( layer_data != NULL );
     D_ASSERT( update != NULL );
     DFBResult ret;

     ret = Xcode_Sd_Vbi_Plane_Update_Region(driver_data, layer_data, surface, update);
  
     D_DEBUG( "XcodeSdVbi0UpdateRegion exit \n" );
     
     return ret;
}

/**********************************************************************************************************************/

DFBResult Xcode_Sd_Vbi_Plane_Update_Region(void *driver_data,
                                        void *layer_data,
                                        CoreSurface *surface,
                                        const DFBRegion *update)
{
    u32 free_entries;
    u32 region_index = 0;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32 tmp;
    DFBResult ret; 

    D_DEBUG("region calculate\n");

    D_DEBUG("update->y1 = %d\n", update->y1);
    D_DEBUG("update->y2 = %d\n", update->y2);
    D_DEBUG("update->x1 = %d\n", update->x1);
    D_DEBUG("update->x2 = %d\n", update->x2);

    D_DEBUG("m_layer_data->flip_height = %d\n", m_layer_data->flip_height);

    region_index = update->y1 / m_layer_data->flip_height;

    D_DEBUG("region_index = %d\n", region_index);

    if (region_index < MAX_VBI_REGION_INDEX )
    {
        //Get frame index
        int curr_frame_index = (region_index & ~0x1);

        //If index doesn't change, return OK immediately
        if(curr_frame_index == m_layer_data->last_frame_index)
        {
            return DFB_OK;
        }

        if(mdrv->hdtosd_fd != -1)
        {
            app_ioctl_command app_cmd;

            //Check there is enough room
            app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_GET_SD_VBI_QUEUE_SIZE;
            ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
            
            if(app_cmd.dfb_sd_size.size < MAX_VBI_REGION_INDEX / 2)
            {
                //Check if only one field is updated, we need to duplicate the other half
                if(update->y2 - update->y1 <= m_layer_data->flip_height && m_layer_data->last_frame_index >= 0)
                {
                    if(region_index == curr_frame_index)
                    {
                        //Top field
                        tmp = m_layer_data->last_frame_index + 1;
                        DFBRectangle rect;
                        rect.x = update->x1;
                        rect.y = tmp * m_layer_data->flip_height;
                        rect.w = update->x2 - update->x1 + 1;
                        rect.h = m_layer_data->flip_height;
                        dfb_gfx_copy_to(surface, surface, &rect, update->x1, update->y1 + m_layer_data->flip_height, true);
                        dfb_gfxcard_sync();
                    }
                    else
                    {
                        //Bottom field
                        tmp = m_layer_data->last_frame_index;
                        DFBRectangle rect;
                        rect.x = update->x1;
                        rect.y = tmp * m_layer_data->flip_height;
                        rect.w = update->x2 - update->x1 + 1;
                        rect.h = m_layer_data->flip_height;
                        dfb_gfx_copy_to(surface, surface, &rect, update->x1, update->y1 - m_layer_data->flip_height, true);
                        dfb_gfxcard_sync();
                        region_index--;
                    }
                }

                //Save last index
                m_layer_data->last_frame_index = curr_frame_index;

                //Set flipping index
                app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_SET_SD_VBI_FLIP_INDEX;
                app_cmd.dfb_sd_index.index = curr_frame_index / 2;
                ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 

                //Update total submitted frames
                m_layer_data->total_submitted_frames++;

                return DFB_OK;
            }

        }

        return DFB_FAILURE;
    }

    return DFB_INVAREA;
}



/**********************************************************************************************************************/

static DFBResult XcodeSdVbi0RemoveRegion( CoreLayer *layer,
                                        void      *driver_data,
                                        void      *layer_data,
					void      *region_data )
{
    DFBResult ret;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     
    D_DEBUG( "XcodeSdVbi0RemoveRegion enter \n" );
 
    disable_sd_vbi_output(mdrv);
    D_DEBUG( "disable sd vbi0 output \n" );
    
    D_DEBUG( "XcodeSdVbi0RemoveRegion exit \n" );

    return  DFB_OK;

}

static DFBResult XcodeSdVbi0ShutdownLayer( CoreLayer *layer,
                                        void       *driver_data,
                                        void       *layer_data )
{
    DFBResult ret;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
 
    D_DEBUG( "XcodeSdVbi0ShutdownLayer enter \n" );
    
    disable_sd_vbi_output(mdrv);
    D_DEBUG( "disable sd vbi0 output \n" );
 
    D_DEBUG( "XcodeSdVbi0ShutdownLayer exit \n" );

    return DFB_OK;

}

static DFBResult XcodeSdVbi0DeallocateSurface( CoreLayer *layer,
                                        void      *driver_data,
                                        void      *layer_data,
                                        CoreLayerRegionConfig *config,
					                    CoreSurface   *surface )
{
    DFBResult ret;
    CoreSurfaceConfig conf;
     
    D_DEBUG( " XcodeSdVbi0DeallocateSurface enter \n" );

    if ((surface) &&
        (dfb_config->preallocate_video0_buffer_number))
    {
        //if the buffer size is not change, don't have to reallocate surface
        conf.flags  = CSCONF_SIZE | CSCONF_FORMAT | CSCONF_CAPS;
        conf.size.w = config->width;
        conf.size.h = config->height;
        conf.format = config->format;
        conf.caps   = DSCAPS_VIDEOONLY;

        ret = dfb_surface_arrange_again( surface, CSAID_LAYER10, CSAF_WRITE|CSAF_READ, &conf );
    }
    else
    {
        ret = DFB_FAILURE;
    }
 
    D_DEBUG( " XcodeSdVbi0DeallocateSurface exit \n" );

    return ret;

}


static DFBResult
XcodeSdVbi0GetCurrentFrameConsume( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int       *frame_consume )
{
    DFBResult ret = DFB_FAILURE;

    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     
    D_DEBUG( "XcodeSdVbi0GetCurrentFrameConsume enter \n" );

    if(mdrv->hdtosd_fd != -1)
    {
        app_ioctl_command app_cmd;

        int current_frame_counter;

        int frames_to_free;

        app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_GET_SD_VBI_QUEUE_SIZE;
        
        ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 

        current_frame_counter = m_layer_data->total_submitted_frames - app_cmd.dfb_sd_size.size;

        //reserve last frame
        if(current_frame_counter > 0)
        {
            current_frame_counter--;
        }

        frames_to_free = current_frame_counter - m_layer_data->last_frame_counter;

        if(frames_to_free < 0)
        {
            frames_to_free = 0;
        }

        m_layer_data->last_frame_counter = current_frame_counter;

        m_layer_data->total_return_frame_to_free += frames_to_free;

        //return consumed fields
        *frame_consume = frames_to_free * 2;

        ret = DFB_OK;
    }

    D_DEBUG( "XcodeSdVbi0GetCurrentFrameConsume exit \n" );

    return ret;
}


static DFBResult
XcodeSdVbi0ClearCurrentFrameConsume( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data)
{
    DFBResult ret;

    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     
    D_DEBUG( "XcodeSdVbi0ClearCurrentFrameConsume enter \n" );

    m_layer_data->last_frame_index = -1;
    m_layer_data->last_frame_counter = 0;
    m_layer_data->total_submitted_frames = 0;
    m_layer_data->total_return_frame_to_free = 0;

    if(mdrv->hdtosd_fd != -1)
    {
        app_ioctl_command app_cmd;

        app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_CLR_SD_VBI_QUEUE;
        
        ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
    }

    D_DEBUG( "XcodeSdVbi0ClearCurrentFrameConsume exit \n" );

    return ret;
}


const DisplayLayerFuncs XcodeSdVbi0LayerFuncs = {
     LayerDataSize:      XcodeSdVbiLayerDataSize,
     InitLayer:          XcodeSdVbi0InitLayer,
     TestRegion:         XcodeSdVbi0TestRegion,
     SetRegion:          XcodeSdVbi0SetRegion,
     FlipRegion:         XcodeSdVbi0FlipRegion,
     UpdateRegion:       XcodeSdVbi0UpdateRegion,
     ShutdownLayer:      XcodeSdVbi0ShutdownLayer,
     RemoveRegion:       XcodeSdVbi0RemoveRegion,
     DeallocateSurface:  (void *)XcodeSdVbi0DeallocateSurface,
     GetCurrentFrameConsume: XcodeSdVbi0GetCurrentFrameConsume,
     ClearFrameConsume: XcodeSdVbi0ClearCurrentFrameConsume,
};

