 /************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_video_vid0.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/


#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include <pthread.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/layers_internal.h>
#include <core/surface.h>
#include <core/palette.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"
#include "xcioctl.h"

extern void CSC_ProgAdjustMatrix(pxc_dfb_driver_data mdrv, int layer_id);

extern int xcode4_g2d_scale_and_rotate ( xc_driver_data *mdrv,
                                         int srcPhys, int dfbSrcFmt, int srcX, int srcY, int srcW, int srcH, int srcPitch,
                                         int dstPhys, int dfbDstFmt, int dstX, int dstY, int dstW, int dstH, int dstPitch,
                                         int rotation );

extern DFBResult XcodeFlipSDFrame(CoreLayer *layer, 
                                         const DFBRegion *update,
                                         CoreSurfaceBufferLock *lock);

extern int XcodeSDFullness(CoreLayer *layer);

extern DFBResult xcode4_flip_interlaced_video(void                  *driver_data,
                                                                                       void                  *layer_data, 
                                                                                       const DFBRegion * region, 
                                                                                       u32 * disp_hd_vid_register_table);

extern DFBResult convert_dfb_format_to_disp_format(const CoreLayerRegionConfig *config, 
                                            int * disp_format);

extern struct display_mode_struct * xc4_find_dve_mode(pxc_dfb_driver_data mdrv);

extern void Calculate_video_sca_inv_ratio(struct display_mode_struct * mode, //output video mode pointer
                                   CoreLayerRegionConfig    * config, //input video 
                                   u32 * video_vsca_inv_ratio,
                                   u32 * video_hsca_inv_ratio,
                                   u32 * vid_vsca_init0,
                                   u32 * vid_vsca_init1,
                                   u32 * vid_vsca_start,
                                   u32 * vid_hsca_init,
                                   u32 * vid_hsca_start,
                                   u32 x_round_down);

extern DFBResult
Xcode4SetScalerRamTable(void                       *driver_data,
                        void                       *layer_data,
                        CoreLayerRegionConfig      *config,
                        struct display_mode_struct   * mode,
                        u32 * disp_hd_vid_register_table,
                        DFBSurfacePixelFormat chroma_in,
                        colorModeParameters *colorParms);

extern DFBResult Set_Layer_Level( void *driver_data,
                 int level,
                 u32 reg_left_shift,
                 u32 reg_mask);

extern inline int Get_Layer_Level( void *driver_data,
                  u32 reg_right_shift,
                  u32 reg_mask);

static u32 disp_hd_vid0_register_table[SIZE_OF_DISP_VID_REGISTER_TABLE];

#define MAX_QUEUE_OVL_FRAMES 5

#define MAX_VID_FIFO_ENTRIES 16

#define FRAMES_TO_HOLD 1

extern int Background_Layer_Alpha_On(pxc_dfb_driver_data mdrv);

extern void Virtual_Layer_Combine_To_Physical_Hd_Gra0(pxc_dfb_driver_data    mdrv,
                                               void                  *layer_data,
                                               CoreSurfaceBufferLock *lock,
                                               int layer_id,
                                               DFBSurfaceFlipFlags flags);

//#define DISP_HIST_VID0_ENABLE

/**********************************************************************************************************************/
void Init_Disp_Vid0_Register_Table()
{
     //set register table
     disp_hd_vid0_register_table[DISP_HD_VID_FORMAT_INDEX] = DISP_HD_VID0_FORMAT;
     disp_hd_vid0_register_table[DISP_HD_VID_BUF_PITCH_INDEX] = DISP_HD_VID0_BUF_PITCH;
     disp_hd_vid0_register_table[DISP_HD_VID_BUF_SIZE_INDEX] = DISP_HD_VID0_BUF_SIZE;
     disp_hd_vid0_register_table[DISP_HD_VID_SIZE_INDEX] = DISP_HD_VID0_SIZE;
     disp_hd_vid0_register_table[DISP_HD_VID_SAMPLES_INDEX] = DISP_HD_VID0_SAMPLES;
     disp_hd_vid0_register_table[DISP_HD_VID_SCALED_SIZE_INDEX] = DISP_HD_VID0_SCALED_SIZE;
     disp_hd_vid0_register_table[DISP_HD_COMP_VID_POS_INDEX] = DISP_HD_COMP_VID0_POS;
     disp_hd_vid0_register_table[DISP_HD_VID_VSCA_INV_RATIO_INDEX] = DISP_HD_VID0_VSCA_INV_RATIO;
     disp_hd_vid0_register_table[DISP_HD_VID_HSCA_INV_RATIO_INDEX] = DISP_HD_VID0_HSCA_INV_RATIO;
     disp_hd_vid0_register_table[DISP_HD_VID_VSCA_INIT0_INDEX] = DISP_HD_VID0_VSCA_INIT0;
     disp_hd_vid0_register_table[DISP_HD_VID_VSCA_INIT1_INDEX] = DISP_HD_VID0_VSCA_INIT1;
     disp_hd_vid0_register_table[DISP_HD_VID_VSCA_START_INDEX] = DISP_HD_VID0_VSCA_START;
     disp_hd_vid0_register_table[DISP_HD_VID_VSCALER_Y_BASE_ADDRESS_INDEX] = DISP_HD_VID0_VSCALER_Y_BASE_ADDRESS;
     disp_hd_vid0_register_table[DISP_HD_VID_VSCALER_CBCR_BASE_ADDRESS_INDEX] = DISP_HD_VID0_VSCALER_CBCR_BASE_ADDRESS;
     disp_hd_vid0_register_table[DISP_HD_VID_HSCALER_Y_BASE_ADDRESS_INDEX] = DISP_HD_VID0_HSCALER_Y_BASE_ADDRESS;
     disp_hd_vid0_register_table[DISP_HD_VID_HSCALER_CBCR_BASE_ADDRESS_INDEX] = DISP_HD_VID0_HSCALER_CBCR_BASE_ADDRESS;
     disp_hd_vid0_register_table[DISP_HD_COMP_ALPHA_VID_SHIFT_INDEX] = ALPHA_V0_SHIFT;
     disp_hd_vid0_register_table[DISP_HD_VID_HSBC0_INDEX] = DISP_HD_VID0_HSBC0;
     disp_hd_vid0_register_table[DISP_HD_VID_HSBC1_INDEX] = DISP_HD_VID0_HSBC1;
     disp_hd_vid0_register_table[DISP_HD_VID_HSBC2_INDEX] = DISP_HD_VID0_HSBC2;
     disp_hd_vid0_register_table[DISP_HD_VID_Y_ADR_INDEX] = DISP_HD_VID0_Y_ADR;
     disp_hd_vid0_register_table[DISP_HD_VID_CB_ADR_INDEX] = DISP_HD_VID0_CB_ADR;
     disp_hd_vid0_register_table[DISP_HD_VID_CR_ADR_INDEX] = DISP_HD_VID0_CR_ADR;
     disp_hd_vid0_register_table[ALPHA_VID_MASK_INDEX] = ALPHA_V0_MASK;

     disp_hd_vid0_register_table[DISP_HD_VID_HSCA_INIT_INDEX] = DISP_HD_VID0_HSCA_INIT;
     disp_hd_vid0_register_table[DISP_HD_VID_HSCA_START_INDEX] = DISP_HD_VID0_HSCA_START;
     disp_hd_vid0_register_table[DISP_HD_CMD_FIFO_INDEX] = DISP_HD_VID0_CMD0_FIFO;
     disp_hd_vid0_register_table[DISP_HD_CURR_CMD_INDEX] = DISP_HD_CURR_CMD0;
     disp_hd_vid0_register_table[DISP_HD_VID_FREE_ENTRIES_MASK_INDEX] = VID0_FREE_ENTRIES_MASK;
     disp_hd_vid0_register_table[DISP_HD_VID_FREE_ENTRIES_SHIFT_INDEX] = VID0_FREE_ENTRIES_SHIFT;
     disp_hd_vid0_register_table[DISP_HD_VID_FORMAT_MAX_REQ_LINE_INDEX] = DEFAULT_VID0_MAX_REQ_LINE;
     disp_hd_vid0_register_table[DISP_HD_VID_VERTICAL_SCALER_TAP_INDEX] = VID0_VERTICAL_SCALER_TAPS;
     disp_hd_vid0_register_table[DISP_HD_COMP_LUMA_KEY_INDEX] = DISP_HD_COMP_LUMA_KEY0;
     disp_hd_vid0_register_table[VID_EMPTY_MASK_INDEX] = VID0_EMPTY_MASK;
     disp_hd_vid0_register_table[HD_CC_VID_MODE_SHIFT_INDEX] = VID0_MODE_SHIFT;
     disp_hd_vid0_register_table[HD_CC_VID_MODE_MASK_INDEX] = VID0_MODE_MASK;
     
    disp_hd_vid0_register_table[DISP_HD_VID_INPUT_Y_LIMIT_INDEX]        =   DISP_HD_VID0_INPUT_Y_LIMIT;
    disp_hd_vid0_register_table[DISP_HD_VID_INPUT_Y_PLUS_MASK_INDEX]    =   DISP_HD_VID0_INPUT_Y_LIMIT_Y_LIMIT_PLUS_MASK;
    disp_hd_vid0_register_table[DISP_HD_VID_INPUT_Y_MINUS_MASK_INDEX]   =   DISP_HD_VID0_INPUT_Y_LIMIT_Y_LIMIT_MINUS_MASK;
    disp_hd_vid0_register_table[DISP_HD_VID_INPUT_Y_PLUS_SHIFT_INDEX]   =   DISP_HD_VID0_INPUT_Y_LIMIT_Y_LIMIT_PLUS_SHIFT;
    disp_hd_vid0_register_table[DISP_HD_VID_INPUT_Y_MINUS_SHIFT_INDEX]  =   DISP_HD_VID0_INPUT_Y_LIMIT_Y_LIMIT_MINUS_SHIFT;
    
    disp_hd_vid0_register_table[DISP_HD_VID_INPUT_CBCR_LIMIT_INDEX]        =   DISP_HD_VID0_INPUT_CBCR_LIMIT;
    disp_hd_vid0_register_table[DISP_HD_VID_INPUT_CBCR_PLUS_MASK_INDEX]    =   DISP_HD_VID0_INPUT_CBCR_LIMIT_CBCR_LIMIT_PLUS_MASK;
    disp_hd_vid0_register_table[DISP_HD_VID_INPUT_CBCR_MINUS_MASK_INDEX]   =   DISP_HD_VID0_INPUT_CBCR_LIMIT_CBCR_LIMIT_MINUS_MASK;
    disp_hd_vid0_register_table[DISP_HD_VID_INPUT_CBCR_PLUS_SHIFT_INDEX]   =   DISP_HD_VID0_INPUT_CBCR_LIMIT_CBCR_LIMIT_PLUS_SHIFT;
    disp_hd_vid0_register_table[DISP_HD_VID_INPUT_CBCR_MINUS_SHIFT_INDEX]  =   DISP_HD_VID0_INPUT_CBCR_LIMIT_CBCR_LIMIT_MINUS_SHIFT;
    
    
}

/**********************************************************************************************************************/

static DFBResult
XcodeVid0InitLayer( CoreLayer                *layer,
                    void                       *driver_data,
                    void                       *layer_data,
                    DFBDisplayLayerDescription *description,
                    DFBDisplayLayerConfig      *config,
                    DFBColorAdjustment         *adjustment )
{
     u32     tmp;
     pthread_mutexattr_t mutex_attr;

     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "xcode DFB driver XcodeVid0InitLayer enter:\n" );


     /* set capabilities and type */
     description->caps = DLCAPS_SURFACE | DLCAPS_ALPHACHANNEL | DLCAPS_OPACITY | 
                         DLCAPS_SCREEN_POSITION | DLCAPS_SCREEN_LOCATION | DLCAPS_LEVELS;

     description->type = DLTF_VIDEO;

     /* set name */
     snprintf( description->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "ViXS Xcode4 Video Plane 0" );

     /* fill out the default configuration */
     config->flags       = DLCONF_WIDTH       | DLCONF_HEIGHT |
                           DLCONF_PIXELFORMAT | DLCONF_BUFFERMODE | DLCONF_OPTIONS | DLCONF_SURFACE_CAPS ;

     if (dfb_config->preallocate_video0_buffer_number)
     {
         config->width       = DEFAULT_VIDEO_BUFFER_WIDTH;
         config->height      = DEFAULT_VIDEO_BUFFER_HEIGHT * dfb_config->preallocate_video0_buffer_number;
     }
     else
     {
         config->width       = XCODE_DEFAULT_WIDTH;
         config->height      = XCODE_DEFAULT_HEIGHT;
     }


     config->pixelformat = DSPF_I420;
     config->buffermode  = DLBM_FRONTONLY;
     config->surface_caps |= DSCAPS_STATIC_ALLOC ;
     config->options     |= DLOP_OPACITY;

     adjustment->flags   = DCAF_ALL;
     adjustment->brightness = LAYER_DEFAULT_BRIGHT;
     adjustment->contrast = LAYER_DEFAULT_CONTRAST;
     adjustment->saturation = LAYER_DEFAULT_SATURATION;
     adjustment->hue = LAYER_DEFAULT_HUE;
     adjustment->luma_filtertaps    =   10;
     adjustment->chroma_filtertaps  =   10;
     
     /* No sharpness modifications */
     adjustment->sharpness  =   adjustment->hsharpness  =   adjustment->vsharpness  =
     adjustment->combine_sharpness  =   LAYER_DEFAULT_SHARPNESS_VALUE;
     
     /* Softens are kept at 1.0 */
     adjustment->luma_soften    =   adjustment->chroma_soften   =   LAYER_DEFAULT_SOFTEN_VALUE;
     
     /* Gains are kept at 1.0 */
     adjustment->luma_gain  =   adjustment->chroma_gain =   LAYER_DEFAULT_GAIN_VALUE;
     
     
     
     m_layer_data->last_frame_counter = 0;
     m_layer_data->total_submitted_frames = 0;
     m_layer_data->total_return_frame_to_free = 0;
     m_layer_data->status = 0;
     m_layer_data->layer_set_return_fail_counter = 0;
     m_layer_data->last_drop_flag = 0;
     m_layer_data->last_frame_index = -1;

     pthread_mutexattr_init( &mutex_attr );
     pthread_mutexattr_setpshared( &mutex_attr, PTHREAD_PROCESS_SHARED );
     pthread_mutex_init(&m_layer_data->layer_mutex, &mutex_attr);

     //set register table

    Init_Disp_Vid0_Register_Table();
     
     //Color Adjustment part (default adjustment parts)
     m_layer_data->colorParams.premulty =   1.0f;
     m_layer_data->colorParams.premultc =   1.0f;
     
     m_layer_data->colorParams.vcsoften  =   1.0f;
     m_layer_data->colorParams.hcsoften  =   1.0f;
     m_layer_data->colorParams.vysoften  =   1.0f;
     m_layer_data->colorParams.hysoften  =   1.0f;
     
     m_layer_data->colorParams.vcsharpen  =   0.0f;
     m_layer_data->colorParams.hcsharpen  =   0.0f;
     m_layer_data->colorParams.vysharpen  =   0.0f;
     m_layer_data->colorParams.hysharpen  =   0.0f;
     
     m_layer_data->colorParams.ntaps_up_c  =   10;
     m_layer_data->colorParams.ntaps_up_y  =   10;
     
     
     

     D_DEBUG( "xcode DFB driver XcodeVid0InitLayer exit:\n" );

     return DFB_OK;
}

static DFBResult
XcodeVid0TestRegion( CoreLayer                  *layer,
                     void                       *driver_data,
                     void                       *layer_data,
                     CoreLayerRegionConfig      *config,
                     CoreLayerRegionConfigFlags *failed )
{
     CoreLayerRegionConfigFlags fail = 0;

     D_DEBUG( "xcode DFB driver XcodeVid0TestRegion enter:\n" );

     D_DEBUG ("surface width %d \n", config->width);
     D_DEBUG ("surface height %d \n", config->height);
     D_DEBUG ("destination x %d \n", config->dest.x);
     D_DEBUG ("destination y %d \n", config->dest.y);
     D_DEBUG ("destination weight %d \n", config->dest.w);
     D_DEBUG ("destination height %d \n", config->dest.h);
     D_DEBUG ("surface caps %d \n", config->surface_caps);
     D_DEBUG ("surface options %d \n", config->options);

     if (config->options & ~XCODE4_VID_SUPPORTED_OPTIONS)
          fail |= CLRCF_OPTIONS;

     switch (config->format) {
          case DSPF_I420:
          case DSPF_YV12:
          case DSPF_YV16:  //replaced DSPF_UYVY
          case DSPF_YUY2://TODO: not consistent with DirectFB pixel format.
               break;

          default:
               fail |= CLRCF_FORMAT;
     }

     if (failed)
          *failed = fail;

     if (fail)
     {
          return DFB_UNSUPPORTED;
     }

     D_DEBUG( "xcode DFB driver XcodeVid0TestRegion exit:\n" );

     return DFB_OK;
}

/***************************************************************************************/

DFBResult
Vid_Set_Color_Adjustment( void               *driver_data,
                          void               *layer_data,
                          DFBColorAdjustment *adj,
                          u32  * disp_hd_vid_register_table)
{

    u32 tmp;
    float sin_hue;
    float cos_hue;
    double hue_radian;
    int int_sin_hue;
    int int_cos_hue;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32 format_bright;
    double  tmpf;
    u32     update_coeff    =   0;
    struct display_mode_struct * display_mode = NULL;

    //set brightness and constrast
    tmp = REG_READ( disp_hd_vid_register_table[DISP_HD_VID_HSBC0_INDEX]);

    if ((adj->flags & DCAF_BRIGHTNESS) &&
        (adj->brightness <= (DISP_HD_VID0_HSBC0_BRIGHT_MASK >> DISP_HD_VID0_HSBC0_BRIGHT_SHIFT)))
    {
        //the format brightness should -511 to 511
        tmp = tmp & (~DISP_HD_VID0_HSBC0_BRIGHT_MASK);
        if (adj->brightness >= LAYER_DEFAULT_BRIGHT)
        {
            format_bright = (adj->brightness - LAYER_DEFAULT_BRIGHT) & (DISP_HD_VID0_HSBC0_BRIGHT_MASK >> DISP_HD_VID0_HSBC0_BRIGHT_SHIFT);

        }
        else
        {
           	format_bright = BRIGHTNESS_SIGN | adj->brightness; //assissign the sign bit
        }
        tmp = tmp | (format_bright << DISP_HD_VID0_HSBC0_BRIGHT_SHIFT);
    }

    if ((adj->flags & DCAF_CONTRAST)  &&
        (adj->contrast <= (DISP_HD_VID0_HSBC0_CONTRAST_MASK >> DISP_HD_VID0_HSBC0_CONTRAST_SHIFT)))
    {
        tmp = tmp & (~DISP_HD_VID0_HSBC0_CONTRAST_MASK);
        tmp = tmp | (adj->contrast  << DISP_HD_VID0_HSBC0_CONTRAST_SHIFT);
    }

    REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_HSBC0_INDEX],
              tmp);

    //set saturation
    tmp = REG_READ( disp_hd_vid_register_table[DISP_HD_VID_HSBC2_INDEX]);

    if ((adj->flags & DCAF_SATURATION) &&
        (adj->saturation <= (DISP_HD_VID0_HSBC2_SAT_MASK >> DISP_HD_VID0_HSBC2_SAT_SHIFT)))
    {
        tmp = tmp & (~DISP_HD_VID0_HSBC2_SAT_MASK);
        tmp = tmp | (adj->saturation << DISP_HD_VID0_HSBC2_SAT_SHIFT);

        REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_HSBC2_INDEX], tmp);

    }

    //set hue
    if (adj->flags & DCAF_HUE )
    {
        
        hue_radian = adj->hue * PI / 180 ;
        sin_hue = sin(hue_radian);
        cos_hue = abs(cos(hue_radian));

        int_cos_hue = cos_hue * (1 << HUE_LEFT_SHIFT) ;
        int_cos_hue = int_cos_hue & DISP_HD_VID0_HSBC1_HUE_COS_MASK;

        int_sin_hue = sin_hue * (1 << HUE_LEFT_SHIFT); 
        int_sin_hue = int_sin_hue & (DISP_HD_VID0_HSBC1_HUE_SIN_MASK >> DISP_HD_VID0_HSBC1_HUE_SIN_SHIFT);

        REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_HSBC1_INDEX],
                  int_cos_hue | (int_sin_hue << DISP_HD_VID0_HSBC1_HUE_SIN_SHIFT));
    }
    
    if (adj->flags & DCAF_LUMA_FILT_TAP_INT)
    {
        if (adj->luma_filtertaps > 10)
        {
            m_layer_data->colorParams.ntaps_up_y    =   10;
        }
        else
        {
            m_layer_data->colorParams.ntaps_up_y    =   adj->luma_filtertaps;
        }
        update_coeff    =   1;
    }
    
    if (adj->flags & DCAF_CHROMA_FILT_TAP_INT)
    {
        if (adj->chroma_filtertaps > 10)
        {
            m_layer_data->colorParams.ntaps_up_c    =   10;
        }
        else
        {
            m_layer_data->colorParams.ntaps_up_c    =   adj->chroma_filtertaps;
        }
        update_coeff    =   1;
    }
    
    if (adj->flags & DCAF_HSHARPNESS_U4P12)
    {
        //4.12 fixed point
        tmpf = ((double)adj->hsharpness) * (1./(4096.0));
        m_layer_data->colorParams.hysharpen =  tmpf; 
        m_layer_data->colorParams.hcsharpen =  tmpf;
        update_coeff    =   1;
    }
    
    if (adj->flags & DCAF_VSHARPNESS_U4P12)
    {
        //4.12 fixed point
        tmpf = ((double)adj->vsharpness) * (1./(4096.0));
        m_layer_data->colorParams.vysharpen =  tmpf; 
        m_layer_data->colorParams.vcsharpen =  tmpf;
        update_coeff    =   1;
    }

    if (adj->flags & DCAF_LUMA_GAIN_U4P12)
    {
        //4.12 fixed point
        tmpf = ((double)adj->luma_gain) * (1./(4096.0));
        m_layer_data->colorParams.premulty =  tmpf; 
        update_coeff    =   1;
    }
    
    if (adj->flags & DCAF_CHROMA_GAIN_U4P12)
    {
        //4.12 fixed point
        tmpf = ((double)adj->chroma_gain) * (1./(4096.0));
        m_layer_data->colorParams.premultc =  tmpf; 
        update_coeff    =   1;
    }
    
    if (adj->flags & DCAF_LUMA_SOFTEN_U0P16)
    {
        //0.16 fixed point
        tmpf = ((double)adj->luma_soften) * (1./(65535.0));
        m_layer_data->colorParams.hysoften =  tmpf; 
        update_coeff    =   1;
    }
    
    if (adj->flags & DCAF_LUMA_SOFTEN_U0P16)
    {
        //0.16 fixed point
        tmpf = ((double)adj->chroma_soften) * (1./(65535.0));
        m_layer_data->colorParams.hcsoften =  tmpf; 
        update_coeff    =   1;
    }
    
    if (update_coeff)
    {
        /* Get the mode */
        display_mode    =   xc4_find_dve_mode(mdrv);
        
        
        pthread_mutex_lock( &mdrv->shared->reentrancy_mutex );
        Xcode4SetScalerRamTable(driver_data, layer_data, &m_layer_data->config, display_mode, 
                                        disp_hd_vid_register_table, m_layer_data->config.format, 
                                        &m_layer_data->colorParams);
        pthread_mutex_unlock( &mdrv->shared->reentrancy_mutex );
    }
    
    
    return DFB_OK;
}


/***************************************************************************************/

DFBResult Xcode_Video_Plane_Update_Region(void                       *driver_data,
                                          void                       *layer_data,
                                          const DFBRegion       *update,
                                          u32 * disp_hd_vid_register_table)
{
    u32 free_entries;
    u32 region_index = 0;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32 tmp;
    DFBResult ret; 
    u32  y_buf_size;
    u32  c_buf_size;
#ifdef DISP_HIST_VID0_ENABLE
    u32 regValue;
#endif 

#if 0
    if (m_layer_data->flip_height != (update->y2 - update->y1))
    {
        m_layer_data->flip_height = update->y2 - update->y1;

        y_buf_size = (m_layer_data->y_pitch * m_layer_data->flip_height) / Y_BUF_SIZE_ALIGNMENT ;

        if ((m_layer_data->format == DSPF_YV12) || //4:2:0
            (m_layer_data->format == DSPF_I420)) //4:2:0
        {

            c_buf_size = (m_layer_data->c_pitch * (m_layer_data->flip_height / 2)) / C_BUF_SIZE_ALIGNMENT;
        }
        else  //4:2:2
        {
            c_buf_size = (m_layer_data->c_pitch * m_layer_data->flip_height) / C_BUF_SIZE_ALIGNMENT; 
        }

        if ((m_layer_data->surface_caps & DSCAPS_VIXS_INTERLACED) ||
            (m_layer_data->surface_caps & DSCAPS_SEPARATED))
        {
            y_buf_size = y_buf_size / 2;
            c_buf_size = c_buf_size / 2;
        }

        REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_BUF_SIZE_INDEX], 
                  ((c_buf_size << DISP_HD_VID0_BUF_SIZE_C_SHIFT) | y_buf_size));

    }
#endif


    free_entries = ((REG_READ(DISP_HD_FIFO_STATUS0)) & disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_MASK_INDEX])
                   >> disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_SHIFT_INDEX];

     
    if  ((free_entries > 0) ||
         (REG_READ(DISP_HD_FIFO_STATUS0) & disp_hd_vid_register_table[VID_EMPTY_MASK_INDEX]))
    {
        if ((m_layer_data->field_select == PROGRESSIVE_FIELD_SEL) ||
            (m_layer_data->field_select == INTERLACED_INTERWOVEN_BOTH_FIELD_INTERLEAVE))
        {
            region_index =  update->y1 / m_layer_data->flip_height;
        }
        else
        {
            region_index =  update->y1 / (m_layer_data->flip_height / 2);
        }

        if (region_index <= MAX_FRAME_INDEX)
        {
            //the current algorithm needs two calls per frame, one indicating the 
            // top field index and the other the bottom field index. However, when
            // we have a progressive output mode, only one call is needed.

                //Add new index
            if (m_layer_data->field_select == PROGRESSIVE_FIELD_SEL)
            {

                tmp = UPDATE_VID0_REGS_MASK |
                      DISP_HD_VID0_CMD0_FIFO_UPDATE_COMP_REGS_MASK |
                      VID0_CMD0_FIFO_WE_MASK |
                      (m_layer_data->field_select << HD_VID0_FLD_SEL_SHIFT) |
                      (region_index << HD_VID0_TOP_IDX_SHIFT) |
                      (region_index << HD_VID0_BOT_IDX_SHIFT);

#ifdef DISP_HIST_VID0_ENABLE
                regValue = UPDATE_HIST_REGS_MASK |
                      DISP_HD_VID0_CMD3_FIFO_UPDATE_COMP_REGS_MASK |
                      VID0_CMD3_FIFO_WE_MASK |
                      (m_layer_data->field_select << HD_VID0_FLD_SEL_SHIFT) |
                      (region_index << HD_VID0_TOP_IDX_SHIFT) |
                      (region_index << HD_VID0_BOT_IDX_SHIFT);
#endif 
            }
            else if (m_layer_data->field_select == INTERLACED_INTERWOVEN_BOTH_FIELD_INTERLEAVE)
            {
                tmp = UPDATE_VID0_REGS_MASK |
                      DISP_HD_VID0_CMD0_FIFO_UPDATE_COMP_REGS_MASK |
                      VID0_CMD0_FIFO_WE_MASK |
                      (INTERLACED_INTERWOVEN_BOTH_FIELD << HD_VID0_FLD_SEL_SHIFT) |
                      (region_index << HD_VID0_TOP_IDX_SHIFT) |
                      (region_index << HD_VID0_BOT_IDX_SHIFT);

#ifdef DISP_HIST_VID0_ENABLE
                regValue = UPDATE_HIST_REGS_MASK |
                      DISP_HD_VID0_CMD3_FIFO_UPDATE_COMP_REGS_MASK |
                      VID0_CMD3_FIFO_WE_MASK |
                      (INTERLACED_INTERWOVEN_BOTH_FIELD << HD_VID0_FLD_SEL_SHIFT) |
                      (region_index << HD_VID0_TOP_IDX_SHIFT) |
                      (region_index << HD_VID0_BOT_IDX_SHIFT);
#endif 
            }
            else
            {
                if (m_layer_data->field_select == INTERWOVEN_FIELD_SWITCH)
                {
                    tmp = UPDATE_VID0_REGS_MASK |
                          DISP_HD_VID0_CMD0_FIFO_UPDATE_COMP_REGS_MASK |
                          VID0_CMD0_FIFO_WE_MASK |
                          (INTERWOVEN_TOP_FIELD << HD_VID0_FLD_SEL_SHIFT) |
                          (region_index << HD_VID0_TOP_IDX_SHIFT);

#ifdef DISP_HIST_VID0_ENABLE
                regValue = UPDATE_HIST_REGS_MASK |
                      DISP_HD_VID0_CMD3_FIFO_UPDATE_COMP_REGS_MASK |
                      VID0_CMD3_FIFO_WE_MASK |
                      (INTERWOVEN_TOP_FIELD << HD_VID0_FLD_SEL_SHIFT) |
                      (region_index << HD_VID0_TOP_IDX_SHIFT);
#endif 
                }
                else if (m_layer_data->field_select == INTERLACED_INTERWOVEN_BOTH_FIELD)
                {
                    m_layer_data->uovl_field_index[m_layer_data->ufield_update] = region_index;
                    m_layer_data->ufield_update = m_layer_data->ufield_update ^ 1;
                    if (m_layer_data->ufield_update == UPDATE_REGION_TOP_INDEX)
                    {
                        tmp = UPDATE_VID0_REGS_MASK |
                              DISP_HD_VID0_CMD0_FIFO_UPDATE_COMP_REGS_MASK |
                              VID0_CMD0_FIFO_WE_MASK |
                              (INTERLACED_INTERWOVEN_BOTH_FIELD << HD_VID0_FLD_SEL_SHIFT) |
                              (m_layer_data->uovl_field_index[UPDATE_REGION_TOP_INDEX] << HD_VID0_TOP_IDX_SHIFT) |
                              (m_layer_data->uovl_field_index[UPDATE_REGION_BOT_INDEX] << HD_VID0_BOT_IDX_SHIFT);

#ifdef DISP_HIST_VID0_ENABLE
                regValue = UPDATE_HIST_REGS_MASK |
                      DISP_HD_VID0_CMD3_FIFO_UPDATE_COMP_REGS_MASK |
                      VID0_CMD3_FIFO_WE_MASK |
                      (INTERLACED_INTERWOVEN_BOTH_FIELD << HD_VID0_FLD_SEL_SHIFT) |
                      (m_layer_data->uovl_field_index[UPDATE_REGION_TOP_INDEX] << HD_VID0_TOP_IDX_SHIFT) |
                      (m_layer_data->uovl_field_index[UPDATE_REGION_BOT_INDEX] << HD_VID0_BOT_IDX_SHIFT);
#endif 
                    }
                }
            }


            tmp |= (UPDATE_VID0_REGS_MASK |
                     DISP_HD_VID0_CMD0_FIFO_UPDATE_COMP_REGS_MASK |
                     VID0_CMD0_FIFO_WE_MASK );

            if (CURRENT_MODE_IS_3D)
            {
                if (disp_hd_vid_register_table[DISP_HD_CMD_FIFO_INDEX] == DISP_HD_VID0_CMD0_FIFO)
                {
                    if(mdrv->hd_vid0_pending_ptr)
                    {
                        //Write cmd fifo for video 0
                        REG_WRITE(DISP_HD_VID0_CMD0_FIFO, mdrv->hd_vid0_pending_cmd);
                        
                        m_layer_data->total_submitted_frames++;
                    }
                    mdrv->hd_vid0_pending_cmd = tmp;
                    mdrv->hd_vid0_pending_ptr = (void *) m_layer_data;
                }
                else if (disp_hd_vid_register_table[DISP_HD_CMD_FIFO_INDEX] == DISP_HD_VID1_CMD0_FIFO)
                {
                    if(mdrv->hd_vid0_pending_ptr)
                    {
                        pthread_mutex_lock(&((pxc_dfb_layer_data)mdrv->hd_vid0_pending_ptr)->layer_mutex);
                        
                        //Write cmd fifo's for video 1
                        REG_WRITE(DISP_HD_VID1_CMD0_FIFO, tmp);
                        
                        //Write cmd fifo for video 0
                        REG_WRITE(DISP_HD_VID0_CMD0_FIFO, mdrv->hd_vid0_pending_cmd);
                        
                        m_layer_data->total_submitted_frames++;
                        ((pxc_dfb_layer_data)mdrv->hd_vid0_pending_ptr)->total_submitted_frames++;
                        
                        pthread_mutex_unlock(&((pxc_dfb_layer_data)mdrv->hd_vid0_pending_ptr)->layer_mutex);

                        //Clear buffered video 0 clip
                        mdrv->hd_vid0_pending_ptr = NULL;
                    }
                    else
                    {
                        //Write cmd fifo's
                        REG_WRITE(disp_hd_vid_register_table[DISP_HD_CMD_FIFO_INDEX], tmp);
            
                        m_layer_data->total_submitted_frames++;
                    }
                }
            }
            else
            {
                //Write cmd fifo's
                REG_WRITE(disp_hd_vid_register_table[DISP_HD_CMD_FIFO_INDEX], tmp);
    
                m_layer_data->total_submitted_frames++;
            }

            if ((dfb_config->check_vid0_playback) &&
                (disp_hd_vid_register_table[DISP_HD_CMD_FIFO_INDEX] == DISP_HD_VID0_CMD0_FIFO))
            {
                REG_WRITE(GIC1_DUMMY_CF_REG, m_layer_data->total_submitted_frames);
            }
            else if ((dfb_config->check_vid1_playback) &&
                (disp_hd_vid_register_table[DISP_HD_CMD_FIFO_INDEX] == DISP_HD_VID1_CMD0_FIFO))
            {
                REG_WRITE(GIC1_DUMMY_CF_REG, m_layer_data->total_submitted_frames);
            }

#ifdef DISP_HIST_VID0_ENABLE
//	    regValue |= HD_HIST_TRANS_EN_MASK | HD_HIST_GEN_EN_MASK;
	    REG_WRITE(DISP_HD_VID0_CMD3_FIFO, regValue);
#endif 
            ret = DFB_OK;
        }
        else
        {
            ret = DFB_FAILURE;
        }
    }
    else
    {
        ret = DFB_FAILURE;
    }

    return ret ;

}



/***************************************************************************************/

DFBResult Xcode_Set_Video_Plane(CoreLayer                  *layer,
                                void                       *driver_data,
                                void                       *layer_data,
                                CoreLayerRegionConfig      *config,
                                CoreSurfaceBufferLock      *lock,
                                CoreSurface                *surface,
                                CoreLayerRegionConfigFlags  updated,
                                u32 * disp_hd_vid_register_table)
{
     int yuv_422=0;
     u32 y_pitch;
     u32 c_pitch;
     u32 y_buf_size;
     u32 c_buf_size;
     u32 horizontal_size16_delay;
     u32 y_base;
     u32 cb_base;
     u32 cr_base;
     u32 video_vsca_inv_ratio;
     u32 video_hsca_inv_ratio;
     u32 vid_vsca_init0;
     u32 vid_vsca_init1;
     u32 vid_vsca_start;
     u32 vid_hsca_init;
     u32 vid_hsca_start;
     u32 field_select;
     u32 vid_format;
     u32 flip_height;
     u32 x_round_down;
     u32 real_width;

     u32 tmp;
     u8 bg_color_y;
     u8 bg_color_cb;
     u8 bg_color_cr;
     u32 total_c_buf_size;
    
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     struct display_mode_struct * display_mode = NULL;
     pxc_dfb_layer_data bg_layer_data;
     CoreLayer *bg_layer;
     void * temp_ptr;
     u32 low_key;
     u32 force_reset_layer = 0;
     u32 y_addr;
     u32 cb_addr;
     u32 cr_addr;
	 u32 max_line_req;

     u32 video0_screen_width;
     u32 video0_screen_height;
     u32 video1_screen_width;
     u32 video1_screen_height;
     u32 layer_alpha_change = 0;
     int key_high;

     //set the related video  plane
     m_layer_data->layer_is_set = 1;
     
     
     m_layer_data->config   =   *config;
     if ((surface) && (lock))
     {
         lock->pitch = config->width;
     }

     //setup the memory interface
     y_base = lock->phys ;

     if (config->source.x < 16)
     {
         x_round_down = 0;
     }
     else
     {
          x_round_down = config->source.x - (config->source.x % VIDEO_LAYER_LUMA_ADDR_ALIGNMENT);
     }

	 if (START_POS_OUT_OF_RANGE)
	 {
	      x_round_down = x_round_down + VIDEO_LAYER_LUMA_ADDR_ALIGNMENT;
	 }

     y_addr = y_base + (lock->pitch * config->source.y) + x_round_down;


     if ((config->format == DSPF_YV12) || //4:2:0
         (config->format == DSPF_I420) || //4:2:0
         (config->format == DSPF_YV16) || //4:2:2   //DSPF_YV16 replaced DSPF_UYVY
         (config->format == DSPF_YUY2))   //4:2:2  //TODO: not consistent with DirectFB pixel format.
     {
         y_pitch = lock->pitch;

         c_pitch = y_pitch >> 1;

         if (config->flip_height)
         {
             flip_height = config->flip_height;
         }
         else
         {
             SET_NUMBER_ALIGNMENT(flip_height, config->source.h, VIDEO_HEIGHT_ALIGNMENT);
         }


         y_buf_size = (y_pitch * flip_height) / Y_BUF_SIZE_ALIGNMENT ;

         if ((config->format == DSPF_YV12) || //4:2:0
             (config->format == DSPF_I420)) //4:2:0
         {

             c_buf_size = (c_pitch * (flip_height / 2)) / C_BUF_SIZE_ALIGNMENT;
             total_c_buf_size = c_pitch * (config->height >> 1);
             //c_buf_size = (c_pitch * flip_height) >> 7 ; 
         }
         else  //4:2:2
         {
             // c_buf_size = (c_pitch * (flip_height / 2)) * 8 /256;
             c_buf_size = (c_pitch * flip_height) / C_BUF_SIZE_ALIGNMENT; 
             total_c_buf_size = c_pitch * config->height;
         }

         if (config->format == DSPF_I420)
         {
             cb_base = y_base + (y_pitch * config->height) ;
             cr_base = cb_base + (c_pitch * (config->height >> 1));

             cb_addr = cb_base + (c_pitch * (config->source.y / 2)) + (x_round_down / 2) ;
             cr_addr = cr_base + (c_pitch * (config->source.y / 2)) + (x_round_down / 2);
         }
         else if (config->format == DSPF_YV12)
         {
             cr_base = y_base + (y_pitch * config->height) ;
             cb_base = cr_base + (c_pitch * (config->height >> 1));

             cb_addr = cb_base + (c_pitch * (config->source.y / 2)) + (x_round_down / 2);
             cr_addr = cr_base + (c_pitch * (config->source.y / 2)) + (x_round_down / 2);

         }
         else if (config->format == DSPF_YUY2)  //TODO: not consistent with DirectFB pixel format.
         {
             cb_base = y_base + (y_pitch * config->height) ;
             cr_base = cb_base + (c_pitch * config->height);
             yuv_422 = DISP_VID_INPUT_422;

             cb_addr = cb_base + (c_pitch * config->source.y) + (x_round_down / 2);
             cr_addr = cr_base + (c_pitch * config->source.y) + (x_round_down / 2);

         }
         else if (config->format == DSPF_YV16)  //DSPF_YV16 replaced DSPF_UYVY
         {
             cr_base = y_base + (y_pitch * config->height) ;
             cb_base = cr_base + (c_pitch * config->height);
             yuv_422 = DISP_VID_INPUT_422;

             cb_addr = cb_base + (c_pitch * config->source.y) + (x_round_down / 2);
             cr_addr = cr_base + (c_pitch * config->source.y) + (x_round_down / 2);

         }

     }

     if (dfb_layer_id(layer) != BACKGROUND_GRA_LAYER_ID)
          surface->config.size.h = config->height;


     //find output display mode
     display_mode = xc4_find_dve_mode(mdrv);
     
     if (!display_mode)
     {
         return DFB_UNSUPPORTED;
     }
     
     if ((config->surface_caps & DSCAPS_VIXS_INTERLACED) ||
         (config->surface_caps & DSCAPS_SEPARATED))
     {
         if (display_mode->scanmode == DSESM_PROGRESSIVE)
         {
             field_select = INTERLACED_INTERWOVEN_BOTH_FIELD;
         }
         else
         {
             field_select = INTERWOVEN_FIELD_SWITCH;
         }

         vid_format = INTERWOVEN_FORMAT;

     }
     else if ((config->surface_caps & DSCAPS_INTERLACED))
     {
         field_select = INTERLACED_INTERWOVEN_BOTH_FIELD_INTERLEAVE;
         vid_format = INTERWOVEN_FORMAT;
     }
     else
     {
         field_select = PROGRESSIVE_FIELD_SEL;
         vid_format = STACKED_FORMAT;
     }

     if ((force_reset_layer) ||
         (m_layer_data->dest_width != config->dest.w) ||
         (m_layer_data->dest_height != config->dest.h) ||
         (m_layer_data->source_x != config->source.x) ||
         (m_layer_data->source_y != config->source.y) ||
         (m_layer_data->source_width != config->source.w) ||
         (m_layer_data->source_height != config->source.h) ||
         (m_layer_data->flip_height != flip_height) ||
         (m_layer_data->field_select != field_select) ||
         (m_layer_data->format != config->format) ||
         (m_layer_data->alpha != (config->opacity & 0xff)))
     {

         if (m_layer_data->alpha != (config->opacity & 0xff))
         {
             layer_alpha_change = 1;
         }

         //set the vidoe scaler/filter coefficient ram table
         if ((m_layer_data->dest_width != config->dest.w) ||
             (m_layer_data->dest_height != config->dest.h) ||
             (m_layer_data->source_width != config->source.w) ||
             (m_layer_data->source_height != config->source.h))
         {
             pthread_mutex_lock( &mdrv->shared->reentrancy_mutex );
             Xcode4SetScalerRamTable(driver_data, layer_data, config, display_mode, disp_hd_vid_register_table, config->format, &m_layer_data->colorParams);
             pthread_mutex_unlock( &mdrv->shared->reentrancy_mutex );
         }

         m_layer_data->buf_width = config->width;
         m_layer_data->buf_height = config->height;
         m_layer_data->dest_width = config->dest.w;
         m_layer_data->dest_height = config->dest.h;
         m_layer_data->source_x = config->source.x;
         m_layer_data->source_y = config->source.y;
         m_layer_data->source_width = config->source.w;
         m_layer_data->source_height = config->source.h;
         m_layer_data->flip_height = flip_height;
         m_layer_data->field_select = field_select;
         m_layer_data->y_pitch = y_pitch;
         m_layer_data->c_pitch = c_pitch;
         m_layer_data->y_start_addr = y_base;
         m_layer_data->u_start_addr = cb_base;
         m_layer_data->v_start_addr = cr_base;
         m_layer_data->format = config->format;
         surface->config.flip_height = flip_height;
         m_layer_data->y_frame_size = m_layer_data->flip_height * m_layer_data->y_pitch;
         m_layer_data->surface_caps = config->surface_caps;
         m_layer_data->alpha = config->opacity & 0x0ff;
         m_layer_data->x = config->dest.x;
         m_layer_data->y = config->dest.y;

	     if ((config->format == DSPF_I420) || 
             (config->format == DSPF_YV12))
         {
             m_layer_data->c_frame_size = m_layer_data->c_pitch * (m_layer_data->flip_height >> 1);
         }
         else if ((config->format == DSPF_YUY2) ||  //TODO: not consistent with DirectFB pixel format.
                  (config->format == DSPF_YV16))  //DSPF_YV16 replaced DSPF_UYVY
         {
           
             m_layer_data->c_frame_size = m_layer_data->c_pitch * m_layer_data->flip_height;
         }
	 
         if (config->surface_caps & DSCAPS_VIXS_INTERLACED)
         {
             surface->config.caps = surface->config.caps | DSCAPS_VIXS_INTERLACED;
         }

         //Setup the memory interface
		 max_line_req = disp_hd_vid_register_table[DISP_HD_VID_FORMAT_MAX_REQ_LINE_INDEX];
         //Setup the memory interface
         tmp = (max_line_req << DISP_HD_VID0_FORMAT_MAX_LINE_REQ_Y_SHIFT) | 
               (max_line_req << DISP_HD_VID0_FORMAT_MAX_LINE_REQ_C_SHIFT) | 
               (yuv_422 << DISP_HD_VID0_FORMAT_INPUT_422_SHIFT) | 
               (vid_format << DISP_HD_VID0_FORMAT_VID_FORMAT_SHIFT) |
               (DISABLE_RANDOM << DISP_HD_VID0_FORMAT_DITHER_SHIFT) |
               (0x80 << DISP_HD_VID0_FORMAT_URGENT_REQ_FLAG_SHIFT);

         if ((config->surface_caps & DSCAPS_VIXS_INTERLACED) ||
             (config->surface_caps & DSCAPS_SEPARATED))
         {
             if (display_mode->scanmode == DSESM_PROGRESSIVE)
             {
                 tmp = tmp | 
                       (DIAG_DEINTERLACED << DISP_HD_VID0_FORMAT_DEINT_EN_SHIFT);
             }
             else
             {
                 tmp = tmp | 
                       (WEAVE_DEINTERLACED << DISP_HD_VID0_FORMAT_DEINT_EN_SHIFT) |
                       (WEAVE_DEINTERLACED << DISP_HD_VID0_FORMAT_DEINT_CBCR_SHIFT);
             }
         }
         else if ((config->surface_caps & DSCAPS_INTERLACED) &&
                  (display_mode->scanmode == DSESM_PROGRESSIVE))
         {
             tmp = tmp | 
                   (HYBRID_DEINTERLACED << DISP_HD_VID0_FORMAT_DEINT_EN_SHIFT) |
                   (WEAVE_DEINTERLACED << DISP_HD_VID0_FORMAT_DEINT_CBCR_SHIFT);
         }

         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_FORMAT_INDEX], tmp );
         
         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_BUF_PITCH_INDEX], 
                   ((c_pitch << 16) | y_pitch));
         
         if ((config->surface_caps & DSCAPS_VIXS_INTERLACED) ||
             (config->surface_caps & DSCAPS_SEPARATED))
         {
             y_buf_size = y_buf_size / 2;
             c_buf_size = c_buf_size / 2;
         }

		 if (START_POS_OUT_OF_RANGE)
		 {
	         real_width = config->source.w +  (((config->source.x - x_round_down) + DISP_INIT_POS) - VIDEO_LAYER_LUMA_ADDR_ALIGNMENT);
		 }
         else
		 {
			 real_width = config->source.w +  (config->source.x - x_round_down);
		 }


         if ((real_width % 16) == 1)
         {
              real_width += 1;
         }

         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_BUF_SIZE_INDEX], 
                   ((c_buf_size << DISP_HD_VID0_BUF_SIZE_C_SHIFT) | y_buf_size));
         
#ifdef DISP_HIST_VID0_ENABLE
{
u32 regValue = 0;

	 regValue = ((config->source.h) << DISP_HIST_SIZE_V_SIZE_SHIFT) |
                    ((config->source.w) << DISP_HIST_SIZE_H_SIZE_SHIFT);
//	 fprintf(stderr, "DISP_HIST_SIZE=%x\n", regValue);
         REG_WRITE(DISP_HIST_SIZE, regValue);

//	 fprintf(stderr, "DISP_HIST_SAMPLES=%x\n", regValue);
         REG_WRITE(DISP_HIST_SAMPLES, config->source.w);

	 regValue = y_pitch | (y_buf_size << DISP_HIST_BUF_SIZE_SHIFT);
//	 fprintf(stderr, "DISP_HIST_BUF=%x\n", regValue);
         REG_WRITE(DISP_HIST_BUF, regValue);

	 regValue = config->source.h * config->source.w;
//	 fprintf(stderr, "DISP_HIST_LENGTH=%x\n", regValue);
         REG_WRITE(DISP_HIST_LENGTH, regValue);

	 // different from vid0 format, just use value for now
//	 fprintf(stderr, "DISP_HIST_FORMAT=%x\n", regValue);
         REG_WRITE(DISP_HIST_FORMAT, 0x0881b000);

//	 fprintf(stderr, "DISP_HIST_ADR=%x\n", regValue);
         REG_WRITE(DISP_HIST_ADR, y_base);
}
#endif

         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_SCALED_SIZE_INDEX], 
                   (config->dest.w | (config->dest.h << DISP_HD_VID0_SCALED_SIZE_V_SCALED_SIZE_SHIFT)));
         
         //Setup scaling
         Calculate_video_sca_inv_ratio(display_mode, 
                                       config, 
                                       &video_vsca_inv_ratio,
                                       &video_hsca_inv_ratio,
                                       &vid_vsca_init0,
                                       &vid_vsca_init1,
                                       &vid_vsca_start,
                                       &vid_hsca_init,
                                       &vid_hsca_start,
                                       x_round_down); 
         
         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_VSCA_INV_RATIO_INDEX], 
                   video_vsca_inv_ratio);
         
         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_HSCA_INV_RATIO_INDEX], 
                   video_hsca_inv_ratio);
         
         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_VSCA_INIT0_INDEX], 
                   vid_vsca_init0);
         
         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_VSCA_INIT1_INDEX], 
                   vid_vsca_init1);
         
         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_VSCA_START_INDEX], 
                   vid_vsca_start);
         
         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_HSCA_INIT_INDEX], 
                   vid_hsca_init);

         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_HSCA_START_INDEX], 
                   vid_hsca_start);

         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_SAMPLES_INDEX], 
                   real_width);

         SET_NUMBER_ALIGNMENT(horizontal_size16_delay, real_width, 16);
         horizontal_size16_delay = (horizontal_size16_delay >> 4);
         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_SIZE_INDEX], 
                   (horizontal_size16_delay << DISP_HD_VID0_SIZE_H_SIZE16_SHIFT) |
                   (config->source.h << DISP_HD_VID0_SIZE_V_SIZE_SHIFT) |
                   real_width);

         if ((y_addr != REG_READ(disp_hd_vid_register_table[DISP_HD_VID_Y_ADR_INDEX])) ||
             (cb_addr != REG_READ(disp_hd_vid_register_table[DISP_HD_VID_CB_ADR_INDEX])) ||
             (cr_addr != REG_READ(disp_hd_vid_register_table[DISP_HD_VID_CR_ADR_INDEX])))
         {

			 //if it's 3d mode, the video 1 address should match as video 0
			 if ((disp_hd_vid_register_table[DISP_HD_VID_Y_ADR_INDEX] == DISP_HD_VID1_Y_ADR) &&
				 (CURRENT_MODE_IS_3D) &&
				 (m_layer_data->total_submitted_frames > 0))
		     {
				 ;
			 }
			 else
			 {
				 REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_Y_ADR_INDEX], 
						   y_addr);

				 REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_CB_ADR_INDEX], 
						   cb_addr);

				 REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_CR_ADR_INDEX], 
					   cr_addr);

			 }
			 force_reset_layer = 1;
         }
     }

     //set the screen position
     if (disp_hd_vid_register_table[DISP_HD_COMP_VID_POS_INDEX] == DISP_HD_COMP_VID1_POS)
     {
         tmp = config->dest.x | ((config->dest.y + 1) << DISP_HD_COMP_VID0_POS_POS_Y_SHIFT);
     }
     else
     {
         tmp = config->dest.x | (config->dest.y << DISP_HD_COMP_VID0_POS_POS_Y_SHIFT);
     }

     if (tmp == REG_READ(disp_hd_vid_register_table[DISP_HD_COMP_VID_POS_INDEX]))
     {
         ;
     }
     else
     {
         REG_WRITE(disp_hd_vid_register_table[DISP_HD_COMP_VID_POS_INDEX], tmp);

     }
               
     /* Setup the video input layers limit ... Y/CBCR */
     
     tmp    =   REG_READ(disp_hd_vid_register_table[DISP_HD_VID_INPUT_Y_LIMIT_INDEX]);
     tmp    &=  ~(  disp_hd_vid_register_table[DISP_HD_VID_INPUT_Y_PLUS_MASK_INDEX] |
                    disp_hd_vid_register_table[DISP_HD_VID_INPUT_Y_MINUS_MASK_INDEX]
                    );
     tmp |= (DISP_HD_Y_PLUS_LIMIT << disp_hd_vid_register_table[DISP_HD_VID_INPUT_Y_PLUS_SHIFT_INDEX] |
             DISP_HD_Y_MINUS_LIMIT << disp_hd_vid_register_table[DISP_HD_VID_INPUT_Y_MINUS_SHIFT_INDEX]
             );
             
     REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_INPUT_Y_LIMIT_INDEX], tmp);
     
     /* Since the CBCR limit registers have no other fields, we do not need to clear it out .. */
     tmp    =   DISP_HD_CBCR_PLUS_LIMIT <<  disp_hd_vid_register_table[DISP_HD_VID_INPUT_CBCR_PLUS_SHIFT_INDEX] |
                DISP_HD_CBCR_MINUS_LIMIT << disp_hd_vid_register_table[DISP_HD_VID_INPUT_CBCR_MINUS_SHIFT_INDEX]
                ;
     REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_INPUT_CBCR_LIMIT_INDEX], tmp);
     
     //set the layer global alpha
     tmp = REG_READ( DISP_HD_COMP_ALPHA);
     tmp = tmp & (~ disp_hd_vid_register_table[ALPHA_VID_MASK_INDEX]);
     tmp = tmp | ((config->opacity & 0x0ff) << disp_hd_vid_register_table[DISP_HD_COMP_ALPHA_VID_SHIFT_INDEX]);
     REG_WRITE( DISP_HD_COMP_ALPHA, tmp);

     if (((config->opacity & 0x0ff) == 0) &&
         (m_layer_data->total_submitted_frames == 0))
     {
         tmp = REG_READ(disp_hd_vid_register_table[DISP_HD_VID_FORMAT_INDEX]) | DISP_HD_VID0_FORMAT_PIPE_DISABLE_MASK;
         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_FORMAT_INDEX], tmp);
     }
     else
     {
         tmp = REG_READ(disp_hd_vid_register_table[DISP_HD_VID_FORMAT_INDEX]) & (~DISP_HD_VID0_FORMAT_PIPE_DISABLE_MASK);
         REG_WRITE(disp_hd_vid_register_table[DISP_HD_VID_FORMAT_INDEX], tmp);

     }

     //set rotation if requested
     if ( config->options & DLOP_ROTATE90 )
     {
        m_layer_data->overlay_rotation = 90;
     }  
     else if ( config->options & DLOP_ROTATE180 )
     {
        m_layer_data->overlay_rotation = 180;
     }  
     else if ( config->options & DLOP_ROTATE270 )
     {
        m_layer_data->overlay_rotation = 270;
     }  
     else
     {
        m_layer_data->overlay_rotation = 0;
     }  

     //check if set luma key
     if(config->options & DLOP_SRC_COLORKEY)
     {
         low_key = 0x1000;
         key_high = (((config->src_key.r - 16) * 16) &  0x1fff) + 0x0c;
         tmp = low_key | VID1_KEY_EN_MASK | VID1_KEY_BG_MASK | (key_high << VID1_KEY_HIGH_SHIFT);
         REG_WRITE (disp_hd_vid_register_table[DISP_HD_COMP_LUMA_KEY_INDEX], tmp);

         //to improve the vid1 setting for luma key
         if (disp_hd_vid_register_table[DISP_HD_COMP_LUMA_KEY_INDEX] == DISP_HD_COMP_LUMA_KEY1)
         {

             REG_WRITE (DISP_HD_OUTPUT_LIMIT, 0x00ff00ff);
             REG_WRITE (DISP_HD_VID0_INPUT_Y_LIMIT, 0x6e1000ff);
             REG_WRITE (DISP_HD_VID0_INPUT_CBCR_LIMIT, 0x00ff);
             REG_WRITE (DISP_HD_VID1_INPUT_Y_LIMIT, 0x6e1000ff);
             REG_WRITE (DISP_HD_VID1_INPUT_CBCR_LIMIT, 0x00ff);

             REG_WRITE (DISP_HD_VID1_HSCA_INIT, 0x08400840);
             REG_WRITE (DISP_HD_VID1_HSCA_START, 0x00000404);
             REG_WRITE (DISP_HD_VID1_VSCA_INIT0, 0x00006020);
             REG_WRITE (DISP_HD_VID1_VSCA_INIT1, 0x00000000);
             REG_WRITE (DISP_HD_VID1_VSCA_START, 0x00000304);
         }

     }
     else
     {
         tmp = REG_READ(disp_hd_vid_register_table[DISP_HD_COMP_LUMA_KEY_INDEX]);
         tmp = tmp & (~VID1_KEY_EN_MASK);
         REG_WRITE (disp_hd_vid_register_table[DISP_HD_COMP_LUMA_KEY_INDEX], tmp);
     }

     m_layer_data->color_space = config->color_space;

     CSC_ProgAdjustMatrix(mdrv, layer->shared->layer_id);  

     if (Background_Layer_Alpha_On(mdrv))
     {
         Virtual_Layer_Combine_To_Physical_Hd_Gra0(mdrv, layer_data, NULL, BACKGROUND_GRA_LAYER_ID, DSFLIP_WAITFORSYNC);
     }
     else if(dfb_config->soft_mirror_sd)
     {
         if(m_layer_data->total_submitted_frames > 0 && m_layer_data->alpha > 0)
         {
            DFBRegion region;
            region.x1 = 0;
            region.y1 = (REG_READ(disp_hd_vid_register_table[DISP_HD_CMD_FIFO_INDEX]) & 0x1F) * m_layer_data->flip_height;
            region.x2 = 0;
            region.y2 = region.y1 + m_layer_data->flip_height;
            XcodeFlipSDFrame(layer, &region, lock);
         }
         else
         {
            XcodeFlipSDFrame(dfb_layer_at(XCODE4_GRA0_LAYER_ID), NULL, NULL);
         }
     }

     return DFB_OK;

}

static DFBResult
XcodeVid0SetRegion( CoreLayer                  *layer,
                    void                       *driver_data,
                    void                       *layer_data,
                    void                       *region_data,
                    CoreLayerRegionConfig      *config,
                    CoreLayerRegionConfigFlags  updated,
                    CoreSurface                *surface,
                    CorePalette                *palette,
                    CoreSurfaceBufferLock      *lock )
{
     int     ret;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     D_ASSERT( driver_data != NULL );
     D_ASSERT( layer_data != NULL );

     D_DEBUG( "XcodeVid0SetRegion enter \n" );
     D_DEBUG ("surface width %d \n", config->width);
     D_DEBUG ("surface height %d \n", config->height);
     D_DEBUG ("destination x %d \n", config->dest.x);
     D_DEBUG ("destination y %d \n", config->dest.y);
     D_DEBUG ("destination width %d \n", config->dest.w);
     D_DEBUG ("destination height %d \n", config->dest.h);
     D_DEBUG ("source x %d \n", config->source.x);
     D_DEBUG ("source y %d \n", config->source.y);
     D_DEBUG ("source weight %d \n", config->source.w);
     D_DEBUG ("source height %d \n", config->source.h);
     D_DEBUG ("surface caps %d \n", config->surface_caps);
     D_DEBUG ("surface pitch %d \n", lock->pitch);
     D_DEBUG ("surface phys 0x%x \n", lock->phys);

     //set hd video0 pipe
     ret =  Xcode_Set_Video_Plane(layer,
                                  driver_data,
                                  layer_data,
                                  config,
                                  lock,
                                  surface,
                                  updated,
                                  disp_hd_vid0_register_table);

     D_DEBUG( "XcodeVid0SetRegion exit \n" );

     return ret;
}

/****************************************************************************************************************************/

DFBResult XcodeVid0SetColorAdjustment(CoreLayer            *layer,
                                void                 *driver_data,
                                void                 *layer_data,
                                DFBColorAdjustment   *adj)
{
    D_DEBUG( "XcodeVid0SetColorAdjustment enter \n" );

    D_DEBUG ("brightness %d \n", adj->brightness);
    D_DEBUG ("contrast  %d \n", adj->contrast );
    D_DEBUG ("hue  %d \n", adj->hue );
    D_DEBUG ("saturation  %d \n", adj->saturation );

    //set hue , bright, contrast, sat
    Vid_Set_Color_Adjustment(driver_data, layer_data, adj, disp_hd_vid0_register_table);

    D_DEBUG( "XcodeVid0SetColorAdjustment exit \n" );

    return DFB_OK;
}

/****************************************************************************************************************************/

static DFBResult
XcodeVid0FlipRegion( CoreLayer             *layer,
                 void                  *driver_data,
                 void                  *layer_data,
                 void                  *region_data,
                 CoreSurface           *surface,
                 DFBSurfaceFlipFlags    flags,
                 CoreSurfaceBufferLock *lock )
{
     DFBResult              ret;
     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

     D_DEBUG( "XcodeVid0FlipRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( mdrv != NULL );

     D_DEBUG( "( 0x%08lx [%d] 0x%04x)\n", 
                 lock->phys, lock->pitch, flags);

     dfb_surface_flip( surface, false );

     D_DEBUG( "XcodeVid0FlipRegion exit \n" );
     return DFB_OK;
}

/****************************************************************************************************************************/

#define VID0_FIFO_MAX_ENTRIES 16

void rotateVideoImage( xc_driver_data *mdrv, xc_layer_data *m_layer_data, const DFBRegion *update )
{
    xc_device_data *dev = (xc_device_data *)mdrv->pdevice_data;
    u32 vidYPhys, vidUPhys, vidVPhys;
    u32 tmpYPhys, tmpVPhys, tmpUPhys;
    u32 srcW = m_layer_data->source_width, srcH = m_layer_data->source_height;
    u32 dstW = m_layer_data->source_width, dstH = m_layer_data->source_height;
    u32 rotation = m_layer_data->overlay_rotation;
    int usedVid0FifoEntries;
    static u32 prev_y1 = 0;

    /* we've already blitted this frame so don't do anything */
    if ( prev_y1 == update->y1 )
    {
        return;
    }

    /* calculate the src/dst addresses to use for the blt to the reserved video region */
    vidYPhys = m_layer_data->y_start_addr + ( m_layer_data->y_pitch * update->y1 );
    vidUPhys = m_layer_data->u_start_addr + ( m_layer_data->c_pitch * ( update->y1 / 2 ) );
    vidVPhys = m_layer_data->v_start_addr + ( m_layer_data->c_pitch * ( update->y1 / 2 ) );

    tmpYPhys = dev->temp_render_physical;
    tmpUPhys = tmpYPhys + ( m_layer_data->y_pitch * m_layer_data->source_height );
    tmpVPhys = tmpUPhys + ( m_layer_data->c_pitch * ( m_layer_data->source_height / 2 ) );

    if ( rotation == 90 || rotation == 270 ) /* 90 or 270 degree rotation need to use a 1/4 size image so G2D can keep up */
    {
        dstW = srcH;
        dstH = srcW;
        srcW = dstW * 2;
        srcH = dstH * 2;
    }

    /* lock G2D */
    pthread_mutex_lock( &mdrv->shared->g2d_mutex );

    /* blit without rotation to temp region */
    /* Y */
    xcode4_g2d_scale_and_rotate( mdrv, 
                                 vidYPhys, DSPF_LUT8, 0, 0, srcW, srcH, m_layer_data->y_pitch,
                                 tmpYPhys, DSPF_LUT8, 0, 0, dstW, dstH, m_layer_data->y_pitch, 0 );
    /* U */
    xcode4_g2d_scale_and_rotate( mdrv, 
                                 vidUPhys, DSPF_LUT8, 0, 0, srcW / 2, srcH / 2, m_layer_data->c_pitch,
                                 tmpUPhys, DSPF_LUT8, 0, 0, dstW / 2, dstH / 2, m_layer_data->c_pitch, 0 );
    /* V */
    xcode4_g2d_scale_and_rotate( mdrv, 
                                 vidVPhys, DSPF_LUT8, 0, 0, srcW / 2, srcH / 2, m_layer_data->c_pitch,
                                 tmpVPhys, DSPF_LUT8, 0, 0, dstW / 2, dstH / 2, m_layer_data->c_pitch, 0 );

    if ( rotation == 90 || rotation == 270 )
    {
        srcW /= 2;
        srcH /= 2;
    }

    /* rotate and put back the image */
    /* Y */
    xcode4_g2d_scale_and_rotate( mdrv,
                                 tmpYPhys, DSPF_LUT8, 0, 0, srcW, srcH, m_layer_data->y_pitch,
                                 vidYPhys, DSPF_LUT8, 0, 0, dstW, dstH, m_layer_data->y_pitch, rotation );
    /* U */
    xcode4_g2d_scale_and_rotate( mdrv,
                                 tmpUPhys, DSPF_LUT8, 0, 0, srcW / 2, srcH / 2, m_layer_data->c_pitch,
                                 vidUPhys, DSPF_LUT8, 0, 0, dstW / 2, dstH / 2, m_layer_data->c_pitch, rotation );
    /* V */
    xcode4_g2d_scale_and_rotate( mdrv,
                                 tmpVPhys, DSPF_LUT8, 0, 0, srcW / 2, srcH / 2, m_layer_data->c_pitch,
                                 vidVPhys, DSPF_LUT8, 0, 0, dstW / 2, dstH / 2, m_layer_data->c_pitch, rotation );

    /* if we are blitting to the buffer about to be displayed then wait until the blits are complete or we may get a tear */
    usedVid0FifoEntries = VID0_FIFO_MAX_ENTRIES - REG_GET_FLD( DISP_HD_FIFO_STATUS0, VID0_FREE_ENTRIES );
 
    if ( usedVid0FifoEntries == 0 )
    {
        while (REG_READ( G2D_GUI_STAT ) & (STAT_FIFO_MASK | STAT_GFX_BUSY_MASK ) )
        {
            usleep(1);
        }
    }

    /* release G2D */
    pthread_mutex_unlock( &mdrv->shared->g2d_mutex );

    prev_y1 = update->y1;
}

/****************************************************************************************************************************/

static DFBResult
XcodeVid0UpdateRegion( CoreLayer             *layer,
                   void                  *driver_data,
                   void                  *layer_data,
                   void                  *region_data,
                   CoreSurface           *surface,
                   const DFBRegion       *update,
                   CoreSurfaceBufferLock *lock )
{

     D_DEBUG( "XcodeVid0UpdateRegion enter \n" );

     D_ASSERT( surface != NULL );
     D_ASSERT( lock != NULL );
     D_ASSERT( driver_data != NULL );
     D_ASSERT( layer_data != NULL );
     D_ASSERT( update != NULL );
     DFBResult ret;

     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;

     /* if overlay rotation has been requested, use G2D to rotate the image */
     if ( m_layer_data->overlay_rotation != 0 )
     {
        rotateVideoImage( mdrv, m_layer_data, update );
     }

     if(dfb_config->soft_mirror_sd)
     {
        if(XcodeSDFullness(layer))
        {
            return DFB_FAILURE;
        }
     }
     if ((m_layer_data->field_select == INTERLACED_INTERWOVEN_BOTH_FIELD_INTERLEAVE) &&
         (REG_READ(DISP_HD_COMP_SIZE) & DISP_HD_COMP_SIZE_INTERLACED_MASK) &&
         !CURRENT_MODE_IS_3D)
     {
         ret = xcode4_flip_interlaced_video(driver_data, 
                                     layer_data, 
                                     update, 
                                     disp_hd_vid0_register_table);
     }
     else
     {
         ret = Xcode_Video_Plane_Update_Region(driver_data, 
                                     layer_data,
                                     update,
                                     disp_hd_vid0_register_table);
     }

     if(ret == DFB_OK && dfb_config->soft_mirror_sd)
     {
          XcodeFlipSDFrame(layer, update, lock);
     }

     D_DEBUG( "XcodeVid0UpdateRegion exit \n" );
     return ret;
}

/**********************************************************************************************************************/

int
XcodeVidLayerDataSize()
{
     return sizeof(xc_layer_data);
}

/**********************************************************************************************************************/

DFBResult xcode4_get_current_frame_consume(void                   *driver_data,
                                           void                   *layer_data,
                                           int                    *frame_consume,
                                           u32 * disp_hd_vid_register_table )
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    int frames_to_free=0;
    int current_frame_counter;
    u32 tmp;
    u32 current_fifo_level;
    bool currentModeIs3d = CURRENT_MODE_IS_3D;

    D_DEBUG(" xcode4_get_current_frame_consume enter:\n");

    if ((dfb_config->check_vid0_playback) &&
        (disp_hd_vid_register_table[DISP_HD_CMD_FIFO_INDEX] == DISP_HD_VID0_CMD0_FIFO))
    {
        tmp = REG_READ(DMA_DUMMY_REG) + 1;
        REG_WRITE(DMA_DUMMY_REG, tmp);
    }
    else if ((dfb_config->check_vid1_playback) &&
        (disp_hd_vid_register_table[DISP_HD_CMD_FIFO_INDEX] == DISP_HD_VID1_CMD0_FIFO))
    {
        tmp = REG_READ(DMA_DUMMY_REG) + 1;
        REG_WRITE(DMA_DUMMY_REG, tmp);
    }

    if ( currentModeIs3d )
    {
        pthread_mutex_lock(&m_layer_data->layer_mutex);
    }

    if (REG_READ(DISP_HD_FIFO_STATUS0) & disp_hd_vid_register_table[VID_EMPTY_MASK_INDEX])
    {
        current_frame_counter = m_layer_data->total_submitted_frames;
    }
    else
    {
        current_fifo_level = MAX_VID_FIFO_ENTRIES - (((REG_READ(DISP_HD_FIFO_STATUS0)) & disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_MASK_INDEX]) >>
                             disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_SHIFT_INDEX]);                                   

        current_frame_counter = m_layer_data->total_submitted_frames - current_fifo_level;
            
    }

    if (current_frame_counter > (m_layer_data->last_frame_counter + FRAMES_TO_HOLD))
    {
         frames_to_free = current_frame_counter - m_layer_data->last_frame_counter;
         m_layer_data->last_frame_counter = current_frame_counter - FRAMES_TO_HOLD;

    }
    else
    {
        frames_to_free=0;
    }

    if (frames_to_free > FRAMES_TO_HOLD)
    {
         //
         //Hold one frame
         // 
         frames_to_free--;
    }
    else
    {
         frames_to_free=0;
    }

    D_DEBUG("frames to free = %x \n", frames_to_free);

    m_layer_data->total_return_frame_to_free += frames_to_free;

    if ((m_layer_data->total_return_frame_to_free >= m_layer_data->total_submitted_frames) &&
        (m_layer_data->total_return_frame_to_free))
    {

        D_WARN("Alarm : total_return_frame_to_free %d is equal or bigger than total_submitted_frames %d \n",
               m_layer_data->total_return_frame_to_free,
               m_layer_data->total_submitted_frames);
    }
    else if ((m_layer_data->total_return_frame_to_free >= current_frame_counter) &&
        (m_layer_data->total_return_frame_to_free))
    {

        D_WARN("Alarm : total_return_frame_to_free %d is equal or bigger than current_frame_counter %d \n",
               m_layer_data->total_return_frame_to_free,
               current_frame_counter);
    }
    else if ((frames_to_free == 0) &&
             (REG_READ(DISP_HD_FIFO_STATUS0) & disp_hd_vid_register_table[VID_EMPTY_MASK_INDEX]) &&
             (m_layer_data->total_submitted_frames > 2) &&
             ((m_layer_data->total_submitted_frames - m_layer_data->total_return_frame_to_free) > FRAMES_TO_HOLD))
    {
        D_INFO("Info : total_return_frame_to_free %d is underflow to total_submitted_frames %d while video fifo is empty \n",
               m_layer_data->total_return_frame_to_free,
               m_layer_data->total_submitted_frames);

    }

    if ( currentModeIs3d )
    {
        pthread_mutex_unlock(&m_layer_data->layer_mutex);
    }

    if ((dfb_config->check_vid0_playback) &&
        (disp_hd_vid_register_table[DISP_HD_CMD_FIFO_INDEX] == DISP_HD_VID0_CMD0_FIFO))
    {
        REG_WRITE(DISP_RFMOD_CRC_COUNT, m_layer_data->total_return_frame_to_free);
    }
    else if ((dfb_config->check_vid1_playback) &&
        (disp_hd_vid_register_table[DISP_HD_CMD_FIFO_INDEX] == DISP_HD_VID1_CMD0_FIFO))
    {
        REG_WRITE(DISP_RFMOD_CRC_COUNT, m_layer_data->total_return_frame_to_free);
    }

    *frame_consume = frames_to_free;

    D_DEBUG(" xcode4_get_current_frame_consume exit:\n");
    return DFB_OK;

}

/**********************************************************************************************************************/

DFBResult xcode4_clear_frame_consume(void                   *driver_data,
                                     void                   *layer_data,
                                     u32 * disp_hd_vid_register_table )
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32 current_fifo_level=0;
    u32 base_frm_cnt;     

    D_DEBUG(" xcode4_clear_frame_consume enter:\n");

    m_layer_data->last_frame_counter = 0;
    m_layer_data->total_submitted_frames = 0;
    m_layer_data->total_return_frame_to_free = 0;
    m_layer_data->last_frame_index = -1;
    m_layer_data->last_drop_flag = 0;

    //wait fifo idle
    current_fifo_level = ((REG_READ(DISP_HD_FIFO_STATUS0)) & disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_MASK_INDEX])
                       >> disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_SHIFT_INDEX];

    while ((current_fifo_level < MAX_VID_FIFO_ENTRIES) &&
           (REG_READ(DISP_HD_COMP_CTRL) & DISP_HD_COMP_CTRL_COMP_EN_MASK))
    {
        usleep(1);
        current_fifo_level = ((REG_READ(DISP_HD_FIFO_STATUS0)) & disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_MASK_INDEX])
                           >> disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_SHIFT_INDEX];
    }

    D_DEBUG(" xcode4_clear_frame_consume exit:\n");

    return DFB_OK;

}

/**********************************************************************************************************************/

DFBResult xcode4_get_fifo_status(void                   *driver_data,
                                 void                   *layer_data,
                                 int                    *fifo_status,
                                 u32 * disp_hd_vid_register_table )
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32 current_fifo_level=0;

    D_DEBUG(" xcode4_get_fifo_status enter:\n");

    current_fifo_level = ((REG_READ(DISP_HD_FIFO_STATUS0)) & disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_MASK_INDEX])
                   >> disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_SHIFT_INDEX];

   
    *fifo_status = MAX_VID_FIFO_ENTRIES - current_fifo_level;

    D_DEBUG(" xcode4_get_fifo_status exit:\n");
    return DFB_OK;

}


/**********************************************************************************************************************/
static DFBResult
XcodeVid0GetCurrentOutputField( CoreLayer *layer,
                                void      *driver_data,
                                void      *layer_data,
                                int       *field )
{
     DFBResult ret;
     
     D_DEBUG( "XcodeVid0GetCurrentOutputField enter \n" );

     //it's a dummy function
     //upper application would use GetCurrentFrameConsume and GetFifoStatus

     D_DEBUG( "XcodeVid0GetCurrentOutputField exit \n" );
     return DFB_OK;
}

/**********************************************************************************************************************/
static DFBResult
Xcode4Vid0SetInputField( CoreLayer *layer,
                         void      *driver_data,
                         void      *layer_data,
                         void      *region_data,
                         int        field )
{
    DFBResult ret; 

    D_DEBUG( "Xcode4Vid0SetInputField enter \n" );
    D_DEBUG( "field  0x%x \n", field );

    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;

    m_layer_data->ufield_update = field & 0x01;
    
    D_DEBUG( "Xcode4Vid0SetInputField exit \n" );

    return DFB_OK;
}

/**********************************************************************************************************************/
static DFBResult
XcodeVid0GetLevel( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int       *level )
{
     
    D_DEBUG( "XcodeVid0GetLevel enter \n" );

    * level = Get_Layer_Level(driver_data, VID0_MUX_SHIFT, VID0_MUX_MASK);

    D_DEBUG( "XcodeVid0GetLevel exit \n" );

    return DFB_OK;
}

/**********************************************************************************************************************/
static DFBResult
XcodeVid0SetLevel( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int        level )
{
    DFBResult ret;
	pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    
    D_DEBUG( "XcodeVid0SetLevel enter \n" );
    D_DEBUG( "level %d \n", level );

    ret = Set_Layer_Level(driver_data, level, VID0_MUX_SHIFT, VID0_MUX_MASK);
    D_DEBUG( "XcodeVid0SetLevel exit \n" );
    
     
    return ret;
}
/**********************************************************************************************************************/
static DFBResult
XcodeVid0GetCurrentFrameConsume( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int       *frame_consume )
{
    DFBResult ret;
     
    D_DEBUG( "XcodeVid0GetCurrentFrameConsume enter \n" );

    ret =  xcode4_get_current_frame_consume( driver_data,
                                      layer_data,
                                      frame_consume,
                                      disp_hd_vid0_register_table );

    D_DEBUG( "XcodeVid0GetCurrentFrameConsume exit \n" );

    return ret;
}

/**********************************************************************************************************************/
static DFBResult
XcodeVid0ClearCurrentFrameConsume( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data)
{
    DFBResult ret;
     
    D_DEBUG( "XcodeVid0ClearCurrentFrameConsume enter \n" );

    ret =  xcode4_clear_frame_consume( driver_data,
                                      layer_data,
                                      disp_hd_vid0_register_table );

    D_DEBUG( "XcodeVid0ClearCurrentFrameConsume exit \n" );

    return ret;
}

/**********************************************************************************************************************/
static DFBResult
XcodeVid0GetFifoStatus( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int       *fifo_status )
{
    DFBResult ret;
    D_ASSERT( driver_data != NULL );
    D_ASSERT( layer_data != NULL );
     
    D_DEBUG( "XcodeVid0GetFifoStatus enter \n" );

    ret =  xcode4_get_fifo_status( driver_data,
                                      layer_data,
                                      fifo_status,
                                      disp_hd_vid0_register_table );

    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32 current_fifo_index;
    current_fifo_index = ((REG_READ(DISP_HD_FIFO_STATUS0)) & VID0_CURR_IDX_MASK)
               >> VID0_CURR_IDX_SHIFT;

    *fifo_status = (*fifo_status) | (current_fifo_index << 16);

    D_DEBUG( "XcodeVid0GetFifoStatus exit \n" );

    return ret;
}

/**********************************************************************************************************************/
static DFBResult
XcodeVid0CheckLayerStatus( CoreLayer *layer,
             void      *driver_data,
             void      *layer_data,
             int       status_bit )
{
    DFBResult ret;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
     
    D_DEBUG( "XcodeVid0CheckLayerStatus enter \n" );

    if (m_layer_data->status & status_bit)
    {
        ret = DFB_OK;
    }
    else
    {
        ret = DFB_FAILURE;
    }

    if (status_bit == LAYER_CHECK_BIT)
    {
        m_layer_data->status = m_layer_data->status | LAYER_CHECK_BIT;
    }


    D_DEBUG( "XcodeVid0CheckLayerStatus exit \n" );

    return ret;
}

/**********************************************************************************************************************/
static DFBResult
XcodeVid0ReallocateSurface( CoreLayer             *layer,
                          void                  *driver_data,
                          void                  *layer_data,
                          void                  *region_data,
                          CoreLayerRegionConfig *config,
                          CoreSurface           *surface )
{
     DFBResult ret;
     pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;

     D_DEBUG( "XcodeVid0ReallocateSurface enter \n" );

     if ((dfb_config->preallocate_video0_buffer_number) &&
         (surface ))
     {
         CoreSurfaceConfig conf;
         conf.flags  = CSCONF_SIZE;
         conf.size.w = config->width;
         conf.size.h = config->height;
         ret = dfb_surface_arrange_again( surface, CSAID_LAYER0 + dfb_layer_id(layer), CSAF_WRITE|CSAF_READ, &conf );
     }
     else
     {
         ret = DFB_FAILURE;
     }

     D_DEBUG( "XcodeVid0ReallocateSurface exit \n" );

     return ret;
}


/**********************************************************************************************************************/

const DisplayLayerFuncs XcodeVid0LayerFuncs = {
     LayerDataSize:      XcodeVidLayerDataSize,
     InitLayer:          XcodeVid0InitLayer,
     TestRegion:         XcodeVid0TestRegion,
     SetRegion:          XcodeVid0SetRegion,
     FlipRegion:         XcodeVid0FlipRegion,
     UpdateRegion:       XcodeVid0UpdateRegion,
     GetCurrentOutputField: XcodeVid0GetCurrentOutputField,
     GetCurrentFrameConsume: XcodeVid0GetCurrentFrameConsume,
     ClearFrameConsume: XcodeVid0ClearCurrentFrameConsume,
     GetCurrentFifoStatus: XcodeVid0GetFifoStatus,
     SetColorAdjustment: XcodeVid0SetColorAdjustment,
     SetInputField:      Xcode4Vid0SetInputField,
     GetLevel:           XcodeVid0GetLevel,
     SetLevel:           XcodeVid0SetLevel,
     CheckLayerStatus:   XcodeVid0CheckLayerStatus,
     ReallocateSurface:  XcodeVid0ReallocateSurface
};

static xcode4_flip_interlaced_frame(pxc_dfb_driver_data mdrv, pxc_dfb_layer_data m_layer_data, const DFBRegion * region, u32 * disp_hd_vid_register_table)
{
    int field, cmd, status, i = 0;
    app_ioctl_command app_cmd;
    app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_GET_HD_DISPLAY_FIELD;

    ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd); 
    field = app_cmd.dfb_hd_field.field;
    status = app_cmd.dfb_hd_field.status;
    cmd = 16 - ((status & disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_MASK_INDEX]) >> disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_SHIFT_INDEX]);

    //We'll not update in odd state
    while(((field + cmd) & 0x1) == 0 && (i++ < 50))
    {    
        usleep(1000);
        ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd); 
        if(field != app_cmd.dfb_hd_field.field)
        {
            field = app_cmd.dfb_hd_field.field;
            status = app_cmd.dfb_hd_field.status;
            cmd = 16 - ((status & disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_MASK_INDEX]) >> disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_SHIFT_INDEX]);
        }
    }

    //Flip twice
    Xcode_Video_Plane_Update_Region(mdrv, m_layer_data, region, disp_hd_vid_register_table);
    Xcode_Video_Plane_Update_Region(mdrv, m_layer_data, region, disp_hd_vid_register_table);
}

DFBResult xcode4_flip_interlaced_video(void                  *driver_data,
                                                                                       void                  *layer_data, 
                                                                                       const DFBRegion * region, 
                                                                                       u32 * disp_hd_vid_register_table)
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    DFBResult ret = DFB_OK;
    int free_entries;
    int index;

    free_entries = ((REG_READ(DISP_HD_FIFO_STATUS0)) & disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_MASK_INDEX])
                   >> disp_hd_vid_register_table[DISP_HD_VID_FREE_ENTRIES_SHIFT_INDEX];

    if(free_entries < 2)
    {
        return DFB_FAILURE;
    }

    if(m_layer_data->flip_height == 0)
    {
        return DFB_FAILURE;
    }

    index = region->y1 / m_layer_data->flip_height;
    
    if(m_layer_data->last_frame_index == index)
    {
        //Get entire frame

        xcode4_flip_interlaced_frame(mdrv, m_layer_data, region, disp_hd_vid_register_table);

        m_layer_data->last_frame_index = -1;
    }
    else if(m_layer_data->last_frame_index == -1)
    {
        //New field

        m_layer_data->last_frame_index = index;
    }
    else
    {
        //Field is only showed once

        if(m_layer_data->last_drop_flag)
        {
            xcode4_flip_interlaced_frame(mdrv,m_layer_data, region, disp_hd_vid_register_table);

            m_layer_data->last_drop_flag = 0;
        }
        else
        {
            m_layer_data->last_drop_flag = 1;
        }

        m_layer_data->last_frame_index = index;
    }

    return ret;
}

