/*
   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
   (c) Copyright 2000-2004  Convergence (integrated media) GmbH

   All rights reserved.

*/

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>

#include <directfb.h>

#include <core/fonts.h>
#include <core/gfxcard.h>
#include <core/surface.h>
#include <core/surface_buffer.h>

#include <gfx/convert.h>

#include <media/idirectfbfont.h>

#include <direct/mem.h>
#include <direct/memcpy.h>
#include <direct/messages.h>
#include <direct/utf8.h>
#include <direct/util.h>

#include <misc/conf.h>
#include <misc/util.h>

#include <olf/fa_encoding.h>
#include <olf/fontavenue.h>

#define STB_FONTGSCALE_MEMSIZE (100 * 100 * sizeof(fa_fixed))

#define FN_CnvExFontCount	(34+45+45)		/* 13悪34ANEC,IBMꂼ45 */

static void * global_font_data=NULL;
int global_font_data_ref =0;

typedef struct {
	u16	nSrcFont;
	u16	nDstFont;
} FN_CnvExFontTable;

static FN_CnvExFontTable FN_CnvSjisToJis[FN_CnvExFontCount] =
{
	/* PR */
	{	0x8740	,	0x7E61	}	,	/*	@	*/
	{	0x8741	,	0x7E62	}	,	/*	A	*/
	{	0x8742	,	0x7E63	}	,	/*	B	*/
	{	0x8743	,	0x7E64	}	,	/*	C	*/
	{	0x8744	,	0x7E65	}	,	/*	D	*/
	{	0x8745	,	0x7E66	}	,	/*	E	*/
	{	0x8746	,	0x7E67	}	,	/*	F	*/
	{	0x8747	,	0x7E68	}	,	/*	G	*/
	{	0x8748	,	0x7E69	}	,	/*	H	*/
	{	0x8749	,	0x7E6A	}	,	/*	I	*/
	{	0x874A	,	0x7E6B	}	,	/*	J	*/
	{	0x874B	,	0x7E6C	}	,	/*	K	*/
	{	0x874C	,	0x7E6D	}	,	/*	L	*/
	{	0x874D	,	0x7E6E	}	,	/*	M	*/
	{	0x874E	,	0x7E6F	}	,	/*	N	*/
	{	0x874F	,	0x7E70	}	,	/*	O	*/
	{	0x8750	,	0x7E2D	}	,	/*	P	*/
	{	0x8751	,	0x7E2E	}	,	/*	Q	*/
	{	0x8752	,	0x7E2F	}	,	/*	R	*/
	{	0x8753	,	0x7E30	}	,	/*	S	*/
	{	0x8754	,	0x7E21	}	,	/*	T	*/
	{	0x8755	,	0x7E22	}	,	/*	U	*/
	{	0x8756	,	0x7E23	}	,	/*	V	*/
	{	0x8757	,	0x7E24	}	,	/*	W	*/
	{	0x8758	,	0x7E25	}	,	/*	X	*/
	{	0x8759	,	0x7E26	}	,	/*	Y	*/
	{	0x875A	,	0x7E27	}	,	/*	Z	*/
	{	0x875B	,	0x7E28	}	,	/*	[	*/
	{	0x875C	,	0x7E29	}	,	/*	\	*/
	{	0x875D	,	0x7E2A	}	,	/*	]	*/
	{	0x8766	,	0x7D4A	}	,	/*	f	*/
	{	0x8767	,	0x7D47	}	,	/*	g	*/
	{	0x8770	,	0x7C2D	}	,	/*	p	*/
	{	0x8771	,	0x7D4B	}	,	/*	q	*/
	{	0x8773	,	0x7D48	}	,	/*	s	*/
	{	0x8775	,	0x7C2B	}	,	/*	u	*/
	{	0x877E	,	0x7D2C	}	,	/*	~	*/
	{	0x8782	,	0x7D2D	}	,	/*		*/
	{	0x8784	,	0x7D2E	}	,	/*		*/
	{	0x878A	,	0x7C4D	}	,	/*		*/
	{	0x878B	,	0x7C4C	}	,	/*		*/
	{	0x878C	,	0x7C4E	}	,	/*		*/
	{	0x878D	,	0x7D29	}	,	/*		*/
	{	0x878E	,	0x7D2A	}	,	/*		*/
	{	0x878F	,	0x7D2B	}	,	/*		*/
	/* mdbgiWX`XQj */
	{	0xED55	,	0x7525	}	,	/*	q	*/
	{	0xED45	,	0x7526	}	,	/*	a	*/
	{	0xED6D	,	0x752B	}	,	/*		*/
	{	0xED74	,	0x7532	}	,	/*		*/
	{	0xED76	,	0x7533	}	,	/*		*/
	{	0xED80	,	0x753A	}	,	/*		*/
	{	0xED8E	,	0x753F	}	,	/*		*/
	{	0xED95	,	0x7540	}	,	/*		*/
	{	0xED9C	,	0x7543	}	,	/*		*/
	{	0xED4B	,	0x7544	}	,	/*	g	*/
	{	0xED9E	,	0x7545	}	,	/*		*/
	{	0xEDA9	,	0x7548	}	,	/*		*/
	{	0xEDB8	,	0x7549	}	,	/*		*/
	{	0xEDC2	,	0x754C	}	,	/*		*/
	{	0xED4A	,	0x754D	}	,	/*	f	*/
	{	0xEDC7	,	0x754E	}	,	/*		*/
	{	0xEDE4	,	0x755E	}	,	/*	C	*/
	{	0xEDF8	,	0x7565	}	,	/*	W	*/
	{	0xEDFA	,	0x7566	}	,	/*	Y	*/
	{	0xEE45	,	0x756A	}	,	/*	a	*/
	{	0xEE46	,	0x756B	}	,	/*	b	*/
	{	0xEE4B	,	0x756F	}	,	/*	g	*/
	{	0xEE4C	,	0x7570	}	,	/*	h	*/
	{	0xEE74	,	0x7628	}	,	/*		*/
	{	0xED44	,	0x762E	}	,	/*	`	*/
	{	0xEE7E	,	0x7630	}	,	/*		*/
	{	0xEE86	,	0x7635	}	,	/*		*/
	{	0xEE8E	,	0x7637	}	,	/*		*/
	{	0xEE9D	,	0x763C	}	,	/*		*/
	{	0xED43	,	0x7640	}	,	/*	_	*/
	{	0xEEBC	,	0x7641	}	,	/*		*/
	{	0xED42	,	0x7642	}	,	/*	^	*/
	{	0xEECC	,	0x7643	}	,	/*		*/
	{	0xEEE0	,	0x7647	}	,	/*		*/
	/* halgiPPT`PPXj */
	{	0xFA71	,	0x7525	}	,	/*	q	*/
	{	0xFA61	,	0x7526	}	,	/*	a	*/
	{	0xFA8A	,	0x752B	}	,	/*		*/
	{	0xFA91	,	0x7532	}	,	/*		*/
	{	0xFA93	,	0x7533	}	,	/*		*/
	{	0xFA9C	,	0x753A	}	,	/*		*/
	{	0xFAAA	,	0x753F	}	,	/*		*/
	{	0xFAB1	,	0x7540	}	,	/*		*/
	{	0xFAB8	,	0x7543	}	,	/*		*/
	{	0xFA67	,	0x7544	}	,	/*	g	*/
	{	0xFABA	,	0x7545	}	,	/*		*/
	{	0xFAC5	,	0x7548	}	,	/*		*/
	{	0xFAD4	,	0x7549	}	,	/*		*/
	{	0xFADE	,	0x754C	}	,	/*		*/
	{	0xFA66	,	0x754D	}	,	/*	f	*/
	{	0xFAE3	,	0x754E	}	,	/*		*/
	{	0xFB43	,	0x755E	}	,	/*	C	*/
	{	0xFB57	,	0x7565	}	,	/*	W	*/
	{	0xFB59	,	0x7566	}	,	/*	Y	*/
	{	0xFB61	,	0x756A	}	,	/*	a	*/
	{	0xFB62	,	0x756B	}	,	/*	b	*/
	{	0xFB67	,	0x756F	}	,	/*	g	*/
	{	0xFB68	,	0x7570	}	,	/*	h	*/
	{	0xFB91	,	0x7628	}	,	/*		*/
	{	0xFA60	,	0x762E	}	,	/*	`	*/
	{	0xFB9B	,	0x7630	}	,	/*		*/
	{	0xFBA2	,	0x7635	}	,	/*		*/
	{	0xFBAA	,	0x7637	}	,	/*		*/
	{	0xFBB9	,	0x763C	}	,	/*		*/
	{	0xFA5F	,	0x7640	}	,	/*	_	*/
	{	0xFBD8	,	0x7641	}	,	/*		*/
	{	0xFA5E	,	0x7642	}	,	/*	^	*/
	{	0xFBE8	,	0x7643	}	,	/*		*/
	{	0xFBFC	,	0x7647	}		/*		*/
};

u16 Single_Byte_Code_To_Kana_Code (u8 code);

typedef struct {
     PTfa_Buffers face;
     Tfa_RequestGlyph tfaRequestGlyph;
     Tfa_Metrics  tfaMetrics;	 
     int          disable_charmap;
     int          fixed_advance;
     bool         fixed_clip;
     int          width;
     int          height;
     int          input_desc_widht;
     u16          indices[256];
} OlfImplData;


static DFBResult
Probe( IDirectFBFont_ProbeContext *ctx );

static DFBResult
Construct( IDirectFBFont      *thiz,
	   ... );

#include <direct/interface_implementation.h>

static pthread_mutex_t olf_mutex     = PTHREAD_MUTEX_INITIALIZER;

DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBFont, OLF )

#define SHIFT_JIS_INPUT

void STBKktjis(u32 *ph, u32 *pl)
{
	if (*ph <= 0x9F) {
		if (*pl < 0x9F)  *ph = (*ph << 1) - 0xE1;
		else             *ph = (*ph << 1) - 0xE0;
	} else {
		if (*pl < 0x9F)  *ph = (*ph << 1) - 0x161;
		else             *ph = (*ph << 1) - 0x160;
	}
	if      (*pl < 0x7F) *pl -= 0x1F;
	else if (*pl < 0x9F) *pl -= 0x20;
	else                 *pl -= 0x7E;
}

u16 SJIS_TO_JIS(u16 sjisCode)
{
     unsigned int jiscode;

	u32	ph, pl ;
	u16	jisCode ;
	ph = (sjisCode>> 8) ;
	pl = (sjisCode& 0x00ff) ;
	STBKktjis (&ph, &pl) ;
	jisCode = ph ;
	jisCode = (jisCode<<8);
	jisCode |= pl ;

	return jisCode ;
}


u16 FN_CnvExFontSJisToJis( u16 code )
{
	u32	i=0;

	/* OtHgϊe[uQƂAvꍇSJISJIS֕ϊ */
	for( i = 0; i < FN_CnvExFontCount; i++ ){
		if( code == FN_CnvSjisToJis[i].nSrcFont ){
			code = FN_CnvSjisToJis[i].nDstFont;
			break;
		}
	}
	return code;
}

/* VtgihrgR[h */
static bool FN_IsSupportExFont(u16 code)
{
	/* 13AmdbgAhalgR[hƂĒ`Ă镶R[hǂ */
	if( (0x8740 <= code && code <= 0x879E) || 	/* 13 */
		(0xED40 <= code && code <= 0xEEFC) || 	/* 89`92 */
		(0xFA40 <= code && code <= 0xFC4B) )  	/* 115`119 */
	{
		return true;
	}
     else 
     {
		return false;
	}
}

#define CHAR_INDEX(c) SJIS_TO_JIS(c)
/**********************************************************************************************************************/

static DFBResult
olfUTF8GetCharacterIndex( CoreFont     *thiz,
                          unsigned int  character,
                          unsigned int *ret_index )
{
     OlfImplData *data = thiz->impl_data;

     D_MAGIC_ASSERT( thiz, CoreFont );

     return DFB_OK;
}


static DFBResult
olfUTF8DecodeText( CoreFont       *thiz,
                   const void     *text,
                   int             length,
                   unsigned int   *ret_indices,
                   int            *ret_num )
{
     int pos = 0, num = 0;
     const u8 *bytes   = text;
     OlfImplData *data = thiz->impl_data;
     DFBTextEncodingID current_encoding = thiz->current_encoding; 


     D_MAGIC_ASSERT( thiz, CoreFont );
     D_ASSERT( text != NULL );
     D_ASSERT( length >= 0 );
     D_ASSERT( ret_indices != NULL );
     D_ASSERT( ret_num != NULL );

     pthread_mutex_lock ( &olf_mutex );

     while (pos < length) {

          //keep shift-jis code here , and transfer into JIS code in olf_get_glyph_info
          if ((data->indices[bytes[pos]]) &&
              (thiz->current_encoding != DTEID_JIS))
          {
              ret_indices[num++] = bytes[pos];
              pos ++;
          }
          else
          {
               ret_indices[num++] = (bytes[pos] << 8) | bytes[pos+1];
               pos = pos + 2;
          }
     }

     pthread_mutex_unlock ( &olf_mutex );

     *ret_num = num;

     return DFB_OK;
}

static const CoreFontEncodingFuncs olf2UTF8Funcs = {
     .GetCharacterIndex = olfUTF8GetCharacterIndex,
     .DecodeText        = olfUTF8DecodeText,
};


/**********************************************************************************************************************/

static DFBResult
olf2Latin1GetCharacterIndex( CoreFont     *thiz,
                            unsigned int  character,
                            unsigned int *ret_index )
{
     OlfImplData *data = thiz->impl_data;

     D_MAGIC_ASSERT( thiz, CoreFont );

     return DFB_OK;
}

static DFBResult
olf2Latin1DecodeText( CoreFont       *thiz,
                     const void     *text,
                     int             length,
                     unsigned int   *ret_indices,
                     int            *ret_num )
{
     int i;
     const u8 *bytes   = text;
     OlfImplData *data = thiz->impl_data;

     D_MAGIC_ASSERT( thiz, CoreFont );
     D_ASSERT( text != NULL );
     D_ASSERT( length >= 0 );
     D_ASSERT( ret_indices != NULL );
     D_ASSERT( ret_num != NULL );

     return DFB_OK;
}

static const CoreFontEncodingFuncs olf2Latin1Funcs = {
     .GetCharacterIndex = olf2Latin1GetCharacterIndex,
     .DecodeText        = olf2Latin1DecodeText,
};

/**********************************************************************************************************************/

static DFBResult
olf_get_glyph_info( CoreFont      *thiz,
                    unsigned int   index,
                    CoreGlyphData *info )
{
     fa_int faSts = FA_SUCCESS;
     DFBResult ret = DFB_OK;
     OlfImplData *data = (OlfImplData*) thiz->impl_data;
     DFBTextEncodingID current_encoding = thiz->current_encoding; 
     unsigned int shift_jis_code;

     if (current_encoding == DTEID_JIS) {
         data->tfaRequestGlyph.in_Code = index;
     }
     else
     {
         if ((index < 256) &&
             (data->indices[index]))
         {
             shift_jis_code = data->indices[index];
             data->tfaRequestGlyph.in_GlyphSize.x = data->input_desc_widht / 2;
         }
         else
         {
             shift_jis_code = index;
             data->tfaRequestGlyph.in_GlyphSize.x = data->input_desc_widht;
         }
    
         if (FN_IsSupportExFont(shift_jis_code)) 
         {
             data->tfaRequestGlyph.in_Code = FN_CnvExFontSJisToJis( shift_jis_code );
         }
         else
         {
             data->tfaRequestGlyph.in_Code = CHAR_INDEX( shift_jis_code );
         }
     }

//#define output_raw_glyph

#ifdef output_raw_glyph
    fa_byte * temp_ptr = NULL;
    int i, j;
#endif

     pthread_mutex_lock ( &olf_mutex );

     faSts = FaGetRegionSize(data->face, &data->tfaRequestGlyph);

     if (faSts == FA_SUCCESS) 
     {

          faSts = FaGetGlyphImage(data->face, NULL, &data->tfaRequestGlyph);


#ifdef output_raw_glyph
          printf("after call  FaGetGlyphImage,  the out_pGlyphBuffer is in_code 0x%x\n", data->tfaRequestGlyph.in_Code);

          printf("data->tfaRequestGlyph.out_pMetrics->AscenderLine %d \n", data->tfaRequestGlyph.out_pMetrics->AscenderLine);
          printf("data->tfaRequestGlyph.out_pMetrics->CapLine %d \n", data->tfaRequestGlyph.out_pMetrics->CapLine);
          printf("data->tfaRequestGlyph.out_pMetrics->BaseLine %d \n", data->tfaRequestGlyph.out_pMetrics->BaseLine);
          printf("data->tfaRequestGlyph.out_pMetrics->MeanLine %d \n", data->tfaRequestGlyph.out_pMetrics->MeanLine);
          printf("data->tfaRequestGlyph.out_pMetrics->DescenderLine %d \n", data->tfaRequestGlyph.out_pMetrics->DescenderLine);
          printf("data->tfaRequestGlyph.out_pMetrics->LeftSideBearing %d \n", data->tfaRequestGlyph.out_pMetrics->LeftSideBearing);
          printf("data->tfaRequestGlyph.out_pMetrics->RightSideBearing %d \n", data->tfaRequestGlyph.out_pMetrics->RightSideBearing);
          printf("data->tfaRequestGlyph.out_pMetrics->UpperSideBearing %d \n", data->tfaRequestGlyph.out_pMetrics->UpperSideBearing);
          printf("data->tfaRequestGlyph.out_pMetrics->UnderSideBearing %d \n", data->tfaRequestGlyph.out_pMetrics->UnderSideBearing);
          printf("data->tfaRequestGlyph.out_pMetrics->BBoxX0 %d \n", data->tfaRequestGlyph.out_pMetrics->BBoxX0);
          printf("data->tfaRequestGlyph.out_pMetrics->BBoxY0 %d \n", data->tfaRequestGlyph.out_pMetrics->BBoxY0);
          printf("data->tfaRequestGlyph.out_pMetrics->BBoxX1 %d \n", data->tfaRequestGlyph.out_pMetrics->BBoxX1);
          printf("data->tfaRequestGlyph.out_pMetrics->BBoxY1 %d \n", data->tfaRequestGlyph.out_pMetrics->BBoxY1);
          printf("data->tfaRequestGlyph.out_GlyphSize.x %d \n", data->tfaRequestGlyph.out_GlyphSize.x);
          printf("data->tfaRequestGlyph.out_GlyphSize.y %d \n", data->tfaRequestGlyph.out_GlyphSize.y);

          printf("  \n");
          temp_ptr = data->tfaRequestGlyph.out_pGlyphBuffer;
          for (i = 0; i < data->tfaRequestGlyph.in_GlyphSize.y; i ++) {
               for (j = 0; j < data->tfaRequestGlyph.in_GlyphSize.x; j ++) {
                    if (*temp_ptr != 0) {
                         printf(" 0x%02x", *temp_ptr);
                    }
                    else
                    {
                        printf("     ");
                    }
                    temp_ptr ++;
               }
          
               printf("  \n");
          }



#endif

          if (faSts != FA_SUCCESS) 
          {
               ret = DFB_FAILURE;

               D_DERROR(faSts,  "fail to Get Glyph Image from OLF font data, index 0x%x, faSts 0x%x\n", index, faSts );
          }
     }
     else
     {
          ret = DFB_FAILURE;
          D_DERROR(faSts,  "fail to Get Region Size from OLF font data, index 0x%x, faSts 0x%x\n", index, faSts );

     }

     pthread_mutex_unlock ( &olf_mutex );

     info->width   = data->tfaRequestGlyph.in_GlyphSize.x;
     info->height  = data->tfaRequestGlyph.in_GlyphSize.y;

     return ret;
}

static DFBResult
olf_render_glyph( CoreFont      *thiz,
              unsigned int   index,
              CoreGlyphData *info )
{
     DFBResult err;
     OlfImplData *data    = thiz->impl_data;
     CoreSurface *surface = info->surface;
     CoreSurfaceBufferLock  lock;
     u8          *src;
     int          y;
     int render_width;

     err = dfb_surface_lock_buffer( surface, CSBR_BACK, CSAID_CPU, CSAF_WRITE, &lock );
     if (err) {
          D_DERROR( err, "DirectFB/FontOLF: Unable to lock surface!\n" );
          return err;
     }

     info->width = data->tfaRequestGlyph.in_GlyphSize.x;
     if (info->width + info->start > surface->config.size.w)
          info->width = surface->config.size.w - info->start;

     info->height = data->tfaRequestGlyph.in_GlyphSize.y;
     if (info->height > surface->config.size.h)
          info->height = surface->config.size.h;

     info->advance = info->width;
     info->left = 0;
     info->top  = 0;

     src = data->tfaRequestGlyph.out_pGlyphBuffer;
     lock.addr += DFB_BYTES_PER_LINE(surface->config.format, info->start);

     render_width = info->width;

     for (y=0; y < info->height; y++) {
          int  i, j, n;
          u8  *dst8  = lock.addr;
          u16 *dst16 = lock.addr;
          u32 *dst32 = lock.addr;

          switch (data->tfaRequestGlyph.in_Command & FA_GSCALE_MASK) {
               case FA_GSCALE_16:
                    switch (surface->config.format) {
                         case DSPF_A8:
                              for (i = 0; i < render_width; i ++) 
                              {
                                  dst8[i] = (src[i] & 0x0f) * 16; //convert grey 16 to grey 256
                              }
                              break;
                         default:
                              D_UNIMPLEMENTED();
                              break;
                    }
                    break;

               default:
                    break;

          }

          src += render_width;

          lock.addr += lock.pitch;
     }

     dfb_surface_unlock_buffer( surface, &lock );

     return DFB_OK;
}


static void
IDirectFBFont_OLF_Destruct( IDirectFBFont *thiz )
{
     IDirectFBFont_data *data = (IDirectFBFont_data*)thiz->priv;

     if (data->font->impl_data) {
          OlfImplData *impl_data = (OlfImplData*) data->font->impl_data;

          pthread_mutex_lock ( &olf_mutex );

          FaFinalize(impl_data->face, NULL);

          pthread_mutex_unlock ( &olf_mutex );

          D_FREE(impl_data->face->pFontBlocks->pFontElements);
          D_FREE(impl_data->face->pFontBlocks);
          D_FREE(impl_data->face->pDecodeBuffer);
          D_FREE(impl_data->face->pGscaleBuffer);
          D_FREE(impl_data->face);
          D_FREE( impl_data );

          data->font->impl_data = NULL;

          global_font_data_ref --;

          if ((global_font_data_ref == 0)  &&
              (global_font_data))
          {
              D_FREE(global_font_data);
              global_font_data = NULL;
          }

     }

     IDirectFBFont_Destruct( thiz );
}

static DirectResult
IDirectFBFont_OLF_Release( IDirectFBFont *thiz )
{
     DIRECT_INTERFACE_GET_DATA(IDirectFBFont)

     if (--data->ref == 0) {
          IDirectFBFont_OLF_Destruct( thiz );
     }

     return DFB_OK;
}

static DFBResult
Probe( IDirectFBFont_ProbeContext *ctx )
{
     D_DEBUG( "DirectFB/OLF: Probe font `%s'.\n", ctx->filename );

     if ((!ctx->filename) &&
         !(ctx->data_buffer))
          return DFB_UNSUPPORTED;

     return DFB_OK;
}

static DFBResult
Construct( IDirectFBFont *thiz,
	   ... )
{
     int i;
     fa_int faSts = FA_SUCCESS;
     fa_int    fa_ret;
     DFBResult           ret;
	 CoreFont           *font;
     CoreDFB            *core;
     char               *filename;
     bool                disable_charmap = false;
     bool                disable_kerning = false;
     bool                load_mono = false;
     DFBFontDescription *desc;
     FILE *font_handle = NULL;
	 void * font_data = NULL;
     int font_data_size;
     PTfa_FontElement font_element;
     PTfa_FontBlock font_block;
     PTfa_Buffers font_buffer;
     OlfImplData * data;
     int GscaleBuffer_width;
     int GscaleBuffer_height;

     va_list tag;
     va_start(tag, thiz);
     core = va_arg(tag, CoreDFB *);
     filename = va_arg(tag, char *);
     desc = va_arg(tag, DFBFontDescription *);
     va_end( tag );

     D_DEBUG( "DirectFB/FontOLF: "
              "Construct font from file `%s' (index %d) at pixel size %d x %d.\n",
              filename,
              (desc->flags & DFDESC_INDEX)  ? desc->index  : 0,
              (desc->flags & DFDESC_WIDTH)  ? desc->width  : 0,
              (desc->flags & DFDESC_HEIGHT) ? desc->height : 0 );



     if (desc->flags & DFDESC_FROM_BUFFER) 
     {
         //font_data = D_CALLOC( 1, desc->data_buffer_size );

         //direct_memcpy ((void *)font_data, desc->data_buffer, desc->data_buffer_size); 

         font_data = desc->data_buffer;
     }
     else
     {

         if (global_font_data == NULL) {

             font_handle = fopen(filename,"rb");

             if ((font_handle == NULL))
             {
                 D_DEBUG(" DirectFB/FontOLF:  can't open font file %s\n", filename); 
                 DIRECT_DEALLOCATE_INTERFACE( thiz );
                 return DFB_FAILURE;
             }
             else
             {

                  //read font date to buffer
                  fseek(font_handle,0,SEEK_END);
                  font_data_size = ftell(font_handle);
                  fseek(font_handle,0,SEEK_SET);

                  font_data = D_CALLOC( 1, font_data_size );

                  fread(font_data, 1 , font_data_size, font_handle);
                  fclose(font_handle);

                  global_font_data = font_data;
                  global_font_data_ref++;
             }
         }
         else
         {
             font_data = global_font_data;
             global_font_data_ref++;

         }
    
     }

     font_element = D_CALLOC(1, sizeof(Tfa_FontElement));
     font_element->pFont =  (fa_dword *)font_data;

     font_block = D_CALLOC(1, sizeof(Tfa_FontBlock));
     font_block->pFontElements = font_element;
     font_block->NumberOfFontElements = 1;
     
     font_buffer = D_CALLOC(1, sizeof (Tfa_Buffers));
     font_buffer->NumberOfFontBlocks = 1;
     font_buffer->pFontBlocks = font_block;
     font_buffer->pDecodeBuffer = (fa_fixed*)D_CALLOC(1, FA_DECODE_BUFSIZE);
     font_buffer->lengthDecodeBuffer = FA_DECODE_BUFSIZE;

     if (desc->flags & DFDESC_HEIGHT)
     {
          GscaleBuffer_height  = desc->height;	 	 
     }
     else if (desc->flags & DFDESC_WIDTH) 
     {
          GscaleBuffer_height = desc->width;	 	 
     }

     if (desc->flags & DFDESC_WIDTH)
     {
          GscaleBuffer_width = desc->width;	 	 
     }
     else if (desc->flags & DFDESC_HEIGHT) 
     {
          GscaleBuffer_width = desc->height;	 	 
     }


 	font_buffer->lengthGscaleBuffer = GscaleBuffer_width * GscaleBuffer_height * sizeof (fa_fixed);
     if (font_buffer->lengthGscaleBuffer < FA_GSCALE_BUFSIZE_MIN) 
     {
         font_buffer->lengthGscaleBuffer = FA_GSCALE_BUFSIZE_MIN;
     }
 	font_buffer->pGscaleBuffer = (fa_fixed*)D_CALLOC(1, font_buffer->lengthGscaleBuffer);

     pthread_mutex_lock ( &olf_mutex );

     fa_ret = FaInitialize(font_buffer, NULL);

     pthread_mutex_unlock ( &olf_mutex );

     if (fa_ret) 
     {
          D_DEBUG(" DirectFB/FontOLF:  can't initizlize the font buffer \n"); 

          FaFinalize(font_buffer, NULL);

          D_FREE(font_data);
          D_FREE(font_element);
          D_FREE(font_block);
          D_FREE(font_buffer->pDecodeBuffer);
          D_FREE(font_buffer->pGscaleBuffer);
          D_FREE(font_buffer);


          DIRECT_DEALLOCATE_INTERFACE( thiz );

          return DFB_FAILURE;
     }

     if (dfb_config->font_format == DSPF_A1 ||
               dfb_config->font_format == DSPF_A1_LSB || 
               dfb_config->font_format == DSPF_ARGB1555 ||
               dfb_config->font_format == DSPF_RGBA5551)
          load_mono = true;

     if (desc->flags & DFDESC_ATTRIBUTES) {
          if (desc->attributes & DFFA_NOCHARMAP)
               disable_charmap = true;
          if (desc->attributes & DFFA_NOKERNING)
               disable_kerning = true;
          if (desc->attributes & DFFA_MONOCHROME)
               load_mono = true;
     }


     ret = dfb_font_create( core, &font );
     if (ret) {
          pthread_mutex_lock ( &olf_mutex );
          FaFinalize(font_buffer, NULL);

          D_FREE(font_data);
          D_FREE(font_element);
          D_FREE(font_block);
          D_FREE(font_buffer->pDecodeBuffer);
          D_FREE(font_buffer->pGscaleBuffer);
          D_FREE(font_buffer);

          pthread_mutex_unlock ( &olf_mutex );
          DIRECT_DEALLOCATE_INTERFACE( thiz );
          return ret;
     }

     D_ASSERT( font->pixel_format == DSPF_ARGB ||
               font->pixel_format == DSPF_AiRGB ||
               font->pixel_format == DSPF_ARGB8565 ||
               font->pixel_format == DSPF_ARGB4444 ||
               font->pixel_format == DSPF_RGBA4444 ||
               font->pixel_format == DSPF_ARGB2554 ||
               font->pixel_format == DSPF_ARGB1555 ||
               font->pixel_format == DSPF_RGBA5551 ||
               font->pixel_format == DSPF_A8 ||
               font->pixel_format == DSPF_A4 ||
               font->pixel_format == DSPF_A1_LSB ||
               font->pixel_format == DSPF_A1 );

     font->descender  = 0;
     font->ascender   = 1;
     font->pixel_format = DSPF_A8;

     D_DEBUG( "DirectFB/FontOLF: height = %d, ascender = %d, descender = %d, maxadvance = %d\n",
              font->height, font->ascender, font->descender, font->maxadvance );

     data = D_CALLOC(1, sizeof(OlfImplData));

     font->GetGlyphData = olf_get_glyph_info;
     font->RenderGlyph  = olf_render_glyph;
     font->current_encoding = DTEID_SHIFT_JIS;
     font->disabled = DFXL_ALL;

     data->face            = font_buffer;
     data->disable_charmap = disable_charmap;
     data->width = desc->width;
     data->height = desc->height;

     data->tfaRequestGlyph.in_EncodingId = FA_ENCODING_JIS_ENCODING;
     data->tfaRequestGlyph.in_FontId = 0;

     data->tfaRequestGlyph.in_GlyphSize.y = data->height;	 	 
     data->tfaRequestGlyph.in_Edge = 0;	 	 	 	 
     data->tfaRequestGlyph.in_Shadow.x = 0;	 	 	 
     data->tfaRequestGlyph.in_Shadow.y = 0;	 	 	 
     data->tfaRequestGlyph.in_Slant.x = 0;	 	 	 
     data->tfaRequestGlyph.in_Slant.y = 0;	 	 	 
     data->tfaRequestGlyph.inout_pRotate = NULL;	 	 
     data->tfaRequestGlyph.in_Weight = 0;	 	 	 

     if (desc->flags & DFDESC_HEIGHT)
     {
          data->tfaRequestGlyph.in_GlyphSize.y = desc->height;	 	 
     }
     else if (desc->flags & DFDESC_WIDTH) 
     {
          data->tfaRequestGlyph.in_GlyphSize.y = desc->width;	 	 
     }

     if (desc->flags & DFDESC_WIDTH)
     {
          data->tfaRequestGlyph.in_GlyphSize.x = desc->width;	 	 
          data->input_desc_widht = desc->width;
     }
     else if (desc->flags & DFDESC_HEIGHT) 
     {
          data->tfaRequestGlyph.in_GlyphSize.x = desc->height;	 	 
          data->input_desc_widht = desc->height;
     }

     data->tfaRequestGlyph.in_Command = (FA_STYLE_STANDARD | FA_DECORA_GRADATION_OFF);
     if (desc->attributes & DFFA_NOKERNING ) 
     {
          data->tfaRequestGlyph.in_Command |= FA_HINT_OFF;
     }
     else
     {
          data->tfaRequestGlyph.in_Command |= FA_HINT_ON;
     }
     
     if (desc->attributes & DFFA_MONOCHROME  ) 
     {
          data->tfaRequestGlyph.in_Command |= FA_GSCALE_16;
     }
     else
     {
          data->tfaRequestGlyph.in_Command |= FA_GSCALE_256;
     }

     data->tfaRequestGlyph.in_pQparam = NULL;	 	 
     data->tfaRequestGlyph.out_pMetrics = &(data->tfaMetrics);
     data->tfaRequestGlyph.inout_lengthGlyphBuffer =  data->tfaRequestGlyph.in_GlyphSize.x * data->tfaRequestGlyph.in_GlyphSize.y;
     data->tfaRequestGlyph.out_pGlyphBuffer  =  (fa_byte*)D_CALLOC(1, data->tfaRequestGlyph.inout_lengthGlyphBuffer );

     font->height     = desc->height;

     if (desc->flags & DFDESC_FIXEDADVANCE) {
          data->fixed_advance = desc->fixed_advance;
          font->maxadvance    = desc->fixed_advance;

          if ((desc->flags & DFDESC_ATTRIBUTES) && (desc->attributes & DFFA_FIXEDCLIP))
               data->fixed_clip = true;
     }

     for (i=0; i<256; i++)
          data->indices[i] = Single_Byte_Code_To_Kana_Code(i);

     font->impl_data = data;

     font->last_encoding = DTEID_JIS;

     data->tfaRequestGlyph.in_Code = 0x3021;
     faSts = FaGetRegionSize(data->face, &data->tfaRequestGlyph);
     if (faSts == FA_SUCCESS) 
     {

          faSts = FaGetGlyphImage(data->face, NULL, &data->tfaRequestGlyph);

          font->ascender = data->tfaRequestGlyph.out_GlyphSize.y - 1;

     }


     dfb_font_register_encoding( font, "UTF8",   &olf2UTF8Funcs,   DTEID_UTF8 );

     IDirectFBFont_Construct( thiz, font );

     thiz->Release = IDirectFBFont_OLF_Release;

     return DFB_OK;
}


u16 Single_Byte_Code_To_Kana_Code (u8 code)
{
     u16 kana_code;

	switch (code) 
     {
	    case 0x20:	kana_code = 0x8140;		break;		/* [ ] */
	    case 0x21:	kana_code = 0x8149;		break;		/* [!] */
	    case 0x22:	kana_code = 0x8168;		break;		/* ["] */
	    case 0x23:	kana_code = 0x8194;		break;		/* [#] */
	    case 0x24:	kana_code = 0x8190;		break;		/* [$] */
	    case 0x25:	kana_code = 0x8193;		break;		/* [%] */
	    case 0x26:	kana_code = 0x8195;		break;		/* [&] */
	    case 0x27:	kana_code = 0x8166;		break;		/* ['] */
	    case 0x28:	kana_code = 0x8169;		break;		/* [(] */
	    case 0x29:	kana_code = 0x816a;		break;		/* [)] */
	    case 0x2a:	kana_code = 0x8196;		break;		/* [*] */
	    case 0x2b:	kana_code = 0x817b;		break;		/* [+] */
	    case 0x2c:	kana_code = 0x8143;		break;		/* [,] */
	    case 0x2d:	kana_code = 0x817c;		break;		/* [~] */
	    case 0x2e:	kana_code = 0x8144;		break;		/* [.] */
	    case 0x2f:	kana_code = 0x815e;		break;		/* [/] */
	    case 0x30:	kana_code = 0x824f;		break;		/* [0] */
	    case 0x31:	kana_code = 0x8250;		break;		/* [1] */
	    case 0x32:	kana_code = 0x8251;		break;		/* [2] */
	    case 0x33:	kana_code = 0x8252;		break;		/* [3] */
	    case 0x34:	kana_code = 0x8253;		break;		/* [4] */
	    case 0x35:	kana_code = 0x8254;		break;		/* [5] */
	    case 0x36:	kana_code = 0x8255;		break;		/* [6] */
	    case 0x37:	kana_code = 0x8256;		break;		/* [7] */
	    case 0x38:	kana_code = 0x8257;		break;		/* [8] */
	    case 0x39:	kana_code = 0x8258;		break;		/* [9] */
	    case 0x3a:	kana_code = 0x8146;		break;		/* [:] */
	    case 0x3b:	kana_code = 0x8147;		break;		/* [;] */
	    case 0x3c:	kana_code = 0x8183;		break;		/* [<] */
	    case 0x3d:	kana_code = 0x8181;		break;		/* [=] */
	    case 0x3e:	kana_code = 0x8184;		break;		/* [>] */
	    case 0x3f:	kana_code = 0x8148;		break;		/* [?] */
	    case 0x40:	kana_code = 0x8197;		break;		/* [@] */
	    case 0x41:	kana_code = 0x8260;		break;		/* [A] */
	    case 0x42:	kana_code = 0x8261;		break;		/* [B] */
	    case 0x43:	kana_code = 0x8262;		break;		/* [C] */
	    case 0x44:	kana_code = 0x8263;		break;		/* [D] */
	    case 0x45:	kana_code = 0x8264;		break;		/* [E] */
	    case 0x46:	kana_code = 0x8265;		break;		/* [F] */
	    case 0x47:	kana_code = 0x8266;		break;		/* [G] */
	    case 0x48:	kana_code = 0x8267;		break;		/* [H] */
	    case 0x49:	kana_code = 0x8268;		break;		/* [I] */
	    case 0x4a:	kana_code = 0x8269;		break;		/* [J] */
	    case 0x4b:	kana_code = 0x826a;		break;		/* [K] */
	    case 0x4c:	kana_code = 0x826b;		break;		/* [L] */
	    case 0x4d:	kana_code = 0x826c;		break;		/* [M] */
	    case 0x4e:	kana_code = 0x826d;		break;		/* [N] */
	    case 0x4f:	kana_code = 0x826e;		break;		/* [O] */
	    case 0x50:	kana_code = 0x826f;		break;		/* [P] */
	    case 0x51:	kana_code = 0x8270;		break;		/* [Q] */
	    case 0x52:	kana_code = 0x8271;		break;		/* [R] */
	    case 0x53:	kana_code = 0x8272;		break;		/* [S] */
	    case 0x54:	kana_code = 0x8273;		break;		/* [T] */
	    case 0x55:	kana_code = 0x8274;		break;		/* [U] */
	    case 0x56:	kana_code = 0x8275;		break;		/* [V] */
	    case 0x57:	kana_code = 0x8276;		break;		/* [W] */
	    case 0x58:	kana_code = 0x8277;		break;		/* [X] */
	    case 0x59:	kana_code = 0x8278;		break;		/* [Y] */
	    case 0x5a:	kana_code = 0x8279;		break;		/* [Z] */
	    case 0x5b:	kana_code = 0x816d;		break;		/* [[] */
	    case 0x5c:	kana_code = 0x818f;		break;		/* [\] */
	    case 0x5d:	kana_code = 0x816e;		break;		/* []] */
	    case 0x5e:	kana_code = 0x814f;		break;		/* [^] */
	    case 0x5f:	kana_code = 0x8151;		break;		/* [_] */
	    case 0x60:	kana_code = 0x8165;		break;		/* ['] */
	    case 0x61:	kana_code = 0x8281;		break;		/* [a] */
	    case 0x62:	kana_code = 0x8282;		break;		/* [b] */
	    case 0x63:	kana_code = 0x8283;		break;		/* [c] */
	    case 0x64:	kana_code = 0x8284;		break;		/* [d] */
	    case 0x65:	kana_code = 0x8285;		break;		/* [e] */
	    case 0x66:	kana_code = 0x8286;		break;		/* [f] */
	    case 0x67:	kana_code = 0x8287;		break;		/* [g] */
	    case 0x68:	kana_code = 0x8288;		break;		/* [h] */
	    case 0x69:	kana_code = 0x8289;		break;		/* [i] */
	    case 0x6a:	kana_code = 0x828a;		break;		/* [j] */
	    case 0x6b:	kana_code = 0x828b;		break;		/* [k] */
	    case 0x6c:	kana_code = 0x828c;		break;		/* [l] */
	    case 0x6d:	kana_code = 0x828d;		break;		/* [m] */
	    case 0x6e:	kana_code = 0x828e;		break;		/* [n] */
	    case 0x6f:	kana_code = 0x828f;		break;		/* [o] */
	    case 0x70:	kana_code = 0x8290;		break;		/* [p] */
	    case 0x71:	kana_code = 0x8291;		break;		/* [q] */
	    case 0x72:	kana_code = 0x8292;		break;		/* [r] */
	    case 0x73:	kana_code = 0x8293;		break;		/* [s] */
	    case 0x74:	kana_code = 0x8294;		break;		/* [t] */
	    case 0x75:	kana_code = 0x8295;		break;		/* [u] */
	    case 0x76:	kana_code = 0x8296;		break;		/* [v] */
	    case 0x77:	kana_code = 0x8297;		break;		/* [w] */
	    case 0x78:	kana_code = 0x8298;		break;		/* [x] */
	    case 0x79:	kana_code = 0x8299;		break;		/* [y] */
	    case 0x7a:	kana_code = 0x829a;		break;		/* [z] */
	    case 0x7b:	kana_code = 0x816f;		break;		/* [{] */
	    case 0x7c:	kana_code = 0x8162;		break;		/* [|] */
	    case 0x7d:	kana_code = 0x8170;		break;		/* [}] */
         case 0x7e:	kana_code = 0x8160;		break;		/* [~] */
         case 0xa1:	kana_code = 0x8142;		break;		/* [?B] */
         case 0xa2:	kana_code = 0x8175;		break;		/* [?u] */
         case 0xa3:	kana_code = 0x8176;		break;		/* [?v] */
         case 0xa4:	kana_code = 0x8141;		break;		/* [?A] */
         case 0xa5:	kana_code = 0x8145;		break;		/* [?E] */
         case 0xb0:	kana_code = 0x815b;		break;		/* [?|] */
         case 0xde:	kana_code = 0x814a;		break;		/* [?J] */
         case 0xdf:	kana_code = 0x814b;		break;		/* [?K] */
         
         case 0xa7:	kana_code = 0x8340;		break;		/* [?@] */
         case 0xa8:	kana_code = 0x8342;		break;		/* [?B] */
         case 0xa9:	kana_code = 0x8344;		break;		/* [?D] */
         case 0xaa:	kana_code = 0x8346;		break;		/* [?F] */
         case 0xab:	kana_code = 0x8348;		break;		/* [?H] */
         case 0xac:	kana_code = 0x8383;		break;		/* [??] */
         case 0xad:	kana_code = 0x8385;		break;		/* [??] */
         case 0xae:	kana_code = 0x8387;		break;		/* [??] */
         case 0xaf:	kana_code = 0x8362;		break;		/* [?b] */
         
         case 0xa6:	kana_code = 0x8392;		break;		/* [??] */
         case 0xb1:	kana_code = 0x8341;		break;		/* [?A] */
         case 0xb2:	kana_code = 0x8343;		break;		/* [?C] */
         case 0xb3:	kana_code = 0x8345;		break;		/* [?E] */
         case 0xb4:	kana_code = 0x8347;		break;		/* [?G] */
         case 0xb5:	kana_code = 0x8349;		break;		/* [?I] */
         case 0xb6:	kana_code = 0x834a;		break;		/* [?J] */
         case 0xb7:	kana_code = 0x834c;		break;		/* [?L] */
         case 0xb8:	kana_code = 0x834e;		break;		/* [?N] */
         case 0xb9:	kana_code = 0x8350;		break;		/* [?P] */
         case 0xba:	kana_code = 0x8352;		break;		/* [?R] */
         case 0xbb:	kana_code = 0x8354;		break;		/* [?T] */
         case 0xbc:	kana_code = 0x8356;		break;		/* [?V] */
         case 0xbd:	kana_code = 0x8358;		break;		/* [?X] */
         case 0xbe:	kana_code = 0x835a;		break;		/* [?Z] */
         case 0xbf:	kana_code = 0x835c;		break;		/* [?\] */
         
         case 0xc0:	kana_code = 0x835e;		break;		/* [?^] */
         case 0xc1:	kana_code = 0x8360;		break;		/* [?`] */
         case 0xc2:	kana_code = 0x8363;		break;		/* [?c] */
         case 0xc3:	kana_code = 0x8365;		break;		/* [?e] */
         case 0xc4:	kana_code = 0x8367;		break;		/* [?g] */
         case 0xc5:	kana_code = 0x8369;		break;		/* [?i] */
         case 0xc6:	kana_code = 0x836a;		break;		/* [?j] */
         case 0xc7:	kana_code = 0x836b;		break;		/* [?k] */
         case 0xc8:	kana_code = 0x836c;		break;		/* [?l] */
         case 0xc9:	kana_code = 0x836d;		break;		/* [?m] */
         case 0xca:	kana_code = 0x836e;		break;		/* [?n] */
         case 0xcb:	kana_code = 0x8371;		break;		/* [?q] */
         case 0xcc:	kana_code = 0x8374;		break;		/* [?t] */
         case 0xcd:	kana_code = 0x8377;		break;		/* [?w] */
         case 0xce:	kana_code = 0x837a;		break;		/* [?z] */
         case 0xcf:	kana_code = 0x837d;		break;		/* [?}] */
         
         case 0xd0:	kana_code = 0x837e;		break;		/* [?~] */
         case 0xd1:	kana_code = 0x8380;		break;		/* [??] */
         case 0xd2:	kana_code = 0x8381;		break;		/* [??] */
         case 0xd3:	kana_code = 0x8382;		break;		/* [??] */
         case 0xd4:	kana_code = 0x8384;		break;		/* [??] */
         case 0xd5:	kana_code = 0x8386;		break;		/* [??] */
         case 0xd6:	kana_code = 0x8388;		break;		/* [??] */
         case 0xd7:	kana_code = 0x8389;		break;		/* [??] */
         case 0xd8:	kana_code = 0x838a;		break;		/* [??] */
         case 0xd9:	kana_code = 0x838b;		break;		/* [??] */
         case 0xda:	kana_code = 0x838c;		break;		/* [??] */
         case 0xdb:	kana_code = 0x838d;		break;		/* [??] */
         case 0xdc:	kana_code = 0x838f;		break;		/* [??] */
         case 0xdd:	kana_code = 0x8393;		break;		/* [??] */
         
	    default:	     kana_code = 0;        break;
	}

     return kana_code;
 }

