/************************************************** 
*
*  Copyright (c) 2006-2012 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xc4_img_api.c  
*
*  DESCRIPTION:   
*               
*
*  AUTHOR:
*
*  HISTORY
*
*  $Log: not supported by cvs2svn $
*  Revision 1.0  2012/02/28 01:17:47  kchan
*  Initial implementation
*
***************************************************/
#include <directfb.h>
#include "xcode4_img_api.h"
#include "drv_if.h"
#include <time.h>

#define MALLOC(t) (t*) malloc(sizeof(t))
#define FREE(p) if (p) free(p);

//#define MSG_INFO(...)  printf(__VA_ARGS__); printf("\n")
#define MSG_INFO(...)  
#define MSG_ERROR(...) printf(__VA_ARGS__); printf("\n")

//IOCTL_NO_TIMEOUT
#define IOCTL_TIMEOUTVALUE_JPEG  20000 //20s control
#define VIDEO_DECODER_SEND_BUFFER_SIZE  0x40000

typedef struct ImgContext
{
    int            h_drv_main;
    int            h_drv_send;
    int            h_drv_recv;

    uint64         h_stream;
    uint8          *send_buf_user_addr;    
    unsigned int   send_buf_phy_addr;
    unsigned int   send_buf_size;
    int            is_initialized;
    int            is_opened;
    int            width;
    int            height;
    int            dst_image_size;
    unsigned int   dst_image_phy_addr;
} ImgContext_t;  


void *xcode_get_mmr_map_addr() 
{
     int h_drv_mmio_base;
     IOCTLDEVFRAMEBUFFER ioctl_devframebuffer;
     void * m_mmio_base;

     h_drv_mmio_base = open(DEVICE_NAME, O_RDWR);
     if (h_drv_mmio_base <= 0) {
          D_ERROR("Fail to open the drive mmio_base handle!");
          return NULL;
     }

     // Get MMR address
     memset(&ioctl_devframebuffer, 0, sizeof(IOCTLDEVFRAMEBUFFER));
     ioctl_devframebuffer.ioctl_header.command = CMD_GET_MMR_ADDR;
     ioctl_devframebuffer.ioctl_header.size_of_buf = sizeof(IOCTLDEVFRAMEBUFFER);
     ioctl_devframebuffer.ioctl_header.time_out_value = 500;
     ioctl_devframebuffer.ioctl_header.return_value = IOCTL_RET_CMD_FAILED;

     ioctl(h_drv_mmio_base, IOCTL_CMD, &ioctl_devframebuffer); 

     if (ioctl_devframebuffer.ioctl_header.return_value == IOCTL_RET_CMD_COMPLETE)
     {
          m_mmio_base = (void *)ioctl_devframebuffer.pbuffer;
     }
     else
     {
          D_ERROR("DFB HW Decoder: GET_MMRADDR failed ret = %d\n",
               ioctl_devframebuffer.ioctl_header.return_value);
          m_mmio_base = NULL;
     }

     if(h_drv_mmio_base > 0) {
          close(h_drv_mmio_base);
     }

     return m_mmio_base;
}


int send_img_open_command(ImgContext_t *ctx, IOCTLSDEVOPENDECODE2 *pioctl_open_decode)
{
    pioctl_open_decode->ioctl_header.command          = CMD2_SDEV_OPEN_DECODE;
    pioctl_open_decode->ioctl_header.size_of_buf      = sizeof(IOCTLSDEVOPENDECODE2);
    pioctl_open_decode->ioctl_header.time_out_value   = IOCTL_TIMEOUTVALUE_JPEG;
    pioctl_open_decode->ioctl_header.return_value     = IOCTL_RET_CMD_FAILED;

    pioctl_open_decode->stream_handle = 0;

    ioctl(ctx->h_drv_main, IOCTL_CMD, pioctl_open_decode);

    if (pioctl_open_decode->ioctl_header.return_value == IOCTL_RET_CMD_COMPLETE)
    {
        ctx->h_stream = pioctl_open_decode->stream_handle;
        return 0;
    }

    MSG_ERROR("Fail to open firmware image decoder, error code: %x!", pioctl_open_decode->ioctl_header.return_value);
    return -1;
}

int send_img_stream_command(ImgContext_t *ctx, IOCTLSDEVSEND *pioctl_send_strm,
                        unsigned int phy_addr, unsigned int *pbufsize, uint64 flags, unsigned int more, uint64 pts, uint64 dts)
{
    pioctl_send_strm->ioctl_header.command          = CMD_SDEV_SEND;
    pioctl_send_strm->ioctl_header.size_of_buf      = sizeof(IOCTLSDEVSEND);
    pioctl_send_strm->ioctl_header.time_out_value   = IOCTL_TIMEOUTVALUE_JPEG;
    pioctl_send_strm->ioctl_header.return_value     = IOCTL_RET_CMD_FAILED;

    pioctl_send_strm->stream_handle                 = ctx->h_stream;
    if (pts)
    {
        flags |= SDEV_IO_PTS;
    }
    if (dts)
    {
        flags |= SDEV_IO_DTS;
    }
    pioctl_send_strm->flags                         = flags;
    pioctl_send_strm->pts                           = pts;
    pioctl_send_strm->dts                           = dts;
    pioctl_send_strm->buffer                        = (pvoid)phy_addr;
    pioctl_send_strm->size                          = *pbufsize;
    //#if defined(XCODE3_CHIP) || defined(XCODE4_CHIP) || defined(XCODE5_CHIP)
    pioctl_send_strm->more                          = more;
    //#endif

    ioctl(ctx->h_drv_send, IOCTL_CMD, pioctl_send_strm);

    if (pioctl_send_strm->ioctl_header.return_value == IOCTL_RET_CMD_COMPLETE)
    {
        *pbufsize = pioctl_send_strm->size;
        return 0;
    }

    MSG_ERROR("Fail to send data to firmware, error code: %x!", pioctl_send_strm->ioctl_header.return_value);
    return -1;
}

int read_img_decode_status_command(ImgContext_t *ctx, IOCTLSDEVRECVFRAMEPTR *pioctl_recv_frame_ptr)
{
    //clock_t start_time, end_time;

    pioctl_recv_frame_ptr->ioctl_header.command          = CMD2_SDEV_RECV_FRAME_PTR;
    pioctl_recv_frame_ptr->ioctl_header.size_of_buf      = sizeof(IOCTLSDEVRECVFRAMEPTR);
    pioctl_recv_frame_ptr->ioctl_header.time_out_value   = IOCTL_TIMEOUTVALUE_JPEG;
    pioctl_recv_frame_ptr->ioctl_header.return_value     = IOCTL_RET_CMD_PROCESSING;

    pioctl_recv_frame_ptr->ioctl_param.mode = SDEV2_IO_VIDEO;
    pioctl_recv_frame_ptr->stream_handle = ctx->h_stream;

    //start_time = clock();    
    do {
        ioctl(ctx->h_drv_recv, IOCTL_CMD, pioctl_recv_frame_ptr);
        //end_time = clock();        
    } while (//(((end_time-start_time) * 1000 / CLOCKS_PER_SEC) < IOCTL_TIMEOUTVALUE_JPEG) &&
             pioctl_recv_frame_ptr->ioctl_header.return_value == IOCTL_RET_CMD_PROCESSING);

    if (pioctl_recv_frame_ptr->ioctl_header.return_value == IOCTL_RET_CMD_COMPLETE)
    {
        return 0;
    }
    else
    {
        MSG_ERROR("Fail to read the decoding status from firmware, error code = %x!", pioctl_recv_frame_ptr->ioctl_header.return_value);
        return -1;
    }
}

int send_img_close_command(ImgContext_t *ctx)
{
    IOCTLSDEVCLOSE2 ioctl_close_strm2;

    memset(&ioctl_close_strm2, 0, sizeof(IOCTLSDEVCLOSE2));

    ioctl_close_strm2.ioctl_header.command             = CMD2_SDEV_CLOSE;
    ioctl_close_strm2.ioctl_header.size_of_buf         = sizeof(IOCTLSDEVCLOSE2);
    ioctl_close_strm2.ioctl_header.time_out_value      = IOCTL_TIMEOUTVALUE_JPEG;
    ioctl_close_strm2.ioctl_header.return_value        = IOCTL_RET_CMD_FAILED;

    // handle of stream to close
    ioctl_close_strm2.stream_handle                    = ctx->h_stream;

    ioctl(ctx->h_drv_main, IOCTL_CMD, &ioctl_close_strm2);

    if (ioctl_close_strm2.ioctl_header.return_value == IOCTL_RET_CMD_COMPLETE)
    {
        return 0;
    }
    else
    {
        MSG_ERROR("Fail to send a close command to firmware, error_code=%x!",ioctl_close_strm2.ioctl_header.return_value);
        return -1;
    }
}

int send_img_stream(ImgContext_t *ctx, unsigned int phy_addr, int *size, uint64 flags, int frame_end, int trick_mode)
{
    IOCTLSDEVSEND ioctl_send_video_strm;
    int ret;

    if ((*size <= 0) && !(flags & SDEV_FLUSH_INPUT))
    {
        return 0;
    }

    memset(&ioctl_send_video_strm, 0, sizeof(IOCTLSDEVSEND));

    ioctl_send_video_strm.trick_mode = trick_mode;

    // no still picture mode?

    ioctl_send_video_strm.playback_speedx100 = 100;
    ioctl_send_video_strm.playback_direction = 0;
    ret = send_img_stream_command(ctx, &ioctl_send_video_strm, phy_addr, (unsigned int *)size, flags, frame_end ? 0 : 1, 0, 0);

    return ret;
}

int read_decode_status(ImgContext_t *ctx)
{
    int ret = 0;
    IOCTLSDEVRECVFRAMEPTR ioctl_recv_fp;

    memset(&ioctl_recv_fp, 0, sizeof(IOCTLSDEVRECVFRAMEPTR));

    ret = read_img_decode_status_command(ctx, &ioctl_recv_fp);
    if (!ret)
    {
        if (ioctl_recv_fp.ioctl_param.sizeVideoFrameY == 0)
        {
            MSG_ERROR("Fail to decode the image in firmware!");

            // According to Jim Wang, the release_buffer() call is needed for video only not img decoding.
            ret = -1;
        }
    }
    return ret;
}

void *img_decode_init()
{
    ImgContext_t *ctx = (ImgContext_t *)MALLOC(ImgContext_t);
    if (ctx)
    {
        memset(ctx, 0, sizeof(*ctx));

        ctx->h_drv_main = open(DEVICE_NAME, O_RDWR);
        if (ctx->h_drv_main > 0)
        {
            ctx->h_drv_send = open(DEVICE_NAME, O_RDWR);
            if (ctx->h_drv_send > 0)
            {
                ctx->h_drv_recv = open(DEVICE_NAME, O_RDWR);
                if (ctx->h_drv_recv > 0)
                {
                    ctx->is_initialized = 1;
                    return ctx;    
                }
                else
                {
                    MSG_ERROR("Fail to open the driver recv handle!");
                }
            }
            else
            {
                MSG_ERROR("Fail to open the driver send handle!");
            }
        }
        else
        {
            MSG_ERROR("Fail to open the driver main handle!");
        }
    }
    else
    {
        MSG_ERROR("Fail to allocate memory for firmware image module context!");
    }

    FREE(ctx);

    return 0;
}

int img_decoder_open(void *c, unsigned int buf_width, unsigned int buf_height, unsigned int buf_size,
                         unsigned int out_buf_add_phy, DFBSurfacePixelFormat format, DFB_HW_img_type img_type)
{
    ImgContext_t *ctx = (ImgContext_t *)c;

    int ret = 0;
    IOCTLSDEVOPENDECODE2 ioctl_open_strm_decode;
    param2_opendecode *param;

    //unsigned int buf_size;
    
    int format_to_pass;

    if (format == DSPF_ARGB)
    {
         //buf_size = buf_width * buf_height * 4; //size for the ARGB
         format_to_pass = RGB;
    }
    else if (format == DSPF_YV16)
    {
         //buf_size = buf_width * buf_height * 2; //size for the YUV422
         format_to_pass = YUV_4_2_2;
    }
    else if (format == DSPF_YUV444P)
    {
         //buf_size = buf_width * buf_height * 3; //size for the YUV444
         format_to_pass = YUV_4_4_4;
    }
    else 
    {
         //buf_size = buf_width * buf_height * 3 / 2; //size for the I420
         format_to_pass = YUV_4_2_0;
    }

    MSG_INFO( "Type: %s, Width: %d, Height: %d Phy Addr: (0x%08x)\n"
	      , dfb_pixelformat_name(format),
	      , buf_width, buf_height, out_buf_add_phy );

    if (ctx->is_initialized)
    {
        memset(&ioctl_open_strm_decode, 0, sizeof(IOCTLSDEVOPENDECODE2));
        param = &ioctl_open_strm_decode.ioctl_param;

        param->mode = SDEV2_MODE_VIDEO 
                     | SDEV2_MODE_VIDEO_FP_OUTPUT 
                     | SDEV2_MODE_HOST_INPUT
                     /* | SDEV2_MODE_VIDEO_FRAME_INPUT*/; 
                     // Don't set frame input flag because upper level app has no idea of the video frame boundaries

        param->inputstreamformat = SDEV2_VIDEO_ES_INPUT;
        param->outputstreamformat = SDEV2_VIDEO_ES_OUTPUT;
        param->req_size = VIDEO_DECODER_SEND_BUFFER_SIZE;

        if(img_type == DFB_HW_JPEG) {
            param->video_in_type = JPEG;
        }
        else if(img_type == DFB_HW_PNG) {
            param->video_in_type = PNG;
        }
        else {
            MSG_ERROR("the image type is neither JPEG nor PNG.");
        }


        //for OPTIMIZED_SEND
        param->soc_shared_input_buffer_size = VIDEO_DECODER_SEND_BUFFER_SIZE;
        param->soc_shared_input_buffer_addr_phy = 0;
        param->soc_shared_input_buffer_addr = 0;

        // for non-SDK 1.0
        param->alloc_decode_buffer_pitch = buf_width;
        param->alloc_decode_buffer_height = buf_height;

        // if buffer is allocated by the application set the following otherwise buffer_mode = 0.
        param->alloc_decode_buffer_mode = 1;
        param->alloc_decode_buffer_addr = (unsigned int)out_buf_add_phy;
        param->alloc_decode_buffer_size = buf_size;

        param->video_out_colorspace_format = format_to_pass;

        //stream_handle is used to bind the hXcodeSend to this hXCode when sending stream
        ret = send_img_open_command(ctx, &ioctl_open_strm_decode);

        // by default OPTIMIZED_SEND
        if (!ret)
        {
            if (param->soc_shared_input_buffer_size)
            {
                ctx->send_buf_user_addr = (unsigned char *)param->soc_shared_input_buffer_addr;
                ctx->send_buf_phy_addr = param->soc_shared_input_buffer_addr_phy;
                ctx->send_buf_size = param->soc_shared_input_buffer_size;
                ctx->width = buf_width;
                ctx->height = buf_height;
                ctx->dst_image_size = buf_size;
                ctx->dst_image_phy_addr = out_buf_add_phy;
                ctx->is_opened = 1;
            }
            else
            {
                MSG_ERROR("Fail to allocate send buffer memory!");
                send_img_close_command(ctx);
                ret = -1;
            }
        }
    }
    else
    {
        MSG_ERROR("firmware image module has not been initialized!");
        ret = -1;
    }

    return ret;
}


int img_decoder_append_data(void *c, unsigned int size)
{
    ImgContext_t *ctx = (ImgContext_t *)c;

    int ret = 0;
    unsigned int curr_phy_addr = ctx->send_buf_phy_addr;
    if (ctx->is_opened)
    {
      //memcpy(ctx->send_buf_user_addr, pbuf, size);
      // uint8 *buf = (uint8*)pbuf;
        uint8 *buf = (uint8*)ctx->send_buf_user_addr;
        MSG_INFO("%08x %08x %08x\n", buf[0], buf[(size-1)>>1], buf[size-1]);

        while(size)
        {
            unsigned int ret_size = size;
            MSG_INFO("send_img_stream, size=%d", ret_size);
            ret = send_img_stream(ctx, curr_phy_addr, &ret_size, SDEV_IO_VIDEO, 0, 0);
            if (ret) break;
            size -= ret_size;
            curr_phy_addr += ret_size;
        }
    }
    else
    {
        MSG_ERROR("Decoder has not been opened!");
        ret = -1;
    }
    return ret;
}

int img_execute_decode(void *c)
{
    ImgContext_t *ctx = (ImgContext_t *)c;

    int ret = 0;
    int size = 0;

    if (ctx->is_opened)
    {
        MSG_INFO("Flushing Command!");
        ret = send_img_stream(ctx, ctx->send_buf_phy_addr, &size, SDEV_IO_VIDEO | SDEV_FLUSH_INPUT, 1, 0);
        
        if (!ret)
        {
            MSG_INFO("Read Status!");
            ret = read_decode_status(ctx);
        }
    }
    else
    {
        MSG_ERROR("Decoder has not been opened!");
        ret = -1;
    }

    return ret; 
}

int img_decoder_close(void *c)
{
    ImgContext_t *ctx = (ImgContext_t *)c;

    int ret = 0;
    if (ctx->is_opened)
    {
        ret = send_img_close_command(ctx);
    }
    else
    {
        MSG_ERROR("Decoder has not been opened!");
        ret = -1;
    }

    return ret;
}

int img_decode_deinit(void *c)
{
    if(!c) {
        MSG_ERROR("image context is not initialized!");
        return 0;
    }
    
    ImgContext_t *ctx = (ImgContext_t *)c;

    if(ctx->h_drv_recv > 0) {
        close(ctx->h_drv_recv);
    }
    if(ctx->h_drv_send > 0) {
        close(ctx->h_drv_send);
    }
    if(ctx->h_drv_main > 0) {
        close(ctx->h_drv_main);
    }

    FREE(ctx);
    
    return 0;
}

void *img_get_send_buf_addr(void *c)
{
    ImgContext_t *ctx = (ImgContext_t *)c;
    return ctx->send_buf_user_addr;
}

unsigned int img_get_send_buf_size(void *c)
{
    ImgContext_t *ctx = (ImgContext_t *)c;
    return ctx->send_buf_size;
}

void img_dump_context_info(void *c)
{
    ImgContext_t *ctx = (ImgContext_t *)c;

    MSG_INFO("h_drv_main=0x%08x", ctx->h_drv_main);
    MSG_INFO("h_drv_send=0x%08x", ctx->h_drv_send);
    MSG_INFO("h_drv_recv=0x%08x", ctx->h_drv_recv);
    MSG_INFO("h_stream=0x%016llx", ctx->h_stream);
    MSG_INFO("send_buf_user_addr=%p", ctx->send_buf_user_addr);
    MSG_INFO("send_buf_phy_addr=0x%08x", ctx->send_buf_phy_addr);
    MSG_INFO("send_buf_size=%d", ctx->send_buf_size);
}

