#include <stdio.h>
#include <stdlib.h>

#include <string.h>

#include <gfx/convert.h>

#include "jpeglib.h"


#define GETJSAMPLE(value)  ((int) (value))

#define ALPHA       3
#define RGB_RED	2	/* Offset of Red in an RGB scanline element */
#define RGB_GREEN	1	/* Offset of Green */
#define RGB_BLUE	0	/* Offset of Blue */
#define RGB_PIXELSIZE	4	/* JSAMPLEs per RGB scanline element */


/*
 * Upsample and color convert for the case of 1:1 horizontal and 1:1 vertical.
 */

void h1v1_merged_upsample(DFBRectangle *dst_rect, int input_pitch,
		int output_pitch, JSAMPROW * input_buf, JSAMPROW output_buf) 
{
	register int y, cred, cgreen, cblue;
	int cb, cr;
	register JSAMPROW outptr;
	JSAMPROW inptr0, inptr1, inptr2;
	JDIMENSION col, col_total;
	JDIMENSION row;

	/* copy these pointers into registers if possible */
	register JSAMPLE * range_limit = sample_range_limit;
	int * Crrtab = Cr_r_tab;
	int * Cbbtab = Cb_b_tab;
	INT32 * Crgtab = Cr_g_tab;
	INT32 * Cbgtab = Cb_g_tab;
	//SHIFT_TEMPS

	int input_pitch_UV = input_pitch;
	int input_pitch_Y_group = input_pitch;
	int output_pitch_Y_group = output_pitch;
	JSAMPROW inptr0_row = input_buf[0];
	JSAMPROW inptr1_row = input_buf[1];
	JSAMPROW inptr2_row = input_buf[2];
	JSAMPROW outptr_row = output_buf + dst_rect->y * output_pitch + dst_rect->x;

	/* Loop for each pair of output pixels */
	row = dst_rect->h;
	col_total = dst_rect->w;
	while (row--) {
		col = col_total;

		inptr0 = inptr0_row;
		inptr1 = inptr1_row;
		inptr2 = inptr2_row;
		outptr = outptr_row;

		while (col--) {
			/* Do the chroma part of the calculation */
			cb = GETJSAMPLE(*inptr1++);
			cr = GETJSAMPLE(*inptr2++);
			cred = Crrtab[cr];
			cgreen = (int) ((Cbgtab[cb] + Crgtab[cr]) >> 16);
			cblue = Cbbtab[cb];
			/* Fetch 1 Y values and emit 1 pixels */
			y = GETJSAMPLE(*inptr0++);
			outptr[RGB_RED] = range_limit[y + cred];
			outptr[RGB_GREEN] = range_limit[y + cgreen];
			outptr[RGB_BLUE] = range_limit[y + cblue];
			outptr[ALPHA] = 0xFF;
			outptr += RGB_PIXELSIZE;
		}

		inptr0_row += input_pitch_Y_group;
		inptr1_row += input_pitch_UV;
		inptr2_row += input_pitch_UV;
		outptr_row += output_pitch_Y_group;

	}
}


/*
 * Upsample and color convert for the case of 2:1 horizontal and 1:1 vertical.
 */

void h2v1_merged_upsample(DFBRectangle *dst_rect, int input_pitch,
		int output_pitch, JSAMPROW * input_buf, JSAMPROW output_buf) 
{
	register int y, cred, cgreen, cblue;
	int cb, cr;
	register JSAMPROW outptr;
	JSAMPROW inptr0, inptr1, inptr2;
	JDIMENSION col, col_total;
	JDIMENSION row;

	/* copy these pointers into registers if possible */
	register JSAMPLE * range_limit = sample_range_limit;
	int * Crrtab = Cr_r_tab;
	int * Cbbtab = Cb_b_tab;
	INT32 * Crgtab = Cr_g_tab;
	INT32 * Cbgtab = Cb_g_tab;
	//SHIFT_TEMPS

	int input_pitch_UV = input_pitch >> 1;
	int input_pitch_Y_group = input_pitch;
	int output_pitch_Y_group = output_pitch;
	JSAMPROW inptr0_row = input_buf[0];
	JSAMPROW inptr1_row = input_buf[1];
	JSAMPROW inptr2_row = input_buf[2];
	JSAMPROW outptr_row = output_buf + dst_rect->y * output_pitch + dst_rect->x;

	/* Loop for each pair of output pixels */
	row = dst_rect->h;
	col_total = ( dst_rect->w ) >> 1;
	while (row--) {
		col = col_total;

		inptr0 = inptr0_row;
		inptr1 = inptr1_row;
		inptr2 = inptr2_row;
		outptr = outptr_row;

		while (col--) {
			/* Do the chroma part of the calculation */
			cb = GETJSAMPLE(*inptr1++);
			cr = GETJSAMPLE(*inptr2++);
			cred = Crrtab[cr];
			cgreen = (int) ((Cbgtab[cb] + Crgtab[cr]) >> 16);
			cblue = Cbbtab[cb];
			/* Fetch 2 Y values and emit 2 pixels */
			y = GETJSAMPLE(*inptr0++);
			outptr[RGB_RED] = range_limit[y + cred];
			outptr[RGB_GREEN] = range_limit[y + cgreen];
			outptr[RGB_BLUE] = range_limit[y + cblue];
			outptr[ALPHA] = 0xFF;
			outptr += RGB_PIXELSIZE;
			y = GETJSAMPLE(*inptr0++);
			outptr[RGB_RED] = range_limit[y + cred];
			outptr[RGB_GREEN] = range_limit[y + cgreen];
			outptr[RGB_BLUE] = range_limit[y + cblue];
			outptr[ALPHA] = 0xFF;
			outptr += RGB_PIXELSIZE;
		}
		/* If image width is odd, do the last output column separately */
		if (dst_rect->w & 1) {
			cb = GETJSAMPLE(*inptr1);
			cr = GETJSAMPLE(*inptr2);
			cred = Crrtab[cr];
			cgreen = (int) ((Cbgtab[cb] + Crgtab[cr]) >> 16);
			cblue = Cbbtab[cb];
			y = GETJSAMPLE(*inptr0);
			outptr[RGB_RED] = range_limit[y + cred];
			outptr[RGB_GREEN] = range_limit[y + cgreen];
			outptr[RGB_BLUE] = range_limit[y + cblue];
			outptr[ALPHA] = 0xFF;
		}

		inptr0_row += input_pitch_Y_group;
		inptr1_row += input_pitch_UV;
		inptr2_row += input_pitch_UV;
		outptr_row += output_pitch_Y_group;

	}
}

/*
 * Upsample and color convert for the case of 2:1 horizontal and 2:1 vertical.
 */

void h2v2_merged_upsample(DFBRectangle *dst_rect, int input_pitch,
		int output_pitch, JSAMPROW * input_buf, JSAMPROW output_buf) 
{
	register int y, cred, cgreen, cblue;
	int cb, cr;
	register JSAMPROW outptr0, outptr1;
	JSAMPROW inptr00, inptr01, inptr1, inptr2;
	JDIMENSION col, col_total;
	JDIMENSION row;

	/* copy these pointers into registers if possible */
	register JSAMPLE * range_limit = sample_range_limit;
	int * Crrtab = Cr_r_tab;
	int * Cbbtab = Cb_b_tab;
	INT32 * Crgtab = Cr_g_tab;
	INT32 * Cbgtab = Cb_g_tab;
	//SHIFT_TEMPS

	int input_pitch_UV = input_pitch >> 1;
	int input_pitch_Y_group = input_pitch << 1;
	int output_pitch_Y_group = output_pitch << 1;

	JSAMPROW inptr00_row = input_buf[0];
	JSAMPROW inptr01_row = input_buf[0] + input_pitch;
	JSAMPROW inptr1_row = input_buf[1];
	JSAMPROW inptr2_row = input_buf[2];
	JSAMPROW outptr0_row = output_buf + dst_rect->y * output_pitch + dst_rect->x;
	JSAMPROW outptr1_row = outptr0_row + output_pitch;
	/* Loop for each group of output pixels */
	row = (dst_rect->h) >> 1;
	col_total = (dst_rect->w) >> 1;
	while (row--) {
		col = col_total;

		inptr00 = inptr00_row;
		inptr01 = inptr01_row;
		inptr1 = inptr1_row;
		inptr2 = inptr2_row;
		outptr0 = outptr0_row;
		outptr1 = outptr1_row;

		while (col--) {
			/* Do the chroma part of the calculation */
			cb = GETJSAMPLE(*inptr1++);
			cr = GETJSAMPLE(*inptr2++);
			cred = Crrtab[cr];
			cgreen = (int)((Cbgtab[cb] + Crgtab[cr]) >> 16);
			cblue = Cbbtab[cb];
			/* Fetch 4 Y values and emit 4 pixels */
			y = GETJSAMPLE(*inptr00++);
			outptr0[RGB_RED] = range_limit[y + cred];
			outptr0[RGB_GREEN] = range_limit[y + cgreen];
			outptr0[RGB_BLUE] = range_limit[y + cblue];
			outptr0[ALPHA] = 0xFF;
			outptr0 += RGB_PIXELSIZE;
			y = GETJSAMPLE(*inptr00++);
			outptr0[RGB_RED] = range_limit[y + cred];
			outptr0[RGB_GREEN] = range_limit[y + cgreen];
			outptr0[RGB_BLUE] = range_limit[y + cblue];
			outptr0[ALPHA] = 0xFF;
			outptr0 += RGB_PIXELSIZE;
			y = GETJSAMPLE(*inptr01++);
			outptr1[RGB_RED] = range_limit[y + cred];
			outptr1[RGB_GREEN] = range_limit[y + cgreen];
			outptr1[RGB_BLUE] = range_limit[y + cblue];
			outptr1[ALPHA] = 0xFF;
			outptr1 += RGB_PIXELSIZE;
			y = GETJSAMPLE(*inptr01++);
			outptr1[RGB_RED] = range_limit[y + cred];
			outptr1[RGB_GREEN] = range_limit[y + cgreen];
			outptr1[RGB_BLUE] = range_limit[y + cblue];
			outptr1[ALPHA] = 0xFF;
			outptr1 += RGB_PIXELSIZE;
		}
		/* If image width is odd, do the last output column separately */
		if (dst_rect->w & 1) {
			cb = GETJSAMPLE(*inptr1);
			cr = GETJSAMPLE(*inptr2);
			cred = Crrtab[cr];
			cgreen = (int)((Cbgtab[cb] + Crgtab[cr]) >> 16);
			cblue = Cbbtab[cb];
			y = GETJSAMPLE(*inptr00);
			outptr0[RGB_RED] = range_limit[y + cred];
			outptr0[RGB_GREEN] = range_limit[y + cgreen];
			outptr0[RGB_BLUE] = range_limit[y + cblue];
			outptr0[ALPHA] = 0xFF;
			y = GETJSAMPLE(*inptr01);
			outptr1[RGB_RED] = range_limit[y + cred];
			outptr1[RGB_GREEN] = range_limit[y + cgreen];
			outptr1[RGB_BLUE] = range_limit[y + cblue];
			outptr1[ALPHA] = 0xFF;
		}
		inptr00_row += input_pitch_Y_group;
		inptr01_row += input_pitch_Y_group;
		inptr1_row += input_pitch_UV;
		inptr2_row += input_pitch_UV;
		outptr0_row += output_pitch_Y_group;
		outptr1_row += output_pitch_Y_group;
	}

    if (dst_rect->h & 1) {
		col = col_total;

		inptr00 = inptr00_row;
		inptr1 = inptr1_row;
		inptr2 = inptr2_row;
		outptr0 = outptr0_row;

		while (col--) {
			/* Do the chroma part of the calculation */
			cb = GETJSAMPLE(*inptr1++);
			cr = GETJSAMPLE(*inptr2++);
			cred = Crrtab[cr];
			cgreen = (int) ((Cbgtab[cb] + Crgtab[cr]) >> 16);
			cblue = Cbbtab[cb];
			/* Fetch 4 Y values and emit 4 pixels */
			y = GETJSAMPLE(*inptr00++);
			outptr0[RGB_RED] = range_limit[y + cred];
			outptr0[RGB_GREEN] = range_limit[y + cgreen];
			outptr0[RGB_BLUE] = range_limit[y + cblue];
			outptr0[ALPHA] = 0xFF;
			outptr0 += RGB_PIXELSIZE;
			y = GETJSAMPLE(*inptr00++);
			outptr0[RGB_RED] = range_limit[y + cred];
			outptr0[RGB_GREEN] = range_limit[y + cgreen];
			outptr0[RGB_BLUE] = range_limit[y + cblue];
			outptr0[ALPHA] = 0xFF;
			outptr0 += RGB_PIXELSIZE;
		}
		/* If image width is odd, do the last output column separately */
		if (dst_rect->w & 1) {
			cb = GETJSAMPLE(*inptr1);
			cr = GETJSAMPLE(*inptr2);
			cred = Crrtab[cr];
			cgreen = (int) ((Cbgtab[cb] + Crgtab[cr]) >> 16);
			cblue = Cbbtab[cb];
			y = GETJSAMPLE(*inptr00);
			outptr0[RGB_RED] = range_limit[y + cred];
			outptr0[RGB_GREEN] = range_limit[y + cgreen];
			outptr0[RGB_BLUE] = range_limit[y + cblue];
			outptr0[ALPHA] = 0xFF;
		}
		inptr00_row += input_pitch_Y_group;
		inptr1_row += input_pitch_UV;
		inptr2_row += input_pitch_UV;
		outptr0_row += output_pitch_Y_group;
	}
        
}

