/****************************************************************************/
/*																			*/
/*	fontavenue.h															*/
/*																			*/
/*	Copyright (C) 2006-2007 NEC Corporation.								*/
/*																			*/
/*	Changes																	*/
/*	2007.03.06	Created														*/
/*	2007.07.20	Updated														*/
/*	2007.10.10	Updated														*/
/*	2007.10.15	Updated														*/
/*																			*/
/****************************************************************************/

#ifndef _FONTAVENUE_H_
#define _FONTAVENUE_H_

#include "fa_encoding.h"

#ifdef FA_FILE_ACCESS
#include <stdio.h>
#endif /* FA_FILE_ACCESS */

#define	FA_FALSE				0
#define	FA_TRUE					(!FA_FALSE)

typedef unsigned char			fa_byte;
typedef unsigned short			fa_word;
typedef unsigned int			fa_dword;
typedef char					fa_char;
typedef short					fa_short;
typedef int						fa_int;
typedef int						fa_fixed;
typedef int						fa_bool;
typedef void					fa_void;

#define FA_SUCCESS								0
#define FA_ERROR								0x80000000
#define FA_E_INVALID_ARGUMENT					(FA_ERROR | 0x00000001)
#define FA_E_INVALID_FONTDATA					(FA_ERROR | 0x00000002)
#define FA_E_INVALID_BOUNDARY_OF_POINTER		(FA_ERROR | 0x00000003)
#define FA_E_INVALID_COMBINATION_OF_FONT_DATA	(FA_ERROR | 0x00000004)
#define FA_E_INVALID_BUFFER						(FA_ERROR | 0x00000005)
#define FA_E_INSUFFICIENT_BUFFER				(FA_ERROR | 0x00000006)
#define FA_E_INVALID_FONTID						(FA_ERROR | 0x00000007)
#define FA_E_INVALID_ENCODING_ID				(FA_ERROR | 0x00000008)
#define FA_E_INVALID_CHARCODE					(FA_ERROR | 0x00000009)
#define FA_E_INVALID_REFERENCE_ID				(FA_ERROR | 0x0000000a)
#define FA_E_INVALID_GLYPHSIZE					(FA_ERROR | 0x0000000b)
#define FA_E_INVALID_EDGE						(FA_ERROR | 0x0000000c)
#define FA_E_INVALID_SHADOW						(FA_ERROR | 0x0000000d)
#define FA_E_INVALID_SLANT						(FA_ERROR | 0x0000000e)
#define FA_E_INVALID_ROTATE						(FA_ERROR | 0x0000000f)
#define FA_E_INVALID_WEIGHT						(FA_ERROR | 0x00000010)
#define FA_E_INVALID_STYLE						(FA_ERROR | 0x00000011)
#define FA_E_INVALID_GSCALE						(FA_ERROR | 0x00000012)
#define FA_E_INVALID_DECORATION					(FA_ERROR | 0x00000013)
#define FA_E_INVALID_COMMAND					(FA_ERROR | 0x00000014)
#define FA_E_INVALID_OUT_GLYPHSIZE				(FA_ERROR | 0x00000015)
#define FA_E_NOTFOUND_ENCODING_ID				(FA_ERROR | 0x00000016)
#define FA_E_NOTFOUND_GLYPH_INDEX				(FA_ERROR | 0x00000017)
#define FA_E_NOTFOUND_GLYPH_DATA				(FA_ERROR | 0x00000018)
#define FA_E_NOTFOUND_BMF_DATA					(FA_ERROR | 0x00000019)
#define FA_E_NOT_IMPLEMENTED					(FA_ERROR | 0x0000001a)

#ifdef FA_FILE_ACCESS
#define FA_E_FAIL_FILE_OPEN						(FA_ERROR | 0x00000100)
#define FA_E_FAIL_FILE_CLOSE					(FA_ERROR | 0x00000101)
#define FA_E_FAIL_FILE_SEEK						(FA_ERROR | 0x00000102)
#define FA_E_FAIL_FILE_READ						(FA_ERROR | 0x00000103)
#endif /* FA_FILE_ACCESS */

#define FA_CACHE_ERROR							(FA_ERROR | 0x40000000)
#define FA_E_INVALID_CACHE_BUFFER				(FA_CACHE_ERROR | 0x00000001)
#define FA_E_INSUFFICIENT_CACHE_BUFFER			(FA_CACHE_ERROR | 0x00000002)
#define FA_E_INVALID_HASHENTRYSIZE				(FA_CACHE_ERROR | 0x00000003)
#define FA_E_INVALID_MEMBLKSIZE					(FA_CACHE_ERROR | 0x00000004)

#define FA_CACHESIZE_KB_MIN		128
#define FA_1K					1024

#define FA_NUMBER_OF_FONT_BLOCKS_MAX	16
#define FA_NUMBER_OF_FONT_ELEMENTS_MAX	8

#define FA_REFERENCE_ID_MIN		0
#define FA_REFERENCE_ID_MAX		7
#define FA_GLYPHSIZE_MIN		2
#define FA_GLYPHSIZE_MAX		256
#define FA_EDGE_MIN				0
#define FA_EDGE_MAX				5
#define FA_SHADOW_MIN			(-5)
#define FA_SHADOW_MAX			5
#define FA_SLANT_MIN			(-45)
#define FA_SLANT_MAX			45
#define FA_ROTATE_MIN			0
#define FA_ROTATE_MAX			360
#define FA_WEIGHT_MIN			(-3)
#define FA_WEIGHT_MAX			3

#define FA_DECORA_GRADATION_ON	0x00000000
#define FA_DECORA_GRADATION_OFF	0x00000001

#define FA_STYLE_STANDARD		0x00000000
#define FA_STYLE_MIDDLE			0x00000100
#define FA_STYLE_SMALL			0x00000200
#define FA_STYLE_PROPO			0x00000300
#define FA_STYLE_FIXED			0x00000400
#define FA_STYLE_MASK			0x00000f00

#define FA_DIR_HORIZONTAL		0x00000000
#define FA_DIR_VERTICAL			0x00080000

#define FA_GSCALE_4				0x00000000
#define FA_GSCALE_16			0x00100000
#define FA_GSCALE_256			0x00200000
#define FA_GSCALE_2				0x00300000
#define FA_GSCALE_MASK			0x00f00000

#define FA_HINT_OFF				0x00000000
#define FA_HINT_ON				0x04000000

#define FA_DECODE_BUFSIZE		8192
#define FA_GSCALE_BUFSIZE_MIN	(64 * 64 * sizeof(fa_fixed))

#define FA_COL_DECORA_MASK_2BPP			0x2
#define FA_COL_GLYPH_MASK_2BPP			0x1

#define FA_COL_DECORA_TO_BACK_4BPP		0x8
#define FA_COL_GLYPH_TO_DECORA_4BPP		0x4
#define FA_COL_DECORA_MASK_4BPP			0xc
#define FA_COL_GLYPH_MASK_4BPP			0x3

#define FA_COL_DECORA_TO_BACK_8BPP		0x80
#define FA_COL_GLYPH_TO_DECORA_8BPP		0x40
#define FA_COL_DECORA_MASK_8BPP			0xc0
#define FA_COL_GLYPH_MASK_8BPP			0x0f

#define FA_COL_DECORA_TO_BACK_16BPP		0x80
#define FA_COL_GLYPH_TO_DECORA_16BPP	0x40
#define FA_COL_DECORA_MASK_16BPP		0xc0
#define FA_COL_GLYPH_MASK_16BPP			0xff

typedef struct _Tfa_FontInfo{
	fa_int						FontByteOrder;
	fa_word						FontVersion;
	fa_word						FontType;
	fa_word						MaxGlyphDataLength;
	fa_word						MaxPass1Length;
	fa_word						MeshSizeX;
	fa_word						MeshSizeY;
	fa_byte						LetterFaceX0;
	fa_byte						LetterFaceY0;
	fa_byte						LetterFaceX1;
	fa_byte						LetterFaceY1;
	fa_word						ItalicAngle;
	fa_byte						UnderlinePosition;
	fa_byte						UnderlineWidth;
	fa_byte						StemThicknessVert;
	fa_byte						StemThicknessHorz;
	fa_dword					CharMappingTableOffset;
	fa_dword					CompositionTableOffset;
	fa_dword					GlyphIndexTableOffset;
	fa_dword					GlyphDataTableOffset;
}Tfa_FontInfo, *PTfa_FontInfo;

typedef	struct _Tfa_Metrics{
	fa_int						AscenderLine;
	fa_int						CapLine;
	fa_int						MeanLine;
	fa_int						BaseLine;
	fa_int						DescenderLine;
	fa_int						LeftSideBearing;
	fa_int						RightSideBearing;
	fa_int						UpperSideBearing;
	fa_int						UnderSideBearing;
	fa_int						BBoxX0;
	fa_int						BBoxY0;
	fa_int						BBoxX1;
	fa_int						BBoxY1;
}Tfa_Metrics, *PTfa_Metrics;

typedef struct _Tfa_FontElement{
#ifndef FA_FILE_ACCESS
	fa_dword					*pFont;
#else /* FA_FILE_ACCESS */
	fa_char						*pFont;
	FILE						*fpFont;
#endif /* FA_FILE_ACCESS */
	fa_word						StartIndex;
	fa_word						EndIndex;
	Tfa_FontInfo				FontInfo;
	Tfa_Metrics					BaseMetrics;
}Tfa_FontElement, *PTfa_FontElement;

typedef struct _Tfa_FontBlock{
	PTfa_FontElement			pFontElements;
	fa_int						NumberOfFontElements;
}Tfa_FontBlock, *PTfa_FontBlock;

typedef struct _Tfa_Buffers{
	PTfa_FontBlock				pFontBlocks;
	fa_int						NumberOfFontBlocks;
	fa_fixed					*pDecodeBuffer;
	fa_dword					lengthDecodeBuffer;
	fa_fixed					*pGscaleBuffer;
	fa_dword					lengthGscaleBuffer;
	fa_int						PlatformEndian;
}Tfa_Buffers, *PTfa_Buffers;

typedef struct _Tfa_CacheBuffer{
	fa_byte						*pCacheBuffer;
	fa_byte						*pWordCacheBuffer;
	fa_dword					lengthCacheBuffer;
	fa_dword					CacheHashSize;
	fa_dword					CacheMemBlockSize;
}Tfa_CacheBuffer, *PTfa_CacheBuffer;

typedef struct _Tfa_iDuo{
	fa_int						x;
	fa_int						y;
}Tfa_iDuo, *PTfa_iDuo;

typedef struct _Tfa_fDuo{
	fa_fixed					x;
	fa_fixed					y;
}Tfa_fDuo, *PTfa_fDuo;

typedef struct _Tfa_Rotate{
	fa_int						Angle;
	Tfa_iDuo					Cdn[4];
}Tfa_Rotate, *PTfa_Rotate;

typedef	struct _Tfa_RequestGlyph{
	fa_int						in_FontId;
	fa_int						in_EncodingId;
	fa_word						in_Code;
	fa_int						in_ReferenceId;
	Tfa_iDuo					in_GlyphSize;
	fa_int						in_Edge;
	Tfa_iDuo					in_Shadow;
	Tfa_iDuo					in_Slant;
	PTfa_Rotate					inout_pRotate;
	fa_int						in_Weight;
	fa_dword					in_Command;
	fa_byte						*in_pQparam;
	Tfa_iDuo					out_GlyphSize;
	PTfa_Metrics				out_pMetrics;
	fa_byte						*out_pGlyphBuffer;
	fa_dword					inout_lengthGlyphBuffer;
}Tfa_RequestGlyph, *PTfa_RequestGlyph;


#ifndef FA_CONVERT_IF

#ifdef WIN32_DLL
#define FAAPI __declspec(dllexport)
#else /* WIN32_DLL */
#define FAAPI
#endif /* WIN32_DLL */

#ifdef __cplusplus
extern "C" {
#endif

FAAPI
fa_int FaInitialize(
	PTfa_Buffers				in_pBuffers,
	PTfa_CacheBuffer			in_pCacheBuffer
);

FAAPI
fa_int FaFinalize(
	PTfa_Buffers				in_pBuffers,
	PTfa_CacheBuffer			in_pCacheBuffer
);

FAAPI
fa_int FaInitializeCache(
	PTfa_CacheBuffer			in_pCacheBuffer
);

FAAPI
fa_int FaFinalizeCache(
	PTfa_CacheBuffer			in_pCacheBuffer
);

FAAPI
fa_int FaGetRegionSize(
	PTfa_Buffers				in_pBuffers,
	PTfa_RequestGlyph			inout_pRequestGlyph
);

FAAPI
fa_int FaGetGlyphImage(
	PTfa_Buffers				in_pBuffers,
	PTfa_CacheBuffer			in_pCacheBuffer,
	PTfa_RequestGlyph			inout_pRequestGlyph
);

FAAPI
fa_int FaDecorateGlyphImage(
	PTfa_Buffers				in_pBuffers,
	fa_byte						*in_pGlyphBuffer,
	fa_dword					in_lengthGlyphBuffer,
	PTfa_RequestGlyph			inout_pRequestGlyph
);

#ifdef __cplusplus
}		/* extern "C" */
#endif

#endif /* FA_CONVERT_IF */

#endif /* _FONTAVENUE_H_ */
