#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>
#include <core/layers_internal.h>

#include <display/idirectfbsurface.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include <math.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"
#include "xcioctl.h"

extern void Wait_For_Fifo_Entries(pxc_dfb_driver_data mdrv,
                           u32 status_reg, 
                           u32 free_ent_mask, 
                           u32 free_ent_shift,
                           u32 requested_fifo_space);

#define RegWrite FES1_REG_WRITE

void Set_Fes1_1080p_to_480p(pxc_dfb_driver_data mdrv)
{
    Wait_For_Fes1_Fifo(14);
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    RegWrite (FES1_POSREG0_INIT,    0x00000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT,    0x00000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT,    0x00000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT,    0x00000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00090000);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00090000);
    RegWrite (FES1_SRC1_SCLH_R_Y,    0x000aaaaa);
    RegWrite (FES1_SRC1_SCLH_GB_UV,  0x000aaaaa);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);

    Wait_For_Fes1_Fifo(67);
    //Vertical scaler Y
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x8be21a43);
    RegWrite (FES1_LUT_PROGM, 0x3c421886);
    RegWrite (FES1_LUT_PROGM, 0xbe21441);
    RegWrite (FES1_LUT_PROGM, 0x3c623088);
    RegWrite (FES1_LUT_PROGM, 0x8be20e3e);
    RegWrite (FES1_LUT_PROGM, 0x3c624889);
    RegWrite (FES1_LUT_PROGM, 0xbe2083b);
    RegWrite (FES1_LUT_PROGM, 0x3c82588b);
    RegWrite (FES1_LUT_PROGM, 0x8be20239);
    RegWrite (FES1_LUT_PROGM, 0x3c82708c);
    RegWrite (FES1_LUT_PROGM, 0xbd9fa36);
    RegWrite (FES1_LUT_PROGM, 0x3ca2888e);
    RegWrite (FES1_LUT_PROGM, 0x8bd9f434);
    RegWrite (FES1_LUT_PROGM, 0x3ca2a08f);
    RegWrite (FES1_LUT_PROGM, 0x8bd1ee31);
    RegWrite (FES1_LUT_PROGM, 0x3cc2b090);
    RegWrite (FES1_LUT_PROGM, 0xbd1e62f);
    RegWrite (FES1_LUT_PROGM, 0x3cc2c892);
    RegWrite (FES1_LUT_PROGM, 0x8bc9e02d);
    RegWrite (FES1_LUT_PROGM, 0x3ce2e093);
    RegWrite (FES1_LUT_PROGM, 0xbc1da2a);
    RegWrite (FES1_LUT_PROGM, 0x3d02f895);
    RegWrite (FES1_LUT_PROGM, 0xbc1d428);
    RegWrite (FES1_LUT_PROGM, 0x3d231096);
    RegWrite (FES1_LUT_PROGM, 0x8bb9cc26);
    RegWrite (FES1_LUT_PROGM, 0x3d232897);
    RegWrite (FES1_LUT_PROGM, 0xbb1c623);
    RegWrite (FES1_LUT_PROGM, 0x3d434099);
    RegWrite (FES1_LUT_PROGM, 0xba9c021);
    RegWrite (FES1_LUT_PROGM, 0x3d63589a);
    RegWrite (FES1_LUT_PROGM, 0x8ba1b81f);
    RegWrite (FES1_LUT_PROGM, 0x3d83709b);
    RegWrite (FES1_LUT_PROGM, 0x8b99b21d);
    RegWrite (FES1_LUT_PROGM, 0x3da3889c);
    RegWrite (FES1_LUT_PROGM, 0xb91ac1b);
    RegWrite (FES1_LUT_PROGM, 0x3dc3a09e);
    RegWrite (FES1_LUT_PROGM, 0xb89a419);
    RegWrite (FES1_LUT_PROGM, 0x3de3b89f);
    RegWrite (FES1_LUT_PROGM, 0x8b819e17);
    RegWrite (FES1_LUT_PROGM, 0x3e03d0a0);
    RegWrite (FES1_LUT_PROGM, 0x8b799815);
    RegWrite (FES1_LUT_PROGM, 0x3e23e8a1);
    RegWrite (FES1_LUT_PROGM, 0x8b699013);
    RegWrite (FES1_LUT_PROGM, 0x3e4400a2);
    RegWrite (FES1_LUT_PROGM, 0xb618a11);
    RegWrite (FES1_LUT_PROGM, 0x3e6418a4);
    RegWrite (FES1_LUT_PROGM, 0xb59840f);
    RegWrite (FES1_LUT_PROGM, 0x3e8438a5);
    RegWrite (FES1_LUT_PROGM, 0xb497c0d);
    RegWrite (FES1_LUT_PROGM, 0x3ea450a6);
    RegWrite (FES1_LUT_PROGM, 0xb41760b);
    RegWrite (FES1_LUT_PROGM, 0x3ec468a7);
    RegWrite (FES1_LUT_PROGM, 0x8b39700a);
    RegWrite (FES1_LUT_PROGM, 0x3f0480a8);
    RegWrite (FES1_LUT_PROGM, 0xb296808);
    RegWrite (FES1_LUT_PROGM, 0x3f2498a9);
    RegWrite (FES1_LUT_PROGM, 0xb196206);
    RegWrite (FES1_LUT_PROGM, 0x3f44b8aa);
    RegWrite (FES1_LUT_PROGM, 0x8b115c05);
    RegWrite (FES1_LUT_PROGM, 0x3f84d0ab);
    RegWrite (FES1_LUT_PROGM, 0x8b015403);
    RegWrite (FES1_LUT_PROGM, 0x3fa4e8ac);
    RegWrite (FES1_LUT_PROGM, 0xaf14e02);
    RegWrite (FES1_LUT_PROGM, 0x3fe508ad);
    RegWrite (FES1_LUT_PROGM, 0xae14800);
    RegWrite (FES1_LUT_PROGM, 0x520ae);

    Wait_For_Fes1_Fifo(67);
    //Vertical scaler CbCr
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x8be21a43);
    RegWrite (FES1_LUT_PROGM, 0x3c421886);
    RegWrite (FES1_LUT_PROGM, 0xbe21441);
    RegWrite (FES1_LUT_PROGM, 0x3c623088);
    RegWrite (FES1_LUT_PROGM, 0x8be20e3e);
    RegWrite (FES1_LUT_PROGM, 0x3c624889);
    RegWrite (FES1_LUT_PROGM, 0xbe2083b);
    RegWrite (FES1_LUT_PROGM, 0x3c82588b);
    RegWrite (FES1_LUT_PROGM, 0x8be20239);
    RegWrite (FES1_LUT_PROGM, 0x3c82708c);
    RegWrite (FES1_LUT_PROGM, 0xbd9fa36);
    RegWrite (FES1_LUT_PROGM, 0x3ca2888e);
    RegWrite (FES1_LUT_PROGM, 0x8bd9f434);
    RegWrite (FES1_LUT_PROGM, 0x3ca2a08f);
    RegWrite (FES1_LUT_PROGM, 0x8bd1ee31);
    RegWrite (FES1_LUT_PROGM, 0x3cc2b090);
    RegWrite (FES1_LUT_PROGM, 0xbd1e62f);
    RegWrite (FES1_LUT_PROGM, 0x3cc2c892);
    RegWrite (FES1_LUT_PROGM, 0x8bc9e02d);
    RegWrite (FES1_LUT_PROGM, 0x3ce2e093);
    RegWrite (FES1_LUT_PROGM, 0xbc1da2a);
    RegWrite (FES1_LUT_PROGM, 0x3d02f895);
    RegWrite (FES1_LUT_PROGM, 0xbc1d428);
    RegWrite (FES1_LUT_PROGM, 0x3d231096);
    RegWrite (FES1_LUT_PROGM, 0x8bb9cc26);
    RegWrite (FES1_LUT_PROGM, 0x3d232897);
    RegWrite (FES1_LUT_PROGM, 0xbb1c623);
    RegWrite (FES1_LUT_PROGM, 0x3d434099);
    RegWrite (FES1_LUT_PROGM, 0xba9c021);
    RegWrite (FES1_LUT_PROGM, 0x3d63589a);
    RegWrite (FES1_LUT_PROGM, 0x8ba1b81f);
    RegWrite (FES1_LUT_PROGM, 0x3d83709b);
    RegWrite (FES1_LUT_PROGM, 0x8b99b21d);
    RegWrite (FES1_LUT_PROGM, 0x3da3889c);
    RegWrite (FES1_LUT_PROGM, 0xb91ac1b);
    RegWrite (FES1_LUT_PROGM, 0x3dc3a09e);
    RegWrite (FES1_LUT_PROGM, 0xb89a419);
    RegWrite (FES1_LUT_PROGM, 0x3de3b89f);
    RegWrite (FES1_LUT_PROGM, 0x8b819e17);
    RegWrite (FES1_LUT_PROGM, 0x3e03d0a0);
    RegWrite (FES1_LUT_PROGM, 0x8b799815);
    RegWrite (FES1_LUT_PROGM, 0x3e23e8a1);
    RegWrite (FES1_LUT_PROGM, 0x8b699013);
    RegWrite (FES1_LUT_PROGM, 0x3e4400a2);
    RegWrite (FES1_LUT_PROGM, 0xb618a11);
    RegWrite (FES1_LUT_PROGM, 0x3e6418a4);
    RegWrite (FES1_LUT_PROGM, 0xb59840f);
    RegWrite (FES1_LUT_PROGM, 0x3e8438a5);
    RegWrite (FES1_LUT_PROGM, 0xb497c0d);
    RegWrite (FES1_LUT_PROGM, 0x3ea450a6);
    RegWrite (FES1_LUT_PROGM, 0xb41760b);
    RegWrite (FES1_LUT_PROGM, 0x3ec468a7);
    RegWrite (FES1_LUT_PROGM, 0x8b39700a);
    RegWrite (FES1_LUT_PROGM, 0x3f0480a8);
    RegWrite (FES1_LUT_PROGM, 0xb296808);
    RegWrite (FES1_LUT_PROGM, 0x3f2498a9);
    RegWrite (FES1_LUT_PROGM, 0xb196206);
    RegWrite (FES1_LUT_PROGM, 0x3f44b8aa);
    RegWrite (FES1_LUT_PROGM, 0x8b115c05);
    RegWrite (FES1_LUT_PROGM, 0x3f84d0ab);
    RegWrite (FES1_LUT_PROGM, 0x8b015403);
    RegWrite (FES1_LUT_PROGM, 0x3fa4e8ac);
    RegWrite (FES1_LUT_PROGM, 0xaf14e02);
    RegWrite (FES1_LUT_PROGM, 0x3fe508ad);
    RegWrite (FES1_LUT_PROGM, 0xae14800);
    RegWrite (FES1_LUT_PROGM, 0x520ae);

    Wait_For_Fes1_Fifo(100);
    //Horzontal scaler Y
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x4289fbc0);
    RegWrite (FES1_LUT_PROGM, 0x48908b1a);
    RegWrite (FES1_LUT_PROGM, 0x35703f64);
    RegWrite (FES1_LUT_PROGM, 0x3e86f9bf);
    RegWrite (FES1_LUT_PROGM, 0x58918b1a);
    RegWrite (FES1_LUT_PROGM, 0x35303fa4);
    RegWrite (FES1_LUT_PROGM, 0x3a84f8bf);
    RegWrite (FES1_LUT_PROGM, 0x70928b1a);
    RegWrite (FES1_LUT_PROGM, 0x34f03fc4);
    RegWrite (FES1_LUT_PROGM, 0x3681f6bf);
    RegWrite (FES1_LUT_PROGM, 0x88938b1a);
    RegWrite (FES1_LUT_PROGM, 0x34b00004);
    RegWrite (FES1_LUT_PROGM, 0x347ff5bf);
    RegWrite (FES1_LUT_PROGM, 0x98948b1a);
    RegWrite (FES1_LUT_PROGM, 0x34b00044);
    RegWrite (FES1_LUT_PROGM, 0x307cf3bf);
    RegWrite (FES1_LUT_PROGM, 0xb0958b1a);
    RegWrite (FES1_LUT_PROGM, 0x34704084);
    RegWrite (FES1_LUT_PROGM, 0x2c7af1be);
    RegWrite (FES1_LUT_PROGM, 0xc8968b22);
    RegWrite (FES1_LUT_PROGM, 0x343040c4);
    RegWrite (FES1_LUT_PROGM, 0x2877f0be);
    RegWrite (FES1_LUT_PROGM, 0xd8978b22);
    RegWrite (FES1_LUT_PROGM, 0x343040e4);
    RegWrite (FES1_LUT_PROGM, 0x2475eebe);
    RegWrite (FES1_LUT_PROGM, 0xf0988b1a);
    RegWrite (FES1_LUT_PROGM, 0x33f04124);
    RegWrite (FES1_LUT_PROGM, 0x2072edbe);
    RegWrite (FES1_LUT_PROGM, 0x8998b1a);
    RegWrite (FES1_LUT_PROGM, 0x33b08165);
    RegWrite (FES1_LUT_PROGM, 0x1c70ebbe);
    RegWrite (FES1_LUT_PROGM, 0x189a8b1a);
    RegWrite (FES1_LUT_PROGM, 0x33b081a5);
    RegWrite (FES1_LUT_PROGM, 0x186eeabe);
    RegWrite (FES1_LUT_PROGM, 0x309b8b1a);
    RegWrite (FES1_LUT_PROGM, 0x3370c1e5);
    RegWrite (FES1_LUT_PROGM, 0x146be8be);
    RegWrite (FES1_LUT_PROGM, 0x489c8b1a);
    RegWrite (FES1_LUT_PROGM, 0x3330c225);
    RegWrite (FES1_LUT_PROGM, 0x1069e7be);
    RegWrite (FES1_LUT_PROGM, 0x589d0b12);
    RegWrite (FES1_LUT_PROGM, 0x33310265);
    RegWrite (FES1_LUT_PROGM, 0xc66e6be);
    RegWrite (FES1_LUT_PROGM, 0x709e0b12);
    RegWrite (FES1_LUT_PROGM, 0x32f102a5);
    RegWrite (FES1_LUT_PROGM, 0x864e4be);
    RegWrite (FES1_LUT_PROGM, 0x809f0b12);
    RegWrite (FES1_LUT_PROGM, 0x32f142e5);
    RegWrite (FES1_LUT_PROGM, 0x461e3be);
    RegWrite (FES1_LUT_PROGM, 0x989f8b0a);
    RegWrite (FES1_LUT_PROGM, 0x32b14325);
    RegWrite (FES1_LUT_PROGM, 0x5fe2be);
    RegWrite (FES1_LUT_PROGM, 0xb0a08b0a);
    RegWrite (FES1_LUT_PROGM, 0x32718365);
    RegWrite (FES1_LUT_PROGM, 0xfa5ce0be);
    RegWrite (FES1_LUT_PROGM, 0xc8a18b09);
    RegWrite (FES1_LUT_PROGM, 0x327183a5);
    RegWrite (FES1_LUT_PROGM, 0xf65adfbf);
    RegWrite (FES1_LUT_PROGM, 0xd8a20b01);
    RegWrite (FES1_LUT_PROGM, 0x3231c3e5);
    RegWrite (FES1_LUT_PROGM, 0xf258debf);
    RegWrite (FES1_LUT_PROGM, 0xf0a30b01);
    RegWrite (FES1_LUT_PROGM, 0x3231c425);
    RegWrite (FES1_LUT_PROGM, 0xee55ddbf);
    RegWrite (FES1_LUT_PROGM, 0xa38af9);
    RegWrite (FES1_LUT_PROGM, 0x31f20466);
    RegWrite (FES1_LUT_PROGM, 0xe853dbbf);
    RegWrite (FES1_LUT_PROGM, 0x18a48af1);
    RegWrite (FES1_LUT_PROGM, 0x31f244a6);
    RegWrite (FES1_LUT_PROGM, 0xe450dabf);
    RegWrite (FES1_LUT_PROGM, 0x30a50af1);
    RegWrite (FES1_LUT_PROGM, 0x31b244e6);
    RegWrite (FES1_LUT_PROGM, 0xe04ed9c0);
    RegWrite (FES1_LUT_PROGM, 0x40a60ae9);
    RegWrite (FES1_LUT_PROGM, 0x31b28526);
    RegWrite (FES1_LUT_PROGM, 0xda4cd8c0);
    RegWrite (FES1_LUT_PROGM, 0x58a68ae1);
    RegWrite (FES1_LUT_PROGM, 0x3172c586);
    RegWrite (FES1_LUT_PROGM, 0xd649d7c0);
    RegWrite (FES1_LUT_PROGM, 0x68a70ad9);
    RegWrite (FES1_LUT_PROGM, 0x317305c6);
    RegWrite (FES1_LUT_PROGM, 0xd247d6c0);
    RegWrite (FES1_LUT_PROGM, 0x80a80ad1);
    RegWrite (FES1_LUT_PROGM, 0x31330606);
    RegWrite (FES1_LUT_PROGM, 0xcc45d5c1);
    RegWrite (FES1_LUT_PROGM, 0x90a88ac9);
    RegWrite (FES1_LUT_PROGM, 0x31334646);
    RegWrite (FES1_LUT_PROGM, 0xc842d4c1);
    RegWrite (FES1_LUT_PROGM, 0xa8a90ac1);
    RegWrite (FES1_LUT_PROGM, 0x30f38686);
    RegWrite (FES1_LUT_PROGM, 0xc240d3c1);
    RegWrite (FES1_LUT_PROGM, 0xb8a98ac1);
    RegWrite (FES1_LUT_PROGM, 0x30f3c6e6);
    RegWrite (FES1_LUT_PROGM, 0xbe3ed2c2);
    RegWrite (FES1_LUT_PROGM, 0xd0aa0ab1);
    RegWrite (FES1_LUT_PROGM, 0x30b40726);
    RegWrite (FES1_LUT_PROGM, 0xba3bd1c2);
    RegWrite (FES1_LUT_PROGM, 0xe8ab0ab1);
    RegWrite (FES1_LUT_PROGM, 0x30b44766);
    
    Wait_For_Fes1_Fifo(100);
    //Horzontal scaler CbCr
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x4289fbc0);
    RegWrite (FES1_LUT_PROGM, 0x48908b1a);
    RegWrite (FES1_LUT_PROGM, 0x35703f64);
    RegWrite (FES1_LUT_PROGM, 0x3e86f9bf);
    RegWrite (FES1_LUT_PROGM, 0x58918b1a);
    RegWrite (FES1_LUT_PROGM, 0x35303fa4);
    RegWrite (FES1_LUT_PROGM, 0x3a84f8bf);
    RegWrite (FES1_LUT_PROGM, 0x70928b1a);
    RegWrite (FES1_LUT_PROGM, 0x34f03fc4);
    RegWrite (FES1_LUT_PROGM, 0x3681f6bf);
    RegWrite (FES1_LUT_PROGM, 0x88938b1a);
    RegWrite (FES1_LUT_PROGM, 0x34b00004);
    RegWrite (FES1_LUT_PROGM, 0x347ff5bf);
    RegWrite (FES1_LUT_PROGM, 0x98948b1a);
    RegWrite (FES1_LUT_PROGM, 0x34b00044);
    RegWrite (FES1_LUT_PROGM, 0x307cf3bf);
    RegWrite (FES1_LUT_PROGM, 0xb0958b1a);
    RegWrite (FES1_LUT_PROGM, 0x34704084);
    RegWrite (FES1_LUT_PROGM, 0x2c7af1be);
    RegWrite (FES1_LUT_PROGM, 0xc8968b22);
    RegWrite (FES1_LUT_PROGM, 0x343040c4);
    RegWrite (FES1_LUT_PROGM, 0x2877f0be);
    RegWrite (FES1_LUT_PROGM, 0xd8978b22);
    RegWrite (FES1_LUT_PROGM, 0x343040e4);
    RegWrite (FES1_LUT_PROGM, 0x2475eebe);
    RegWrite (FES1_LUT_PROGM, 0xf0988b1a);
    RegWrite (FES1_LUT_PROGM, 0x33f04124);
    RegWrite (FES1_LUT_PROGM, 0x2072edbe);
    RegWrite (FES1_LUT_PROGM, 0x8998b1a);
    RegWrite (FES1_LUT_PROGM, 0x33b08165);
    RegWrite (FES1_LUT_PROGM, 0x1c70ebbe);
    RegWrite (FES1_LUT_PROGM, 0x189a8b1a);
    RegWrite (FES1_LUT_PROGM, 0x33b081a5);
    RegWrite (FES1_LUT_PROGM, 0x186eeabe);
    RegWrite (FES1_LUT_PROGM, 0x309b8b1a);
    RegWrite (FES1_LUT_PROGM, 0x3370c1e5);
    RegWrite (FES1_LUT_PROGM, 0x146be8be);
    RegWrite (FES1_LUT_PROGM, 0x489c8b1a);
    RegWrite (FES1_LUT_PROGM, 0x3330c225);
    RegWrite (FES1_LUT_PROGM, 0x1069e7be);
    RegWrite (FES1_LUT_PROGM, 0x589d0b12);
    RegWrite (FES1_LUT_PROGM, 0x33310265);
    RegWrite (FES1_LUT_PROGM, 0xc66e6be);
    RegWrite (FES1_LUT_PROGM, 0x709e0b12);
    RegWrite (FES1_LUT_PROGM, 0x32f102a5);
    RegWrite (FES1_LUT_PROGM, 0x864e4be);
    RegWrite (FES1_LUT_PROGM, 0x809f0b12);
    RegWrite (FES1_LUT_PROGM, 0x32f142e5);
    RegWrite (FES1_LUT_PROGM, 0x461e3be);
    RegWrite (FES1_LUT_PROGM, 0x989f8b0a);
    RegWrite (FES1_LUT_PROGM, 0x32b14325);
    RegWrite (FES1_LUT_PROGM, 0x5fe2be);
    RegWrite (FES1_LUT_PROGM, 0xb0a08b0a);
    RegWrite (FES1_LUT_PROGM, 0x32718365);
    RegWrite (FES1_LUT_PROGM, 0xfa5ce0be);
    RegWrite (FES1_LUT_PROGM, 0xc8a18b09);
    RegWrite (FES1_LUT_PROGM, 0x327183a5);
    RegWrite (FES1_LUT_PROGM, 0xf65adfbf);
    RegWrite (FES1_LUT_PROGM, 0xd8a20b01);
    RegWrite (FES1_LUT_PROGM, 0x3231c3e5);
    RegWrite (FES1_LUT_PROGM, 0xf258debf);
    RegWrite (FES1_LUT_PROGM, 0xf0a30b01);
    RegWrite (FES1_LUT_PROGM, 0x3231c425);
    RegWrite (FES1_LUT_PROGM, 0xee55ddbf);
    RegWrite (FES1_LUT_PROGM, 0xa38af9);
    RegWrite (FES1_LUT_PROGM, 0x31f20466);
    RegWrite (FES1_LUT_PROGM, 0xe853dbbf);
    RegWrite (FES1_LUT_PROGM, 0x18a48af1);
    RegWrite (FES1_LUT_PROGM, 0x31f244a6);
    RegWrite (FES1_LUT_PROGM, 0xe450dabf);
    RegWrite (FES1_LUT_PROGM, 0x30a50af1);
    RegWrite (FES1_LUT_PROGM, 0x31b244e6);
    RegWrite (FES1_LUT_PROGM, 0xe04ed9c0);
    RegWrite (FES1_LUT_PROGM, 0x40a60ae9);
    RegWrite (FES1_LUT_PROGM, 0x31b28526);
    RegWrite (FES1_LUT_PROGM, 0xda4cd8c0);
    RegWrite (FES1_LUT_PROGM, 0x58a68ae1);
    RegWrite (FES1_LUT_PROGM, 0x3172c586);
    RegWrite (FES1_LUT_PROGM, 0xd649d7c0);
    RegWrite (FES1_LUT_PROGM, 0x68a70ad9);
    RegWrite (FES1_LUT_PROGM, 0x317305c6);
    RegWrite (FES1_LUT_PROGM, 0xd247d6c0);
    RegWrite (FES1_LUT_PROGM, 0x80a80ad1);
    RegWrite (FES1_LUT_PROGM, 0x31330606);
    RegWrite (FES1_LUT_PROGM, 0xcc45d5c1);
    RegWrite (FES1_LUT_PROGM, 0x90a88ac9);
    RegWrite (FES1_LUT_PROGM, 0x31334646);
    RegWrite (FES1_LUT_PROGM, 0xc842d4c1);
    RegWrite (FES1_LUT_PROGM, 0xa8a90ac1);
    RegWrite (FES1_LUT_PROGM, 0x30f38686);
    RegWrite (FES1_LUT_PROGM, 0xc240d3c1);
    RegWrite (FES1_LUT_PROGM, 0xb8a98ac1);
    RegWrite (FES1_LUT_PROGM, 0x30f3c6e6);
    RegWrite (FES1_LUT_PROGM, 0xbe3ed2c2);
    RegWrite (FES1_LUT_PROGM, 0xd0aa0ab1);
    RegWrite (FES1_LUT_PROGM, 0x30b40726);
    RegWrite (FES1_LUT_PROGM, 0xba3bd1c2);
    RegWrite (FES1_LUT_PROGM, 0xe8ab0ab1);
    RegWrite (FES1_LUT_PROGM, 0x30b44766);
}


void Set_Fes1_1080p_to_576p(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00078000);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00078000);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x000aaaab);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x000aaaab);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x911a8bde);
    RegWrite (FES1_LUT_PROGM, 0x333ef0a2);
    RegWrite (FES1_LUT_PROGM, 0x911a7fda);
    RegWrite (FES1_LUT_PROGM, 0x32ff10a5);
    RegWrite (FES1_LUT_PROGM, 0x912273d6);
    RegWrite (FES1_LUT_PROGM, 0x32df30a8);
    RegWrite (FES1_LUT_PROGM, 0x912269d3);
    RegWrite (FES1_LUT_PROGM, 0x329f50ab);
    RegWrite (FES1_LUT_PROGM, 0x112a5dcf);
    RegWrite (FES1_LUT_PROGM, 0x327f70af);
    RegWrite (FES1_LUT_PROGM, 0x112a51cb);
    RegWrite (FES1_LUT_PROGM, 0x325f98b2);
    RegWrite (FES1_LUT_PROGM, 0x112a45c8);
    RegWrite (FES1_LUT_PROGM, 0x321fb8b5);
    RegWrite (FES1_LUT_PROGM, 0x112a3bc4);
    RegWrite (FES1_LUT_PROGM, 0x31ffd8b8);
    RegWrite (FES1_LUT_PROGM, 0x11222dc1);
    RegWrite (FES1_LUT_PROGM, 0x31c000bb);
    RegWrite (FES1_LUT_PROGM, 0x111a21be);
    RegWrite (FES1_LUT_PROGM, 0x31a028be);
    RegWrite (FES1_LUT_PROGM, 0x912217ba);
    RegWrite (FES1_LUT_PROGM, 0x318048c1);
    RegWrite (FES1_LUT_PROGM, 0x111209b7);
    RegWrite (FES1_LUT_PROGM, 0x316070c4);
    RegWrite (FES1_LUT_PROGM, 0x1109ffb4);
    RegWrite (FES1_LUT_PROGM, 0x314098c7);
    RegWrite (FES1_LUT_PROGM, 0x1101f3b1);
    RegWrite (FES1_LUT_PROGM, 0x3140c0ca);
    RegWrite (FES1_LUT_PROGM, 0x10f9e5af);
    RegWrite (FES1_LUT_PROGM, 0x3120e8cd);
    RegWrite (FES1_LUT_PROGM, 0x10f1d9ac);
    RegWrite (FES1_LUT_PROGM, 0x310110d0);
    RegWrite (FES1_LUT_PROGM, 0x90e1cda9);
    RegWrite (FES1_LUT_PROGM, 0x310138d2);
    RegWrite (FES1_LUT_PROGM, 0x90d9c1a7);
    RegWrite (FES1_LUT_PROGM, 0x30e168d5);
    RegWrite (FES1_LUT_PROGM, 0x10c9b5a4);
    RegWrite (FES1_LUT_PROGM, 0x30e190d8);
    RegWrite (FES1_LUT_PROGM, 0x90b9a9a2);
    RegWrite (FES1_LUT_PROGM, 0x30e1c0da);
    RegWrite (FES1_LUT_PROGM, 0x10a99ba0);
    RegWrite (FES1_LUT_PROGM, 0x30e1e8dd);
    RegWrite (FES1_LUT_PROGM, 0x10998f9d);
    RegWrite (FES1_LUT_PROGM, 0x30e218e0);
    RegWrite (FES1_LUT_PROGM, 0x9081839b);
    RegWrite (FES1_LUT_PROGM, 0x30e240e2);
    RegWrite (FES1_LUT_PROGM, 0x10717799);
    RegWrite (FES1_LUT_PROGM, 0x30e270e5);
    RegWrite (FES1_LUT_PROGM, 0x10596998);
    RegWrite (FES1_LUT_PROGM, 0x30e298e7);
    RegWrite (FES1_LUT_PROGM, 0x90415d96);
    RegWrite (FES1_LUT_PROGM, 0x30e2c8e9);
    RegWrite (FES1_LUT_PROGM, 0x10295194);
    RegWrite (FES1_LUT_PROGM, 0x3102f8ec);
    RegWrite (FES1_LUT_PROGM, 0x10114593);
    RegWrite (FES1_LUT_PROGM, 0x310328ee);
    RegWrite (FES1_LUT_PROGM, 0x8ff93991);
    RegWrite (FES1_LUT_PROGM, 0x312358f0);
    RegWrite (FES1_LUT_PROGM, 0x8fe12b90);
    RegWrite (FES1_LUT_PROGM, 0x314388f2);
    RegWrite (FES1_LUT_PROGM, 0x8fc11f8e);
    RegWrite (FES1_LUT_PROGM, 0x3143b8f4);
    RegWrite (FES1_LUT_PROGM, 0x8fa9138d);
    RegWrite (FES1_LUT_PROGM, 0x3163e8f6);
    RegWrite (FES1_LUT_PROGM, 0x8f89078c);
    RegWrite (FES1_LUT_PROGM, 0x318418f8);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x911a8bde);
    RegWrite (FES1_LUT_PROGM, 0x333ef0a2);
    RegWrite (FES1_LUT_PROGM, 0x911a7fda);
    RegWrite (FES1_LUT_PROGM, 0x32ff10a5);
    RegWrite (FES1_LUT_PROGM, 0x912273d6);
    RegWrite (FES1_LUT_PROGM, 0x32df30a8);
    RegWrite (FES1_LUT_PROGM, 0x912269d3);
    RegWrite (FES1_LUT_PROGM, 0x329f50ab);
    RegWrite (FES1_LUT_PROGM, 0x112a5dcf);
    RegWrite (FES1_LUT_PROGM, 0x327f70af);
    RegWrite (FES1_LUT_PROGM, 0x112a51cb);
    RegWrite (FES1_LUT_PROGM, 0x325f98b2);
    RegWrite (FES1_LUT_PROGM, 0x112a45c8);
    RegWrite (FES1_LUT_PROGM, 0x321fb8b5);
    RegWrite (FES1_LUT_PROGM, 0x112a3bc4);
    RegWrite (FES1_LUT_PROGM, 0x31ffd8b8);
    RegWrite (FES1_LUT_PROGM, 0x11222dc1);
    RegWrite (FES1_LUT_PROGM, 0x31c000bb);
    RegWrite (FES1_LUT_PROGM, 0x111a21be);
    RegWrite (FES1_LUT_PROGM, 0x31a028be);
    RegWrite (FES1_LUT_PROGM, 0x912217ba);
    RegWrite (FES1_LUT_PROGM, 0x318048c1);
    RegWrite (FES1_LUT_PROGM, 0x111209b7);
    RegWrite (FES1_LUT_PROGM, 0x316070c4);
    RegWrite (FES1_LUT_PROGM, 0x1109ffb4);
    RegWrite (FES1_LUT_PROGM, 0x314098c7);
    RegWrite (FES1_LUT_PROGM, 0x1101f3b1);
    RegWrite (FES1_LUT_PROGM, 0x3140c0ca);
    RegWrite (FES1_LUT_PROGM, 0x10f9e5af);
    RegWrite (FES1_LUT_PROGM, 0x3120e8cd);
    RegWrite (FES1_LUT_PROGM, 0x10f1d9ac);
    RegWrite (FES1_LUT_PROGM, 0x310110d0);
    RegWrite (FES1_LUT_PROGM, 0x90e1cda9);
    RegWrite (FES1_LUT_PROGM, 0x310138d2);
    RegWrite (FES1_LUT_PROGM, 0x90d9c1a7);
    RegWrite (FES1_LUT_PROGM, 0x30e168d5);
    RegWrite (FES1_LUT_PROGM, 0x10c9b5a4);
    RegWrite (FES1_LUT_PROGM, 0x30e190d8);
    RegWrite (FES1_LUT_PROGM, 0x90b9a9a2);
    RegWrite (FES1_LUT_PROGM, 0x30e1c0da);
    RegWrite (FES1_LUT_PROGM, 0x10a99ba0);
    RegWrite (FES1_LUT_PROGM, 0x30e1e8dd);
    RegWrite (FES1_LUT_PROGM, 0x10998f9d);
    RegWrite (FES1_LUT_PROGM, 0x30e218e0);
    RegWrite (FES1_LUT_PROGM, 0x9081839b);
    RegWrite (FES1_LUT_PROGM, 0x30e240e2);
    RegWrite (FES1_LUT_PROGM, 0x10717799);
    RegWrite (FES1_LUT_PROGM, 0x30e270e5);
    RegWrite (FES1_LUT_PROGM, 0x10596998);
    RegWrite (FES1_LUT_PROGM, 0x30e298e7);
    RegWrite (FES1_LUT_PROGM, 0x90415d96);
    RegWrite (FES1_LUT_PROGM, 0x30e2c8e9);
    RegWrite (FES1_LUT_PROGM, 0x10295194);
    RegWrite (FES1_LUT_PROGM, 0x3102f8ec);
    RegWrite (FES1_LUT_PROGM, 0x10114593);
    RegWrite (FES1_LUT_PROGM, 0x310328ee);
    RegWrite (FES1_LUT_PROGM, 0x8ff93991);
    RegWrite (FES1_LUT_PROGM, 0x312358f0);
    RegWrite (FES1_LUT_PROGM, 0x8fe12b90);
    RegWrite (FES1_LUT_PROGM, 0x314388f2);
    RegWrite (FES1_LUT_PROGM, 0x8fc11f8e);
    RegWrite (FES1_LUT_PROGM, 0x3143b8f4);
    RegWrite (FES1_LUT_PROGM, 0x8fa9138d);
    RegWrite (FES1_LUT_PROGM, 0x3163e8f6);
    RegWrite (FES1_LUT_PROGM, 0x8f89078c);
    RegWrite (FES1_LUT_PROGM, 0x318418f8);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x8c7dd3a8);
    RegWrite (FES1_LUT_PROGM, 0xe8a30d02);
    RegWrite (FES1_LUT_PROGM, 0x396a3a63);
    RegWrite (FES1_LUT_PROGM, 0x887ad1a8);
    RegWrite (FES1_LUT_PROGM, 0xa48d02);
    RegWrite (FES1_LUT_PROGM, 0x3929faa4);
    RegWrite (FES1_LUT_PROGM, 0x8477cfa8);
    RegWrite (FES1_LUT_PROGM, 0x20a60d0a);
    RegWrite (FES1_LUT_PROGM, 0x38a9fac4);
    RegWrite (FES1_LUT_PROGM, 0x7e73cea9);
    RegWrite (FES1_LUT_PROGM, 0x38a78d0a);
    RegWrite (FES1_LUT_PROGM, 0x3869fb04);
    RegWrite (FES1_LUT_PROGM, 0x7a70cca9);
    RegWrite (FES1_LUT_PROGM, 0x58a90d0a);
    RegWrite (FES1_LUT_PROGM, 0x3829bb44);
    RegWrite (FES1_LUT_PROGM, 0x766dcaa9);
    RegWrite (FES1_LUT_PROGM, 0x70aa8d12);
    RegWrite (FES1_LUT_PROGM, 0x37a9bb84);
    RegWrite (FES1_LUT_PROGM, 0x706ac9aa);
    RegWrite (FES1_LUT_PROGM, 0x90ab8d12);
    RegWrite (FES1_LUT_PROGM, 0x3769bbc4);
    RegWrite (FES1_LUT_PROGM, 0x6c67c7aa);
    RegWrite (FES1_LUT_PROGM, 0xa8ad0d12);
    RegWrite (FES1_LUT_PROGM, 0x37297c04);
    RegWrite (FES1_LUT_PROGM, 0x6664c5ab);
    RegWrite (FES1_LUT_PROGM, 0xc8ae8d12);
    RegWrite (FES1_LUT_PROGM, 0x36a97c44);
    RegWrite (FES1_LUT_PROGM, 0x6061c4ab);
    RegWrite (FES1_LUT_PROGM, 0xe0b00d12);
    RegWrite (FES1_LUT_PROGM, 0x36697c84);
    RegWrite (FES1_LUT_PROGM, 0x5c5ec2ac);
    RegWrite (FES1_LUT_PROGM, 0xb10d12);
    RegWrite (FES1_LUT_PROGM, 0x36297cc5);
    RegWrite (FES1_LUT_PROGM, 0x565bc1ac);
    RegWrite (FES1_LUT_PROGM, 0x18b28d12);
    RegWrite (FES1_LUT_PROGM, 0x35e97d05);
    RegWrite (FES1_LUT_PROGM, 0x5057bfad);
    RegWrite (FES1_LUT_PROGM, 0x38b40d12);
    RegWrite (FES1_LUT_PROGM, 0x35697d45);
    RegWrite (FES1_LUT_PROGM, 0x4c54bead);
    RegWrite (FES1_LUT_PROGM, 0x50b50d0a);
    RegWrite (FES1_LUT_PROGM, 0x35297d85);
    RegWrite (FES1_LUT_PROGM, 0x4651bdae);
    RegWrite (FES1_LUT_PROGM, 0x70b68d0a);
    RegWrite (FES1_LUT_PROGM, 0x34e93de5);
    RegWrite (FES1_LUT_PROGM, 0x404ebbaf);
    RegWrite (FES1_LUT_PROGM, 0x88b78d0a);
    RegWrite (FES1_LUT_PROGM, 0x34693e25);
    RegWrite (FES1_LUT_PROGM, 0x3c4bbaaf);
    RegWrite (FES1_LUT_PROGM, 0xa8b90d02);
    RegWrite (FES1_LUT_PROGM, 0x34293e65);
    RegWrite (FES1_LUT_PROGM, 0x3448b9b0);
    RegWrite (FES1_LUT_PROGM, 0xc0ba0d02);
    RegWrite (FES1_LUT_PROGM, 0x33e97ec5);
    RegWrite (FES1_LUT_PROGM, 0x3045b8b1);
    RegWrite (FES1_LUT_PROGM, 0xe0bb0cfa);
    RegWrite (FES1_LUT_PROGM, 0x33a97f05);
    RegWrite (FES1_LUT_PROGM, 0x2a42b6b2);
    RegWrite (FES1_LUT_PROGM, 0xf8bc0cf2);
    RegWrite (FES1_LUT_PROGM, 0x33297f45);
    RegWrite (FES1_LUT_PROGM, 0x243fb5b2);
    RegWrite (FES1_LUT_PROGM, 0x18bd8cf2);
    RegWrite (FES1_LUT_PROGM, 0x32e97fa6);
    RegWrite (FES1_LUT_PROGM, 0x1e3cb4b3);
    RegWrite (FES1_LUT_PROGM, 0x30be8cea);
    RegWrite (FES1_LUT_PROGM, 0x32a97fe6);
    RegWrite (FES1_LUT_PROGM, 0x1839b3b4);
    RegWrite (FES1_LUT_PROGM, 0x50bf8ce2);
    RegWrite (FES1_LUT_PROGM, 0x32698046);
    RegWrite (FES1_LUT_PROGM, 0x1236b2b5);
    RegWrite (FES1_LUT_PROGM, 0x68c08cda);
    RegWrite (FES1_LUT_PROGM, 0x32298086);
    RegWrite (FES1_LUT_PROGM, 0xc33b1b6);
    RegWrite (FES1_LUT_PROGM, 0x88c18cda);
    RegWrite (FES1_LUT_PROGM, 0x31a980e6);
    RegWrite (FES1_LUT_PROGM, 0x430b0b7);
    RegWrite (FES1_LUT_PROGM, 0xa0c28cca);
    RegWrite (FES1_LUT_PROGM, 0x3169c126);
    RegWrite (FES1_LUT_PROGM, 0xfe2dafb7);
    RegWrite (FES1_LUT_PROGM, 0xb8c30cc1);
    RegWrite (FES1_LUT_PROGM, 0x3129c186);
    RegWrite (FES1_LUT_PROGM, 0xf82aaeb8);
    RegWrite (FES1_LUT_PROGM, 0xd8c40cb9);
    RegWrite (FES1_LUT_PROGM, 0x30ea01e6);
    RegWrite (FES1_LUT_PROGM, 0xf227adb9);
    RegWrite (FES1_LUT_PROGM, 0xf0c50cb1);
    RegWrite (FES1_LUT_PROGM, 0x30aa0226);
    RegWrite (FES1_LUT_PROGM, 0xec25adba);
    RegWrite (FES1_LUT_PROGM, 0x10c60ca9);
    RegWrite (FES1_LUT_PROGM, 0x306a4287);
    RegWrite (FES1_LUT_PROGM, 0xe422acbb);
    RegWrite (FES1_LUT_PROGM, 0x28c70c99);
    RegWrite (FES1_LUT_PROGM, 0x302a42e7);
    RegWrite (FES1_LUT_PROGM, 0xde1fabbc);
    RegWrite (FES1_LUT_PROGM, 0x40c78c91);
    RegWrite (FES1_LUT_PROGM, 0x2faa8327);
    RegWrite (FES1_LUT_PROGM, 0xd81caabd);
    RegWrite (FES1_LUT_PROGM, 0x60c80c81);
    RegWrite (FES1_LUT_PROGM, 0x2f6a8387);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x8c7dd3a8);
    RegWrite (FES1_LUT_PROGM, 0xe8a30d02);
    RegWrite (FES1_LUT_PROGM, 0x396a3a63);
    RegWrite (FES1_LUT_PROGM, 0x887ad1a8);
    RegWrite (FES1_LUT_PROGM, 0xa48d02);
    RegWrite (FES1_LUT_PROGM, 0x3929faa4);
    RegWrite (FES1_LUT_PROGM, 0x8477cfa8);
    RegWrite (FES1_LUT_PROGM, 0x20a60d0a);
    RegWrite (FES1_LUT_PROGM, 0x38a9fac4);
    RegWrite (FES1_LUT_PROGM, 0x7e73cea9);
    RegWrite (FES1_LUT_PROGM, 0x38a78d0a);
    RegWrite (FES1_LUT_PROGM, 0x3869fb04);
    RegWrite (FES1_LUT_PROGM, 0x7a70cca9);
    RegWrite (FES1_LUT_PROGM, 0x58a90d0a);
    RegWrite (FES1_LUT_PROGM, 0x3829bb44);
    RegWrite (FES1_LUT_PROGM, 0x766dcaa9);
    RegWrite (FES1_LUT_PROGM, 0x70aa8d12);
    RegWrite (FES1_LUT_PROGM, 0x37a9bb84);
    RegWrite (FES1_LUT_PROGM, 0x706ac9aa);
    RegWrite (FES1_LUT_PROGM, 0x90ab8d12);
    RegWrite (FES1_LUT_PROGM, 0x3769bbc4);
    RegWrite (FES1_LUT_PROGM, 0x6c67c7aa);
    RegWrite (FES1_LUT_PROGM, 0xa8ad0d12);
    RegWrite (FES1_LUT_PROGM, 0x37297c04);
    RegWrite (FES1_LUT_PROGM, 0x6664c5ab);
    RegWrite (FES1_LUT_PROGM, 0xc8ae8d12);
    RegWrite (FES1_LUT_PROGM, 0x36a97c44);
    RegWrite (FES1_LUT_PROGM, 0x6061c4ab);
    RegWrite (FES1_LUT_PROGM, 0xe0b00d12);
    RegWrite (FES1_LUT_PROGM, 0x36697c84);
    RegWrite (FES1_LUT_PROGM, 0x5c5ec2ac);
    RegWrite (FES1_LUT_PROGM, 0xb10d12);
    RegWrite (FES1_LUT_PROGM, 0x36297cc5);
    RegWrite (FES1_LUT_PROGM, 0x565bc1ac);
    RegWrite (FES1_LUT_PROGM, 0x18b28d12);
    RegWrite (FES1_LUT_PROGM, 0x35e97d05);
    RegWrite (FES1_LUT_PROGM, 0x5057bfad);
    RegWrite (FES1_LUT_PROGM, 0x38b40d12);
    RegWrite (FES1_LUT_PROGM, 0x35697d45);
    RegWrite (FES1_LUT_PROGM, 0x4c54bead);
    RegWrite (FES1_LUT_PROGM, 0x50b50d0a);
    RegWrite (FES1_LUT_PROGM, 0x35297d85);
    RegWrite (FES1_LUT_PROGM, 0x4651bdae);
    RegWrite (FES1_LUT_PROGM, 0x70b68d0a);
    RegWrite (FES1_LUT_PROGM, 0x34e93de5);
    RegWrite (FES1_LUT_PROGM, 0x404ebbaf);
    RegWrite (FES1_LUT_PROGM, 0x88b78d0a);
    RegWrite (FES1_LUT_PROGM, 0x34693e25);
    RegWrite (FES1_LUT_PROGM, 0x3c4bbaaf);
    RegWrite (FES1_LUT_PROGM, 0xa8b90d02);
    RegWrite (FES1_LUT_PROGM, 0x34293e65);
    RegWrite (FES1_LUT_PROGM, 0x3448b9b0);
    RegWrite (FES1_LUT_PROGM, 0xc0ba0d02);
    RegWrite (FES1_LUT_PROGM, 0x33e97ec5);
    RegWrite (FES1_LUT_PROGM, 0x3045b8b1);
    RegWrite (FES1_LUT_PROGM, 0xe0bb0cfa);
    RegWrite (FES1_LUT_PROGM, 0x33a97f05);
    RegWrite (FES1_LUT_PROGM, 0x2a42b6b2);
    RegWrite (FES1_LUT_PROGM, 0xf8bc0cf2);
    RegWrite (FES1_LUT_PROGM, 0x33297f45);
    RegWrite (FES1_LUT_PROGM, 0x243fb5b2);
    RegWrite (FES1_LUT_PROGM, 0x18bd8cf2);
    RegWrite (FES1_LUT_PROGM, 0x32e97fa6);
    RegWrite (FES1_LUT_PROGM, 0x1e3cb4b3);
    RegWrite (FES1_LUT_PROGM, 0x30be8cea);
    RegWrite (FES1_LUT_PROGM, 0x32a97fe6);
    RegWrite (FES1_LUT_PROGM, 0x1839b3b4);
    RegWrite (FES1_LUT_PROGM, 0x50bf8ce2);
    RegWrite (FES1_LUT_PROGM, 0x32698046);
    RegWrite (FES1_LUT_PROGM, 0x1236b2b5);
    RegWrite (FES1_LUT_PROGM, 0x68c08cda);
    RegWrite (FES1_LUT_PROGM, 0x32298086);
    RegWrite (FES1_LUT_PROGM, 0xc33b1b6);
    RegWrite (FES1_LUT_PROGM, 0x88c18cda);
    RegWrite (FES1_LUT_PROGM, 0x31a980e6);
    RegWrite (FES1_LUT_PROGM, 0x430b0b7);
    RegWrite (FES1_LUT_PROGM, 0xa0c28cca);
    RegWrite (FES1_LUT_PROGM, 0x3169c126);
    RegWrite (FES1_LUT_PROGM, 0xfe2dafb7);
    RegWrite (FES1_LUT_PROGM, 0xb8c30cc1);
    RegWrite (FES1_LUT_PROGM, 0x3129c186);
    RegWrite (FES1_LUT_PROGM, 0xf82aaeb8);
    RegWrite (FES1_LUT_PROGM, 0xd8c40cb9);
    RegWrite (FES1_LUT_PROGM, 0x30ea01e6);
    RegWrite (FES1_LUT_PROGM, 0xf227adb9);
    RegWrite (FES1_LUT_PROGM, 0xf0c50cb1);
    RegWrite (FES1_LUT_PROGM, 0x30aa0226);
    RegWrite (FES1_LUT_PROGM, 0xec25adba);
    RegWrite (FES1_LUT_PROGM, 0x10c60ca9);
    RegWrite (FES1_LUT_PROGM, 0x306a4287);
    RegWrite (FES1_LUT_PROGM, 0xe422acbb);
    RegWrite (FES1_LUT_PROGM, 0x28c70c99);
    RegWrite (FES1_LUT_PROGM, 0x302a42e7);
    RegWrite (FES1_LUT_PROGM, 0xde1fabbc);
    RegWrite (FES1_LUT_PROGM, 0x40c78c91);
    RegWrite (FES1_LUT_PROGM, 0x2faa8327);
    RegWrite (FES1_LUT_PROGM, 0xd81caabd);
    RegWrite (FES1_LUT_PROGM, 0x60c80c81);
    RegWrite (FES1_LUT_PROGM, 0x2f6a8387);
}

void Set_Fes1_720p_to_480p(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00060000);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00060000);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x00071c72);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x00071c72);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x16aa596a);
    RegWrite (FES1_LUT_PROGM, 0x1b5096);
    RegWrite (FES1_LUT_PROGM, 0x96ba4568);
    RegWrite (FES1_LUT_PROGM, 0x3f9b609b);
    RegWrite (FES1_LUT_PROGM, 0x96ca3366);
    RegWrite (FES1_LUT_PROGM, 0x3f1b68a1);
    RegWrite (FES1_LUT_PROGM, 0x16da1f64);
    RegWrite (FES1_LUT_PROGM, 0x3e9b70a7);
    RegWrite (FES1_LUT_PROGM, 0x16ea0d63);
    RegWrite (FES1_LUT_PROGM, 0x3e1b80ad);
    RegWrite (FES1_LUT_PROGM, 0x96f1f961);
    RegWrite (FES1_LUT_PROGM, 0x3d9b90b2);
    RegWrite (FES1_LUT_PROGM, 0x96f9e760);
    RegWrite (FES1_LUT_PROGM, 0x3d1ba8b8);
    RegWrite (FES1_LUT_PROGM, 0x9701d35f);
    RegWrite (FES1_LUT_PROGM, 0x3c9bb8be);
    RegWrite (FES1_LUT_PROGM, 0x1701bf5e);
    RegWrite (FES1_LUT_PROGM, 0x3c1bd0c4);
    RegWrite (FES1_LUT_PROGM, 0x1701ad5e);
    RegWrite (FES1_LUT_PROGM, 0x3b9be8ca);
    RegWrite (FES1_LUT_PROGM, 0x96f9995d);
    RegWrite (FES1_LUT_PROGM, 0x3b1c00cf);
    RegWrite (FES1_LUT_PROGM, 0x96f1855d);
    RegWrite (FES1_LUT_PROGM, 0x3a7c20d5);
    RegWrite (FES1_LUT_PROGM, 0x96f1715d);
    RegWrite (FES1_LUT_PROGM, 0x39fc38db);
    RegWrite (FES1_LUT_PROGM, 0x16e15d5d);
    RegWrite (FES1_LUT_PROGM, 0x397c58e1);
    RegWrite (FES1_LUT_PROGM, 0x96d14b5e);
    RegWrite (FES1_LUT_PROGM, 0x38fc78e6);
    RegWrite (FES1_LUT_PROGM, 0x96c9375e);
    RegWrite (FES1_LUT_PROGM, 0x385ca0ec);
    RegWrite (FES1_LUT_PROGM, 0x16b1235f);
    RegWrite (FES1_LUT_PROGM, 0x37dcc0f2);
    RegWrite (FES1_LUT_PROGM, 0x9699115f);
    RegWrite (FES1_LUT_PROGM, 0x375ce8f7);
    RegWrite (FES1_LUT_PROGM, 0x1680fd61);
    RegWrite (FES1_LUT_PROGM, 0x36dd10fd);
    RegWrite (FES1_LUT_PROGM, 0x1660e962);
    RegWrite (FES1_LUT_PROGM, 0x363d4102);
    RegWrite (FES1_LUT_PROGM, 0x1648d763);
    RegWrite (FES1_LUT_PROGM, 0x35bd6908);
    RegWrite (FES1_LUT_PROGM, 0x1620c365);
    RegWrite (FES1_LUT_PROGM, 0x353d990d);
    RegWrite (FES1_LUT_PROGM, 0x1600b167);
    RegWrite (FES1_LUT_PROGM, 0x34bdc912);
    RegWrite (FES1_LUT_PROGM, 0x95e09f68);
    RegWrite (FES1_LUT_PROGM, 0x343df917);
    RegWrite (FES1_LUT_PROGM, 0x95b08b6a);
    RegWrite (FES1_LUT_PROGM, 0x33be291c);
    RegWrite (FES1_LUT_PROGM, 0x1580796d);
    RegWrite (FES1_LUT_PROGM, 0x333e6121);
    RegWrite (FES1_LUT_PROGM, 0x1550676f);
    RegWrite (FES1_LUT_PROGM, 0x32be9926);
    RegWrite (FES1_LUT_PROGM, 0x95205572);
    RegWrite (FES1_LUT_PROGM, 0x323ed12a);
    RegWrite (FES1_LUT_PROGM, 0x14e84574);
    RegWrite (FES1_LUT_PROGM, 0x31df092f);
    RegWrite (FES1_LUT_PROGM, 0x94b83377);
    RegWrite (FES1_LUT_PROGM, 0x315f4933);
    RegWrite (FES1_LUT_PROGM, 0x1480217a);
    RegWrite (FES1_LUT_PROGM, 0x30ff8138);
    RegWrite (FES1_LUT_PROGM, 0x1440117d);
    RegWrite (FES1_LUT_PROGM, 0x307fc13c);
    RegWrite (FES1_LUT_PROGM, 0x14000180);
    RegWrite (FES1_LUT_PROGM, 0x30000140);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x16aa596a);
    RegWrite (FES1_LUT_PROGM, 0x1b5096);
    RegWrite (FES1_LUT_PROGM, 0x96ba4568);
    RegWrite (FES1_LUT_PROGM, 0x3f9b609b);
    RegWrite (FES1_LUT_PROGM, 0x96ca3366);
    RegWrite (FES1_LUT_PROGM, 0x3f1b68a1);
    RegWrite (FES1_LUT_PROGM, 0x16da1f64);
    RegWrite (FES1_LUT_PROGM, 0x3e9b70a7);
    RegWrite (FES1_LUT_PROGM, 0x16ea0d63);
    RegWrite (FES1_LUT_PROGM, 0x3e1b80ad);
    RegWrite (FES1_LUT_PROGM, 0x96f1f961);
    RegWrite (FES1_LUT_PROGM, 0x3d9b90b2);
    RegWrite (FES1_LUT_PROGM, 0x96f9e760);
    RegWrite (FES1_LUT_PROGM, 0x3d1ba8b8);
    RegWrite (FES1_LUT_PROGM, 0x9701d35f);
    RegWrite (FES1_LUT_PROGM, 0x3c9bb8be);
    RegWrite (FES1_LUT_PROGM, 0x1701bf5e);
    RegWrite (FES1_LUT_PROGM, 0x3c1bd0c4);
    RegWrite (FES1_LUT_PROGM, 0x1701ad5e);
    RegWrite (FES1_LUT_PROGM, 0x3b9be8ca);
    RegWrite (FES1_LUT_PROGM, 0x96f9995d);
    RegWrite (FES1_LUT_PROGM, 0x3b1c00cf);
    RegWrite (FES1_LUT_PROGM, 0x96f1855d);
    RegWrite (FES1_LUT_PROGM, 0x3a7c20d5);
    RegWrite (FES1_LUT_PROGM, 0x96f1715d);
    RegWrite (FES1_LUT_PROGM, 0x39fc38db);
    RegWrite (FES1_LUT_PROGM, 0x16e15d5d);
    RegWrite (FES1_LUT_PROGM, 0x397c58e1);
    RegWrite (FES1_LUT_PROGM, 0x96d14b5e);
    RegWrite (FES1_LUT_PROGM, 0x38fc78e6);
    RegWrite (FES1_LUT_PROGM, 0x96c9375e);
    RegWrite (FES1_LUT_PROGM, 0x385ca0ec);
    RegWrite (FES1_LUT_PROGM, 0x16b1235f);
    RegWrite (FES1_LUT_PROGM, 0x37dcc0f2);
    RegWrite (FES1_LUT_PROGM, 0x9699115f);
    RegWrite (FES1_LUT_PROGM, 0x375ce8f7);
    RegWrite (FES1_LUT_PROGM, 0x1680fd61);
    RegWrite (FES1_LUT_PROGM, 0x36dd10fd);
    RegWrite (FES1_LUT_PROGM, 0x1660e962);
    RegWrite (FES1_LUT_PROGM, 0x363d4102);
    RegWrite (FES1_LUT_PROGM, 0x1648d763);
    RegWrite (FES1_LUT_PROGM, 0x35bd6908);
    RegWrite (FES1_LUT_PROGM, 0x1620c365);
    RegWrite (FES1_LUT_PROGM, 0x353d990d);
    RegWrite (FES1_LUT_PROGM, 0x1600b167);
    RegWrite (FES1_LUT_PROGM, 0x34bdc912);
    RegWrite (FES1_LUT_PROGM, 0x95e09f68);
    RegWrite (FES1_LUT_PROGM, 0x343df917);
    RegWrite (FES1_LUT_PROGM, 0x95b08b6a);
    RegWrite (FES1_LUT_PROGM, 0x33be291c);
    RegWrite (FES1_LUT_PROGM, 0x1580796d);
    RegWrite (FES1_LUT_PROGM, 0x333e6121);
    RegWrite (FES1_LUT_PROGM, 0x1550676f);
    RegWrite (FES1_LUT_PROGM, 0x32be9926);
    RegWrite (FES1_LUT_PROGM, 0x95205572);
    RegWrite (FES1_LUT_PROGM, 0x323ed12a);
    RegWrite (FES1_LUT_PROGM, 0x14e84574);
    RegWrite (FES1_LUT_PROGM, 0x31df092f);
    RegWrite (FES1_LUT_PROGM, 0x94b83377);
    RegWrite (FES1_LUT_PROGM, 0x315f4933);
    RegWrite (FES1_LUT_PROGM, 0x1480217a);
    RegWrite (FES1_LUT_PROGM, 0x30ff8138);
    RegWrite (FES1_LUT_PROGM, 0x1440117d);
    RegWrite (FES1_LUT_PROGM, 0x307fc13c);
    RegWrite (FES1_LUT_PROGM, 0x14000180);
    RegWrite (FES1_LUT_PROGM, 0x30000140);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x6bc4a938);
    RegWrite (FES1_LUT_PROGM, 0x209a916a);
    RegWrite (FES1_LUT_PROGM, 0x8ce353e);
    RegWrite (FES1_LUT_PROGM, 0x5bc0ab39);
    RegWrite (FES1_LUT_PROGM, 0x409d115a);
    RegWrite (FES1_LUT_PROGM, 0x90db4fe);
    RegWrite (FES1_LUT_PROGM, 0x4fbdad3a);
    RegWrite (FES1_LUT_PROGM, 0x58a09152);
    RegWrite (FES1_LUT_PROGM, 0x98d74be);
    RegWrite (FES1_LUT_PROGM, 0x41bab03b);
    RegWrite (FES1_LUT_PROGM, 0x78a3114a);
    RegWrite (FES1_LUT_PROGM, 0x9ccf47e);
    RegWrite (FES1_LUT_PROGM, 0x31b7b23c);
    RegWrite (FES1_LUT_PROGM, 0x98a6113a);
    RegWrite (FES1_LUT_PROGM, 0xa4cb45e);
    RegWrite (FES1_LUT_PROGM, 0x23b4b43d);
    RegWrite (FES1_LUT_PROGM, 0xb8a91132);
    RegWrite (FES1_LUT_PROGM, 0xa8c341e);
    RegWrite (FES1_LUT_PROGM, 0x17b1b73e);
    RegWrite (FES1_LUT_PROGM, 0xd8ac112a);
    RegWrite (FES1_LUT_PROGM, 0xb0bb3de);
    RegWrite (FES1_LUT_PROGM, 0x9aeb93f);
    RegWrite (FES1_LUT_PROGM, 0xf8af111a);
    RegWrite (FES1_LUT_PROGM, 0xb4b73be);
    RegWrite (FES1_LUT_PROGM, 0xfbacbb40);
    RegWrite (FES1_LUT_PROGM, 0x20b19109);
    RegWrite (FES1_LUT_PROGM, 0xb8af37f);
    RegWrite (FES1_LUT_PROGM, 0xeda9be40);
    RegWrite (FES1_LUT_PROGM, 0x40b490f9);
    RegWrite (FES1_LUT_PROGM, 0xc0a735f);
    RegWrite (FES1_LUT_PROGM, 0xdfa7c041);
    RegWrite (FES1_LUT_PROGM, 0x60b790e9);
    RegWrite (FES1_LUT_PROGM, 0xc49f31f);
    RegWrite (FES1_LUT_PROGM, 0xd1a4c342);
    RegWrite (FES1_LUT_PROGM, 0x88ba10e1);
    RegWrite (FES1_LUT_PROGM, 0xc89b2ff);
    RegWrite (FES1_LUT_PROGM, 0xc3a2c542);
    RegWrite (FES1_LUT_PROGM, 0xb0bc90c9);
    RegWrite (FES1_LUT_PROGM, 0xcc932df);
    RegWrite (FES1_LUT_PROGM, 0xb7a0c843);
    RegWrite (FES1_LUT_PROGM, 0xd0bf90b9);
    RegWrite (FES1_LUT_PROGM, 0xd08b29f);
    RegWrite (FES1_LUT_PROGM, 0xa99eca44);
    RegWrite (FES1_LUT_PROGM, 0xf8c210a9);
    RegWrite (FES1_LUT_PROGM, 0xd88327f);
    RegWrite (FES1_LUT_PROGM, 0x9b9ccd44);
    RegWrite (FES1_LUT_PROGM, 0x20c49091);
    RegWrite (FES1_LUT_PROGM, 0xdc7b240);
    RegWrite (FES1_LUT_PROGM, 0x8d9acf44);
    RegWrite (FES1_LUT_PROGM, 0x48c79081);
    RegWrite (FES1_LUT_PROGM, 0xe073220);
    RegWrite (FES1_LUT_PROGM, 0x7f99d245);
    RegWrite (FES1_LUT_PROGM, 0x70ca1069);
    RegWrite (FES1_LUT_PROGM, 0xe46b200);
    RegWrite (FES1_LUT_PROGM, 0x7397d445);
    RegWrite (FES1_LUT_PROGM, 0x98cc9051);
    RegWrite (FES1_LUT_PROGM, 0xe8631e0);
    RegWrite (FES1_LUT_PROGM, 0x6596d745);
    RegWrite (FES1_LUT_PROGM, 0xc0cf1039);
    RegWrite (FES1_LUT_PROGM, 0xec571c0);
    RegWrite (FES1_LUT_PROGM, 0x5994d945);
    RegWrite (FES1_LUT_PROGM, 0xe8d19021);
    RegWrite (FES1_LUT_PROGM, 0xf04f1c0);
    RegWrite (FES1_LUT_PROGM, 0x4b93dc46);
    RegWrite (FES1_LUT_PROGM, 0x10d49011);
    RegWrite (FES1_LUT_PROGM, 0xf4471a1);
    RegWrite (FES1_LUT_PROGM, 0x3f92de46);
    RegWrite (FES1_LUT_PROGM, 0x40d68ff1);
    RegWrite (FES1_LUT_PROGM, 0xf43f181);
    RegWrite (FES1_LUT_PROGM, 0x3191e146);
    RegWrite (FES1_LUT_PROGM, 0x68d90fd9);
    RegWrite (FES1_LUT_PROGM, 0xf837181);
    RegWrite (FES1_LUT_PROGM, 0x2590e346);
    RegWrite (FES1_LUT_PROGM, 0x90db8fc1);
    RegWrite (FES1_LUT_PROGM, 0xfc2b161);
    RegWrite (FES1_LUT_PROGM, 0x198fe646);
    RegWrite (FES1_LUT_PROGM, 0xc0de0fa1);
    RegWrite (FES1_LUT_PROGM, 0x10023161);
    RegWrite (FES1_LUT_PROGM, 0xb8ee945);
    RegWrite (FES1_LUT_PROGM, 0xf0e00f89);
    RegWrite (FES1_LUT_PROGM, 0x1001b161);
    RegWrite (FES1_LUT_PROGM, 0xff8deb45);
    RegWrite (FES1_LUT_PROGM, 0x18e20f68);
    RegWrite (FES1_LUT_PROGM, 0x1040f162);
    RegWrite (FES1_LUT_PROGM, 0xf38cee45);
    RegWrite (FES1_LUT_PROGM, 0x48e48f48);
    RegWrite (FES1_LUT_PROGM, 0x10807142);
    RegWrite (FES1_LUT_PROGM, 0xe78cf045);
    RegWrite (FES1_LUT_PROGM, 0x78e68f30);
    RegWrite (FES1_LUT_PROGM, 0x10bff142);
    RegWrite (FES1_LUT_PROGM, 0xd98bf245);
    RegWrite (FES1_LUT_PROGM, 0xa0e88f10);
    RegWrite (FES1_LUT_PROGM, 0x10ff3142);
    RegWrite (FES1_LUT_PROGM, 0xcd8bf544);
    RegWrite (FES1_LUT_PROGM, 0xd0eb0ef0);
    RegWrite (FES1_LUT_PROGM, 0x10feb142);
    RegWrite (FES1_LUT_PROGM, 0xc18bf744);
    RegWrite (FES1_LUT_PROGM, 0xed0ed0);
    RegWrite (FES1_LUT_PROGM, 0x113df163);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x6bc4a938);
    RegWrite (FES1_LUT_PROGM, 0x209a916a);
    RegWrite (FES1_LUT_PROGM, 0x8ce353e);
    RegWrite (FES1_LUT_PROGM, 0x5bc0ab39);
    RegWrite (FES1_LUT_PROGM, 0x409d115a);
    RegWrite (FES1_LUT_PROGM, 0x90db4fe);
    RegWrite (FES1_LUT_PROGM, 0x4fbdad3a);
    RegWrite (FES1_LUT_PROGM, 0x58a09152);
    RegWrite (FES1_LUT_PROGM, 0x98d74be);
    RegWrite (FES1_LUT_PROGM, 0x41bab03b);
    RegWrite (FES1_LUT_PROGM, 0x78a3114a);
    RegWrite (FES1_LUT_PROGM, 0x9ccf47e);
    RegWrite (FES1_LUT_PROGM, 0x31b7b23c);
    RegWrite (FES1_LUT_PROGM, 0x98a6113a);
    RegWrite (FES1_LUT_PROGM, 0xa4cb45e);
    RegWrite (FES1_LUT_PROGM, 0x23b4b43d);
    RegWrite (FES1_LUT_PROGM, 0xb8a91132);
    RegWrite (FES1_LUT_PROGM, 0xa8c341e);
    RegWrite (FES1_LUT_PROGM, 0x17b1b73e);
    RegWrite (FES1_LUT_PROGM, 0xd8ac112a);
    RegWrite (FES1_LUT_PROGM, 0xb0bb3de);
    RegWrite (FES1_LUT_PROGM, 0x9aeb93f);
    RegWrite (FES1_LUT_PROGM, 0xf8af111a);
    RegWrite (FES1_LUT_PROGM, 0xb4b73be);
    RegWrite (FES1_LUT_PROGM, 0xfbacbb40);
    RegWrite (FES1_LUT_PROGM, 0x20b19109);
    RegWrite (FES1_LUT_PROGM, 0xb8af37f);
    RegWrite (FES1_LUT_PROGM, 0xeda9be40);
    RegWrite (FES1_LUT_PROGM, 0x40b490f9);
    RegWrite (FES1_LUT_PROGM, 0xc0a735f);
    RegWrite (FES1_LUT_PROGM, 0xdfa7c041);
    RegWrite (FES1_LUT_PROGM, 0x60b790e9);
    RegWrite (FES1_LUT_PROGM, 0xc49f31f);
    RegWrite (FES1_LUT_PROGM, 0xd1a4c342);
    RegWrite (FES1_LUT_PROGM, 0x88ba10e1);
    RegWrite (FES1_LUT_PROGM, 0xc89b2ff);
    RegWrite (FES1_LUT_PROGM, 0xc3a2c542);
    RegWrite (FES1_LUT_PROGM, 0xb0bc90c9);
    RegWrite (FES1_LUT_PROGM, 0xcc932df);
    RegWrite (FES1_LUT_PROGM, 0xb7a0c843);
    RegWrite (FES1_LUT_PROGM, 0xd0bf90b9);
    RegWrite (FES1_LUT_PROGM, 0xd08b29f);
    RegWrite (FES1_LUT_PROGM, 0xa99eca44);
    RegWrite (FES1_LUT_PROGM, 0xf8c210a9);
    RegWrite (FES1_LUT_PROGM, 0xd88327f);
    RegWrite (FES1_LUT_PROGM, 0x9b9ccd44);
    RegWrite (FES1_LUT_PROGM, 0x20c49091);
    RegWrite (FES1_LUT_PROGM, 0xdc7b240);
    RegWrite (FES1_LUT_PROGM, 0x8d9acf44);
    RegWrite (FES1_LUT_PROGM, 0x48c79081);
    RegWrite (FES1_LUT_PROGM, 0xe073220);
    RegWrite (FES1_LUT_PROGM, 0x7f99d245);
    RegWrite (FES1_LUT_PROGM, 0x70ca1069);
    RegWrite (FES1_LUT_PROGM, 0xe46b200);
    RegWrite (FES1_LUT_PROGM, 0x7397d445);
    RegWrite (FES1_LUT_PROGM, 0x98cc9051);
    RegWrite (FES1_LUT_PROGM, 0xe8631e0);
    RegWrite (FES1_LUT_PROGM, 0x6596d745);
    RegWrite (FES1_LUT_PROGM, 0xc0cf1039);
    RegWrite (FES1_LUT_PROGM, 0xec571c0);
    RegWrite (FES1_LUT_PROGM, 0x5994d945);
    RegWrite (FES1_LUT_PROGM, 0xe8d19021);
    RegWrite (FES1_LUT_PROGM, 0xf04f1c0);
    RegWrite (FES1_LUT_PROGM, 0x4b93dc46);
    RegWrite (FES1_LUT_PROGM, 0x10d49011);
    RegWrite (FES1_LUT_PROGM, 0xf4471a1);
    RegWrite (FES1_LUT_PROGM, 0x3f92de46);
    RegWrite (FES1_LUT_PROGM, 0x40d68ff1);
    RegWrite (FES1_LUT_PROGM, 0xf43f181);
    RegWrite (FES1_LUT_PROGM, 0x3191e146);
    RegWrite (FES1_LUT_PROGM, 0x68d90fd9);
    RegWrite (FES1_LUT_PROGM, 0xf837181);
    RegWrite (FES1_LUT_PROGM, 0x2590e346);
    RegWrite (FES1_LUT_PROGM, 0x90db8fc1);
    RegWrite (FES1_LUT_PROGM, 0xfc2b161);
    RegWrite (FES1_LUT_PROGM, 0x198fe646);
    RegWrite (FES1_LUT_PROGM, 0xc0de0fa1);
    RegWrite (FES1_LUT_PROGM, 0x10023161);
    RegWrite (FES1_LUT_PROGM, 0xb8ee945);
    RegWrite (FES1_LUT_PROGM, 0xf0e00f89);
    RegWrite (FES1_LUT_PROGM, 0x1001b161);
    RegWrite (FES1_LUT_PROGM, 0xff8deb45);
    RegWrite (FES1_LUT_PROGM, 0x18e20f68);
    RegWrite (FES1_LUT_PROGM, 0x1040f162);
    RegWrite (FES1_LUT_PROGM, 0xf38cee45);
    RegWrite (FES1_LUT_PROGM, 0x48e48f48);
    RegWrite (FES1_LUT_PROGM, 0x10807142);
    RegWrite (FES1_LUT_PROGM, 0xe78cf045);
    RegWrite (FES1_LUT_PROGM, 0x78e68f30);
    RegWrite (FES1_LUT_PROGM, 0x10bff142);
    RegWrite (FES1_LUT_PROGM, 0xd98bf245);
    RegWrite (FES1_LUT_PROGM, 0xa0e88f10);
    RegWrite (FES1_LUT_PROGM, 0x10ff3142);
    RegWrite (FES1_LUT_PROGM, 0xcd8bf544);
    RegWrite (FES1_LUT_PROGM, 0xd0eb0ef0);
    RegWrite (FES1_LUT_PROGM, 0x10feb142);
    RegWrite (FES1_LUT_PROGM, 0xc18bf744);
    RegWrite (FES1_LUT_PROGM, 0xed0ed0);
    RegWrite (FES1_LUT_PROGM, 0x113df163);
}

void Set_Fes1_720p_to_576p(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00050000);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00050000);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x00071c72);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x00071c72);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x9a618b60);
    RegWrite (FES1_LUT_PROGM, 0xd5b0062);
    RegWrite (FES1_LUT_PROGM, 0x9a597164);
    RegWrite (FES1_LUT_PROGM, 0xd5ae869);
    RegWrite (FES1_LUT_PROGM, 0x9a595567);
    RegWrite (FES1_LUT_PROGM, 0xd1ad070);
    RegWrite (FES1_LUT_PROGM, 0x9a593b6b);
    RegWrite (FES1_LUT_PROGM, 0xcfab877);
    RegWrite (FES1_LUT_PROGM, 0x1a49216f);
    RegWrite (FES1_LUT_PROGM, 0xcdaa87f);
    RegWrite (FES1_LUT_PROGM, 0x1a410773);
    RegWrite (FES1_LUT_PROGM, 0xc9a9086);
    RegWrite (FES1_LUT_PROGM, 0x1a28ed77);
    RegWrite (FES1_LUT_PROGM, 0xc7a808d);
    RegWrite (FES1_LUT_PROGM, 0x9a20d37b);
    RegWrite (FES1_LUT_PROGM, 0xc3a7094);
    RegWrite (FES1_LUT_PROGM, 0x9a00b980);
    RegWrite (FES1_LUT_PROGM, 0xbfa609b);
    RegWrite (FES1_LUT_PROGM, 0x19e8a184);
    RegWrite (FES1_LUT_PROGM, 0xbba58a3);
    RegWrite (FES1_LUT_PROGM, 0x99d08989);
    RegWrite (FES1_LUT_PROGM, 0xb5a50aa);
    RegWrite (FES1_LUT_PROGM, 0x19b0718e);
    RegWrite (FES1_LUT_PROGM, 0xb1a48b2);
    RegWrite (FES1_LUT_PROGM, 0x19885993);
    RegWrite (FES1_LUT_PROGM, 0xaba40b9);
    RegWrite (FES1_LUT_PROGM, 0x99684398);
    RegWrite (FES1_LUT_PROGM, 0xa5a40c0);
    RegWrite (FES1_LUT_PROGM, 0x19382d9d);
    RegWrite (FES1_LUT_PROGM, 0x9fa40c8);
    RegWrite (FES1_LUT_PROGM, 0x190817a2);
    RegWrite (FES1_LUT_PROGM, 0x99a40cf);
    RegWrite (FES1_LUT_PROGM, 0x18d001a8);
    RegWrite (FES1_LUT_PROGM, 0x91a40d6);
    RegWrite (FES1_LUT_PROGM, 0x98a7edad);
    RegWrite (FES1_LUT_PROGM, 0x8ba48dd);
    RegWrite (FES1_LUT_PROGM, 0x186fd7b2);
    RegWrite (FES1_LUT_PROGM, 0x83a50e5);
    RegWrite (FES1_LUT_PROGM, 0x1837c3b8);
    RegWrite (FES1_LUT_PROGM, 0x7ba60ec);
    RegWrite (FES1_LUT_PROGM, 0x97f7b1be);
    RegWrite (FES1_LUT_PROGM, 0x73a68f3);
    RegWrite (FES1_LUT_PROGM, 0x97b79dc3);
    RegWrite (FES1_LUT_PROGM, 0x6ba78fa);
    RegWrite (FES1_LUT_PROGM, 0x97778bc9);
    RegWrite (FES1_LUT_PROGM, 0x63a9101);
    RegWrite (FES1_LUT_PROGM, 0x172779ce);
    RegWrite (FES1_LUT_PROGM, 0x59aa108);
    RegWrite (FES1_LUT_PROGM, 0x16df69d4);
    RegWrite (FES1_LUT_PROGM, 0x51ab90f);
    RegWrite (FES1_LUT_PROGM, 0x168f59da);
    RegWrite (FES1_LUT_PROGM, 0x47ad916);
    RegWrite (FES1_LUT_PROGM, 0x963f49df);
    RegWrite (FES1_LUT_PROGM, 0x3faf11c);
    RegWrite (FES1_LUT_PROGM, 0x15ef39e5);
    RegWrite (FES1_LUT_PROGM, 0x35b1123);
    RegWrite (FES1_LUT_PROGM, 0x95972bea);
    RegWrite (FES1_LUT_PROGM, 0x2bb3929);
    RegWrite (FES1_LUT_PROGM, 0x153f1df0);
    RegWrite (FES1_LUT_PROGM, 0x21b5930);
    RegWrite (FES1_LUT_PROGM, 0x14e70ff5);
    RegWrite (FES1_LUT_PROGM, 0x17b8136);
    RegWrite (FES1_LUT_PROGM, 0x148703fb);
    RegWrite (FES1_LUT_PROGM, 0xbbb13c);
    RegWrite (FES1_LUT_PROGM, 0x942ef600);
    RegWrite (FES1_LUT_PROGM, 0x1bd942);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x9a618b60);
    RegWrite (FES1_LUT_PROGM, 0xd5b0062);
    RegWrite (FES1_LUT_PROGM, 0x9a597164);
    RegWrite (FES1_LUT_PROGM, 0xd5ae869);
    RegWrite (FES1_LUT_PROGM, 0x9a595567);
    RegWrite (FES1_LUT_PROGM, 0xd1ad070);
    RegWrite (FES1_LUT_PROGM, 0x9a593b6b);
    RegWrite (FES1_LUT_PROGM, 0xcfab877);
    RegWrite (FES1_LUT_PROGM, 0x1a49216f);
    RegWrite (FES1_LUT_PROGM, 0xcdaa87f);
    RegWrite (FES1_LUT_PROGM, 0x1a410773);
    RegWrite (FES1_LUT_PROGM, 0xc9a9086);
    RegWrite (FES1_LUT_PROGM, 0x1a28ed77);
    RegWrite (FES1_LUT_PROGM, 0xc7a808d);
    RegWrite (FES1_LUT_PROGM, 0x9a20d37b);
    RegWrite (FES1_LUT_PROGM, 0xc3a7094);
    RegWrite (FES1_LUT_PROGM, 0x9a00b980);
    RegWrite (FES1_LUT_PROGM, 0xbfa609b);
    RegWrite (FES1_LUT_PROGM, 0x19e8a184);
    RegWrite (FES1_LUT_PROGM, 0xbba58a3);
    RegWrite (FES1_LUT_PROGM, 0x99d08989);
    RegWrite (FES1_LUT_PROGM, 0xb5a50aa);
    RegWrite (FES1_LUT_PROGM, 0x19b0718e);
    RegWrite (FES1_LUT_PROGM, 0xb1a48b2);
    RegWrite (FES1_LUT_PROGM, 0x19885993);
    RegWrite (FES1_LUT_PROGM, 0xaba40b9);
    RegWrite (FES1_LUT_PROGM, 0x99684398);
    RegWrite (FES1_LUT_PROGM, 0xa5a40c0);
    RegWrite (FES1_LUT_PROGM, 0x19382d9d);
    RegWrite (FES1_LUT_PROGM, 0x9fa40c8);
    RegWrite (FES1_LUT_PROGM, 0x190817a2);
    RegWrite (FES1_LUT_PROGM, 0x99a40cf);
    RegWrite (FES1_LUT_PROGM, 0x18d001a8);
    RegWrite (FES1_LUT_PROGM, 0x91a40d6);
    RegWrite (FES1_LUT_PROGM, 0x98a7edad);
    RegWrite (FES1_LUT_PROGM, 0x8ba48dd);
    RegWrite (FES1_LUT_PROGM, 0x186fd7b2);
    RegWrite (FES1_LUT_PROGM, 0x83a50e5);
    RegWrite (FES1_LUT_PROGM, 0x1837c3b8);
    RegWrite (FES1_LUT_PROGM, 0x7ba60ec);
    RegWrite (FES1_LUT_PROGM, 0x97f7b1be);
    RegWrite (FES1_LUT_PROGM, 0x73a68f3);
    RegWrite (FES1_LUT_PROGM, 0x97b79dc3);
    RegWrite (FES1_LUT_PROGM, 0x6ba78fa);
    RegWrite (FES1_LUT_PROGM, 0x97778bc9);
    RegWrite (FES1_LUT_PROGM, 0x63a9101);
    RegWrite (FES1_LUT_PROGM, 0x172779ce);
    RegWrite (FES1_LUT_PROGM, 0x59aa108);
    RegWrite (FES1_LUT_PROGM, 0x16df69d4);
    RegWrite (FES1_LUT_PROGM, 0x51ab90f);
    RegWrite (FES1_LUT_PROGM, 0x168f59da);
    RegWrite (FES1_LUT_PROGM, 0x47ad916);
    RegWrite (FES1_LUT_PROGM, 0x963f49df);
    RegWrite (FES1_LUT_PROGM, 0x3faf11c);
    RegWrite (FES1_LUT_PROGM, 0x15ef39e5);
    RegWrite (FES1_LUT_PROGM, 0x35b1123);
    RegWrite (FES1_LUT_PROGM, 0x95972bea);
    RegWrite (FES1_LUT_PROGM, 0x2bb3929);
    RegWrite (FES1_LUT_PROGM, 0x153f1df0);
    RegWrite (FES1_LUT_PROGM, 0x21b5930);
    RegWrite (FES1_LUT_PROGM, 0x14e70ff5);
    RegWrite (FES1_LUT_PROGM, 0x17b8136);
    RegWrite (FES1_LUT_PROGM, 0x148703fb);
    RegWrite (FES1_LUT_PROGM, 0xbbb13c);
    RegWrite (FES1_LUT_PROGM, 0x942ef600);
    RegWrite (FES1_LUT_PROGM, 0x1bd942);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x6bc4a938);
    RegWrite (FES1_LUT_PROGM, 0x209a916a);
    RegWrite (FES1_LUT_PROGM, 0x8ce353e);
    RegWrite (FES1_LUT_PROGM, 0x5bc0ab39);
    RegWrite (FES1_LUT_PROGM, 0x409d115a);
    RegWrite (FES1_LUT_PROGM, 0x90db4fe);
    RegWrite (FES1_LUT_PROGM, 0x4fbdad3a);
    RegWrite (FES1_LUT_PROGM, 0x58a09152);
    RegWrite (FES1_LUT_PROGM, 0x98d74be);
    RegWrite (FES1_LUT_PROGM, 0x41bab03b);
    RegWrite (FES1_LUT_PROGM, 0x78a3114a);
    RegWrite (FES1_LUT_PROGM, 0x9ccf47e);
    RegWrite (FES1_LUT_PROGM, 0x31b7b23c);
    RegWrite (FES1_LUT_PROGM, 0x98a6113a);
    RegWrite (FES1_LUT_PROGM, 0xa4cb45e);
    RegWrite (FES1_LUT_PROGM, 0x23b4b43d);
    RegWrite (FES1_LUT_PROGM, 0xb8a91132);
    RegWrite (FES1_LUT_PROGM, 0xa8c341e);
    RegWrite (FES1_LUT_PROGM, 0x17b1b73e);
    RegWrite (FES1_LUT_PROGM, 0xd8ac112a);
    RegWrite (FES1_LUT_PROGM, 0xb0bb3de);
    RegWrite (FES1_LUT_PROGM, 0x9aeb93f);
    RegWrite (FES1_LUT_PROGM, 0xf8af111a);
    RegWrite (FES1_LUT_PROGM, 0xb4b73be);
    RegWrite (FES1_LUT_PROGM, 0xfbacbb40);
    RegWrite (FES1_LUT_PROGM, 0x20b19109);
    RegWrite (FES1_LUT_PROGM, 0xb8af37f);
    RegWrite (FES1_LUT_PROGM, 0xeda9be40);
    RegWrite (FES1_LUT_PROGM, 0x40b490f9);
    RegWrite (FES1_LUT_PROGM, 0xc0a735f);
    RegWrite (FES1_LUT_PROGM, 0xdfa7c041);
    RegWrite (FES1_LUT_PROGM, 0x60b790e9);
    RegWrite (FES1_LUT_PROGM, 0xc49f31f);
    RegWrite (FES1_LUT_PROGM, 0xd1a4c342);
    RegWrite (FES1_LUT_PROGM, 0x88ba10e1);
    RegWrite (FES1_LUT_PROGM, 0xc89b2ff);
    RegWrite (FES1_LUT_PROGM, 0xc3a2c542);
    RegWrite (FES1_LUT_PROGM, 0xb0bc90c9);
    RegWrite (FES1_LUT_PROGM, 0xcc932df);
    RegWrite (FES1_LUT_PROGM, 0xb7a0c843);
    RegWrite (FES1_LUT_PROGM, 0xd0bf90b9);
    RegWrite (FES1_LUT_PROGM, 0xd08b29f);
    RegWrite (FES1_LUT_PROGM, 0xa99eca44);
    RegWrite (FES1_LUT_PROGM, 0xf8c210a9);
    RegWrite (FES1_LUT_PROGM, 0xd88327f);
    RegWrite (FES1_LUT_PROGM, 0x9b9ccd44);
    RegWrite (FES1_LUT_PROGM, 0x20c49091);
    RegWrite (FES1_LUT_PROGM, 0xdc7b240);
    RegWrite (FES1_LUT_PROGM, 0x8d9acf44);
    RegWrite (FES1_LUT_PROGM, 0x48c79081);
    RegWrite (FES1_LUT_PROGM, 0xe073220);
    RegWrite (FES1_LUT_PROGM, 0x7f99d245);
    RegWrite (FES1_LUT_PROGM, 0x70ca1069);
    RegWrite (FES1_LUT_PROGM, 0xe46b200);
    RegWrite (FES1_LUT_PROGM, 0x7397d445);
    RegWrite (FES1_LUT_PROGM, 0x98cc9051);
    RegWrite (FES1_LUT_PROGM, 0xe8631e0);
    RegWrite (FES1_LUT_PROGM, 0x6596d745);
    RegWrite (FES1_LUT_PROGM, 0xc0cf1039);
    RegWrite (FES1_LUT_PROGM, 0xec571c0);
    RegWrite (FES1_LUT_PROGM, 0x5994d945);
    RegWrite (FES1_LUT_PROGM, 0xe8d19021);
    RegWrite (FES1_LUT_PROGM, 0xf04f1c0);
    RegWrite (FES1_LUT_PROGM, 0x4b93dc46);
    RegWrite (FES1_LUT_PROGM, 0x10d49011);
    RegWrite (FES1_LUT_PROGM, 0xf4471a1);
    RegWrite (FES1_LUT_PROGM, 0x3f92de46);
    RegWrite (FES1_LUT_PROGM, 0x40d68ff1);
    RegWrite (FES1_LUT_PROGM, 0xf43f181);
    RegWrite (FES1_LUT_PROGM, 0x3191e146);
    RegWrite (FES1_LUT_PROGM, 0x68d90fd9);
    RegWrite (FES1_LUT_PROGM, 0xf837181);
    RegWrite (FES1_LUT_PROGM, 0x2590e346);
    RegWrite (FES1_LUT_PROGM, 0x90db8fc1);
    RegWrite (FES1_LUT_PROGM, 0xfc2b161);
    RegWrite (FES1_LUT_PROGM, 0x198fe646);
    RegWrite (FES1_LUT_PROGM, 0xc0de0fa1);
    RegWrite (FES1_LUT_PROGM, 0x10023161);
    RegWrite (FES1_LUT_PROGM, 0xb8ee945);
    RegWrite (FES1_LUT_PROGM, 0xf0e00f89);
    RegWrite (FES1_LUT_PROGM, 0x1001b161);
    RegWrite (FES1_LUT_PROGM, 0xff8deb45);
    RegWrite (FES1_LUT_PROGM, 0x18e20f68);
    RegWrite (FES1_LUT_PROGM, 0x1040f162);
    RegWrite (FES1_LUT_PROGM, 0xf38cee45);
    RegWrite (FES1_LUT_PROGM, 0x48e48f48);
    RegWrite (FES1_LUT_PROGM, 0x10807142);
    RegWrite (FES1_LUT_PROGM, 0xe78cf045);
    RegWrite (FES1_LUT_PROGM, 0x78e68f30);
    RegWrite (FES1_LUT_PROGM, 0x10bff142);
    RegWrite (FES1_LUT_PROGM, 0xd98bf245);
    RegWrite (FES1_LUT_PROGM, 0xa0e88f10);
    RegWrite (FES1_LUT_PROGM, 0x10ff3142);
    RegWrite (FES1_LUT_PROGM, 0xcd8bf544);
    RegWrite (FES1_LUT_PROGM, 0xd0eb0ef0);
    RegWrite (FES1_LUT_PROGM, 0x10feb142);
    RegWrite (FES1_LUT_PROGM, 0xc18bf744);
    RegWrite (FES1_LUT_PROGM, 0xed0ed0);
    RegWrite (FES1_LUT_PROGM, 0x113df163);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x6bc4a938);
    RegWrite (FES1_LUT_PROGM, 0x209a916a);
    RegWrite (FES1_LUT_PROGM, 0x8ce353e);
    RegWrite (FES1_LUT_PROGM, 0x5bc0ab39);
    RegWrite (FES1_LUT_PROGM, 0x409d115a);
    RegWrite (FES1_LUT_PROGM, 0x90db4fe);
    RegWrite (FES1_LUT_PROGM, 0x4fbdad3a);
    RegWrite (FES1_LUT_PROGM, 0x58a09152);
    RegWrite (FES1_LUT_PROGM, 0x98d74be);
    RegWrite (FES1_LUT_PROGM, 0x41bab03b);
    RegWrite (FES1_LUT_PROGM, 0x78a3114a);
    RegWrite (FES1_LUT_PROGM, 0x9ccf47e);
    RegWrite (FES1_LUT_PROGM, 0x31b7b23c);
    RegWrite (FES1_LUT_PROGM, 0x98a6113a);
    RegWrite (FES1_LUT_PROGM, 0xa4cb45e);
    RegWrite (FES1_LUT_PROGM, 0x23b4b43d);
    RegWrite (FES1_LUT_PROGM, 0xb8a91132);
    RegWrite (FES1_LUT_PROGM, 0xa8c341e);
    RegWrite (FES1_LUT_PROGM, 0x17b1b73e);
    RegWrite (FES1_LUT_PROGM, 0xd8ac112a);
    RegWrite (FES1_LUT_PROGM, 0xb0bb3de);
    RegWrite (FES1_LUT_PROGM, 0x9aeb93f);
    RegWrite (FES1_LUT_PROGM, 0xf8af111a);
    RegWrite (FES1_LUT_PROGM, 0xb4b73be);
    RegWrite (FES1_LUT_PROGM, 0xfbacbb40);
    RegWrite (FES1_LUT_PROGM, 0x20b19109);
    RegWrite (FES1_LUT_PROGM, 0xb8af37f);
    RegWrite (FES1_LUT_PROGM, 0xeda9be40);
    RegWrite (FES1_LUT_PROGM, 0x40b490f9);
    RegWrite (FES1_LUT_PROGM, 0xc0a735f);
    RegWrite (FES1_LUT_PROGM, 0xdfa7c041);
    RegWrite (FES1_LUT_PROGM, 0x60b790e9);
    RegWrite (FES1_LUT_PROGM, 0xc49f31f);
    RegWrite (FES1_LUT_PROGM, 0xd1a4c342);
    RegWrite (FES1_LUT_PROGM, 0x88ba10e1);
    RegWrite (FES1_LUT_PROGM, 0xc89b2ff);
    RegWrite (FES1_LUT_PROGM, 0xc3a2c542);
    RegWrite (FES1_LUT_PROGM, 0xb0bc90c9);
    RegWrite (FES1_LUT_PROGM, 0xcc932df);
    RegWrite (FES1_LUT_PROGM, 0xb7a0c843);
    RegWrite (FES1_LUT_PROGM, 0xd0bf90b9);
    RegWrite (FES1_LUT_PROGM, 0xd08b29f);
    RegWrite (FES1_LUT_PROGM, 0xa99eca44);
    RegWrite (FES1_LUT_PROGM, 0xf8c210a9);
    RegWrite (FES1_LUT_PROGM, 0xd88327f);
    RegWrite (FES1_LUT_PROGM, 0x9b9ccd44);
    RegWrite (FES1_LUT_PROGM, 0x20c49091);
    RegWrite (FES1_LUT_PROGM, 0xdc7b240);
    RegWrite (FES1_LUT_PROGM, 0x8d9acf44);
    RegWrite (FES1_LUT_PROGM, 0x48c79081);
    RegWrite (FES1_LUT_PROGM, 0xe073220);
    RegWrite (FES1_LUT_PROGM, 0x7f99d245);
    RegWrite (FES1_LUT_PROGM, 0x70ca1069);
    RegWrite (FES1_LUT_PROGM, 0xe46b200);
    RegWrite (FES1_LUT_PROGM, 0x7397d445);
    RegWrite (FES1_LUT_PROGM, 0x98cc9051);
    RegWrite (FES1_LUT_PROGM, 0xe8631e0);
    RegWrite (FES1_LUT_PROGM, 0x6596d745);
    RegWrite (FES1_LUT_PROGM, 0xc0cf1039);
    RegWrite (FES1_LUT_PROGM, 0xec571c0);
    RegWrite (FES1_LUT_PROGM, 0x5994d945);
    RegWrite (FES1_LUT_PROGM, 0xe8d19021);
    RegWrite (FES1_LUT_PROGM, 0xf04f1c0);
    RegWrite (FES1_LUT_PROGM, 0x4b93dc46);
    RegWrite (FES1_LUT_PROGM, 0x10d49011);
    RegWrite (FES1_LUT_PROGM, 0xf4471a1);
    RegWrite (FES1_LUT_PROGM, 0x3f92de46);
    RegWrite (FES1_LUT_PROGM, 0x40d68ff1);
    RegWrite (FES1_LUT_PROGM, 0xf43f181);
    RegWrite (FES1_LUT_PROGM, 0x3191e146);
    RegWrite (FES1_LUT_PROGM, 0x68d90fd9);
    RegWrite (FES1_LUT_PROGM, 0xf837181);
    RegWrite (FES1_LUT_PROGM, 0x2590e346);
    RegWrite (FES1_LUT_PROGM, 0x90db8fc1);
    RegWrite (FES1_LUT_PROGM, 0xfc2b161);
    RegWrite (FES1_LUT_PROGM, 0x198fe646);
    RegWrite (FES1_LUT_PROGM, 0xc0de0fa1);
    RegWrite (FES1_LUT_PROGM, 0x10023161);
    RegWrite (FES1_LUT_PROGM, 0xb8ee945);
    RegWrite (FES1_LUT_PROGM, 0xf0e00f89);
    RegWrite (FES1_LUT_PROGM, 0x1001b161);
    RegWrite (FES1_LUT_PROGM, 0xff8deb45);
    RegWrite (FES1_LUT_PROGM, 0x18e20f68);
    RegWrite (FES1_LUT_PROGM, 0x1040f162);
    RegWrite (FES1_LUT_PROGM, 0xf38cee45);
    RegWrite (FES1_LUT_PROGM, 0x48e48f48);
    RegWrite (FES1_LUT_PROGM, 0x10807142);
    RegWrite (FES1_LUT_PROGM, 0xe78cf045);
    RegWrite (FES1_LUT_PROGM, 0x78e68f30);
    RegWrite (FES1_LUT_PROGM, 0x10bff142);
    RegWrite (FES1_LUT_PROGM, 0xd98bf245);
    RegWrite (FES1_LUT_PROGM, 0xa0e88f10);
    RegWrite (FES1_LUT_PROGM, 0x10ff3142);
    RegWrite (FES1_LUT_PROGM, 0xcd8bf544);
    RegWrite (FES1_LUT_PROGM, 0xd0eb0ef0);
    RegWrite (FES1_LUT_PROGM, 0x10feb142);
    RegWrite (FES1_LUT_PROGM, 0xc18bf744);
    RegWrite (FES1_LUT_PROGM, 0xed0ed0);
    RegWrite (FES1_LUT_PROGM, 0x113df163);
}

void Set_Fes1_576p_to_480p(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x0004cccd);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x0004cccd);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x1b214d70);
    RegWrite (FES1_LUT_PROGM, 0xdfb8053);
    RegWrite (FES1_LUT_PROGM, 0x1b213175);
    RegWrite (FES1_LUT_PROGM, 0xdfb605a);
    RegWrite (FES1_LUT_PROGM, 0x9b19137a);
    RegWrite (FES1_LUT_PROGM, 0xdfb4061);
    RegWrite (FES1_LUT_PROGM, 0x1b10f97f);
    RegWrite (FES1_LUT_PROGM, 0xdfb2069);
    RegWrite (FES1_LUT_PROGM, 0x1af8dd84);
    RegWrite (FES1_LUT_PROGM, 0xdfb0070);
    RegWrite (FES1_LUT_PROGM, 0x9ae0c189);
    RegWrite (FES1_LUT_PROGM, 0xdfae877);
    RegWrite (FES1_LUT_PROGM, 0x1ac8a78f);
    RegWrite (FES1_LUT_PROGM, 0xddac87f);
    RegWrite (FES1_LUT_PROGM, 0x9aa88d94);
    RegWrite (FES1_LUT_PROGM, 0xddab086);
    RegWrite (FES1_LUT_PROGM, 0x1a88739a);
    RegWrite (FES1_LUT_PROGM, 0xdba988e);
    RegWrite (FES1_LUT_PROGM, 0x9a685b9f);
    RegWrite (FES1_LUT_PROGM, 0xd9a8895);
    RegWrite (FES1_LUT_PROGM, 0x1a4043a5);
    RegWrite (FES1_LUT_PROGM, 0xd5a709d);
    RegWrite (FES1_LUT_PROGM, 0x9a082bab);
    RegWrite (FES1_LUT_PROGM, 0xd3a60a4);
    RegWrite (FES1_LUT_PROGM, 0x19e013b0);
    RegWrite (FES1_LUT_PROGM, 0xcfa50ac);
    RegWrite (FES1_LUT_PROGM, 0x19bffdb6);
    RegWrite (FES1_LUT_PROGM, 0xcda48b4);
    RegWrite (FES1_LUT_PROGM, 0x9987e7bc);
    RegWrite (FES1_LUT_PROGM, 0xc9a38bb);
    RegWrite (FES1_LUT_PROGM, 0x1947d1c2);
    RegWrite (FES1_LUT_PROGM, 0xc3a38c3);
    RegWrite (FES1_LUT_PROGM, 0x990fbdc8);
    RegWrite (FES1_LUT_PROGM, 0xbfa30ca);
    RegWrite (FES1_LUT_PROGM, 0x18cfa9ce);
    RegWrite (FES1_LUT_PROGM, 0xb9a28d2);
    RegWrite (FES1_LUT_PROGM, 0x987f95d4);
    RegWrite (FES1_LUT_PROGM, 0xb3a28d9);
    RegWrite (FES1_LUT_PROGM, 0x184781da);
    RegWrite (FES1_LUT_PROGM, 0xafa28e1);
    RegWrite (FES1_LUT_PROGM, 0x97ff6fe0);
    RegWrite (FES1_LUT_PROGM, 0xa9a30e8);
    RegWrite (FES1_LUT_PROGM, 0x97a75fe6);
    RegWrite (FES1_LUT_PROGM, 0xa1a38ef);
    RegWrite (FES1_LUT_PROGM, 0x175f4deb);
    RegWrite (FES1_LUT_PROGM, 0x9ba40f7);
    RegWrite (FES1_LUT_PROGM, 0x17173df1);
    RegWrite (FES1_LUT_PROGM, 0x93a48ff);
    RegWrite (FES1_LUT_PROGM, 0x16bf2df7);
    RegWrite (FES1_LUT_PROGM, 0x8ba5906);
    RegWrite (FES1_LUT_PROGM, 0x96671ffd);
    RegWrite (FES1_LUT_PROGM, 0x83a690c);
    RegWrite (FES1_LUT_PROGM, 0x960f1002);
    RegWrite (FES1_LUT_PROGM, 0x7ba8113);
    RegWrite (FES1_LUT_PROGM, 0x15b70408);
    RegWrite (FES1_LUT_PROGM, 0x73a911b);
    RegWrite (FES1_LUT_PROGM, 0x954ef60d);
    RegWrite (FES1_LUT_PROGM, 0x6bab121);
    RegWrite (FES1_LUT_PROGM, 0x94f6ea13);
    RegWrite (FES1_LUT_PROGM, 0x61ac928);
    RegWrite (FES1_LUT_PROGM, 0x9496e018);
    RegWrite (FES1_LUT_PROGM, 0x59ae92f);
    RegWrite (FES1_LUT_PROGM, 0x1436d41d);
    RegWrite (FES1_LUT_PROGM, 0x4fb0936);
    RegWrite (FES1_LUT_PROGM, 0x13c6cc22);
    RegWrite (FES1_LUT_PROGM, 0x45b313c);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x1b214d70);
    RegWrite (FES1_LUT_PROGM, 0xdfb8053);
    RegWrite (FES1_LUT_PROGM, 0x1b213175);
    RegWrite (FES1_LUT_PROGM, 0xdfb605a);
    RegWrite (FES1_LUT_PROGM, 0x9b19137a);
    RegWrite (FES1_LUT_PROGM, 0xdfb4061);
    RegWrite (FES1_LUT_PROGM, 0x1b10f97f);
    RegWrite (FES1_LUT_PROGM, 0xdfb2069);
    RegWrite (FES1_LUT_PROGM, 0x1af8dd84);
    RegWrite (FES1_LUT_PROGM, 0xdfb0070);
    RegWrite (FES1_LUT_PROGM, 0x9ae0c189);
    RegWrite (FES1_LUT_PROGM, 0xdfae877);
    RegWrite (FES1_LUT_PROGM, 0x1ac8a78f);
    RegWrite (FES1_LUT_PROGM, 0xddac87f);
    RegWrite (FES1_LUT_PROGM, 0x9aa88d94);
    RegWrite (FES1_LUT_PROGM, 0xddab086);
    RegWrite (FES1_LUT_PROGM, 0x1a88739a);
    RegWrite (FES1_LUT_PROGM, 0xdba988e);
    RegWrite (FES1_LUT_PROGM, 0x9a685b9f);
    RegWrite (FES1_LUT_PROGM, 0xd9a8895);
    RegWrite (FES1_LUT_PROGM, 0x1a4043a5);
    RegWrite (FES1_LUT_PROGM, 0xd5a709d);
    RegWrite (FES1_LUT_PROGM, 0x9a082bab);
    RegWrite (FES1_LUT_PROGM, 0xd3a60a4);
    RegWrite (FES1_LUT_PROGM, 0x19e013b0);
    RegWrite (FES1_LUT_PROGM, 0xcfa50ac);
    RegWrite (FES1_LUT_PROGM, 0x19bffdb6);
    RegWrite (FES1_LUT_PROGM, 0xcda48b4);
    RegWrite (FES1_LUT_PROGM, 0x9987e7bc);
    RegWrite (FES1_LUT_PROGM, 0xc9a38bb);
    RegWrite (FES1_LUT_PROGM, 0x1947d1c2);
    RegWrite (FES1_LUT_PROGM, 0xc3a38c3);
    RegWrite (FES1_LUT_PROGM, 0x990fbdc8);
    RegWrite (FES1_LUT_PROGM, 0xbfa30ca);
    RegWrite (FES1_LUT_PROGM, 0x18cfa9ce);
    RegWrite (FES1_LUT_PROGM, 0xb9a28d2);
    RegWrite (FES1_LUT_PROGM, 0x987f95d4);
    RegWrite (FES1_LUT_PROGM, 0xb3a28d9);
    RegWrite (FES1_LUT_PROGM, 0x184781da);
    RegWrite (FES1_LUT_PROGM, 0xafa28e1);
    RegWrite (FES1_LUT_PROGM, 0x97ff6fe0);
    RegWrite (FES1_LUT_PROGM, 0xa9a30e8);
    RegWrite (FES1_LUT_PROGM, 0x97a75fe6);
    RegWrite (FES1_LUT_PROGM, 0xa1a38ef);
    RegWrite (FES1_LUT_PROGM, 0x175f4deb);
    RegWrite (FES1_LUT_PROGM, 0x9ba40f7);
    RegWrite (FES1_LUT_PROGM, 0x17173df1);
    RegWrite (FES1_LUT_PROGM, 0x93a48ff);
    RegWrite (FES1_LUT_PROGM, 0x16bf2df7);
    RegWrite (FES1_LUT_PROGM, 0x8ba5906);
    RegWrite (FES1_LUT_PROGM, 0x96671ffd);
    RegWrite (FES1_LUT_PROGM, 0x83a690c);
    RegWrite (FES1_LUT_PROGM, 0x960f1002);
    RegWrite (FES1_LUT_PROGM, 0x7ba8113);
    RegWrite (FES1_LUT_PROGM, 0x15b70408);
    RegWrite (FES1_LUT_PROGM, 0x73a911b);
    RegWrite (FES1_LUT_PROGM, 0x954ef60d);
    RegWrite (FES1_LUT_PROGM, 0x6bab121);
    RegWrite (FES1_LUT_PROGM, 0x94f6ea13);
    RegWrite (FES1_LUT_PROGM, 0x61ac928);
    RegWrite (FES1_LUT_PROGM, 0x9496e018);
    RegWrite (FES1_LUT_PROGM, 0x59ae92f);
    RegWrite (FES1_LUT_PROGM, 0x1436d41d);
    RegWrite (FES1_LUT_PROGM, 0x4fb0936);
    RegWrite (FES1_LUT_PROGM, 0x13c6cc22);
    RegWrite (FES1_LUT_PROGM, 0x45b313c);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
}

void Set_Fes1_576p_to_576p(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fdfe008);
    RegWrite (FES1_LUT_PROGM, 0xbfc008);
    RegWrite (FES1_LUT_PROGM, 0x9f9fc210);
    RegWrite (FES1_LUT_PROGM, 0x17f8010);
    RegWrite (FES1_LUT_PROGM, 0x9f77a617);
    RegWrite (FES1_LUT_PROGM, 0x21f4018);
    RegWrite (FES1_LUT_PROGM, 0x1f2f8a1e);
    RegWrite (FES1_LUT_PROGM, 0x2bf0021);
    RegWrite (FES1_LUT_PROGM, 0x1ee77025);
    RegWrite (FES1_LUT_PROGM, 0x35ec02a);
    RegWrite (FES1_LUT_PROGM, 0x9eaf582c);
    RegWrite (FES1_LUT_PROGM, 0x41e8032);
    RegWrite (FES1_LUT_PROGM, 0x9e574032);
    RegWrite (FES1_LUT_PROGM, 0x4be403b);
    RegWrite (FES1_LUT_PROGM, 0x9dff2a38);
    RegWrite (FES1_LUT_PROGM, 0x55e0044);
    RegWrite (FES1_LUT_PROGM, 0x9da7163e);
    RegWrite (FES1_LUT_PROGM, 0x5fdc04d);
    RegWrite (FES1_LUT_PROGM, 0x1d570244);
    RegWrite (FES1_LUT_PROGM, 0x69d8857);
    RegWrite (FES1_LUT_PROGM, 0x1cf6f049);
    RegWrite (FES1_LUT_PROGM, 0x71d4860);
    RegWrite (FES1_LUT_PROGM, 0x9c86e04e);
    RegWrite (FES1_LUT_PROGM, 0x7bd1069);
    RegWrite (FES1_LUT_PROGM, 0x9c26d053);
    RegWrite (FES1_LUT_PROGM, 0x83cd072);
    RegWrite (FES1_LUT_PROGM, 0x1bbec257);
    RegWrite (FES1_LUT_PROGM, 0x8dc987c);
    RegWrite (FES1_LUT_PROGM, 0x9b4eb45c);
    RegWrite (FES1_LUT_PROGM, 0x95c6085);
    RegWrite (FES1_LUT_PROGM, 0x1adea85f);
    RegWrite (FES1_LUT_PROGM, 0x9dc288f);
    RegWrite (FES1_LUT_PROGM, 0x9a669e63);
    RegWrite (FES1_LUT_PROGM, 0xa5bf898);
    RegWrite (FES1_LUT_PROGM, 0x99ee9466);
    RegWrite (FES1_LUT_PROGM, 0xadbc0a2);
    RegWrite (FES1_LUT_PROGM, 0x19768c69);
    RegWrite (FES1_LUT_PROGM, 0xb3b90ac);
    RegWrite (FES1_LUT_PROGM, 0x98fe846c);
    RegWrite (FES1_LUT_PROGM, 0xbbb60b5);
    RegWrite (FES1_LUT_PROGM, 0x187e7e6e);
    RegWrite (FES1_LUT_PROGM, 0xc1b30bf);
    RegWrite (FES1_LUT_PROGM, 0x17fe7870);
    RegWrite (FES1_LUT_PROGM, 0xc7b08c9);
    RegWrite (FES1_LUT_PROGM, 0x97767472);
    RegWrite (FES1_LUT_PROGM, 0xcdae0d2);
    RegWrite (FES1_LUT_PROGM, 0x96e67074);
    RegWrite (FES1_LUT_PROGM, 0xd3ab8db);
    RegWrite (FES1_LUT_PROGM, 0x96666e75);
    RegWrite (FES1_LUT_PROGM, 0xd7a90e5);
    RegWrite (FES1_LUT_PROGM, 0x15de6c76);
    RegWrite (FES1_LUT_PROGM, 0xdba70ef);
    RegWrite (FES1_LUT_PROGM, 0x15466c77);
    RegWrite (FES1_LUT_PROGM, 0xdfa50f8);
    RegWrite (FES1_LUT_PROGM, 0x14be6c77);
    RegWrite (FES1_LUT_PROGM, 0xe3a3102);
    RegWrite (FES1_LUT_PROGM, 0x142e6e77);
    RegWrite (FES1_LUT_PROGM, 0xe7a190b);
    RegWrite (FES1_LUT_PROGM, 0x13a67077);
    RegWrite (FES1_LUT_PROGM, 0xe9a0115);
    RegWrite (FES1_LUT_PROGM, 0x93167277);
    RegWrite (FES1_LUT_PROGM, 0xeb9e91e);
    RegWrite (FES1_LUT_PROGM, 0x927e7676);
    RegWrite (FES1_LUT_PROGM, 0xed9d927);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fdfe008);
    RegWrite (FES1_LUT_PROGM, 0xbfc008);
    RegWrite (FES1_LUT_PROGM, 0x9f9fc210);
    RegWrite (FES1_LUT_PROGM, 0x17f8010);
    RegWrite (FES1_LUT_PROGM, 0x9f77a617);
    RegWrite (FES1_LUT_PROGM, 0x21f4018);
    RegWrite (FES1_LUT_PROGM, 0x1f2f8a1e);
    RegWrite (FES1_LUT_PROGM, 0x2bf0021);
    RegWrite (FES1_LUT_PROGM, 0x1ee77025);
    RegWrite (FES1_LUT_PROGM, 0x35ec02a);
    RegWrite (FES1_LUT_PROGM, 0x9eaf582c);
    RegWrite (FES1_LUT_PROGM, 0x41e8032);
    RegWrite (FES1_LUT_PROGM, 0x9e574032);
    RegWrite (FES1_LUT_PROGM, 0x4be403b);
    RegWrite (FES1_LUT_PROGM, 0x9dff2a38);
    RegWrite (FES1_LUT_PROGM, 0x55e0044);
    RegWrite (FES1_LUT_PROGM, 0x9da7163e);
    RegWrite (FES1_LUT_PROGM, 0x5fdc04d);
    RegWrite (FES1_LUT_PROGM, 0x1d570244);
    RegWrite (FES1_LUT_PROGM, 0x69d8857);
    RegWrite (FES1_LUT_PROGM, 0x1cf6f049);
    RegWrite (FES1_LUT_PROGM, 0x71d4860);
    RegWrite (FES1_LUT_PROGM, 0x9c86e04e);
    RegWrite (FES1_LUT_PROGM, 0x7bd1069);
    RegWrite (FES1_LUT_PROGM, 0x9c26d053);
    RegWrite (FES1_LUT_PROGM, 0x83cd072);
    RegWrite (FES1_LUT_PROGM, 0x1bbec257);
    RegWrite (FES1_LUT_PROGM, 0x8dc987c);
    RegWrite (FES1_LUT_PROGM, 0x9b4eb45c);
    RegWrite (FES1_LUT_PROGM, 0x95c6085);
    RegWrite (FES1_LUT_PROGM, 0x1adea85f);
    RegWrite (FES1_LUT_PROGM, 0x9dc288f);
    RegWrite (FES1_LUT_PROGM, 0x9a669e63);
    RegWrite (FES1_LUT_PROGM, 0xa5bf898);
    RegWrite (FES1_LUT_PROGM, 0x99ee9466);
    RegWrite (FES1_LUT_PROGM, 0xadbc0a2);
    RegWrite (FES1_LUT_PROGM, 0x19768c69);
    RegWrite (FES1_LUT_PROGM, 0xb3b90ac);
    RegWrite (FES1_LUT_PROGM, 0x98fe846c);
    RegWrite (FES1_LUT_PROGM, 0xbbb60b5);
    RegWrite (FES1_LUT_PROGM, 0x187e7e6e);
    RegWrite (FES1_LUT_PROGM, 0xc1b30bf);
    RegWrite (FES1_LUT_PROGM, 0x17fe7870);
    RegWrite (FES1_LUT_PROGM, 0xc7b08c9);
    RegWrite (FES1_LUT_PROGM, 0x97767472);
    RegWrite (FES1_LUT_PROGM, 0xcdae0d2);
    RegWrite (FES1_LUT_PROGM, 0x96e67074);
    RegWrite (FES1_LUT_PROGM, 0xd3ab8db);
    RegWrite (FES1_LUT_PROGM, 0x96666e75);
    RegWrite (FES1_LUT_PROGM, 0xd7a90e5);
    RegWrite (FES1_LUT_PROGM, 0x15de6c76);
    RegWrite (FES1_LUT_PROGM, 0xdba70ef);
    RegWrite (FES1_LUT_PROGM, 0x15466c77);
    RegWrite (FES1_LUT_PROGM, 0xdfa50f8);
    RegWrite (FES1_LUT_PROGM, 0x14be6c77);
    RegWrite (FES1_LUT_PROGM, 0xe3a3102);
    RegWrite (FES1_LUT_PROGM, 0x142e6e77);
    RegWrite (FES1_LUT_PROGM, 0xe7a190b);
    RegWrite (FES1_LUT_PROGM, 0x13a67077);
    RegWrite (FES1_LUT_PROGM, 0xe9a0115);
    RegWrite (FES1_LUT_PROGM, 0x93167277);
    RegWrite (FES1_LUT_PROGM, 0xeb9e91e);
    RegWrite (FES1_LUT_PROGM, 0x927e7676);
    RegWrite (FES1_LUT_PROGM, 0xed9d927);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
}

void Set_Fes1_480p_to_480p(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fdfe008);
    RegWrite (FES1_LUT_PROGM, 0xbfc008);
    RegWrite (FES1_LUT_PROGM, 0x9f9fc210);
    RegWrite (FES1_LUT_PROGM, 0x17f8010);
    RegWrite (FES1_LUT_PROGM, 0x9f77a617);
    RegWrite (FES1_LUT_PROGM, 0x21f4018);
    RegWrite (FES1_LUT_PROGM, 0x1f2f8a1e);
    RegWrite (FES1_LUT_PROGM, 0x2bf0021);
    RegWrite (FES1_LUT_PROGM, 0x1ee77025);
    RegWrite (FES1_LUT_PROGM, 0x35ec02a);
    RegWrite (FES1_LUT_PROGM, 0x9eaf582c);
    RegWrite (FES1_LUT_PROGM, 0x41e8032);
    RegWrite (FES1_LUT_PROGM, 0x9e574032);
    RegWrite (FES1_LUT_PROGM, 0x4be403b);
    RegWrite (FES1_LUT_PROGM, 0x9dff2a38);
    RegWrite (FES1_LUT_PROGM, 0x55e0044);
    RegWrite (FES1_LUT_PROGM, 0x9da7163e);
    RegWrite (FES1_LUT_PROGM, 0x5fdc04d);
    RegWrite (FES1_LUT_PROGM, 0x1d570244);
    RegWrite (FES1_LUT_PROGM, 0x69d8857);
    RegWrite (FES1_LUT_PROGM, 0x1cf6f049);
    RegWrite (FES1_LUT_PROGM, 0x71d4860);
    RegWrite (FES1_LUT_PROGM, 0x9c86e04e);
    RegWrite (FES1_LUT_PROGM, 0x7bd1069);
    RegWrite (FES1_LUT_PROGM, 0x9c26d053);
    RegWrite (FES1_LUT_PROGM, 0x83cd072);
    RegWrite (FES1_LUT_PROGM, 0x1bbec257);
    RegWrite (FES1_LUT_PROGM, 0x8dc987c);
    RegWrite (FES1_LUT_PROGM, 0x9b4eb45c);
    RegWrite (FES1_LUT_PROGM, 0x95c6085);
    RegWrite (FES1_LUT_PROGM, 0x1adea85f);
    RegWrite (FES1_LUT_PROGM, 0x9dc288f);
    RegWrite (FES1_LUT_PROGM, 0x9a669e63);
    RegWrite (FES1_LUT_PROGM, 0xa5bf898);
    RegWrite (FES1_LUT_PROGM, 0x99ee9466);
    RegWrite (FES1_LUT_PROGM, 0xadbc0a2);
    RegWrite (FES1_LUT_PROGM, 0x19768c69);
    RegWrite (FES1_LUT_PROGM, 0xb3b90ac);
    RegWrite (FES1_LUT_PROGM, 0x98fe846c);
    RegWrite (FES1_LUT_PROGM, 0xbbb60b5);
    RegWrite (FES1_LUT_PROGM, 0x187e7e6e);
    RegWrite (FES1_LUT_PROGM, 0xc1b30bf);
    RegWrite (FES1_LUT_PROGM, 0x17fe7870);
    RegWrite (FES1_LUT_PROGM, 0xc7b08c9);
    RegWrite (FES1_LUT_PROGM, 0x97767472);
    RegWrite (FES1_LUT_PROGM, 0xcdae0d2);
    RegWrite (FES1_LUT_PROGM, 0x96e67074);
    RegWrite (FES1_LUT_PROGM, 0xd3ab8db);
    RegWrite (FES1_LUT_PROGM, 0x96666e75);
    RegWrite (FES1_LUT_PROGM, 0xd7a90e5);
    RegWrite (FES1_LUT_PROGM, 0x15de6c76);
    RegWrite (FES1_LUT_PROGM, 0xdba70ef);
    RegWrite (FES1_LUT_PROGM, 0x15466c77);
    RegWrite (FES1_LUT_PROGM, 0xdfa50f8);
    RegWrite (FES1_LUT_PROGM, 0x14be6c77);
    RegWrite (FES1_LUT_PROGM, 0xe3a3102);
    RegWrite (FES1_LUT_PROGM, 0x142e6e77);
    RegWrite (FES1_LUT_PROGM, 0xe7a190b);
    RegWrite (FES1_LUT_PROGM, 0x13a67077);
    RegWrite (FES1_LUT_PROGM, 0xe9a0115);
    RegWrite (FES1_LUT_PROGM, 0x93167277);
    RegWrite (FES1_LUT_PROGM, 0xeb9e91e);
    RegWrite (FES1_LUT_PROGM, 0x927e7676);
    RegWrite (FES1_LUT_PROGM, 0xed9d927);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fdfe008);
    RegWrite (FES1_LUT_PROGM, 0xbfc008);
    RegWrite (FES1_LUT_PROGM, 0x9f9fc210);
    RegWrite (FES1_LUT_PROGM, 0x17f8010);
    RegWrite (FES1_LUT_PROGM, 0x9f77a617);
    RegWrite (FES1_LUT_PROGM, 0x21f4018);
    RegWrite (FES1_LUT_PROGM, 0x1f2f8a1e);
    RegWrite (FES1_LUT_PROGM, 0x2bf0021);
    RegWrite (FES1_LUT_PROGM, 0x1ee77025);
    RegWrite (FES1_LUT_PROGM, 0x35ec02a);
    RegWrite (FES1_LUT_PROGM, 0x9eaf582c);
    RegWrite (FES1_LUT_PROGM, 0x41e8032);
    RegWrite (FES1_LUT_PROGM, 0x9e574032);
    RegWrite (FES1_LUT_PROGM, 0x4be403b);
    RegWrite (FES1_LUT_PROGM, 0x9dff2a38);
    RegWrite (FES1_LUT_PROGM, 0x55e0044);
    RegWrite (FES1_LUT_PROGM, 0x9da7163e);
    RegWrite (FES1_LUT_PROGM, 0x5fdc04d);
    RegWrite (FES1_LUT_PROGM, 0x1d570244);
    RegWrite (FES1_LUT_PROGM, 0x69d8857);
    RegWrite (FES1_LUT_PROGM, 0x1cf6f049);
    RegWrite (FES1_LUT_PROGM, 0x71d4860);
    RegWrite (FES1_LUT_PROGM, 0x9c86e04e);
    RegWrite (FES1_LUT_PROGM, 0x7bd1069);
    RegWrite (FES1_LUT_PROGM, 0x9c26d053);
    RegWrite (FES1_LUT_PROGM, 0x83cd072);
    RegWrite (FES1_LUT_PROGM, 0x1bbec257);
    RegWrite (FES1_LUT_PROGM, 0x8dc987c);
    RegWrite (FES1_LUT_PROGM, 0x9b4eb45c);
    RegWrite (FES1_LUT_PROGM, 0x95c6085);
    RegWrite (FES1_LUT_PROGM, 0x1adea85f);
    RegWrite (FES1_LUT_PROGM, 0x9dc288f);
    RegWrite (FES1_LUT_PROGM, 0x9a669e63);
    RegWrite (FES1_LUT_PROGM, 0xa5bf898);
    RegWrite (FES1_LUT_PROGM, 0x99ee9466);
    RegWrite (FES1_LUT_PROGM, 0xadbc0a2);
    RegWrite (FES1_LUT_PROGM, 0x19768c69);
    RegWrite (FES1_LUT_PROGM, 0xb3b90ac);
    RegWrite (FES1_LUT_PROGM, 0x98fe846c);
    RegWrite (FES1_LUT_PROGM, 0xbbb60b5);
    RegWrite (FES1_LUT_PROGM, 0x187e7e6e);
    RegWrite (FES1_LUT_PROGM, 0xc1b30bf);
    RegWrite (FES1_LUT_PROGM, 0x17fe7870);
    RegWrite (FES1_LUT_PROGM, 0xc7b08c9);
    RegWrite (FES1_LUT_PROGM, 0x97767472);
    RegWrite (FES1_LUT_PROGM, 0xcdae0d2);
    RegWrite (FES1_LUT_PROGM, 0x96e67074);
    RegWrite (FES1_LUT_PROGM, 0xd3ab8db);
    RegWrite (FES1_LUT_PROGM, 0x96666e75);
    RegWrite (FES1_LUT_PROGM, 0xd7a90e5);
    RegWrite (FES1_LUT_PROGM, 0x15de6c76);
    RegWrite (FES1_LUT_PROGM, 0xdba70ef);
    RegWrite (FES1_LUT_PROGM, 0x15466c77);
    RegWrite (FES1_LUT_PROGM, 0xdfa50f8);
    RegWrite (FES1_LUT_PROGM, 0x14be6c77);
    RegWrite (FES1_LUT_PROGM, 0xe3a3102);
    RegWrite (FES1_LUT_PROGM, 0x142e6e77);
    RegWrite (FES1_LUT_PROGM, 0xe7a190b);
    RegWrite (FES1_LUT_PROGM, 0x13a67077);
    RegWrite (FES1_LUT_PROGM, 0xe9a0115);
    RegWrite (FES1_LUT_PROGM, 0x93167277);
    RegWrite (FES1_LUT_PROGM, 0xeb9e91e);
    RegWrite (FES1_LUT_PROGM, 0x927e7676);
    RegWrite (FES1_LUT_PROGM, 0xed9d927);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
}

void Set_Fes1_480p_to_576p(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00035555);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00035555);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fffe603);
    RegWrite (FES1_LUT_PROGM, 0x1fe807);
    RegWrite (FES1_LUT_PROGM, 0x1ff7ce06);
    RegWrite (FES1_LUT_PROGM, 0x1fc80e);
    RegWrite (FES1_LUT_PROGM, 0x1fefb609);
    RegWrite (FES1_LUT_PROGM, 0x1fa815);
    RegWrite (FES1_LUT_PROGM, 0x1fd7a00c);
    RegWrite (FES1_LUT_PROGM, 0x1f881d);
    RegWrite (FES1_LUT_PROGM, 0x9fb78a0e);
    RegWrite (FES1_LUT_PROGM, 0x3f6824);
    RegWrite (FES1_LUT_PROGM, 0x1f8f7611);
    RegWrite (FES1_LUT_PROGM, 0x3f482d);
    RegWrite (FES1_LUT_PROGM, 0x1f5f6413);
    RegWrite (FES1_LUT_PROGM, 0x3f2835);
    RegWrite (FES1_LUT_PROGM, 0x1f275215);
    RegWrite (FES1_LUT_PROGM, 0x5f083e);
    RegWrite (FES1_LUT_PROGM, 0x9eef4217);
    RegWrite (FES1_LUT_PROGM, 0x5ee046);
    RegWrite (FES1_LUT_PROGM, 0x1eaf3218);
    RegWrite (FES1_LUT_PROGM, 0x7ec050);
    RegWrite (FES1_LUT_PROGM, 0x1e5f241a);
    RegWrite (FES1_LUT_PROGM, 0x9e9859);
    RegWrite (FES1_LUT_PROGM, 0x9e0f181b);
    RegWrite (FES1_LUT_PROGM, 0x9e7062);
    RegWrite (FES1_LUT_PROGM, 0x9dbf0c1c);
    RegWrite (FES1_LUT_PROGM, 0xbe486c);
    RegWrite (FES1_LUT_PROGM, 0x9d67001d);
    RegWrite (FES1_LUT_PROGM, 0xde2076);
    RegWrite (FES1_LUT_PROGM, 0x9cf6f81e);
    RegWrite (FES1_LUT_PROGM, 0xfdf880);
    RegWrite (FES1_LUT_PROGM, 0x9c8ef01f);
    RegWrite (FES1_LUT_PROGM, 0x11dd08a);
    RegWrite (FES1_LUT_PROGM, 0x9c2ee81f);
    RegWrite (FES1_LUT_PROGM, 0x13da895);
    RegWrite (FES1_LUT_PROGM, 0x1bb6e220);
    RegWrite (FES1_LUT_PROGM, 0x13d80a0);
    RegWrite (FES1_LUT_PROGM, 0x9b36dc20);
    RegWrite (FES1_LUT_PROGM, 0x15d60aa);
    RegWrite (FES1_LUT_PROGM, 0x1ab6d820);
    RegWrite (FES1_LUT_PROGM, 0x19d38b5);
    RegWrite (FES1_LUT_PROGM, 0x1a3ed620);
    RegWrite (FES1_LUT_PROGM, 0x1bd10c0);
    RegWrite (FES1_LUT_PROGM, 0x19b6d220);
    RegWrite (FES1_LUT_PROGM, 0x1dce8cb);
    RegWrite (FES1_LUT_PROGM, 0x192ed220);
    RegWrite (FES1_LUT_PROGM, 0x1fcc0d6);
    RegWrite (FES1_LUT_PROGM, 0x1896d01f);
    RegWrite (FES1_LUT_PROGM, 0x21ca0e1);
    RegWrite (FES1_LUT_PROGM, 0x1806d21f);
    RegWrite (FES1_LUT_PROGM, 0x23c78ec);
    RegWrite (FES1_LUT_PROGM, 0x976ed21e);
    RegWrite (FES1_LUT_PROGM, 0x25c58f7);
    RegWrite (FES1_LUT_PROGM, 0x16d6d41d);
    RegWrite (FES1_LUT_PROGM, 0x29c3103);
    RegWrite (FES1_LUT_PROGM, 0x962ed81d);
    RegWrite (FES1_LUT_PROGM, 0x2bc190d);
    RegWrite (FES1_LUT_PROGM, 0x958edc1c);
    RegWrite (FES1_LUT_PROGM, 0x2dbf918);
    RegWrite (FES1_LUT_PROGM, 0x14e6e01b);
    RegWrite (FES1_LUT_PROGM, 0x2fbd923);
    RegWrite (FES1_LUT_PROGM, 0x143ee41a);
    RegWrite (FES1_LUT_PROGM, 0x31bc12e);
    RegWrite (FES1_LUT_PROGM, 0x938eea19);
    RegWrite (FES1_LUT_PROGM, 0x33ba938);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fffe603);
    RegWrite (FES1_LUT_PROGM, 0x1fe807);
    RegWrite (FES1_LUT_PROGM, 0x1ff7ce06);
    RegWrite (FES1_LUT_PROGM, 0x1fc80e);
    RegWrite (FES1_LUT_PROGM, 0x1fefb609);
    RegWrite (FES1_LUT_PROGM, 0x1fa815);
    RegWrite (FES1_LUT_PROGM, 0x1fd7a00c);
    RegWrite (FES1_LUT_PROGM, 0x1f881d);
    RegWrite (FES1_LUT_PROGM, 0x9fb78a0e);
    RegWrite (FES1_LUT_PROGM, 0x3f6824);
    RegWrite (FES1_LUT_PROGM, 0x1f8f7611);
    RegWrite (FES1_LUT_PROGM, 0x3f482d);
    RegWrite (FES1_LUT_PROGM, 0x1f5f6413);
    RegWrite (FES1_LUT_PROGM, 0x3f2835);
    RegWrite (FES1_LUT_PROGM, 0x1f275215);
    RegWrite (FES1_LUT_PROGM, 0x5f083e);
    RegWrite (FES1_LUT_PROGM, 0x9eef4217);
    RegWrite (FES1_LUT_PROGM, 0x5ee046);
    RegWrite (FES1_LUT_PROGM, 0x1eaf3218);
    RegWrite (FES1_LUT_PROGM, 0x7ec050);
    RegWrite (FES1_LUT_PROGM, 0x1e5f241a);
    RegWrite (FES1_LUT_PROGM, 0x9e9859);
    RegWrite (FES1_LUT_PROGM, 0x9e0f181b);
    RegWrite (FES1_LUT_PROGM, 0x9e7062);
    RegWrite (FES1_LUT_PROGM, 0x9dbf0c1c);
    RegWrite (FES1_LUT_PROGM, 0xbe486c);
    RegWrite (FES1_LUT_PROGM, 0x9d67001d);
    RegWrite (FES1_LUT_PROGM, 0xde2076);
    RegWrite (FES1_LUT_PROGM, 0x9cf6f81e);
    RegWrite (FES1_LUT_PROGM, 0xfdf880);
    RegWrite (FES1_LUT_PROGM, 0x9c8ef01f);
    RegWrite (FES1_LUT_PROGM, 0x11dd08a);
    RegWrite (FES1_LUT_PROGM, 0x9c2ee81f);
    RegWrite (FES1_LUT_PROGM, 0x13da895);
    RegWrite (FES1_LUT_PROGM, 0x1bb6e220);
    RegWrite (FES1_LUT_PROGM, 0x13d80a0);
    RegWrite (FES1_LUT_PROGM, 0x9b36dc20);
    RegWrite (FES1_LUT_PROGM, 0x15d60aa);
    RegWrite (FES1_LUT_PROGM, 0x1ab6d820);
    RegWrite (FES1_LUT_PROGM, 0x19d38b5);
    RegWrite (FES1_LUT_PROGM, 0x1a3ed620);
    RegWrite (FES1_LUT_PROGM, 0x1bd10c0);
    RegWrite (FES1_LUT_PROGM, 0x19b6d220);
    RegWrite (FES1_LUT_PROGM, 0x1dce8cb);
    RegWrite (FES1_LUT_PROGM, 0x192ed220);
    RegWrite (FES1_LUT_PROGM, 0x1fcc0d6);
    RegWrite (FES1_LUT_PROGM, 0x1896d01f);
    RegWrite (FES1_LUT_PROGM, 0x21ca0e1);
    RegWrite (FES1_LUT_PROGM, 0x1806d21f);
    RegWrite (FES1_LUT_PROGM, 0x23c78ec);
    RegWrite (FES1_LUT_PROGM, 0x976ed21e);
    RegWrite (FES1_LUT_PROGM, 0x25c58f7);
    RegWrite (FES1_LUT_PROGM, 0x16d6d41d);
    RegWrite (FES1_LUT_PROGM, 0x29c3103);
    RegWrite (FES1_LUT_PROGM, 0x962ed81d);
    RegWrite (FES1_LUT_PROGM, 0x2bc190d);
    RegWrite (FES1_LUT_PROGM, 0x958edc1c);
    RegWrite (FES1_LUT_PROGM, 0x2dbf918);
    RegWrite (FES1_LUT_PROGM, 0x14e6e01b);
    RegWrite (FES1_LUT_PROGM, 0x2fbd923);
    RegWrite (FES1_LUT_PROGM, 0x143ee41a);
    RegWrite (FES1_LUT_PROGM, 0x31bc12e);
    RegWrite (FES1_LUT_PROGM, 0x938eea19);
    RegWrite (FES1_LUT_PROGM, 0x33ba938);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
}

void Set_Fes1_1080i_to_480i(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00090000);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00090000);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x000aaaab);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x000aaaab);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x8d124e33);
    RegWrite (FES1_LUT_PROGM, 0x35219893);
    RegWrite (FES1_LUT_PROGM, 0xd124630);
    RegWrite (FES1_LUT_PROGM, 0x3541b895);
    RegWrite (FES1_LUT_PROGM, 0x8d0a3e2c);
    RegWrite (FES1_LUT_PROGM, 0x3561d096);
    RegWrite (FES1_LUT_PROGM, 0x8d0a3829);
    RegWrite (FES1_LUT_PROGM, 0x3581f098);
    RegWrite (FES1_LUT_PROGM, 0xd0a3025);
    RegWrite (FES1_LUT_PROGM, 0x35a2089a);
    RegWrite (FES1_LUT_PROGM, 0x8d022822);
    RegWrite (FES1_LUT_PROGM, 0x35c2289b);
    RegWrite (FES1_LUT_PROGM, 0xcfa221f);
    RegWrite (FES1_LUT_PROGM, 0x35e2409d);
    RegWrite (FES1_LUT_PROGM, 0x8cfa1a1c);
    RegWrite (FES1_LUT_PROGM, 0x3602609e);
    RegWrite (FES1_LUT_PROGM, 0xcf21218);
    RegWrite (FES1_LUT_PROGM, 0x362278a0);
    RegWrite (FES1_LUT_PROGM, 0x8cf20a15);
    RegWrite (FES1_LUT_PROGM, 0x364298a1);
    RegWrite (FES1_LUT_PROGM, 0xcea0212);
    RegWrite (FES1_LUT_PROGM, 0x3662b8a3);
    RegWrite (FES1_LUT_PROGM, 0x8ce1fc0f);
    RegWrite (FES1_LUT_PROGM, 0x3682d0a4);
    RegWrite (FES1_LUT_PROGM, 0xcd9f40c);
    RegWrite (FES1_LUT_PROGM, 0x36a2f0a6);
    RegWrite (FES1_LUT_PROGM, 0x8cd1ec09);
    RegWrite (FES1_LUT_PROGM, 0x36c310a7);
    RegWrite (FES1_LUT_PROGM, 0xcc9e406);
    RegWrite (FES1_LUT_PROGM, 0x370330a9);
    RegWrite (FES1_LUT_PROGM, 0x8cc1dc03);
    RegWrite (FES1_LUT_PROGM, 0x372348aa);
    RegWrite (FES1_LUT_PROGM, 0x8cb9d400);
    RegWrite (FES1_LUT_PROGM, 0x376368ab);
    RegWrite (FES1_LUT_PROGM, 0xcb1cdfd);
    RegWrite (FES1_LUT_PROGM, 0x378388ad);
    RegWrite (FES1_LUT_PROGM, 0x8ca1c5fa);
    RegWrite (FES1_LUT_PROGM, 0x37a3a8ae);
    RegWrite (FES1_LUT_PROGM, 0x8c99bdf8);
    RegWrite (FES1_LUT_PROGM, 0x37e3c8af);
    RegWrite (FES1_LUT_PROGM, 0xc91b7f5);
    RegWrite (FES1_LUT_PROGM, 0x3823e0b1);
    RegWrite (FES1_LUT_PROGM, 0xc81aff2);
    RegWrite (FES1_LUT_PROGM, 0x384400b2);
    RegWrite (FES1_LUT_PROGM, 0x8c79a7ef);
    RegWrite (FES1_LUT_PROGM, 0x388420b3);
    RegWrite (FES1_LUT_PROGM, 0x8c699fed);
    RegWrite (FES1_LUT_PROGM, 0x38c440b4);
    RegWrite (FES1_LUT_PROGM, 0xc6197ea);
    RegWrite (FES1_LUT_PROGM, 0x38e460b6);
    RegWrite (FES1_LUT_PROGM, 0xc518fe8);
    RegWrite (FES1_LUT_PROGM, 0x392480b7);
    RegWrite (FES1_LUT_PROGM, 0xc4187e5);
    RegWrite (FES1_LUT_PROGM, 0x3964a0b8);
    RegWrite (FES1_LUT_PROGM, 0x8c397fe3);
    RegWrite (FES1_LUT_PROGM, 0x39a4c0b9);
    RegWrite (FES1_LUT_PROGM, 0x8c2977e0);
    RegWrite (FES1_LUT_PROGM, 0x39e4e0ba);
    RegWrite (FES1_LUT_PROGM, 0x8c196fde);
    RegWrite (FES1_LUT_PROGM, 0x3a24f8bb);
    RegWrite (FES1_LUT_PROGM, 0x8c0967dc);
    RegWrite (FES1_LUT_PROGM, 0x3a6518bc);
    RegWrite (FES1_LUT_PROGM, 0x8bf95fda);
    RegWrite (FES1_LUT_PROGM, 0x3aa538bd);
    RegWrite (FES1_LUT_PROGM, 0x8be957d7);
    RegWrite (FES1_LUT_PROGM, 0x3ae558be);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x8d124e33);
    RegWrite (FES1_LUT_PROGM, 0x35219893);
    RegWrite (FES1_LUT_PROGM, 0xd124630);
    RegWrite (FES1_LUT_PROGM, 0x3541b895);
    RegWrite (FES1_LUT_PROGM, 0x8d0a3e2c);
    RegWrite (FES1_LUT_PROGM, 0x3561d096);
    RegWrite (FES1_LUT_PROGM, 0x8d0a3829);
    RegWrite (FES1_LUT_PROGM, 0x3581f098);
    RegWrite (FES1_LUT_PROGM, 0xd0a3025);
    RegWrite (FES1_LUT_PROGM, 0x35a2089a);
    RegWrite (FES1_LUT_PROGM, 0x8d022822);
    RegWrite (FES1_LUT_PROGM, 0x35c2289b);
    RegWrite (FES1_LUT_PROGM, 0xcfa221f);
    RegWrite (FES1_LUT_PROGM, 0x35e2409d);
    RegWrite (FES1_LUT_PROGM, 0x8cfa1a1c);
    RegWrite (FES1_LUT_PROGM, 0x3602609e);
    RegWrite (FES1_LUT_PROGM, 0xcf21218);
    RegWrite (FES1_LUT_PROGM, 0x362278a0);
    RegWrite (FES1_LUT_PROGM, 0x8cf20a15);
    RegWrite (FES1_LUT_PROGM, 0x364298a1);
    RegWrite (FES1_LUT_PROGM, 0xcea0212);
    RegWrite (FES1_LUT_PROGM, 0x3662b8a3);
    RegWrite (FES1_LUT_PROGM, 0x8ce1fc0f);
    RegWrite (FES1_LUT_PROGM, 0x3682d0a4);
    RegWrite (FES1_LUT_PROGM, 0xcd9f40c);
    RegWrite (FES1_LUT_PROGM, 0x36a2f0a6);
    RegWrite (FES1_LUT_PROGM, 0x8cd1ec09);
    RegWrite (FES1_LUT_PROGM, 0x36c310a7);
    RegWrite (FES1_LUT_PROGM, 0xcc9e406);
    RegWrite (FES1_LUT_PROGM, 0x370330a9);
    RegWrite (FES1_LUT_PROGM, 0x8cc1dc03);
    RegWrite (FES1_LUT_PROGM, 0x372348aa);
    RegWrite (FES1_LUT_PROGM, 0x8cb9d400);
    RegWrite (FES1_LUT_PROGM, 0x376368ab);
    RegWrite (FES1_LUT_PROGM, 0xcb1cdfd);
    RegWrite (FES1_LUT_PROGM, 0x378388ad);
    RegWrite (FES1_LUT_PROGM, 0x8ca1c5fa);
    RegWrite (FES1_LUT_PROGM, 0x37a3a8ae);
    RegWrite (FES1_LUT_PROGM, 0x8c99bdf8);
    RegWrite (FES1_LUT_PROGM, 0x37e3c8af);
    RegWrite (FES1_LUT_PROGM, 0xc91b7f5);
    RegWrite (FES1_LUT_PROGM, 0x3823e0b1);
    RegWrite (FES1_LUT_PROGM, 0xc81aff2);
    RegWrite (FES1_LUT_PROGM, 0x384400b2);
    RegWrite (FES1_LUT_PROGM, 0x8c79a7ef);
    RegWrite (FES1_LUT_PROGM, 0x388420b3);
    RegWrite (FES1_LUT_PROGM, 0x8c699fed);
    RegWrite (FES1_LUT_PROGM, 0x38c440b4);
    RegWrite (FES1_LUT_PROGM, 0xc6197ea);
    RegWrite (FES1_LUT_PROGM, 0x38e460b6);
    RegWrite (FES1_LUT_PROGM, 0xc518fe8);
    RegWrite (FES1_LUT_PROGM, 0x392480b7);
    RegWrite (FES1_LUT_PROGM, 0xc4187e5);
    RegWrite (FES1_LUT_PROGM, 0x3964a0b8);
    RegWrite (FES1_LUT_PROGM, 0x8c397fe3);
    RegWrite (FES1_LUT_PROGM, 0x39a4c0b9);
    RegWrite (FES1_LUT_PROGM, 0x8c2977e0);
    RegWrite (FES1_LUT_PROGM, 0x39e4e0ba);
    RegWrite (FES1_LUT_PROGM, 0x8c196fde);
    RegWrite (FES1_LUT_PROGM, 0x3a24f8bb);
    RegWrite (FES1_LUT_PROGM, 0x8c0967dc);
    RegWrite (FES1_LUT_PROGM, 0x3a6518bc);
    RegWrite (FES1_LUT_PROGM, 0x8bf95fda);
    RegWrite (FES1_LUT_PROGM, 0x3aa538bd);
    RegWrite (FES1_LUT_PROGM, 0x8be957d7);
    RegWrite (FES1_LUT_PROGM, 0x3ae558be);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x8c7dd3a8);
    RegWrite (FES1_LUT_PROGM, 0xe8a30d02);
    RegWrite (FES1_LUT_PROGM, 0x396a3a63);
    RegWrite (FES1_LUT_PROGM, 0x887ad1a8);
    RegWrite (FES1_LUT_PROGM, 0xa48d02);
    RegWrite (FES1_LUT_PROGM, 0x3929faa4);
    RegWrite (FES1_LUT_PROGM, 0x8477cfa8);
    RegWrite (FES1_LUT_PROGM, 0x20a60d0a);
    RegWrite (FES1_LUT_PROGM, 0x38a9fac4);
    RegWrite (FES1_LUT_PROGM, 0x7e73cea9);
    RegWrite (FES1_LUT_PROGM, 0x38a78d0a);
    RegWrite (FES1_LUT_PROGM, 0x3869fb04);
    RegWrite (FES1_LUT_PROGM, 0x7a70cca9);
    RegWrite (FES1_LUT_PROGM, 0x58a90d0a);
    RegWrite (FES1_LUT_PROGM, 0x3829bb44);
    RegWrite (FES1_LUT_PROGM, 0x766dcaa9);
    RegWrite (FES1_LUT_PROGM, 0x70aa8d12);
    RegWrite (FES1_LUT_PROGM, 0x37a9bb84);
    RegWrite (FES1_LUT_PROGM, 0x706ac9aa);
    RegWrite (FES1_LUT_PROGM, 0x90ab8d12);
    RegWrite (FES1_LUT_PROGM, 0x3769bbc4);
    RegWrite (FES1_LUT_PROGM, 0x6c67c7aa);
    RegWrite (FES1_LUT_PROGM, 0xa8ad0d12);
    RegWrite (FES1_LUT_PROGM, 0x37297c04);
    RegWrite (FES1_LUT_PROGM, 0x6664c5ab);
    RegWrite (FES1_LUT_PROGM, 0xc8ae8d12);
    RegWrite (FES1_LUT_PROGM, 0x36a97c44);
    RegWrite (FES1_LUT_PROGM, 0x6061c4ab);
    RegWrite (FES1_LUT_PROGM, 0xe0b00d12);
    RegWrite (FES1_LUT_PROGM, 0x36697c84);
    RegWrite (FES1_LUT_PROGM, 0x5c5ec2ac);
    RegWrite (FES1_LUT_PROGM, 0xb10d12);
    RegWrite (FES1_LUT_PROGM, 0x36297cc5);
    RegWrite (FES1_LUT_PROGM, 0x565bc1ac);
    RegWrite (FES1_LUT_PROGM, 0x18b28d12);
    RegWrite (FES1_LUT_PROGM, 0x35e97d05);
    RegWrite (FES1_LUT_PROGM, 0x5057bfad);
    RegWrite (FES1_LUT_PROGM, 0x38b40d12);
    RegWrite (FES1_LUT_PROGM, 0x35697d45);
    RegWrite (FES1_LUT_PROGM, 0x4c54bead);
    RegWrite (FES1_LUT_PROGM, 0x50b50d0a);
    RegWrite (FES1_LUT_PROGM, 0x35297d85);
    RegWrite (FES1_LUT_PROGM, 0x4651bdae);
    RegWrite (FES1_LUT_PROGM, 0x70b68d0a);
    RegWrite (FES1_LUT_PROGM, 0x34e93de5);
    RegWrite (FES1_LUT_PROGM, 0x404ebbaf);
    RegWrite (FES1_LUT_PROGM, 0x88b78d0a);
    RegWrite (FES1_LUT_PROGM, 0x34693e25);
    RegWrite (FES1_LUT_PROGM, 0x3c4bbaaf);
    RegWrite (FES1_LUT_PROGM, 0xa8b90d02);
    RegWrite (FES1_LUT_PROGM, 0x34293e65);
    RegWrite (FES1_LUT_PROGM, 0x3448b9b0);
    RegWrite (FES1_LUT_PROGM, 0xc0ba0d02);
    RegWrite (FES1_LUT_PROGM, 0x33e97ec5);
    RegWrite (FES1_LUT_PROGM, 0x3045b8b1);
    RegWrite (FES1_LUT_PROGM, 0xe0bb0cfa);
    RegWrite (FES1_LUT_PROGM, 0x33a97f05);
    RegWrite (FES1_LUT_PROGM, 0x2a42b6b2);
    RegWrite (FES1_LUT_PROGM, 0xf8bc0cf2);
    RegWrite (FES1_LUT_PROGM, 0x33297f45);
    RegWrite (FES1_LUT_PROGM, 0x243fb5b2);
    RegWrite (FES1_LUT_PROGM, 0x18bd8cf2);
    RegWrite (FES1_LUT_PROGM, 0x32e97fa6);
    RegWrite (FES1_LUT_PROGM, 0x1e3cb4b3);
    RegWrite (FES1_LUT_PROGM, 0x30be8cea);
    RegWrite (FES1_LUT_PROGM, 0x32a97fe6);
    RegWrite (FES1_LUT_PROGM, 0x1839b3b4);
    RegWrite (FES1_LUT_PROGM, 0x50bf8ce2);
    RegWrite (FES1_LUT_PROGM, 0x32698046);
    RegWrite (FES1_LUT_PROGM, 0x1236b2b5);
    RegWrite (FES1_LUT_PROGM, 0x68c08cda);
    RegWrite (FES1_LUT_PROGM, 0x32298086);
    RegWrite (FES1_LUT_PROGM, 0xc33b1b6);
    RegWrite (FES1_LUT_PROGM, 0x88c18cda);
    RegWrite (FES1_LUT_PROGM, 0x31a980e6);
    RegWrite (FES1_LUT_PROGM, 0x430b0b7);
    RegWrite (FES1_LUT_PROGM, 0xa0c28cca);
    RegWrite (FES1_LUT_PROGM, 0x3169c126);
    RegWrite (FES1_LUT_PROGM, 0xfe2dafb7);
    RegWrite (FES1_LUT_PROGM, 0xb8c30cc1);
    RegWrite (FES1_LUT_PROGM, 0x3129c186);
    RegWrite (FES1_LUT_PROGM, 0xf82aaeb8);
    RegWrite (FES1_LUT_PROGM, 0xd8c40cb9);
    RegWrite (FES1_LUT_PROGM, 0x30ea01e6);
    RegWrite (FES1_LUT_PROGM, 0xf227adb9);
    RegWrite (FES1_LUT_PROGM, 0xf0c50cb1);
    RegWrite (FES1_LUT_PROGM, 0x30aa0226);
    RegWrite (FES1_LUT_PROGM, 0xec25adba);
    RegWrite (FES1_LUT_PROGM, 0x10c60ca9);
    RegWrite (FES1_LUT_PROGM, 0x306a4287);
    RegWrite (FES1_LUT_PROGM, 0xe422acbb);
    RegWrite (FES1_LUT_PROGM, 0x28c70c99);
    RegWrite (FES1_LUT_PROGM, 0x302a42e7);
    RegWrite (FES1_LUT_PROGM, 0xde1fabbc);
    RegWrite (FES1_LUT_PROGM, 0x40c78c91);
    RegWrite (FES1_LUT_PROGM, 0x2faa8327);
    RegWrite (FES1_LUT_PROGM, 0xd81caabd);
    RegWrite (FES1_LUT_PROGM, 0x60c80c81);
    RegWrite (FES1_LUT_PROGM, 0x2f6a8387);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x8c7dd3a8);
    RegWrite (FES1_LUT_PROGM, 0xe8a30d02);
    RegWrite (FES1_LUT_PROGM, 0x396a3a63);
    RegWrite (FES1_LUT_PROGM, 0x887ad1a8);
    RegWrite (FES1_LUT_PROGM, 0xa48d02);
    RegWrite (FES1_LUT_PROGM, 0x3929faa4);
    RegWrite (FES1_LUT_PROGM, 0x8477cfa8);
    RegWrite (FES1_LUT_PROGM, 0x20a60d0a);
    RegWrite (FES1_LUT_PROGM, 0x38a9fac4);
    RegWrite (FES1_LUT_PROGM, 0x7e73cea9);
    RegWrite (FES1_LUT_PROGM, 0x38a78d0a);
    RegWrite (FES1_LUT_PROGM, 0x3869fb04);
    RegWrite (FES1_LUT_PROGM, 0x7a70cca9);
    RegWrite (FES1_LUT_PROGM, 0x58a90d0a);
    RegWrite (FES1_LUT_PROGM, 0x3829bb44);
    RegWrite (FES1_LUT_PROGM, 0x766dcaa9);
    RegWrite (FES1_LUT_PROGM, 0x70aa8d12);
    RegWrite (FES1_LUT_PROGM, 0x37a9bb84);
    RegWrite (FES1_LUT_PROGM, 0x706ac9aa);
    RegWrite (FES1_LUT_PROGM, 0x90ab8d12);
    RegWrite (FES1_LUT_PROGM, 0x3769bbc4);
    RegWrite (FES1_LUT_PROGM, 0x6c67c7aa);
    RegWrite (FES1_LUT_PROGM, 0xa8ad0d12);
    RegWrite (FES1_LUT_PROGM, 0x37297c04);
    RegWrite (FES1_LUT_PROGM, 0x6664c5ab);
    RegWrite (FES1_LUT_PROGM, 0xc8ae8d12);
    RegWrite (FES1_LUT_PROGM, 0x36a97c44);
    RegWrite (FES1_LUT_PROGM, 0x6061c4ab);
    RegWrite (FES1_LUT_PROGM, 0xe0b00d12);
    RegWrite (FES1_LUT_PROGM, 0x36697c84);
    RegWrite (FES1_LUT_PROGM, 0x5c5ec2ac);
    RegWrite (FES1_LUT_PROGM, 0xb10d12);
    RegWrite (FES1_LUT_PROGM, 0x36297cc5);
    RegWrite (FES1_LUT_PROGM, 0x565bc1ac);
    RegWrite (FES1_LUT_PROGM, 0x18b28d12);
    RegWrite (FES1_LUT_PROGM, 0x35e97d05);
    RegWrite (FES1_LUT_PROGM, 0x5057bfad);
    RegWrite (FES1_LUT_PROGM, 0x38b40d12);
    RegWrite (FES1_LUT_PROGM, 0x35697d45);
    RegWrite (FES1_LUT_PROGM, 0x4c54bead);
    RegWrite (FES1_LUT_PROGM, 0x50b50d0a);
    RegWrite (FES1_LUT_PROGM, 0x35297d85);
    RegWrite (FES1_LUT_PROGM, 0x4651bdae);
    RegWrite (FES1_LUT_PROGM, 0x70b68d0a);
    RegWrite (FES1_LUT_PROGM, 0x34e93de5);
    RegWrite (FES1_LUT_PROGM, 0x404ebbaf);
    RegWrite (FES1_LUT_PROGM, 0x88b78d0a);
    RegWrite (FES1_LUT_PROGM, 0x34693e25);
    RegWrite (FES1_LUT_PROGM, 0x3c4bbaaf);
    RegWrite (FES1_LUT_PROGM, 0xa8b90d02);
    RegWrite (FES1_LUT_PROGM, 0x34293e65);
    RegWrite (FES1_LUT_PROGM, 0x3448b9b0);
    RegWrite (FES1_LUT_PROGM, 0xc0ba0d02);
    RegWrite (FES1_LUT_PROGM, 0x33e97ec5);
    RegWrite (FES1_LUT_PROGM, 0x3045b8b1);
    RegWrite (FES1_LUT_PROGM, 0xe0bb0cfa);
    RegWrite (FES1_LUT_PROGM, 0x33a97f05);
    RegWrite (FES1_LUT_PROGM, 0x2a42b6b2);
    RegWrite (FES1_LUT_PROGM, 0xf8bc0cf2);
    RegWrite (FES1_LUT_PROGM, 0x33297f45);
    RegWrite (FES1_LUT_PROGM, 0x243fb5b2);
    RegWrite (FES1_LUT_PROGM, 0x18bd8cf2);
    RegWrite (FES1_LUT_PROGM, 0x32e97fa6);
    RegWrite (FES1_LUT_PROGM, 0x1e3cb4b3);
    RegWrite (FES1_LUT_PROGM, 0x30be8cea);
    RegWrite (FES1_LUT_PROGM, 0x32a97fe6);
    RegWrite (FES1_LUT_PROGM, 0x1839b3b4);
    RegWrite (FES1_LUT_PROGM, 0x50bf8ce2);
    RegWrite (FES1_LUT_PROGM, 0x32698046);
    RegWrite (FES1_LUT_PROGM, 0x1236b2b5);
    RegWrite (FES1_LUT_PROGM, 0x68c08cda);
    RegWrite (FES1_LUT_PROGM, 0x32298086);
    RegWrite (FES1_LUT_PROGM, 0xc33b1b6);
    RegWrite (FES1_LUT_PROGM, 0x88c18cda);
    RegWrite (FES1_LUT_PROGM, 0x31a980e6);
    RegWrite (FES1_LUT_PROGM, 0x430b0b7);
    RegWrite (FES1_LUT_PROGM, 0xa0c28cca);
    RegWrite (FES1_LUT_PROGM, 0x3169c126);
    RegWrite (FES1_LUT_PROGM, 0xfe2dafb7);
    RegWrite (FES1_LUT_PROGM, 0xb8c30cc1);
    RegWrite (FES1_LUT_PROGM, 0x3129c186);
    RegWrite (FES1_LUT_PROGM, 0xf82aaeb8);
    RegWrite (FES1_LUT_PROGM, 0xd8c40cb9);
    RegWrite (FES1_LUT_PROGM, 0x30ea01e6);
    RegWrite (FES1_LUT_PROGM, 0xf227adb9);
    RegWrite (FES1_LUT_PROGM, 0xf0c50cb1);
    RegWrite (FES1_LUT_PROGM, 0x30aa0226);
    RegWrite (FES1_LUT_PROGM, 0xec25adba);
    RegWrite (FES1_LUT_PROGM, 0x10c60ca9);
    RegWrite (FES1_LUT_PROGM, 0x306a4287);
    RegWrite (FES1_LUT_PROGM, 0xe422acbb);
    RegWrite (FES1_LUT_PROGM, 0x28c70c99);
    RegWrite (FES1_LUT_PROGM, 0x302a42e7);
    RegWrite (FES1_LUT_PROGM, 0xde1fabbc);
    RegWrite (FES1_LUT_PROGM, 0x40c78c91);
    RegWrite (FES1_LUT_PROGM, 0x2faa8327);
    RegWrite (FES1_LUT_PROGM, 0xd81caabd);
    RegWrite (FES1_LUT_PROGM, 0x60c80c81);
    RegWrite (FES1_LUT_PROGM, 0x2f6a8387);
}

void Set_Fes1_1080i_to_576i(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00078000);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00078000);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x000aaaab);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x000aaaab);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x911a8bde);
    RegWrite (FES1_LUT_PROGM, 0x333ef0a2);
    RegWrite (FES1_LUT_PROGM, 0x911a7fda);
    RegWrite (FES1_LUT_PROGM, 0x32ff10a5);
    RegWrite (FES1_LUT_PROGM, 0x912273d6);
    RegWrite (FES1_LUT_PROGM, 0x32df30a8);
    RegWrite (FES1_LUT_PROGM, 0x912269d3);
    RegWrite (FES1_LUT_PROGM, 0x329f50ab);
    RegWrite (FES1_LUT_PROGM, 0x112a5dcf);
    RegWrite (FES1_LUT_PROGM, 0x327f70af);
    RegWrite (FES1_LUT_PROGM, 0x112a51cb);
    RegWrite (FES1_LUT_PROGM, 0x325f98b2);
    RegWrite (FES1_LUT_PROGM, 0x112a45c8);
    RegWrite (FES1_LUT_PROGM, 0x321fb8b5);
    RegWrite (FES1_LUT_PROGM, 0x112a3bc4);
    RegWrite (FES1_LUT_PROGM, 0x31ffd8b8);
    RegWrite (FES1_LUT_PROGM, 0x11222dc1);
    RegWrite (FES1_LUT_PROGM, 0x31c000bb);
    RegWrite (FES1_LUT_PROGM, 0x111a21be);
    RegWrite (FES1_LUT_PROGM, 0x31a028be);
    RegWrite (FES1_LUT_PROGM, 0x912217ba);
    RegWrite (FES1_LUT_PROGM, 0x318048c1);
    RegWrite (FES1_LUT_PROGM, 0x111209b7);
    RegWrite (FES1_LUT_PROGM, 0x316070c4);
    RegWrite (FES1_LUT_PROGM, 0x1109ffb4);
    RegWrite (FES1_LUT_PROGM, 0x314098c7);
    RegWrite (FES1_LUT_PROGM, 0x1101f3b1);
    RegWrite (FES1_LUT_PROGM, 0x3140c0ca);
    RegWrite (FES1_LUT_PROGM, 0x10f9e5af);
    RegWrite (FES1_LUT_PROGM, 0x3120e8cd);
    RegWrite (FES1_LUT_PROGM, 0x10f1d9ac);
    RegWrite (FES1_LUT_PROGM, 0x310110d0);
    RegWrite (FES1_LUT_PROGM, 0x90e1cda9);
    RegWrite (FES1_LUT_PROGM, 0x310138d2);
    RegWrite (FES1_LUT_PROGM, 0x90d9c1a7);
    RegWrite (FES1_LUT_PROGM, 0x30e168d5);
    RegWrite (FES1_LUT_PROGM, 0x10c9b5a4);
    RegWrite (FES1_LUT_PROGM, 0x30e190d8);
    RegWrite (FES1_LUT_PROGM, 0x90b9a9a2);
    RegWrite (FES1_LUT_PROGM, 0x30e1c0da);
    RegWrite (FES1_LUT_PROGM, 0x10a99ba0);
    RegWrite (FES1_LUT_PROGM, 0x30e1e8dd);
    RegWrite (FES1_LUT_PROGM, 0x10998f9d);
    RegWrite (FES1_LUT_PROGM, 0x30e218e0);
    RegWrite (FES1_LUT_PROGM, 0x9081839b);
    RegWrite (FES1_LUT_PROGM, 0x30e240e2);
    RegWrite (FES1_LUT_PROGM, 0x10717799);
    RegWrite (FES1_LUT_PROGM, 0x30e270e5);
    RegWrite (FES1_LUT_PROGM, 0x10596998);
    RegWrite (FES1_LUT_PROGM, 0x30e298e7);
    RegWrite (FES1_LUT_PROGM, 0x90415d96);
    RegWrite (FES1_LUT_PROGM, 0x30e2c8e9);
    RegWrite (FES1_LUT_PROGM, 0x10295194);
    RegWrite (FES1_LUT_PROGM, 0x3102f8ec);
    RegWrite (FES1_LUT_PROGM, 0x10114593);
    RegWrite (FES1_LUT_PROGM, 0x310328ee);
    RegWrite (FES1_LUT_PROGM, 0x8ff93991);
    RegWrite (FES1_LUT_PROGM, 0x312358f0);
    RegWrite (FES1_LUT_PROGM, 0x8fe12b90);
    RegWrite (FES1_LUT_PROGM, 0x314388f2);
    RegWrite (FES1_LUT_PROGM, 0x8fc11f8e);
    RegWrite (FES1_LUT_PROGM, 0x3143b8f4);
    RegWrite (FES1_LUT_PROGM, 0x8fa9138d);
    RegWrite (FES1_LUT_PROGM, 0x3163e8f6);
    RegWrite (FES1_LUT_PROGM, 0x8f89078c);
    RegWrite (FES1_LUT_PROGM, 0x318418f8);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x911a8bde);
    RegWrite (FES1_LUT_PROGM, 0x333ef0a2);
    RegWrite (FES1_LUT_PROGM, 0x911a7fda);
    RegWrite (FES1_LUT_PROGM, 0x32ff10a5);
    RegWrite (FES1_LUT_PROGM, 0x912273d6);
    RegWrite (FES1_LUT_PROGM, 0x32df30a8);
    RegWrite (FES1_LUT_PROGM, 0x912269d3);
    RegWrite (FES1_LUT_PROGM, 0x329f50ab);
    RegWrite (FES1_LUT_PROGM, 0x112a5dcf);
    RegWrite (FES1_LUT_PROGM, 0x327f70af);
    RegWrite (FES1_LUT_PROGM, 0x112a51cb);
    RegWrite (FES1_LUT_PROGM, 0x325f98b2);
    RegWrite (FES1_LUT_PROGM, 0x112a45c8);
    RegWrite (FES1_LUT_PROGM, 0x321fb8b5);
    RegWrite (FES1_LUT_PROGM, 0x112a3bc4);
    RegWrite (FES1_LUT_PROGM, 0x31ffd8b8);
    RegWrite (FES1_LUT_PROGM, 0x11222dc1);
    RegWrite (FES1_LUT_PROGM, 0x31c000bb);
    RegWrite (FES1_LUT_PROGM, 0x111a21be);
    RegWrite (FES1_LUT_PROGM, 0x31a028be);
    RegWrite (FES1_LUT_PROGM, 0x912217ba);
    RegWrite (FES1_LUT_PROGM, 0x318048c1);
    RegWrite (FES1_LUT_PROGM, 0x111209b7);
    RegWrite (FES1_LUT_PROGM, 0x316070c4);
    RegWrite (FES1_LUT_PROGM, 0x1109ffb4);
    RegWrite (FES1_LUT_PROGM, 0x314098c7);
    RegWrite (FES1_LUT_PROGM, 0x1101f3b1);
    RegWrite (FES1_LUT_PROGM, 0x3140c0ca);
    RegWrite (FES1_LUT_PROGM, 0x10f9e5af);
    RegWrite (FES1_LUT_PROGM, 0x3120e8cd);
    RegWrite (FES1_LUT_PROGM, 0x10f1d9ac);
    RegWrite (FES1_LUT_PROGM, 0x310110d0);
    RegWrite (FES1_LUT_PROGM, 0x90e1cda9);
    RegWrite (FES1_LUT_PROGM, 0x310138d2);
    RegWrite (FES1_LUT_PROGM, 0x90d9c1a7);
    RegWrite (FES1_LUT_PROGM, 0x30e168d5);
    RegWrite (FES1_LUT_PROGM, 0x10c9b5a4);
    RegWrite (FES1_LUT_PROGM, 0x30e190d8);
    RegWrite (FES1_LUT_PROGM, 0x90b9a9a2);
    RegWrite (FES1_LUT_PROGM, 0x30e1c0da);
    RegWrite (FES1_LUT_PROGM, 0x10a99ba0);
    RegWrite (FES1_LUT_PROGM, 0x30e1e8dd);
    RegWrite (FES1_LUT_PROGM, 0x10998f9d);
    RegWrite (FES1_LUT_PROGM, 0x30e218e0);
    RegWrite (FES1_LUT_PROGM, 0x9081839b);
    RegWrite (FES1_LUT_PROGM, 0x30e240e2);
    RegWrite (FES1_LUT_PROGM, 0x10717799);
    RegWrite (FES1_LUT_PROGM, 0x30e270e5);
    RegWrite (FES1_LUT_PROGM, 0x10596998);
    RegWrite (FES1_LUT_PROGM, 0x30e298e7);
    RegWrite (FES1_LUT_PROGM, 0x90415d96);
    RegWrite (FES1_LUT_PROGM, 0x30e2c8e9);
    RegWrite (FES1_LUT_PROGM, 0x10295194);
    RegWrite (FES1_LUT_PROGM, 0x3102f8ec);
    RegWrite (FES1_LUT_PROGM, 0x10114593);
    RegWrite (FES1_LUT_PROGM, 0x310328ee);
    RegWrite (FES1_LUT_PROGM, 0x8ff93991);
    RegWrite (FES1_LUT_PROGM, 0x312358f0);
    RegWrite (FES1_LUT_PROGM, 0x8fe12b90);
    RegWrite (FES1_LUT_PROGM, 0x314388f2);
    RegWrite (FES1_LUT_PROGM, 0x8fc11f8e);
    RegWrite (FES1_LUT_PROGM, 0x3143b8f4);
    RegWrite (FES1_LUT_PROGM, 0x8fa9138d);
    RegWrite (FES1_LUT_PROGM, 0x3163e8f6);
    RegWrite (FES1_LUT_PROGM, 0x8f89078c);
    RegWrite (FES1_LUT_PROGM, 0x318418f8);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x8c7dd3a8);
    RegWrite (FES1_LUT_PROGM, 0xe8a30d02);
    RegWrite (FES1_LUT_PROGM, 0x396a3a63);
    RegWrite (FES1_LUT_PROGM, 0x887ad1a8);
    RegWrite (FES1_LUT_PROGM, 0xa48d02);
    RegWrite (FES1_LUT_PROGM, 0x3929faa4);
    RegWrite (FES1_LUT_PROGM, 0x8477cfa8);
    RegWrite (FES1_LUT_PROGM, 0x20a60d0a);
    RegWrite (FES1_LUT_PROGM, 0x38a9fac4);
    RegWrite (FES1_LUT_PROGM, 0x7e73cea9);
    RegWrite (FES1_LUT_PROGM, 0x38a78d0a);
    RegWrite (FES1_LUT_PROGM, 0x3869fb04);
    RegWrite (FES1_LUT_PROGM, 0x7a70cca9);
    RegWrite (FES1_LUT_PROGM, 0x58a90d0a);
    RegWrite (FES1_LUT_PROGM, 0x3829bb44);
    RegWrite (FES1_LUT_PROGM, 0x766dcaa9);
    RegWrite (FES1_LUT_PROGM, 0x70aa8d12);
    RegWrite (FES1_LUT_PROGM, 0x37a9bb84);
    RegWrite (FES1_LUT_PROGM, 0x706ac9aa);
    RegWrite (FES1_LUT_PROGM, 0x90ab8d12);
    RegWrite (FES1_LUT_PROGM, 0x3769bbc4);
    RegWrite (FES1_LUT_PROGM, 0x6c67c7aa);
    RegWrite (FES1_LUT_PROGM, 0xa8ad0d12);
    RegWrite (FES1_LUT_PROGM, 0x37297c04);
    RegWrite (FES1_LUT_PROGM, 0x6664c5ab);
    RegWrite (FES1_LUT_PROGM, 0xc8ae8d12);
    RegWrite (FES1_LUT_PROGM, 0x36a97c44);
    RegWrite (FES1_LUT_PROGM, 0x6061c4ab);
    RegWrite (FES1_LUT_PROGM, 0xe0b00d12);
    RegWrite (FES1_LUT_PROGM, 0x36697c84);
    RegWrite (FES1_LUT_PROGM, 0x5c5ec2ac);
    RegWrite (FES1_LUT_PROGM, 0xb10d12);
    RegWrite (FES1_LUT_PROGM, 0x36297cc5);
    RegWrite (FES1_LUT_PROGM, 0x565bc1ac);
    RegWrite (FES1_LUT_PROGM, 0x18b28d12);
    RegWrite (FES1_LUT_PROGM, 0x35e97d05);
    RegWrite (FES1_LUT_PROGM, 0x5057bfad);
    RegWrite (FES1_LUT_PROGM, 0x38b40d12);
    RegWrite (FES1_LUT_PROGM, 0x35697d45);
    RegWrite (FES1_LUT_PROGM, 0x4c54bead);
    RegWrite (FES1_LUT_PROGM, 0x50b50d0a);
    RegWrite (FES1_LUT_PROGM, 0x35297d85);
    RegWrite (FES1_LUT_PROGM, 0x4651bdae);
    RegWrite (FES1_LUT_PROGM, 0x70b68d0a);
    RegWrite (FES1_LUT_PROGM, 0x34e93de5);
    RegWrite (FES1_LUT_PROGM, 0x404ebbaf);
    RegWrite (FES1_LUT_PROGM, 0x88b78d0a);
    RegWrite (FES1_LUT_PROGM, 0x34693e25);
    RegWrite (FES1_LUT_PROGM, 0x3c4bbaaf);
    RegWrite (FES1_LUT_PROGM, 0xa8b90d02);
    RegWrite (FES1_LUT_PROGM, 0x34293e65);
    RegWrite (FES1_LUT_PROGM, 0x3448b9b0);
    RegWrite (FES1_LUT_PROGM, 0xc0ba0d02);
    RegWrite (FES1_LUT_PROGM, 0x33e97ec5);
    RegWrite (FES1_LUT_PROGM, 0x3045b8b1);
    RegWrite (FES1_LUT_PROGM, 0xe0bb0cfa);
    RegWrite (FES1_LUT_PROGM, 0x33a97f05);
    RegWrite (FES1_LUT_PROGM, 0x2a42b6b2);
    RegWrite (FES1_LUT_PROGM, 0xf8bc0cf2);
    RegWrite (FES1_LUT_PROGM, 0x33297f45);
    RegWrite (FES1_LUT_PROGM, 0x243fb5b2);
    RegWrite (FES1_LUT_PROGM, 0x18bd8cf2);
    RegWrite (FES1_LUT_PROGM, 0x32e97fa6);
    RegWrite (FES1_LUT_PROGM, 0x1e3cb4b3);
    RegWrite (FES1_LUT_PROGM, 0x30be8cea);
    RegWrite (FES1_LUT_PROGM, 0x32a97fe6);
    RegWrite (FES1_LUT_PROGM, 0x1839b3b4);
    RegWrite (FES1_LUT_PROGM, 0x50bf8ce2);
    RegWrite (FES1_LUT_PROGM, 0x32698046);
    RegWrite (FES1_LUT_PROGM, 0x1236b2b5);
    RegWrite (FES1_LUT_PROGM, 0x68c08cda);
    RegWrite (FES1_LUT_PROGM, 0x32298086);
    RegWrite (FES1_LUT_PROGM, 0xc33b1b6);
    RegWrite (FES1_LUT_PROGM, 0x88c18cda);
    RegWrite (FES1_LUT_PROGM, 0x31a980e6);
    RegWrite (FES1_LUT_PROGM, 0x430b0b7);
    RegWrite (FES1_LUT_PROGM, 0xa0c28cca);
    RegWrite (FES1_LUT_PROGM, 0x3169c126);
    RegWrite (FES1_LUT_PROGM, 0xfe2dafb7);
    RegWrite (FES1_LUT_PROGM, 0xb8c30cc1);
    RegWrite (FES1_LUT_PROGM, 0x3129c186);
    RegWrite (FES1_LUT_PROGM, 0xf82aaeb8);
    RegWrite (FES1_LUT_PROGM, 0xd8c40cb9);
    RegWrite (FES1_LUT_PROGM, 0x30ea01e6);
    RegWrite (FES1_LUT_PROGM, 0xf227adb9);
    RegWrite (FES1_LUT_PROGM, 0xf0c50cb1);
    RegWrite (FES1_LUT_PROGM, 0x30aa0226);
    RegWrite (FES1_LUT_PROGM, 0xec25adba);
    RegWrite (FES1_LUT_PROGM, 0x10c60ca9);
    RegWrite (FES1_LUT_PROGM, 0x306a4287);
    RegWrite (FES1_LUT_PROGM, 0xe422acbb);
    RegWrite (FES1_LUT_PROGM, 0x28c70c99);
    RegWrite (FES1_LUT_PROGM, 0x302a42e7);
    RegWrite (FES1_LUT_PROGM, 0xde1fabbc);
    RegWrite (FES1_LUT_PROGM, 0x40c78c91);
    RegWrite (FES1_LUT_PROGM, 0x2faa8327);
    RegWrite (FES1_LUT_PROGM, 0xd81caabd);
    RegWrite (FES1_LUT_PROGM, 0x60c80c81);
    RegWrite (FES1_LUT_PROGM, 0x2f6a8387);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x8c7dd3a8);
    RegWrite (FES1_LUT_PROGM, 0xe8a30d02);
    RegWrite (FES1_LUT_PROGM, 0x396a3a63);
    RegWrite (FES1_LUT_PROGM, 0x887ad1a8);
    RegWrite (FES1_LUT_PROGM, 0xa48d02);
    RegWrite (FES1_LUT_PROGM, 0x3929faa4);
    RegWrite (FES1_LUT_PROGM, 0x8477cfa8);
    RegWrite (FES1_LUT_PROGM, 0x20a60d0a);
    RegWrite (FES1_LUT_PROGM, 0x38a9fac4);
    RegWrite (FES1_LUT_PROGM, 0x7e73cea9);
    RegWrite (FES1_LUT_PROGM, 0x38a78d0a);
    RegWrite (FES1_LUT_PROGM, 0x3869fb04);
    RegWrite (FES1_LUT_PROGM, 0x7a70cca9);
    RegWrite (FES1_LUT_PROGM, 0x58a90d0a);
    RegWrite (FES1_LUT_PROGM, 0x3829bb44);
    RegWrite (FES1_LUT_PROGM, 0x766dcaa9);
    RegWrite (FES1_LUT_PROGM, 0x70aa8d12);
    RegWrite (FES1_LUT_PROGM, 0x37a9bb84);
    RegWrite (FES1_LUT_PROGM, 0x706ac9aa);
    RegWrite (FES1_LUT_PROGM, 0x90ab8d12);
    RegWrite (FES1_LUT_PROGM, 0x3769bbc4);
    RegWrite (FES1_LUT_PROGM, 0x6c67c7aa);
    RegWrite (FES1_LUT_PROGM, 0xa8ad0d12);
    RegWrite (FES1_LUT_PROGM, 0x37297c04);
    RegWrite (FES1_LUT_PROGM, 0x6664c5ab);
    RegWrite (FES1_LUT_PROGM, 0xc8ae8d12);
    RegWrite (FES1_LUT_PROGM, 0x36a97c44);
    RegWrite (FES1_LUT_PROGM, 0x6061c4ab);
    RegWrite (FES1_LUT_PROGM, 0xe0b00d12);
    RegWrite (FES1_LUT_PROGM, 0x36697c84);
    RegWrite (FES1_LUT_PROGM, 0x5c5ec2ac);
    RegWrite (FES1_LUT_PROGM, 0xb10d12);
    RegWrite (FES1_LUT_PROGM, 0x36297cc5);
    RegWrite (FES1_LUT_PROGM, 0x565bc1ac);
    RegWrite (FES1_LUT_PROGM, 0x18b28d12);
    RegWrite (FES1_LUT_PROGM, 0x35e97d05);
    RegWrite (FES1_LUT_PROGM, 0x5057bfad);
    RegWrite (FES1_LUT_PROGM, 0x38b40d12);
    RegWrite (FES1_LUT_PROGM, 0x35697d45);
    RegWrite (FES1_LUT_PROGM, 0x4c54bead);
    RegWrite (FES1_LUT_PROGM, 0x50b50d0a);
    RegWrite (FES1_LUT_PROGM, 0x35297d85);
    RegWrite (FES1_LUT_PROGM, 0x4651bdae);
    RegWrite (FES1_LUT_PROGM, 0x70b68d0a);
    RegWrite (FES1_LUT_PROGM, 0x34e93de5);
    RegWrite (FES1_LUT_PROGM, 0x404ebbaf);
    RegWrite (FES1_LUT_PROGM, 0x88b78d0a);
    RegWrite (FES1_LUT_PROGM, 0x34693e25);
    RegWrite (FES1_LUT_PROGM, 0x3c4bbaaf);
    RegWrite (FES1_LUT_PROGM, 0xa8b90d02);
    RegWrite (FES1_LUT_PROGM, 0x34293e65);
    RegWrite (FES1_LUT_PROGM, 0x3448b9b0);
    RegWrite (FES1_LUT_PROGM, 0xc0ba0d02);
    RegWrite (FES1_LUT_PROGM, 0x33e97ec5);
    RegWrite (FES1_LUT_PROGM, 0x3045b8b1);
    RegWrite (FES1_LUT_PROGM, 0xe0bb0cfa);
    RegWrite (FES1_LUT_PROGM, 0x33a97f05);
    RegWrite (FES1_LUT_PROGM, 0x2a42b6b2);
    RegWrite (FES1_LUT_PROGM, 0xf8bc0cf2);
    RegWrite (FES1_LUT_PROGM, 0x33297f45);
    RegWrite (FES1_LUT_PROGM, 0x243fb5b2);
    RegWrite (FES1_LUT_PROGM, 0x18bd8cf2);
    RegWrite (FES1_LUT_PROGM, 0x32e97fa6);
    RegWrite (FES1_LUT_PROGM, 0x1e3cb4b3);
    RegWrite (FES1_LUT_PROGM, 0x30be8cea);
    RegWrite (FES1_LUT_PROGM, 0x32a97fe6);
    RegWrite (FES1_LUT_PROGM, 0x1839b3b4);
    RegWrite (FES1_LUT_PROGM, 0x50bf8ce2);
    RegWrite (FES1_LUT_PROGM, 0x32698046);
    RegWrite (FES1_LUT_PROGM, 0x1236b2b5);
    RegWrite (FES1_LUT_PROGM, 0x68c08cda);
    RegWrite (FES1_LUT_PROGM, 0x32298086);
    RegWrite (FES1_LUT_PROGM, 0xc33b1b6);
    RegWrite (FES1_LUT_PROGM, 0x88c18cda);
    RegWrite (FES1_LUT_PROGM, 0x31a980e6);
    RegWrite (FES1_LUT_PROGM, 0x430b0b7);
    RegWrite (FES1_LUT_PROGM, 0xa0c28cca);
    RegWrite (FES1_LUT_PROGM, 0x3169c126);
    RegWrite (FES1_LUT_PROGM, 0xfe2dafb7);
    RegWrite (FES1_LUT_PROGM, 0xb8c30cc1);
    RegWrite (FES1_LUT_PROGM, 0x3129c186);
    RegWrite (FES1_LUT_PROGM, 0xf82aaeb8);
    RegWrite (FES1_LUT_PROGM, 0xd8c40cb9);
    RegWrite (FES1_LUT_PROGM, 0x30ea01e6);
    RegWrite (FES1_LUT_PROGM, 0xf227adb9);
    RegWrite (FES1_LUT_PROGM, 0xf0c50cb1);
    RegWrite (FES1_LUT_PROGM, 0x30aa0226);
    RegWrite (FES1_LUT_PROGM, 0xec25adba);
    RegWrite (FES1_LUT_PROGM, 0x10c60ca9);
    RegWrite (FES1_LUT_PROGM, 0x306a4287);
    RegWrite (FES1_LUT_PROGM, 0xe422acbb);
    RegWrite (FES1_LUT_PROGM, 0x28c70c99);
    RegWrite (FES1_LUT_PROGM, 0x302a42e7);
    RegWrite (FES1_LUT_PROGM, 0xde1fabbc);
    RegWrite (FES1_LUT_PROGM, 0x40c78c91);
    RegWrite (FES1_LUT_PROGM, 0x2faa8327);
    RegWrite (FES1_LUT_PROGM, 0xd81caabd);
    RegWrite (FES1_LUT_PROGM, 0x60c80c81);
    RegWrite (FES1_LUT_PROGM, 0x2f6a8387);
}

void Set_Fes1_576i_to_480i(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x0004cccd);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x0004cccd);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x1b214d70);
    RegWrite (FES1_LUT_PROGM, 0xdfb8053);
    RegWrite (FES1_LUT_PROGM, 0x1b213175);
    RegWrite (FES1_LUT_PROGM, 0xdfb605a);
    RegWrite (FES1_LUT_PROGM, 0x9b19137a);
    RegWrite (FES1_LUT_PROGM, 0xdfb4061);
    RegWrite (FES1_LUT_PROGM, 0x1b10f97f);
    RegWrite (FES1_LUT_PROGM, 0xdfb2069);
    RegWrite (FES1_LUT_PROGM, 0x1af8dd84);
    RegWrite (FES1_LUT_PROGM, 0xdfb0070);
    RegWrite (FES1_LUT_PROGM, 0x9ae0c189);
    RegWrite (FES1_LUT_PROGM, 0xdfae877);
    RegWrite (FES1_LUT_PROGM, 0x1ac8a78f);
    RegWrite (FES1_LUT_PROGM, 0xddac87f);
    RegWrite (FES1_LUT_PROGM, 0x9aa88d94);
    RegWrite (FES1_LUT_PROGM, 0xddab086);
    RegWrite (FES1_LUT_PROGM, 0x1a88739a);
    RegWrite (FES1_LUT_PROGM, 0xdba988e);
    RegWrite (FES1_LUT_PROGM, 0x9a685b9f);
    RegWrite (FES1_LUT_PROGM, 0xd9a8895);
    RegWrite (FES1_LUT_PROGM, 0x1a4043a5);
    RegWrite (FES1_LUT_PROGM, 0xd5a709d);
    RegWrite (FES1_LUT_PROGM, 0x9a082bab);
    RegWrite (FES1_LUT_PROGM, 0xd3a60a4);
    RegWrite (FES1_LUT_PROGM, 0x19e013b0);
    RegWrite (FES1_LUT_PROGM, 0xcfa50ac);
    RegWrite (FES1_LUT_PROGM, 0x19bffdb6);
    RegWrite (FES1_LUT_PROGM, 0xcda48b4);
    RegWrite (FES1_LUT_PROGM, 0x9987e7bc);
    RegWrite (FES1_LUT_PROGM, 0xc9a38bb);
    RegWrite (FES1_LUT_PROGM, 0x1947d1c2);
    RegWrite (FES1_LUT_PROGM, 0xc3a38c3);
    RegWrite (FES1_LUT_PROGM, 0x990fbdc8);
    RegWrite (FES1_LUT_PROGM, 0xbfa30ca);
    RegWrite (FES1_LUT_PROGM, 0x18cfa9ce);
    RegWrite (FES1_LUT_PROGM, 0xb9a28d2);
    RegWrite (FES1_LUT_PROGM, 0x987f95d4);
    RegWrite (FES1_LUT_PROGM, 0xb3a28d9);
    RegWrite (FES1_LUT_PROGM, 0x184781da);
    RegWrite (FES1_LUT_PROGM, 0xafa28e1);
    RegWrite (FES1_LUT_PROGM, 0x97ff6fe0);
    RegWrite (FES1_LUT_PROGM, 0xa9a30e8);
    RegWrite (FES1_LUT_PROGM, 0x97a75fe6);
    RegWrite (FES1_LUT_PROGM, 0xa1a38ef);
    RegWrite (FES1_LUT_PROGM, 0x175f4deb);
    RegWrite (FES1_LUT_PROGM, 0x9ba40f7);
    RegWrite (FES1_LUT_PROGM, 0x17173df1);
    RegWrite (FES1_LUT_PROGM, 0x93a48ff);
    RegWrite (FES1_LUT_PROGM, 0x16bf2df7);
    RegWrite (FES1_LUT_PROGM, 0x8ba5906);
    RegWrite (FES1_LUT_PROGM, 0x96671ffd);
    RegWrite (FES1_LUT_PROGM, 0x83a690c);
    RegWrite (FES1_LUT_PROGM, 0x960f1002);
    RegWrite (FES1_LUT_PROGM, 0x7ba8113);
    RegWrite (FES1_LUT_PROGM, 0x15b70408);
    RegWrite (FES1_LUT_PROGM, 0x73a911b);
    RegWrite (FES1_LUT_PROGM, 0x954ef60d);
    RegWrite (FES1_LUT_PROGM, 0x6bab121);
    RegWrite (FES1_LUT_PROGM, 0x94f6ea13);
    RegWrite (FES1_LUT_PROGM, 0x61ac928);
    RegWrite (FES1_LUT_PROGM, 0x9496e018);
    RegWrite (FES1_LUT_PROGM, 0x59ae92f);
    RegWrite (FES1_LUT_PROGM, 0x1436d41d);
    RegWrite (FES1_LUT_PROGM, 0x4fb0936);
    RegWrite (FES1_LUT_PROGM, 0x13c6cc22);
    RegWrite (FES1_LUT_PROGM, 0x45b313c);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x1b214d70);
    RegWrite (FES1_LUT_PROGM, 0xdfb8053);
    RegWrite (FES1_LUT_PROGM, 0x1b213175);
    RegWrite (FES1_LUT_PROGM, 0xdfb605a);
    RegWrite (FES1_LUT_PROGM, 0x9b19137a);
    RegWrite (FES1_LUT_PROGM, 0xdfb4061);
    RegWrite (FES1_LUT_PROGM, 0x1b10f97f);
    RegWrite (FES1_LUT_PROGM, 0xdfb2069);
    RegWrite (FES1_LUT_PROGM, 0x1af8dd84);
    RegWrite (FES1_LUT_PROGM, 0xdfb0070);
    RegWrite (FES1_LUT_PROGM, 0x9ae0c189);
    RegWrite (FES1_LUT_PROGM, 0xdfae877);
    RegWrite (FES1_LUT_PROGM, 0x1ac8a78f);
    RegWrite (FES1_LUT_PROGM, 0xddac87f);
    RegWrite (FES1_LUT_PROGM, 0x9aa88d94);
    RegWrite (FES1_LUT_PROGM, 0xddab086);
    RegWrite (FES1_LUT_PROGM, 0x1a88739a);
    RegWrite (FES1_LUT_PROGM, 0xdba988e);
    RegWrite (FES1_LUT_PROGM, 0x9a685b9f);
    RegWrite (FES1_LUT_PROGM, 0xd9a8895);
    RegWrite (FES1_LUT_PROGM, 0x1a4043a5);
    RegWrite (FES1_LUT_PROGM, 0xd5a709d);
    RegWrite (FES1_LUT_PROGM, 0x9a082bab);
    RegWrite (FES1_LUT_PROGM, 0xd3a60a4);
    RegWrite (FES1_LUT_PROGM, 0x19e013b0);
    RegWrite (FES1_LUT_PROGM, 0xcfa50ac);
    RegWrite (FES1_LUT_PROGM, 0x19bffdb6);
    RegWrite (FES1_LUT_PROGM, 0xcda48b4);
    RegWrite (FES1_LUT_PROGM, 0x9987e7bc);
    RegWrite (FES1_LUT_PROGM, 0xc9a38bb);
    RegWrite (FES1_LUT_PROGM, 0x1947d1c2);
    RegWrite (FES1_LUT_PROGM, 0xc3a38c3);
    RegWrite (FES1_LUT_PROGM, 0x990fbdc8);
    RegWrite (FES1_LUT_PROGM, 0xbfa30ca);
    RegWrite (FES1_LUT_PROGM, 0x18cfa9ce);
    RegWrite (FES1_LUT_PROGM, 0xb9a28d2);
    RegWrite (FES1_LUT_PROGM, 0x987f95d4);
    RegWrite (FES1_LUT_PROGM, 0xb3a28d9);
    RegWrite (FES1_LUT_PROGM, 0x184781da);
    RegWrite (FES1_LUT_PROGM, 0xafa28e1);
    RegWrite (FES1_LUT_PROGM, 0x97ff6fe0);
    RegWrite (FES1_LUT_PROGM, 0xa9a30e8);
    RegWrite (FES1_LUT_PROGM, 0x97a75fe6);
    RegWrite (FES1_LUT_PROGM, 0xa1a38ef);
    RegWrite (FES1_LUT_PROGM, 0x175f4deb);
    RegWrite (FES1_LUT_PROGM, 0x9ba40f7);
    RegWrite (FES1_LUT_PROGM, 0x17173df1);
    RegWrite (FES1_LUT_PROGM, 0x93a48ff);
    RegWrite (FES1_LUT_PROGM, 0x16bf2df7);
    RegWrite (FES1_LUT_PROGM, 0x8ba5906);
    RegWrite (FES1_LUT_PROGM, 0x96671ffd);
    RegWrite (FES1_LUT_PROGM, 0x83a690c);
    RegWrite (FES1_LUT_PROGM, 0x960f1002);
    RegWrite (FES1_LUT_PROGM, 0x7ba8113);
    RegWrite (FES1_LUT_PROGM, 0x15b70408);
    RegWrite (FES1_LUT_PROGM, 0x73a911b);
    RegWrite (FES1_LUT_PROGM, 0x954ef60d);
    RegWrite (FES1_LUT_PROGM, 0x6bab121);
    RegWrite (FES1_LUT_PROGM, 0x94f6ea13);
    RegWrite (FES1_LUT_PROGM, 0x61ac928);
    RegWrite (FES1_LUT_PROGM, 0x9496e018);
    RegWrite (FES1_LUT_PROGM, 0x59ae92f);
    RegWrite (FES1_LUT_PROGM, 0x1436d41d);
    RegWrite (FES1_LUT_PROGM, 0x4fb0936);
    RegWrite (FES1_LUT_PROGM, 0x13c6cc22);
    RegWrite (FES1_LUT_PROGM, 0x45b313c);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
}

void Set_Fes1_576i_to_576i(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fdfe008);
    RegWrite (FES1_LUT_PROGM, 0xbfc008);
    RegWrite (FES1_LUT_PROGM, 0x9f9fc210);
    RegWrite (FES1_LUT_PROGM, 0x17f8010);
    RegWrite (FES1_LUT_PROGM, 0x9f77a617);
    RegWrite (FES1_LUT_PROGM, 0x21f4018);
    RegWrite (FES1_LUT_PROGM, 0x1f2f8a1e);
    RegWrite (FES1_LUT_PROGM, 0x2bf0021);
    RegWrite (FES1_LUT_PROGM, 0x1ee77025);
    RegWrite (FES1_LUT_PROGM, 0x35ec02a);
    RegWrite (FES1_LUT_PROGM, 0x9eaf582c);
    RegWrite (FES1_LUT_PROGM, 0x41e8032);
    RegWrite (FES1_LUT_PROGM, 0x9e574032);
    RegWrite (FES1_LUT_PROGM, 0x4be403b);
    RegWrite (FES1_LUT_PROGM, 0x9dff2a38);
    RegWrite (FES1_LUT_PROGM, 0x55e0044);
    RegWrite (FES1_LUT_PROGM, 0x9da7163e);
    RegWrite (FES1_LUT_PROGM, 0x5fdc04d);
    RegWrite (FES1_LUT_PROGM, 0x1d570244);
    RegWrite (FES1_LUT_PROGM, 0x69d8857);
    RegWrite (FES1_LUT_PROGM, 0x1cf6f049);
    RegWrite (FES1_LUT_PROGM, 0x71d4860);
    RegWrite (FES1_LUT_PROGM, 0x9c86e04e);
    RegWrite (FES1_LUT_PROGM, 0x7bd1069);
    RegWrite (FES1_LUT_PROGM, 0x9c26d053);
    RegWrite (FES1_LUT_PROGM, 0x83cd072);
    RegWrite (FES1_LUT_PROGM, 0x1bbec257);
    RegWrite (FES1_LUT_PROGM, 0x8dc987c);
    RegWrite (FES1_LUT_PROGM, 0x9b4eb45c);
    RegWrite (FES1_LUT_PROGM, 0x95c6085);
    RegWrite (FES1_LUT_PROGM, 0x1adea85f);
    RegWrite (FES1_LUT_PROGM, 0x9dc288f);
    RegWrite (FES1_LUT_PROGM, 0x9a669e63);
    RegWrite (FES1_LUT_PROGM, 0xa5bf898);
    RegWrite (FES1_LUT_PROGM, 0x99ee9466);
    RegWrite (FES1_LUT_PROGM, 0xadbc0a2);
    RegWrite (FES1_LUT_PROGM, 0x19768c69);
    RegWrite (FES1_LUT_PROGM, 0xb3b90ac);
    RegWrite (FES1_LUT_PROGM, 0x98fe846c);
    RegWrite (FES1_LUT_PROGM, 0xbbb60b5);
    RegWrite (FES1_LUT_PROGM, 0x187e7e6e);
    RegWrite (FES1_LUT_PROGM, 0xc1b30bf);
    RegWrite (FES1_LUT_PROGM, 0x17fe7870);
    RegWrite (FES1_LUT_PROGM, 0xc7b08c9);
    RegWrite (FES1_LUT_PROGM, 0x97767472);
    RegWrite (FES1_LUT_PROGM, 0xcdae0d2);
    RegWrite (FES1_LUT_PROGM, 0x96e67074);
    RegWrite (FES1_LUT_PROGM, 0xd3ab8db);
    RegWrite (FES1_LUT_PROGM, 0x96666e75);
    RegWrite (FES1_LUT_PROGM, 0xd7a90e5);
    RegWrite (FES1_LUT_PROGM, 0x15de6c76);
    RegWrite (FES1_LUT_PROGM, 0xdba70ef);
    RegWrite (FES1_LUT_PROGM, 0x15466c77);
    RegWrite (FES1_LUT_PROGM, 0xdfa50f8);
    RegWrite (FES1_LUT_PROGM, 0x14be6c77);
    RegWrite (FES1_LUT_PROGM, 0xe3a3102);
    RegWrite (FES1_LUT_PROGM, 0x142e6e77);
    RegWrite (FES1_LUT_PROGM, 0xe7a190b);
    RegWrite (FES1_LUT_PROGM, 0x13a67077);
    RegWrite (FES1_LUT_PROGM, 0xe9a0115);
    RegWrite (FES1_LUT_PROGM, 0x93167277);
    RegWrite (FES1_LUT_PROGM, 0xeb9e91e);
    RegWrite (FES1_LUT_PROGM, 0x927e7676);
    RegWrite (FES1_LUT_PROGM, 0xed9d927);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fdfe008);
    RegWrite (FES1_LUT_PROGM, 0xbfc008);
    RegWrite (FES1_LUT_PROGM, 0x9f9fc210);
    RegWrite (FES1_LUT_PROGM, 0x17f8010);
    RegWrite (FES1_LUT_PROGM, 0x9f77a617);
    RegWrite (FES1_LUT_PROGM, 0x21f4018);
    RegWrite (FES1_LUT_PROGM, 0x1f2f8a1e);
    RegWrite (FES1_LUT_PROGM, 0x2bf0021);
    RegWrite (FES1_LUT_PROGM, 0x1ee77025);
    RegWrite (FES1_LUT_PROGM, 0x35ec02a);
    RegWrite (FES1_LUT_PROGM, 0x9eaf582c);
    RegWrite (FES1_LUT_PROGM, 0x41e8032);
    RegWrite (FES1_LUT_PROGM, 0x9e574032);
    RegWrite (FES1_LUT_PROGM, 0x4be403b);
    RegWrite (FES1_LUT_PROGM, 0x9dff2a38);
    RegWrite (FES1_LUT_PROGM, 0x55e0044);
    RegWrite (FES1_LUT_PROGM, 0x9da7163e);
    RegWrite (FES1_LUT_PROGM, 0x5fdc04d);
    RegWrite (FES1_LUT_PROGM, 0x1d570244);
    RegWrite (FES1_LUT_PROGM, 0x69d8857);
    RegWrite (FES1_LUT_PROGM, 0x1cf6f049);
    RegWrite (FES1_LUT_PROGM, 0x71d4860);
    RegWrite (FES1_LUT_PROGM, 0x9c86e04e);
    RegWrite (FES1_LUT_PROGM, 0x7bd1069);
    RegWrite (FES1_LUT_PROGM, 0x9c26d053);
    RegWrite (FES1_LUT_PROGM, 0x83cd072);
    RegWrite (FES1_LUT_PROGM, 0x1bbec257);
    RegWrite (FES1_LUT_PROGM, 0x8dc987c);
    RegWrite (FES1_LUT_PROGM, 0x9b4eb45c);
    RegWrite (FES1_LUT_PROGM, 0x95c6085);
    RegWrite (FES1_LUT_PROGM, 0x1adea85f);
    RegWrite (FES1_LUT_PROGM, 0x9dc288f);
    RegWrite (FES1_LUT_PROGM, 0x9a669e63);
    RegWrite (FES1_LUT_PROGM, 0xa5bf898);
    RegWrite (FES1_LUT_PROGM, 0x99ee9466);
    RegWrite (FES1_LUT_PROGM, 0xadbc0a2);
    RegWrite (FES1_LUT_PROGM, 0x19768c69);
    RegWrite (FES1_LUT_PROGM, 0xb3b90ac);
    RegWrite (FES1_LUT_PROGM, 0x98fe846c);
    RegWrite (FES1_LUT_PROGM, 0xbbb60b5);
    RegWrite (FES1_LUT_PROGM, 0x187e7e6e);
    RegWrite (FES1_LUT_PROGM, 0xc1b30bf);
    RegWrite (FES1_LUT_PROGM, 0x17fe7870);
    RegWrite (FES1_LUT_PROGM, 0xc7b08c9);
    RegWrite (FES1_LUT_PROGM, 0x97767472);
    RegWrite (FES1_LUT_PROGM, 0xcdae0d2);
    RegWrite (FES1_LUT_PROGM, 0x96e67074);
    RegWrite (FES1_LUT_PROGM, 0xd3ab8db);
    RegWrite (FES1_LUT_PROGM, 0x96666e75);
    RegWrite (FES1_LUT_PROGM, 0xd7a90e5);
    RegWrite (FES1_LUT_PROGM, 0x15de6c76);
    RegWrite (FES1_LUT_PROGM, 0xdba70ef);
    RegWrite (FES1_LUT_PROGM, 0x15466c77);
    RegWrite (FES1_LUT_PROGM, 0xdfa50f8);
    RegWrite (FES1_LUT_PROGM, 0x14be6c77);
    RegWrite (FES1_LUT_PROGM, 0xe3a3102);
    RegWrite (FES1_LUT_PROGM, 0x142e6e77);
    RegWrite (FES1_LUT_PROGM, 0xe7a190b);
    RegWrite (FES1_LUT_PROGM, 0x13a67077);
    RegWrite (FES1_LUT_PROGM, 0xe9a0115);
    RegWrite (FES1_LUT_PROGM, 0x93167277);
    RegWrite (FES1_LUT_PROGM, 0xeb9e91e);
    RegWrite (FES1_LUT_PROGM, 0x927e7676);
    RegWrite (FES1_LUT_PROGM, 0xed9d927);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
}

void Set_Fes1_480i_to_480i(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fdfe008);
    RegWrite (FES1_LUT_PROGM, 0xbfc008);
    RegWrite (FES1_LUT_PROGM, 0x9f9fc210);
    RegWrite (FES1_LUT_PROGM, 0x17f8010);
    RegWrite (FES1_LUT_PROGM, 0x9f77a617);
    RegWrite (FES1_LUT_PROGM, 0x21f4018);
    RegWrite (FES1_LUT_PROGM, 0x1f2f8a1e);
    RegWrite (FES1_LUT_PROGM, 0x2bf0021);
    RegWrite (FES1_LUT_PROGM, 0x1ee77025);
    RegWrite (FES1_LUT_PROGM, 0x35ec02a);
    RegWrite (FES1_LUT_PROGM, 0x9eaf582c);
    RegWrite (FES1_LUT_PROGM, 0x41e8032);
    RegWrite (FES1_LUT_PROGM, 0x9e574032);
    RegWrite (FES1_LUT_PROGM, 0x4be403b);
    RegWrite (FES1_LUT_PROGM, 0x9dff2a38);
    RegWrite (FES1_LUT_PROGM, 0x55e0044);
    RegWrite (FES1_LUT_PROGM, 0x9da7163e);
    RegWrite (FES1_LUT_PROGM, 0x5fdc04d);
    RegWrite (FES1_LUT_PROGM, 0x1d570244);
    RegWrite (FES1_LUT_PROGM, 0x69d8857);
    RegWrite (FES1_LUT_PROGM, 0x1cf6f049);
    RegWrite (FES1_LUT_PROGM, 0x71d4860);
    RegWrite (FES1_LUT_PROGM, 0x9c86e04e);
    RegWrite (FES1_LUT_PROGM, 0x7bd1069);
    RegWrite (FES1_LUT_PROGM, 0x9c26d053);
    RegWrite (FES1_LUT_PROGM, 0x83cd072);
    RegWrite (FES1_LUT_PROGM, 0x1bbec257);
    RegWrite (FES1_LUT_PROGM, 0x8dc987c);
    RegWrite (FES1_LUT_PROGM, 0x9b4eb45c);
    RegWrite (FES1_LUT_PROGM, 0x95c6085);
    RegWrite (FES1_LUT_PROGM, 0x1adea85f);
    RegWrite (FES1_LUT_PROGM, 0x9dc288f);
    RegWrite (FES1_LUT_PROGM, 0x9a669e63);
    RegWrite (FES1_LUT_PROGM, 0xa5bf898);
    RegWrite (FES1_LUT_PROGM, 0x99ee9466);
    RegWrite (FES1_LUT_PROGM, 0xadbc0a2);
    RegWrite (FES1_LUT_PROGM, 0x19768c69);
    RegWrite (FES1_LUT_PROGM, 0xb3b90ac);
    RegWrite (FES1_LUT_PROGM, 0x98fe846c);
    RegWrite (FES1_LUT_PROGM, 0xbbb60b5);
    RegWrite (FES1_LUT_PROGM, 0x187e7e6e);
    RegWrite (FES1_LUT_PROGM, 0xc1b30bf);
    RegWrite (FES1_LUT_PROGM, 0x17fe7870);
    RegWrite (FES1_LUT_PROGM, 0xc7b08c9);
    RegWrite (FES1_LUT_PROGM, 0x97767472);
    RegWrite (FES1_LUT_PROGM, 0xcdae0d2);
    RegWrite (FES1_LUT_PROGM, 0x96e67074);
    RegWrite (FES1_LUT_PROGM, 0xd3ab8db);
    RegWrite (FES1_LUT_PROGM, 0x96666e75);
    RegWrite (FES1_LUT_PROGM, 0xd7a90e5);
    RegWrite (FES1_LUT_PROGM, 0x15de6c76);
    RegWrite (FES1_LUT_PROGM, 0xdba70ef);
    RegWrite (FES1_LUT_PROGM, 0x15466c77);
    RegWrite (FES1_LUT_PROGM, 0xdfa50f8);
    RegWrite (FES1_LUT_PROGM, 0x14be6c77);
    RegWrite (FES1_LUT_PROGM, 0xe3a3102);
    RegWrite (FES1_LUT_PROGM, 0x142e6e77);
    RegWrite (FES1_LUT_PROGM, 0xe7a190b);
    RegWrite (FES1_LUT_PROGM, 0x13a67077);
    RegWrite (FES1_LUT_PROGM, 0xe9a0115);
    RegWrite (FES1_LUT_PROGM, 0x93167277);
    RegWrite (FES1_LUT_PROGM, 0xeb9e91e);
    RegWrite (FES1_LUT_PROGM, 0x927e7676);
    RegWrite (FES1_LUT_PROGM, 0xed9d927);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fdfe008);
    RegWrite (FES1_LUT_PROGM, 0xbfc008);
    RegWrite (FES1_LUT_PROGM, 0x9f9fc210);
    RegWrite (FES1_LUT_PROGM, 0x17f8010);
    RegWrite (FES1_LUT_PROGM, 0x9f77a617);
    RegWrite (FES1_LUT_PROGM, 0x21f4018);
    RegWrite (FES1_LUT_PROGM, 0x1f2f8a1e);
    RegWrite (FES1_LUT_PROGM, 0x2bf0021);
    RegWrite (FES1_LUT_PROGM, 0x1ee77025);
    RegWrite (FES1_LUT_PROGM, 0x35ec02a);
    RegWrite (FES1_LUT_PROGM, 0x9eaf582c);
    RegWrite (FES1_LUT_PROGM, 0x41e8032);
    RegWrite (FES1_LUT_PROGM, 0x9e574032);
    RegWrite (FES1_LUT_PROGM, 0x4be403b);
    RegWrite (FES1_LUT_PROGM, 0x9dff2a38);
    RegWrite (FES1_LUT_PROGM, 0x55e0044);
    RegWrite (FES1_LUT_PROGM, 0x9da7163e);
    RegWrite (FES1_LUT_PROGM, 0x5fdc04d);
    RegWrite (FES1_LUT_PROGM, 0x1d570244);
    RegWrite (FES1_LUT_PROGM, 0x69d8857);
    RegWrite (FES1_LUT_PROGM, 0x1cf6f049);
    RegWrite (FES1_LUT_PROGM, 0x71d4860);
    RegWrite (FES1_LUT_PROGM, 0x9c86e04e);
    RegWrite (FES1_LUT_PROGM, 0x7bd1069);
    RegWrite (FES1_LUT_PROGM, 0x9c26d053);
    RegWrite (FES1_LUT_PROGM, 0x83cd072);
    RegWrite (FES1_LUT_PROGM, 0x1bbec257);
    RegWrite (FES1_LUT_PROGM, 0x8dc987c);
    RegWrite (FES1_LUT_PROGM, 0x9b4eb45c);
    RegWrite (FES1_LUT_PROGM, 0x95c6085);
    RegWrite (FES1_LUT_PROGM, 0x1adea85f);
    RegWrite (FES1_LUT_PROGM, 0x9dc288f);
    RegWrite (FES1_LUT_PROGM, 0x9a669e63);
    RegWrite (FES1_LUT_PROGM, 0xa5bf898);
    RegWrite (FES1_LUT_PROGM, 0x99ee9466);
    RegWrite (FES1_LUT_PROGM, 0xadbc0a2);
    RegWrite (FES1_LUT_PROGM, 0x19768c69);
    RegWrite (FES1_LUT_PROGM, 0xb3b90ac);
    RegWrite (FES1_LUT_PROGM, 0x98fe846c);
    RegWrite (FES1_LUT_PROGM, 0xbbb60b5);
    RegWrite (FES1_LUT_PROGM, 0x187e7e6e);
    RegWrite (FES1_LUT_PROGM, 0xc1b30bf);
    RegWrite (FES1_LUT_PROGM, 0x17fe7870);
    RegWrite (FES1_LUT_PROGM, 0xc7b08c9);
    RegWrite (FES1_LUT_PROGM, 0x97767472);
    RegWrite (FES1_LUT_PROGM, 0xcdae0d2);
    RegWrite (FES1_LUT_PROGM, 0x96e67074);
    RegWrite (FES1_LUT_PROGM, 0xd3ab8db);
    RegWrite (FES1_LUT_PROGM, 0x96666e75);
    RegWrite (FES1_LUT_PROGM, 0xd7a90e5);
    RegWrite (FES1_LUT_PROGM, 0x15de6c76);
    RegWrite (FES1_LUT_PROGM, 0xdba70ef);
    RegWrite (FES1_LUT_PROGM, 0x15466c77);
    RegWrite (FES1_LUT_PROGM, 0xdfa50f8);
    RegWrite (FES1_LUT_PROGM, 0x14be6c77);
    RegWrite (FES1_LUT_PROGM, 0xe3a3102);
    RegWrite (FES1_LUT_PROGM, 0x142e6e77);
    RegWrite (FES1_LUT_PROGM, 0xe7a190b);
    RegWrite (FES1_LUT_PROGM, 0x13a67077);
    RegWrite (FES1_LUT_PROGM, 0xe9a0115);
    RegWrite (FES1_LUT_PROGM, 0x93167277);
    RegWrite (FES1_LUT_PROGM, 0xeb9e91e);
    RegWrite (FES1_LUT_PROGM, 0x927e7676);
    RegWrite (FES1_LUT_PROGM, 0xed9d927);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
}

void Set_Fes1_480i_to_576i(pxc_dfb_driver_data mdrv)
{
    //FES Register settings
    //=== Input Field 0 / Output Field 0: 
    Wait_For_Fes1_Fifo(14);
    RegWrite (FES1_POSREG0_INIT, 0x04000000);
    RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 0: 
    RegWrite (FES1_POSREG1_INIT, 0x04000000);
    RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 0 / Output Field 1: 
    //RegWrite (FES1_POSREG0_INIT, 0x04000000);
    //RegWrite (FES1_POSREG0_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== Input Field 1 / Output Field 1: 
    //RegWrite (FES1_POSREG1_INIT, 0x04000000);
    //RegWrite (FES1_POSREG1_UV_INIT, 0x00000000);
    //RegWrite (FES1_NUMTAPS_HELD, 0x03050305);
    //=== General: 
    RegWrite (FES1_SRC1_SCLV_R_Y, 0x00035555);
    RegWrite (FES1_SRC1_SCLV_GB_UV, 0x00035555);
    RegWrite (FES1_SRC1_SCLH_R_Y, 0x00040000);
    RegWrite (FES1_SRC1_SCLH_GB_UV, 0x00040000);
    RegWrite (FES1_SRC1_FOLDV_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDV_GB_UV, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_R_Y, 0x00002040);
    RegWrite (FES1_SRC1_FOLDH_GB_UV, 0x00002040);
    //Vertical scaler Y
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fffe603);
    RegWrite (FES1_LUT_PROGM, 0x1fe807);
    RegWrite (FES1_LUT_PROGM, 0x1ff7ce06);
    RegWrite (FES1_LUT_PROGM, 0x1fc80e);
    RegWrite (FES1_LUT_PROGM, 0x1fefb609);
    RegWrite (FES1_LUT_PROGM, 0x1fa815);
    RegWrite (FES1_LUT_PROGM, 0x1fd7a00c);
    RegWrite (FES1_LUT_PROGM, 0x1f881d);
    RegWrite (FES1_LUT_PROGM, 0x9fb78a0e);
    RegWrite (FES1_LUT_PROGM, 0x3f6824);
    RegWrite (FES1_LUT_PROGM, 0x1f8f7611);
    RegWrite (FES1_LUT_PROGM, 0x3f482d);
    RegWrite (FES1_LUT_PROGM, 0x1f5f6413);
    RegWrite (FES1_LUT_PROGM, 0x3f2835);
    RegWrite (FES1_LUT_PROGM, 0x1f275215);
    RegWrite (FES1_LUT_PROGM, 0x5f083e);
    RegWrite (FES1_LUT_PROGM, 0x9eef4217);
    RegWrite (FES1_LUT_PROGM, 0x5ee046);
    RegWrite (FES1_LUT_PROGM, 0x1eaf3218);
    RegWrite (FES1_LUT_PROGM, 0x7ec050);
    RegWrite (FES1_LUT_PROGM, 0x1e5f241a);
    RegWrite (FES1_LUT_PROGM, 0x9e9859);
    RegWrite (FES1_LUT_PROGM, 0x9e0f181b);
    RegWrite (FES1_LUT_PROGM, 0x9e7062);
    RegWrite (FES1_LUT_PROGM, 0x9dbf0c1c);
    RegWrite (FES1_LUT_PROGM, 0xbe486c);
    RegWrite (FES1_LUT_PROGM, 0x9d67001d);
    RegWrite (FES1_LUT_PROGM, 0xde2076);
    RegWrite (FES1_LUT_PROGM, 0x9cf6f81e);
    RegWrite (FES1_LUT_PROGM, 0xfdf880);
    RegWrite (FES1_LUT_PROGM, 0x9c8ef01f);
    RegWrite (FES1_LUT_PROGM, 0x11dd08a);
    RegWrite (FES1_LUT_PROGM, 0x9c2ee81f);
    RegWrite (FES1_LUT_PROGM, 0x13da895);
    RegWrite (FES1_LUT_PROGM, 0x1bb6e220);
    RegWrite (FES1_LUT_PROGM, 0x13d80a0);
    RegWrite (FES1_LUT_PROGM, 0x9b36dc20);
    RegWrite (FES1_LUT_PROGM, 0x15d60aa);
    RegWrite (FES1_LUT_PROGM, 0x1ab6d820);
    RegWrite (FES1_LUT_PROGM, 0x19d38b5);
    RegWrite (FES1_LUT_PROGM, 0x1a3ed620);
    RegWrite (FES1_LUT_PROGM, 0x1bd10c0);
    RegWrite (FES1_LUT_PROGM, 0x19b6d220);
    RegWrite (FES1_LUT_PROGM, 0x1dce8cb);
    RegWrite (FES1_LUT_PROGM, 0x192ed220);
    RegWrite (FES1_LUT_PROGM, 0x1fcc0d6);
    RegWrite (FES1_LUT_PROGM, 0x1896d01f);
    RegWrite (FES1_LUT_PROGM, 0x21ca0e1);
    RegWrite (FES1_LUT_PROGM, 0x1806d21f);
    RegWrite (FES1_LUT_PROGM, 0x23c78ec);
    RegWrite (FES1_LUT_PROGM, 0x976ed21e);
    RegWrite (FES1_LUT_PROGM, 0x25c58f7);
    RegWrite (FES1_LUT_PROGM, 0x16d6d41d);
    RegWrite (FES1_LUT_PROGM, 0x29c3103);
    RegWrite (FES1_LUT_PROGM, 0x962ed81d);
    RegWrite (FES1_LUT_PROGM, 0x2bc190d);
    RegWrite (FES1_LUT_PROGM, 0x958edc1c);
    RegWrite (FES1_LUT_PROGM, 0x2dbf918);
    RegWrite (FES1_LUT_PROGM, 0x14e6e01b);
    RegWrite (FES1_LUT_PROGM, 0x2fbd923);
    RegWrite (FES1_LUT_PROGM, 0x143ee41a);
    RegWrite (FES1_LUT_PROGM, 0x31bc12e);
    RegWrite (FES1_LUT_PROGM, 0x938eea19);
    RegWrite (FES1_LUT_PROGM, 0x33ba938);
    //Vertical scaler CbCr
    Wait_For_Fes1_Fifo(66);
    RegWrite (FES1_CONFIG_CTRL, 0x3000);
    RegWrite (FES1_LUT_PROGM, 0x20000000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x1fffe603);
    RegWrite (FES1_LUT_PROGM, 0x1fe807);
    RegWrite (FES1_LUT_PROGM, 0x1ff7ce06);
    RegWrite (FES1_LUT_PROGM, 0x1fc80e);
    RegWrite (FES1_LUT_PROGM, 0x1fefb609);
    RegWrite (FES1_LUT_PROGM, 0x1fa815);
    RegWrite (FES1_LUT_PROGM, 0x1fd7a00c);
    RegWrite (FES1_LUT_PROGM, 0x1f881d);
    RegWrite (FES1_LUT_PROGM, 0x9fb78a0e);
    RegWrite (FES1_LUT_PROGM, 0x3f6824);
    RegWrite (FES1_LUT_PROGM, 0x1f8f7611);
    RegWrite (FES1_LUT_PROGM, 0x3f482d);
    RegWrite (FES1_LUT_PROGM, 0x1f5f6413);
    RegWrite (FES1_LUT_PROGM, 0x3f2835);
    RegWrite (FES1_LUT_PROGM, 0x1f275215);
    RegWrite (FES1_LUT_PROGM, 0x5f083e);
    RegWrite (FES1_LUT_PROGM, 0x9eef4217);
    RegWrite (FES1_LUT_PROGM, 0x5ee046);
    RegWrite (FES1_LUT_PROGM, 0x1eaf3218);
    RegWrite (FES1_LUT_PROGM, 0x7ec050);
    RegWrite (FES1_LUT_PROGM, 0x1e5f241a);
    RegWrite (FES1_LUT_PROGM, 0x9e9859);
    RegWrite (FES1_LUT_PROGM, 0x9e0f181b);
    RegWrite (FES1_LUT_PROGM, 0x9e7062);
    RegWrite (FES1_LUT_PROGM, 0x9dbf0c1c);
    RegWrite (FES1_LUT_PROGM, 0xbe486c);
    RegWrite (FES1_LUT_PROGM, 0x9d67001d);
    RegWrite (FES1_LUT_PROGM, 0xde2076);
    RegWrite (FES1_LUT_PROGM, 0x9cf6f81e);
    RegWrite (FES1_LUT_PROGM, 0xfdf880);
    RegWrite (FES1_LUT_PROGM, 0x9c8ef01f);
    RegWrite (FES1_LUT_PROGM, 0x11dd08a);
    RegWrite (FES1_LUT_PROGM, 0x9c2ee81f);
    RegWrite (FES1_LUT_PROGM, 0x13da895);
    RegWrite (FES1_LUT_PROGM, 0x1bb6e220);
    RegWrite (FES1_LUT_PROGM, 0x13d80a0);
    RegWrite (FES1_LUT_PROGM, 0x9b36dc20);
    RegWrite (FES1_LUT_PROGM, 0x15d60aa);
    RegWrite (FES1_LUT_PROGM, 0x1ab6d820);
    RegWrite (FES1_LUT_PROGM, 0x19d38b5);
    RegWrite (FES1_LUT_PROGM, 0x1a3ed620);
    RegWrite (FES1_LUT_PROGM, 0x1bd10c0);
    RegWrite (FES1_LUT_PROGM, 0x19b6d220);
    RegWrite (FES1_LUT_PROGM, 0x1dce8cb);
    RegWrite (FES1_LUT_PROGM, 0x192ed220);
    RegWrite (FES1_LUT_PROGM, 0x1fcc0d6);
    RegWrite (FES1_LUT_PROGM, 0x1896d01f);
    RegWrite (FES1_LUT_PROGM, 0x21ca0e1);
    RegWrite (FES1_LUT_PROGM, 0x1806d21f);
    RegWrite (FES1_LUT_PROGM, 0x23c78ec);
    RegWrite (FES1_LUT_PROGM, 0x976ed21e);
    RegWrite (FES1_LUT_PROGM, 0x25c58f7);
    RegWrite (FES1_LUT_PROGM, 0x16d6d41d);
    RegWrite (FES1_LUT_PROGM, 0x29c3103);
    RegWrite (FES1_LUT_PROGM, 0x962ed81d);
    RegWrite (FES1_LUT_PROGM, 0x2bc190d);
    RegWrite (FES1_LUT_PROGM, 0x958edc1c);
    RegWrite (FES1_LUT_PROGM, 0x2dbf918);
    RegWrite (FES1_LUT_PROGM, 0x14e6e01b);
    RegWrite (FES1_LUT_PROGM, 0x2fbd923);
    RegWrite (FES1_LUT_PROGM, 0x143ee41a);
    RegWrite (FES1_LUT_PROGM, 0x31bc12e);
    RegWrite (FES1_LUT_PROGM, 0x938eea19);
    RegWrite (FES1_LUT_PROGM, 0x33ba938);
    //Horzontal scaler Y
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0x800);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
    //Horzontal scaler CbCr
    Wait_For_Fes1_Fifo(100);
    RegWrite (FES1_CONFIG_CTRL, 0xc00);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0x2000);
    RegWrite (FES1_LUT_PROGM, 0x0);
    RegWrite (FES1_LUT_PROGM, 0xe008fb04);
    RegWrite (FES1_LUT_PROGM, 0xc0081fef);
    RegWrite (FES1_LUT_PROGM, 0xff00bf);
    RegWrite (FES1_LUT_PROGM, 0xc210f608);
    RegWrite (FES1_LUT_PROGM, 0x80109fc7);
    RegWrite (FES1_LUT_PROGM, 0x1be017f);
    RegWrite (FES1_LUT_PROGM, 0xa617f00c);
    RegWrite (FES1_LUT_PROGM, 0x40191f9f);
    RegWrite (FES1_LUT_PROGM, 0x2bd021f);
    RegWrite (FES1_LUT_PROGM, 0x8a1eeb0f);
    RegWrite (FES1_LUT_PROGM, 0x219f6f);
    RegWrite (FES1_LUT_PROGM, 0x37c02bf);
    RegWrite (FES1_LUT_PROGM, 0x7026e713);
    RegWrite (FES1_LUT_PROGM, 0xb82a9f37);
    RegWrite (FES1_LUT_PROGM, 0x43b037e);
    RegWrite (FES1_LUT_PROGM, 0x562ce217);
    RegWrite (FES1_LUT_PROGM, 0x78339f0f);
    RegWrite (FES1_LUT_PROGM, 0x4fa041e);
    RegWrite (FES1_LUT_PROGM, 0x3e33dd1a);
    RegWrite (FES1_LUT_PROGM, 0x383c9ebf);
    RegWrite (FES1_LUT_PROGM, 0x5b904be);
    RegWrite (FES1_LUT_PROGM, 0x2839d91e);
    RegWrite (FES1_LUT_PROGM, 0xf8459e77);
    RegWrite (FES1_LUT_PROGM, 0x678455d);
    RegWrite (FES1_LUT_PROGM, 0x123fd521);
    RegWrite (FES1_LUT_PROGM, 0xb84f1e27);
    RegWrite (FES1_LUT_PROGM, 0x73745fd);
    RegWrite (FES1_LUT_PROGM, 0xfe45d124);
    RegWrite (FES1_LUT_PROGM, 0x78589dde);
    RegWrite (FES1_LUT_PROGM, 0x7f6469d);
    RegWrite (FES1_LUT_PROGM, 0xea4bcd27);
    RegWrite (FES1_LUT_PROGM, 0x38621d8e);
    RegWrite (FES1_LUT_PROGM, 0x8b5873d);
    RegWrite (FES1_LUT_PROGM, 0xda50c92a);
    RegWrite (FES1_LUT_PROGM, 0x6b9d26);
    RegWrite (FES1_LUT_PROGM, 0x93487dd);
    RegWrite (FES1_LUT_PROGM, 0xc855c62d);
    RegWrite (FES1_LUT_PROGM, 0xc0759cd6);
    RegWrite (FES1_LUT_PROGM, 0x9f3c87c);
    RegWrite (FES1_LUT_PROGM, 0xba5ac22f);
    RegWrite (FES1_LUT_PROGM, 0x807f9c76);
    RegWrite (FES1_LUT_PROGM, 0xab2c91c);
    RegWrite (FES1_LUT_PROGM, 0xac5ebf32);
    RegWrite (FES1_LUT_PROGM, 0x48891bfe);
    RegWrite (FES1_LUT_PROGM, 0xb32099c);
    RegWrite (FES1_LUT_PROGM, 0xa062bc34);
    RegWrite (FES1_LUT_PROGM, 0x10931b96);
    RegWrite (FES1_LUT_PROGM, 0xbb14a1c);
    RegWrite (FES1_LUT_PROGM, 0x9466b936);
    RegWrite (FES1_LUT_PROGM, 0xd89d1b2e);
    RegWrite (FES1_LUT_PROGM, 0xc708a9b);
    RegWrite (FES1_LUT_PROGM, 0x8869b738);
    RegWrite (FES1_LUT_PROGM, 0xa0a79abe);
    RegWrite (FES1_LUT_PROGM, 0xcefcb1b);
    RegWrite (FES1_LUT_PROGM, 0x806cb43a);
    RegWrite (FES1_LUT_PROGM, 0x70b11a3e);
    RegWrite (FES1_LUT_PROGM, 0xd6f4b9b);
    RegWrite (FES1_LUT_PROGM, 0x786fb23c);
    RegWrite (FES1_LUT_PROGM, 0x38bb19c6);
    RegWrite (FES1_LUT_PROGM, 0xdee8c1b);
    RegWrite (FES1_LUT_PROGM, 0x7072b03d);
    RegWrite (FES1_LUT_PROGM, 0x8c59946);
    RegWrite (FES1_LUT_PROGM, 0xe6e0c7b);
    RegWrite (FES1_LUT_PROGM, 0x6a74ae3f);
    RegWrite (FES1_LUT_PROGM, 0xd8cf98ce);
    RegWrite (FES1_LUT_PROGM, 0xeed4cfa);
    RegWrite (FES1_LUT_PROGM, 0x6676ad40);
    RegWrite (FES1_LUT_PROGM, 0xb0d99846);
    RegWrite (FES1_LUT_PROGM, 0xf2ccd5a);
    RegWrite (FES1_LUT_PROGM, 0x6278ac41);
    RegWrite (FES1_LUT_PROGM, 0x88e397b6);
    RegWrite (FES1_LUT_PROGM, 0xf6c8d9a);
    RegWrite (FES1_LUT_PROGM, 0x6079ab42);
    RegWrite (FES1_LUT_PROGM, 0x60ed972e);
    RegWrite (FES1_LUT_PROGM, 0xfec0dfa);
    RegWrite (FES1_LUT_PROGM, 0x5e7aaa43);
    RegWrite (FES1_LUT_PROGM, 0x38f816ae);
    RegWrite (FES1_LUT_PROGM, 0x102b8e3a);
    RegWrite (FES1_LUT_PROGM, 0x5e7ba943);
    RegWrite (FES1_LUT_PROGM, 0x2101160e);
    RegWrite (FES1_LUT_PROGM, 0x106b4e7a);
    RegWrite (FES1_LUT_PROGM, 0x5c7ca844);
    RegWrite (FES1_LUT_PROGM, 0xf90b9586);
    RegWrite (FES1_LUT_PROGM, 0x10aaced9);
    RegWrite (FES1_LUT_PROGM, 0x5e7ca844);
    RegWrite (FES1_LUT_PROGM, 0xe11514ee);
    RegWrite (FES1_LUT_PROGM, 0x10eacef9);
    RegWrite (FES1_LUT_PROGM, 0x607ca844);
    RegWrite (FES1_LUT_PROGM, 0xc91f945e);
    RegWrite (FES1_LUT_PROGM, 0x10ea8f39);
    RegWrite (FES1_LUT_PROGM, 0x647ba844);
    RegWrite (FES1_LUT_PROGM, 0xb12913ce);
    RegWrite (FES1_LUT_PROGM, 0x112a4f59);
    RegWrite (FES1_LUT_PROGM, 0x667ba844);
    RegWrite (FES1_LUT_PROGM, 0x99331336);
    RegWrite (FES1_LUT_PROGM, 0x112a0f79);
}


int gcf (int a, int b) {
  while (1) {
    a = a % b;
    if (a == 0)
      return b;
    b = b % a;
    if (b == 0)
      return a;
  }
}

int lcm (int a, int b) {
  int n;
  for (n = 1;;n++) {
    if ((n%a == 0) && (n%b == 0))
      return n;
  }
}


int format_pos_value(double fract)
{
    int pos_value;

    pos_value = (int)(fract*4096+0.5);
	if (pos_value > 65535) 
    {
        pos_value = 65535;
    }

	return pos_value;
}


void Set_Fes1_Parameter(pxc_dfb_driver_data mdrv,
						int vsize_in, 
						int vsize_out, 
						int hsize_in, 
						int hsize_out, 
						int interlaced_in, 
						int interlaced_out, 
						int chroma_in, 
						int chroma_out, 
						int genLh, 
						int genLv)
{

	double hratio = (double)hsize_in / hsize_out;
	double vratio = (double)vsize_in / vsize_out;
	double y00, y10;
	double cbcr00, cbcr10;
	int y00_integer, y10_integer;
	int cbcr00_integer, cbcr10_integer;
	double y00_fract, y10_fract;
	double cbcr00_fract, cbcr10_fract;
	double y01, y11;
	double cbcr01, cbcr11;
	int y01_integer, y11_integer;
	int cbcr01_integer, cbcr11_integer;
	double y01_fract, y11_fract;
	double cbcr01_fract, cbcr11_fract;

	double vmult, hmult;

    int Lv_y    = 64;
	int Lh_y    = 64;
	int Lv_cbcr = 64;
	int Lh_cbcr = 64;
    bool mirror_h = 1;
    bool mirror_v = 1;
    bool trho  = 0;


    if ((chroma_in == 420) &&
        (chroma_out == 444))
	{
        vmult = 2;
        hmult = 2;
	}
    else if ((chroma_in == 420) &&
        (chroma_out == 422))
	{
        vmult = 1;
        hmult = 2;
	}
    else
    {
        D_DEBUG ("Error in chroma types in Set_Fes1_Parameter\n");
        return;
    }

	// Set defaults here for vscaler
	double y_default = 3.0;
	double cbcr_default;
	int y_slice = 1;
	int cbcr_slice = 1;
	if(chroma_in == chroma_out)
	{
	  cbcr_default = 3.0;
	}
	else
	{
	  cbcr_default = 2.75;
	  cbcr_slice = 4;
	}

	if((interlaced_in == 1) )
	{
	  y00 = y_default;
	  y10 = y_default - 0.5;
	  y_slice = 2;
	  cbcr00 = cbcr_default + 0.125;
	  cbcr10 = cbcr_default - 0.5 - 0.125;
	  cbcr_slice = 8;

	  if(interlaced_out == 0)   // I-P
	  {
	  }
	  else                  // I-I
	  {
		// this case is probably never used.  Just scale fields
		// might need to look at y_slice/cbcr_slice for this case, it might not be right???
		y00 = y_default;
		y10 = y_default;
		cbcr00 = cbcr_default;
		cbcr10 = cbcr_default;

		// /2 below is due to fact that FES does not know output will be interlaced.  Need to set output=720:240:1:60, for example to output 480i
		y01 = y_default;
		y11 = y_default;
		cbcr01 = cbcr_default;
		cbcr11 = cbcr_default;
		y01 = y01 + vratio/2;
		cbcr01 = cbcr01 + vratio/2/2;
		y11 = y11 + vratio/2;
		cbcr11 = cbcr11 + vratio/2/2;
	  }
	}
	else
	{
	  y00 = y_default;
	  y10 = y_default;
	  cbcr00 = cbcr_default;
	  cbcr10 = cbcr_default;
	  // Need to also set these variables, or else bug fix screws things
	  y01 = y_default;
	  y11 = y_default;
	  cbcr01 = cbcr_default;
	  cbcr11 = cbcr_default;

	  if(interlaced_out == 0)  // P-P
	  {
		y00 = y_default;
		y10 = y_default;
		cbcr00 = cbcr_default;
		cbcr10 = cbcr_default;
	  }
	  else                    // P-I
	  {
		y01 = y00;
		cbcr01 = cbcr00;
		y11 = y10;
		cbcr11 = cbcr10;

		// /2 below is due to fact that FES does not know output will be interlaced.  Need to set output=720:240:1:60, for example to output 480i
		y01 = y01 + vratio/2;
		cbcr01 = cbcr01 + vratio/2/2;
		y11 = y11 + vratio/2;
		cbcr11 = cbcr11 + vratio/2/2;
	  }
	}

	// bug fix.  In viper, FES hangs if NUM_TAPS_HELD<3.
	double min = 100;
	if(y00 < min)
	{
	  min = y00;
	}
	if(cbcr00 < min)
	{
	  min = cbcr00;
	}
	if(y10 < min)
	{
	  min = y10;
	}
	if(cbcr10 < min)
	{
	  min = cbcr10;
	}
	if(y01 < min)
	{
	  min = y01;
	}
	if(cbcr01 < min)
	{
	  min = cbcr01;
	}
	if(y11 < min)
	{
	  min = y11;
	}
	if(cbcr11 < min)
	{
	  min = cbcr11;
	}
	if(min < 3.0) 
	{
	  y00+=(3.0-min);
	  y01+=(3.0-min);
	  y10+=(3.0-min);
	  y11+=(3.0-min);
	  cbcr00+=(3.0-min);
	  cbcr01+=(3.0-min);
	  cbcr10+=(3.0-min);
	  cbcr11+=(3.0-min);
	}


	y00_integer = (int)y00;
	y10_integer = (int)y10;
	cbcr00_integer = (int)(cbcr00);
	cbcr10_integer = (int)(cbcr10);

	y00_fract = y00 - (int)(y00);
	y10_fract = y10 - (int)(y10);
	cbcr00_fract = cbcr00 - (int)(cbcr00);
	cbcr10_fract = cbcr10 - (int)(cbcr10);

	y01_integer = (int)(y01);
	y11_integer = (int)(y11);
	cbcr01_integer = (int)(cbcr01);
	cbcr11_integer = (int)(cbcr11);

	y01_fract = y01 - (int)(y01);
	y11_fract = y11 - (int)(y11);
	cbcr01_fract = cbcr01 - (int)(cbcr01);
	cbcr11_fract = cbcr11 - (int)(cbcr11);

	//  cerr << "cbcr00       = " << cbcr00 << endl;
	//  cerr << "cbcr01       = " << cbcr10 << endl;
	//  cerr << "cbcr00_fract = " << cbcr00_fract << endl;
	//  cerr << "cbcr01_fract = " << cbcr10_fract << endl;
	//  cerr << "y_slice    = " << y_slice << endl;
	//  cerr << "cbcr_slice = " << cbcr_slice << endl;

	if (genLh) {
	  Lh_y = hsize_out/gcf(hsize_in, hsize_out);
	  Lh_y = lcm(Lh_y, 1);
	  Lh_cbcr = ((int)(hsize_out*hmult))/gcf(hsize_in, ((int)(hsize_out*hmult)));
	  Lh_cbcr = lcm(Lh_cbcr, 1);
	  if ((Lh_cbcr >= 64) || (Lh_y >= 64)) {
		mirror_h = 0;
		if (Lh_y >= 64) Lh_y = 64;
		if (Lh_cbcr >= 64) Lh_cbcr = 64;
	  }
	  //    cerr << "Lh_y = " << Lh_y << endl;
	  //    cerr << "Lh_cbcr = " << Lh_cbcr << endl;
	}
	if (genLv) {
	  Lv_y = vsize_out/gcf(vsize_in, vsize_out);
	  Lv_y = lcm(Lv_y, y_slice);
	  Lv_cbcr = ((int)(vsize_out*vmult))/gcf(vsize_in, ((int)(vsize_out*vmult)));
	  Lv_cbcr = lcm(Lv_cbcr, cbcr_slice);
	  if ((Lv_cbcr >= 64) || (Lv_y >= 64)) {
		mirror_v = 0;
		if (Lv_y >= 64) Lv_y = 64;
		if (Lv_cbcr >= 64) Lv_cbcr = 64;
	  }
	  //    cerr << "Lv_y = " << Lv_y << endl;
	  //    cerr << "Lv_cbcr = " << Lv_cbcr << endl;
	}

    Wait_For_Fes1_Fifo(14);

	// "# === Input Field 0 / Output Field 0: " 
	int init_v;
	init_v = format_pos_value(y00_fract);
	REG_WRITE(FES1_POSREG0_INIT,  init_v << FES1_POSREG0_INIT_FES1_POSREG_INIT_V_SHIFT);

	init_v = format_pos_value(cbcr00_fract);
	REG_WRITE (FES1_POSREG0_UV_INIT, init_v << FES1_POSREG0_UV_INIT_FES1_POSREG_INIT_V_SHIFT);

	RegWrite (FES1_NUMTAPS_HELD, (cbcr00_integer << FES1_NUMTAPS_HELD_FES1_VTAPS_UV_SHIFT) | 
			                     (5 << FES1_NUMTAPS_HELD_FES1_HTAPS_UV_SHIFT) |
			                     (y00_integer << FES1_NUMTAPS_HELD_FES1_VTAPS_SHIFT) | 5);

	// "# === Input Field 1 / Output Field 0: 
	init_v = format_pos_value(y10_fract);
	RegWrite (FES1_POSREG1_INIT, init_v << FES1_POSREG1_INIT_FES1_POSREG_INIT_V_SHIFT);

	init_v = format_pos_value(cbcr10_fract);
	RegWrite (FES1_POSREG1_UV_INIT, init_v << FES1_POSREG1_UV_INIT_FES1_POSREG_INIT_V_SHIFT);

	RegWrite (FES1_NUMTAPS_HELD, (cbcr10_integer << FES1_NUMTAPS_HELD_FES1_VTAPS_UV_SHIFT) |
			                     (5 << FES1_NUMTAPS_HELD_FES1_HTAPS_UV_SHIFT) |
			                     (y10_integer << FES1_NUMTAPS_HELD_FES1_VTAPS_SHIFT) | 5);



	// === General: 
	int scale_ratio;
	if (trho) 
    {
        scale_ratio = (int)(vratio*pow(2.,18.));
    }
    else
    {
        scale_ratio = (int)(vratio*pow(2.,18.)+0.5);
    }
	int rmax;
	rmax = (int)(pow(2.,22)-1);
	if(scale_ratio > rmax)
	{
	  scale_ratio = rmax;
	}
	RegWrite (FES1_SRC1_SCLV_R_Y, scale_ratio & FES1_SRC1_SCLV_R_Y_FES1_SRC1_SCLV_MASK);


	if (trho) scale_ratio = (int)(1./vmult*vratio*pow(2.,18.));
	else scale_ratio = (int)(1./vmult*vratio*pow(2.,18.)+0.5);
	if(scale_ratio > rmax)
	{
	  scale_ratio = rmax;
	}
	RegWrite (FES1_SRC1_SCLV_GB_UV, scale_ratio & FES1_SRC1_SCLV_GB_UV_FES1_SRC1_SCLV_MASK);

	if (trho) scale_ratio = (int)(hratio*pow(2.,18.));
	else scale_ratio = (int)(hratio*pow(2.,18.)+0.5);
	if(scale_ratio > rmax)
	{
	  scale_ratio = rmax;
	}
	RegWrite (FES1_SRC1_SCLH_R_Y, scale_ratio & FES1_SRC1_SCLH_R_Y_FES1_SRC1_SCLH_MASK);
	RegWrite (FES1_SRC1_SCLH_GB_UV, scale_ratio & FES1_SRC1_SCLH_GB_UV_FES1_SRC1_SCLH_MASK);


	int Lmirrored = mirror_v ? (Lv_y/2) : Lv_y;
	scale_ratio = (Lmirrored<<8) + Lv_y;
	RegWrite (FES1_SRC1_FOLDV_R_Y, scale_ratio & (FES1_SRC1_FOLDV_R_Y_FES1_MAX_PHASE_MASK |
												  FES1_SRC1_FOLDV_R_Y_FES1_TRIG_PHASE_MASK));

	Lmirrored = mirror_v ? Lv_cbcr/2 : Lv_cbcr;
	scale_ratio = (Lmirrored<<8) + Lv_cbcr;
	RegWrite (FES1_SRC1_FOLDV_GB_UV, scale_ratio & (FES1_SRC1_FOLDV_GB_UV_FES1_MAX_PHASE_MASK |
													FES1_SRC1_FOLDV_GB_UV_FES1_TRIG_PHASE_MASK) );


	Lmirrored = mirror_h ? Lh_y/2 : Lh_y;
	scale_ratio = (Lmirrored<<8) + Lh_y;
	RegWrite (FES1_SRC1_FOLDH_R_Y, scale_ratio & (FES1_SRC1_FOLDH_R_Y_FES1_MAX_PHASE_MASK | 
												  FES1_SRC1_FOLDH_R_Y_FES1_TRIG_PHASE_MASK));

	Lmirrored = mirror_h ? Lh_cbcr/2 : Lh_cbcr;
	scale_ratio = (Lmirrored<<8) + Lh_cbcr;
	RegWrite (FES1_SRC1_FOLDH_GB_UV, scale_ratio & (FES1_SRC1_FOLDH_GB_UV_FES1_MAX_PHASE_MASK | 
													FES1_SRC1_FOLDH_GB_UV_FES1_TRIG_PHASE_MASK));

	return;

}

