/*
*   ViXS XCode4210 Component VENC driver
*   This file contains the programming scripts
*   for the various modes a user may want to use
*   to set up the component output 
*
*/


#ifndef __XCODE4_COMP_VENC_H__
#define __XCODE4_COMP_VENC_H__
#include <config.h>

#include <asm/types.h>

#include <stdio.h>
#include <sys/mman.h>
#include <sys/ioctl.h>

#include <directfb.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>

#include <math.h>

#include <misc/conf.h>

#include <gfx/convert.h>

#include "viper.h"
#include "xcmmio.h"
#include "xcode4_osd.h"
#include "xcioctl.h"
#include "xcode4_adv7511.h"

/* Entry in a table (for a single register write) */
typedef struct {
    u32     reg;
    u32     value;
    u32     mask;
} comp_reg_entry_t;

#include <directfb.h>


/* The excerpt below is an e-mail explaining the lower nibble of DISP_OUTPUT_MUX ... */
/*
grp1 is for the hd path, grp2 for the sd.  We never us the sd path for anything but sd, however it is capable of supporting hd.

This register is the oversample ratio as a power of 2.  The output rate is always ~297 for hd and ~216 MHz for sd.   
Since SD rate is 27 the ovs is 216/27 = 8 so the register would be 3 (2^3).  
For 1080i/720p the sample rate is 74.25 so 297/74.25 = 4 and the reg should be 2 (2^2).  
For 1080p it's 297/148.5 = 2 so reg is 1 (2^1).

Jim
*/


static const comp_reg_entry_t comp480p60 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30250000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20430000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0FFF8006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207E8003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20960000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x04000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31930000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33298000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30338000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21BC8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31DB0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30338000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21BC8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x332D0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30540000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20588000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20600006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207E8003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x317F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21BC8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31DB0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30520000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21BC8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31DB0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30338000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30540000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20588000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20600006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207E8003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x74000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x395E7FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x6500F8, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x268, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    //{DISP_HD_VENC0_YCMPN, 0x03C00093, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    //{DISP_HD_VENC0_CB2GAIN, 0x80450045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000C0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000181, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    //{DISP_HD_VENC0_DVE_VSYNC_START, 0x5a7ff003, 0xFFFFFFFF},
    //{DISP_HD_VENC0_DVE_VSYNC_END, 0x037ff006, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x13, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};

static const comp_reg_entry_t comp576p50 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20040000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30240000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20460000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0FFF8006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207E8003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20960000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x04000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20040000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31940000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33298000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x395E7FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x920133, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x268, 0xFFFFFFFF},
    //{DISP_HD_VENC0_YCMPN, 0x03C00093, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    //{DISP_HD_VENC0_CB2GAIN, 0x80450045, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x13, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};

static const comp_reg_entry_t comp720p24 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24F60000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x350A0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x451E0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x258C0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x07FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24F60000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x350A0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x451E0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x258C0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x380C0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x39487FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x40013F, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x273, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    //{DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    //{DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x13, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};


static const comp_reg_entry_t comp720p25 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24A38000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x34B78000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x44CB8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x25398000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x07FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24A38000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x34B78000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x44CB8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x25398000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x37B98000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x27FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x39487FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x40013F, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x273, 0xFFFFFFFF},
    //{DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    //{DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x13, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};


static const comp_reg_entry_t comp720p30 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23598000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x336D8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x43818000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x07FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23598000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x336D8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x43818000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x366F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x27FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x39487FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x40013F, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x273, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x13, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};

static const comp_reg_entry_t comp720p50 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20C58000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30D98000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x40ED8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x215B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x07FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20C58000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30D98000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x40ED8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x215B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33DB8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x27FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x39487FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x40013F, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x273, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x12, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};


static const comp_reg_entry_t comp720p60 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20208000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30348000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x40488000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20B68000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x07FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20208000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30348000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x40488000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20B68000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33368000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x27FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x39487FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x40013F, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x273, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x12, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};

static const comp_reg_entry_t comp1080i25 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x411B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21660000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x411B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21478000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x32FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23838000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33998000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x43AF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23DB8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x3FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x306B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x411B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21478000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x32FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23838000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33998000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x43AF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23DB8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x3FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x306B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x411B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23838000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33998000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x43AF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x411B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23838000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33998000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x43AF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23DB8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x3FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x306B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x411B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21478000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x32FF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x23838000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33998000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x43AF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x39487FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x40013F, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x273, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A3, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00004640, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00004666, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000046E3, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00004707, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00008CC0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x12, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};


static const comp_reg_entry_t comp1080i30 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20138000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30298000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x403F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x208A8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20138000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30298000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x403F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x32238000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x22398000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x324F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x42658000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x22918000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x34498000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20138000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30298000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x403F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x22398000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x324F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x42658000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x22918000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x34498000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20138000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30298000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x403F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x32238000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x22398000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x324F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x42658000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20138000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30298000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x403F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x22398000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x324F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x42658000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x39487FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x40013F, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x273, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000C7, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00004640, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00004663, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000046E1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00004705, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00008CC0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x12, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};

static const comp_reg_entry_t comp1080p24 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21268000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x313C8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x41528000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x219D0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21268000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x313C8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x41528000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x217E8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x355C8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x39487FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x40013F, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x273, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x12, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};


static const comp_reg_entry_t comp1080p25 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x411B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21660000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x411B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21478000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x35258000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x39487FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x40013F, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x273, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x12, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};

static const comp_reg_entry_t comp1080p50 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x411B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21660000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20EF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x411B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21478000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x35258000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x39487FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x40013F, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x273, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x11, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};


static const comp_reg_entry_t comp1080p60 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20138000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30298000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x403F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x208A0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20138000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30298000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x403F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x34498000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x39487FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x40013F, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x273, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x11, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};


static const comp_reg_entry_t comp480i30 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20148001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30548001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20588001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20600001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20820001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x208D0001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x04000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20148000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31818000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21C18000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x332E8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20148000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30348000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21C18000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31E18000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20148000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30348000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21C18000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x332E8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20148001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30548001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20588001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20600001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20820001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20960001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x04000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20148000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31818000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21C18000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31E18000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20148000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30548000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20600000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20820001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20960000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21C18000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31E18000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20148000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30348000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20148002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30548000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20588000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20600006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207E8003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x74000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040BAD0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x395E7FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x3200F8, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x268, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000C1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000122, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002C4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020C0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E6, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002122, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002143, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002181, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021E2, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002207, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002384, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x13, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};



static const comp_reg_entry_t comp576i25 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20108002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30500000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x205D8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207C0001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20A00000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x04000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20108000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31810000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21C08000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33310000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20108000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30300000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21C08000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31E00000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20108000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31810000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21C08000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31E00000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20108002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30500000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x205D8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207C0001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20A00000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x04000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20108000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30300000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21C08000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33310000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20108000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30500000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x205D8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207C0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20A00000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x01B00000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21C08000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31E00000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20108002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30500000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x205D8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207C0001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20A00000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x04000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20108000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30300000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x2A098ACB, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0xF80BCD0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x285E7FFF, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x280133, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x268, 0xFFFFFFFF},
    //{DISP_HD_VENC0_YCMPN, 0x03C00093, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    //{DISP_HD_VENC0_CB2GAIN, 0x80450045, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A2, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000C4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002C0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E8, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000026C0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002702, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002725, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002761, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000027A2, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000027C9, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00004DC0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00004DE6, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00004E42, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x139, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x1, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x13, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};


/* Structure to contain the associated DirectFB settings and an appropriate VENC script ... */
typedef struct {
    DFBScreenEncoderScanMode            scan_mode;
    DFBScreenEncoderFrequency           frequency;
    DFBScreenOutputResolution           resolution;
    const comp_reg_entry_t              *venc_script;
    
} venc_table_entry_t;


int   Xcode4_SetUpComponentVenc(pxc_dfb_driver_data mdrv, struct display_mode_struct *pdisplay_mode, const DFBScreenOutputConfig *config);

#endif          //__XCODE4_COMP_VENC_H__
