/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_osd.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/


#include <config.h>

#include <dfb_types.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>


#include <directfb.h>

#include <direct/messages.h>
#include <direct/util.h>

#include <core/coredefs.h>
#include <core/coretypes.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>

#include <core/layers_internal.h>

#include <gfx/convert.h>
#include <gfx/util.h>

#include <misc/conf.h>

#include <math.h>

#include "viper.h"
#include "xcmmio.h"
#include "xc4fb_if.h"
#include "xcode4_2d.h"
#include "xcode4_osd.h"

void adjust_frequency_response(u32 start_address, pxc_dfb_driver_data mdrv);

void set_filter_for_vid0_nonscale(pxc_dfb_driver_data mdrv);

void set_filter_for_vid1_nonscale(pxc_dfb_driver_data mdrv);

extern void Set_Downscale_Vertical_Fes0_Coef(pxc_dfb_driver_data mdrv);
extern void Set_Downscale_Horizontal_Fes0_Coef(pxc_dfb_driver_data mdrv);

extern void Wait_For_Fifo_Entries(pxc_dfb_driver_data mdrv,
                           u32 status_reg, 
                           u32 free_ent_mask, 
						   u32 free_ent_shift,
                           u32 requested_fifo_space);

extern void set_fes0_coefficient_table(u32 base_address, double htable[][FES0_HORIZONAL_SCALER_TAPS], int upscaling_fractor_L, int ntaps, pxc_dfb_driver_data mdrv);
extern void set_fes1_coefficient_table(u32 base_address, double htable[][FES1_HORIZONAL_SCALER_TAPS], int upscaling_fractor_L, int ntaps, pxc_dfb_driver_data mdrv);

extern void Set_Fes1_Parameter(pxc_dfb_driver_data mdrv);

static double htablev_y[UPSAMPLING_FACTOR][VID0_VERTICAL_SCALER_TAPS];
static double htablev_cbcr[UPSAMPLING_FACTOR][VID0_VERTICAL_SCALER_TAPS];
static double htableh_y[UPSAMPLING_FACTOR][HORIZONAL_SCALER_TAPS];
static double htableh_cbcr[UPSAMPLING_FACTOR][HORIZONAL_SCALER_TAPS];

#ifdef FES_DRIVER

static double kaiser_256[FES_UPSAMPLING_FACTOR*FES1_VERTICAL_SCALER_TAPS];
static double kaiser_384[FES_UPSAMPLING_FACTOR*FES0_VERTICAL_SCALER_TAPS];
static double kaiser_640[FES_UPSAMPLING_FACTOR*FES0_HORIZONAL_SCALER_TAPS];
#endif

#define MAX_FACTORIAL   100
/* 
*   Since factorial invocations for the filter coefficient computations
*   are bounded to [0 to 99], we can used a fixed size table to greatly increase 
*   the performance.
*
*   If the above conditions is no longer true, the table method may need to be
*   dropped. However, this provides the biggest speed increase in the algorithm.
*
*/
static double factorial_table[MAX_FACTORIAL];

#define VECTOR_NUMBER 32

DFBResult
Xcode4InitAuxOSDFunctions(void)
{
    /* We can use this function to precompute the factorial tables (and other tables) */
    u32 i;
    double  f;

    /* Precompute factorial table */
    for (i = 0 ; i < MAX_FACTORIAL ; i++)
    {
        if (i == 0)
        {
            f = 1.;
        }
        else
        {
            f = ((double)(i)) * factorial_table[i - 1];
        }
        
        factorial_table[i] = f;
        //D_INFO("factorial[%u] = %lf\n", i, f);
    }
    
    return DFB_OK;
}                          


void CSC_ProgAdjustMatrix(pxc_dfb_driver_data mdrv, int layer_id)
{
    u32 tmp;

    if(layer_id >= dfb_layer_num())
    {
        return;
    }

    CoreLayer * layer = dfb_layer_at(layer_id);
    pxc_dfb_layer_data layer_data = (pxc_dfb_layer_data) layer->layer_data;

    if(!layer_data)
    {
        return;
    }
    
    DFBColorSpace layer_space = layer_data->color_space;
    DFBColorSpace screen_space = mdrv->shared->current_hd_color_space;

    tmp = REG_READ(DISP_HD_CC);
    switch(layer_id)
    {
        case XCODE4_GRA0_LAYER_ID:
            tmp &= ~DISP_HD_CC_GRA0_MODE_MASK;
            if(screen_space == DSCS_709)
            {
                tmp = tmp | (GRA_RGB255_TO_709 << DISP_HD_CC_GRA0_MODE_SHIFT);
            }
            else
            {
                tmp = tmp | (GRA_RGB255_TO_601 << DISP_HD_CC_GRA0_MODE_SHIFT);
            }
            break;
        case XCODE4_GRA1_LAYER_ID:
            tmp &= ~GRA1_MODE_MASK;
            if(screen_space == DSCS_709)
            {
                tmp = tmp | (GRA_RGB255_TO_709 << GRA1_MODE_SHIFT);
            }
            else
            {
                tmp = tmp | (GRA_RGB255_TO_601 << GRA1_MODE_SHIFT);
            }
            break;
        case XCODE4_VID0_LAYER_ID:
            tmp &= ~VID0_MODE_MASK;
            if(layer_space == DSCS_601 && screen_space == DSCS_709)
            {
                tmp = tmp | (YUV601_TO_709 << VID0_MODE_SHIFT);
            }
            else if(layer_space == DSCS_709 && screen_space == DSCS_601)
            {
                tmp = tmp | (YUV709_TO_601 << VID0_MODE_SHIFT);
            }
            else
            {
                tmp = tmp | (YUV_PASS_THROUGH << VID0_MODE_SHIFT);
            }
            break;
        case XCODE4_VID1_LAYER_ID:
            tmp &= ~VID1_MODE_MASK;
            if(layer_space == DSCS_601 && screen_space == DSCS_709)
            {
                tmp = tmp | (YUV601_TO_709 << VID1_MODE_SHIFT);
            }
            else if(layer_space == DSCS_709 && screen_space == DSCS_601)
            {
                tmp = tmp | (YUV709_TO_601 << VID1_MODE_SHIFT);
            }
            else
            {
                tmp = tmp | (YUV_PASS_THROUGH << VID1_MODE_SHIFT);
            }
            break;
        default:
            return;
    }
    REG_WRITE(DISP_HD_CC, tmp);

#if 0
    if(layer_id == XCODE4_VID0_LAYER_ID)
    {
        if (layer_space == DSCS_601 && screen_space == DSCS_709)
        {
            /* This will correct for the small error in the fixed function CC */
            REG_WRITE(DISP_HD_OCC_C11, 0x00002000);
            REG_WRITE(DISP_HD_OCC_C13, 0x02000000);
            REG_WRITE(DISP_HD_OCC_C21, 0x20000000);
            REG_WRITE(DISP_HD_OCC_C23, 0x10000000);
            REG_WRITE(DISP_HD_OCC_C31, 0xff7a0000);
            REG_WRITE(DISP_HD_OCC_C33, 0x1000200f);
        }
        else
        {
            REG_WRITE(DISP_HD_OCC_C11, 0x00002000);
            REG_WRITE(DISP_HD_OCC_C13, 0x02000000);
            REG_WRITE(DISP_HD_OCC_C21, 0x20000000);
            REG_WRITE(DISP_HD_OCC_C23, 0x10000000);
            REG_WRITE(DISP_HD_OCC_C31, 0x00000000);
            REG_WRITE(DISP_HD_OCC_C33, 0x10002000);
        }
    }
#endif

    //Follow Brad's setting in LG_BLURAY_4210-1367
    if(layer_id == XCODE4_VID0_LAYER_ID)
    {
        if(layer_space == DSCS_601 && screen_space == DSCS_709)
        {
            REG_WRITE(DISP_HD_OCC_C11, 0x00032000);
            REG_WRITE(DISP_HD_OCC_C13, 0x0200fffa);
            REG_WRITE(DISP_HD_OCC_C21, 0x20010000);
            REG_WRITE(DISP_HD_OCC_C23, 0x10000006);
            REG_WRITE(DISP_HD_OCC_C31, 0x00040000);
            REG_WRITE(DISP_HD_OCC_C33, 0x10002002);
        }
        else if(layer_space == DSCS_709 && screen_space == DSCS_601)
        {
            REG_WRITE(DISP_HD_OCC_C11, 0xfffd2000);
            REG_WRITE(DISP_HD_OCC_C13, 0x02000004);
            REG_WRITE(DISP_HD_OCC_C21, 0x1fff0000);
            REG_WRITE(DISP_HD_OCC_C23, 0x1000fffa);
            REG_WRITE(DISP_HD_OCC_C31, 0xfffc0000);
            REG_WRITE(DISP_HD_OCC_C33, 0x10001fff);
        }
        else
        {
            REG_WRITE(DISP_HD_OCC_C11, 0x00002000);
            REG_WRITE(DISP_HD_OCC_C13, 0x02000000);
            REG_WRITE(DISP_HD_OCC_C21, 0x20000000);
            REG_WRITE(DISP_HD_OCC_C23, 0x10000000);
            REG_WRITE(DISP_HD_OCC_C31, 0x00000000);
            REG_WRITE(DISP_HD_OCC_C33, 0x10002000);
        }
    }

    return;
}


/*****************************************************************************************************************
Function Name: convert_dfb_format_to_disp_format

Parameters:
           * config ->dfb core layer region config
           * disp_format -> color format of Xcode4 DISP graphics pipe
           

Description:
           convert dfb color format to Xcode4 DISP color format for graphic pipe

Returns:
           true or false

******************************************************************************************************************/

DFBResult convert_dfb_format_to_disp_format(const CoreLayerRegionConfig *config, 
                                            int * disp_format,
					    int * ignorePixelAlpha)
{
     switch (config->format) 
     {
          case DSPF_RGB32:
	      *disp_format = DISP_HD_GRA0_FORMAT_ARGB;
              *ignorePixelAlpha = 1;
	      /* Tell upper levels to ignore pixel alpha values */	
	      break;
          case DSPF_ARGB:
              *disp_format = DISP_HD_GRA0_FORMAT_ARGB;
              *ignorePixelAlpha = 0;
	      break;

          case DSPF_ARGB1555:
	      *ignorePixelAlpha = 0;
	      *disp_format = DISP_HD_GRA0_FORMAT_RGB1555;
	      break;
	    
	  case DSPF_RGB555:
              *disp_format = DISP_HD_GRA0_FORMAT_RGB1555;
              *ignorePixelAlpha = 1;
	      /* The above line tells all levels above to ignore the
	         the alpha value that is specified in the pixel value 
	      */
	      break;

          case DSPF_RGB16:
              *disp_format = DISP_HD_GRA0_FORMAT_RGB565;
              *ignorePixelAlpha = 0;
	      break;

          case DSPF_RGB24:
              *disp_format = DISP_HD_GRA0_FORMAT_RGB24;
              *ignorePixelAlpha = 0;
	      break;

          case DSPF_LUT8:
              *disp_format = DISP_HD_GRA0_FORMAT_CLUT8;
	      *ignorePixelAlpha = 0;
              break;

          case DSPF_LUT2:
              *disp_format = DISP_HD_GRA0_FORMAT_CLUT2;
              *ignorePixelAlpha	=   0;
	      break;

          default:
              D_BUG( "unexpected pixelformat! %08x \n", config->format );
              return DFB_UNSUPPORTED;
     }

     return DFB_OK;
}

/*****************************************************************************************************************
Function Name: Calculate_gfx_hsca_inv_ratio

Parameters:
           input_h_size -> input horizonal size
           output_h_size -> output horizonal size
           

Description:
           calculate the horizonal scaler invert ratio for graphic pipe

Returns:
           horizonal invert ratio

*****************************************************************************************************************/

u32 Calculate_gfx_hsca_inv_ratio(u32 input_h_size, u32 output_h_size)
{
    double hsca_ratio;
    u32 hsca_inv_ratio;

    hsca_ratio = ((double)input_h_size) / ((double)output_h_size);

    hsca_inv_ratio = (int) (hsca_ratio * (1 << DISP_HD_INV_RATIO_SHIFT) + 0.5);

    return hsca_inv_ratio;
}

/*****************************************************************************************************************
Function Name: Calculate_gfx_vsca_inv_ratio

Parameters:
           scanmode -> indicated if the output signal is interlace
           input_v_size -> input vertical size
           output_v_size -> output vertical size
           * gra_vsca_inv_ratio -> vertical scaler invert ratio for graphics pipe
           * gra_vsca_init -> init position of vertical scaler, graphic pipe
           * gra_vsca_start -> vertical scaler start pos for graphic pipe

Description:
           calculate the vertical scaler invert ratio, vertical scaler start pos for graphic pipe

Returns:
           void
*****************************************************************************************************************/

void Calculate_gfx_vsca_inv_ratio
           (DFBScreenEncoderScanMode scanmode ,
            u32 input_v_size, 
            u32 output_v_size,
            u32 * gra_vsca_inv_ratio,
            u32 * gra_vsca_init,
            u32 * gra_vsca_start)
{

    double vsca_ratio;
    u32 vsca_inv_ratio;
    double y00, y01;
    u32 y00_int, y01_int;
    u32 b00, b01;
    

    vsca_ratio = ((double)input_v_size) / ((double)output_v_size);

    y00 = 1.0;
    y01 = y00;

    y00_int = 0;
    y01_int = 0;
    
    y01 = y01 + vsca_ratio;

    if(y00 >= 1)
    {
      y00_int = (int) (y00);
      y00 = y00 - y00_int;
    }

    if(y01 >= 1)
    {
      y01_int = (int) (y01);
      y01 = y01 - y01_int;
    }

    *gra_vsca_inv_ratio = (int) (vsca_ratio * (1 << DISP_HD_INV_RATIO_SHIFT) + 0.5);

    b01 = (int) (y01 * 256 + 0.5);
    b00 = (int) (y00 * 256 + 0.5);

    if (scanmode == DSESM_INTERLACED )
    {
        *gra_vsca_init = (b01 << DISP_HD_GRA0_VSCA_INIT_POSREG_INIT1_SHIFT) | b00;
        *gra_vsca_start = (y01_int << DISP_HD_GRA0_VSCA_START_START_POS1_SHIFT) | y00_int;
    }
    else
    {
       *gra_vsca_init = DEFAULT_GRAPHIC_VSCA_INIT;
       *gra_vsca_start = DEFAULT_GRAPHIC_VSCA_START;
    }


    return ;
}

/*****************************************************************************************************************

Function Name: Calculate_video_sca_inv_ratio

Parameters:
           * mode -> output video mode pointer
           * config -> input video layer config pointer
           * video_vsca_inv_ratio -> vertical scaler invert ratio for video pipe
           * video_hsca_inv_ratio -> horizonal scaler invert ratio for video pipe
           * vid_vsca_init0 -> video scaler init0
           * vid_vsca_init1 -> video scaler init1

Description:
           calculate the vertical scaler invert ratio, vertical scaler start pos for graphic pipe

Returns:
           void

*****************************************************************************************************************/

void Calculate_video_sca_inv_ratio(struct display_mode_struct * mode, //output video mode pointer
                                   CoreLayerRegionConfig    * config, //input video 
                                   u32 * video_vsca_inv_ratio,
                                   u32 * video_hsca_inv_ratio,
                                   u32 * vid_vsca_init0,
                                   u32 * vid_vsca_init1,
                                   u32 * vid_vsca_start,
                                   u32 * vid_hsca_init,
                                   u32 * vid_hsca_start,
                                   u32 x_round_down)
{
    double vratio;
    double hratio;
    double y_default = 4.0;
    double cbcr_default = 3.75;
    double y00, y01, y10, y11;
    double cbcr00, cbcr01, cbcr10, cbcr11;
    int y00_int, y01_int, y10_int, y11_int;
    int cbcr00_int, cbcr01_int, cbcr10_int, cbcr11_int;
    double y00_fract, y01_fract, y10_fract, y11_fract;
    double cbcr00_fract, cbcr01_fract, cbcr10_fract, cbcr11_fract;
    u32 hsca_luma_start;
    u32 hsca_chroma_start;
    u32 start_pos;

    //calculate the vertical scale invert ratio

    vratio = ((double) config->source.h) / ((double)config->dest.h);
    * video_vsca_inv_ratio = ((vratio * (1 << DISP_HD_INV_RATIO_SHIFT)) + 0.5);

    //calculate the horizonal scale invert ratio
    hratio = ((double) config->source.w) / ((double)config->dest.w);
    * video_hsca_inv_ratio = ((hratio * (1 << DISP_HD_INV_RATIO_SHIFT)) + 0.5);

    // Set defaults here for vscaler
    if ((config->surface_caps & DSCAPS_VIXS_INTERLACED) ||
	    (config->surface_caps & DSCAPS_SEPARATED) )
    {
        y00 = y_default;
        cbcr00 = cbcr_default + 0.125;
        y10 = y_default - 0.5;
        cbcr10 = cbcr_default - 0.5 - 0.125;
        if(mode->scanmode == DSESM_INTERLACED)
        {
            y01 = y00;
            cbcr01 = cbcr00;
            y11 = y10;
            cbcr11 = cbcr10;
            
            y01 = y01 + vratio/2;
            cbcr01 = cbcr01 + vratio/4;
            y11 = y11 + vratio/2;
            cbcr11 = cbcr11 + vratio/4;
        }
        else
        {
            y01 = 0;
            cbcr01 = 0;
            y11 = 0;
            cbcr11 = 0;
        }
    }
    else
    {
        y00 = y_default;
        cbcr00 = cbcr_default;
        y10 = 0;
        cbcr10 = 0;
        if(mode->scanmode == DSESM_INTERLACED)
        {
            y01 = y00;
            cbcr01 = cbcr00;
            y11 = y10;
            cbcr11 = cbcr10;
            
            y01 = y01 + vratio;
            cbcr01 = cbcr01 + vratio/2;
            y11 = y11 + vratio;
            cbcr11 = cbcr11 + vratio/2;
        }
        else
        {
            y01 = 0;
            cbcr01 = 0;
            y11 = 0;
            cbcr11 = 0;
        }
    }

    y00_int = (int)(y00);
    y01_int = (int)(y01);
    y10_int = (int)(y10);
    y11_int = (int)(y11);
    cbcr00_int = (int)(cbcr00);
    cbcr01_int = (int)(cbcr01);
    cbcr10_int = (int)(cbcr10);
    cbcr11_int = (int)(cbcr11);
    
    y00_fract = y00 - (int)(y00);
    y01_fract = y01 - (int)(y01);
    y10_fract = y10 - (int)(y10);
    y11_fract = y11 - (int)(y11);
    cbcr00_fract = cbcr00 - (int)(cbcr00);
    cbcr01_fract = cbcr01 - (int)(cbcr01);
    cbcr10_fract = cbcr10 - (int)(cbcr10);
    cbcr11_fract = cbcr11 - (int)(cbcr11);
    
    
    int b3 = (int)(cbcr01_fract*256+0.5);
    int b2 = (int)(y01_fract*256+0.5);
    int b1 = (int)(cbcr00_fract*256+0.5);
    int b0 = (int)(y00_fract*256+0.5);
    *vid_vsca_init0 = b0 |
                      (b1 << DISP_HD_VID0_VSCA_INIT0_POSREG_INIT_CBCR00_SHIFT) |
                      (b2 << DISP_HD_VID0_VSCA_INIT0_POSREG_INIT_Y01_SHIFT) |
                      (b3 << DISP_HD_VID0_VSCA_INIT0_POSREG_INIT_CBCR01_SHIFT);
    
    b3 = (int)(cbcr11_fract*256+0.5);
    b2 = (int)(y11_fract*256+0.5);
    b1 = (int)(cbcr10_fract*256+0.5);
    b0 = (int)(y10_fract*256+0.5);
    * vid_vsca_init1 = b0 |
                       (b1 << DISP_HD_VID0_VSCA_INIT1_POSREG_INIT_CBCR10_SHIFT) |
                       (b2 << DISP_HD_VID0_VSCA_INIT1_POSREG_INIT_Y11_SHIFT) |
                       (b3 << DISP_HD_VID0_VSCA_INIT1_POSREG_INIT_CBCR11_SHIFT);

    //printf ("RegWrite (%s_VSCA_START, 0x%x%x%x%x%x%x%x%x);\n", prefix.c_str(), 
    //cbcr11_int,cbcr01_int,y11_int,y01_int,cbcr10_int,cbcr00_int,y10_int,y00_int);
    * vid_vsca_start = y00_int |
                       (y10_int << DISP_HD_VID0_VSCA_START_START_POS10_SHIFT) |
                       (cbcr00_int << DISP_HD_VID0_VSCA_START_START_CBCR_POS00_SHIFT) |
                       (cbcr10_int << DISP_HD_VID0_VSCA_START_START_CBCR_POS10_SHIFT) |
                       (y01_int << DISP_HD_VID0_VSCA_START_START_POS01_SHIFT) |
                       (y11_int << DISP_HD_VID0_VSCA_START_START_POS11_SHIFT) |
                       (cbcr01_int << DISP_HD_VID0_VSCA_START_START_CBCR_POS01_SHIFT) |
                       (cbcr11_int << DISP_HD_VID0_VSCA_START_START_CBCR_POS11_SHIFT);

    * vid_hsca_init = 0x00000000;

    if (START_POS_OUT_OF_RANGE)
    {
        start_pos = ((config->source.x - x_round_down) + DISP_INIT_POS) - VIDEO_LAYER_LUMA_ADDR_ALIGNMENT; 
    }
    else
    {
        start_pos = config->source.x - x_round_down;
    }

    hsca_luma_start = DISP_INIT_POS + start_pos;
    hsca_chroma_start = DISP_INIT_POS + (start_pos/2);

    * vid_hsca_start = hsca_luma_start | (hsca_chroma_start << DISP_HD_VID0_HSCA_START_START_CBCR_POS_SHIFT);

    return ;
}
    
    
/*****************************************************************************************************************
Function Name: create_coefficient_table

Parameters:
           * upsampling_factor_L -> upsampling factor
           * ntaps -> tap number for interpolation
           * size_in -> input size
           * size_out -> output size
           * htable -> array to save the scaler coefficient

Description:
           calculate the scaler filter coefficient 

Returns:
           void
*****************************************************************************************************************/



void create_coefficient_table(int upsampling_factor_L, int ntaps, int size_in, int size_out, double htable[][HORIZONAL_SCALER_TAPS], int srckey)
{
    double rho = ((double)size_out) / ((double)size_in);
    int total_ir = upsampling_factor_L * ntaps;
    double h[total_ir];
    double x;
    double step;
    int i;
    int j;
    int k;
	int lanczos_alpha;
	int lanczos_pos_gate;
	int lanczos_neg_gate;
	double sample_point ;
	double dc_part;

    if (ntaps > VID1_VERTICAL_SCALER_TAPS)
	{
	    lanczos_alpha = 3;
	}
	else
	{
	    lanczos_alpha = 2;
	}

	lanczos_pos_gate = lanczos_alpha * upsampling_factor_L;
	lanczos_neg_gate = 0 - lanczos_pos_gate ;

    if(rho == 0)
        step = rho / upsampling_factor_L;
    else
        step = 1./upsampling_factor_L;
    
    if(rho == 0)
    {
        // sinc
        D_DEBUG( "make sinc function \n" );
        for(i = 0; i < total_ir; i++)
        {
            x = (i-ntaps/2.* upsampling_factor_L) * step*PI;
        
            if(x == 0) 
                h[i] = 1;
            else 
                h[i] = sin(x)/x;
        }
    }
    else if(srckey)
    {
        // zoh
        D_DEBUG( "make zoh function \n" );
        for(i = 0; i < total_ir; i++)
        {
            if ((i - total_ir/2) < (-upsampling_factor_L/2))
                h[i] = 0;
            else if ((i - total_ir/2) >= (upsampling_factor_L/2)) 
                h[i] = 0;
            else 
                h[i] = 1;

        }
    }
    else
    {
        // Lanczos
        D_DEBUG( "make Lanczos function \n" );
        for(i = 0; i < total_ir; i++)
        {
            x = (i-ntaps/2.*upsampling_factor_L)*step*PI;
			sample_point = i- ntaps/2*upsampling_factor_L;

            if(sample_point == 0) 
			{
                h[i] = 1;
			}
            else if (sample_point < lanczos_neg_gate) 
			{
                h[i] = 0;
			}
            else if (sample_point > lanczos_pos_gate) 
			{
                h[i] = 0;
			}
            else
            { 
                h[i] = sin(x)/x*sin(x/lanczos_alpha)/(x/lanczos_alpha);
			}
        }
    }
    
    // create table
    for(i = 0; i < upsampling_factor_L; i++)
    {
        for(j = 0; j < ntaps; j++)
        {
            k = upsampling_factor_L*j + i;
            htable[i][j] = h[k];
        }
    }

    // Normalize each phase to 1.0
    for(i = 0; i < upsampling_factor_L; i++)
    {
        dc_part = 0;
        for(j = 0; j < ntaps; j++)
        {
            dc_part += htable[i][j];
        }
        for(j = 0; j < ntaps; j++)
        {
            htable[i][j] = htable[i][j]/dc_part;
        }
    }
}

/*****************************************************************************************************************
Function Name: set_coefficient_disp_ram_table

Parameters:
           * base_address -> base address of HD RAM	table
           * htable -> coefficient table
           * upscaling_fractor_L -> upsampling factor
           * ntaps -> tap number for interpolation
           * mdrv -> xcode4 driver pointer

Description:
           calculate the scaler filter coefficient 

Returns:
           void
*****************************************************************************************************************/
void set_coefficient_disp_ram_table(u32 base_address, double htable[][HORIZONAL_SCALER_TAPS], int upscaling_fractor_L, int ntaps, pxc_dfb_driver_data mdrv)
{
  
    int Lmirrored = upscaling_fractor_L/2+1;
    u32 u12_coef_value;
    u32 u4_coef_value[VECTOR_NUMBER];
    u32 u4_index;
    //u32 total_hex;
    //u32 attach_zero_length;
    u32 reg_addr;
    u32 u32_coef;
	u32 u4_coef_count;
	u32 t;
    int i, j, k;

    

    //set the base address of scaler
    REG_WRITE_AND_POLL( DISP_HD_RAM_ADDR, base_address);
    
    for(i = 0; i < Lmirrored; i++)
    {

        u4_index = 0;
		memset (&u4_coef_value[0], 0, sizeof(u32) * VECTOR_NUMBER);

        for(j = 0; j < ntaps; j++)
        {
            // rounding to the u12 value
            if(htable[i][j] < 0)
            {
                u12_coef_value = (int)(htable[i][j] * pow(2,10)-0.5);
                if (u12_coef_value < (-pow(2,11)))
                {
                    u12_coef_value = (int) (-pow(2.,11.));
                }
            }
            else
            {
                u12_coef_value = (int)(htable[i][j] * pow(2,10)+0.5);
                if(u12_coef_value >= pow(2,11))
                {
                    u12_coef_value = (int)(pow(2.,11.)-1);
                }

            }

            u12_coef_value = u12_coef_value & 0xfff;
            u4_coef_value[u4_index++] = u12_coef_value & 0x0f;
            u4_coef_value[u4_index++] = (u12_coef_value & 0x0f0) >> 4;
            u4_coef_value[u4_index++] = (u12_coef_value & 0x0f00) >> 8;
        }

        u4_coef_count = ceil((ntaps*3)/8.)*8;
		/*
        total_hex = ceil((ntaps*3)/8.)*8;

        attach_zero_length = total_hex-ntaps*3;
        for(k = 0; k < attach_zero_length; k++)
        {
            u4_value[u4_index++] = 0;
        
        }

        u4_coef_count = total_hex;
		*/
        
        for( u4_index = u4_coef_count; u4_index > 0; u4_index = u4_index - 8)
        {
            u32_coef = 0;
            for (k = u4_index; k > (u4_index - 8); k --)
        	{
                u32_coef = (u32_coef << 4 ) | u4_coef_value[k-1];
        
        	}
        
            t = u4_index / 8 - 1;
        
		    switch (t)
			{
			    case 0:
				    reg_addr = DISP_HD_RAM_WDATA;
					break;

			    case 1:
				    reg_addr = DISP_HD_RAM_WDATA1;
					break;

			    case 2:
				    reg_addr = DISP_HD_RAM_WDATA2;
					break;

			    case 3:
				    reg_addr = DISP_HD_RAM_WDATA3;
					break;

			    default:
				    reg_addr = DISP_HD_RAM_WDATA;
					break;
			}

            REG_WRITE_AND_POLL (reg_addr, u32_coef);
        }
    }

    return;
}

/*****************************************************************************************************************
Function Name: my_round

*****************************************************************************************************************/
double my_round(double x, int N, int I)
{
    int h;
    int F = N - 1 - I;
    unsigned int    mF;
    unsigned int    mNm1;
    
    double   fF;
    double   fNm1;
    /* 
    *   We can replace the pow(2, x) functions with 1 << x,
    *   provided that x is bounded in 32-bits 
    */
    mF      =   (1 << F);
    mNm1    =   (1 << (N-1));
    
    fF      = (double)mF;
    fNm1    = (double)mNm1;

    if(x < 0)
    {
        h = (int)(x*pow(2.,F)-0.5);
        if(h < -pow(2.,N-1))
        {
            h = (int)(-pow(2.,N-1));
        }
    }
    else 
    {
        h = (int)(x*pow(2.,F)+0.5);
        if(h >= pow(2., N-1))
        {
            h = (int)(pow(2.,N-1)-1);
        }
    }
    
    /* h * pow(2, -F) = h * 1/(2^F) */
    return (h*pow(2.,-F));
}

double factorial(double m)
{
  //if(m>0)
  //  return (m*factorial(m-1));
  //else
  //  return (1);
  return factorial_table[((int)m)];
}

double bessel (double x)
{
  double I = 0;
  double num;
  double den;
  int m;
  for( m = 0; m < 100; m++)
  {
    num = pow(x/2.,2.0*m);
    //den = factorial(m)*gamma_function(m+1);
    den = factorial(m)*factorial(m);
    I+=num/den;
  }
  return (I);
}

double kaiser(int32_t N, double beta, int32_t n)
{
  int32_t M = N-1;
  double alpha = M/2.;
  double num = bessel(beta*sqrt((1-pow((n-alpha)/alpha,2.0))));
  double den = bessel(beta);
  double ret = num/den;
  return ret;
}

void preset_CoefParams(scaleCoefParam * CoefParams_ptr)
{
    if(CoefParams_ptr)
    {
        CoefParams_ptr->type = LANCZOS3_FILTER;
        CoefParams_ptr->lpf = 1.0;
        CoefParams_ptr->beta = 0;
        CoefParams_ptr->lpf0 = 1.0;
        CoefParams_ptr->rho = 1.0;
    }
}

void assign_scaler_autotype(int32_t ntaps, scaleCoefParam * coefParams_ptr, int srckey)
{

  if(coefParams_ptr)
  {
      if(srckey)
      {
          coefParams_ptr->type = ZOH_FILTER;
      }
      else if (ntaps < 4)
      {
        coefParams_ptr->type = LINEAR_FILTER;
      }
      else if(ntaps < 6)
      {
        coefParams_ptr->type = LANCZOS2_FILTER;
      }
      else
      {
        coefParams_ptr->type = LANCZOS3_FILTER;
      }
  }
}

void create_coefficient_table_version_2 (void     *layer_data,
                                         u32 upsampling_factor_L,
                                         double htable[][HORIZONAL_SCALER_TAPS], 
                                         u32 maxtaps, 
                                         u32 ntaps, 
                                         bool interlaced_in, bool interlaced_out, double premult,  
                                         scaleCoefParam * coefParams, 
                                         double soften,
                                         double sharpen)
{
  pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;  
  int total_ir = upsampling_factor_L*(ntaps);
  double hint[total_ir];
  double gint[total_ir];
  double ggint[total_ir * 2];
  double gain;
  double *g;
  double *h;
  
  
  double x;
  double w;
  double step;
  double fc;
  u32 i, j, k;
  double lanczos_alpha;
  double lanczos_pos_gate;
  double lanczos_neg_gate;
  double sample_point ;
  
  double dch = 0;
  double dch_recip  =   0;
  
  double dcg = 0;
  double dcg_recip  =   0;
  
  double sumr = 0, sumi = 0;
  double norm, lin;
  double f0, stepbpf;
  
  /* 
  * This function seems to be in use for computation of both FES and DISP related
  * scaling co-efficients, as such we cannot completely remove the internal
  * g and h double float arrays.
  *
  */
  if (layer_data)
  {
    g   =   (double *)&m_layer_data->g;
    h   =   (double *)&m_layer_data->h;
  }
  else
  {
    g   =   (double *)&gint;
    h   =   (double *)&hint;
  }
  
    if (coefParams->type != ZOH_FILTER)
    {
      if(coefParams->rho <= 1)
        fc = sqrt(soften) * coefParams->lpf * coefParams->lpf0 * coefParams->rho;
      else
        fc = sqrt(soften) * coefParams->lpf * coefParams->lpf0 * 1.;
    }
    else
    {
        fc = coefParams->lpf * coefParams->lpf0 * 1.;
    }
  step = fc / upsampling_factor_L;
  
  /* DC sum value will be calculated in-loop */
  dch = 0;
  
  if(coefParams->type == RANDOM_FILTER)
  {
    // random
    D_DEBUG( "Making random coefficients \n");
    for(i = 0; i < total_ir; i++)
    {
      double r = (double)rand()/(double)(RAND_MAX);
      r = (r-0.5)*2;  // +/-1
      h[i] = 2*r;
      dch += h[i];
    }
  }
  else if(coefParams->type == SINC_FILTER)
  {
    // sinc
    D_DEBUG ("Making windowed sinc function\n");
    for(i = 0; i < total_ir; i++)
    {
      x = (i-total_ir/2.)*step*PI;
      w = kaiser(total_ir+1,coefParams->beta,i);
      if(i == total_ir/2.) h[i] = 1*w;
      else h[i] = sin(x)/x * w;
      
      dch +=h[i];
    }
  }
  else if(coefParams->type == ZOH_FILTER)
  {
    // Zero-order hold
    D_DEBUG( "Making zero-order hold function \n");
    for( i = 0; i < total_ir; i++)
    {
      x = (i-total_ir/2.)*step;
      if (i-total_ir/2 < 0) h[i] = 0;
      else if (i-total_ir/2 >= +2/2*upsampling_factor_L) h[i] = 0;
      else h[i] = 1;
      
      dch +=h[i];
    }
  }
  else if(coefParams->type == LINEAR_FILTER)
  {
    // Linear
    D_DEBUG ("Making Linear function, \n");
    for(i = 0; i < total_ir; i++)
    {
      x = (i-total_ir/2.)*step;
      if (i-total_ir/2 < -2/2*upsampling_factor_L) h[i] = 0;
      else if (i-total_ir/2 > +2/2*upsampling_factor_L) h[i] = 0;
      else h[i] = 1-fabs(x);
      
      dch +=h[i];
    }
  }
  else if((coefParams->type == LANCZOS2_FILTER) || (coefParams->type == LANCZOS3_FILTER))
  {
    int l;
    if(coefParams->type == LANCZOS2_FILTER) 
    {
        lanczos_alpha = 2;
    }
    else if (coefParams->type == LANCZOS3_FILTER)
    {
        lanczos_alpha = 3;
    }

	lanczos_pos_gate = lanczos_alpha * upsampling_factor_L;
	lanczos_neg_gate = 0 - lanczos_pos_gate ;

    // Lanczos2 or Lanczos3
    D_DEBUG ( "Making Lanczos\n");

    for(i = 0; i < total_ir; i++)
    {
      x = (i-(total_ir/2.))*step*PI;

      sample_point = i- (total_ir/2.);

      if(sample_point == 0) 
      {
          h[i] = 1;
          dch +=h[i];
      }
      else if (sample_point < lanczos_neg_gate) 
      {
          h[i] = 0;
      }
      else if (sample_point > lanczos_pos_gate) 
      {
          h[i] = 0;
      }
      else
      { 
          h[i] = sin(x)/x*sin(x/lanczos_alpha)/(x/lanczos_alpha);
          
          dch +=h[i];
      }

      //printf("the h[i] is %f \n", h[i]);
    }
  }
  else
  {
      D_ERROR( "Error: Unknown filter type \n");
      return;
  }

 
  //bool normalize = 1;
  
  /* 
  * Seems like this will sum a bandpass filter with a centre freq (of 3fc/4) with
  * the already created low pass filter 
  */
  
  dch_recip   =   1./dch;
  
  if (sharpen > 0 && coefParams->type != ZOH_FILTER)
  {
    
    if (coefParams->rho <= 1.)
    {
        f0 = 1/2. * fc;
        stepbpf =   fc / upsampling_factor_L;
    }
    else
    {
        f0      = 1/2. ;
        stepbpf =   1./upsampling_factor_L;
    }
    
    for (i = 0 ; i < total_ir * 2 ; i++)
    {
        x = (i-total_ir*2/2.)*PI;
        //w = kaiser(total_ir+1,coefParams->beta, i);
        w = 1;
        //w = kaiser(total_ir+1, 4, i);
        if (i == total_ir*2/2.) ggint[i] = 1*w;
        else ggint[i] = sin(x*stepbpf*0.25*0.5)/(x*stepbpf*0.25*0.5) * w * cos(x*f0*stepbpf*0.5);
        /* Cutoff is 0.25x of LPF */
    }
    
    for (i = 0 ; i < total_ir*2 ; i++)
    {
        //Transfer odd samples over
        if ((i % 2) == 1) g[i/2] = ggint[i];
    }
    //Find max gain of bandpass filter, which is at f0
    sumr = 0;
    sumi = 0;
    
    for (i = 0 ; i < total_ir ; i++)
    {
        x = (i-total_ir/2.)*f0/upsampling_factor_L*PI;
        sumr = sumr + g[i] * cos(x);
        sumi = sumi + g[i] * sin(x);
    }
    
    norm = sqrt(sumr*sumr + sumi*sumi);
    lin = pow(10., sharpen/20);
    
    if (lin > 2) lin = 2;
    gain = (lin-1)/norm;
    
    //if (m_layer_data)
    //{
    //    m_layer_data->mulfg =   (lin - 1);
    //    /* Note that this */
    //}
    
    dcg =   0;
    for (i = 0 ; i < total_ir ; i++)
    {
        g[i] = g[i] * gain;
        g[i] = (h[i] * dch_recip) + g[i]; 
        
        dcg += g[i];
    }

    dcg_recip   =   upsampling_factor_L / (dcg );
    
    for (i = 0 ; i < total_ir ; i++)
    {
        //g[i] = g[i] *gain;
        g[i] =  (g[i] * dcg_recip) ;
    }
  }
  
  /* 
  * At this point:
  *
  * -h = Un-normalized LPF filter co-efficients (no sharpness)
  * -g = Un-normalized LPF+BPF filter co-efficients (sharpness applied)
  * 
  * dcg = DC response of filter (for LPF+BPF)
  * dch = DC response of filter (for BPF)
  *
  * dcg_recip = Reciprocal of DC response for G
  * dch_recip = Reciprocal of DC response for H
  *
  *
  * NOTE: LPF only filter does not need its co-efficients to be normalized. 
  * Only the LPF + BPF filter needs to be normalized
  *
  *
  */
  // create table
  for( i = 0; i < upsampling_factor_L; i++)
  {
    for( j = 0; j < ntaps; j++)
    {
      k = upsampling_factor_L * j + i;
      //htable[i][j] = round(h[k],12,1);
      if (sharpen > 0)  htable[i][j] = g[k];
      else              htable[i][j] = h[k];
    }
    
    
    for( j = ntaps; j < maxtaps-ntaps; j++)
    {
      htable[i][j] = 0;
    }
  }
  
  // Normalize each phase to 1.0
 
    for(i = 0; i < upsampling_factor_L; i++)
    {
      double dc = 0;
      for(j = 0; j < ntaps; j++)
      {
        //dc += htable[i][j];
        dc += my_round(htable[i][j],12,1);
      }
      for(j = 0; j < ntaps; j++)
      {
        htable[i][j] = htable[i][j]/dc;
      }
    }
 

  // Multiplier
  for(i = 0; i < upsampling_factor_L; i++)
  {
    for(j = 0; j < ntaps; j++)
    {
      htable[i][j] = htable[i][j] * premult;
    }
  }

  // Put in fixed point
  for( i = 0; i < upsampling_factor_L; i++)
  {
    for( j = 0; j < ntaps; j++)
    {
      htable[i][j] = my_round(htable[i][j],12,1);
    
      //printf("the htable[i][j] is %f \n", htable[i][j]);

      //if(i == 0) cout << htable[i][j] << " " << endl;
    }
  }
}



DFBResult
Xcode4InitLayerScaleParms(void    *driver_data,
                          void    *layer_data            
)
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    u32 i;
    
    for (i = 0 ; i < VIDEO_SCL_MAX_TAPS ; i++)
    {
        /* Clear the filter impulse response h = scale, g = (scale + sharpness) */
        m_layer_data->g[i]  =   0;
        m_layer_data->h[i]  =   0;
    }
    
    m_layer_data->vY.type  =   LANCZOS3_FILTER;
    m_layer_data->vY.lpf   =   1.0;
    m_layer_data->vY.beta  =   0;
    m_layer_data->vY.lpf0  =   1.0;
    m_layer_data->vY.rho   =   1.0;
    
    m_layer_data->vC.type  =   LANCZOS3_FILTER;
    m_layer_data->vC.lpf   =   1.0;
    m_layer_data->vC.beta  =   0;
    m_layer_data->vC.lpf0  =   1.0;
    m_layer_data->vC.rho   =   1.0;
    
    
    m_layer_data->hY.type  =   LANCZOS3_FILTER;
    m_layer_data->hY.lpf   =   1.0;
    m_layer_data->hY.beta  =   0;
    m_layer_data->hY.lpf0  =   1.0;
    m_layer_data->hY.rho   =   1.0;
    
    m_layer_data->hC.type  =   LANCZOS3_FILTER;
    m_layer_data->hC.lpf   =   1.0;
    m_layer_data->hC.beta  =   0;
    m_layer_data->hC.lpf0  =   1.0;
    m_layer_data->hC.rho   =   1.0;
    
    return DFB_OK;
}
/*****************************************************************************************************************
Function Name: Xcode4SetScalerRamTable

Parameters:
           * driver_data -> pointer to DISP register spaces
           * config -> display layer patameter (for input)
           * mode -> output display mode table
           chroma_in -> input video encoding format

Description:
           Set Scaler Ram table for video pipe, using for scaler/filter 

Returns:   DFB_OK if it's successful
           
*****************************************************************************************************************/
DFBResult
Xcode4SetScalerRamTable(void                       *driver_data,
                        void                       *layer_data,
                        CoreLayerRegionConfig      *config,
                        struct display_mode_struct   * mode,
                        u32 * disp_hd_vid_register_table,
                        DFBSurfacePixelFormat chroma_in,
                        colorModeParameters *colorParms)
{
    double vmult;
    double hmult;
    double seconds;
    
    /* Soften factors */
    double vCsoften, vYsoften;
    double hCsoften, hYsoften;
    
    /* Sharpen factors */
    double vCsharpen, vYsharpen;
    double hCsharpen, hYsharpen;
   
    
    /* Premultiply factors */
    double vCpremult, vYpremult;
    double hCpremult, hYpremult;
    
    /* Upscaling taps (I guess for downscale, we always use all taps?) */
    u32    y_upscale_taps;
    u32    c_upscale_taps;
    
    u32    start_time, end_time;
    
    
    vCpremult   =   vYpremult   =   1.0f;
    
    if (colorParms)
    {
        hCpremult   =   colorParms->premultc;
        hYpremult   =   colorParms->premulty;
        
        //vSoften     =   colorParms->vsoften;
        //hSoften     =   colorParms->hsoften;
        
        y_upscale_taps  =   colorParms->ntaps_up_y;
        c_upscale_taps  =   colorParms->ntaps_up_c;

        vYsharpen        =   colorParms->vysharpen;
        hYsharpen        =   colorParms->hysharpen;
        vCsharpen        =   colorParms->vcsharpen;
        hCsharpen        =   colorParms->hcsharpen;
        
        vYsoften        =   colorParms->vysoften;
        hYsoften        =   colorParms->hysoften;
        vCsoften        =   colorParms->vcsoften;
        hCsoften        =   colorParms->hcsoften;
        
        if (y_upscale_taps > 10)    y_upscale_taps  =   10;
        if (c_upscale_taps > 10)    c_upscale_taps  =   10;
    }
    else
    {
        /* Defaults (i.e no modification) */
        hCpremult   =   hYpremult   =   1.0f;
        
        vCsharpen    =   0.0f; 
        hCsharpen    =   0.0f;
        vYsharpen    =   0.0f; 
        hYsharpen    =   0.0f;
        
        
        vCsoften     =   1.0f; 
        hCsoften     =   1.0f;
        vYsoften     =   1.0f; 
        hYsoften     =   1.0f;
        y_upscale_taps  =   c_upscale_taps  =   10;
    }
    
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    pxc_dfb_layer_data m_layer_data = (pxc_dfb_layer_data) layer_data;
    
    int srckey = ((config->options & DLOP_SRC_COLORKEY) &&
                   (config->source.w >= config->dest.w) &&
                   (config->source.h >= config->dest.h));
    
    start_time  =   REG_READ(CG_SYSTEM_CNT);
    
     
    if (Dfb_Format_420(chroma_in))
    {
        vmult = 2;
        hmult = 2;
    }
    else if (Dfb_Format_422(chroma_in))
    {
        vmult = 1;
        hmult = 2;
    }
    else
    {
        D_DEBUG ("Error in chroma types \n");
        return DFB_UNSUPPORTED;
    }

    if ((Dfb_Format_420(chroma_in)) &&
        (config->source.w == config->dest.w) &&
        (config->source.h == config->dest.h) &&
        (!srckey) &&
        (!colorParms)
        )
    {
        if (disp_hd_vid_register_table[DISP_HD_VID_FORMAT_INDEX] == DISP_HD_VID0_FORMAT)
        {
            set_filter_for_vid0_nonscale(mdrv);
            adjust_frequency_response(disp_hd_vid_register_table[DISP_HD_VID_HSCALER_CBCR_BASE_ADDRESS_INDEX], mdrv);
        }
        else if (disp_hd_vid_register_table[DISP_HD_VID_FORMAT_INDEX] == DISP_HD_VID1_FORMAT)
        {
            set_filter_for_vid1_nonscale(mdrv);
            adjust_frequency_response(disp_hd_vid_register_table[DISP_HD_VID_HSCALER_CBCR_BASE_ADDRESS_INDEX], mdrv);
        }
    }
    else
    {
        
        preset_CoefParams(&m_layer_data->vY);
        preset_CoefParams(&m_layer_data->vC);
        preset_CoefParams(&m_layer_data->hY);
        preset_CoefParams(&m_layer_data->hC);
        
        m_layer_data->vY.rho    =   (double)config->dest.h / config->source.h;
        m_layer_data->vC.rho    =   (double)config->dest.h*vmult / config->source.h;
        m_layer_data->hY.rho    =   (double)config->dest.w / config->source.w;
        m_layer_data->hC.rho    =   (double)config->dest.w*hmult / config->source.w;
        
        if ((config->source.w == 1280) && 
            (config->source.h == 720) &&
            (config->dest.w == 720) &&
            (config->dest.h == 480)) // 720p -> 480p
        {
          m_layer_data->vY.lpf0 = 0.65;
          m_layer_data->hY.lpf0 = 0.85;
          m_layer_data->hY.beta = 1.80;
        }
        else if ((config->source.w == 1920) && 
            (config->source.h == 1080) &&
            (config->dest.w == 720) &&
            (config->dest.h == 480)) // 1080p -> 480p
        {
          m_layer_data->vY.lpf0 = 0.9;
          m_layer_data->vY.beta = 1.8;
          m_layer_data->vC.lpf0 = 0.9;
          m_layer_data->vC.beta = 1.8;
          
          m_layer_data->hY.lpf0 = 0.95;
          m_layer_data->hY.beta = 1.80;
          m_layer_data->hC.lpf0 = 0.95;
          m_layer_data->hC.beta = 1.80;
        }
        else if ((config->source.w == 1920) && 
            (config->source.h == 1080) &&
            (config->dest.w == 1280) &&
            (config->dest.h == 720)) // 1080p -> 720p
        {
          m_layer_data->vY.lpf0 = 0.75;
          m_layer_data->vY.beta = 1.80;
          m_layer_data->hY.lpf0 = 0.72;
          m_layer_data->hY.beta = 8.00;
        }
        
        // ###### vertical scaler0 - Y
        assign_scaler_autotype(disp_hd_vid_register_table[DISP_HD_VID_VERTICAL_SCALER_TAP_INDEX], &m_layer_data->vY, srckey);
        create_coefficient_table_version_2(layer_data,
                                           UPSAMPLING_FACTOR, 
                                           (void *)htablev_y,
                                           disp_hd_vid_register_table[DISP_HD_VID_VERTICAL_SCALER_TAP_INDEX], 
                                           disp_hd_vid_register_table[DISP_HD_VID_VERTICAL_SCALER_TAP_INDEX], 
                                           0,
                                           0,
                                           vYpremult, /* Vertical scaler Y gain */
                                           &m_layer_data->vY, vYsoften, vYsharpen);
        
        
        
        set_coefficient_disp_ram_table(disp_hd_vid_register_table[DISP_HD_VID_VSCALER_Y_BASE_ADDRESS_INDEX], 
                                  (void *)htablev_y, 
                                  UPSAMPLING_FACTOR,  
                                  disp_hd_vid_register_table[DISP_HD_VID_VERTICAL_SCALER_TAP_INDEX], 
                                  mdrv);
        
        
        // Vertical scaler  - CbCr
        assign_scaler_autotype(disp_hd_vid_register_table[DISP_HD_VID_VERTICAL_SCALER_TAP_INDEX], &m_layer_data->vC, srckey);
        create_coefficient_table_version_2(layer_data,
                                           UPSAMPLING_FACTOR, 
                                           (void *)htablev_cbcr,
                                           disp_hd_vid_register_table[DISP_HD_VID_VERTICAL_SCALER_TAP_INDEX], 
                                           disp_hd_vid_register_table[DISP_HD_VID_VERTICAL_SCALER_TAP_INDEX], 
                                           0,
                                           0,
                                           vCpremult, /* Vertical scaler Cb/Cr gain */
                                           &m_layer_data->vC, vCsoften, vCsharpen);
        
        set_coefficient_disp_ram_table(disp_hd_vid_register_table[DISP_HD_VID_VSCALER_CBCR_BASE_ADDRESS_INDEX], 
                                  (void *)htablev_cbcr, 
                                  UPSAMPLING_FACTOR,  
                                  disp_hd_vid_register_table[DISP_HD_VID_VERTICAL_SCALER_TAP_INDEX], 
                                  mdrv);
        
        //Horizonal scaler - Y
        assign_scaler_autotype(y_upscale_taps, &m_layer_data->hY, srckey);
        create_coefficient_table_version_2(layer_data,
                                           UPSAMPLING_FACTOR, 
                                           htableh_y,
                                           HORIZONAL_SCALER_TAPS, 
                                           y_upscale_taps, 
                                           0,
                                           0,
                                           hYpremult, /* Horizontal scaler Y gain */
                                           &m_layer_data->hY, hYsoften, hYsharpen);
        
        set_coefficient_disp_ram_table(disp_hd_vid_register_table[DISP_HD_VID_HSCALER_Y_BASE_ADDRESS_INDEX], 
                                  htableh_y, 
                                  UPSAMPLING_FACTOR,  
                                  y_upscale_taps, 
                                  mdrv);
        
        //Horizonal scaler - CbCr
        assign_scaler_autotype(c_upscale_taps, &m_layer_data->hC, srckey);
        create_coefficient_table_version_2(layer_data,
                                           UPSAMPLING_FACTOR, 
                                           htableh_cbcr,
                                           HORIZONAL_SCALER_TAPS, 
                                           c_upscale_taps, 
                                           0,
                                           0,
                                           hCpremult, /* Horizontal scaler CbCr gain */
                                           &m_layer_data->hC, hCsoften, hCsharpen);
        
        set_coefficient_disp_ram_table(disp_hd_vid_register_table[DISP_HD_VID_HSCALER_CBCR_BASE_ADDRESS_INDEX], 
                                  htableh_cbcr, 
                                  UPSAMPLING_FACTOR,  
                                  c_upscale_taps, 
                                  mdrv);
    }
    
    end_time    =   REG_READ(CG_SYSTEM_CNT);
    
    seconds = ((double)(end_time - start_time)) / 1687500.0;
    
    D_INFO("Co-efficient calculation time: %.4lf\n", seconds);
    return DFB_OK;
     
}
     

/*****************************************************************************************************************
Function Name: SetFes1ScalerCoef

Parameters:

Returns:   DFB_OK if it's successful
           
*****************************************************************************************************************/
DFBResult
SetFes1ScalerCoef(pxc_dfb_driver_data mdrv, 
                  int in_width,
                  int in_height,
                  int in_format,
                  int out_width,
                  int out_height,
                  int out_format)
{
    double vmult;
    double hmult;

    scaleCoefParam coefParamsVY;
    scaleCoefParam coefParamsVC;
    scaleCoefParam coefParamsHY;
    scaleCoefParam coefParamsHC;


    if ((in_format == 420) &&
        (out_format == 444))
	{
        vmult = 2;
        hmult = 2;
	}
    else if ((in_format == 420) &&
        (out_format == 422))
	{
        vmult = 1;
        hmult = 2;
	}
    else
    {
        D_DEBUG ("Error in chroma types in SetFes1ScalerCoef\n");
        return DFB_UNSUPPORTED;
    }

    //clear previous setting
    memset ((void *)&(htablev_y[0][0]), 
            0,
            FES_UPSAMPLING_FACTOR * FES1_VERTICAL_SCALER_TAPS * sizeof(double));

    memset ((void *)&(htablev_cbcr[0][0]), 
            0,
            FES_UPSAMPLING_FACTOR * FES1_VERTICAL_SCALER_TAPS * sizeof(double));

    memset ((void *)&(htableh_y[0][0]), 
            0,
            FES_UPSAMPLING_FACTOR * FES1_HORIZONAL_SCALER_TAPS * sizeof(double));

    memset ((void *)&(htableh_cbcr[0][0]), 
            0,
            FES_UPSAMPLING_FACTOR * FES1_HORIZONAL_SCALER_TAPS * sizeof(double));

    preset_CoefParams(&coefParamsVY);
    preset_CoefParams(&coefParamsVC);
    preset_CoefParams(&coefParamsHY);
    preset_CoefParams(&coefParamsHC);

    coefParamsVY.rho = (double)out_height / in_height;
    coefParamsVC.rho = (double)out_height*vmult / in_height;
    coefParamsHY.rho = (double)out_width / in_width;
    coefParamsHC.rho = (double)out_width*hmult / in_width;

    if ((in_width == 1280) && 
        (in_height == 720) &&
        (out_height == 480 || out_height == 576)) // 720p -> 480p
    {
      coefParamsVY.lpf0=0.65;
      coefParamsHY.lpf0=0.85;
      coefParamsHY.beta=1.8;
    }
    else if ((in_width == 1920) && 
             (in_height == 1080) &&
             (out_height == 480 || out_height == 576)) // 1080p -> 480p
    {
      coefParamsVY.lpf0=0.9;
      coefParamsVY.beta=1.8;
      coefParamsVC.lpf0=0.9;  
      coefParamsVC.beta=1.8;
      coefParamsHY.lpf0=0.95;
      coefParamsHY.beta=1.8;
      coefParamsHC.lpf0=0.95;
      coefParamsHC.beta=1.8;
    }
    else if ((in_width == 1920) && 
             (in_height == 1080) &&
             (out_width == 1280) &&
             (out_height == 720)) // 1080p -> 720p
    {
      coefParamsVY.lpf0=0.75;
      coefParamsVY.beta=1.8;
      coefParamsHY.lpf0=0.72;
      coefParamsHY.beta=1.8;
    }
    else if (in_width < out_width)
    {
      coefParamsHY.lpf0=1.35;
      coefParamsHC.lpf0=1.35;
    }

    // ###### vertical scaler0 - Y
    assign_scaler_autotype(FES1_VERTICAL_SCALER_TAPS, &coefParamsVY, 0);
    create_coefficient_table_version_2(NULL,
                                       FES_UPSAMPLING_FACTOR, 
                                       (void *)htablev_y,
                                       FES1_VERTICAL_SCALER_TAPS, 
                                       FES1_VERTICAL_SCALER_TAPS, 
                                       0, 0, 1.0,
                                       &coefParamsVY, 1, 0);

    set_fes1_coefficient_table(FES1_COEF_VERTICAL_Y_BASE_ADDR, 
                              (void *)htablev_y, 
                              FES_UPSAMPLING_FACTOR,  
                              FES1_VERTICAL_SCALER_TAPS, 
                              mdrv);


    // Vertical scaler  - CbCr
    assign_scaler_autotype(FES1_VERTICAL_SCALER_TAPS, &coefParamsVC, 0);
    create_coefficient_table_version_2(NULL,
                                       FES_UPSAMPLING_FACTOR, 
                                       (void *)htablev_cbcr,
                                       FES1_VERTICAL_SCALER_TAPS, 
                                       FES1_VERTICAL_SCALER_TAPS, 
                                       0, 0, 1.0, 
                                       &coefParamsVC, 1, 0);

    set_fes1_coefficient_table(FES1_COEF_VERTICAL_CBCR_BASE_ADDR, 
                              (void *)htablev_cbcr, 
                              FES_UPSAMPLING_FACTOR,  
                              FES1_VERTICAL_SCALER_TAPS,
                              mdrv);


    //Horizonal scaler - Y
    assign_scaler_autotype(FES1_HORIZONAL_SCALER_TAPS, &coefParamsHY, 0);
    create_coefficient_table_version_2(NULL,
                                       FES_UPSAMPLING_FACTOR, 
                                       (void *)htableh_y,
                                       FES1_HORIZONAL_SCALER_TAPS, 
                                       FES1_HORIZONAL_SCALER_TAPS, 
                                       0, 0, 1.0, 
                                       &coefParamsHY, 1, 0.25);

    set_fes1_coefficient_table(FES1_COEF_HOREIZONAL_Y_BASE_ADDR, 
                              (void *)htableh_y, 
                              FES_UPSAMPLING_FACTOR,  
                              FES1_HORIZONAL_SCALER_TAPS, 
                              mdrv);


    //Horizonal scaler - CbCr
    assign_scaler_autotype(FES1_HORIZONAL_SCALER_TAPS, &coefParamsHC, 0);
    create_coefficient_table_version_2(NULL,
                                       FES_UPSAMPLING_FACTOR, 
                                       (void *)htableh_cbcr,
                                       FES1_HORIZONAL_SCALER_TAPS, 
                                       FES1_HORIZONAL_SCALER_TAPS, 
                                       0, 0, 1.0, 
                                       &coefParamsHC, 1, 0.25);

    set_fes1_coefficient_table(FES1_COEF_HOREIZONAL_CBCR_BASE_ADDR, 
                              (void *)htableh_cbcr, 
                              FES_UPSAMPLING_FACTOR,  
                              FES1_HORIZONAL_SCALER_TAPS, 
                              mdrv);
    return DFB_OK;
     
}


#ifdef FES_DRIVER
/*****************************************************************************************************************
Function Name: Set_Kaiser_Table

*****************************************************************************************************************/
void Set_Kaiser_Table()
		  SetFesScalerCoef
{

    kaiser_256[0] = 0.789848; 
    kaiser_256[1] = 0.792925; 
    kaiser_256[2] = 0.795983; 
    kaiser_256[3] = 0.799022; 
    kaiser_256[4] = 0.802044; 
    kaiser_256[5] = 0.805046; 
    kaiser_256[6] = 0.808030; 
    kaiser_256[7] = 0.810995; 
    kaiser_256[8] = 0.813941; 
    kaiser_256[9] = 0.816868; 
    kaiser_256[10] = 0.819776; 
    kaiser_256[11] = 0.822664; 
    kaiser_256[12] = 0.825533; 
    kaiser_256[13] = 0.828382; 
    kaiser_256[14] = 0.831212; 
    kaiser_256[15] = 0.834021; 
    kaiser_256[16] = 0.836811; 
    kaiser_256[17] = 0.839580; 
    kaiser_256[18] = 0.842330; 
    kaiser_256[19] = 0.845058; 
    kaiser_256[20] = 0.847767; 
    kaiser_256[21] = 0.850455; 
    kaiser_256[22] = 0.853122; 
    kaiser_256[23] = 0.855768; 
    kaiser_256[24] = 0.858393; 
    kaiser_256[25] = 0.860998; 
    kaiser_256[26] = 0.863581; 
    kaiser_256[27] = 0.866143; 
    kaiser_256[28] = 0.868683; 
    kaiser_256[29] = 0.871202; 
    kaiser_256[30] = 0.873700; 
    kaiser_256[31] = 0.876175; 
    kaiser_256[32] = 0.878629; 
    kaiser_256[33] = 0.881061; 
    kaiser_256[34] = 0.883471; 
    kaiser_256[35] = 0.885859; 
    kaiser_256[36] = 0.888224; 
    kaiser_256[37] = 0.890567; 
    kaiser_256[38] = 0.892888; 
    kaiser_256[39] = 0.895186; 
    kaiser_256[40] = 0.897461; 
    kaiser_256[41] = 0.899714; 
    kaiser_256[42] = 0.901944; 
    kaiser_256[43] = 0.904151; 
    kaiser_256[44] = 0.906335; 
    kaiser_256[45] = 0.908496; 
    kaiser_256[46] = 0.910633; 
    kaiser_256[47] = 0.912747; 
    kaiser_256[48] = 0.914838; 
    kaiser_256[49] = 0.916905; 
    kaiser_256[50] = 0.918949; 
    kaiser_256[51] = 0.920969; 
    kaiser_256[52] = 0.922965; 
    kaiser_256[53] = 0.924937; 
    kaiser_256[54] = 0.926885; 
    kaiser_256[55] = 0.928810; 
    kaiser_256[56] = 0.930710; 
    kaiser_256[57] = 0.932586; 
    kaiser_256[58] = 0.934438; 
    kaiser_256[59] = 0.936265; 
    kaiser_256[60] = 0.938068; 
    kaiser_256[61] = 0.939846; 
    kaiser_256[62] = 0.941600; 
    kaiser_256[63] = 0.943329; 
    kaiser_256[64] = 0.945033; 
    kaiser_256[65] = 0.946713; 
    kaiser_256[66] = 0.948367; 
    kaiser_256[67] = 0.949997; 
    kaiser_256[68] = 0.951601; 
    kaiser_256[69] = 0.953181; 
    kaiser_256[70] = 0.954735; 
    kaiser_256[71] = 0.956264; 
    kaiser_256[72] = 0.957768; 
    kaiser_256[73] = 0.959246; 
    kaiser_256[74] = 0.960699; 
    kaiser_256[75] = 0.962126; 
    kaiser_256[76] = 0.963528; 
    kaiser_256[77] = 0.964904; 
    kaiser_256[78] = 0.966254; 
    kaiser_256[79] = 0.967578; 
    kaiser_256[80] = 0.968877; 
    kaiser_256[81] = 0.970150; 
    kaiser_256[82] = 0.971397; 
    kaiser_256[83] = 0.972618; 
    kaiser_256[84] = 0.973813; 
    kaiser_256[85] = 0.974982; 
    kaiser_256[86] = 0.976124; 
    kaiser_256[87] = 0.977241; 
    kaiser_256[88] = 0.978331; 
    kaiser_256[89] = 0.979395; 
    kaiser_256[90] = 0.980433; 
    kaiser_256[91] = 0.981444; 
    kaiser_256[92] = 0.982429; 
    kaiser_256[93] = 0.983387; 
    kaiser_256[94] = 0.984319; 
    kaiser_256[95] = 0.985224; 
    kaiser_256[96] = 0.986103; 
    kaiser_256[97] = 0.986955; 
    kaiser_256[98] = 0.987780; 
    kaiser_256[99] = 0.988579; 
    kaiser_256[100] = 0.989350; 
    kaiser_256[101] = 0.990096; 
    kaiser_256[102] = 0.990814; 
    kaiser_256[103] = 0.991505; 
    kaiser_256[104] = 0.992170; 
    kaiser_256[105] = 0.992808; 
    kaiser_256[106] = 0.993418; 
    kaiser_256[107] = 0.994002; 
    kaiser_256[108] = 0.994559; 
    kaiser_256[109] = 0.995089; 
    kaiser_256[110] = 0.995591; 
    kaiser_256[111] = 0.996067; 
    kaiser_256[112] = 0.996516; 
    kaiser_256[113] = 0.996937; 
    kaiser_256[114] = 0.997332; 
    kaiser_256[115] = 0.997699; 
    kaiser_256[116] = 0.998039; 
    kaiser_256[117] = 0.998352; 
    kaiser_256[118] = 0.998638; 
    kaiser_256[119] = 0.998897; 
    kaiser_256[120] = 0.999128; 
    kaiser_256[121] = 0.999333; 
    kaiser_256[122] = 0.999510; 
    kaiser_256[123] = 0.999659; 
    kaiser_256[124] = 0.999782; 
    kaiser_256[125] = 0.999877; 
    kaiser_256[126] = 0.999946; 
    kaiser_256[127] = 0.999986; 
    kaiser_256[128] = 1.000000; 
    kaiser_256[129] = 0.999986; 
    kaiser_256[130] = 0.999946; 
    kaiser_256[131] = 0.999877; 
    kaiser_256[132] = 0.999782; 
    kaiser_256[133] = 0.999659; 
    kaiser_256[134] = 0.999510; 
    kaiser_256[135] = 0.999333; 
    kaiser_256[136] = 0.999128; 
    kaiser_256[137] = 0.998897; 
    kaiser_256[138] = 0.998638; 
    kaiser_256[139] = 0.998352; 
    kaiser_256[140] = 0.998039; 
    kaiser_256[141] = 0.997699; 
    kaiser_256[142] = 0.997332; 
    kaiser_256[143] = 0.996937; 
    kaiser_256[144] = 0.996516; 
    kaiser_256[145] = 0.996067; 
    kaiser_256[146] = 0.995591; 
    kaiser_256[147] = 0.995089; 
    kaiser_256[148] = 0.994559; 
    kaiser_256[149] = 0.994002; 
    kaiser_256[150] = 0.993418; 
    kaiser_256[151] = 0.992808; 
    kaiser_256[152] = 0.992170; 
    kaiser_256[153] = 0.991505; 
    kaiser_256[154] = 0.990814; 
    kaiser_256[155] = 0.990096; 
    kaiser_256[156] = 0.989350; 
    kaiser_256[157] = 0.988579; 
    kaiser_256[158] = 0.987780; 
    kaiser_256[159] = 0.986955; 
    kaiser_256[160] = 0.986103; 
    kaiser_256[161] = 0.985224; 
    kaiser_256[162] = 0.984319; 
    kaiser_256[163] = 0.983387; 
    kaiser_256[164] = 0.982429; 
    kaiser_256[165] = 0.981444; 
    kaiser_256[166] = 0.980433; 
    kaiser_256[167] = 0.979395; 
    kaiser_256[168] = 0.978331; 
    kaiser_256[169] = 0.977241; 
    kaiser_256[170] = 0.976124; 
    kaiser_256[171] = 0.974982; 
    kaiser_256[172] = 0.973813; 
    kaiser_256[173] = 0.972618; 
    kaiser_256[174] = 0.971397; 
    kaiser_256[175] = 0.970150; 
    kaiser_256[176] = 0.968877; 
    kaiser_256[177] = 0.967578; 
    kaiser_256[178] = 0.966254; 
    kaiser_256[179] = 0.964904; 
    kaiser_256[180] = 0.963528; 
    kaiser_256[181] = 0.962126; 
    kaiser_256[182] = 0.960699; 
    kaiser_256[183] = 0.959246; 
    kaiser_256[184] = 0.957768; 
    kaiser_256[185] = 0.956264; 
    kaiser_256[186] = 0.954735; 
    kaiser_256[187] = 0.953181; 
    kaiser_256[188] = 0.951601; 
    kaiser_256[189] = 0.949997; 
    kaiser_256[190] = 0.948367; 
    kaiser_256[191] = 0.946713; 
    kaiser_256[192] = 0.945033; 
    kaiser_256[193] = 0.943329; 
    kaiser_256[194] = 0.941600; 
    kaiser_256[195] = 0.939846; 
    kaiser_256[196] = 0.938068; 
    kaiser_256[197] = 0.936265; 
    kaiser_256[198] = 0.934438; 
    kaiser_256[199] = 0.932586; 
    kaiser_256[200] = 0.930710; 
    kaiser_256[201] = 0.928810; 
    kaiser_256[202] = 0.926885; 
    kaiser_256[203] = 0.924937; 
    kaiser_256[204] = 0.922965; 
    kaiser_256[205] = 0.920969; 
    kaiser_256[206] = 0.918949; 
    kaiser_256[207] = 0.916905; 
    kaiser_256[208] = 0.914838; 
    kaiser_256[209] = 0.912747; 
    kaiser_256[210] = 0.910633; 
    kaiser_256[211] = 0.908496; 
    kaiser_256[212] = 0.906335; 
    kaiser_256[213] = 0.904151; 
    kaiser_256[214] = 0.901944; 
    kaiser_256[215] = 0.899714; 
    kaiser_256[216] = 0.897461; 
    kaiser_256[217] = 0.895186; 
    kaiser_256[218] = 0.892888; 
    kaiser_256[219] = 0.890567; 
    kaiser_256[220] = 0.888224; 
    kaiser_256[221] = 0.885859; 
    kaiser_256[222] = 0.883471; 
    kaiser_256[223] = 0.881061; 
    kaiser_256[224] = 0.878629; 
    kaiser_256[225] = 0.876175; 
    kaiser_256[226] = 0.873700; 
    kaiser_256[227] = 0.871202; 
    kaiser_256[228] = 0.868683; 
    kaiser_256[229] = 0.866143; 
    kaiser_256[230] = 0.863581; 
    kaiser_256[231] = 0.860998; 
    kaiser_256[232] = 0.858393; 
    kaiser_256[233] = 0.855768; 
    kaiser_256[234] = 0.853122; 
    kaiser_256[235] = 0.850455; 
    kaiser_256[236] = 0.847767; 
    kaiser_256[237] = 0.845058; 
    kaiser_256[238] = 0.842330; 
    kaiser_256[239] = 0.839580; 
    kaiser_256[240] = 0.836811; 
    kaiser_256[241] = 0.834021; 
    kaiser_256[242] = 0.831212; 
    kaiser_256[243] = 0.828382; 
    kaiser_256[244] = 0.825533; 
    kaiser_256[245] = 0.822664; 
    kaiser_256[246] = 0.819776; 
    kaiser_256[247] = 0.816868; 
    kaiser_256[248] = 0.813941; 
    kaiser_256[249] = 0.810995; 
    kaiser_256[250] = 0.808030; 
    kaiser_256[251] = 0.805046; 
    kaiser_256[252] = 0.802044; 
    kaiser_256[253] = 0.799022; 
    kaiser_256[254] = 0.795983; 
    kaiser_256[255] = 0.792925; 
    kaiser_384[0] = 0.789848; 
    kaiser_384[1] = 0.791901; 
    kaiser_384[2] = 0.793946; 
    kaiser_384[3] = 0.795983; 
    kaiser_384[4] = 0.798011; 
    kaiser_384[5] = 0.800032; 
    kaiser_384[6] = 0.802044; 
    kaiser_384[7] = 0.804047; 
    kaiser_384[8] = 0.806043; 
    kaiser_384[9] = 0.808030; 
    kaiser_384[10] = 0.810009; 
    kaiser_384[11] = 0.811979; 
    kaiser_384[12] = 0.813941; 
    kaiser_384[13] = 0.815895; 
    kaiser_384[14] = 0.817840; 
    kaiser_384[15] = 0.819776; 
    kaiser_384[16] = 0.821704; 
    kaiser_384[17] = 0.823623; 
    kaiser_384[18] = 0.825533; 
    kaiser_384[19] = 0.827435; 
    kaiser_384[20] = 0.829328; 
    kaiser_384[21] = 0.831212; 
    kaiser_384[22] = 0.833087; 
    kaiser_384[23] = 0.834953; 
    kaiser_384[24] = 0.836811; 
    kaiser_384[25] = 0.838659; 
    kaiser_384[26] = 0.840499; 
    kaiser_384[27] = 0.842330; 
    kaiser_384[28] = 0.844151; 
    kaiser_384[29] = 0.845964; 
    kaiser_384[30] = 0.847767; 
    kaiser_384[31] = 0.849561; 
    kaiser_384[32] = 0.851346; 
    kaiser_384[33] = 0.853122; 
    kaiser_384[34] = 0.854888; 
    kaiser_384[35] = 0.856646; 
    kaiser_384[36] = 0.858393; 
    kaiser_384[37] = 0.860132; 
    kaiser_384[38] = 0.861861; 
    kaiser_384[39] = 0.863581; 
    kaiser_384[40] = 0.865291; 
    kaiser_384[41] = 0.866992; 
    kaiser_384[42] = 0.868683; 
    kaiser_384[43] = 0.870365; 
    kaiser_384[44] = 0.872037; 
    kaiser_384[45] = 0.873700; 
    kaiser_384[46] = 0.875352; 
    kaiser_384[47] = 0.876996; 
    kaiser_384[48] = 0.878629; 
    kaiser_384[49] = 0.880253; 
    kaiser_384[50] = 0.881867; 
    kaiser_384[51] = 0.883471; 
    kaiser_384[52] = 0.885065; 
    kaiser_384[53] = 0.886650; 
    kaiser_384[54] = 0.888224; 
    kaiser_384[55] = 0.889789; 
    kaiser_384[56] = 0.891343; 
    kaiser_384[57] = 0.892888; 
    kaiser_384[58] = 0.894422; 
    kaiser_384[59] = 0.895947; 
    kaiser_384[60] = 0.897461; 
    kaiser_384[61] = 0.898966; 
    kaiser_384[62] = 0.900460; 
    kaiser_384[63] = 0.901944; 
    kaiser_384[64] = 0.903418; 
    kaiser_384[65] = 0.904881; 
    kaiser_384[66] = 0.906335; 
    kaiser_384[67] = 0.907778; 
    kaiser_384[68] = 0.909211; 
    kaiser_384[69] = 0.910633; 
    kaiser_384[70] = 0.912045; 
    kaiser_384[71] = 0.913447; 
    kaiser_384[72] = 0.914838; 
    kaiser_384[73] = 0.916219; 
    kaiser_384[74] = 0.917589; 
    kaiser_384[75] = 0.918949; 
    kaiser_384[76] = 0.920298; 
    kaiser_384[77] = 0.921637; 
    kaiser_384[78] = 0.922965; 
    kaiser_384[79] = 0.924282; 
    kaiser_384[80] = 0.925589; 
    kaiser_384[81] = 0.926885; 
    kaiser_384[82] = 0.928171; 
    kaiser_384[83] = 0.929446; 
    kaiser_384[84] = 0.930710; 
    kaiser_384[85] = 0.931963; 
    kaiser_384[86] = 0.933206; 
    kaiser_384[87] = 0.934438; 
    kaiser_384[88] = 0.935659; 
    kaiser_384[89] = 0.936869; 
    kaiser_384[90] = 0.938068; 
    kaiser_384[91] = 0.939256; 
    kaiser_384[92] = 0.940433; 
    kaiser_384[93] = 0.941600; 
    kaiser_384[94] = 0.942755; 
    kaiser_384[95] = 0.943900; 
    kaiser_384[96] = 0.945033; 
    kaiser_384[97] = 0.946156; 
    kaiser_384[98] = 0.947267; 
    kaiser_384[99] = 0.948367; 
    kaiser_384[100] = 0.949456; 
    kaiser_384[101] = 0.950534; 
    kaiser_384[102] = 0.951601; 
    kaiser_384[103] = 0.952657; 
    kaiser_384[104] = 0.953702; 
    kaiser_384[105] = 0.954735; 
    kaiser_384[106] = 0.955757; 
    kaiser_384[107] = 0.956768; 
    kaiser_384[108] = 0.957768; 
    kaiser_384[109] = 0.958756; 
    kaiser_384[110] = 0.959733; 
    kaiser_384[111] = 0.960699; 
    kaiser_384[112] = 0.961653; 
    kaiser_384[113] = 0.962596; 
    kaiser_384[114] = 0.963528; 
    kaiser_384[115] = 0.964448; 
    kaiser_384[116] = 0.965357; 
    kaiser_384[117] = 0.966254; 
    kaiser_384[118] = 0.967140; 
    kaiser_384[119] = 0.968014; 
    kaiser_384[120] = 0.968877; 
    kaiser_384[121] = 0.969729; 
    kaiser_384[122] = 0.970569; 
    kaiser_384[123] = 0.971397; 
    kaiser_384[124] = 0.972214; 
    kaiser_384[125] = 0.973019; 
    kaiser_384[126] = 0.973813; 
    kaiser_384[127] = 0.974595; 
    kaiser_384[128] = 0.975366; 
    kaiser_384[129] = 0.976124; 
    kaiser_384[130] = 0.976872; 
    kaiser_384[131] = 0.977607; 
    kaiser_384[132] = 0.978331; 
    kaiser_384[133] = 0.979043; 
    kaiser_384[134] = 0.979744; 
    kaiser_384[135] = 0.980433; 
    kaiser_384[136] = 0.981110; 
    kaiser_384[137] = 0.981775; 
    kaiser_384[138] = 0.982429; 
    kaiser_384[139] = 0.983070; 
    kaiser_384[140] = 0.983700; 
    kaiser_384[141] = 0.984319; 
    kaiser_384[142] = 0.984925; 
    kaiser_384[143] = 0.985520; 
    kaiser_384[144] = 0.986103; 
    kaiser_384[145] = 0.986674; 
    kaiser_384[146] = 0.987233; 
    kaiser_384[147] = 0.987780; 
    kaiser_384[148] = 0.988315; 
    kaiser_384[149] = 0.988839; 
    kaiser_384[150] = 0.989350; 
    kaiser_384[151] = 0.989850; 
    kaiser_384[152] = 0.990338; 
    kaiser_384[153] = 0.990814; 
    kaiser_384[154] = 0.991278; 
    kaiser_384[155] = 0.991730; 
    kaiser_384[156] = 0.992170; 
    kaiser_384[157] = 0.992598; 
    kaiser_384[158] = 0.993014; 
    kaiser_384[159] = 0.993418; 
    kaiser_384[160] = 0.993810; 
    kaiser_384[161] = 0.994191; 
    kaiser_384[162] = 0.994559; 
    kaiser_384[163] = 0.994915; 
    kaiser_384[164] = 0.995259; 
    kaiser_384[165] = 0.995591; 
    kaiser_384[166] = 0.995912; 
    kaiser_384[167] = 0.996220; 
    kaiser_384[168] = 0.996516; 
    kaiser_384[169] = 0.996800; 
    kaiser_384[170] = 0.997072; 
    kaiser_384[171] = 0.997332; 
    kaiser_384[172] = 0.997580; 
    kaiser_384[173] = 0.997816; 
    kaiser_384[174] = 0.998039; 
    kaiser_384[175] = 0.998251; 
    kaiser_384[176] = 0.998451; 
    kaiser_384[177] = 0.998638; 
    kaiser_384[178] = 0.998814; 
    kaiser_384[179] = 0.998977; 
    kaiser_384[180] = 0.999128; 
    kaiser_384[181] = 0.999268; 
    kaiser_384[182] = 0.999395; 
    kaiser_384[183] = 0.999510; 
    kaiser_384[184] = 0.999613; 
    kaiser_384[185] = 0.999703; 
    kaiser_384[186] = 0.999782; 
    kaiser_384[187] = 0.999849; 
    kaiser_384[188] = 0.999903; 
    kaiser_384[189] = 0.999946; 
    kaiser_384[190] = 0.999976; 
    kaiser_384[191] = 0.999994; 
    kaiser_384[192] = 1.000000; 
    kaiser_384[193] = 0.999994; 
    kaiser_384[194] = 0.999976; 
    kaiser_384[195] = 0.999946; 
    kaiser_384[196] = 0.999903; 
    kaiser_384[197] = 0.999849; 
    kaiser_384[198] = 0.999782; 
    kaiser_384[199] = 0.999703; 
    kaiser_384[200] = 0.999613; 
    kaiser_384[201] = 0.999510; 
    kaiser_384[202] = 0.999395; 
    kaiser_384[203] = 0.999268; 
    kaiser_384[204] = 0.999128; 
    kaiser_384[205] = 0.998977; 
    kaiser_384[206] = 0.998814; 
    kaiser_384[207] = 0.998638; 
    kaiser_384[208] = 0.998451; 
    kaiser_384[209] = 0.998251; 
    kaiser_384[210] = 0.998039; 
    kaiser_384[211] = 0.997816; 
    kaiser_384[212] = 0.997580; 
    kaiser_384[213] = 0.997332; 
    kaiser_384[214] = 0.997072; 
    kaiser_384[215] = 0.996800; 
    kaiser_384[216] = 0.996516; 
    kaiser_384[217] = 0.996220; 
    kaiser_384[218] = 0.995912; 
    kaiser_384[219] = 0.995591; 
    kaiser_384[220] = 0.995259; 
    kaiser_384[221] = 0.994915; 
    kaiser_384[222] = 0.994559; 
    kaiser_384[223] = 0.994191; 
    kaiser_384[224] = 0.993810; 
    kaiser_384[225] = 0.993418; 
    kaiser_384[226] = 0.993014; 
    kaiser_384[227] = 0.992598; 
    kaiser_384[228] = 0.992170; 
    kaiser_384[229] = 0.991730; 
    kaiser_384[230] = 0.991278; 
    kaiser_384[231] = 0.990814; 
    kaiser_384[232] = 0.990338; 
    kaiser_384[233] = 0.989850; 
    kaiser_384[234] = 0.989350; 
    kaiser_384[235] = 0.988839; 
    kaiser_384[236] = 0.988315; 
    kaiser_384[237] = 0.987780; 
    kaiser_384[238] = 0.987233; 
    kaiser_384[239] = 0.986674; 
    kaiser_384[240] = 0.986103; 
    kaiser_384[241] = 0.985520; 
    kaiser_384[242] = 0.984925; 
    kaiser_384[243] = 0.984319; 
    kaiser_384[244] = 0.983700; 
    kaiser_384[245] = 0.983070; 
    kaiser_384[246] = 0.982429; 
    kaiser_384[247] = 0.981775; 
    kaiser_384[248] = 0.981110; 
    kaiser_384[249] = 0.980433; 
    kaiser_384[250] = 0.979744; 
    kaiser_384[251] = 0.979043; 
    kaiser_384[252] = 0.978331; 
    kaiser_384[253] = 0.977607; 
    kaiser_384[254] = 0.976872; 
    kaiser_384[255] = 0.976124; 
    kaiser_384[256] = 0.975366; 
    kaiser_384[257] = 0.974595; 
    kaiser_384[258] = 0.973813; 
    kaiser_384[259] = 0.973019; 
    kaiser_384[260] = 0.972214; 
    kaiser_384[261] = 0.971397; 
    kaiser_384[262] = 0.970569; 
    kaiser_384[263] = 0.969729; 
    kaiser_384[264] = 0.968877; 
    kaiser_384[265] = 0.968014; 
    kaiser_384[266] = 0.967140; 
    kaiser_384[267] = 0.966254; 
    kaiser_384[268] = 0.965357; 
    kaiser_384[269] = 0.964448; 
    kaiser_384[270] = 0.963528; 
    kaiser_384[271] = 0.962596; 
    kaiser_384[272] = 0.961653; 
    kaiser_384[273] = 0.960699; 
    kaiser_384[274] = 0.959733; 
    kaiser_384[275] = 0.958756; 
    kaiser_384[276] = 0.957768; 
    kaiser_384[277] = 0.956768; 
    kaiser_384[278] = 0.955757; 
    kaiser_384[279] = 0.954735; 
    kaiser_384[280] = 0.953702; 
    kaiser_384[281] = 0.952657; 
    kaiser_384[282] = 0.951601; 
    kaiser_384[283] = 0.950534; 
    kaiser_384[284] = 0.949456; 
    kaiser_384[285] = 0.948367; 
    kaiser_384[286] = 0.947267; 
    kaiser_384[287] = 0.946156; 
    kaiser_384[288] = 0.945033; 
    kaiser_384[289] = 0.943900; 
    kaiser_384[290] = 0.942755; 
    kaiser_384[291] = 0.941600; 
    kaiser_384[292] = 0.940433; 
    kaiser_384[293] = 0.939256; 
    kaiser_384[294] = 0.938068; 
    kaiser_384[295] = 0.936869; 
    kaiser_384[296] = 0.935659; 
    kaiser_384[297] = 0.934438; 
    kaiser_384[298] = 0.933206; 
    kaiser_384[299] = 0.931963; 
    kaiser_384[300] = 0.930710; 
    kaiser_384[301] = 0.929446; 
    kaiser_384[302] = 0.928171; 
    kaiser_384[303] = 0.926885; 
    kaiser_384[304] = 0.925589; 
    kaiser_384[305] = 0.924282; 
    kaiser_384[306] = 0.922965; 
    kaiser_384[307] = 0.921637; 
    kaiser_384[308] = 0.920298; 
    kaiser_384[309] = 0.918949; 
    kaiser_384[310] = 0.917589; 
    kaiser_384[311] = 0.916219; 
    kaiser_384[312] = 0.914838; 
    kaiser_384[313] = 0.913447; 
    kaiser_384[314] = 0.912045; 
    kaiser_384[315] = 0.910633; 
    kaiser_384[316] = 0.909211; 
    kaiser_384[317] = 0.907778; 
    kaiser_384[318] = 0.906335; 
    kaiser_384[319] = 0.904881; 
    kaiser_384[320] = 0.903418; 
    kaiser_384[321] = 0.901944; 
    kaiser_384[322] = 0.900460; 
    kaiser_384[323] = 0.898966; 
    kaiser_384[324] = 0.897461; 
    kaiser_384[325] = 0.895947; 
    kaiser_384[326] = 0.894422; 
    kaiser_384[327] = 0.892888; 
    kaiser_384[328] = 0.891343; 
    kaiser_384[329] = 0.889789; 
    kaiser_384[330] = 0.888224; 
    kaiser_384[331] = 0.886650; 
    kaiser_384[332] = 0.885065; 
    kaiser_384[333] = 0.883471; 
    kaiser_384[334] = 0.881867; 
    kaiser_384[335] = 0.880253; 
    kaiser_384[336] = 0.878629; 
    kaiser_384[337] = 0.876996; 
    kaiser_384[338] = 0.875352; 
    kaiser_384[339] = 0.873700; 
    kaiser_384[340] = 0.872037; 
    kaiser_384[341] = 0.870365; 
    kaiser_384[342] = 0.868683; 
    kaiser_384[343] = 0.866992; 
    kaiser_384[344] = 0.865291; 
    kaiser_384[345] = 0.863581; 
    kaiser_384[346] = 0.861861; 
    kaiser_384[347] = 0.860132; 
    kaiser_384[348] = 0.858393; 
    kaiser_384[349] = 0.856646; 
    kaiser_384[350] = 0.854888; 
    kaiser_384[351] = 0.853122; 
    kaiser_384[352] = 0.851346; 
    kaiser_384[353] = 0.849561; 
    kaiser_384[354] = 0.847767; 
    kaiser_384[355] = 0.845964; 
    kaiser_384[356] = 0.844151; 
    kaiser_384[357] = 0.842330; 
    kaiser_384[358] = 0.840499; 
    kaiser_384[359] = 0.838659; 
    kaiser_384[360] = 0.836811; 
    kaiser_384[361] = 0.834953; 
    kaiser_384[362] = 0.833087; 
    kaiser_384[363] = 0.831212; 
    kaiser_384[364] = 0.829328; 
    kaiser_384[365] = 0.827435; 
    kaiser_384[366] = 0.825533; 
    kaiser_384[367] = 0.823623; 
    kaiser_384[368] = 0.821704; 
    kaiser_384[369] = 0.819776; 
    kaiser_384[370] = 0.817840; 
    kaiser_384[371] = 0.815895; 
    kaiser_384[372] = 0.813941; 
    kaiser_384[373] = 0.811979; 
    kaiser_384[374] = 0.810009; 
    kaiser_384[375] = 0.808030; 
    kaiser_384[376] = 0.806043; 
    kaiser_384[377] = 0.804047; 
    kaiser_384[378] = 0.802044; 
    kaiser_384[379] = 0.800032; 
    kaiser_384[380] = 0.798011; 
    kaiser_384[381] = 0.795983; 
    kaiser_384[382] = 0.793946; 
    kaiser_384[383] = 0.791901; 
    kaiser_640[0] = 0.789848; 
    kaiser_640[1] = 0.791081; 
    kaiser_640[2] = 0.792311; 
    kaiser_640[3] = 0.793538; 
    kaiser_640[4] = 0.794762; 
    kaiser_640[5] = 0.795983; 
    kaiser_640[6] = 0.797201; 
    kaiser_640[7] = 0.798416; 
    kaiser_640[8] = 0.799628; 
    kaiser_640[9] = 0.800837; 
    kaiser_640[10] = 0.802044; 
    kaiser_640[11] = 0.803247; 
    kaiser_640[12] = 0.804447; 
    kaiser_640[13] = 0.805645; 
    kaiser_640[14] = 0.806839; 
    kaiser_640[15] = 0.808030; 
    kaiser_640[16] = 0.809219; 
    kaiser_640[17] = 0.810404; 
    kaiser_640[18] = 0.811586; 
    kaiser_640[19] = 0.812765; 
    kaiser_640[20] = 0.813941; 
    kaiser_640[21] = 0.815114; 
    kaiser_640[22] = 0.816284; 
    kaiser_640[23] = 0.817451; 
    kaiser_640[24] = 0.818615; 
    kaiser_640[25] = 0.819776; 
    kaiser_640[26] = 0.820934; 
    kaiser_640[27] = 0.822088; 
    kaiser_640[28] = 0.823240; 
    kaiser_640[29] = 0.824388; 
    kaiser_640[30] = 0.825533; 
    kaiser_640[31] = 0.826675; 
    kaiser_640[32] = 0.827814; 
    kaiser_640[33] = 0.828950; 
    kaiser_640[34] = 0.830082; 
    kaiser_640[35] = 0.831212; 
    kaiser_640[36] = 0.832338; 
    kaiser_640[37] = 0.833461; 
    kaiser_640[38] = 0.834581; 
    kaiser_640[39] = 0.835697; 
    kaiser_640[40] = 0.836811; 
    kaiser_640[41] = 0.837921; 
    kaiser_640[42] = 0.839028; 
    kaiser_640[43] = 0.840132; 
    kaiser_640[44] = 0.841232; 
    kaiser_640[45] = 0.842330; 
    kaiser_640[46] = 0.843424; 
    kaiser_640[47] = 0.844514; 
    kaiser_640[48] = 0.845602; 
    kaiser_640[49] = 0.846686; 
    kaiser_640[50] = 0.847767; 
    kaiser_640[51] = 0.848844; 
    kaiser_640[52] = 0.849919; 
    kaiser_640[53] = 0.850990; 
    kaiser_640[54] = 0.852057; 
    kaiser_640[55] = 0.853122; 
    kaiser_640[56] = 0.854183; 
    kaiser_640[57] = 0.855241; 
    kaiser_640[58] = 0.856295; 
    kaiser_640[59] = 0.857346; 
    kaiser_640[60] = 0.858393; 
    kaiser_640[61] = 0.859438; 
    kaiser_640[62] = 0.860479; 
    kaiser_640[63] = 0.861516; 
    kaiser_640[64] = 0.862550; 
    kaiser_640[65] = 0.863581; 
    kaiser_640[66] = 0.864608; 
    kaiser_640[67] = 0.865632; 
    kaiser_640[68] = 0.866653; 
    kaiser_640[69] = 0.867670; 
    kaiser_640[70] = 0.868683; 
    kaiser_640[71] = 0.869693; 
    kaiser_640[72] = 0.870700; 
    kaiser_640[73] = 0.871703; 
    kaiser_640[74] = 0.872703; 
    kaiser_640[75] = 0.873700; 
    kaiser_640[76] = 0.874692; 
    kaiser_640[77] = 0.875682; 
    kaiser_640[78] = 0.876668; 
    kaiser_640[79] = 0.877650; 
    kaiser_640[80] = 0.878629; 
    kaiser_640[81] = 0.879604; 
    kaiser_640[82] = 0.880576; 
    kaiser_640[83] = 0.881545; 
    kaiser_640[84] = 0.882510; 
    kaiser_640[85] = 0.883471; 
    kaiser_640[86] = 0.884429; 
    kaiser_640[87] = 0.885383; 
    kaiser_640[88] = 0.886333; 
    kaiser_640[89] = 0.887281; 
    kaiser_640[90] = 0.888224; 
    kaiser_640[91] = 0.889164; 
    kaiser_640[92] = 0.890100; 
    kaiser_640[93] = 0.891033; 
    kaiser_640[94] = 0.891962; 
    kaiser_640[95] = 0.892888; 
    kaiser_640[96] = 0.893810; 
    kaiser_640[97] = 0.894728; 
    kaiser_640[98] = 0.895643; 
    kaiser_640[99] = 0.896554; 
    kaiser_640[100] = 0.897461; 
    kaiser_640[101] = 0.898365; 
    kaiser_640[102] = 0.899265; 
    kaiser_640[103] = 0.900162; 
    kaiser_640[104] = 0.901055; 
    kaiser_640[105] = 0.901944; 
    kaiser_640[106] = 0.902830; 
    kaiser_640[107] = 0.903711; 
    kaiser_640[108] = 0.904590; 
    kaiser_640[109] = 0.905464; 
    kaiser_640[110] = 0.906335; 
    kaiser_640[111] = 0.907202; 
    kaiser_640[112] = 0.908065; 
    kaiser_640[113] = 0.908925; 
    kaiser_640[114] = 0.909781; 
    kaiser_640[115] = 0.910633; 
    kaiser_640[116] = 0.911481; 
    kaiser_640[117] = 0.912326; 
    kaiser_640[118] = 0.913167; 
    kaiser_640[119] = 0.914004; 
    kaiser_640[120] = 0.914838; 
    kaiser_640[121] = 0.915668; 
    kaiser_640[122] = 0.916494; 
    kaiser_640[123] = 0.917316; 
    kaiser_640[124] = 0.918134; 
    kaiser_640[125] = 0.918949; 
    kaiser_640[126] = 0.919760; 
    kaiser_640[127] = 0.920567; 
    kaiser_640[128] = 0.921370; 
    kaiser_640[129] = 0.922169; 
    kaiser_640[130] = 0.922965; 
    kaiser_640[131] = 0.923757; 
    kaiser_640[132] = 0.924545; 
    kaiser_640[133] = 0.925329; 
    kaiser_640[134] = 0.926109; 
    kaiser_640[135] = 0.926885; 
    kaiser_640[136] = 0.927658; 
    kaiser_640[137] = 0.928427; 
    kaiser_640[138] = 0.929192; 
    kaiser_640[139] = 0.929953; 
    kaiser_640[140] = 0.930710; 
    kaiser_640[141] = 0.931463; 
    kaiser_640[142] = 0.932213; 
    kaiser_640[143] = 0.932958; 
    kaiser_640[144] = 0.933700; 
    kaiser_640[145] = 0.934438; 
    kaiser_640[146] = 0.935171; 
    kaiser_640[147] = 0.935901; 
    kaiser_640[148] = 0.936627; 
    kaiser_640[149] = 0.937350; 
    kaiser_640[150] = 0.938068; 
    kaiser_640[151] = 0.938782; 
    kaiser_640[152] = 0.939492; 
    kaiser_640[153] = 0.940199; 
    kaiser_640[154] = 0.940901; 
    kaiser_640[155] = 0.941600; 
    kaiser_640[156] = 0.942294; 
    kaiser_640[157] = 0.942985; 
    kaiser_640[158] = 0.943672; 
    kaiser_640[159] = 0.944354; 
    kaiser_640[160] = 0.945033; 
    kaiser_640[161] = 0.945708; 
    kaiser_640[162] = 0.946379; 
    kaiser_640[163] = 0.947046; 
    kaiser_640[164] = 0.947708; 
    kaiser_640[165] = 0.948367; 
    kaiser_640[166] = 0.949022; 
    kaiser_640[167] = 0.949673; 
    kaiser_640[168] = 0.950320; 
    kaiser_640[169] = 0.950963; 
    kaiser_640[170] = 0.951601; 
    kaiser_640[171] = 0.952236; 
    kaiser_640[172] = 0.952867; 
    kaiser_640[173] = 0.953494; 
    kaiser_640[174] = 0.954116; 
    kaiser_640[175] = 0.954735; 
    kaiser_640[176] = 0.955350; 
    kaiser_640[177] = 0.955960; 
    kaiser_640[178] = 0.956567; 
    kaiser_640[179] = 0.957169; 
    kaiser_640[180] = 0.957768; 
    kaiser_640[181] = 0.958362; 
    kaiser_640[182] = 0.958952; 
    kaiser_640[183] = 0.959538; 
    kaiser_640[184] = 0.960121; 
    kaiser_640[185] = 0.960699; 
    kaiser_640[186] = 0.961273; 
    kaiser_640[187] = 0.961842; 
    kaiser_640[188] = 0.962408; 
    kaiser_640[189] = 0.962970; 
    kaiser_640[190] = 0.963528; 
    kaiser_640[191] = 0.964081; 
    kaiser_640[192] = 0.964630; 
    kaiser_640[193] = 0.965176; 
    kaiser_640[194] = 0.965717; 
    kaiser_640[195] = 0.966254; 
    kaiser_640[196] = 0.966787; 
    kaiser_640[197] = 0.967316; 
    kaiser_640[198] = 0.967840; 
    kaiser_640[199] = 0.968361; 
    kaiser_640[200] = 0.968877; 
    kaiser_640[201] = 0.969389; 
    kaiser_640[202] = 0.969898; 
    kaiser_640[203] = 0.970402; 
    kaiser_640[204] = 0.970901; 
    kaiser_640[205] = 0.971397; 
    kaiser_640[206] = 0.971889; 
    kaiser_640[207] = 0.972376; 
    kaiser_640[208] = 0.972859; 
    kaiser_640[209] = 0.973338; 
    kaiser_640[210] = 0.973813; 
    kaiser_640[211] = 0.974284; 
    kaiser_640[212] = 0.974750; 
    kaiser_640[213] = 0.975212; 
    kaiser_640[214] = 0.975670; 
    kaiser_640[215] = 0.976124; 
    kaiser_640[216] = 0.976574; 
    kaiser_640[217] = 0.977020; 
    kaiser_640[218] = 0.977461; 
    kaiser_640[219] = 0.977898; 
    kaiser_640[220] = 0.978331; 
    kaiser_640[221] = 0.978760; 
    kaiser_640[222] = 0.979184; 
    kaiser_640[223] = 0.979605; 
    kaiser_640[224] = 0.980021; 
    kaiser_640[225] = 0.980433; 
    kaiser_640[226] = 0.980840; 
    kaiser_640[227] = 0.981244; 
    kaiser_640[228] = 0.981643; 
    kaiser_640[229] = 0.982038; 
    kaiser_640[230] = 0.982429; 
    kaiser_640[231] = 0.982815; 
    kaiser_640[232] = 0.983197; 
    kaiser_640[233] = 0.983575; 
    kaiser_640[234] = 0.983949; 
    kaiser_640[235] = 0.984319; 
    kaiser_640[236] = 0.984684; 
    kaiser_640[237] = 0.985045; 
    kaiser_640[238] = 0.985402; 
    kaiser_640[239] = 0.985754; 
    kaiser_640[240] = 0.986103; 
    kaiser_640[241] = 0.986447; 
    kaiser_640[242] = 0.986786; 
    kaiser_640[243] = 0.987122; 
    kaiser_640[244] = 0.987453; 
    kaiser_640[245] = 0.987780; 
    kaiser_640[246] = 0.988103; 
    kaiser_640[247] = 0.988421; 
    kaiser_640[248] = 0.988735; 
    kaiser_640[249] = 0.989045; 
    kaiser_640[250] = 0.989350; 
    kaiser_640[251] = 0.989652; 
    kaiser_640[252] = 0.989949; 
    kaiser_640[253] = 0.990241; 
    kaiser_640[254] = 0.990530; 
    kaiser_640[255] = 0.990814; 
    kaiser_640[256] = 0.991094; 
    kaiser_640[257] = 0.991369; 
    kaiser_640[258] = 0.991640; 
    kaiser_640[259] = 0.991907; 
    kaiser_640[260] = 0.992170; 
    kaiser_640[261] = 0.992428; 
    kaiser_640[262] = 0.992682; 
    kaiser_640[263] = 0.992932; 
    kaiser_640[264] = 0.993177; 
    kaiser_640[265] = 0.993418; 
    kaiser_640[266] = 0.993655; 
    kaiser_640[267] = 0.993887; 
    kaiser_640[268] = 0.994116; 
    kaiser_640[269] = 0.994339; 
    kaiser_640[270] = 0.994559; 
    kaiser_640[271] = 0.994774; 
    kaiser_640[272] = 0.994985; 
    kaiser_640[273] = 0.995191; 
    kaiser_640[274] = 0.995394; 
    kaiser_640[275] = 0.995591; 
    kaiser_640[276] = 0.995785; 
    kaiser_640[277] = 0.995974; 
    kaiser_640[278] = 0.996159; 
    kaiser_640[279] = 0.996340; 
    kaiser_640[280] = 0.996516; 
    kaiser_640[281] = 0.996688; 
    kaiser_640[282] = 0.996855; 
    kaiser_640[283] = 0.997018; 
    kaiser_640[284] = 0.997177; 
    kaiser_640[285] = 0.997332; 
    kaiser_640[286] = 0.997482; 
    kaiser_640[287] = 0.997628; 
    kaiser_640[288] = 0.997769; 
    kaiser_640[289] = 0.997907; 
    kaiser_640[290] = 0.998039; 
    kaiser_640[291] = 0.998168; 
    kaiser_640[292] = 0.998292; 
    kaiser_640[293] = 0.998412; 
    kaiser_640[294] = 0.998527; 
    kaiser_640[295] = 0.998638; 
    kaiser_640[296] = 0.998745; 
    kaiser_640[297] = 0.998847; 
    kaiser_640[298] = 0.998945; 
    kaiser_640[299] = 0.999039; 
    kaiser_640[300] = 0.999128; 
    kaiser_640[301] = 0.999213; 
    kaiser_640[302] = 0.999294; 
    kaiser_640[303] = 0.999370; 
    kaiser_640[304] = 0.999442; 
    kaiser_640[305] = 0.999510; 
    kaiser_640[306] = 0.999573; 
    kaiser_640[307] = 0.999632; 
    kaiser_640[308] = 0.999686; 
    kaiser_640[309] = 0.999736; 
    kaiser_640[310] = 0.999782; 
    kaiser_640[311] = 0.999823; 
    kaiser_640[312] = 0.999861; 
    kaiser_640[313] = 0.999893; 
    kaiser_640[314] = 0.999922; 
    kaiser_640[315] = 0.999946; 
    kaiser_640[316] = 0.999965; 
    kaiser_640[317] = 0.999980; 
    kaiser_640[318] = 0.999991; 
    kaiser_640[319] = 0.999998; 
    kaiser_640[320] = 1.000000; 
    kaiser_640[321] = 0.999998; 
    kaiser_640[322] = 0.999991; 
    kaiser_640[323] = 0.999980; 
    kaiser_640[324] = 0.999965; 
    kaiser_640[325] = 0.999946; 
    kaiser_640[326] = 0.999922; 
    kaiser_640[327] = 0.999893; 
    kaiser_640[328] = 0.999861; 
    kaiser_640[329] = 0.999823; 
    kaiser_640[330] = 0.999782; 
    kaiser_640[331] = 0.999736; 
    kaiser_640[332] = 0.999686; 
    kaiser_640[333] = 0.999632; 
    kaiser_640[334] = 0.999573; 
    kaiser_640[335] = 0.999510; 
    kaiser_640[336] = 0.999442; 
    kaiser_640[337] = 0.999370; 
    kaiser_640[338] = 0.999294; 
    kaiser_640[339] = 0.999213; 
    kaiser_640[340] = 0.999128; 
    kaiser_640[341] = 0.999039; 
    kaiser_640[342] = 0.998945; 
    kaiser_640[343] = 0.998847; 
    kaiser_640[344] = 0.998745; 
    kaiser_640[345] = 0.998638; 
    kaiser_640[346] = 0.998527; 
    kaiser_640[347] = 0.998412; 
    kaiser_640[348] = 0.998292; 
    kaiser_640[349] = 0.998168; 
    kaiser_640[350] = 0.998039; 
    kaiser_640[351] = 0.997907; 
    kaiser_640[352] = 0.997769; 
    kaiser_640[353] = 0.997628; 
    kaiser_640[354] = 0.997482; 
    kaiser_640[355] = 0.997332; 
    kaiser_640[356] = 0.997177; 
    kaiser_640[357] = 0.997018; 
    kaiser_640[358] = 0.996855; 
    kaiser_640[359] = 0.996688; 
    kaiser_640[360] = 0.996516; 
    kaiser_640[361] = 0.996340; 
    kaiser_640[362] = 0.996159; 
    kaiser_640[363] = 0.995974; 
    kaiser_640[364] = 0.995785; 
    kaiser_640[365] = 0.995591; 
    kaiser_640[366] = 0.995394; 
    kaiser_640[367] = 0.995191; 
    kaiser_640[368] = 0.994985; 
    kaiser_640[369] = 0.994774; 
    kaiser_640[370] = 0.994559; 
    kaiser_640[371] = 0.994339; 
    kaiser_640[372] = 0.994116; 
    kaiser_640[373] = 0.993887; 
    kaiser_640[374] = 0.993655; 
    kaiser_640[375] = 0.993418; 
    kaiser_640[376] = 0.993177; 
    kaiser_640[377] = 0.992932; 
    kaiser_640[378] = 0.992682; 
    kaiser_640[379] = 0.992428; 
    kaiser_640[380] = 0.992170; 
    kaiser_640[381] = 0.991907; 
    kaiser_640[382] = 0.991640; 
    kaiser_640[383] = 0.991369; 
    kaiser_640[384] = 0.991094; 
    kaiser_640[385] = 0.990814; 
    kaiser_640[386] = 0.990530; 
    kaiser_640[387] = 0.990241; 
    kaiser_640[388] = 0.989949; 
    kaiser_640[389] = 0.989652; 
    kaiser_640[390] = 0.989350; 
    kaiser_640[391] = 0.989045; 
    kaiser_640[392] = 0.988735; 
    kaiser_640[393] = 0.988421; 
    kaiser_640[394] = 0.988103; 
    kaiser_640[395] = 0.987780; 
    kaiser_640[396] = 0.987453; 
    kaiser_640[397] = 0.987122; 
    kaiser_640[398] = 0.986786; 
    kaiser_640[399] = 0.986447; 
    kaiser_640[400] = 0.986103; 
    kaiser_640[401] = 0.985754; 
    kaiser_640[402] = 0.985402; 
    kaiser_640[403] = 0.985045; 
    kaiser_640[404] = 0.984684; 
    kaiser_640[405] = 0.984319; 
    kaiser_640[406] = 0.983949; 
    kaiser_640[407] = 0.983575; 
    kaiser_640[408] = 0.983197; 
    kaiser_640[409] = 0.982815; 
    kaiser_640[410] = 0.982429; 
    kaiser_640[411] = 0.982038; 
    kaiser_640[412] = 0.981643; 
    kaiser_640[413] = 0.981244; 
    kaiser_640[414] = 0.980840; 
    kaiser_640[415] = 0.980433; 
    kaiser_640[416] = 0.980021; 
    kaiser_640[417] = 0.979605; 
    kaiser_640[418] = 0.979184; 
    kaiser_640[419] = 0.978760; 
    kaiser_640[420] = 0.978331; 
    kaiser_640[421] = 0.977898; 
    kaiser_640[422] = 0.977461; 
    kaiser_640[423] = 0.977020; 
    kaiser_640[424] = 0.976574; 
    kaiser_640[425] = 0.976124; 
    kaiser_640[426] = 0.975670; 
    kaiser_640[427] = 0.975212; 
    kaiser_640[428] = 0.974750; 
    kaiser_640[429] = 0.974284; 
    kaiser_640[430] = 0.973813; 
    kaiser_640[431] = 0.973338; 
    kaiser_640[432] = 0.972859; 
    kaiser_640[433] = 0.972376; 
    kaiser_640[434] = 0.971889; 
    kaiser_640[435] = 0.971397; 
    kaiser_640[436] = 0.970901; 
    kaiser_640[437] = 0.970402; 
    kaiser_640[438] = 0.969898; 
    kaiser_640[439] = 0.969389; 
    kaiser_640[440] = 0.968877; 
    kaiser_640[441] = 0.968361; 
    kaiser_640[442] = 0.967840; 
    kaiser_640[443] = 0.967316; 
    kaiser_640[444] = 0.966787; 
    kaiser_640[445] = 0.966254; 
    kaiser_640[446] = 0.965717; 
    kaiser_640[447] = 0.965176; 
    kaiser_640[448] = 0.964630; 
    kaiser_640[449] = 0.964081; 
    kaiser_640[450] = 0.963528; 
    kaiser_640[451] = 0.962970; 
    kaiser_640[452] = 0.962408; 
    kaiser_640[453] = 0.961842; 
    kaiser_640[454] = 0.961273; 
    kaiser_640[455] = 0.960699; 
    kaiser_640[456] = 0.960121; 
    kaiser_640[457] = 0.959538; 
    kaiser_640[458] = 0.958952; 
    kaiser_640[459] = 0.958362; 
    kaiser_640[460] = 0.957768; 
    kaiser_640[461] = 0.957169; 
    kaiser_640[462] = 0.956567; 
    kaiser_640[463] = 0.955960; 
    kaiser_640[464] = 0.955350; 
    kaiser_640[465] = 0.954735; 
    kaiser_640[466] = 0.954116; 
    kaiser_640[467] = 0.953494; 
    kaiser_640[468] = 0.952867; 
    kaiser_640[469] = 0.952236; 
    kaiser_640[470] = 0.951601; 
    kaiser_640[471] = 0.950963; 
    kaiser_640[472] = 0.950320; 
    kaiser_640[473] = 0.949673; 
    kaiser_640[474] = 0.949022; 
    kaiser_640[475] = 0.948367; 
    kaiser_640[476] = 0.947708; 
    kaiser_640[477] = 0.947046; 
    kaiser_640[478] = 0.946379; 
    kaiser_640[479] = 0.945708; 
    kaiser_640[480] = 0.945033; 
    kaiser_640[481] = 0.944354; 
    kaiser_640[482] = 0.943672; 
    kaiser_640[483] = 0.942985; 
    kaiser_640[484] = 0.942294; 
    kaiser_640[485] = 0.941600; 
    kaiser_640[486] = 0.940901; 
    kaiser_640[487] = 0.940199; 
    kaiser_640[488] = 0.939492; 
    kaiser_640[489] = 0.938782; 
    kaiser_640[490] = 0.938068; 
    kaiser_640[491] = 0.937350; 
    kaiser_640[492] = 0.936627; 
    kaiser_640[493] = 0.935901; 
    kaiser_640[494] = 0.935171; 
    kaiser_640[495] = 0.934438; 
    kaiser_640[496] = 0.933700; 
    kaiser_640[497] = 0.932958; 
    kaiser_640[498] = 0.932213; 
    kaiser_640[499] = 0.931463; 
    kaiser_640[500] = 0.930710; 
    kaiser_640[501] = 0.929953; 
    kaiser_640[502] = 0.929192; 
    kaiser_640[503] = 0.928427; 
    kaiser_640[504] = 0.927658; 
    kaiser_640[505] = 0.926885; 
    kaiser_640[506] = 0.926109; 
    kaiser_640[507] = 0.925329; 
    kaiser_640[508] = 0.924545; 
    kaiser_640[509] = 0.923757; 
    kaiser_640[510] = 0.922965; 
    kaiser_640[511] = 0.922169; 
    kaiser_640[512] = 0.921370; 
    kaiser_640[513] = 0.920567; 
    kaiser_640[514] = 0.919760; 
    kaiser_640[515] = 0.918949; 
    kaiser_640[516] = 0.918134; 
    kaiser_640[517] = 0.917316; 
    kaiser_640[518] = 0.916494; 
    kaiser_640[519] = 0.915668; 
    kaiser_640[520] = 0.914838; 
    kaiser_640[521] = 0.914004; 
    kaiser_640[522] = 0.913167; 
    kaiser_640[523] = 0.912326; 
    kaiser_640[524] = 0.911481; 
    kaiser_640[525] = 0.910633; 
    kaiser_640[526] = 0.909781; 
    kaiser_640[527] = 0.908925; 
    kaiser_640[528] = 0.908065; 
    kaiser_640[529] = 0.907202; 
    kaiser_640[530] = 0.906335; 
    kaiser_640[531] = 0.905464; 
    kaiser_640[532] = 0.904590; 
    kaiser_640[533] = 0.903711; 
    kaiser_640[534] = 0.902830; 
    kaiser_640[535] = 0.901944; 
    kaiser_640[536] = 0.901055; 
    kaiser_640[537] = 0.900162; 
    kaiser_640[538] = 0.899265; 
    kaiser_640[539] = 0.898365; 
    kaiser_640[540] = 0.897461; 
    kaiser_640[541] = 0.896554; 
    kaiser_640[542] = 0.895643; 
    kaiser_640[543] = 0.894728; 
    kaiser_640[544] = 0.893810; 
    kaiser_640[545] = 0.892888; 
    kaiser_640[546] = 0.891962; 
    kaiser_640[547] = 0.891033; 
    kaiser_640[548] = 0.890100; 
    kaiser_640[549] = 0.889164; 
    kaiser_640[550] = 0.888224; 
    kaiser_640[551] = 0.887281; 
    kaiser_640[552] = 0.886333; 
    kaiser_640[553] = 0.885383; 
    kaiser_640[554] = 0.884429; 
    kaiser_640[555] = 0.883471; 
    kaiser_640[556] = 0.882510; 
    kaiser_640[557] = 0.881545; 
    kaiser_640[558] = 0.880576; 
    kaiser_640[559] = 0.879604; 
    kaiser_640[560] = 0.878629; 
    kaiser_640[561] = 0.877650; 
    kaiser_640[562] = 0.876668; 
    kaiser_640[563] = 0.875682; 
    kaiser_640[564] = 0.874692; 
    kaiser_640[565] = 0.873700; 
    kaiser_640[566] = 0.872703; 
    kaiser_640[567] = 0.871703; 
    kaiser_640[568] = 0.870700; 
    kaiser_640[569] = 0.869693; 
    kaiser_640[570] = 0.868683; 
    kaiser_640[571] = 0.867670; 
    kaiser_640[572] = 0.866653; 
    kaiser_640[573] = 0.865632; 
    kaiser_640[574] = 0.864608; 
    kaiser_640[575] = 0.863581; 
    kaiser_640[576] = 0.862550; 
    kaiser_640[577] = 0.861516; 
    kaiser_640[578] = 0.860479; 
    kaiser_640[579] = 0.859438; 
    kaiser_640[580] = 0.858393; 
    kaiser_640[581] = 0.857346; 
    kaiser_640[582] = 0.856295; 
    kaiser_640[583] = 0.855241; 
    kaiser_640[584] = 0.854183; 
    kaiser_640[585] = 0.853122; 
    kaiser_640[586] = 0.852057; 
    kaiser_640[587] = 0.850990; 
    kaiser_640[588] = 0.849919; 
    kaiser_640[589] = 0.848844; 
    kaiser_640[590] = 0.847767; 
    kaiser_640[591] = 0.846686; 
    kaiser_640[592] = 0.845602; 
    kaiser_640[593] = 0.844514; 
    kaiser_640[594] = 0.843424; 
    kaiser_640[595] = 0.842330; 
    kaiser_640[596] = 0.841232; 
    kaiser_640[597] = 0.840132; 
    kaiser_640[598] = 0.839028; 
    kaiser_640[599] = 0.837921; 
    kaiser_640[600] = 0.836811; 
    kaiser_640[601] = 0.835697; 
    kaiser_640[602] = 0.834581; 
    kaiser_640[603] = 0.833461; 
    kaiser_640[604] = 0.832338; 
    kaiser_640[605] = 0.831212; 
    kaiser_640[606] = 0.830082; 
    kaiser_640[607] = 0.828950; 
    kaiser_640[608] = 0.827814; 
    kaiser_640[609] = 0.826675; 
    kaiser_640[610] = 0.825533; 
    kaiser_640[611] = 0.824388; 
    kaiser_640[612] = 0.823240; 
    kaiser_640[613] = 0.822088; 
    kaiser_640[614] = 0.820934; 
    kaiser_640[615] = 0.819776; 
    kaiser_640[616] = 0.818615; 
    kaiser_640[617] = 0.817451; 
    kaiser_640[618] = 0.816284; 
    kaiser_640[619] = 0.815114; 
    kaiser_640[620] = 0.813941; 
    kaiser_640[621] = 0.812765; 
    kaiser_640[622] = 0.811586; 
    kaiser_640[623] = 0.810404; 
    kaiser_640[624] = 0.809219; 
    kaiser_640[625] = 0.808030; 
    kaiser_640[626] = 0.806839; 
    kaiser_640[627] = 0.805645; 
    kaiser_640[628] = 0.804447; 
    kaiser_640[629] = 0.803247; 
    kaiser_640[630] = 0.802044; 
    kaiser_640[631] = 0.800837; 
    kaiser_640[632] = 0.799628; 
    kaiser_640[633] = 0.798416; 
    kaiser_640[634] = 0.797201; 
    kaiser_640[635] = 0.795983; 
    kaiser_640[636] = 0.794762; 
    kaiser_640[637] = 0.793538; 
    kaiser_640[638] = 0.792311; 
    kaiser_640[639] = 0.791081; 
    
}

#endif


void adjust_frequency_response(u32 start_address, pxc_dfb_driver_data mdrv)
{

    D_DEBUG ("try to adjust the frequency response for \n");

    REG_WRITE_AND_POLL (DISP_HD_RAM_ADDR, start_address);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000040);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00003ffc);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x007ff13f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd010ff90);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x04ffd002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00005ff8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00efe23f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xb020ff10);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x09ffb003);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00008ff4);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x015fd43f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x7031fea0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0dff8005);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0000aff0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01cfc63f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x3042fe20);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x11ff5007);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0000cfec);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x022fb93e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xe054fdb0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x16ff3008);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0000ffe8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x028fad3e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x7065fd30);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1aff000a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00011fe5);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x02efa13e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0078fcb0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1ffed00c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00013fe1);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x034f963d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xa08afc40);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x23fea00d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00015fde);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x039f8b3d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x109dfbc0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x27fe800f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00017fdb);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x03ff813c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x80b0fb40);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x2cfe5010);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00019fd8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x044f783b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xf0c4fad0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x30fe2012);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0001bfd5);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x048f6f3b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x40d8fa50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x34fe0014);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0001dfd2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x04df673a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xa0ecf9e0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x38fdd015);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0001efcf);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x051f5f39);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xf100f960);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x3cfdb017);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00020fcd);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x055f5939);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1114f8f0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x40fd8018);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00021fcb);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x058f5238);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x3128f890);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x44fd601a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00022fc9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x05cf4d37);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x713df810);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x48fd401b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00024fc7);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x05ff4736);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xb153f7b0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x4bfd201d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00025fc5);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x062f4335);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xa167f740);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x4ffcf01e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00026fc3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x064f3f34);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xc17cf6e0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x52fcd01f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00026fc2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x066f3b33);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xc191f680);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x55fcb020);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00027fc1);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x068f3932);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xc1a6f620);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x59fca022);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00028fc0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06af3631);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xb1bbf5c0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x5bfc8023);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00028fbf);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06bf3430);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xa1d0f570);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x5efc6024);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00029fbe);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06cf332f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x91e5f520);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x61fc5025);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00029fbe);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06df332e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x61f9f4e0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x63fc3025);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00029fbd);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06ef322d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x520ff490);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x65fc2026);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00029fbd);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06ef322c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x3224f450);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x67fc1027);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00029fbd);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06ef332a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xf238f410);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x69fc0028);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00029fbd);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06ef3429);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd24df3d0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6afbf028);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00029fbd);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06df3628);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x8260f3b0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6bfbe029);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00029fbe);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06cf3827);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x4274f380);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6cfbe029);

}

void set_filter_for_vid0_nonscale(pxc_dfb_driver_data mdrv)
{

    D_DEBUG ("try to set filter for vid0  \n");

    REG_WRITE_AND_POLL (DISP_HD_RAM_ADDR, 0x10000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x00004000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x7ff13fb0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x10ff8005);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xffe23f50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x20ff1009);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6fd43ef0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x31fe900e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xcfc63e80);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x42fe2012);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x3fb93e00);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x53fda017);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x9fad3d80);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x64fd201b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xffa23ce0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x76fcb020);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000003);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x5f973c50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x88fc3024);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000003);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xaf8d3bb0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x9afbc029);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000003);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xff833af0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xadfb402d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000004);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x4f7a3a50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xbffad031);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000004);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x9f723990);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xd2fa6036);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000004);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xdf6a38b0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xe5f9f03a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000005);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1f6337e0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xf8f9803e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000005);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x5f5d3711);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0bf91042);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000005);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x9f573631);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1ff8a045);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000005);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xcf523541);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x32f84049);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000005);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xff4d3451);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x45f7e04c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1f493351);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x58f77050);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x4f453271);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6cf71053);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6f423161);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x7ff6c056);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x8f3f3061);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x93f66059);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x9f3d2f61);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xa7f6105c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xbf3c2e41);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xbaf5c05e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xcf3b2d31);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xcef58061);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xcf3a2c21);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xe2f53063);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xdf3a2b01);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xf5f4f065);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xdf3a29e2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x08f4b067);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xdf3b28c2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1bf48068);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xdf3c2792);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x2ef4506a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xcf3e2672);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x41f4206b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xcf402552);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x55f4006c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_ADDR, 0x20000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x00004000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x3ff33ff0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0effd000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6fe73fe0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1cff9000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x9fdb3fd0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x2aff5000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xcfd03fa0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x3aff1000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xefc53f60);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x49fed001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1fbb3f10);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x5afe9001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x3fb23eb0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6afe5001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x5fa93e40);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x7cfe1002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x7fa13dd0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x8dfdc002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x8f993d50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xa0fd8003);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xaf923cb0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xb2fd3004);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xbf8c3c10);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xc5fce004);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xcf863b70);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xd9fc9005);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xdf803ac0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xedfc4006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xef7c39e1);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x01fbf007);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xff783911);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x15fba008);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xff743851);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x2bfb5009);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0f713761);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x40fb0009);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0f6e3661);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x55fac00a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0f6c3561);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6afa700c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0f6b3471);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x80fa200d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0f693361);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x96f9d00e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000002);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0f693251);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xacf9800f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xff683121);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xc2f94010);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xff693001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xd8f8f011);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xef692ed1);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xeff8b012);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xdf6a2da2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x06f86014);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xdf6c2c52);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1bf83015);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xcf6e2b12);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x31f7f016);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xbf7029c2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x46f7b017);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xaf722872);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x5cf78018);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000001);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x9f752712);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x71f75019);
    REG_WRITE_AND_POLL (DISP_HD_RAM_ADDR, 0x30000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000040);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00004ffb);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x008ff03f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd010ff80);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x05ffc003);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00008ff6);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x010fe13f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x8021ff00);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0bff8006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0000cff0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x017fd33f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x3032fe80);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x10ff400a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0000ffeb);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01efc53e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd043fe00);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x15ff000d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00013fe7);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x026fb83e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6055fd70);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1bfec010);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00017fe2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x02cfab3e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1067fcf0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x20fe8013);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0001afdd);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x033f9f3d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x7079fc70);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x25fe4016);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0001efd9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x039f943c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xe08bfbf0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x2afe1019);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00021fd5);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x03ff893c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x409efb70);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x2ffdd01c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00024fd1);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x045f7f3b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xb0b1faf0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x34fd901f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00027fcd);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x04bf753b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x10c4fa70);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x39fd6022);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0002afc9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x050f6d3a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x40d7fa00);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x3efd2024);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0002dfc6);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x055f6439);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xa0ebf980);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x43fcf027);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0002ffc2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x05af5d38);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xe0fff900);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x48fcb02a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00032fbf);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x05ef5637);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xf112f890);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x4cfc802c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00034fbc);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x062f5037);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x2126f820);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x50fc502e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00036fb9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x066f4a36);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x513af7b0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x54fc2031);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00038fb7);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x069f4435);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x714ff740);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x58fbf033);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0003afb4);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06cf4034);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x7162f6e0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x5cfbd035);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0003cfb2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06ff3c33);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x8176f670);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x60fba037);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0003dfb0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x072f3832);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x818bf610);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x63fb8039);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0003ffae);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x074f3531);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x919ff5b0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x67fb503b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00040fad);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x076f3330);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x81b3f560);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6afb303c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00041fac);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x078f312f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x61c7f510);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6cfb203d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00042fab);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x079f302e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x51dbf4c0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6ffb003f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00043faa);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07af2f2d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x51f0f470);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x71fae040);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00043fa9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07bf2f2c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1202f440);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x73fad041);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00044fa8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07cf2e2b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0217f3f0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x76fab042);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00044fa8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07cf2f29);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd22af3c0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x77fab043);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00044fa8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07cf3028);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xb23ff390);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x79faa043);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00044fa8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07bf3227);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x9252f360);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x7afa9044);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00044fa8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07bf3326);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6266f330);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x7bfa8044);
    REG_WRITE_AND_POLL (DISP_HD_RAM_ADDR, 0x40000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000040);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x003ff33f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xf00effd0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x006fe73f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xe01cff90);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x009fdb3f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd02aff50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00cfd03f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xa03aff10);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00efc53f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6049fed0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x01000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x011fbb3f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x105afe90);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x01000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x013fb23e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xb06afe50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x01000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x015fa93e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x407cfe10);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x02000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x017fa13d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd08dfdc0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x02000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x018f993d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x50a0fd80);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x03000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01af923c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xb0b2fd30);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x04000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01bf8c3c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x10c5fce0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x04000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01cf863b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x70d9fc90);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x05000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01df803a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xc0edfc40);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x06000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ef7c39);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xe101fbf0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x07000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ff7839);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1115fba0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x08000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ff7438);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x512bfb50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x09000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f7137);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6140fb00);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x09000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f6e36);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6155fac0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0a000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f6c35);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x616afa70);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0c000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f6b34);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x7180fa20);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0d000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f6933);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6196f9d0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0e000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f6932);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x51acf980);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0f000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ff6831);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x21c2f940);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x10000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ff6930);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x01d8f8f0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x11000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ef692e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd1eff8b0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x12000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01df6a2d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xa206f860);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x14000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01df6c2c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x521bf830);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x15000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01cf6e2b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1231f7f0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x16000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01bf7029);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xc246f7b0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x17000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01af7228);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x725cf780);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x18000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x019f7527);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1271f750);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x19000000);
    
    return;    
}

void set_filter_for_vid1_nonscale(pxc_dfb_driver_data mdrv)
{
    D_DEBUG ("try to set filter for vid1  \n");

    REG_WRITE_AND_POLL (DISP_HD_RAM_ADDR, 0x90000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x00000004);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000ff14);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x06010ff9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fe24);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0c020ff2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fd34);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x11031fea);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fc54);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x14043fe3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fb74);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x17056fdb);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000faa4);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1a069fd3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f9d4);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1b07dfcb);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f914);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1b092fc3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f854);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1a0a7fba);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f794);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x180bdfb2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f6e4);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x150d4fa9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f644);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x110ebfa1);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f5a4);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0c102f98);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f514);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0411af90);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f493);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xfd132f87);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f413);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xf514bf7f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f393);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xec165f77);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f323);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xe117ef6e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f2d3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xd4198f66);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f273);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xc91b2f5e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f233);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xba1ccf57);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f1f3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xab1e6f50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f1b3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x9b201f49);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f183);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x8b21bf42);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f173);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x79235f3b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f153);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x67250f35);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f153);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x52269f30);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f153);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x3d282f2b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f153);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x2929cf26);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f163);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x142b6f21);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f182);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xfd2cef1e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000f1b2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xe52e5f1b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_ADDR, 0xa0000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x00000004);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000ff63);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xff00b000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fed3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xfd016000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fe43);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xfc021fff);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fdc3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xf702efff);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fd53);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xf203bffe);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fce3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xeb048ffe);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fc83);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xe4057ffd);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fc33);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xdd065ffc);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fbe3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xd4075ffa);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fb93);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xc9085ff9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fb53);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xbe095ff8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fb23);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xb10a6ff6);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fb03);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xa40b7ff4);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fad3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x970c9ff2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fab3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x880dcff0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000faa3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x7a0effee);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fa93);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6a102fec);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fa93);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x58115fe9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fa93);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x4812afe6);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fa93);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x3613efe4);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fa93);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x23152fe1);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000faa3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x11167fde);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fac2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xfd17cfdb);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fad2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xe9191fd8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000faf2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xd51a7fd5);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fb12);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xc11bdfd2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fb32);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0xab1d2fcf);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fb52);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x961e8fcc);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fb82);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x811fefc9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fba2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6c214fc6);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fbd2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x5522afc3);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0000fc02);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x40240fc0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_ADDR, 0xb0000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000040);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00004ffb);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x008ff03f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd010ff80);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x05ffc003);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00008ff6);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x010fe13f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x8021ff00);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0bff8006);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0000cff0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x017fd33f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x3032fe80);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x10ff400a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0000ffeb);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01efc53e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd043fe00);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x15ff000d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00013fe7);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x026fb83e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6055fd70);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x1bfec010);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00017fe2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x02cfab3e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1067fcf0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x20fe8013);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0001afdd);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x033f9f3d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x7079fc70);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x25fe4016);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0001efd9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x039f943c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xe08bfbf0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x2afe1019);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00021fd5);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x03ff893c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x409efb70);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x2ffdd01c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00024fd1);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x045f7f3b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xb0b1faf0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x34fd901f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00027fcd);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x04bf753b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x10c4fa70);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x39fd6022);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0002afc9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x050f6d3a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x40d7fa00);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x3efd2024);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0002dfc6);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x055f6439);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xa0ebf980);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x43fcf027);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0002ffc2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x05af5d38);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xe0fff900);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x48fcb02a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00032fbf);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x05ef5637);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xf112f890);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x4cfc802c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00034fbc);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x062f5037);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x2126f820);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x50fc502e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00036fb9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x066f4a36);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x513af7b0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x54fc2031);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00038fb7);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x069f4435);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x714ff740);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x58fbf033);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0003afb4);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06cf4034);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x7162f6e0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x5cfbd035);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0003cfb2);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x06ff3c33);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x8176f670);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x60fba037);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0003dfb0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x072f3832);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x818bf610);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x63fb8039);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x0003ffae);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x074f3531);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x919ff5b0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x67fb503b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00040fad);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x076f3330);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x81b3f560);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6afb303c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00041fac);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x078f312f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x61c7f510);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6cfb203d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00042fab);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x079f302e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x51dbf4c0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x6ffb003f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00043faa);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07af2f2d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x51f0f470);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x71fae040);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00043fa9);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07bf2f2c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1202f440);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x73fad041);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00044fa8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07cf2e2b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x0217f3f0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x76fab042);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00044fa8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07cf2f29);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd22af3c0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x77fab043);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00044fa8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07cf3028);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xb23ff390);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x79faa043);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00044fa8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07bf3227);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x9252f360);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x7afa9044);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00044fa8);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x07bf3326);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6266f330);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x7bfa8044);
    REG_WRITE_AND_POLL (DISP_HD_RAM_ADDR, 0xc0000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00000040);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x003ff33f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xf00effd0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x006fe73f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xe01cff90);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x009fdb3f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd02aff50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00cfd03f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xa03aff10);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x00efc53f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6049fed0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x01000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x011fbb3f);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x105afe90);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x01000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x013fb23e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xb06afe50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x01000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x015fa93e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x407cfe10);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x02000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x017fa13d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd08dfdc0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x02000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x018f993d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x50a0fd80);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x03000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01af923c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xb0b2fd30);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x04000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01bf8c3c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x10c5fce0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x04000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01cf863b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x70d9fc90);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x05000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01df803a);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xc0edfc40);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x06000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ef7c39);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xe101fbf0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x07000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ff7839);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1115fba0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x08000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ff7438);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x512bfb50);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x09000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f7137);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6140fb00);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x09000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f6e36);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6155fac0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0a000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f6c35);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x616afa70);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0c000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f6b34);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x7180fa20);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0d000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f6933);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x6196f9d0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0e000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x020f6932);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x51acf980);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x0f000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ff6831);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x21c2f940);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x10000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ff6930);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x01d8f8f0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x11000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01ef692e);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xd1eff8b0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x12000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01df6a2d);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xa206f860);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x14000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01df6c2c);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x521bf830);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x15000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01cf6e2b);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1231f7f0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x16000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01bf7029);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0xc246f7b0);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x17000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x01af7228);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x725cf780);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x18000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA3, 0x00000000);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA2, 0x019f7527);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA1, 0x1271f750);
    REG_WRITE_AND_POLL (DISP_HD_RAM_WDATA, 0x19000000);
    
}


