#include <sys/ioctl.h>

/************************************************** 
*
*  Copyright (c) 2006-2007 VIXS Systems Inc. All rights reserved
*
*  FILENAME: xcode4_screen.c  
*
*  DESCRIPTION: Contains Xcode-IIII DirectFB driver implemntation  
*               
*  AUTHOR:
*
*  HISTORY
*
*
***************************************************/
#include <config.h>

#include <asm/types.h>

#include <stdio.h>
#include <sys/mman.h>
#include <sys/ioctl.h>

#include <directfb.h>

#include <core/screens.h>
#include <core/state.h>
#include <core/gfxcard.h>
#include <core/layers.h>
#include <core/surface.h>
#include <core/palette.h>
#include <core/screens_internal.h>

#include <math.h>

#include <misc/conf.h>

#include <gfx/convert.h>

#include "viper.h"
#include "xcmmio.h"
#include "xcode4_osd.h"
#include "xcioctl.h"
#include "xcode4_adv7511.h"
#include "xcode4_hd_comp_venc.h"

/////////////////////////////////////////////////////

typedef u8 byte;
/* byte must be 8 bits */

/* int must be at least 16 bits */

/* long must be at least 32 bits */

#define UPPER_NIBBLE( x ) \
        (((128|64|32|16) & (x)) >> 4)

#define LOWER_NIBBLE( x ) \
        ((1|2|4|8) & (x))

#define COMBINE_HI_8LO( hi, lo ) \
        ( (((unsigned)hi) << 8) | (unsigned)lo )

#define COMBINE_HI_4LO( hi, lo ) \
        ( (((unsigned)hi) << 4) | (unsigned)lo )

const byte edid_v1_header[] = { 0x00, 0xff, 0xff, 0xff,
                                0xff, 0xff, 0xff, 0x00 };

const byte edid_v1_descriptor_flag[] = { 0x00, 0x00 };


#define EDID_LENGTH                             0x80

#define EDID_HEADER                             0x00
#define EDID_HEADER_END                         0x07

#define ID_MANUFACTURER_NAME                    0x08
#define ID_MANUFACTURER_NAME_END                0x09
#define ID_MODEL				0x0a

#define ID_SERIAL_NUMBER			0x0c

#define MANUFACTURE_WEEK			0x10
#define MANUFACTURE_YEAR			0x11

#define EDID_STRUCT_VERSION                     0x12
#define EDID_STRUCT_REVISION                    0x13

#define DPMS_FLAGS				0x18
#define ESTABLISHED_TIMING_1                    0x23
#define ESTABLISHED_TIMING_2                    0x24
#define MANUFACTURERS_TIMINGS                   0x25

/* Standard Timing Identification */
#define STANDARD_TIMING_ID_START		0x26
#define STANDARD_TIMING_ID_END			0x35

#define DETAILED_TIMING_DESCRIPTIONS_START      0x36
#define DETAILED_TIMING_DESCRIPTION_SIZE        18
#define NO_DETAILED_TIMING_DESCRIPTIONS         4

#define DETAILED_TIMING_DESCRIPTION_1           0x36
#define DETAILED_TIMING_DESCRIPTION_2           0x48
#define DETAILED_TIMING_DESCRIPTION_3           0x5a
#define DETAILED_TIMING_DESCRIPTION_4           0x6c

#define EXTENSION_FLAG				0x7e

#define PIXEL_CLOCK_LO     (unsigned)dtd[ 0 ]
#define PIXEL_CLOCK_HI     (unsigned)dtd[ 1 ]
#define PIXEL_CLOCK        (COMBINE_HI_8LO( PIXEL_CLOCK_HI,PIXEL_CLOCK_LO )*10000)

#define H_ACTIVE_LO        (unsigned)dtd[ 2 ]

#define H_BLANKING_LO      (unsigned)dtd[ 3 ]

#define H_ACTIVE_HI        UPPER_NIBBLE( (unsigned)dtd[ 4 ] )

#define H_ACTIVE           COMBINE_HI_8LO( H_ACTIVE_HI, H_ACTIVE_LO )

#define H_BLANKING_HI      LOWER_NIBBLE( (unsigned)dtd[ 4 ] )

#define H_BLANKING         COMBINE_HI_8LO( H_BLANKING_HI, H_BLANKING_LO )


#define V_ACTIVE_LO        (unsigned)dtd[ 5 ]

#define V_BLANKING_LO      (unsigned)dtd[ 6 ]

#define V_ACTIVE_HI        UPPER_NIBBLE( (unsigned)dtd[ 7 ] )

#define V_ACTIVE           COMBINE_HI_8LO( V_ACTIVE_HI, V_ACTIVE_LO )

#define V_BLANKING_HI      LOWER_NIBBLE( (unsigned)dtd[ 7 ] )

#define V_BLANKING         COMBINE_HI_8LO( V_BLANKING_HI, V_BLANKING_LO )


#define H_SYNC_OFFSET_LO   (unsigned)dtd[ 8 ]
#define H_SYNC_WIDTH_LO    (unsigned)dtd[ 9 ]

#define V_SYNC_OFFSET_LO   UPPER_NIBBLE( (unsigned)dtd[ 10 ] )
#define V_SYNC_WIDTH_LO    LOWER_NIBBLE( (unsigned)dtd[ 10 ] )

#define V_SYNC_WIDTH_HI    ((unsigned)dtd[ 11 ] & (1|2))
#define V_SYNC_OFFSET_HI   (((unsigned)dtd[ 11 ] & (4|8)) >> 2)

#define H_SYNC_WIDTH_HI    (((unsigned)dtd[ 11 ] & (16|32)) >> 4)
#define H_SYNC_OFFSET_HI   (((unsigned)dtd[ 11 ] & (64|128)) >> 6)


#define V_SYNC_WIDTH       COMBINE_HI_4LO( V_SYNC_WIDTH_HI, V_SYNC_WIDTH_LO )
#define V_SYNC_OFFSET      COMBINE_HI_4LO( V_SYNC_OFFSET_HI, V_SYNC_OFFSET_LO )

#define H_SYNC_WIDTH       COMBINE_HI_4LO( H_SYNC_WIDTH_HI, H_SYNC_WIDTH_LO )
#define H_SYNC_OFFSET      COMBINE_HI_4LO( H_SYNC_OFFSET_HI, H_SYNC_OFFSET_LO )

#define H_SIZE_LO          (unsigned)dtd[ 12 ]
#define V_SIZE_LO          (unsigned)dtd[ 13 ]

#define H_SIZE_HI          UPPER_NIBBLE( (unsigned)dtd[ 14 ] )
#define V_SIZE_HI          LOWER_NIBBLE( (unsigned)dtd[ 14 ] )

#define H_SIZE             COMBINE_HI_8LO( H_SIZE_HI, H_SIZE_LO )
#define V_SIZE             COMBINE_HI_8LO( V_SIZE_HI, V_SIZE_LO )

#define H_BORDER           (unsigned)dtd[ 15 ]
#define V_BORDER           (unsigned)dtd[ 16 ]

#define FLAGS              (unsigned)dtd[ 17 ]

#define INTERLACED         (FLAGS&128)
#define SYNC_TYPE	   (FLAGS&3<<3)  /* bits 4,3 */
#define SYNC_SEPARATE	   (3<<3)
#define HSYNC_POSITIVE	   (FLAGS & 4)
#define VSYNC_POSITIVE     (FLAGS & 2)

#define MONITOR_NAME            0xfc
#define MONITOR_LIMITS          0xfd
#define UNKNOWN_DESCRIPTOR      -1
#define DETAILED_TIMING_BLOCK   -2


#define DESCRIPTOR_DATA         5
#define V_MIN_RATE              block[ 5 ]
#define V_MAX_RATE              block[ 6 ]
#define H_MIN_RATE              block[ 7 ]
#define H_MAX_RATE              block[ 8 ]

#define MAX_PIXEL_CLOCK         (((int)block[ 9 ]) * 10)
#define GTF_SUPPORT		block[10]

#define DPMS_ACTIVE_OFF		(1 << 5)
#define DPMS_SUSPEND		(1 << 6)
#define DPMS_STANDBY		(1 << 7)

#define DISP_EDID

#ifdef DISP_EDID
#define DISP_EDID_DATA printf
#endif

/////////////////////////////////////////////////////

void Set_HD_Component_Hard_Code_720p(pxc_dfb_driver_data mdrv);

void Set_HD_Component_Hard_Code(pxc_dfb_driver_data mdrv);

void Set_HD_Component_Hard_Code_1080i(pxc_dfb_driver_data mdrv);

void ViperHDDACPowerUp(pxc_dfb_driver_data mdrv);
void ViperHDDACPowerDown(pxc_dfb_driver_data mdrv);

static u32 disp_hd_comp_register_table[SIZE_OF_DISP_COMP_REGISTER_TABLE];

extern void RESET_TOGGLE(pxc_dfb_driver_data mdrv, u32 reg_addr, u32 mask);

void Xcode4_Set_DVE_656_Output(pxc_dfb_driver_data mdrv,
                               struct display_mode_struct * mode,
                               struct dve_656_reg_struct * dve_656_reg_ptr);

void Xcode4_Set_HD_DVE_656_2_Output_Order(pxc_dfb_driver_data mdrv,
                               const DFBScreenOutputConfig *config);

int get_edid_data(pxc_dfb_driver_data mdrv, DFBScreenOutputEdidConfig *config);

extern void Set_Sd_Comp_As_HardCode(pxc_dfb_driver_data mdrv );

static  screen_mode_alias_type_t    get_alias_type(const DFBScreenOutputConfig *config);

static  const u8  *get_csc_array(const struct display_mode_struct *mode, DFBScreenOutputSignals out_signals);

/* Clock management/power off */
static  void        Clk656On(pxc_dfb_driver_data mdrv);
static  void        Clk656Off(pxc_dfb_driver_data mdrv);
static  void        HDMIOn(pxc_dfb_driver_data mdrv);
static  void        HDMIOff(pxc_dfb_driver_data mdrv);

#define PRESET_LINES 1

extern void ViperSDDACPowerDown(pxc_dfb_driver_data mdrv);
extern void ViperSDDACPowerUp(pxc_dfb_driver_data mdrv);

extern void CSC_ProgAdjustMatrix(pxc_dfb_driver_data mdrv, int layer_id);

//
//Timings based of CEA-861-D, includes borders with blanking interval
// 
static struct display_mode_struct xc4_mode_list[] = 
{
    //640x480p 60Hz  25.200MHz
	{DSOSB_4x3, DSESM_PROGRESSIVE, DSEF_60HZ, DSOR_640_480, 1,
	 640, 640, 480, 800, 525, 16, 96, 10, 2, 516, 0, 1, 0, VGA_DENOMINATOR_25P2MHZ,
	 3.579545454354957, 25.200, 0, 0, 0,
	 282, 240, 16, 0, 1, 1, 1, 1, .f2off = 0, .field_length = 0 
	},
	
	//640x480p 59.94Hz 25.175MHz
	{DSOSB_4x3, DSESM_PROGRESSIVE, DSEF_59_94HZ, DSOR_640_480, 1,
	 640, 640, 480, 800, 525, 16, 96, 10, 2, 516, 0, 1, 0, VGA_DENOMINATOR_25P175MHZ,
	 3.579545454354957, 25.200, 0, 0, 0,
	 282, 240, 16, 0, 1, 1, 1, 1, .f2off = 0, .field_length = 0 
	},
	
	//720(1440)x480i 60Hz	27.027M 4:3
    {DSOSB_4x3, DSESM_INTERLACED, DSEF_60HZ, DSOR_720_480, 6,
     720, 1440, 480, 1716, 525, 38, 124, 4, 3, 525, 262, 4, 266, SD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 9, 272,
     282, 240, 16, 0, 1, 1, 2, 1, .f2off = 624, .field_length = 0 },
	 
	//720(1440)x480i 59.94Hz 27M 4:3
    {DSOSB_4x3, DSESM_INTERLACED, DSEF_59_94HZ , DSOR_720_480, 6,
      720, 1440, 480, 1716, 525, 38, 124, 4, 3, 525, 262, 4, 266, SD_DENOMINATOR_NO_FRAC,
     3.579545454354957, 27.000, 0, 9, 272,
     282, 240, 16, 0, 1, 1, 2, 1, .f2off = 624, .field_length = 0 }, 
	
	//720(1440)x480i 60Hz	27.027M 16:9
	{DSOSB_16x9, DSESM_INTERLACED, DSEF_60HZ, DSOR_720_480, 7,
     720, 1440, 480, 1716, 525, 38, 124, 4, 3, 525, 262, 4, 266, SD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 9, 272,
     282, 240, 16, 0, 1, 1, 2, 1, .f2off = 624, .field_length = 0 },
	 
	//720(1440)x480i 59.94Hz 27M 16:9
    {DSOSB_16x9, DSESM_INTERLACED, DSEF_59_94HZ , DSOR_720_480, 7,
      720, 1440, 480, 1716, 525, 38, 124, 4, 3, 525, 262, 4, 266, SD_DENOMINATOR_NO_FRAC,
     3.579545454354957, 27.000, 0, 9, 272,
     282, 240, 16, 0, 1, 1, 2, 1, .f2off = 624, .field_length = 0 }, 
	
	
	/* For pixel repetition there is no need to modify timing parameters, HDMI core/PHY
	*  will perform the repetition, pr=4 720x480i 60Hz and 59.94Hz, 4:3, 16:9	
	*/
	//720(2880)x480i 60Hz	27.027M	4:3
	{DSOSB_4x3, DSESM_INTERLACED, DSEF_60HZ, DSOR_2880_480, 10,
     720, 1440, 480, 1716, 525, 38, 124, 4, 3, 525, 262, 4, 266, SD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 9, 272,
     282, 240, 16, 0, 1, 1, 2, 2, .f2off = 624, .field_length = 0 },
	
	//720(2880)x480i 60Hz	27.027M	16:9
	{DSOSB_16x9, DSESM_INTERLACED, DSEF_60HZ, DSOR_2880_480, 11,
     720, 1440, 480, 1716, 525, 38, 124, 4, 3, 525, 262, 4, 266, SD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 9, 272,
     282, 240, 16, 0, 1, 1, 2, 2, .f2off = 624, .field_length = 0 },	
	
	//720(2880)x480i 59.94Hz	27M	4:3
	{DSOSB_4x3, DSESM_INTERLACED, DSEF_59_94HZ, DSOR_2880_480, 10,
     720, 1440, 480, 1716, 525, 38, 124, 4, 3, 525, 262, 4, 266, SD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 9, 272,
     282, 240, 16, 0, 1, 1, 2, 2, .f2off = 624, .field_length = 0 },
	
	//720(2880)x480i 59.94Hz	27M	16:9
	{DSOSB_16x9, DSESM_INTERLACED, DSEF_59_94HZ, DSOR_2880_480, 11,
     720, 1440, 480, 1716, 525, 38, 124, 4, 3, 525, 262, 4, 266, SD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 9, 272,
     282, 240, 16, 0, 1, 1, 2, 2, .f2off = 624, .field_length = 0 },	
	
	/* 720x480p pixel rep 1 */
	 
	//720x480p 60Hz 27.027M 16:9
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_60HZ , DSOR_720_480, 3, 
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //720x480p 59.94Hz 27M 16:9
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_59_94HZ , DSOR_720_480, 3,
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 27, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
	 
	//720x480p 60Hz 27.027M 4:3
    {DSOSB_4x3, DSESM_PROGRESSIVE , DSEF_60HZ , DSOR_720_480, 2, 
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //720x480p 59.94Hz 27M 4:3
    {DSOSB_4x3, DSESM_PROGRESSIVE , DSEF_59_94HZ , DSOR_720_480, 2,
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 27, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
	
	
	/* 720x480p pixel rep 2 */
	 
	//720(1440)x480p 60Hz 27.027M 16:9
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_60HZ , DSOR_1440_480, 15, 
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 2, .f2off = 0, .field_length = 0 },

    //720(1440)x480p 59.94Hz 27M 16:9
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_59_94HZ , DSOR_1440_480, 15,
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 27, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 2, .f2off = 0, .field_length = 0 },
	 
	//720(1440)x480p 60Hz 27.027M 4:3
    {DSOSB_4x3, DSESM_PROGRESSIVE , DSEF_60HZ , DSOR_1440_480, 14, 
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 2, .f2off = 0, .field_length = 0 },

    //720(1440)x480p 59.94Hz 27M 4:3
    {DSOSB_4x3, DSESM_PROGRESSIVE , DSEF_59_94HZ , DSOR_1440_480, 14,
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 27, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 2, .f2off = 0, .field_length = 0 },
	 
	 
	/* 720x480p pixel rep 4 */
	 
	//720(2880)x480p 60Hz 27.027M 16:9
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_60HZ , DSOR_2880_480, 36, 
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 4, .f2off = 0, .field_length = 0 },

    //720(2880)x480p 59.94Hz 27M 16:9
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_59_94HZ , DSOR_2880_480, 36,
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 27, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 4, .f2off = 0, .field_length = 0 },
	 
	//720(2880)x480p 60Hz 27.027M 4:3
    {DSOSB_4x3, DSESM_PROGRESSIVE , DSEF_60HZ , DSOR_2880_480, 35, 
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 27.027, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 4, .f2off = 0, .field_length = 0 },

    //720(2880)x480p 59.94Hz 27M 4:3
    {DSOSB_4x3, DSESM_PROGRESSIVE , DSEF_59_94HZ , DSOR_2880_480, 35,
      720, 720, 480, 858, 525, 16, 62, 9, 6, 523, 0, 7, 0, SD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 27, 0, 0, 0,
     282, 240, 16, 0, 1, 1, 1, 4, .f2off = 0, .field_length = 0 }, 
	 
	 
	
    //720(1440)x576i 50Hz 27M 4:3
    {DSOSB_4x3, DSESM_INTERLACED, DSEF_50HZ, DSOR_720_576, 21,
     720, 1440, 576, 1728, 625, 24, 126, 2, 3, 624, 311, 1, 313, SD_DENOMINATOR_NO_FRAC,
     4.43361875, 27, 0, 5, 318,
     252, 252, 16, 0, 1, 1, 2, 1, .f2off = 604, .field_length = 0 },
	
	//720(1440)x576i 50Hz 27M 16:9	
	{DSOSB_16x9, DSESM_INTERLACED, DSEF_50HZ, DSOR_720_576, 22,
     720, 1440, 576, 1728, 625, 24, 126, 2, 3, 624, 311, 1, 313, SD_DENOMINATOR_NO_FRAC,
     4.43361875, 27, 0, 5, 318,
     252, 252, 16, 0, 1, 1, 2, 1, .f2off = 604, .field_length = 0 }, 
	
	//720(2880)x576i 50Hz 27M 4:3
    {DSOSB_4x3, DSESM_INTERLACED, DSEF_50HZ, DSOR_2880_576, 25,
     720, 1440, 576, 1728, 625, 24, 126, 2, 3, 624, 311, 1, 313, SD_DENOMINATOR_NO_FRAC,
     4.43361875, 27, 0, 5, 318,
     252, 252, 16, 0, 1, 1, 2, 2, .f2off = 604, .field_length = 0 },
	
	//720(2880)x576i 50Hz 27M 16:9	
	{DSOSB_16x9, DSESM_INTERLACED, DSEF_50HZ, DSOR_2880_576, 26,
     720, 1440, 576, 1728, 625, 24, 126, 2, 3, 624, 311, 1, 313, SD_DENOMINATOR_NO_FRAC,
     4.43361875, 27, 0, 5, 318,
     252, 252, 16, 0, 1, 1, 2, 2, .f2off = 604, .field_length = 0 }, 	
	 
	
	//720x576p 50Hz 27M 4:3
	{DSOSB_4x3, DSESM_PROGRESSIVE , DSEF_50HZ, DSOR_720_576, 17,
     720, 720, 576, 864, 625, 12, 64, 5, 5, 621, 0, 1, 0, SD_DENOMINATOR_NO_FRAC, 
     .pol_hsync = 1, .pol_vsync = 1, .incoming_factor = 1, .outgoing_factor = 1, 
     .f2off = 0, .field_length = 0 },
	 
    //720x576p 50Hz 27M 16:9
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_50HZ, DSOR_720_576, 18,
     720, 720, 576, 864, 625, 12, 64, 5, 5, 621, 0, 1, 0, SD_DENOMINATOR_NO_FRAC, 
     .pol_hsync = 1, .pol_vsync = 1, .incoming_factor = 1, .outgoing_factor = 1, 
     .f2off = 0, .field_length = 0 },
	
	
		
	//720(1440)x576p 50Hz 27M 4:3
	{DSOSB_4x3, DSESM_PROGRESSIVE , DSEF_50HZ, DSOR_1440_576, 29,
     720, 720, 576, 864, 625, 12, 64, 5, 5, 621, 0, 1, 0, SD_DENOMINATOR_NO_FRAC, 
     .pol_hsync = 1, .pol_vsync = 1, .incoming_factor = 1, .outgoing_factor = 2, 
     .f2off = 0, .field_length = 0 },
	 
    //720(1440)x576p 50Hz 27M 16:9
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_50HZ, DSOR_1440_576, 30,
     720, 720, 576, 864, 625, 12, 64, 5, 5, 621, 0, 1, 0, SD_DENOMINATOR_NO_FRAC, 
     .pol_hsync = 1, .pol_vsync = 1, .incoming_factor = 1, .outgoing_factor = 2, 
     .f2off = 0, .field_length = 0 },
	 
	//720(2880)x576p 50Hz 27M 4:3
	{DSOSB_4x3, DSESM_PROGRESSIVE , DSEF_50HZ, DSOR_2880_576, 37,
     720, 720, 576, 864, 625, 12, 64, 5, 5, 621, 0, 1, 0, SD_DENOMINATOR_NO_FRAC, 
     .pol_hsync = 1, .pol_vsync = 1, .incoming_factor = 1, .outgoing_factor = 4, 
     .f2off = 0, .field_length = 0 },
	 
    //720(2880)x576p 50Hz 27M 16:9
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_50HZ, DSOR_2880_576, 38,
     720, 720, 576, 864, 625, 12, 64, 5, 5, 621, 0, 1, 0, SD_DENOMINATOR_NO_FRAC, 
     .pol_hsync = 1, .pol_vsync = 1, .incoming_factor = 1, .outgoing_factor = 4, 
     .f2off = 0, .field_length = 0 }, 
	
    //1280x720p 25Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_25HZ, DSOR_1280_720, 61,
     1280, 1280, 720, 3960, 750, 2420, 40, 5, 5, 746, 0, 1, 0, HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 74.25, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
    
    //1280x720p 30Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_30HZ, DSOR_1280_720, 62,
     1280, 1280, 720, 3300, 750, 1760, 40, 5, 5, 746, 0, 1, 0, HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 74.25, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
    
    //1280x720p 29.97Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_29_97HZ, DSOR_1280_720, 62,
     1280, 1280, 720, 3300, 750, 1760, 40, 5, 5, 746, 0, 1, 0, HD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 74.176, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
	
    //1280x720p 60Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_60HZ, DSOR_1280_720, 4,
     1280, 1280, 720, 1650, 750, 110, 40, 5, 5, 746, 0, 1, 0, HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 74.25, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //1280x720p 59.94Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_59_94HZ, DSOR_1280_720, 4,
     1280, 1280, 720, 1650, 750, 110, 40, 5, 5, 746, 0, 1, 0, HD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 74.176, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //1280x720p 50Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_50HZ, DSOR_1280_720, 19,
     1280, 1280, 720, 1980, 750, 440, 40, 5, 5, 746, 0, 1, 0, HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 74.250, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //1920x1080i 60Hz
    {DSOSB_16x9, DSESM_INTERLACED  , DSEF_60HZ, DSOR_1920_1080, 5,
     1920, 1920, 1080, 2200, 1125, 88, 44, 2, 5, 1124, 561, 1, 563, HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 74.250, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 912, .field_length = 0 },

    //1920x1080i 59.94Hz
    {DSOSB_16x9, DSESM_INTERLACED  , DSEF_59_94HZ, DSOR_1920_1080, 5,
     1920, 1920, 1080, 2200, 1125, 88, 44, 2, 5, 1124, 561, 1, 563, HD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 74.176, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 912, .field_length = 0 },

    //1920x1080i 50Hz
    {DSOSB_16x9, DSESM_INTERLACED  , DSEF_50HZ, DSOR_1920_1080, 20,
     1920, 1920, 1080, 2640, 1125, 528, 44, 2, 5, 1124, 561, 1, 563, HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 74.250, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 1132, .field_length = 0 },

    //1920x1080p 60Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_60HZ, DSOR_1920_1080, 16,
     1920, 1920, 1080, 2200, 1125, 88, 44, 4, 5, 1122, 0, 1, 0, FULL_HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 148.500, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //1920x1080p 59.94Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_59_94HZ, DSOR_1920_1080, 16,
     1920, 1920, 1080, 2200, 1125, 88, 44, 4, 5, 1122, 0, 1, 0, FULL_HD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 148.352, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //1920x1080p 50Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_50HZ, DSOR_1920_1080, 31,
     1920, 1920, 1080, 2640, 1125, 528, 44, 4, 5, 1122, 0, 1, 0, FULL_HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 148.500, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //1920x1080p 30Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_30HZ , DSOR_1920_1080, 34,
     1920, 1920, 1080, 2200, 1125, 88, 44, 4, 5, 1122, 0, 1, 0, HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 74.176, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //1920x1080p 29.97Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_29_97HZ , DSOR_1920_1080, 34,
     1920, 1920, 1080, 2200, 1125, 88, 44, 4, 5, 1122, 0, 1, 0, HD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 74.176, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
    
    //1920x1080p 25Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_25HZ , DSOR_1920_1080, 33,
     1920, 1920, 1080, 2640, 1125, 528, 44, 4, 5, 1122, 0, 1, 0, HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 74.250, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
    
    
    //1920x1080p 24Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_24HZ , DSOR_1920_1080, 32,
     1920, 1920, 1080, 2750, 1125, 638, 44, 4, 5, 1122, 0, 1, 0, HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 74.250, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //1920x1080p 23.98Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_23_976HZ, DSOR_1920_1080, 32,
     1920, 1920, 1080, 2750, 1125, 638, 44, 4, 5, 1122, 0, 1, 0, HD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 74.176, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //1280x1470 60Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_60HZ, DSOR_1280_1470, 4,
     1280, 1280, 1470, 1650, 1500, 110, 40, 5, 5, 1496, 0, 1, 0, FULL_HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 148.5, 0, 0, 0,
     252, 252, 16, 488, 0, 0, 1, 1, .f2off = 0, .field_length = 0 },

    //1280x1470p 59.94Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_59_94HZ, DSOR_1280_1470, 4,
     1280, 1280, 1470, 1650, 1500, 110, 40, 5, 5, 1496, 0, 1, 0, FULL_HD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 148.5, 0, 0, 0,
     252, 252, 16, 488, 0, 0, 1, 1, .f2off = 0, .field_length = 0 },

    //1920x1080p 24Hz
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_24HZ , DSOR_1920_1080_3D_SIDE_BY_SIDE, 32,
     1920, 1920, 1080, 2750, 1125, 638, 44, 4, 5, 1122, 0, 1, 0, HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 74.250, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
     
    //1920x2250p 24Hz (1080p framepacking for external HDMI only)
    {DSOSB_16x9, DSESM_PROGRESSIVE, DSEF_24HZ, DSOR_1920_2205, 32,
     1920, 1920, 2205, 2750, 2250, 638, 44, 4, 5, 2247, 0, 1, 0, FULL_HD_DENOMINATOR_NO_FRAC, 
     //for analog video encoder
     3.579545454354957, 148.5, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
	
    //1920x2250p 23.976Hz (1080p framepacking 23.976Hz)
	{DSOSB_16x9, DSESM_PROGRESSIVE, DSEF_23_976HZ, DSOR_1920_2205, 32,
     1920, 1920, 2205, 2750, 2250, 638, 44, 4, 5, 2247, 0, 1, 0, FULL_HD_DENOMINATOR_WITH_FRAC, 
     //for analog video encoder
     3.579545454354957, 148.35, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 }, 
     
    //1920x1080i 59.94Hz frame packing
    {DSOSB_16x9, DSESM_INTERLACED  , DSEF_59_94HZ, DSOR_1920_2205, 5,
     1920, 1920, 2228, 2200, 2250, 88, 44, 2, 5, 2249, 561, 1, 563, FULL_HD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 148.35, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 912, .field_length = 540},
    
    //1920x1080i 60Hz frame packing
    {DSOSB_16x9, DSESM_INTERLACED  , DSEF_60HZ, DSOR_1920_2205, 5,
     1920, 1920, 2228, 2200, 2250, 88, 44, 2, 5, 2249, 561, 1, 563, FULL_HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 148.35, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 912, .field_length = 540},
    
    //1920x1080i 50Hz frame packing
    {DSOSB_16x9, DSESM_INTERLACED  , DSEF_50HZ, DSOR_1920_2205, 20,
     1920, 1920, 2228, 2640, 2250, 528, 44, 2, 5, 2249, 561, 1, 563, FULL_HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 74.250, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 1132, .field_length = 540},

    //1920x1080p DSEF_23_976HZ
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_23_976HZ , DSOR_1920_1080_3D_SIDE_BY_SIDE, 32,
     1920, 1920, 1080, 2750, 1125, 638, 44, 4, 5, 1122, 0, 1, 0, HD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 74.250, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
    
    //1920x1080p 30Hz frame packing
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_30HZ , DSOR_1920_2205, 34,
     1920, 1920, 2205, 2200, 2250, 88, 44, 4, 5, 2247, 0, 1, 0, FULL_HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 148.5, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },

    //1920x1080p 29.97Hz frame packing
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_29_97HZ , DSOR_1920_2205, 34,
     1920, 1920, 2205, 2200, 2250, 88, 44, 4, 5, 2247, 0, 1, 0, FULL_HD_DENOMINATOR_WITH_FRAC,
     //for analog video encoder
     3.579545454354957, 148.35, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
    
    //1920x1080p 25Hz frame packing
    {DSOSB_16x9, DSESM_PROGRESSIVE , DSEF_25HZ , DSOR_1920_2205, 33,
     1920, 1920, 2205, 2640, 2250, 528, 44, 4, 5, 2247, 0, 1, 0, FULL_HD_DENOMINATOR_NO_FRAC,
     //for analog video encoder
     3.579545454354957, 148.5, 0, 0, 0,
     252, 252, 16, 488, 1, 1, 1, 1, .f2off = 0, .field_length = 0 },
    
};

#define MAX_MODE_INDEX  61






/* 
*   Alias table to alias 2D modes to half res 3D modes ... 
*   This entire table will need to be scanned for each mode
*   during the mode_list entry search, so it will make
*   lookups more expensive ... 
*/
static screen_mode_alias_t mode_aliases[] = 
{
    {DSOR_1920_1080, DSOR_1920_1080_3D_SIDE_BY_SIDE},
    {DSOR_1920_1080, DSOR_1920_1080_3D_TOP_AND_BOT},
    {DSOR_1280_720, DSOR_1280_720_3D_SIDE_BY_SIDE},
    {DSOR_1280_720, DSOR_1280_720_3D_TOP_AND_BOT},
    {DSOR_720_480, DSOR_720_480_3D_SIDE_BY_SIDE},
    {DSOR_720_480, DSOR_720_480_3D_TOP_AND_BOT},
    {DSOR_720_576, DSOR_720_576_3D_SIDE_BY_SIDE},
    {DSOR_720_576, DSOR_720_576_3D_TOP_AND_BOT},
    {DSOR_1440_480, DSOR_1440_480_3D_SIDE_BY_SIDE},
    {DSOR_1440_480, DSOR_1440_480_3D_TOP_AND_BOT},
    {DSOR_1440_576, DSOR_1440_576_3D_SIDE_BY_SIDE},
    {DSOR_1440_576, DSOR_1440_576_3D_TOP_AND_BOT},
};

/* Static constant variable which holds the entire size of the table ... */
static const int mode_aliases_size   =   sizeof(mode_aliases) / sizeof(screen_mode_alias_t);

/* Script to do 20% boost
*  With address and data pairs...
*  The end is marked with both address and data 
*  equal to 0xFFFFFFFF
*/
u32 phy_boost_20[] =
{
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0xB0,
    0x3001, 0x01,
    /* 0xB0 to reg 0x24*/
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0x80,
    0x3001, 0x01,
    
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0x90,
    0x3001, 0x01,
    
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0xA0,
    0x3001, 0x01,
    
    /*
    *
    *   Below are the register write changes to the PHY
    *   based on investigation from JimKelton on the Toshiba board
    *   0x05 -> Reg 0x21
    *   0x0C -> Reg 0x20
    */
    0x3001, 0x01,
    0x3002, 0x21,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, (PHY_VALUE_GAIN_HI << PHY_MASK_SHIFT) | 0x05,
    0x3001, 0x01,
    
    0x3001, 0x01,
    0x3002, 0x20,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, (PHY_VALUE_PREEMPHASIS_HI << PHY_MASK_SHIFT)  | 0x12,
    0x3001, 0x01,
    
    0xFFFFFFFF, 0xFFFFFFFF
};


/* 
*   This PHY initialization script is used in place
*   If the pixel clock is below 148.5
*/
u32 phy_boost_20_b1485[] =
{
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0xB0,
    0x3001, 0x01,
    /* 0xB0 to reg 0x24*/
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0x80,
    0x3001, 0x01,
    
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0x90,
    0x3001, 0x01,
    
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0xA0,
    0x3001, 0x01,
    
    /*
    *
    *   Below are the register write changes to the PHY
    *   based on investigation from JimKelton on the Toshiba board
    *   0x06 -> Reg 0x21
    *   0x0C -> Reg 0x20
    */
    /*
    *    We failed one test in the most recent run at Allion, test 7-2 which checks that the single sided low level voltage is between 2.7 to 2.9 volts. 
    *    It was a violation of the minimum voltage level on a line at the lowest data rate (27 MHz). 
    *    All 8 tmds lines failed by about 6-10 mV. 
    *    I beleive an easy fix is to reduce the gain one increment on all but the highests rates. 
    *    o, for video modes which are at 148.5 or higher clock rates continue to set the gain register (phy register 0x21) to "0x5." 
    *    For all other rates set the gain register to "0x6." Note that 6 is a lower gain then 5, this is not a typo. 
    */
    0x3001, 0x01,
    0x3002, 0x21,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, (PHY_VALUE_GAIN_LO << PHY_MASK_SHIFT) | 0x06,
    0x3001, 0x01,
    
    0x3001, 0x01,
    0x3002, 0x20,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, (PHY_VALUE_PREEMPHASIS_LO << PHY_MASK_SHIFT) | 0x12,
    0x3001, 0x01,
    
    0xFFFFFFFF, 0xFFFFFFFF
};


/* Script to do 40% boost
*  With address and data pairs...
*  The end is marked with both address and data 
*  equal to 0xFFFFFFFF
*/
u32 phy_boost_40[] =
{
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0xB0,
    0x3001, 0x01,
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0x80,
    0x3001, 0x01,
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0x90,
    0x3001, 0x01,
    0x3001, 0x01,
    0x3002, 0x24,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0xA0,
    0x3001, 0x01,
    0x3001, 0x01,
    0x3002, 0x21,
    0x3001, 0x11,
    0x3001, 0x10,
    0x3001, 0x00,
    0x3002, 0x04,
    0x3001, 0x01,
    0xFFFFFFFF, 0xFFFFFFFF
};

u8  xvycc_gmd_packet_body[] =
{
    0x92,
    0x9B,
    0x52,
    0xF4,
    0x8D,
    0x72,
    0x96,
    0x8C,
    0xC2,
    0x92,
    0x00,
    0x00

};

u32 xvycc_gmd_length    =   sizeof(xvycc_gmd_packet_body) / sizeof(u8);


char vendor_specific[LENGTH_OF_VENDOR_SPECIFIC] = 
{'V', 'I', 'x', 'S', ' ', 'S', 'y', 's',
 't', 'e', 'm', ' ', 'I', 'n', 'c', '.',
 ' ', 'V', 'i', 'p', 'e', 'r', ' ', ' '
};

char source_product_descriptor_vendor_name[LENGTH_OF_SPD_VENDOR_NAME] =
{'V', 'i', 'X', 'S', ' ', 'I', 'n', 'c'};

char source_product_descriptor_product_name[LENGTH_OF_SPD_PRODUCT_NAME] =
{'X', 'c', 'o', 'd', 'e', '4', ' ', 'S', 'D', 'K', ' ', 'D', 'E', 'M', 'O', ' '};

char iscr1_packet[LENGTH_OF_ISCR1]=
{0x47, 0xa7, 0x89, 0xaa, 0x07, 0x00, 0xff, 0x87,
 0xf1, 0x11, 0xff, 0x05, 0xa5, 0xbc, 0xb5, 0x07
};

char iscr2_packet[LENGTH_OF_ISCR2]=
{
0x54, 0xf7, 0xff, 0x41, 0x00, 0xdc, 0xc7, 0x79,
0x44, 0xf1, 0x0f, 0xaa, 0xab, 0x12, 0x0a, 0xff
};


/* HDMI YPbPr -> RGB Color Space Conversion Matricies */
/* 
* 601/709, limited (16 -235), non-limitied (0 - 255), 8/10/12-bit
* These program HDMI registers 0x4102 -> 0x4119 (inclusive)
*/


const u8 hdmi_csc_8_709[] = 
{
    0x12,
    0xA1,
    0x77,
    0x73,
    0x7C,
    0x97,
    0x00,
    0x4D,
    0x12,
    0xA1,
    0x1C,
    0xB1,
    0x00,
    0x00,
    0x7F,
    0x08,
    0x12,
    0xA1,
    0x00,
    0x00,
    0x21,
    0xD5,
    0x7E,
    0xDE,
    0x42        /* CSC_SCALE */
};


const u8 hdmi_csc_8_601[] =
{
    0x12,
    0xA1,
    0x72,
    0xFF,
    0x79,
    0xBE,
    0x00,
    0x87,
    0x12,
    0xA1,
    0x19,
    0x8B,
    0x00,
    0x00,
    0x7F,
    0x21,
    0x12,
    0xA1,
    0x00,
    0x00,
    0x20,
    0x44,
    0x7E,
    0xEB,
    0x42        /* CSC_SCALE */
};


/* CSC_SCALE = 0x41 */
const u8 hdmi_csc_l8_709[] = 
{
    0x20, 
	0x00, 
	0x71, 
	0x50, 
	0x7A, 
	0x25, 
	0x00,   
	0xA4,   
	0x20, 
	0x00, 
	0x31, 
	0x48, 
	0x00, 
	0x00, 
    0x7E,   
    0x76,   
	0x20, 
	0x00, 
	0x00,
	0x00, 
	0x3A, 
	0x1D, 
	0x7E,   
	0x2F,
    0x41            /* CSC_SCALE */
};

/* CSC_SCALE = 0x41 */
const u8 hdmi_csc_l8_601[] =
{
    0x20,
    0x00,
    0x69,
    0xAA,
    0x75,
    0x3F,
    0x01,
    0x09,
    0x20,
    0x00,
    0x2B,
    0xDF,
    0x00,
    0x00,
    0x7E,
    0xA1,
    0x20,
    0x00,
    0x00,
    0x00,
    0x37,
    0x6D,
    0x7E,
    0x45,
    0x41            /* CSC_SCALE */
};

/* CSC_SCALE = 0x52 */
const u8 hdmi_csc_10_709[] = 
{
    0x12,
    0xA1,
    0x77,
    0x73,
    0x7C,
    0x97,
    0x01,
    0x34,
    0x12,
    0xA1,
    0x1C,
    0xB1,
    0x00,
    0x00,
    0x7C,
    0x20,
    0x12,
    0xA1,
    0x00,
    0x00,
    0x21,
    0xD5,
    0x7B,
    0x78,
    0x52        /* CSC_SCALE */
};

/* CSC_SCALE = 0x52 */
const u8 hdmi_csc_10_601[] =
{
    0x12,
    0xA1,
    0x72,
    0xFF,
    0x79,
    0xBE,
    0x02,
    0x1C,
    0x12,
    0xA1,
    0x19,
    0x8B,
    0x00,
    0x00,
    0x7C,
    0x84,
    0x12,
    0xA1,
    0x00,
    0x00,
    0x20,
    0x44,
    0x7B,
    0xAC,
    0x52            /* CSC_SCALE */
};


/* CSC_SCALE = 0x51 */
const u8 hdmi_csc_l10_709[] = 
{
    0x20,
    0x00,
    0x71,
    0x50,
    0x7A,
    0x25,
    0x02,
    0x90,
    0x20,
    0x00,
    0x31,
    0x48,
    0x00,
    0x00,
    0x79,
    0xD8,
    0x20,
    0x00,
    0x00,
    0x00,
    0x3A,
    0x1D,
    0x78,
    0xBC,
    0x51            /* CSC_SCALE */
};

/* CSC_SCALE = 0x51 */
const u8 hdmi_csc_l10_601[] =
{
    0x20,
    0x00,
    0x69,
    0xAA,
    0x75,
    0x3F,
    0x04,
    0x24,
    0x20,
    0x00,
    0x2B,
    0xDF,
    0x00,
    0x00,
    0x7A,
    0x84,
    0x20,
    0x00,
    0x00,
    0x00,
    0x37,
    0x6D,
    0x79,
    0x14,
    0x51            /* CSC_SCALE */
};


/* CSC_SCALE = 0x62 */
const u8 hdmi_csc_12_709[] = 
{
    0x12,
    0xA1,
    0x77,
    0x73,
    0x7C,
    0x97,
    0x04,
    0xD0,
    0x12,
    0xA1,
    0x1C,
    0xB1,
    0x00,
    0x00,
    0x70,
    0x80,
    0x12,
    0xA1,
    0x00,
    0x00,
    0x21,
    0xD5,
    0x6D,
    0xE0,
    0x62            /* CSC_SCALE */
};

/* CSC_SCALE = 0x62 */
const u8 hdmi_csc_12_601[] =
{
    0x12,
    0xA1,
    0x72,
    0xFF,
    0x79,
    0xBE,
    0x08,
    0x70,
    0x12,
    0xA1,
    0x19,
    0x8B,
    0x00,
    0x00,
    0x72,
    0x10,
    0x12,
    0xA1,
    0x00,
    0x00,
    0x20,
    0x44,
    0x6E,
    0xB0,
    0x62            /* CSC_SCALE */
};


/* CSC_SCALE = 0x61 */
const u8 hdmi_csc_l12_709[] = 
{
    0x20,
    0x00,
    0x71,
    0x50,
    0x7A,
    0x25,
    0x0A,
    0x40,
    0x20,
    0x00,
    0x31,
    0x48,
    0x00,
    0x00,
    0x67,
    0x60,
    0x20,
    0x00,
    0x00,
    0x00,
    0x3A,
    0x1D,
    0x62,
    0xF0,
    0x61            /* CSC_SCALE */
};

/* CSC_SCALE = 0x61 */
const u8 hdmi_csc_l12_601[] =
{
    0x20,
    0x00,
    0x69,
    0xAA,
    0x75,
    0x3F,
    0x10,
    0x90,
    0x20,
    0x00,
    0x2B,
    0xDF,
    0x00,
    0x00,
    0x6A,
    0x10,
    0x20,
    0x00,
    0x00,
    0x00,
    0x37,
    0x6D,
    0x64,
    0x50,
    0x61            /* CSC_SCALE */
};



/* functions to read and write HDMI registers. This forces all index/data read/writes */
/* through a single function in the kernel.                                           */
void hdmi_reg_write( u32 reg, u32 value, int handle )
{
    app_ioctl_command cmd;

    cmd.uIOCTLCommand = CMD_APP_IOCTL_WRITE_HDMI_REG;
    cmd.app_hdmi_reg.index    = reg;
    cmd.app_hdmi_reg.contents = value;
    ioctl( handle, CMD_APPLICATION_IOCTL, &cmd );
}

u32 hdmi_reg_read( u32 reg, int handle )
{
    app_ioctl_command cmd;

    cmd.uIOCTLCommand = CMD_APP_IOCTL_READ_HDMI_REG;
    cmd.app_hdmi_reg.index = reg;
    ioctl( handle, CMD_APPLICATION_IOCTL, &cmd );
    return cmd.app_hdmi_reg.contents;
}

void set_hd_comp_enable(pxc_dfb_driver_data mdrv, int enable)
{
    u32 tmp;
    if(!enable)
    {
        tmp = REG_READ(DISP_HD_COMP_CTRL);
        tmp &= ~DISP_HD_COMP_CTRL_COMP_MODE_MASK;
        tmp &= ~DISP_HD_COMP_CTRL_COMP_EN_MASK;
        tmp |= 0x2 << DISP_HD_COMP_CTRL_COMP_MODE_SHIFT;
        REG_WRITE(DISP_HD_COMP_CTRL, tmp);
        REG_WRITE(DISP_HD_VID0_CMD0_FIFO, REG_READ(DISP_HD_VID0_CMD0_FIFO) | DISP_HD_VID0_CMD0_FIFO_UPDATE_COMP_REGS_MASK);
        for(tmp = 0; tmp < 5; tmp++)
        {
            usleep(20000);
            if(REG_READ(DISP_HD_FIFO_STATUS0) & VID0_EMPTY_MASK)
            {
                break;
            }
        }
    }
    else
    {
        tmp = REG_READ(DISP_HD_COMP_CTRL);
        tmp &= ~DISP_HD_COMP_CTRL_COMP_MODE_MASK;
        tmp |= DISP_HD_COMP_CTRL_COMP_EN_MASK;
        REG_WRITE(DISP_HD_COMP_CTRL, tmp);
    }
}

/***********************************************************************************************/
void Init_Disp_Hd_Comp_Register()
{
    disp_hd_comp_register_table[DISP_COMP_SIZE_INDEX] = DISP_HD_COMP_SIZE;
    disp_hd_comp_register_table[DISP_COMP_GLOBAL_SIZE_INDEX] = DISP_HD_COMP_GLOBAL_SIZE;
    disp_hd_comp_register_table[DISP_COMP_START_PIX_POS_INDEX] = DISP_HD_COMP_START_PIX_POS;
    disp_hd_comp_register_table[DISP_COMP_START_FLD_POS_INDEX] = DISP_HD_COMP_START_FLD_POS;
    disp_hd_comp_register_table[DISP_COMP_START_VBI_POS_INDEX] = DISP_HD_COMP_START_VBI_POS;
    disp_hd_comp_register_table[DISP_COMP_STOP_VBI_POS_INDEX] = DISP_HD_COMP_STOP_VBI_POS;
    disp_hd_comp_register_table[DISP_COMP_START_HANC_POS_INDEX] = DISP_HD_COMP_START_HANC_POS;
    disp_hd_comp_register_table[DISP_COMP_STOP_HANC_POS_INDEX] = DISP_HD_COMP_STOP_HANC_POS;
    disp_hd_comp_register_table[DISP_VBI0_HANC0_FORMAT_INDEX] = DISP_HD_VBI0_HANC0_FORMAT;
    disp_hd_comp_register_table[DISP_VENC0_DVE_FIELD_BNDRY_INDEX] = DISP_HD_VENC0_DVE_FIELD_BNDRY;
    disp_hd_comp_register_table[DISP_VENC0_DVE_VBLANK_START_BNDRY_INDEX] = DISP_HD_VENC0_DVE_VBLANK_START_BNDRY;
    disp_hd_comp_register_table[DISP_VENC0_DVE_VBLANK_END_BNDRY_INDEX] = DISP_HD_VENC0_DVE_VBLANK_END_BNDRY;
    disp_hd_comp_register_table[DISP_VENC0_DVE_CNTRL_INDEX] = DISP_HD_VENC0_DVE_CNTRL;
    disp_hd_comp_register_table[DISP_GENLOCK_PCR_INDEX] = DISP_HD_GENLOCK_PCR;
    disp_hd_comp_register_table[DISP_GENLOCK_NUM_INDEX] = DISP_HD_GENLOCK_NUM;
    disp_hd_comp_register_table[DISP_GENLOCK_DEN_INDEX] = DISP_HD_GENLOCK_DEN;

    disp_hd_comp_register_table[DISP_VENC0_DVE_VSYNC_START_INDEX] = DISP_HD_VENC0_DVE_VSYNC_START;
    disp_hd_comp_register_table[DISP_VENC0_DVE_VSYNC_END_INDEX] = DISP_HD_VENC0_DVE_VSYNC_END;
    disp_hd_comp_register_table[DISP_VENC0_DVE_HSYNC_INDEX] = DISP_HD_VENC0_DVE_HSYNC;
    disp_hd_comp_register_table[DISP_OUTPUT_LIMIT_INDEX]    = DISP_HD_OUTPUT_LIMIT;
}
/***********************************************************************************************/
static  screen_mode_alias_type_t    get_alias_type(const DFBScreenOutputConfig *config)
{
    u32   i;  
    
    for (i = 0 ; i < mode_aliases_size ; i++)
    {
    
        if (config->resolution == mode_aliases[i].configResolution 
            )
        {
            /* Determine the type and return ... */
            if (config->resolution == DSOR_1920_1080_3D_SIDE_BY_SIDE ||
                config->resolution == DSOR_1280_720_3D_SIDE_BY_SIDE  ||
                config->resolution == DSOR_720_480_3D_SIDE_BY_SIDE   ||
                config->resolution == DSOR_720_576_3D_SIDE_BY_SIDE   ||
                config->resolution == DSOR_1440_480_3D_SIDE_BY_SIDE  ||
                config->resolution == DSOR_1440_576_3D_SIDE_BY_SIDE
               )
            {
                return ALIAS_3D_SIDE_BY_SIDE;
            }
            else if 
            (
                config->resolution == DSOR_1920_1080_3D_TOP_AND_BOT   ||
                config->resolution == DSOR_1280_720_3D_TOP_AND_BOT    ||
                config->resolution == DSOR_720_480_3D_TOP_AND_BOT     ||
                config->resolution == DSOR_720_576_3D_TOP_AND_BOT     ||
                config->resolution == DSOR_1440_480_3D_TOP_AND_BOT    ||
                config->resolution == DSOR_1440_576_3D_TOP_AND_BOT    
            )
            {
                return ALIAS_3D_TOP_AND_BOT;
            }
        }
    
    }
    
    /* If we get here, do NOT */
    return ALIAS_NONE;
}


static  const u8  *   get_csc_array(const struct display_mode_struct *mode, DFBScreenOutputSignals out_signals)
{
    const u8  *ptr    =   (u8 *)0;
    
    /* NOTE: We even return data for YCbCr to fix */
    if (mode->denominator == SD_DENOMINATOR_NO_FRAC ||
        mode->denominator == SD_DENOMINATOR_WITH_FRAC ||
        mode->denominator == VGA_DENOMINATOR_25P175MHZ ||
        mode->denominator == VGA_DENOMINATOR_25P2MHZ
       )
    {
        /* SD portions, use 601 conversion matricies */
        switch (out_signals)
        {
            case    DSOS_RGB:
            case    DSOS_YCBCR:
                ptr =   (const u8 *)&hdmi_csc_8_601;
                D_DEBUG("[HDMICSC]: Select 601 -> RGB 8-bit full range\n");
                break;
            
            case    DSOS_RGBL:
                ptr =   (const u8 *)&hdmi_csc_l8_601;
                D_DEBUG("[HDMICSC]: Select 601 -> RGB 8-bit limited range\n");
                break;
                
            case    DSOS_RGB30:
            case    DSOS_YCBCR30:
                ptr =   (const u8 *)&hdmi_csc_10_601;
                D_DEBUG("[HDMICSC]: Select 601 -> RGB 10-bit full range\n");
                break;
            
            case    DSOS_RGBL30:
                ptr =   (const u8 *)&hdmi_csc_l10_601;
                D_DEBUG("[HDMICSC]: Select 601 -> RGB 10-bit limited range\n");
                break;
                
            case    DSOS_RGB36:
            case    DSOS_YCBCR36:
                ptr =   (const u8 *)&hdmi_csc_12_601;
                D_DEBUG("[HDMICSC]: Select 601 -> RGB 12-bit full range\n");
                break;
            
            case    DSOS_RGBL36:
                ptr =   (const u8 *)&hdmi_csc_l12_601;
                D_DEBUG("[HDMICSC]: Select 601 -> RGB 12-bit limited range\n");
                break;
            
            default:
                ptr =   (u8 *)0;
                D_DEBUG("[HDMICSC]: Select NULL (likely 4:2:2 selected) \n");
                break;
        }
    
    }
    else
    {
        /* HD portions, use 709 conversion matricies */
        switch (out_signals)
        {
            case    DSOS_RGB:
            case    DSOS_YCBCR:
                ptr =   (const u8 *)&hdmi_csc_8_709;
                D_DEBUG("[HDMICSC]: Select 709 -> RGB 8-bit full range\n");
                break;
            
            case    DSOS_RGBL:
                ptr =   (const u8 *)&hdmi_csc_l8_709;
                D_DEBUG("[HDMICSC]: Select 709 -> RGB 8-bit limited range\n");
                break;
                
            case    DSOS_RGB30:
            case    DSOS_YCBCR30:
                ptr =   (const u8 *)&hdmi_csc_10_709;
                D_DEBUG("[HDMICSC]: Select 709 -> RGB 10-bit full range\n");
                break;
            
            case    DSOS_RGBL30:
                ptr =   (const u8 *)&hdmi_csc_l10_709;
                D_DEBUG("[HDMICSC]: Select 709 -> RGB 10-bit limited range\n");
                break;
                
            case    DSOS_RGB36:
            case    DSOS_YCBCR36:
                ptr =   (const u8 *)&hdmi_csc_12_709;
                D_DEBUG("[HDMICSC]: Select 709 -> RGB 12-bit full range\n");
                break;
            
            case    DSOS_RGBL36:
                ptr =   (const u8 *)&hdmi_csc_l12_709;
                D_DEBUG("[HDMICSC]: Select 709 -> RGB 12-bit limited range\n");
                break;
            
            default:
                ptr =   (u8 *)0;
                D_DEBUG("[HDMICSC]: Select NULL (likely 4:2:2 selected) \n");
                break;
        }
    }

    return (ptr);
}

/***********************************************************************************************/

int xc_find_mode_index(const DFBScreenEncoderConfig * config)
{
    int mode_index;
    u32 i, mode_found;


    for (mode_index = 0; mode_index <= MAX_MODE_INDEX; mode_index ++)
    {
        if ((config->resolution == xc4_mode_list[mode_index].resolution)  &&
            (config->frequency == xc4_mode_list[mode_index].frequency) &&
            (config->scanmode == xc4_mode_list[mode_index].scanmode) &&
            (config->slow_blanking == xc4_mode_list[mode_index].slow_blanking))
        {
            D_DEBUG("Non-aliased mode found ... %u\n", mode_index);
            break;
        }
        else
        {
        //else if ( (config->resolution == DSOR_1920_1080_3D_SIDE_BY_SIDE) &&
        //          (DSOR_1920_1080 == xc4_mode_list[mode_index].resolution) &&
        //          (config->frequency == xc4_mode_list[mode_index].frequency) &&
        //          (config->scanmode == xc4_mode_list[mode_index].scanmode) &&
        //          (config->slow_blanking == xc4_mode_list[mode_index].slow_blanking)
        //        )
        //{
        //    /* 
        //    *   If the configured resolution is 1920_1080_3D_SIDE_BY_SIDE and we find a 
        //    *   similar resolution constant, we should take it as is and then simply 
        //    *   configure the VSDB packet to notify the TV of the 3D mode
        //    */
        //    break;
            mode_found  =   0;
            if ((config->frequency == xc4_mode_list[mode_index].frequency) &&
                  (config->scanmode == xc4_mode_list[mode_index].scanmode) &&
                  (config->slow_blanking == xc4_mode_list[mode_index].slow_blanking)
                )
        {
                for (i = 0 ; i < mode_aliases_size ; i++)
                {
                    if ( config->resolution ==   mode_aliases[i].configResolution  &&
                         mode_aliases[i].listResolution == xc4_mode_list[mode_index].resolution 
                       )
                    {
                        /* Need to set this flag so we can break out of outter most loop ... */
                        mode_found  =   1;
                        D_DEBUG("Aliased mode found ... %u\n", mode_index);
            break;
        }
                }
            
                if (mode_found != 0) break;
            
            }
            
            
        }
                   
    }

   
    if (mode_index > MAX_MODE_INDEX)
        mode_index = INDEX_UNKNOWN;
 
 //   printf("mode_index = 0x%x\n", mode_index);

    return (mode_index);
}

/***********************************************************************************************/

int xc_layer_search_mode_index(CoreLayerRegionConfig * config)
{
    int mode_index = 0;
    DFBScreenEncoderScanMode scanmode;

    if ((config->surface_caps & DSCAPS_VIXS_INTERLACED) ||
        (config->surface_caps & DSCAPS_SEPARATED))
    {
         scanmode = DSESM_INTERLACED ;
    }
    else
    {
         scanmode = DSESM_PROGRESSIVE ;
    }

    for (mode_index = 0; mode_index <= MAX_MODE_INDEX; mode_index ++)
    {
        if ((xc4_mode_list[mode_index].xres == config->width) &&
            (xc4_mode_list[mode_index].v_active == config->height) &&
            (xc4_mode_list[mode_index].scanmode == scanmode))
            break;
    }

    if (mode_index > MAX_MODE_INDEX)
        mode_index = INDEX_UNKNOWN;

    return mode_index;
}


/**********************************************************************************************************************/

static DFBResult
XcodeInitScreen( CoreScreen           *screen,
                 CoreGraphicsDevice   *device,
                 void                 *driver_data,
                 void                 *screen_data,
                 DFBScreenDescription *description )
{
     u32 tmp;

     pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    
     

     D_DEBUG( "XcodeInitScreen enter \n");

     /* Set the screen capabilities. */
     description->caps = DSCCAPS_VSYNC | DSCCAPS_ENCODERS | DSCCAPS_OUTPUTS;

     /* Set the screen name. */
     snprintf( description->name, DFB_SCREEN_DESC_NAME_LENGTH, "ViXS XCODE4 Screen" );

     /* Set number of encoders and outputs. */
     description->encoders = 1;
#ifndef  OUTPUT_INTHDMI_ONLY     
     description->outputs   = DISP_HD_OUTPUTS;
#else
     description->outputs   = DISP_HD_LIMIT_OUTPUTS;    
#endif     
     mdrv->shared->mode_index = INDEX_UNKNOWN;
     mdrv->shared->last_mode_index = INDEX_UNKNOWN;
     
     

     
     mdrv->shared->last_out_signals[DISP_HD_DVE656_2_OUTPUT] = DSOS_NONE ;
     mdrv->shared->last_out_signals[DISP_HD_HDMI_OUTPUT] = DSOS_NONE ;
     mdrv->shared->last_out_signals[DISP_HD_COMPONENT_OUTPUT] = DSOS_NONE ;
     
     
#ifdef ENABLE_ADV7511_OUTPUT
     if (ADV7511_Init())
     {
	/* Error */
	
     }
#endif 

     //set disp hd every layse default alpha value and mode
     REG_WRITE(DISP_HD_COMP_ALPHA , 0);

     tmp = REG_READ(DISP_HD_COMP_ALPHA_MODE ) | ALPHA_MODE_G0_MASK | ALPHA_MODE_G1_MASK;
     REG_WRITE(DISP_HD_COMP_ALPHA_MODE, tmp);

     //set the default level for every layer
     tmp = REG_READ(DISP_HD_COMP_CTRL );
     tmp = tmp & ~(VID0_MUX_MASK | GRA0_MUX_MASK | VID1_MUX_MASK |GRA1_MUX_MASK);
     tmp = tmp | DEFAULT_VID0_LEVEL
               | (DEFAULT_GRA0_LEVEL << GRA0_MUX_SHIFT)
               | (DEFAULT_VID1_LEVEL << VID1_MUX_SHIFT)
               | (DEFAULT_GRA1_LEVEL << GRA1_MUX_SHIFT);

     REG_WRITE(DISP_HD_COMP_CTRL, tmp );

     screen->shared->mode = DSPM_ON;
     
     D_DEBUG( "XcodeInitScreen exit \n");
     return DFB_OK;
}

/**************************************************************************************************/

static DFBResult
Xcode4TestEncoderConfig( CoreScreen                   *screen,
                        void                         *driver_data,
                        void                         *screen_data,
                        int                           encoder,
                        const DFBScreenEncoderConfig *config,
                        DFBScreenEncoderConfigFlags  *failed )
{

    DFBResult result=DFB_UNSUPPORTED;

    D_DEBUG( "Xcode4TestEncoderConfig enter \n");
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    if ((config->flags & DSECONF_SCANMODE ) &&
        (config->flags & DSECONF_OUT_SIGNALS) &&
        (config->flags & DSECONF_RESOLUTION) &&
        (config->flags & DSECONF_FREQUENCY))
    {
         mdrv->shared->mode_index = xc_find_mode_index(config);

         if (mdrv->shared->mode_index != INDEX_UNKNOWN)
         {
             result = DFB_OK;
         }
    }
    //}

    return result;
}

//get dve mode from xc4_mode_list
struct display_mode_struct * xc4_get_mode(mode_index)
{
    struct display_mode_struct * mode = NULL;
    
    if (mode_index != INDEX_UNKNOWN)
    {
        mode = &(xc4_mode_list[mode_index]);
    }

    return mode;
}

/**************************************************************************************************
check if this CEA861-D mode is with fractional Vblank (eg. formats 5,6,7,10,11,20,21,22)
**************************************************************************************************/
u32 RV_Blank_In_OSC(struct display_mode_struct *mode)
{
    u32 r_v_blank_in_osc;

    if ((mode->resolution  == DSOR_1920_1080) &&
        (mode->scanmode == DSESM_INTERLACED  ) &&
        ((mode->frequency == DSEF_60HZ) || (mode->frequency == DSEF_59_94HZ)))
    {
        r_v_blank_in_osc = MODE_WITH_FRACTIONAL_VBLANK;
    }
    else if ((mode->resolution  == DSOR_720_480 ) &&
             (mode->scanmode == DSESM_INTERLACED  ) &&
             ((mode->frequency == DSEF_60HZ) || (mode->frequency == DSEF_59_94HZ)))
    {
        r_v_blank_in_osc = MODE_WITH_FRACTIONAL_VBLANK;
    }
    else if ((mode->resolution  == DSOR_1920_1080 ) &&
             (mode->scanmode == DSESM_INTERLACED  ) &&
             (mode->frequency == DSEF_50HZ))
    {
        r_v_blank_in_osc = MODE_WITH_FRACTIONAL_VBLANK;
    }
    else if ((mode->resolution  == DSOR_720_576  ) &&
             (mode->scanmode == DSESM_INTERLACED  ) &&
             (mode->frequency == DSEF_50HZ))
    {
        r_v_blank_in_osc = MODE_WITH_FRACTIONAL_VBLANK;
    }
    else
    {
        r_v_blank_in_osc = MODE_WITHOUT_FRACTIONAL_VBLANK;
    }

    return r_v_blank_in_osc;
}

/**************************************************************************************************/
u32 get_hdmi_video_refresh_rate(struct display_mode_struct * mode)
{
    u32 refresh_rate = 0;

    switch (mode->frequency)
    {
        case DSEF_25HZ:
            refresh_rate = 0x0ea25;
            break;

        case DSEF_29_97HZ :
            refresh_rate = 0x29970;
            break;

        case DSEF_50HZ  :
            refresh_rate = 0x0ea50;
            break;

        case DSEF_59_94HZ :
            refresh_rate = 0x59940;
            break;

        case DSEF_60HZ  :
            refresh_rate = 0x0ea60;
            break;

        case DSEF_24HZ  :
            refresh_rate = 0x0ea24;
            break;

        case DSEF_23_976HZ  :
            refresh_rate = 0x23976;
            break;

        default :
            refresh_rate = 0x0ea60;
            break;
    }

    return (refresh_rate);
    
}

/**************************************************************************************************
this function set the DISP encoder for component output
**************************************************************************************************/
void fill_vertical_timing_array(pxc_dfb_driver_data mdrv,
                                struct display_mode_struct * mode)
{
    u32 hd_venc0_addr;
    u32 tmp;
    u32 field1_vbi;
    u32 field1_vblank_length;
    u32 field1_active;
    u32 field2_vblank_length;
    u32 field2_vbi;
    u32 field2_active;
    u32 transition_line;
    u32 field2_equalization_to_serration;
    u32 field2_serration_to_equalization;
    u32 field2_signal_equalization;
    u32 field1_half_picture;
    u32 current_line;


    hd_venc0_addr = VENC_VERTICAL_STATE_START_ADDR;

    //for sd vertical timing
    if ((mode->resolution == DSOR_720_480 ) ||
        (mode->resolution == DSOR_720_576))
    {
        if (mode->scanmode == DSESM_INTERLACED)
        {
            if (mode->resolution == DSOR_720_480 )
            {
                //it's in the V_STATE_EQ_EQ from 1 to (field1_vsync_start - 1)
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_EQ_EQ, (mode->field1_vsync_start - 1));
            
                //it's in the V_STATE_BR_BR from field1_vsync_start to (field1_vsync_start + vsync_active_pulse_width - 1)
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_BR_BR, 
                                           (mode->field1_vsync_start + mode->vsync_active_pulse_width - 1));
            
            }
            else if (mode->resolution == DSOR_720_576 )
            {
                //it's in the V_STATE_BR_BR from field1_vsync_start to (field1_vsync_start + vsync_active_pulse_width - 2)
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_BR_BR, 
                                           (mode->field1_vsync_start + mode->vsync_active_pulse_width - 2));
            
            	//it's the V_STATE_BR_EQ on the line mode->field1_vsync_start + mode->vsync_active_pulse_width - 1
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_BR_EQ, 
                                           (mode->field1_vsync_start + mode->vsync_active_pulse_width - 1));
            
            }
            
            //it's in the V_STATE_EQ_EQ to field1_post_equalization
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_EQ_EQ, mode->field1_post_equalization);
            
            if (mode->resolution == DSOR_720_480 )
            {
                //it's in the V_STATE_NP, 
                field1_vblank_length = (mode->v_total - mode->v_active) / 2;
                field1_vbi = (mode->field1_vblank_start + field1_vblank_length) % mode->v_total;
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_NP, field1_vbi);
            
                //it's in the V_STATE_ACTIVE_VIDEO for the field1
                field1_active =  field1_vbi + (mode->v_active / 2);
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_ACTIVE_VIDEO, field1_active);
            
            }
            else if (mode->resolution == DSOR_720_576 )
            {
                //it's in the V_STATE_NP, 
                field1_vblank_length = (mode->v_total - mode->v_active) / 2;
                field1_vbi = (mode->field1_vblank_start + field1_vblank_length) % mode->v_total - 1;
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_NP, field1_vbi);
            
                field1_half_picture = field1_vbi + 1;
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_NP_HP, field1_half_picture);
            
                //it's in the V_STATE_ACTIVE_VIDEO for the field1
                field1_active =  field1_half_picture + (mode->v_active / 2) - 1;
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_ACTIVE_VIDEO, field1_active);
            }
            
            if (mode->resolution == DSOR_720_480 )
            {
                //it's in the V_STATE_NP_EQ for field transition
                transition_line = field1_active + 1;
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_ACTIVE_VIDEO, transition_line);
            }
                
            //it's in the field2 V_STATE_EQ_EQ state from (transition_line + 1) to (field2_equalization_to_serration -1)
            field2_equalization_to_serration = mode->field2_vsync_start;
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_EQ_EQ, (field2_equalization_to_serration - 1));
            
            //it's in the field2 V_STATE_EQ_BR state at field2_equalization_to_serration line
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_EQ_BR, field2_equalization_to_serration);
            
            //it's in the field2 V_STATE_BR_BR from field2_equalization_to_serration + 1 to field2_equalization_to_serration - 1
            field2_serration_to_equalization = mode->field2_vsync_start + mode->vsync_active_pulse_width;
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_BR_BR, (field2_serration_to_equalization - 1));
            
            if (mode->resolution == DSOR_720_480 )
            {
                //it's in the field2 V_STATE_BR_EQ at line field2_serration_to_equalization
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_BR_EQ, field2_serration_to_equalization);
            }
            
            //it's in the V_STATE_EQ_EQ from field2_serration_to_equalization + 1 to field2_single_equalization - 1
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_EQ_EQ, mode->field2_single_equalization - 1);
            
            //it's in the V_STATE_EQ at line field2_single_equalization
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_EQ, mode->field2_single_equalization);
            
            //it's in the V_STATE_NP, 
            field2_vblank_length = (mode->v_total - mode->v_active) / 2;
            field2_vbi = (mode->field2_vblank_start + field2_vblank_length) % mode->v_total;
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_NP, field2_vbi);
            
            if (mode->resolution == DSOR_720_480 )
            {
                //it's in the V_STATE_ACTIVE_VIDEO for the field2
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_ACTIVE_VIDEO, mode->v_total);
            }
            else if (mode->resolution == DSOR_720_576 )
            {
                field2_active = field2_vbi + (mode->v_active / 2) - 1;
                //it's in the V_STATE_ACTIVE_VIDEO for the field2
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_ACTIVE_VIDEO, field2_active);
            
                //it's in the V_STATE_HP_EQ at line field2_active + 1
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HP_EQ, field2_active + 1);
            
                //it's in the V_STATE_EQ_EQ to the end of field2
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_EQ_EQ, mode->v_total);
            }
        }
        else if (mode->scanmode == DSESM_PROGRESSIVE)
        {
        
            if (mode->field1_vsync_start > 1)
        	{
                //it's in the V_STATE_NP, from 1 to mode->field1_vsync_start;
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_NP, (mode->field1_vsync_start - 1));
        	}
        
            //it's in the V_STATE_BR, from field1_vsync_start to (field1_vsync_start + vsync_active_pulse_width - 1)
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_BR, (mode->field1_vsync_start + mode->vsync_active_pulse_width - 1));
        
            field1_vblank_length = (mode->v_total - mode->v_active);
            field1_vbi = ((mode->field1_vblank_start + field1_vblank_length) % mode->v_total) - 1;
            //it's in the V_STATE_NP, 
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_NP, field1_vbi);
        
            //it's in the V_STATE_ACTIVE_VIDEO to line (field1_vbi + mode->v_active) 
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_ACTIVE_VIDEO, (field1_vbi + mode->v_active));
        
            if ((field1_vbi + mode->v_active) < mode->v_total)
            {
                //it's in the V_STATE_NP to end of frame
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_NP, mode->v_total);
            }
        }
    }

    //for hd vertical timing
    else if ((mode->resolution == DSOR_1280_720 ) ||
             (mode->resolution == DSOR_1920_1080))
    {
        if (mode->scanmode == DSESM_PROGRESSIVE)
        {
            //it's in state V_STATE_HD_SYNC_PROG, for vertcal sync, from  field1_vsync_start to (field1_vsync_start + vsync_active_pulse_width - 1) 
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_SYNC_PROG, (mode->field1_vsync_start + mode->vsync_active_pulse_width - 1));
            
            //it's in state V_STATE_HD_VBI, for vblank line 
            field1_vblank_length = (mode->v_total - mode->v_active);
            field1_vbi = ((mode->field1_vblank_start + field1_vblank_length) % mode->v_total) - 1;
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_VBI, field1_vbi);
            
            //it's in the V_STATE_HD_ACTIVE to line (field1_vbi + mode->v_active) 
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_ACTIVE, (field1_vbi + mode->v_active));
            
            if ((field1_vbi + mode->v_active) < mode->v_total)
            {
                //it's in the V_STATE_HD_VBI to end of frame
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_VBI, mode->v_total);
            }
        }
        else if (mode->scanmode == DSESM_INTERLACED)
        {
            //it's in state V_STATE_HD_SYNC_INT, for vertcal sync, from  field1_vsync_start to (field1_vsync_start + vsync_active_pulse_width - 1) 
            current_line = mode->field1_vsync_start + mode->vsync_active_pulse_width - 1;
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_SYNC_INT, current_line);

            current_line ++;
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_FIELD1_SYNC_TO_BLANK_INT, current_line);

            field1_vblank_length = (mode->v_total - mode->v_active) / 2;
            field1_vbi = ((mode->field1_vblank_start + field1_vblank_length) % mode->v_total) -1;
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_VBI, field1_vbi);
            
            //it's in the V_STATE_TR_ACTIVE to line (field1_vbi + mode->v_active) 
            current_line = field1_vbi + (mode->v_active / 2);
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_ACTIVE, current_line);

            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_VBI, (mode->field2_vsync_start - 1));

            //transition line
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_FIELD2_BLANK_TO_SYNC_INT, mode->field2_vsync_start);

            //field2 sync line
            current_line = mode->field2_vsync_start + mode->vsync_active_pulse_width;
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_SYNC_INT, current_line);

            //field2 vbi
            current_line = mode->field2_vblank_start + ((mode->v_total - mode->v_active) / 2);
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_VBI, current_line);

            //field2 active
            current_line = current_line + (mode->v_active / 2);
            WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_ACTIVE, current_line);

            if (current_line < mode->v_total)
            {
                //it's in the V_STATE_HD_VBI to end of frame
                WRITE_HD_VENC0_VERTICAL_TIMING(V_STATE_HD_VBI, mode->v_total);
            }
        }
    }

    return;
}

/**************************************************************************************************
this function set the DISP encoder for composite output (CVBS), 
**************************************************************************************************/
void Set_VEnc_Horizontal_Timing(pxc_dfb_driver_data mdrv,
                                struct display_mode_struct * mode)
{
    u32 hd_venc0_addr;
    u32 tmp;
    u32 transition_sample;
    u32 broad_pulse_width;
    u32 narrow_pulse_width;
    u32 half_location_per_line;
    u32 sample_value;

    narrow_pulse_width = mode->hsync_active_pulse_width / 2;
    broad_pulse_width = mode->h_total - mode->hsync_active_edge_delay - mode->hsync_active_pulse_width;
    half_location_per_line = mode->h_total / 2;
    
    hd_venc0_addr = VENC_HORIZONTAL_STATE_START_ADDR;

    

    if ((mode->resolution == DSOR_1920_1080) &&
        (mode->scanmode == DSESM_INTERLACED))
    {
        //event 0: 	V_STATE_HD_FIELD2_BLANK_TO_SYNC_INT
        SET_HD_VENC_HORIZONTAL_STATE(0, mode->hsync_active_pulse_width, H_STATE_SYNC2); 

        sample_value = (mode->h_total / 2) - mode->hsync_active_pulse_width;
        SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);

        sample_value = sample_value + mode->hsync_active_pulse_width;
        SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_SYNC1);

        sample_value = sample_value + mode->hsync_active_pulse_width;
        SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_SYNC2);

        sample_value = sample_value + mode->hsync_active_edge_delay;
        SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);

        sample_value = mode->h_total - (mode->hsync_active_pulse_width * 2);
        SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_SYNC1);

        sample_value = mode->h_total - mode->hsync_active_pulse_width;
        SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);

        SET_HD_VENC_HORIZONTAL_STATE(0, mode->h_total, H_STATE_SYNC1);

        CLEAR_EMPTY_EVENT;
    }
    else
    {
        //event 0: define the line with two narrow negative equalization pulse
        SET_TWO_NEGATIVE_PULSE(H_STATE_BLANK,
                               H_STATE_SYNC1,
                               mode->hsync_active_edge_delay,
                               narrow_pulse_width,
                               half_location_per_line,
                               narrow_pulse_width,
                               mode->h_total);
    }

    //event 1: define the line with two broad negative pulse for serration
    SET_TWO_NEGATIVE_PULSE(H_STATE_BLANK,
                           H_STATE_SYNC1,
                           mode->hsync_active_edge_delay,
                           broad_pulse_width,
                           half_location_per_line,
                           broad_pulse_width,
                           mode->h_total);

    //event 2: define the line with one normal negative pulse
    SET_ONE_NEGATIVE_PULSE(H_STATE_BLANK,
                           H_STATE_SYNC1,
                           mode->hsync_active_edge_delay,
                           mode->hsync_active_pulse_width,
                           mode->h_total);

    //event 3: define the line that is active video 
    SET_ONE_NEGATIVE_PULSE_WITH_ACTIVE(H_STATE_BLANK,
                                       H_STATE_SYNC1,
                                       mode->hsync_active_edge_delay,
                                       mode->hsync_active_pulse_width,
                                       (mode->h_total - mode->h_active),
                                       mode->h_total);

    //event 4 : define the line with one normal negative pulse followed by one narrow negative pulse
    SET_TWO_NEGATIVE_PULSE(H_STATE_BLANK,
                           H_STATE_SYNC1,
                           mode->hsync_active_edge_delay,
                           mode->hsync_active_pulse_width,
                           half_location_per_line,
                           narrow_pulse_width,
                           mode->h_total);

    //event 5: define the line with one narrow negative pulse followed by one broad negative pulse
    SET_TWO_NEGATIVE_PULSE(H_STATE_BLANK,
                           H_STATE_SYNC1,
                           mode->hsync_active_edge_delay,
                           narrow_pulse_width,
                           half_location_per_line,
                           broad_pulse_width,
                           mode->h_total);

    //event 6: define line with one broad negative pulse followed by one narrow negative pulse
    SET_TWO_NEGATIVE_PULSE(H_STATE_BLANK,
                           H_STATE_SYNC1,
                           mode->hsync_active_edge_delay,
                           broad_pulse_width,
                           half_location_per_line,
                           narrow_pulse_width,
                           mode->h_total);

    //event 7: define the line with one narrow negative pulse
    SET_ONE_NEGATIVE_PULSE(H_STATE_BLANK,
                           H_STATE_SYNC1,
                           mode->hsync_active_edge_delay,
                           narrow_pulse_width,
                           mode->h_total);

    //event 8: define the line with one normal pulse folllowed by half pixture of samples
    SET_ONE_NEGATIVE_PULSE_WITH_ACTIVE(H_STATE_BLANK,
                                       H_STATE_SYNC1,
                                       mode->hsync_active_edge_delay,
                                       mode->hsync_active_pulse_width,
                                       ((mode->h_total - mode->h_active) + (mode->h_active / 2)),
                                       mode->h_total);

    //event 9: define the line with half pixture of samples followed by one narrow negative pulse
    SET_HALT_PIC_WITH_NARROW_PULSE(H_STATE_BLANK,
                                   H_STATE_SYNC1,
                                   mode->hsync_active_edge_delay,
                                   mode->hsync_active_pulse_width,
                                   (mode->h_total - mode->h_active),
                                   mode->h_total);

    //event 10: define the line with one broad negative pulse 
    SET_ONE_NEGATIVE_PULSE(H_STATE_BLANK,
                           H_STATE_SYNC1,
                           mode->hsync_active_edge_delay,
                           broad_pulse_width,
                           mode->h_total);

    //event 11: define the line with hd vertical sync, progressive
    SET_HD_VENC_HORIZONTAL_STATE(0, mode->hsync_active_pulse_width, H_STATE_SYNC2); 
    sample_value = mode->h_total - mode->hsync_active_edge_delay - mode->h_active;
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);
    sample_value = sample_value + mode->h_active; //broad pulse
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_SYNC1);
    sample_value = sample_value + (mode->hsync_active_edge_delay - mode->hsync_active_pulse_width);
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);
    SET_HD_VENC_HORIZONTAL_STATE(0, mode->h_total, H_STATE_SYNC1);
    CLEAR_EMPTY_EVENT;

    //event 12: define the line with hd vblank
    SET_HD_VENC_HORIZONTAL_STATE(0, mode->hsync_active_pulse_width, H_STATE_SYNC2); 
    sample_value = mode->h_total - mode->hsync_active_pulse_width;
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);
    SET_HD_VENC_HORIZONTAL_STATE(0, mode->h_total, H_STATE_SYNC1); 
    CLEAR_EMPTY_EVENT;

    //event 13: define the line with hd active video
    SET_HD_VENC_HORIZONTAL_STATE(0, mode->hsync_active_pulse_width, H_STATE_SYNC2); 
    sample_value = mode->h_total - mode->hsync_active_edge_delay - mode->h_active;
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);
    sample_value = sample_value + mode->h_active; //active 
    SET_HD_VENC_HORIZONTAL_STATE(ADD_BLACK_TO_COMPOSITE, sample_value, H_STATE_ACTIVE_VIDEO);
    sample_value = sample_value + (mode->hsync_active_edge_delay - mode->hsync_active_pulse_width);
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);
    SET_HD_VENC_HORIZONTAL_STATE(0, mode->h_total, H_STATE_SYNC1);
    CLEAR_EMPTY_EVENT;

    //event 14: define the line with hd sync , interlaced
    SET_HD_VENC_HORIZONTAL_STATE(0, mode->hsync_active_pulse_width, H_STATE_SYNC2); 
    sample_value = mode->hsync_active_edge_delay + mode->hsync_active_pulse_width;
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);
    sample_value = (mode->h_total / 2) - mode->hsync_active_edge_delay; //broad pulse
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_SYNC1);
    sample_value = (mode->h_total / 2) - mode->hsync_active_pulse_width;
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);
    sample_value = (mode->h_total / 2);
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_SYNC1);
    sample_value = sample_value + mode->hsync_active_pulse_width;
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_SYNC2);
    sample_value = mode->h_total - mode->hsync_active_pulse_width;
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);
    SET_HD_VENC_HORIZONTAL_STATE(0, mode->h_total, H_STATE_SYNC1);
    CLEAR_EMPTY_EVENT;

    //event 15: define the line hd for hd, two tri-level pulse, for interlace
    SET_HD_VENC_HORIZONTAL_STATE(0, mode->hsync_active_pulse_width, H_STATE_SYNC2); 
    sample_value = (mode->h_total / 2) - mode->hsync_active_pulse_width;
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);
    sample_value = sample_value + mode->hsync_active_pulse_width;
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_SYNC1);
    sample_value = sample_value + mode->hsync_active_pulse_width;
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_SYNC2);
    sample_value = mode->h_total - mode->hsync_active_pulse_width;
    SET_HD_VENC_HORIZONTAL_STATE(0, sample_value, H_STATE_BLANK);
    SET_HD_VENC_HORIZONTAL_STATE(0, mode->h_total, H_STATE_SYNC1);
    CLEAR_EMPTY_EVENT;


    CLEAR_EMPTY_HORIZONTAL_TIMING;

    return;
}

/**************************************************************************************************
this function set the DISP encoder for composite output (CVBS), 
**************************************************************************************************/
void Disp_Set_VEnc_Composite(pxc_dfb_driver_data mdrv,
                             const DFBScreenEncoderConfig *config,
                             struct display_mode_struct * mode)
{
    u32 tmp;
    u32 venc_cb_freq;

    //set the horizontal timing state array
    if (mode->resolution == DSOR_720_480 )
    {
        ;
    }

    //the dcs will be disabled
    REG_WRITE(DISP_HD_VENC0_DCS_CONFIG, (SD_COMPOSITE << DISP_HD_VENC0_DCS_CONFIG_SD_COMPOSITE_SHIFT));

    //set the subcaiirer frequency and phase
    venc_cb_freq = round((mode->subcarrier_freq / mode->sample_freq) * (0x01 << 31));
    REG_WRITE(DISP_SD_VENC0_CB_FREQ, venc_cb_freq);
    REG_WRITE(DISP_SD_VENC0_CB_PHASE, mode->subcarrier_phase);

    //set the ratio of component to composite DAC gain
    tmp = COMPOSITE_Y_GAIN | 
          (COMPOSITE_V_GAIN << DISP_HD_VENC0_CMPST_GAIN_CMPST_V_GAIN_SHIFT) |
          (COMPOSITE_U_GAIN << DISP_HD_VENC0_CMPST_GAIN_CMPST_U_GAIN_SHIFT) |
          (COMPOSITE_CB_GAIN << DISP_HD_VENC0_CMPST_GAIN_CMPST_CB_GAIN_SHIFT);

    REG_WRITE(DISP_HD_VENC0_CMPST_GAIN, tmp);
    
    //set the level of black, blank, sync1, sync2
    REG_WRITE(DISP_HD_VENC0_BLACK, COMPOSITE_DAC_BLACK_LEVEL);

    tmp = COMPOSITE_DAC_BLANK_LEVEL | (COMPOSITE_DAC_SYNC_STEP <<  DISP_HD_VENC0_BLANK_SYNC_STEP_SHIFT);
    REG_WRITE(DISP_HD_VENC0_BLANK, tmp);

    REG_WRITE(DISP_HD_VENC0_SYNC1, COMPOSITE_DAC_SYNC1_LEVEL);

    REG_WRITE(DISP_HD_VENC0_SYNC2, COMPOSITE_DAC_SYNC2_LEVEL);


    tmp = DISP_HD_VENC0_YCMPN_YCMPNGAIN | (YCMPN_YCMPNOFF_FOR_COMPOSITE << DISP_HD_VENC0_YCMPN_YCMPNOFF_SHIFT);
    REG_WRITE(DISP_HD_VENC0_YCMPN, tmp);

    tmp = DISP_HD_VENC0_CB2GAIN_CB2PBGAIN | (DISP_HD_VENC0_CB2GAIN_CB2PRGAIN << DISP_HD_VENC0_CB2GAIN_CB2PRGAIN_SHIFT);
    REG_WRITE(DISP_HD_VENC0_CB2GAIN, tmp);

    //set the vertical timing state array
    if (mode->resolution == DSOR_720_480 )
    {
         fill_vertical_timing_array(mdrv, mode);
    }

    //set the field boundary 
    REG_WRITE(DISP_HD_VENC0_FIELDBNDRY, mode->field2_vblank_start);

    //set the coefficient y composite
    REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS0, DEFAULT_Y_COMPOSITE0_COEF);
    REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS1, DEFAULT_Y_COMPOSITE1_COEF);
    REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS2, DEFAULT_Y_COMPOSITE2_COEF);
    REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS0, DEFAULT_UV_COMPOSITE0_COEF);
    REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS1, DEFAULT_UV_COMPOSITE1_COEF);
    REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS2, DEFAULT_UV_COMPOSITE2_COEF);
    
    //set the MACROVISION parameter
    
    return;
}

/**************************************************************************************************
this function set the DISP encoder for component output
**************************************************************************************************/
void Disp_Set_VEnc_Component(pxc_dfb_driver_data mdrv,
                            const DFBScreenEncoderConfig *config,
                            struct display_mode_struct * mode)
{
    u32 tmp;
    u32 hd_venc0_addr;
    u32 field1_vblank_length;
    u32 field_rate_50 = 0;
    u32 field_config;
    u32 field_config_shift;


    //Set the field resolution, frame rate
    if (mode->frequency == DSEF_50HZ )
    {
        field_rate_50 = FIELD_RATE_50_ENABLE;
    }

    switch (mode->resolution)
    {
        case DSOR_720_480:
        case DSOR_720_576:
            if (mode->scanmode == DSESM_INTERLACED )
            {
                field_config = SD_COMPONENT_I;
                field_config_shift =  DISP_HD_VENC0_DCS_CONFIG_SD_COMPONENT_I_SHIFT;
            }
            else
            {
                field_config = SD_COMPONENT_P;
                field_config_shift =  DISP_HD_VENC0_DCS_CONFIG_SD_COMPONENT_P_SHIFT;
            }
            break;

        case DSOR_1280_720 :
            field_config = HD_720P;
            field_config_shift =  DISP_HD_VENC0_DCS_CONFIG_HD_720P_SHIFT;
            break;

        case DSOR_1920_1080 :
            if (mode->scanmode == DSESM_INTERLACED )
            {
                field_config = HD_1080I;
                field_config_shift =  DISP_HD_VENC0_DCS_CONFIG_HD_1080I_SHIFT;
            }
            else
            {
                field_config = HD_1080P;
                field_config_shift =  DISP_HD_VENC0_DCS_CONFIG_HD_1080P_SHIFT;
            }
            break;
    }

    //the dcs will be disabled
    tmp = field_rate_50 | (field_config << field_config_shift);
    REG_WRITE(DISP_HD_VENC0_DCS_CONFIG, tmp);

     

    //set the level of black, blank, sync1, sync2
    REG_WRITE(DISP_HD_VENC0_BLACK, mode->black_level);

    tmp = mode->blank_level | (COMPONENT_DAC_SYNC_STEP <<  DISP_HD_VENC0_BLANK_SYNC_STEP_SHIFT);
    REG_WRITE(DISP_HD_VENC0_BLANK, tmp);

    REG_WRITE(DISP_HD_VENC0_SYNC1, mode->sync1_level);

    REG_WRITE(DISP_HD_VENC0_SYNC2, mode->sync2_level);

    tmp = DISP_HD_VENC0_YCMPN_YCMPNGAIN | (YCMPN_YCMPNOFF_FOR_COMPONENT << DISP_HD_VENC0_YCMPN_YCMPNOFF_SHIFT);
    REG_WRITE(DISP_HD_VENC0_YCMPN, tmp);

    tmp = DISP_HD_VENC0_CB2GAIN_CB2PBGAIN | (DISP_HD_VENC0_CB2GAIN_CB2PRGAIN << DISP_HD_VENC0_CB2GAIN_CB2PRGAIN_SHIFT);
    REG_WRITE(DISP_HD_VENC0_CB2GAIN, tmp);

    //set the vertical timing state array






    return;
}

/**************************************************************************************************
this function set the hdmi encoder of CI24210, 
**************************************************************************************************/
void Disp_Set_656_HDMI_Encoder(pxc_dfb_driver_data mdrv,
                               struct display_mode_struct * mode, 
							   const DFBScreenOutputConfig *config)
{
    int timeout;
    u32 in_I_P;
    u32 r_v_blank_in_osc;
    u32 tmp;
    u32 dvi_modez;
    u32 de_in_polarity;
    u32 vertical_active_width;
    u32 field1_vblank_length;
    u32 refresh_rate;
    u32 picture_aspect_ratio;
    u32 reg;
    u32 addr, data;
    DFBScreenOutputSignals internal_out_signals;
    double clock;
    u32 *phy_init_list;
    u32 spin_counter = 0;
    u32 h_active;
    u32 h_total;
    u32 h_active_edge_delay;
    u32 h_active_pulse_width;
    const u8  *csc_array;
    
    u32 str_cnt;

    /* Used as timeout to wait for Tx PHY Lock */
    u32 system_cnt_start;
    u32 current_system_cnt;
    u32 force_deep_color_off_hint;
	
    app_ioctl_command app_cmd;
    pid_t       current_pid;
    pthread_t   current_thread;
    
    
    
    REG_WRITE(DISP_HDMI_STATUS, 0x0f);
    REG_WRITE(DISP_HDMI_RDDATA, 0x0f);

    //frequency divide frac
    hdmi_reg_write(HDMI_MC_SFRDIV, 0x01, mdrv->fb0_handle);

    //Set the Color depth configuration
	/* 
	* For 480i, there is no need for repetition since the oversampling is done in the
	* DISP, however for 4x mode, we need to enable the repeat on the HDMI core PHY/PLL
    *
	*/
    
    
    /* Shut off any packets we are sending over HDMI ... */
    //hdmi_reg_write(HDMI_FC_DATAUTO0, DEFAULT_RDRB_MODE, mdrv->fb0_handle);

    //Configures the Frame Composer (FC) RDRB frame interpolation
    //hdmi_reg_write(HDMI_FC_DATAUTO1, 0x00, mdrv->fb0_handle);

    //hdmi_reg_write(HDMI_FC_DATAUTO2, 0x00, mdrv->fb0_handle);

    //hdmi_reg_write(HDMI_FC_DATAUTO3, 0x00, mdrv->fb0_handle);
    
    //usleep(50000);
    
    /* Shut off the HDCP engine if necessary ... */
    //hdmi_reg_write(HDMI_MC_CLKGAT, CLKGATE_HDCP_MASK, mdrv->fb0_handle);
    //app_cmd.uIOCTLCommand = CMD_APP_IOCTL_DISABLE_HDCP;
    //if (mdrv->fb0_handle != -1)
    //{
    //    ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd); 
    //}
        
    app_cmd.uIOCTLCommand =    CMD_APP_IOCTL_FORCE_DEEP_COLOR_OFF_HINT;
    if (mdrv->fb0_handle != -1)
    {
        /* Obtain force_deep_color_off_hint from FB driver ... */
        force_deep_color_off_hint = ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd);
        
        if (force_deep_color_off_hint != 0)
        {
            D_INFO("Force deep color off hint flag is set.\n");
        }
        
        if (dfb_config->disable_deep_color_hint != 0)
        {
            force_deep_color_off_hint = 0;
            /* Disable the hint to force deep color off .... */
        }
    }
    /* Gate off HDCP clock .. */
    //data = hdmi_reg_read(HDMI_MC_CLKGAT, mdrv->fb0_handle);
    //data|=  CLKGATE_HDCP_MASK;
    //hdmi_reg_write(HDMI_MC_CLKGAT, data, mdrv->fb0_handle);
    //sleep(3);
    //usleep(250000);
    /* Try sleeping for 3s after we instruct FB driver to shut off HDCP ... */
    
    /* Offer an option to force */
    if (dfb_config->force_deep_color == 0 && dfb_config->disable_deep_color == 0 &&
        force_deep_color_off_hint == 0)
    {
        internal_out_signals    =   config->out_signals & ~(DSOS_VGA | DSOS_YC | DSOS_CVBS | DSOS_HDMI | DSOS_656);
    }
    else if (dfb_config->force_deep_color == 1 && dfb_config->disable_deep_color == 0 &&
        force_deep_color_off_hint == 0)
    {
        internal_out_signals    =   DSOS_YCBCR36;
    }
    else if (dfb_config->disable_deep_color == 1 || 
             (force_deep_color_off_hint != 0 && 
             (mode->denominator == FULL_HD_DENOMINATOR_NO_FRAC || 
              mode->denominator == FULL_HD_DENOMINATOR_WITH_FRAC)
             )
            )
    {
        /* If we have DSOS_RGB30/RGB36 or DSOS_YCBCR30/36, set to the respective 24-bit modes ... */
        if (
            config->out_signals == DSOS_RGB30 ||
            config->out_signals == DSOS_RGB36
           )
        {
            internal_out_signals = DSOS_RGB;
            D_INFO("Forcing 24-bit RGB.\n");
        }
        else if (
                config->out_signals == DSOS_RGBL30 ||
                config->out_signals == DSOS_RGBL36
                )
        {
            internal_out_signals    =   DSOS_RGBL;
            D_INFO("Forcing 24-bit limited range RGB.\n");
        }
        else if (
                config->out_signals == DSOS_YCBCR30 ||
                config->out_signals == DSOS_YCBCR36
                )
        {
            internal_out_signals = DSOS_YCBCR;
            D_INFO("Forcing 24-bit YCbCr 4:4:4. \n");
        }
        else if (
                config->out_signals == DSOS_YC422_10 ||
                config->out_signals == DSOS_YC422_12
                )
        {
            internal_out_signals = DSOS_YC422;
            D_INFO("Forcing 8-bit per component YCC 4:2:2. \n");
        }
        else
        {
            internal_out_signals = config->out_signals & ~(DSOS_VGA | DSOS_YC | DSOS_CVBS | DSOS_HDMI | DSOS_656);
        }
          
    }
   
    
	switch (mode->outgoing_factor)
	{
		case 1:
			tmp = NO_PIXEL_REPETITION;
			break;
			
		case 2:
			tmp = PIXEL_REPEAT_ONCE;
			break;
			
		case 4:
			tmp = PIXEL_REPEAT_THREE;
			break;
			
		default:
			tmp = NO_PIXEL_REPETITION;
			D_DEBUG("HDMI: Mode has improper outgoing_factor, pixel repetition is disabled...\n");
			break;
	
	}
	
    switch (internal_out_signals)
	{
		case DSOS_RGB30:
        case DSOS_RGBL30:
		case DSOS_YCBCR30:
        case DSOS_YC422_10:
			//hdmi_reg_write(HDMI_TX_INVID0, YCBCR_444_10, mdrv->fb0_handle);
			tmp |= (DEPTH_30_BIT_10_PER_COMPONENT << COLOR_DEPTH_SHIFT);
			break;
			
		case DSOS_RGB36:
        case DSOS_RGBL36:
		case DSOS_YCBCR36:
        case DSOS_YC422_12:
			//hdmi_reg_write(HDMI_TX_INVID0, YCBCR_444_12, mdrv->fb0_handle);
			tmp |= (DEPTH_36_BIT_12_PER_COMPONENT << COLOR_DEPTH_SHIFT);
			break;
			
		default:
			//Set the Video input mode selection
			//hdmi_reg_write(HDMI_TX_INVID0, YCBCR_444_8, mdrv->fb0_handle);
			tmp |= (DEPTH_24_BIT_8_PER_COMPONENT << COLOR_DEPTH_SHIFT);
			break;
	
	
	}
    
    
    /* Check if we are entering deep color if so, write the following flags to the register () */
    switch (internal_out_signals)
    {
        case DSOS_RGB30:
        case DSOS_RGB36:
        case DSOS_RGBL30:
        case DSOS_RGBL36:
        case DSOS_YCBCR30:
        case DSOS_YCBCR36:
            /* Should come out to 0x46 ...  */
            hdmi_reg_write(HDMI_VP_CONF,   (BYPASS_ENABLE << BYPASS_ENABLE_SHIFT) |
                                                (DATA_FROM_VIDEO_PACKETIZER  << BYPASS_SELECT_SHIFT) |
                                                (DATA_FROM_8_BIT_BYPASS_BLOCK),
                                                mdrv->fb0_handle);
            usleep(75000);
            /* 75ms delay ... */
            break;
        
        case DSOS_YC422:
        case DSOS_YC422_10:
        case DSOS_YC422_12:
            
            hdmi_reg_write(HDMI_VP_CONF,  (YUV_422_ENABLE   << YUV_422_ENABLE_SHIFT) |
                                            (DATA_FROM_VIDEO_PACKETIZER  << BYPASS_SELECT_SHIFT) |
                                            (DATA_FROM_YCC422_REMAP_BLOCK),
                                            mdrv->fb0_handle);
            
            usleep(75000);
            break;
        

            
        default:
            break;
    }
    
    
     hdmi_reg_write(HDMI_VP_PR_CD, tmp, mdrv->fb0_handle);
    
    
   

    //physical reset (pull PHY out of reset)
    hdmi_reg_write (HDMI_MC_PHYRSTZ, HDMI_PHYSICAL_RESET, mdrv->fb0_handle);
   
    
    /* The HDMI Tx PHY has been pulled out of the reset, wait for PLL to lock
    * Then follow programming sequence for 20% PHY boost
    *
    */
    system_cnt_start = REG_READ(CG_SYSTEM_CNT);
    current_system_cnt = system_cnt_start;
    
	tmp = 0;

    /* Determine the clock rate, based on base dot clock and color depth */
    switch (mode->denominator)
    {
        case  SD_DENOMINATOR_NO_FRAC:
            /* 27.027 */
            clock = 27.027;
            break;
        
        case SD_DENOMINATOR_WITH_FRAC:
            /* 27 */
            clock = 27.000;
            break;
        
        case HD_DENOMINATOR_NO_FRAC:
            /* 74.25 */
            clock = 74.25;
            break;
        
        case HD_DENOMINATOR_WITH_FRAC:
            /* 74.176 */
            clock = 74.176;
            break;
            
        case FULL_HD_DENOMINATOR_WITH_FRAC:
            /* 148.35*/
            clock = 148.35;
            break;
            
        case FULL_HD_DENOMINATOR_NO_FRAC:
            /* 148.5 */
            clock = 148.5;
            break;
            
        case VGA_DENOMINATOR_25P2MHZ:
            clock = 25.2;
            break;
            
        case VGA_DENOMINATOR_25P175MHZ:
            clock = 25.175;
            break;
      
            
        default:
            D_DEBUG("NOTE: Invalid PLL Denominator set for this mode!!\n");
            clock = 27.00;
            break;
    }
    
    switch (internal_out_signals)
    {
        case DSOS_YCBCR30:
        case DSOS_RGB30:
        case DSOS_RGBL30:
            /* For HDMI, this is x1.25 */
            clock*= 1.25;
            D_DEBUG("30-bit: clock x 1.25 !!\n");
            break;
            
        case DSOS_YCBCR36:
        case DSOS_RGB36:
        case DSOS_RGBL36:
            /* For HDMI, this is x1.5 */
            clock*=1.5;
            D_DEBUG("36-bit: clock x 1.5 !!\n");
            break;
            
        default:
            clock*=1.0;
            D_DEBUG("24-bit: clock x 1.0 !!\n");
            break;
    
    }
    
    
    D_DEBUG("Approx. TMDS clock rate: %f \n", clock);
    
    if (clock < 145.0000)
    {
        /* Use a different PHY gain for modes which have clock rate below 148.5 ... */
        phy_init_list   =   (u32 *)&phy_boost_20_b1485;
        D_DEBUG("Using PHY_BOOST_20_B1485 for PHY init... \n");
    }
    else
    {
        /* Use the original PHY gains for modes 148.5 and above... */
        phy_init_list   =   (u32 *)&phy_boost_20;
        D_DEBUG("Using PHY_BOOST_20 for PHY init... \n");
    }
    
    if (internal_out_signals == DSOS_NONE )
    {
        //set the hdmi power down
        hdmi_reg_write(HDMI_PHY_CONF0, DISP_HDMI_POWER_DOWN, mdrv->fb0_handle);
    }
	else
	{
		
		//set the hdmi power down
		hdmi_reg_write(HDMI_PHY_CONF0, DISP_HDMI_POWER_DOWN, mdrv->fb0_handle);
	
		usleep(100000);
    
    //Source PHY Registers
    hdmi_reg_write (HDMI_PHY_CONF0, 0xfe, mdrv->fb0_handle);
	}

    //test clear signal
    hdmi_reg_write(HDMI_PHY_TST0, 0x20, mdrv->fb0_handle);
    hdmi_reg_write(HDMI_PHY_TST0, 0x00, mdrv->fb0_handle);

	while ( current_system_cnt < (system_cnt_start + PHY_TIMEOUT) )
    {
        current_system_cnt = REG_READ(CG_SYSTEM_CNT);
       
        tmp = hdmi_reg_read(HDMI_PHY_STAT0, mdrv->fb0_handle);
        if (tmp & HDMI_TX_PHY_LOCK_MASK)
        {
            break; /* Break the loop*/
        }
    
    }
/* The wait for the PHY PLL lock was also included twice, removing one instance ... */
#if 0
    while ( current_system_cnt < (system_cnt_start + PHY_TIMEOUT) )
    {
        current_system_cnt = REG_READ(CG_SYSTEM_CNT);
       
        tmp = hdmi_reg_read(HDMI_PHY_STAT0, mdrv->fb0_handle);
        if (tmp & HDMI_TX_PHY_LOCK_MASK)
        {
            break; /* Break the loop*/
        }
    
    }
#endif 

/* This was included twice during the merge process, this will remove one instance of this ...*/    
#if 0    
    tmp = hdmi_reg_read(HDMI_PHY_STAT0, mdrv->fb0_handle);
    if (!(tmp & HDMI_TX_PHY_LOCK_MASK))
    {
        /* PHY poll timeout...*/
        D_DEBUG("HDMI Tx PHY Lock timeout!!\n");
    }
    
    reg = 0;
    addr = phy_boost_40[reg];
    data = phy_boost_40[reg+1];
    while (addr != 0xFFFFFFFF && data != 0xFFFFFFFF)
    {
        D_DEBUG("Addr: 0x%X, Data: 0x%X\n", addr, data);
		/* Write data*/
        hdmi_reg_write(addr, data, mdrv->fb0_handle);
        reg+=2;
        addr = phy_boost_40[reg];
        data = phy_boost_40[reg+1];
		usleep(5);	/* Insert small delay, just to be sure..*/
    }
#endif    
  
	
    /* The PHY boost for this has been reverted to 20% ... */
    tmp = hdmi_reg_read(HDMI_PHY_STAT0, mdrv->fb0_handle);
    if (!(tmp & HDMI_TX_PHY_LOCK_MASK))
    {
        /* PHY poll timeout...*/
        D_DEBUG("HDMI Tx PHY Lock timeout!!\n");
    }
    
/* 
*   The following snippet of code programs the PHY settings.
*   They are absolutely necessary.
*/

// #if 0
    reg = 0;
    addr = phy_init_list[reg];
    data = phy_init_list[reg+1];
    while (addr != 0xFFFFFFFF && data != 0xFFFFFFFF)
    {
		D_DEBUG("Addr: 0x%X, Data: 0x%X\n", addr, data);
        
        /* Intercept the data (gain/pre-emphasis override ---) ... */
        if (data & PHY_MASK_ALL)
        {
            tmp =   data >> PHY_MASK_SHIFT;
            data &= (~PHY_MASK_ALL);
            
            /* We have two possible gain and two possible pre-emphasis values ... */
            
            if (tmp == PHY_VALUE_GAIN_HI && dfb_config->hdmi_phy_gain_hi != -1) 
            {
                D_WARN("HDMI PHY [HI] gain override (old: 0x%X, new: 0x%X)\n", 
                        (data & ~PHY_MASK_ALL),
                        dfb_config->hdmi_phy_gain_hi);
                        
                data =  dfb_config->hdmi_phy_gain_hi;
            }
            else if (tmp == PHY_VALUE_GAIN_LO && dfb_config->hdmi_phy_gain_lo != -1) 
            {
                D_WARN("HDMI PHY [LO] gain override (old: 0x%X, new: 0x%X)\n", 
                        (data & ~PHY_MASK_ALL),
                        dfb_config->hdmi_phy_gain_lo);
                        
                data =  dfb_config->hdmi_phy_gain_lo;
            }
            else if (tmp == PHY_VALUE_PREEMPHASIS_HI && dfb_config->hdmi_phy_preemphasis_hi != -1)
            {
                D_WARN("HDMI PHY [HI] pre-emphasis override (old: 0x%X, new: 0x%X)\n", 
                        (data & ~PHY_MASK_ALL),
                        dfb_config->hdmi_phy_preemphasis_hi);
                
                data =  dfb_config->hdmi_phy_preemphasis_hi;
            }
            else if (tmp == PHY_VALUE_PREEMPHASIS_LO && dfb_config->hdmi_phy_preemphasis_lo != -1)
            {
                D_WARN("HDMI PHY [LO] pre-emphasis override (old: 0x%X, new: 0x%X)\n", 
                        (data & ~PHY_MASK_ALL),
                        dfb_config->hdmi_phy_preemphasis_lo);
                
                data =  dfb_config->hdmi_phy_preemphasis_lo;
            }
        
        }
        
        
        /* Write data*/
        hdmi_reg_write(addr, data, mdrv->fb0_handle);
        reg+=2;
        addr = phy_init_list[reg];
        data = phy_init_list[reg+1];
		usleep(5);
		
    }
	
//#endif

    //input output pixel repetition ratio
    if ((mode->h_active / mode->xres) == 2 || 
		 mode->incoming_factor == 2
		)
    {
        /* 
		*	This has to be set to OUTPUT_REPEAT_ONCE since it is transmitted over AVI Inforame,
		*	even though repeat is done in DISP
		*/
		tmp = (INPUT_REPEAT_ONCE << INCOMING_PR_FACTOR_SHIFT) ;
    }
	else
    {
        tmp = (INPUT_NO_REPEAT << INCOMING_PR_FACTOR_SHIFT) ;
    }
	
	/* This needs to be set correctly as it is being sent over the AVI
	*  InfoFrame
	*/
	switch (mode->incoming_factor * mode->outgoing_factor)
	{
		case 1:	
			tmp|= OUTPUT_NO_REPEAT;
			break;
		
		case 2:
			tmp|= OUTPUT_REPEAT_ONCE;
			break;
			
		case 4:
			tmp|= OUTPUT_REPEAT_THREE;
			break;
			
		default:
			D_DEBUG("HDMI: Repeat is not 1, 2 or 4!! Sending no repeat...\n");
			tmp|= OUTPUT_NO_REPEAT;
			break;
	}
	
	hdmi_reg_write(HDMI_FC_PRCONF, tmp, mdrv->fb0_handle);

    

    //YCC422 remap of the Video Packetizer
    switch (internal_out_signals)
    {
        case DSOS_YC422_10:
            hdmi_reg_write(HDMI_VP_REMAP, YCC422_20BIT, mdrv->fb0_handle);
            break;
        
        case DSOS_YC422_12:
            hdmi_reg_write(HDMI_VP_REMAP, YCC422_24BIT, mdrv->fb0_handle);
            break;
        
        default:
            hdmi_reg_write(HDMI_VP_REMAP, YCC422_16BIT, mdrv->fb0_handle);
            break;
        
    
    }
    

    //Video Packetizer output selection, bypass select, YCC422 enable, pixel packing enabling.
	
    
    //FC_INVIDCONF configure the Interlaced/progressive, Vblank variation and polarity of all video synchronism of the input video signal.
    in_I_P = 
    (mode->scanmode == DSESM_INTERLACED && mode->resolution != DSOR_1920_2205 	) ? HDMI_INTERLACE_INPUT : HDMI_PROGRESSIVE_INPUT;
    r_v_blank_in_osc = RV_Blank_In_OSC(mode);
    dvi_modez = NONE_DVI_MODE;
    de_in_polarity = DATA_ENABLE_INPUT;
    tmp = in_I_P | 
          (r_v_blank_in_osc << R_V_BLANK_IN_OSC_SHIFT) | 
          (dvi_modez << DVI_MODEZ_SHIFT) |
          (de_in_polarity << DE_IN_POLARITY_SHIFT);
    
    switch (dfb_config->hdcp_mode)
    {
        case 0:
            tmp |= HDCP_KEEPOUT_MASK;
            break;
        
        case 2:
            if (config->out_connectors == DSOC_HDCP)
                tmp |= HDCP_KEEPOUT_MASK;
            break;
        
        default:
            break;
    }
		   
	/* 
	*	reg is used to configure the HDCP polarity registers 
	*	DE is always positive
	*	
	*/
	reg = DE_POS_MASK;
	
	
	if (mode->denominator == HD_DENOMINATOR_WITH_FRAC ||
		mode->denominator == HD_DENOMINATOR_NO_FRAC  ||
		mode->denominator == FULL_HD_DENOMINATOR_WITH_FRAC ||
		mode->denominator == FULL_HD_DENOMINATOR_NO_FRAC ||
		mdrv->asic_flags & VIPER_ASIC_A11
	   )
	{
	
		/* If we are running on an A11 ASIC or using HD, keep positive */
		tmp |= (HSYNC_IN_POLARITY_MASK | VSYNC_IN_POLARITY_MASK);
		reg |= (VSYNC_POS_MASK | HSYNC_POS_MASK);
	}
	else
	{
		/* Clear to 0 to keep negative*/
		
		tmp &= ~(HSYNC_IN_POLARITY_MASK | VSYNC_IN_POLARITY_MASK);
		reg &= ~(VSYNC_POS_MASK | HSYNC_POS_MASK);
	}
    
    reg |=  (UNENCRYPT_COLOR_BLUE << UNENCRYPT_COLOR_SHIFT);
	
    /* Configure HDCP polarity settings for this mode (important) */
	hdmi_reg_write(HDMI_A_HDCPPOLCFG, reg, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_INVIDCONF, tmp, mdrv->fb0_handle);
	
	//Hactive positions will need to be modified
	//They need to be multiplied by the outgoing_factor
	//of the mode to be set..
    h_active    =   mode->h_active *    mode->outgoing_factor;
    h_total     =   mode->h_total  *    mode->outgoing_factor;
	
    //set the Input video Horizontal active pixel region width
    tmp = h_active & FC_INHACTIV0_MASK;
    hdmi_reg_write(HDMI_FC_INHACTIV0, tmp, mdrv->fb0_handle);

    tmp = (h_active & FC_INHACTIV1_MASK) >> FC_INHACTIV1_SHIFT;
    hdmi_reg_write(HDMI_FC_INHACTIV1, tmp, mdrv->fb0_handle);

    //set the Input video Horizontal blanking pixel region width
    tmp = (h_total - h_active)  &  FC_INIHBLANK0_MASK;
    hdmi_reg_write(HDMI_FC_INHBLANK0, tmp, mdrv->fb0_handle);

    tmp = ((h_total - h_active) & FC_INIHBLANK1_MASK) >> FC_INIHBLANK1_SHIFT;
    hdmi_reg_write(HDMI_FC_INHBLANK1, tmp, mdrv->fb0_handle);

    //set Input video Vertical active pixel region width:
    if (mode->scanmode == DSESM_INTERLACED &&
        mode->resolution != DSOR_1920_2205 )
    {
        vertical_active_width = mode->v_active >> 1;
        field1_vblank_length = (mode->v_total - mode->v_active) / 2;
    }
    else
    {
        vertical_active_width = mode->v_active;
        field1_vblank_length = mode->v_total - mode->v_active;
    }
    tmp = vertical_active_width & FC_INVACTIV0_MASK;
    hdmi_reg_write(HDMI_FC_INVACTIV0, tmp, mdrv->fb0_handle);
	
	if (mdrv->asic_flags & VIPER_ASIC_A11)
	{
		/* A11 */
		tmp = (vertical_active_width & FC_INVACTIV1_MASK) >> FC_INVACTIV1_SHIFT;
		hdmi_reg_write(HDMI_FC_INVACTIV1, tmp, mdrv->fb0_handle);
	}
	else
	{
		/* A21 */
		tmp = (vertical_active_width & FC_INVACTIV1_MASK_A21) >> FC_INVACTIV1_SHIFT;
		hdmi_reg_write(HDMI_FC_INVACTIV1, tmp, mdrv->fb0_handle);
	
	}

    //set Input video Vertical blanking pixel region width
    if ((mode->denominator == FULL_HD_DENOMINATOR_NO_FRAC) ||
        (mode->denominator == FULL_HD_DENOMINATOR_WITH_FRAC) ||
        (mode->denominator == HD_DENOMINATOR_WITH_FRAC) ||
        (mode->denominator == HD_DENOMINATOR_NO_FRAC))
    {
		/* This is only done on A11 chip */
		if (mdrv->asic_flags & VIPER_ASIC_A11)
			field1_vblank_length = field1_vblank_length - mode->vsync_active_pulse_width;
    }
    hdmi_reg_write(HDMI_FC_INVBLANK, field1_vblank_length, mdrv->fb0_handle);
    
    
    h_active_edge_delay     =   mode->hsync_active_edge_delay   * mode->outgoing_factor;
    
    h_active_pulse_width    =   mode->hsync_active_pulse_width  * mode->outgoing_factor;   
    //Set Input video Hsync active edge delay
    tmp = h_active_edge_delay & FC_HSYNCINDELAY0_MASK;
    hdmi_reg_write(HDMI_FC_HSYNCINDELAY0, tmp, mdrv->fb0_handle);

    tmp = (h_active_edge_delay & FC_HSYNCINDELAY1_MASK) >> FC_HSYNCINDELAY1_SHIFT;
    hdmi_reg_write(HDMI_FC_HSYNCINDELAY1, tmp, mdrv->fb0_handle);

    //set Input video Hsync active pulse width
    tmp = h_active_pulse_width & FC_HSYNCINWIDTH0_MASK;
    hdmi_reg_write(HDMI_FC_HSYNCINWIDTH0, tmp, mdrv->fb0_handle);

    tmp = (h_active_pulse_width & FC_HSYNCINWIDTH1_MASK) >> FC_HSYNCINWIDTH1_SHIFT;
    hdmi_reg_write(HDMI_FC_HSYNCINWIDTH1, tmp, mdrv->fb0_handle);

    //set Input video Vsync active edge delay
    hdmi_reg_write(HDMI_FC_VSYNCINDELAY, mode->vsync_active_edge_delay, mdrv->fb0_handle);

    //set Input video Vsync active pulse width
    tmp = mode->vsync_active_pulse_width & FC_VSYNCINWIDTH_MASK;
    hdmi_reg_write(HDMI_FC_VSYNCINWIDTH, tmp, mdrv->fb0_handle);

    //set video refresh rate
    refresh_rate = get_hdmi_video_refresh_rate(mode);

    tmp = refresh_rate & FC_INFREQ0_MASK;
    hdmi_reg_write(HDMI_FC_INFREQ0, tmp, mdrv->fb0_handle);

    tmp = (refresh_rate & FC_INFREQ1_MASK) >> FC_INFREQ1_SHIFT;
    hdmi_reg_write(HDMI_FC_INFREQ1, tmp, mdrv->fb0_handle);

    tmp = (refresh_rate & FC_INFREQ2_MASK) >> FC_INFREQ2_SHIFT;
    hdmi_reg_write(HDMI_FC_INFREQ2, tmp, mdrv->fb0_handle);
    
    
    
    /* Retrieve the current CSC array */
    csc_array   =   get_csc_array(mode, internal_out_signals);
    
    
    /* Program CSC array in HDMI Tx ONLY if return value is non-zero */
    if (csc_array)
    {
        for (reg = HDMI_CSC_COEFA1_MSB ; reg <= HDMI_CSC_COEFC4_LSB; reg++)
		{
			hdmi_reg_write(reg, csc_array[reg - HDMI_CSC_COEFA1_MSB], mdrv->fb0_handle);
			/* Update the matrix to perform CSC */
		}
        
        /* 
        *   The appropriate CSC_SCALE value for the conversion matrix is directly 
        *   after the matrix elements in the array. 
        */
        hdmi_reg_write(HDMI_CSC_SCALE, csc_array[reg - HDMI_CSC_COEFA1_MSB], mdrv->fb0_handle);
        D_DEBUG("HDMI_CSC_SCALE = 0x%X\n", csc_array[reg - HDMI_CSC_COEFA1_MSB]);
    }
    
    /* VP_STUFF register ... */
    switch (internal_out_signals)
	{
	
		case DSOS_RGB:
        case DSOS_RGBL:
        case DSOS_RGB30:
        case DSOS_RGBL30:
		case DSOS_RGB36:
        case DSOS_RGBL36:
			
			/* Enable CSC */
			hdmi_reg_write(HDMI_MC_FLOWCTRL, COLOR_CONVERT_ENABLE, mdrv->fb0_handle);
			hdmi_reg_write(HDMI_FC_AVICONF0, AVICONF_RGB, mdrv->fb0_handle);
			
			/* Configure CSC output */
			//tmp = 1 | (COLOR_DEPTH_36_BIT_PER_PIXEL<< COLOR_SPACE_CONV_COLOR_DEPTH_SHIFT);
			//hdmi_reg_write(HDMI_CSC_SCALE, tmp, mdrv->fb0_handle);
			
			//Set the controls the Pixel repetition, pixel packing and YCC422 stuffing
			hdmi_reg_write(HDMI_VP_STUFF, 0x00, mdrv->fb0_handle);
			break;
			
		case DSOS_YCBCR30:
        case DSOS_YCBCR36:
			hdmi_reg_write(HDMI_MC_FLOWCTRL, COLOR_CONVERT_BYPASS, mdrv->fb0_handle);
			hdmi_reg_write(HDMI_FC_AVICONF0, AVICONF_YCBCR_444, mdrv->fb0_handle);
			
            //tmp = 1 | (COLOR_DEPTH_36_BIT_PER_PIXEL << COLOR_SPACE_CONV_COLOR_DEPTH_SHIFT);
			//hdmi_reg_write(HDMI_CSC_SCALE, tmp, mdrv->fb0_handle);
			
			//Set the controls the Pixel repetition, pixel packing and YCC422 stuffing
			hdmi_reg_write(HDMI_VP_STUFF, 0x00, mdrv->fb0_handle);
			
			break;
        
        case DSOS_YC422:
        case DSOS_YC422_10:
        case DSOS_YC422_12:
            hdmi_reg_write(HDMI_MC_FLOWCTRL, COLOR_CONVERT_BYPASS, mdrv->fb0_handle);
			hdmi_reg_write(HDMI_FC_AVICONF0, AVICONF_YCBCR_422, mdrv->fb0_handle);
            break;
        
        
			
		default:
			
			//bypass the color conversion 
			hdmi_reg_write(HDMI_MC_FLOWCTRL, COLOR_CONVERT_BYPASS, mdrv->fb0_handle);
			hdmi_reg_write(HDMI_FC_AVICONF0, AVICONF_YCBCR_444, mdrv->fb0_handle);
			
			/* Configure CSC output */
			//tmp = 1 | (COLOR_DEPTH_24_BIT_PER_PIXEL<< COLOR_SPACE_CONV_COLOR_DEPTH_SHIFT);
			//hdmi_reg_write(HDMI_CSC_SCALE, tmp, mdrv->fb0_handle);
			
			//Set the controls the Pixel repetition, pixel packing and YCC422 stuffing
			hdmi_reg_write(HDMI_VP_STUFF, 0x00, mdrv->fb0_handle);
			/* TODO: This is a HACK!!! Modify the DISP_HD output limits... */
            if (internal_out_signals == DSOS_XVYCC)
            {
                REG_WRITE(DISP_HD_OUTPUT_LIMIT, 0x01FE01FE);
            /* Video layers need to be changed as well.... */
            //    REG_WRITE(DISP_HD_INPUT_Y_LIMIT, 0x6E1000FF);
            //    REG_WRITE(DISP_HD_INPUT_CBCR_LIMIT, 0xFF);
            }
			break;
	}
    
    /* FC_GCP register will need to be updated here as well ... */
    
    
    /* Update the VP_CONF register ... */
    switch (internal_out_signals)
    {
        case DSOS_RGB30:
        case DSOS_RGBL30:
        case DSOS_YCBCR30:
        case DSOS_RGB36:
        case DSOS_RGBL36:
        case DSOS_YCBCR36:
            /* 
            *	For bigger bit modes (larger than 8), we need to 
            *	Take data from pixel packer
            *	Disable bypass block
            *	Enable pixel packer
            */
            //tmp =      (PIXEL_PACKET_ENABLE << PIXEL_PACKET_ENABLE_SHIFT)   |
            //           (DATA_FROM_VIDEO_PACKETIZER << BYPASS_SELECT_SHIFT)  |
            //           DATA_FROM_PIXEL_PACKING_BLOCK;
            
            //tmp =      (PIXEL_PACKET_ENABLE << PIXEL_PACKET_ENABLE_SHIFT)   |
            //          (DATA_FROM_VIDEO_PACKETIZER << BYPASS_SELECT_SHIFT)  |
            //           DATA_FROM_8_BIT_BYPASS_BLOCK;
            
            /* Enable pixel packing block but do not switch video output ... */
            //hdmi_reg_write(HDMI_VP_CONF, tmp, mdrv->fb0_handle);
            
            tmp =      (PIXEL_PACKET_ENABLE << PIXEL_PACKET_ENABLE_SHIFT)   |
                       (DATA_FROM_VIDEO_PACKETIZER << BYPASS_SELECT_SHIFT)  |
                       DATA_FROM_PIXEL_PACKING_BLOCK;
            
            
            /* Sleep for 75 ms ... */
            usleep(75000);
            break;
        
        case DSOS_YC422:
        case DSOS_YC422_10:
        case DSOS_YC422_12:
            tmp =   (YUV_422_ENABLE   << YUV_422_ENABLE_SHIFT) |
                    (DATA_FROM_VIDEO_PACKETIZER  << BYPASS_SELECT_SHIFT) |
                    (DATA_FROM_YCC422_REMAP_BLOCK);
            /* Sleep for 75 ms ... */
            usleep(75000);                               
            break;
            
        default:
            tmp =      (BYPASS_ENABLE << BYPASS_ENABLE_SHIFT) |
                       (DATA_FROM_VIDEO_PACKETIZER  << BYPASS_SELECT_SHIFT) |
                       (DATA_FROM_8_BIT_BYPASS_BLOCK);
            break;
    }
	
   
		  
    hdmi_reg_write(HDMI_VP_CONF, tmp, mdrv->fb0_handle);

    
    
    /* Updates TX_INVID0 register ... */
    switch (internal_out_signals)
	{
		case DSOS_RGB30:
        case DSOS_RGBL30:
		case DSOS_YCBCR30:
        	hdmi_reg_write(HDMI_TX_INVID0, YCBCR_444_10, mdrv->fb0_handle);
			tmp = (DEPTH_30_BIT_10_PER_COMPONENT << COLOR_DEPTH_SHIFT);
			break;
			
		case DSOS_RGB36:
        case DSOS_RGBL36:
		case DSOS_YCBCR36:
        	hdmi_reg_write(HDMI_TX_INVID0, YCBCR_444_12, mdrv->fb0_handle);
			tmp = (DEPTH_36_BIT_12_PER_COMPONENT << COLOR_DEPTH_SHIFT);
			break;
			
		default:
			//Set the Video input mode selection
			hdmi_reg_write(HDMI_TX_INVID0, YCBCR_444_8, mdrv->fb0_handle);
			tmp = (DEPTH_24_BIT_8_PER_COMPONENT << COLOR_DEPTH_SHIFT);
			break;
	
	
	}
    

    //set the control period minimum duration
    //hdmi_reg_write(HDMI_FC_CTRLDUR, HDMI_CONTROL_PERIOD_MINIMUM_DURATION, mdrv->fb0_handle);

    //set the extended control period minimum duration
    hdmi_reg_write(HDMI_FC_EXCTRLDUR, HDMI_EXTENDED_CONTROL_PERIOD_MINIMUM_DURATION, mdrv->fb0_handle);

    //set the maximum spacing between consecutive extended control period
    hdmi_reg_write(HDMI_FC_EXCTRLSPAC, HDMI_EXT_CTRL_PERIOD_SPACE, mdrv->fb0_handle);

    //set the 8bits that are going to fill the channel 0 data lines not used to transmit the preamble
    hdmi_reg_write(HDMI_FC_CH0PREAM, HDMI_CH0_PREAMBLE_FILTER, mdrv->fb0_handle);

    //set the 6bits that are going to fill the channel 1 data lines not used to transmit the preamble
    hdmi_reg_write(HDMI_FC_CH1PREAM, HDMI_CH1_PREAMBLE_FILTER, mdrv->fb0_handle);

    //set the 6bits that are going to fill the channel 2 data lines not used to transmit the preamble
    hdmi_reg_write(HDMI_FC_CH2PREAM, HDMI_CH2_PREAMBLE_FILTER, mdrv->fb0_handle);

    //the General Control Packet A/V mute indicators and the default phase
    //hdmi_reg_write(HDMI_FC_GCP, 0x00, mdrv->fb0_handle);
    
   

    //define the colormetry, picture aspect ratio, active format ratio
    if (mode->slow_blanking == DSOSB_4x3)
    {
        picture_aspect_ratio = PICTURE_ASPECT_RATIO_4X3;
    }
    else if (mode->slow_blanking == DSOSB_16x9)
    {
        picture_aspect_ratio = PICTURE_ASPECT_RATIO_16X9;
    }
    /* We need to indicate the correct colorimetry in the HDMI info packet ... */
    
    
    tmp = picture_aspect_ratio << PICTURE_ASPECT_RATIO_SHIFT | 
    (SAME_AS_PICTURE_ASPECT_RATIO << ACTIVE_ASPECT_RATIO_SHIFT);
     
    if (internal_out_signals    !=  DSOS_XVYCC)
    {
        if (mdrv->shared->current_hd_color_space ==  DSCS_601)
        {
            tmp    |=  (COLORIMETRY_ITU601 << COLORIMETRY_SHIFT);
        }
        else if (mdrv->shared->current_hd_color_space  ==  DSCS_709)
        {
            tmp   |=  (COLORIMETRY_ITU709 << COLORIMETRY_SHIFT);
        }
        else
        {
            /* Assume 709*/
            tmp    |=  (COLORIMETRY_ITU709 << COLORIMETRY_SHIFT);
        }
    }
    else
    {
        tmp   |=  (COLORIMETRY_XVYCC << COLORIMETRY_SHIFT);
        
        /* 
        *   Set C0 and C1 to indicate that there is valid extended colorimetry information 
        *   Only enabled in XVYCC mode ... 
        */
    }
   
    /* Append colorimetry settings if XVYCC is in... */       
    hdmi_reg_write(HDMI_FC_AVICONF1, tmp, mdrv->fb0_handle);

    //colorimetry, RGB Quantization Range
    if (internal_out_signals != DSOS_XVYCC)
    {
        tmp = 0;
        switch (internal_out_signals)
        {
            case DSOS_RGBL:
            case DSOS_RGBL30:
            case DSOS_RGBL36:
                tmp |= QUANTIZATION_RANGE_LIMITED << QUANTIZATION_RANGE_SHIFT;
                break;
            case DSOS_RGB:
            case DSOS_RGB30:
            case DSOS_RGB36:
                tmp |= QUANTIZATION_RANGE_FULL << QUANTIZATION_RANGE_SHIFT;
                break;
        }
        hdmi_reg_write(HDMI_FC_AVICONF2, tmp, mdrv->fb0_handle);
    }
    else
    {
        /* 
        *   This parameter needs to be setup based on the output color space,
        *   whether it is 709 or 601... 
        */
        if (mdrv->shared->current_hd_color_space  ==  DSCS_601)
        {
            /* ITU601 color space ... */
            hdmi_reg_write(HDMI_FC_AVICONF2, 
                COLORIMETRY_XVYCC601    << EXTENDED_COLORIMETRY_SHIFT, 
                mdrv->fb0_handle);
            
        }
        else if (mdrv->shared->current_hd_color_space == DSCS_709)
        {
            /* ITU709 color space ... */
            hdmi_reg_write(HDMI_FC_AVICONF2, 
                COLORIMETRY_XVYCC709    << EXTENDED_COLORIMETRY_SHIFT, 
                mdrv->fb0_handle);
           
        }
        else
        {
            /* Stick with ITU709 color space in the unknown case ... */
            hdmi_reg_write(HDMI_FC_AVICONF2, 
                COLORIMETRY_XVYCC709    << EXTENDED_COLORIMETRY_SHIFT, 
                mdrv->fb0_handle);
        }
        
    }
    //set the video id code as table 3 in the CEA-861-D	
    hdmi_reg_write(HDMI_FC_AVIVID, mode->video_id_code, mdrv->fb0_handle);

    //AVI infoFrame End of Top Bar value
    hdmi_reg_write(HDMI_FC_AVIETB0, 0, mdrv->fb0_handle);
    hdmi_reg_write(HDMI_FC_AVIETB1, 0, mdrv->fb0_handle);

    //AVI infoFrame Start of Bottom Bar value
    hdmi_reg_write(HDMI_FC_AVISBB0, 0, mdrv->fb0_handle);
    hdmi_reg_write(HDMI_FC_AVISBB1, 0, mdrv->fb0_handle);

    //AVI infoFrame End of Left Bar value
    hdmi_reg_write(HDMI_FC_AVIELB0, 0, mdrv->fb0_handle);
    hdmi_reg_write(HDMI_FC_AVIELB1, 0, mdrv->fb0_handle);

    //AVI infoFrame End of Start of Right Bar value
    hdmi_reg_write(HDMI_FC_AVISRB0, 0, mdrv->fb0_handle);
    hdmi_reg_write(HDMI_FC_AVISRB1, 0, mdrv->fb0_handle);

    //set the video id code as table 3 in the CEA-861-D	
    //set the vendor id 
    
    if ((mode->resolution == DSOR_1280_1470) ||
        (mode->resolution == DSOR_1920_1080_3D_SIDE_BY_SIDE) ||
		(mode->resolution == DSOR_1920_2205) || 
        (config->resolution == DSOR_1920_1080_3D_SIDE_BY_SIDE) ||
        get_alias_type(config) != ALIAS_NONE )
	{
        //set 3d mode
        hdmi_reg_write(HDMI_FC_VSDIEEEID0, 0x03, mdrv->fb0_handle);

        hdmi_reg_write(HDMI_FC_VSDIEEEID1, 0x0c, mdrv->fb0_handle);
        
        hdmi_reg_write(HDMI_FC_VSDIEEEID2, 0x00, mdrv->fb0_handle);
        
        
    }
    else
    {
        hdmi_reg_write(HDMI_FC_VSDIEEEID0, VIXS_ID, mdrv->fb0_handle);

        hdmi_reg_write(HDMI_FC_VSDIEEEID1, 0, mdrv->fb0_handle);
        
        hdmi_reg_write(HDMI_FC_VSDIEEEID2, 0, mdrv->fb0_handle);
       
    }


    //set the Source Product Descriptor infoFrame description device field
    if (dfb_config->device_information == -1)
    {
        hdmi_reg_write(HDMI_FC_SPDDEVICEINF, BLU_RAY_DISC, mdrv->fb0_handle);
    }
    else
    {
        hdmi_reg_write(HDMI_FC_SPDDEVICEINF, dfb_config->device_information, mdrv->fb0_handle);
    }

    //the number of high priority packets
    hdmi_reg_write(HDMI_FC_CTRLQHIGH, 0x0f, mdrv->fb0_handle);

    //the number of low priority packets
    hdmi_reg_write(HDMI_FC_CTRLQLOW, 0x03, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_ACP0, 0x01, mdrv->fb0_handle);

    //set the ISRC  
    hdmi_reg_write(HDMI_FC_ISCR1_0, ISRC_VALID | ISRC_CONT, mdrv->fb0_handle);

    //set the RDRB mode
    

    //Configures the Frame Composer (FC) RDRB frame interpolation
    hdmi_reg_write(HDMI_FC_DATAUTO1, 0x00, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_DATAUTO2, 0x10, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_DATAUTO3, 0x0f, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_RDRB0, 0, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_RDRB1, 0, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_RDRB2, 0, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_RDRB3, 0, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_RDRB4, 0, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_RDRB5, 0, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_RDRB5, 0, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_RDRB7, 0, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_GMD_EN, 0, mdrv->fb0_handle);

    hdmi_reg_write(HDMI_FC_GMD_CONF, 0x10, mdrv->fb0_handle);

    //disable interpolation and decimation
    hdmi_reg_write(HDMI_CSC_CFG, 0x00, mdrv->fb0_handle);

    //Color space converter color depth configuration
  
	/* Read the value at VP_STUFF */
    data = hdmi_reg_read(HDMI_VP_STUFF, mdrv->fb0_handle);
    D_DEBUG("HDMI_VP_STUFF before programming.. 0x%X\n", data);
	/* Pass through for CSC */
	
    
    
   

    //set the color space conversion
    for (reg = HDMI_TX_INSTUFFING; reg <= HDMI_TX_BCBDATA1; reg++)
    {
        hdmi_reg_write(reg, 0, mdrv->fb0_handle);
    }

    //set the Vendor Specific infoFrame 24 bytes specific payload
    for (reg = HDMI_FC_VSDPAYLOAD0; reg <= HDMI_FC_VSDPAYLOAD23; reg++)
    {

        hdmi_reg_write(reg, 0, mdrv->fb0_handle);
    }

    // set the 3d frame packING mode
    if (mode->resolution == DSOR_1280_1470 ||
		mode->resolution == DSOR_1920_2205
		)
		
    {
        /* Signal frame packing to the TV ... */
        hdmi_reg_write(HDMI_FC_VSDPAYLOAD0, STRUCTURE_3D, mdrv->fb0_handle);
        tmp = FRAME_PACKING_3D << MODE_SHIFT_3D;
        hdmi_reg_write(HDMI_FC_VSDPAYLOAD1, tmp, mdrv->fb0_handle);
        hdmi_reg_write(HDMI_FC_DATAUTO0, VSDB_RDRB_MODE, mdrv->fb0_handle);
    }
    //set 3d side by side
    else if (mode->resolution == DSOR_1920_1080_3D_SIDE_BY_SIDE ||
             config->resolution == DSOR_1920_1080_3D_SIDE_BY_SIDE ||
             get_alias_type(config) == ALIAS_3D_SIDE_BY_SIDE )
    {
        hdmi_reg_write(HDMI_FC_VSDPAYLOAD0, STRUCTURE_3D, mdrv->fb0_handle);
        /* Signal side by side to the TV ... */
        tmp = SIDE_BY_SIDE_3D << MODE_SHIFT_3D;
        hdmi_reg_write(HDMI_FC_VSDPAYLOAD1, tmp, mdrv->fb0_handle);
        hdmi_reg_write(HDMI_FC_DATAUTO0, VSDB_RDRB_MODE, mdrv->fb0_handle);
    }
    else if (get_alias_type(config) == ALIAS_3D_TOP_AND_BOT)
    {
        /* Signal top and bottom to the TV ... */
        hdmi_reg_write(HDMI_FC_VSDPAYLOAD0, STRUCTURE_3D, mdrv->fb0_handle);
        tmp = TOP_AND_BOTTOM_3D << MODE_SHIFT_3D;
        hdmi_reg_write(HDMI_FC_VSDPAYLOAD1, tmp, mdrv->fb0_handle);
        hdmi_reg_write(HDMI_FC_DATAUTO0, VSDB_RDRB_MODE, mdrv->fb0_handle);
    }
    else
    {
        /* 2D mode.. do nothing */
        hdmi_reg_write(HDMI_FC_DATAUTO0, DEFAULT_RDRB_MODE, mdrv->fb0_handle);
    }
    
    

    //set the control period minimum duration
    hdmi_reg_write(HDMI_FC_CTRLDUR, HDMI_CONTROL_PERIOD_MINIMUM_DURATION, mdrv->fb0_handle);
    
    str_cnt = 0;

  /*********************David Li added 2011-03-01 ********************************/
   //set the Source Product Descriptor infoFrame 8 bytes Vendor name
    if( dfb_config->vendor_name)
    {
       for (reg = HDMI_FC_SPDVENDORNAME0; reg <= HDMI_FC_SPDVENDORNAME7; reg++, str_cnt++)
       {
          if (str_cnt < dfb_config->vendor_length)
          {
            hdmi_reg_write(reg, dfb_config->vendor_name[reg - HDMI_FC_SPDVENDORNAME0], mdrv->fb0_handle);
            //D_INFO("reg[0x%X] = val[0x%X]\n", reg, dfb_config->vendor_name[reg - HDMI_FC_SPDVENDORNAME0]);
          }
          else
          {
            /* Force to 0 */
            hdmi_reg_write(reg, 0x00, mdrv->fb0_handle);
            //D_INFO("reg[0x%X] = val[0x%X]\n", reg, 0);
          }
       }
    }else{
       for (reg = HDMI_FC_SPDVENDORNAME0; reg <= HDMI_FC_SPDVENDORNAME7; reg++)
       {
          hdmi_reg_write(reg, source_product_descriptor_vendor_name[reg - HDMI_FC_SPDVENDORNAME0], mdrv->fb0_handle);
          //D_INFO("reg[0x%X] = val[0x%X]\n", reg, dfb_config->vendor_name[reg - HDMI_FC_SPDVENDORNAME0]);
       }
    }
    
    str_cnt = 0;
    //set the Source Product Descriptor infoFrame 16 bytes Product name
    if( dfb_config->product_name)
    {
       for (reg = HDMI_FC_SPDPRODUCTNAME0; reg <= HDMI_FC_SPDPRODUCTNAME15; reg++, str_cnt++)
       {
          if (str_cnt < dfb_config->product_length)
          {
            hdmi_reg_write(reg, dfb_config->product_name[reg - HDMI_FC_SPDPRODUCTNAME0], mdrv->fb0_handle);
            //D_INFO("reg[0x%X] = val[0x%X]\n", reg, dfb_config->product_name[reg - HDMI_FC_SPDPRODUCTNAME0]);
          }
          else
          {
            /* Force to 0 */
            hdmi_reg_write(reg, 0x00, mdrv->fb0_handle);
            //D_INFO("reg[0x%X] = val[0x%X]\n", reg, 0);
          }
       }
    }else{
       for (reg = HDMI_FC_SPDPRODUCTNAME0; reg <= HDMI_FC_SPDPRODUCTNAME15; reg++)
       {
          hdmi_reg_write(reg, source_product_descriptor_product_name[reg - HDMI_FC_SPDPRODUCTNAME0], mdrv->fb0_handle);
          //D_INFO("reg[0x%X] = val[0x%X]\n", reg, dfb_config->vendor_name[reg - HDMI_FC_SPDVENDORNAME0]);
       }
    }
/***************************************************************************************/
    
    /* Program the XVYCC packet descriptors (if necessary).. */
    if (internal_out_signals == DSOS_XVYCC)
    {
        hdmi_reg_write(HDMI_FC_GMD_EN, GMD_ENABLE_PACKET_MASK, mdrv->fb0_handle);
        /* Enable the sending of the Gamut Metadata descriptor packet... */
        
        /* Set this to default values...  */
        hdmi_reg_write(HDMI_FC_GMD_CONF, HDMI_FC_GMD_CONF_DEFAULT, mdrv->fb0_handle);
        
        /* Update profile register ... */
        hdmi_reg_write(HDMI_FC_GMD_HB,
                ((GMD_HB_PROFILE_P0 << GMD_HB_PROFILE_SHIFT) & GMD_HB_PROFILE_MASK) |
                ((0 << GMD_HB_AFFECTED_SHIFT) & GMD_HB_AFFECTED_MASK),   /* Fix at zero for now */
                mdrv->fb0_handle);
        D_DEBUG("GMD body programming...\n");
        /* Program GMD packet body into the registers... */
        for (reg = HDMI_FC_GMD_PB0 ; reg  < (HDMI_FC_GMD_PB0 + xvycc_gmd_length) ; reg++)
        {
            hdmi_reg_write(reg, xvycc_gmd_packet_body[reg - HDMI_FC_GMD_PB0], mdrv->fb0_handle);
            D_DEBUG("HDMI[0x%X] = 0x%X\n", reg, xvycc_gmd_packet_body[reg - HDMI_FC_GMD_PB0]);
        
        }
        
        /* Perform a GMD packet update... flag is self clearing...  */
        hdmi_reg_write(HDMI_FC_GMD_UP, GMD_UPDATE_PACKET_MASK, mdrv->fb0_handle);
        
        
        
    }
    else
    {
        tmp = hdmi_reg_read(HDMI_FC_GMD_EN, mdrv->fb0_handle);
        tmp &=  (~GMD_ENABLE_PACKET_MASK);
        hdmi_reg_write(HDMI_FC_GMD_EN, tmp, mdrv->fb0_handle);
        /* Disable the GMD packet.. */
    
    }
    

    //set the ISCR1 packet
    for (reg = HDMI_FC_ISCR1_1; reg <= HDMI_FC_ISCR1_16; reg++)
    {
        hdmi_reg_write(reg, iscr1_packet[reg - HDMI_FC_ISCR1_1], mdrv->fb0_handle);
    }

    //set the ISCR2 packet
    for (reg = HDMI_FC_ISCR2_0; reg <= HDMI_FC_ISCR2_15; reg++)
    {
        hdmi_reg_write(reg, iscr2_packet[reg - HDMI_FC_ISCR2_0], mdrv->fb0_handle);
    }

    //set the the data island packet insertion for  SPD, VSD, ISCR2, ISCR1 packets when FC_DATAUTO0 bit is in manual mode for the packet requested
    //tmp = SPD_TX | VSD_TX | ISCR2_TX | ISCR1_TX;
    //hdmi_reg_write(HDMI_FC_DATMAN, tmp, mdrv->fb0_handle);

    if ((mode->denominator == FULL_HD_DENOMINATOR_NO_FRAC) || 
        (mode->denominator == FULL_HD_DENOMINATOR_WITH_FRAC) ||
        (mode->denominator == HD_DENOMINATOR_NO_FRAC) ||
        (mode->denominator == HD_DENOMINATOR_WITH_FRAC))
    {
        hdmi_reg_write(HDMI_A_TIMER2SETUP0, 0x01, mdrv->fb0_handle);
        hdmi_reg_write(HDMI_A_TIMER2SETUP1, 0x08, mdrv->fb0_handle);
    }

    /* 
    *   This part should not be written to..., it messes up the HDMI certification 
    *   I have blocked it out completely...
    */
#if 0    
    tmp = ENHANCED_LINK_VERIFICATION_MASK |
          BYPASSES_DATA_ENCRYPTION_MASK |
          RX_DETECT_MASK | HDMI_DVI_MASK;
    hdmi_reg_write(HDMI_A_HDCPCFG0, tmp, mdrv->fb0_handle);
#endif
    
    //set the control period minimum duration
    hdmi_reg_write(HDMI_FC_CTRLDUR, HDMI_CONTROL_PERIOD_MINIMUM_DURATION, mdrv->fb0_handle);
    
    /*
    *   The current codes for dfb_config->hdcp_mode are ... :
    *   0   =   always-on
    *   1   =   always-off
    *   2   =   app-set
    */
   /* Print out current PID/thread ... */
   current_pid      =   getpid();
   current_thread   =   pthread_self();
   
   D_DEBUG("PID: 0x%X, Thread: 0x%X\n", current_pid, current_thread);
   
   
    if ( dfb_config->hdcp_mode == 0 ||
        (config->out_connectors == DSOC_HDCP && dfb_config->hdcp_mode == 2)
        )
    {
        /* Re-enable the HDCP clock... */
        data = hdmi_reg_read(HDMI_MC_CLKGAT, mdrv->fb0_handle);
        data&=  (~CLKGATE_HDCP_MASK);
        hdmi_reg_write(HDMI_MC_CLKGAT, data, mdrv->fb0_handle);
        
        
        app_cmd.uIOCTLCommand = CMD_APP_IOCTL_ENABLE_HDCP;
        if (mdrv->fb0_handle != -1)
            ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd); 
        
        D_INFO("HDCP on..\n");
    }
    else if ( (config->out_connectors == DSOC_HDMI && dfb_config->hdcp_mode == 2) ||
              dfb_config->hdcp_mode == 1
            )
    {
        /* Disable the HDCP engine clock if HDCP is not enabled! */
        data = hdmi_reg_read(HDMI_MC_CLKGAT, mdrv->fb0_handle);
        data|=  CLKGATE_HDCP_MASK;
        hdmi_reg_write(HDMI_MC_CLKGAT, data, mdrv->fb0_handle);
        
        app_cmd.uIOCTLCommand = CMD_APP_IOCTL_DISABLE_HDCP;
        if (mdrv->fb0_handle != -1)
            ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd); 
            
        D_INFO("HDCP off..\n");
    }

    /* 
    *   In here we can simply issue a simple pixel packing block reset in order 
    *   to protect against the staircasing issue ... 
    */
      
	
    if (mdrv->fb0_handle != -1)
    {	
        /* Tell FB driver to not pass this event up to application */
        app_cmd.uIOCTLCommand   =   CMD_APP_IOCTL_REFRESH_HDMI;
        ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd);
        
        if (dfb_config->hdcp_mode == 0 ||
            (config->out_connectors == DSOC_HDCP && dfb_config->hdcp_mode == 2)
        )
        {
            app_cmd.uIOCTLCommand   =   CMD_APP_IOCTL_HDCP_AUTHENTICATED;
            data = ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd);
            
            spin_counter = 0;
            while (data == 0)
            {
                data = ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd);
                usleep(250000);
                spin_counter++;
                
                /* Defined in xcode4_osd.h ... */
                if (spin_counter == HDCP_SPIN_WAIT_LIMIT)
                {
                    D_INFO("HDCP Authentication fatal error ... video output may not work\n");
                    break;
                }
                
                
            }
            if (spin_counter != HDCP_SPIN_WAIT_LIMIT) D_INFO("HDCP Authentication completed...\n");

            //Authenticated
            if(data == 1)
            {
                app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_UNBLANK_HDMI;
                if (mdrv->fb0_handle != -1)
                {
                    ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd); 
                }
            }
        }
        else 
        {
            app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_UNBLANK_HDMI;
            if (mdrv->fb0_handle != -1)
            {
                ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd); 
            }
            usleep(50000);  /* Sleep for 50 ms */
        }
    }	
    
    
    return;
}

static void Disp_Set_Clock(pxc_dfb_driver_data mdrv, int denominator)
{
    int gen27_num, gen27_den;
    int gen90_num, gen90_den;
    int pll0;
    int vclk_div, ratio_div;
    int tmp;
    int div;

    switch(denominator)
    {
        case FULL_HD_DENOMINATOR_NO_FRAC: //148.5MHz
            pll0 = HD_CG1_PLL0_FREQ_NO_FRAC;
            vclk_div = 0x404040C0;
            ratio_div = 2;
            gen90_num = FULL_HD_NO_FRAC_GENLOCK_NUM;
            gen90_den = FULL_HD_NO_FRAC_GENLOCK_DEN;
            gen27_num = FULL_HD_PCR_NO_FRAC_NUM;
            gen27_den = FULL_HD_PCR_NO_FRAC_DEN;
            break;
        case FULL_HD_DENOMINATOR_WITH_FRAC: //148.35MHz
            pll0 = HD_CG1_PLL0_FREQ_WITH_FRAC;
            vclk_div = 0x404040C0;
            ratio_div = 2;
            gen90_num = FULL_HD_WITH_FRAC_GENLOCK_NUM;
            gen90_den = FULL_HD_WITH_FRAC_GENLOCK_DEN;
            gen27_num = FULL_HD_PCR_WITH_FRAC_NUM;
            gen27_den = FULL_HD_PCR_WITH_FRAC_DEN;
            break;
        case HD_DENOMINATOR_NO_FRAC: //74.25MHz
            pll0 = HD_CG1_PLL0_FREQ_NO_FRAC;
            vclk_div = 0x808080C0;
            ratio_div = 4;
            gen90_num = HD_NO_FRAC_GENLOCK_NUM;
            gen90_den = HD_NO_FRAC_GENLOCK_DEN;
            gen27_num = HD_PCR_NO_FRAC_NUM;
            gen27_den = HD_PCR_NO_FRAC_DEN;
            break;
        case HD_DENOMINATOR_WITH_FRAC: //74.176MHz
            pll0 = HD_CG1_PLL0_FREQ_WITH_FRAC;
            vclk_div = 0x808080C0;
            ratio_div = 4;
            gen90_num = HD_WITH_FRAC_GENLOCK_NUM;
            gen90_den = HD_WITH_FRAC_GENLOCK_DEN;
            gen27_num = HD_PCR_WITH_FRAC_NUM;
            gen27_den = HD_PCR_WITH_FRAC_DEN;
            break;
        case SD_DENOMINATOR_NO_FRAC: //27MHz
            pll0 = SD_CG1_PLL1_FREQ_NO_FRAC;
            vclk_div = 0xC0C0C0C0;
            ratio_div = 8;
            gen90_num = SD_NO_FRAC_GENLOCK_NUM;
            gen90_den = SD_NO_FRAC_GENLOCK_DEN;
            gen27_num = SD_PCR_NO_FRAC_NUM;
            gen27_den = SD_PCR_NO_FRAC_DEN;
            break;
        case SD_DENOMINATOR_WITH_FRAC: //27.027MHz
            pll0 = SD_CG1_PLL1_FREQ_WITH_FRAC;
            vclk_div = 0xC0C0C0C0;
            ratio_div = 8;
            gen90_num = SD_WITH_FRAC_GENLOCK_NUM;
            gen90_den = SD_WITH_FRAC_GENLOCK_DEN;
            gen27_num = SD_PCR_WITH_FRAC_NUM;
            gen27_den = SD_PCR_WITH_FRAC_DEN;
            break;
        case VGA_DENOMINATOR_25P175MHZ: //25.175MHz
            pll0 = VGA_CG1_PLL1_FREQ_25_175MHZ;
            vclk_div = 0xC0C0C0C0;
            ratio_div = 8;
            gen90_num = VGA_25P175_GENLOCK_NUM;
            gen90_den = VGA_25P175_GENLOCK_DEN;
            gen27_num = VGA_25P175_PCR_NUM;
            gen27_den = VGA_25P175_PCR_DEN;
            break;
        case VGA_DENOMINATOR_25P2MHZ: //25.2MHz
            pll0 = VGA_CG1_PLL1_FREQ_25_2MHZ;
            vclk_div = 0xC0C0C0C0;
            ratio_div = 8;
            gen90_num = VGA_25P2_GENLOCK_NUM;
            gen90_den = VGA_25P2_GENLOCK_DEN;
            gen27_num = VGA_25P2_PCR_NUM;
            gen27_den = VGA_25P2_PCR_DEN;
            break;
        default:
            return;
    }

    DFB_LOCK_CLK_SRC_SEL; 
    RegMaskWrite(CG1_CLK_SRC_SEL4,0x00000000,0xFFFF0000);
    RegMaskWrite(CG1_CLK_SRC_SEL5, vclk_div, 0xFFFFFFFF);
    RegMaskWrite(CG1_CLK_SRC_SEL7,0xC0000000,0xFF000000);
    DFB_UNLOCK_CLK_SRC_SEL; 

    REG_WRITE(CG1_PLL0_FREQ, pll0);
    REG_WRITE(CG1_PLL5_FREQ, pll0);
    tmp = REG_READ(CG1_PLL0_CTRL);
    tmp |= CG1_PLL0_CTRL_FRAC_ENABLE_MASK;
    REG_WRITE(CG1_PLL0_CTRL, tmp);

    REG_WRITE(CG1_MODDIV0_NUM, gen27_num);
    REG_WRITE(CG1_MODDIV0_DEN, gen27_den * ratio_div);
    REG_WRITE(CG1_MODDIV0_CTRL, 2);
    REG_WRITE(CG1_PLL1_FREQ, SD_CG1_PLL1_FREQ_NO_FRAC);
    tmp = REG_READ(CG1_PLL1_CTRL) | (2 << CG1_PLL1_CTRL_REFCLK_SEL_SHIFT);
    REG_WRITE(CG1_PLL1_CTRL, tmp);

    REG_WRITE(DISP_HD_GENLOCK_NUM, gen90_num);
    REG_WRITE(DISP_HD_GENLOCK_DEN, gen90_den);

    REG_WRITE(DISP_HD_GENLOCK_PCR, ((gen27_num << DISP_HD_GENLOCK_PCR_NUM_SHIFT) & DISP_HD_GENLOCK_PCR_NUM_MASK) |
        ((gen27_den << DISP_HD_GENLOCK_PCR_DEN_SHIFT) & DISP_HD_GENLOCK_PCR_DEN_MASK));
}


/**************************************************************************************************/

void Disp_Compositor_Dve_SetMode(pxc_dfb_driver_data mdrv, 
                            const DFBScreenEncoderConfig *config,
                            u32 * disp_comp_register_table)
{

    struct display_mode_struct * mode;
    u32 tmp;
    u32 start_pix_pos;
    u32 interlace = 0;
    u32 field1_position;
    u32 field2_position;
    u32 start_field_position;
    u32 field1_vblank_start;
    u32 field2_vblank_start;
    u32 field1_vblank_end;
    u32 field2_vblank_end;
    u32 field1_vblank_length;
    u32 field2_vblank_length;
    u32 line_a_interrupt;
    u32 line_b_interrupt;
    u32 reg_value;
    static u32 firstflag = 1;

    struct dve_656_reg_struct dve_656_reg;
	/* 90kHz STC NUM/DEN */
	u32	genlock_num;
	u32 genlock_den;
	
	/* 27MHz PCR NUM/DEN */
	u32 genlock_pcr_num;
	u32 genlock_pcr_den;

    //start to set the compositor
    set_hd_comp_enable(mdrv, 0);

#if 0
    DISP_LOCK_CG_RESET_REG;
    RESET_TOGGLE(mdrv, CG_RESET_REG, DISP_RESET_MASK);
    DISP_UNLOCK_CG_RESET_REG;

    //reset the SFT 
    REG_WRITE( DISP_SFT_RESET, 0x0000);

    usleep(1);

    REG_WRITE( DISP_SFT_RESET, 0xffff);

    usleep(1);

    REG_WRITE( DISP_SFT_RESET, 0xffff);

    usleep(1);
#endif
    
    if (disp_comp_register_table[DISP_COMP_SIZE_INDEX] == DISP_HD_COMP_SIZE)
    {
        mode = &(xc4_mode_list[mdrv->shared->mode_index]);
    }
    else
    {
        mode = &(xc4_mode_list[mdrv->sd_mode_index]);
    }

    start_pix_pos = 0;

    if (mode->scanmode == DSESM_INTERLACED && mode->resolution != DSOR_1920_2205)
    {
        interlace = DISP_HD_COMP_SIZE_INTERLACED;

        start_pix_pos = mode->h_total - mode->h_active;

        field1_vblank_length = (mode->v_total - mode->v_active) / 2;
        field2_vblank_length = field1_vblank_length + 1;
        field1_position = (mode->field1_vblank_start + field1_vblank_length) % mode->v_total;
        field2_position = mode->field2_vblank_start + field2_vblank_length;
        start_field_position =  (field2_position << DISP_HD_COMP_START_FLD_POS_FLD2_POS_Y_SHIFT) | field1_position;
    }
    else
    {
        field1_vblank_length = mode->v_total - mode->v_active;
        field2_position = 0;
        start_pix_pos = mode->h_total - mode->h_active;
        
        if (mode->field_length == 0)
        {
            start_field_position = DISP_HD_COMP_START_FLD_POS_FLD2_POS_Y_MASK | 
                               ((mode->field1_vblank_start + field1_vblank_length) % mode->v_total);
        }
        else
        {
            //Compositor counts from 0 -> (global_y_size-1), the register below
            field2_position = (mode->field_length * 2) + field1_vblank_length + (field1_vblank_length + 1);
            field2_position += field1_vblank_length - 1;
            D_DEBUG("field2_position [interlacedFP] = 0x%X\n", field2_position);
            start_field_position = (field2_position << DISP_HD_COMP_START_FLD_POS_FLD2_POS_Y_SHIFT) | 
                               ((mode->field1_vblank_start + field1_vblank_length) % mode->v_total);
        }
    }

    if ((mode->h_active / mode->xres) == 2)
    {
        //for sd, set 2x oversample
        tmp = mode->xres | 
              (mode->v_active << DISP_HD_COMP_SIZE_SIZE_Y_SHIFT) | 
              (DISP_HD_COMP_SIZE_OSR << DISP_HD_COMP_SIZE_OSR_SHIFT) |
              (interlace << DISP_HD_COMP_SIZE_INTERLACED_SHIFT); 
    }
	else if (mode->scanmode == DSESM_INTERLACED &&
             mode->resolution == DSOR_1920_2205)
             
    {
        D_DEBUG("Interlaced frame packing..\n");
        //Different calculation for interlaced frame packing
        //The middle active space seems to be ignored in the active video calculation
        //tmp = mode->xres | 
        reg_value = mode->v_total - mode->v_active;
        tmp  = reg_value;           //Active video space (-.5, simply add +1 to get +.5)
        D_DEBUG("Active video space = 0x%X\n", tmp);
        
        //value to write in height is v_total - 
        tmp = mode->v_total - (2 * tmp);
        D_DEBUG("COMP_VHEIGHT = 0x%X\n", tmp);
        
        tmp = mode->xres | 
              (tmp << DISP_HD_COMP_SIZE_SIZE_Y_SHIFT) |
              (1 << DISP_HD_COMP_SIZE_INTERLACED_SHIFT);
        
    }
    else
    {
        //for hd, no oversampling on Display for other modes (4x)
        tmp = mode->xres | 
              (mode->v_active << DISP_HD_COMP_SIZE_SIZE_Y_SHIFT) | 
              (interlace << DISP_HD_COMP_SIZE_INTERLACED_SHIFT); 
    
    }

    REG_WRITE( disp_comp_register_table[DISP_COMP_SIZE_INDEX], tmp);

    if ((mode->h_active / mode->xres) == 2)
    {
        REG_WRITE( disp_comp_register_table[DISP_COMP_GLOBAL_SIZE_INDEX], 
                   (mode->h_total / 2) | (mode->v_total << DISP_HD_COMP_GLOBAL_SIZE_SIZE_Y_SHIFT));

        start_pix_pos = start_pix_pos /2;
    }
    else
    {
        REG_WRITE( disp_comp_register_table[DISP_COMP_GLOBAL_SIZE_INDEX], 
                   mode->h_total | (mode->v_total << DISP_HD_COMP_GLOBAL_SIZE_SIZE_Y_SHIFT));
    }

    REG_WRITE( disp_comp_register_table[DISP_COMP_START_PIX_POS_INDEX], 
               start_pix_pos);

    REG_WRITE( disp_comp_register_table[DISP_COMP_START_FLD_POS_INDEX], start_field_position);

    
  //no vbi and ancillary data
    REG_WRITE( disp_comp_register_table[DISP_COMP_START_VBI_POS_INDEX], 0xffffff);
    REG_WRITE( disp_comp_register_table[DISP_COMP_STOP_VBI_POS_INDEX], 0xffffff);
    REG_WRITE( disp_comp_register_table[DISP_COMP_START_HANC_POS_INDEX], 0x0);
    REG_WRITE( disp_comp_register_table[DISP_COMP_STOP_HANC_POS_INDEX], 0x0);
    
    REG_WRITE(disp_comp_register_table[DISP_OUTPUT_LIMIT_INDEX], 0x01FE01FE);

  // vbi is enabled
    if (firstflag)
    {
        REG_WRITE( disp_comp_register_table[DISP_VBI0_HANC0_FORMAT_INDEX], 0x8080800a);
        firstflag = 0;
    }else{
        reg_value = REG_READ(disp_comp_register_table[DISP_VBI0_HANC0_FORMAT_INDEX]) & DISP_HD_VBI0_HANC0_FORMAT_VBI_PIPE_DISABLE_MASK;
        reg_value |= 0x80800a;
        REG_WRITE( disp_comp_register_table[DISP_VBI0_HANC0_FORMAT_INDEX], reg_value);
    }   
 
    //set DVE output registers

    //try to set DVE 656 2 output
    dve_656_reg.dve_field_bndry_reg = disp_comp_register_table[DISP_VENC0_DVE_FIELD_BNDRY_INDEX];
    dve_656_reg.dve_vblank_start_bndry_reg = disp_comp_register_table[DISP_VENC0_DVE_VBLANK_START_BNDRY_INDEX];
    dve_656_reg.dve_vblank_end_bndry_reg = disp_comp_register_table[DISP_VENC0_DVE_VBLANK_END_BNDRY_INDEX];
    dve_656_reg.dve_vsync_start_reg = disp_comp_register_table[DISP_VENC0_DVE_VSYNC_START_INDEX];
    dve_656_reg.dve_vsync_end_reg = disp_comp_register_table[DISP_VENC0_DVE_VSYNC_END_INDEX];
    dve_656_reg.dve_hsync_reg = disp_comp_register_table[DISP_VENC0_DVE_HSYNC_INDEX];
    dve_656_reg.dve_cntrl_reg = disp_comp_register_table[DISP_VENC0_DVE_CNTRL_INDEX];
    
    /* 
    *   TODO: Initialize this to a non-zero value so it does not go to the 656_2 output automatically
    *   However, this will need to be cleaned up and changed...
    */
    mdrv->output    =   100;

    Xcode4_Set_DVE_656_Output(mdrv, mode, &dve_656_reg);

    tmp = REG_READ(DISP_OUTPUT_MUX);
    if (disp_comp_register_table[DISP_COMP_SIZE_INDEX] == DISP_HD_COMP_SIZE)
    {
        tmp = tmp & (~O656_SD_MASK);
    }
    else if (disp_comp_register_table[DISP_COMP_SIZE_INDEX] == DISP_SD_COMP_SIZE)
    {
        tmp = tmp | O656_SD_MASK;
    }
    REG_WRITE(DISP_OUTPUT_MUX, tmp);

    tmp = REG_READ(DISP_SIMULATION);
    tmp = tmp | (COEF_AUTO_GENERATE << COEF_AUTO_GENERATE_SHIFT);
    REG_WRITE (DISP_SIMULATION, tmp);


    DISP_LOCK_CG_CLOCK_STOP;
    
    //REG_WRITE(CG1_CLK_SRC_EN0,0xffffffff);
    //REG_WRITE(CG1_CLK_STOP0,0);
    //REG_WRITE(CG1_BLK_CLK_STOP0,0);
    
    DISP_UNLOCK_CG_CLOCK_STOP;
    
    /* SDK Release ... */
    
    RegMaskWrite(CG1_CLK_SRC_SEL7,CG1_PLL4_DIV_5, VSCLK_SRC_SEL_MASK);
//    RegMaskWrite(RBM_PADU_CTRL, VO_SELECT, PADU_CTRL_HDMI_VO_MASK);
	
	/* 
		Add code here to set up the NUM/DENOM for the Genlocking 
		The clock needs to be brought down to 90kHz
	*/

    D_DEBUG("mode->denominator = 0x%x\n", mode->denominator);

    Disp_Set_Clock(mdrv, mode->denominator);
    
    if (mode->scanmode == DSESM_PROGRESSIVE)
    {
        if (mode->field1_vblank_start > PRESET_LINES)
        {
            line_a_interrupt = mode->field1_vblank_start - PRESET_LINES;
            line_b_interrupt = line_a_interrupt + 1;
        }
        else
        {
            line_a_interrupt = mode->v_total - PRESET_LINES;
            line_b_interrupt = line_a_interrupt + 1;
        }

        tmp = (line_a_interrupt & DISP_HD_LINE_INTERRUPT_A_MASK) | 
              ((line_b_interrupt << DISP_HD_LINE_INTERRUPT_B_SHIFT) & DISP_HD_LINE_INTERRUPT_B_MASK);

        REG_WRITE(DISP_HD_LINE_INTERRUPT, tmp);
    }
    else
    {
        tmp = (mode->field1_vblank_start & DISP_HD_LINE_INTERRUPT_A_MASK) | 
              ((mode->field2_vblank_start << DISP_HD_LINE_INTERRUPT_B_SHIFT) & DISP_HD_LINE_INTERRUPT_B_MASK);
        REG_WRITE(DISP_HD_LINE_INTERRUPT, tmp);
    }

    if ((mode->scanmode == DSESM_PROGRESSIVE) &&
        (disp_comp_register_table[DISP_COMP_GLOBAL_SIZE_INDEX] == DISP_HD_COMP_GLOBAL_SIZE))
    {
        REG_WRITE(DISP_HD_VID1_VSCA_START, 0x304);
        REG_WRITE(DISP_HD_VID0_VSCA_START, 0x304);
    }
    else if ((mode->scanmode == DSESM_INTERLACED) &&
        (disp_comp_register_table[DISP_COMP_GLOBAL_SIZE_INDEX] == DISP_HD_COMP_GLOBAL_SIZE))
    {
        REG_WRITE(DISP_HD_VID1_VSCA_START, 0x34443334);
        REG_WRITE(DISP_HD_VID0_VSCA_START, 0x34443334);
    }

    set_hd_comp_enable(mdrv, 1);

    return ;
}

/**************************************************************************************************/

static DFBResult Xcode4SetEncoderConfig ( CoreScreen                   *screen,
                                          void                         *driver_data,
                                          void                         *screen_data,
                                          int                           encoder,
                                          const DFBScreenEncoderConfig *config )
{
    DFBResult result = DFB_UNSUPPORTED;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    u32 tmp;
    u32 bg_color_y, bg_color_cb, bg_color_cr;
    DFBConfigLayer *conf  = &dfb_config->layers[0];
    struct display_mode_struct * mode = NULL;
    app_ioctl_command app_cmd;

    D_DEBUG( "Xcode4SetEncoderConfig enter \n");
    D_DEBUG( "encoder tv_standard 0x%x\n", config->tv_standard);
    D_DEBUG( "encoder out_signals  0x%x \n", config->out_signals );
    D_DEBUG( "encoder out_connectors 0x%x  \n", config->out_connectors );
    D_DEBUG( "encoder slow_blanking  0x%x  \n", config->slow_blanking  );
    D_DEBUG( "encoder scanmode  0x%x \n", config->scanmode   );
    D_DEBUG( "encoder frequency  0x%x \n", config->frequency    );
    D_DEBUG( "encoder resolution 0x%x \n", config->resolution  );

#ifdef FORMAL_DFB_DRIVER
    if (mdrv->shared->mode_index != INDEX_UNKNOWN)
    {
        /* Based on the currently set video mode, set the color space (DSCS_709 or DSCS_601) */
        mode = xc4_get_mode(mdrv->shared->mode_index);
        if (mode)
        {
            DFBColorSpace screen_space;
            if (mode->denominator   ==  SD_DENOMINATOR_NO_FRAC ||
                mode->denominator   ==  SD_DENOMINATOR_WITH_FRAC ||
                mode->denominator   ==  VGA_DENOMINATOR_25P2MHZ ||
                mode->denominator   ==  VGA_DENOMINATOR_25P175MHZ
               )
            {
                /* SD resolutions output DSCS_601 */
                screen_space  =   DSCS_601;
            }
            else
            {
                /* HD resolutions output DSCS_709 */
                screen_space  =   DSCS_709;
            }
            if(screen_space != mdrv->shared->current_hd_color_space)
            {
                mdrv->shared->current_hd_color_space = screen_space;
                
                CSC_ProgAdjustMatrix(mdrv, XCODE4_GRA0_LAYER_ID);
                CSC_ProgAdjustMatrix(mdrv, XCODE4_GRA1_LAYER_ID);
                CSC_ProgAdjustMatrix(mdrv, XCODE4_VID0_LAYER_ID);
                CSC_ProgAdjustMatrix(mdrv, XCODE4_VID1_LAYER_ID);
            }
        }
    }
    
    if ((mdrv->shared->mode_index != INDEX_UNKNOWN) &&
        (mdrv->shared->mode_index != mdrv->shared->last_mode_index))
    //if (1)
    {
        
        //app_cmd.uIOCTLCommand = CMD_APP_IOCTL_DISABLE_HDCP;
        //if (mdrv->fb0_handle != -1)
        //{
        //    ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd); 
        //}
        mdrv->shared->last_mode_index = mdrv->shared->mode_index;
        mdrv->shared->last_mode_index_out[DISP_HD_DVE656_2_OUTPUT] =   INDEX_UNKNOWN;
        mdrv->shared->last_mode_index_out[DISP_HD_HDMI_OUTPUT] = INDEX_UNKNOWN;
        mdrv->shared->last_mode_index_out[DISP_HD_COMPONENT_OUTPUT] = INDEX_UNKNOWN;
        
        
        D_DEBUG("Encoder modeindex: 0x%X\n", mdrv->shared->mode_index);
        app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_BLANK_HDMI;
        if (mdrv->fb0_handle != -1)
        {
            ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd); 
        }
        usleep(50000);  /* Sleep for 50 ms to ensure HDMI is OK ... */
        
        /* Unblank will happen OK either (when HDCP is finally done, or end of mode change ... )*/
        result = DFB_OK;

        Disp_Compositor_Dve_SetMode(mdrv, config, disp_hd_comp_register_table);

        mdrv->shared->current_hd_resolution = config->resolution;
        mdrv->shared->current_hd_scanmode   = config->scanmode;

        //set screen background color
        tmp = REG_READ(DISP_HD_COMP_ALPHA_MODE);
        tmp = tmp & 
              (~DISP_HD_COMP_ALPHA_MODE_BACKGROUND_Y_MASK) &
              (~DISP_HD_COMP_ALPHA_MODE_BACKGROUND_CB_MASK) &
              (~DISP_HD_COMP_ALPHA_MODE_BACKGROUND_CR_MASK);
    
        RGB_TO_YCBCR(conf->background.color.r, 
                     conf->background.color.g,
                     conf->background.color.b,
                     bg_color_y,
                     bg_color_cb,
                     bg_color_cr);
    
        bg_color_y = bg_color_y & 0x0ff;
        bg_color_cb = bg_color_cb & 0x0ff;
        bg_color_cr = bg_color_cr & 0x0ff;
    
        tmp = tmp | 
              (bg_color_y << DISP_HD_COMP_ALPHA_MODE_BACKGROUND_Y_SHIFT) |
              (bg_color_cb << DISP_HD_COMP_ALPHA_MODE_BACKGROUND_CB_SHIFT) |
              (bg_color_cr << DISP_HD_COMP_ALPHA_MODE_BACKGROUND_CR_SHIFT);
    
        REG_WRITE(DISP_HD_COMP_ALPHA_MODE, tmp);

        //make the sd has the same background as hd
        tmp = REG_READ(DISP_SD_COMP_ALPHA_MODE);

        tmp = tmp & (~DISP_SD_COMP_ALPHA_MODE_BACKGROUND_Y_MASK);
        tmp = tmp & (~DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CB_MASK);
        tmp = tmp & (~DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CR_MASK);

        tmp = tmp | 
              (bg_color_y << DISP_SD_COMP_ALPHA_MODE_BACKGROUND_Y_SHIFT) |
              (bg_color_cb << DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CB_SHIFT) |
              (bg_color_cr << DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CR_SHIFT);

        REG_WRITE(DISP_SD_COMP_ALPHA_MODE, tmp);

        //update SD parameter
        if (mdrv->hdtosd_fd != -1)
        {
            if ((config->resolution == DSOR_1920_2205) ||
                (config->resolution == DSOR_1280_1470))
            {
                app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_DISABLE_HD2SD;
                ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
                ViperSDDACPowerDown(mdrv);
            }
            else
            {
                app_cmd.uIOCTLCommand = CMD_APP_IOCTL_GET_SD_SCREEN_MODE;
                ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
                
                app_cmd.uIOCTLCommand = CMD_APP_IOCTL_SET_SD_SCREEN_MODE;
                app_cmd.dfb_sd_screen_mode.width = mode->xres;
                app_cmd.dfb_sd_screen_mode.height = mode->v_active;
                app_cmd.dfb_sd_screen_mode.interlaced = (mode->scanmode == DSESM_INTERLACED);
                app_cmd.dfb_sd_screen_mode.double_frame = ((mode->frequency & (DSEF_25HZ | DSEF_29_97HZ | DSEF_30HZ | DSEF_24HZ | DSEF_23_976HZ)) != 0);
                ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
                
                mdrv->shared->sd_frame_entries.double_frame = app_cmd.dfb_sd_screen_mode.double_frame;

                if ((mdrv->shared->current_sd_power_mode == DSPM_ON) &&
                    (REG_READ(DISP_SD_GRA0_FORMAT) & DISP_SD_GRA0_FORMAT_PIPE_DISABLE_MASK))
                {
                    app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_ENABLE_HD2SD;
                    ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
                    ViperSDDACPowerUp(mdrv);
                }
            }
        }
    }
    else if ((mdrv->shared->mode_index != INDEX_UNKNOWN) &&
             (mdrv->shared->mode_index == mdrv->shared->last_mode_index))
    {
        result = DFB_OK;
    }
#else
    
    result = DFB_OK;

#endif

    
    D_DEBUG( "Xcode4SetEncoderConfig exit \n");
    return result;

}

/**************************************************************************************************/

static DFBResult
Xcode4TestOutputConfig( CoreScreen                  *screen,
                        void                        *driver_data,
                        void                        *screen_data,
                        int                          output,
                        const DFBScreenOutputConfig *config,
                        DFBScreenOutputConfigFlags  *failed )
{

    DFBResult result = DFB_UNSUPPORTED;
    struct display_mode_struct * mode = NULL;

    D_DEBUG( "Xcode4TestOutputConfig enter \n");
    D_DEBUG( "config->resolution 0x%x  \n", config->resolution);
    D_DEBUG( "config->slow_blanking 0x%x  \n", config->slow_blanking);

    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    if (mdrv->shared->mode_index != INDEX_UNKNOWN)
    {
        mode = xc4_get_mode(mdrv->shared->mode_index);
        
        /* DSOR_1920_1080 modes and DSOR_1920_1080_3D_SIDE_BY_SIDE need to be interchangeable... */
        if ((mode) ||
            ((mode->resolution == config->resolution)  ||
             ((mode->resolution == DSOR_1920_1080) && (config->resolution == DSOR_1920_1080_3D_SIDE_BY_SIDE))) && 
            (mode->slow_blanking == config->slow_blanking))
        {
            result = DFB_OK;
        }
    }

    D_DEBUG( "Xcode4TestOutputConfig exit \n");
    return result;
}

/**************************************************************************************************/

static DFBResult 
Xcode4SetOutputConfig ( CoreScreen                  *screen,
                        void                        *driver_data,
                        void                        *screen_data,
                        int                          output,
                        const DFBScreenOutputConfig *config )
{
    
    D_DEBUG( "Xcode4SetOutputConfig enter \n");

    DFBResult result = DFB_UNSUPPORTED;
    struct display_mode_struct * mode = NULL;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    struct dve_656_reg_struct dve_656_reg;
    u32 tmp, timeout, tmp_status;
    DFBScreenOutputConfig interlaced_config;
    app_ioctl_command app_cmd;
    int ret;
    
    D_DEBUG( "Xcode4SetOutputConfig enter \n");
    D_DEBUG( "output flags 0x%x\n", config->flags);
    D_DEBUG( "output encoder  0x%x \n", config->encoder );
    D_DEBUG( "output out_signals 0x%x  \n", config->out_signals );
    
    D_DEBUG( "output connectors  0x%x \n", config->out_connectors );
    D_DEBUG( "output resolution 0x%x \n", config->resolution );
    
    
    interlaced_config.flags                   = config->flags;
    interlaced_config.out_signals         = config->out_signals;
    interlaced_config.out_connectors  = config->out_connectors;
    interlaced_config.slow_blanking     = config->slow_blanking;  
    interlaced_config.resolution           = config->resolution;
    interlaced_config.output_order       = config->output_order;
    interlaced_config.bus_type             = config->bus_type;

    if ((mdrv->shared->mode_index != INDEX_UNKNOWN) &&
        ((config->out_signals != mdrv->shared->last_out_signals[output]) ||
         (mdrv->shared->mode_index != mdrv->shared->last_mode_index_out[output]) ||
         (output != DISP_HD_HDMI_OUTPUT) ||
         (mdrv->shared->last_output_init != output)  ))
    //if (mdrv->shared->mode_index != INDEX_UNKNOWN)
    {
        mode = xc4_get_mode(mdrv->shared->mode_index);

        mdrv->shared->last_out_signals[output] = config->out_signals;
        mdrv->shared->last_mode_index_out[output] = mdrv->shared->mode_index;
        D_DEBUG("Port: 0x%X, modeindex: 0x%X, signal: 0x%X\n", output, mdrv->shared->mode_index, config->out_signals);
		/*
		*	This code sets the mux to output video out,
		*	which conflicts with the HPD signal of the internal
		*	HDMI PHY.
		*	
		*	If an external output device is enabled, we must mask
		*	the internal HPD interrupt signal.
		*	
		*	To do this, read PHY_MASK0 (0x3006), or it with 0x2 and then
		*	write it back.
		*	
		*	In addition, temporarily disable all display interrupts
		*	to prevent locking
		*/
		//tmp = hdmi_reg_read(HDMI_PHY_MASK0, mdrv->fb0_handle);
		
        
        /* This needs to be done all the time!!! */
        mdrv->output = output;
        //return DFB_OK;

#ifndef LG_BDR_4210
        if (output == DISP_HD_DVE656_2_OUTPUT)
        {
            //try to set DVE 656 2 output
            dve_656_reg.dve_field_bndry_reg = DISP_HD_VENC0_6562_FIELD_BNDRY;
            dve_656_reg.dve_vblank_start_bndry_reg = DISP_HD_VENC0_6562_VBLANK_START_BNDRY;
            dve_656_reg.dve_vblank_end_bndry_reg = DISP_HD_VENC0_6562_VBLANK_END_BNDRY;
            dve_656_reg.dve_vsync_start_reg = DISP_HD_VENC0_6562_VSYNC_START;
            dve_656_reg.dve_vsync_end_reg = DISP_HD_VENC0_6562_VSYNC_END;
            dve_656_reg.dve_hsync_reg = DISP_HD_VENC0_6562_HSYNC;
            dve_656_reg.dve_cntrl_reg = DISP_HD_VENC0_6562_CNTRL;
            Xcode4_Set_DVE_656_Output(mdrv, mode, &dve_656_reg);
			

            if ((config->flags & DSOCONF_BUS_TYPE) &&
                (config->flags & DSOCONF_OUTPUT_ORDER))
            {
            	    if ( mode->scanmode == DSESM_INTERLACED )
            	    {	    
            	    	    if (mode->resolution == DSOR_1920_1080)
            	    	    {
            	    	    	   interlaced_config.bus_type = DSBT_DUAL;
            	    	    	   interlaced_config.output_order = DSSO_CBYCR;
            	    	    	   interlaced_config.out_signals = DSOS_656_10;
            	    	    	   Xcode4_Set_HD_DVE_656_2_Output_Order(mdrv, &interlaced_config);
            	    	    }
            	    	    else if ((mode->resolution == DSOR_720_480) || (mode->resolution == DSOR_720_576) )
                            {
                            	   interlaced_config.bus_type = DSBT_SINGLE;
            	    	    	   interlaced_config.output_order = DSSO_CBYCR;
            	    	    	   interlaced_config.out_signals = DSOS_656_10; 
            	    	    	   Xcode4_Set_HD_DVE_656_2_Output_Order(mdrv, &interlaced_config);
                            } 
                            else 
                            {
                            	    Xcode4_Set_HD_DVE_656_2_Output_Order(mdrv, config);
                            }	    
            	    }
            	    else 
            	    {
            	             Xcode4_Set_HD_DVE_656_2_Output_Order(mdrv, config);
            	    }
            }

			/* Disable the interrupt */
			/* Change PADU to mux out video signals (for video output) */
            
			D_INFO("ViXS Viper set 656 mode[%u]: resolution 0x%x, scanmode 0x%x, slowblanking 0x%x, frequency 0x%x, out signal 0x%x, out connectors 0x%x \n", 
                output,
                mode->resolution, mode->scanmode, 
                mode->slow_blanking, mode->frequency, 
                config->out_signals,
                config->out_connectors);
			
            if (config->out_signals != DSOS_NONE)
            {
                if(config->out_connectors == DSOC_CVBS)
                {
                    struct display_mode_struct * mode = xc4_get_mode(mdrv->shared->mode_index);
                    // DISP_LOCK_CG_CLOCK_STOP;
                    // tmp =   REG_READ(CG1_CLK_STOP0);
                    // tmp &= (~DISP_656CLK_STOP_MASK);
                    // REG_WRITE(CG1_CLK_STOP0, tmp);
                    // D_INFO("Starting 656 clock ...\n");
                    // DISP_UNLOCK_CG_CLOCK_STOP;
                    Clk656On(mdrv);
                    //Special output format for CVBS
                    if(mode->xres == 720 && mode->scanmode == DSESM_INTERLACED)
                    {
                        REG_WRITE(DISP_HD_VENC0_6562_CNTRL , 0x20);
                    }
                    else
                    {
                        REG_WRITE(DISP_HD_VENC0_6562_CNTRL , 0x32);
                    }

                    //Set to loop mode
                    if (mdrv->hdtosd_fd != -1)
                    {                    
                        app_ioctl_command app_cmd;
                        app_cmd.uIOCTLCommand = CMD_APP_IOCTL_GET_SD_SCREEN_MODE;
                        ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
                        
                        app_cmd.uIOCTLCommand = CMD_APP_IOCTL_SET_SD_SCREEN_MODE;
                        app_cmd.dfb_sd_screen_mode.loop = 1;
                        app_cmd.dfb_sd_screen_mode.width = mode->xres;
                        app_cmd.dfb_sd_screen_mode.height = mode->v_active;
                        app_cmd.dfb_sd_screen_mode.interlaced = (mode->scanmode == DSESM_INTERLACED);
                        ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
                    }
                }
                else
                {
                    if (mdrv->hdtosd_fd != -1)
                    {
                        app_ioctl_command app_cmd;
                        app_cmd.uIOCTLCommand = CMD_APP_IOCTL_GET_SD_SCREEN_MODE;
                        ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 

                        app_cmd.uIOCTLCommand = CMD_APP_IOCTL_SET_SD_SCREEN_MODE;
                        app_cmd.dfb_sd_screen_mode.loop = 0;
                        ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
                    }
                    /* Enable Video out only when external syncs are required ... */
                    
                    if (config->out_signals == DSOS_656 ||
                        config->out_signals == DSOS_656_10 ||
                        config->out_signals == DSOS_656_12
                        )
                    {
                        /* External sync */
                        hdmi_reg_write(HDMI_PHY_MASK0, 0xF3, mdrv->fb0_handle);
                        RegMaskWrite(RBM_PADU_CTRL, VO_SELECT, PADU_CTRL_HDMI_VO_MASK);
                    }
                    else
                    {
                        /* Embedded sync */
                        RegMaskWrite(RBM_PADU_CTRL, INT_HDMI_SELECT, PADU_CTRL_HDMI_VO_MASK);
                        hdmi_reg_write(HDMI_PHY_MASK0, 0xF1, mdrv->fb0_handle);
                    }
                    // DISP_LOCK_CG_CLOCK_STOP;
                    // tmp =   REG_READ(CG1_CLK_STOP0);
                    // tmp &= (~DISP_656CLK_STOP_MASK);
                    // REG_WRITE(CG1_CLK_STOP0, tmp);
                    // D_INFO("Starting 656 clock ...\n");
                    // DISP_UNLOCK_CG_CLOCK_STOP;
                    Clk656On(mdrv);
                    
                    
                    
    #ifdef     ENABLE_ADV7511_OUTPUT
                    ret = ADV7511_SetMode(mode);
    #endif

                }
			}
            else
            {
                /* Shut off 656 clock, no output ... */
                // DISP_LOCK_CG_CLOCK_STOP;
                // tmp =   REG_READ(CG1_CLK_STOP0);
                // tmp |= (DISP_656CLK_STOP_MASK);
                // REG_WRITE(CG1_CLK_STOP0, tmp);
                // D_INFO("Stopping 656 clock ...\n");
                // DISP_UNLOCK_CG_CLOCK_STOP;
                Clk656Off(mdrv);
                //hdmi_reg_write(HDMI_PHY_MASK0, 0xF1, mdrv->fb0_handle);
                RegMaskWrite(RBM_PADU_CTRL, INT_HDMI_SELECT, PADU_CTRL_HDMI_VO_MASK);

                if (mdrv->hdtosd_fd != -1)
                {
                    app_ioctl_command app_cmd;
                    app_cmd.uIOCTLCommand = CMD_APP_IOCTL_GET_SD_SCREEN_MODE;
                    ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 

                    app_cmd.uIOCTLCommand = CMD_APP_IOCTL_SET_SD_SCREEN_MODE;
                    app_cmd.dfb_sd_screen_mode.loop = 0;
                    ioctl(mdrv->hdtosd_fd, CMD_APPLICATION_IOCTL, &app_cmd); 
                }
            }
			result = DFB_OK;
            
        }
        else 
#endif
        if (output == DISP_HD_HDMI_OUTPUT)
        {
            app_cmd.uIOCTLCommand = CMD_DFB_IOCTL_BLANK_HDMI;
            if (mdrv->fb0_handle != -1)
            {
                ioctl(mdrv->fb0_handle, CMD_APPLICATION_IOCTL, &app_cmd); 
            }
            usleep(50000);  /* Sleep for 50 ms to ensure HDMI is OK ... */

            D_INFO("ViXS Viper set hdmi mode[%u]: resolution 0x%x, scanmode 0x%x, slowblanking 0x%x, frequency 0x%x, out signal 0x%x \n",
                output,
                mode->resolution, mode->scanmode, 
                mode->slow_blanking, mode->frequency, 
                config->out_signals);
            
            /* Only perform HDMI initialization on DSOS_NONE ... */
            if (config->out_signals != DSOS_NONE)
            {
                HDMIOn(mdrv);
                
                Disp_Set_656_HDMI_Encoder(mdrv, mode, config);
                
                /* Disable video output over external bus */
                RegMaskWrite(RBM_PADU_CTRL, INT_HDMI_SELECT, PADU_CTRL_HDMI_VO_MASK);
                usleep(10000);
			
                /* Enable the HPD interrupt */
                hdmi_reg_write(HDMI_PHY_MASK0, 0xF1, mdrv->fb0_handle);
                usleep(10000);
                D_INFO("**** HDMI ON\n");
            }
            else
            {
                //set the hdmi power down
                //hdmi_reg_write(HDMI_PHY_CONF0, DISP_HDMI_POWER_DOWN, mdrv->fb0_handle);
                HDMIOff(mdrv);
                usleep(100000);
                //test clear signal
                //hdmi_reg_write(HDMI_PHY_TST0, 0x20, mdrv->fb0_handle);
                //hdmi_reg_write(HDMI_PHY_TST0, 0x00, mdrv->fb0_handle);
                D_INFO("**** HDMI OFF\n");
            }
            
         	result	=   DFB_OK;
        }
#ifndef LG_BDR_4210
        else if (output == DISP_HD_COMPONENT_OUTPUT)
        {
            
            if (config->out_signals == DSOS_NONE )
            {
                //power the component interface
                //REG_WRITE(DISP_VDAC01_CNTL, 0);
                ViperHDDACPowerDown(mdrv);
                D_INFO("**** COMPONENT OFF\n");
            }
            else
            {
                Xcode4_SetUpComponentVenc(mdrv, mode, config);
                ViperHDDACPowerUp(mdrv);
                D_INFO("**** COMPONENT ON\n");
            }
            result = DFB_OK;
        }
#endif        
        /* Set the last initialized output ... */
        mdrv->shared->last_output_init  =   output; 
    }
    
    
    D_DEBUG( "Xcode4SetOutputConfig exit \n");

    return DFB_OK;

}


/**************************************************************************************************/

static DFBResult
Xcode4InitEncoder( CoreScreen                  *screen,
                  void                        *driver_data,
                  void                        *screen_data,
                  int                          encoder,
                  DFBScreenEncoderDescription *description,
                  DFBScreenEncoderConfig      *config )
{
    DFBResult result = DFB_UNSUPPORTED;
    app_ioctl_command dfb_ioctl_cmd;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    DFBConfigLayer *conf;
    int i;

    D_DEBUG( "Xcode4InitEncoder enter \n");

    //set the initialize display mode as CEA-861-D 
    config->flags = DSECONF_TV_STANDARD |
                    DSECONF_OUT_SIGNALS |
                    DSECONF_SCANMODE |
                    DSECONF_ADJUSTMENT |
                    DSECONF_FREQUENCY |
                    DSECONF_CONNECTORS  |
                    DSECONF_SLOW_BLANKING |
                    DSECONF_RESOLUTION ;

    config->tv_standard = DSETV_NTSC;
    config->out_signals = DSOS_YCBCR;
    config->out_connectors = DSOC_COMPONENT;

    if ((dfb_config->mode.width == 0) &&
        (dfb_config->mode.height == 0) &&
        (dfb_config->layers[XCODE4_VID1_LAYER_ID].init == true))
    {
        dfb_config->mode.width = 1920;
        dfb_config->mode.height = 1080;

        conf = &dfb_config->layers[XCODE4_VID1_LAYER_ID];
        conf->config.width  = DEFAULT_VID1_BUFFER_WIDTH;
        conf->config.height = DEFAULT_VID1_BUFFER_HEIGHT;
    }

    for (i = 0; i < D_ARRAY_SIZE(dfb_config->layers); i ++)
    {
        dfb_config->layers[i].background.color.a     = 0;
        dfb_config->layers[i].background.color.r     = 0;
        dfb_config->layers[i].background.color.g     = 0;
        dfb_config->layers[i].background.color.b     = 0;
        dfb_config->layers[i].background.color_index = -1;
        dfb_config->layers[i].background.mode        = DLBM_COLOR;
    }

    if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 480))
    {
        config->slow_blanking = DSOSB_4x3;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_720_480;
    }
    if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 481))
    {
        config->slow_blanking = DSOSB_16x9;
        config->scanmode = DSESM_PROGRESSIVE;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_720_480;
    }

    else if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 576))
    {
        config->slow_blanking = DSOSB_4x3;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_50HZ;
        config->resolution = DSOR_720_576;
    }
    else if ((dfb_config->mode.width == 1280) && (dfb_config->mode.height == 720))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_PROGRESSIVE;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_1280_720;
    }
    else if ((dfb_config->mode.width == 1280) && (dfb_config->mode.height == 721))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_PROGRESSIVE;
        config->frequency = DSEF_50HZ;
        config->resolution = DSOR_1280_720;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1080))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1081))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_INTERLACED;
        config->frequency = DSEF_50HZ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1082))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_PROGRESSIVE ;
        config->frequency = DSEF_60HZ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1083))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_PROGRESSIVE ;
        config->frequency = DSEF_50HZ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 2205))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->scanmode = DSESM_PROGRESSIVE ;
        config->frequency = DSEF_24HZ;
        config->resolution = DSOR_1920_2205;
    }
    else
    {
        D_DEBUG( "Xcode4InitEncoder unsupport display mode \n");
    }

    mdrv->shared->mode_index = xc_find_mode_index(config);

    if (mdrv->shared->mode_index != INDEX_UNKNOWN)
    {
        result = DFB_OK;
    }

    Init_Disp_Hd_Comp_Register();


    if (mdrv->fb0_handle != -1)
	{
        //
        //Inform FB driver to  layer region reset
        // 
        dfb_ioctl_cmd.uSize = sizeof(app_ioctl_command);
        dfb_ioctl_cmd.uIOCTLCommand = CMD_DFB_IOCTL_SET_DFB_VIDEO_START_PHYS_ADDR;
        if (dfb_config->extra_video_phys)
        {
            dfb_ioctl_cmd.video_buffer_info.dfb_start_phys_addr = dfb_config->extra_video_phys;
        }
        else
        {
            dfb_ioctl_cmd.video_buffer_info.dfb_start_phys_addr = dfb_config->video_phys;
        }
	    
        if (ioctl( mdrv->fb0_handle,CMD_APPLICATION_IOCTL , &dfb_ioctl_cmd ) )
        {
             D_ERROR("failed to inform FB driver of CMD_DFB_IOCTL_SET_DFB_VIDEO_START_PHYS_ADDR \n");
        }
	}

    D_DEBUG( "Xcode4InitEncoder exit \n");
    return result;
}

/**************************************************************************************************/

static DFBResult
Xcode4InitOutput( CoreScreen                 *screen,
                 void                       *driver_data,
                 void                       *screen_data,
                 int                         output,
                 DFBScreenOutputDescription *description,
                 DFBScreenOutputConfig      *config )
{
    D_DEBUG( "Xcode4InitOutput enter \n");

    DFBResult result = DFB_OK;
    /* Set the output capabilities. */
    description->caps = DSOCAPS_ALL;

    /* Set supported output connectors and signals. */
    description->all_connectors = DSOC_VGA | 
                                  DSOC_SCART |   
                                  DSOC_YC |  
                                  DSOC_CVBS |
                                  DSOC_SCART2 |   
                                  DSOC_COMPONENT |
                                  DSOC_HDMI;

    description->all_signals  = DSOS_YC |
    							DSOS_CVBS |
    							DSOS_YCBCR |
    							DSOS_HDMI |
                                DSOS_656 ;
                          

    description->all_resolutions = DSOR_ALL;



    /* Set default configuration. */
    config->flags = DSOCONF_ALL;
    config->signal_degrade  =   DSD_FULLRESOLUTION;

    config->out_signals = DSOS_NONE;
    
    
    if (output == DISP_HD_HDMI_OUTPUT)
    {
        config->out_connectors = DSOC_HDMI;
    }
    else 
    {
        config->out_connectors = DSOC_COMPONENT;
    }

    if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 480))
    {
        config->slow_blanking = DSOSB_4x3;
        config->resolution = DSOR_720_480;
    }
    else if ((dfb_config->mode.width == 720) && (dfb_config->mode.height == 576))
    {
        config->slow_blanking = DSOSB_4x3;
        config->resolution = DSOR_720_576;
    }
    else if ((dfb_config->mode.width == 1280) && (dfb_config->mode.height == 720))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1280_720;
    }
    else if ((dfb_config->mode.width == 1280) && (dfb_config->mode.height == 721))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1280_720;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1080))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1081))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1082))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1920_1080;
    }
    else if ((dfb_config->mode.width == 1920) && (dfb_config->mode.height == 1083))
    {
        config->slow_blanking = DSOSB_16x9 ;
        config->resolution = DSOR_1920_1080;
    }
    else
    {
        D_DEBUG( "Xcode4InitOutput unsupport output mode \n");
        result = DFB_UNSUPPORTED;
    }

    config->bus_type = DSBT_TRIPLE;
    config->output_order = DSSO_CBYCR;

    //Disable analog output by default
    Clk656Off((pxc_dfb_driver_data) driver_data);
    ViperHDDACPowerDown((pxc_dfb_driver_data) driver_data);

    D_DEBUG( "Xcode4InitOutput exit \n");

    return result;
}

static DFBResult
Xcode4GetScreenSize( CoreScreen *screen,
                     void       *driver_data,
                     void       *screen_data,
                     int        *ret_width,
                     int        *ret_height )
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    struct display_mode_struct * mode = NULL;
    DFBConfigLayer *conf;

    D_DEBUG( "Xcode4GetScreenSize enter \n");

    D_ASSERT( ret_width != NULL );
    D_ASSERT( ret_height != NULL );

    if (mdrv->shared->mode_index != INDEX_UNKNOWN)
    {
        mode = xc4_get_mode(mdrv->shared->mode_index);
        *ret_width  = mode->xres;
        *ret_height = mode->v_active;
    }
    else
    {
        conf = dfb_config->config_layer;
        *ret_width  = conf->config.width;
        *ret_height = conf->config.height;
    }

    D_DEBUG( "Xcode4GetScreenSize exit \n");

    return DFB_OK;
}


static DFBResult
Xcode4SetPowerMode( CoreScreen         *screen,
                   void               *driver_data,
                   void               *screen_data,
                   DFBScreenPowerMode  mode )
{
     
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    u32 tmp;

    tmp = REG_READ(DISP_HD_COMP_CTRL);
    tmp = tmp & (~DISP_HD_COMP_CTRL_COMP_MODE_MASK);
    switch (mode) 
    {
        case DSPM_OFF:
            tmp =  tmp & (~DISP_HD_COMP_CTRL_COMP_EN_MASK);
            break;

        case DSPM_SUSPEND:
        case DSPM_STANDBY:
            tmp =  tmp | (COMP_MODE_DISP_BACKGROUND << DISP_HD_COMP_CTRL_COMP_MODE_SHIFT);
            break;

        case DSPM_ON:
            tmp =  tmp | 
                   (COMP_MODE_CMD_REPEAT << DISP_HD_COMP_CTRL_COMP_MODE_SHIFT) |
                   (DISP_HD_COMP_CTRL_COMP_EN << DISP_HD_COMP_CTRL_COMP_EN_SHIFT);
            break;

        default:
             D_DEBUG( "unknown power mode" );
             return DFB_INVARG;
     }
     
     REG_WRITE( DISP_HD_COMP_CTRL, tmp );

     return DFB_OK;
}


static  void        Clk656On(pxc_dfb_driver_data mdrv)
{
    u32 tmp;
    DISP_LOCK_CG_CLOCK_STOP;
    tmp =   REG_READ(CG1_CLK_STOP0);
    tmp &= (~DISP_656CLK_STOP_MASK);
    REG_WRITE(CG1_CLK_STOP0, tmp);
    //D_INFO("Starting 656 clock ...\n");
    DISP_UNLOCK_CG_CLOCK_STOP;
    return;
}
static  void        Clk656Off(pxc_dfb_driver_data mdrv)
{
    u32 tmp;
    DISP_LOCK_CG_CLOCK_STOP;
    tmp =   REG_READ(CG1_CLK_STOP0);
    tmp |= (DISP_656CLK_STOP_MASK);
    REG_WRITE(CG1_CLK_STOP0, tmp);
    //D_INFO("Stopping 656 clock ...\n");
    DISP_UNLOCK_CG_CLOCK_STOP;
    return;
}
static  void        HDMIOn(pxc_dfb_driver_data mdrv)
{
    hdmi_reg_write(HDMI_PHY_CONF0, 0xFE, mdrv->fb0_handle);
    hdmi_reg_write(HDMI_MC_CLKGAT, 0x00, mdrv->fb0_handle);
    return;
}
static  void        HDMIOff(pxc_dfb_driver_data mdrv)
{
    hdmi_reg_write(HDMI_PHY_CONF0, DISP_HDMI_POWER_DOWN, mdrv->fb0_handle);
    
    hdmi_reg_write(HDMI_MC_CLKGAT,  CLKGATE_HDCP_MASK | CLKGATE_CSC_MASK | CLKGATE_AUD_MASK |
                                    CLKGATE_PREP_MASK | CLKGATE_TMDS_MASK | CLKGATE_PCLK_MASK, 
                                    mdrv->fb0_handle);
    return;
}



/**************************************************************************************************/

void Xcode4_Set_DVE_656_Output(pxc_dfb_driver_data mdrv,
                               struct display_mode_struct * mode,
                               struct dve_656_reg_struct * dve_656_reg_ptr)

{

    u32 tmp;
    u32 field1_vblank_end;
    u32 field2_vblank_end;
    u32 field1_vblank_length;
    u32 field2_vblank_length;
    u32 hsync_start;
    u32 hsync_end;
    u32 field1_vsync_start;
    u32 field1_vsync_end;
    u32 field2_vsync_start;
    u32 field2_vsync_end;
    u32 f2offlow;
    u32 f2offhigh;
	u32 f2off;
    u32	vsync_start, vsync_end;
    u32 total_active_lines;
    
    //return;
    static u32 isfirst = 1;
    //set the timing parameters for DVE

    
    if (mode->scanmode == DSESM_INTERLACED && 
        mode->resolution != DSOR_1920_2205)
        
    {      
        /* Straight interlaced case */
        
        //if (0)
        if (mdrv->output == 0)   
        {
            if (mode->resolution == DSOR_1920_1080 )
            {
                REG_WRITE(DISP_HD_VENC0_6562_FIELD_BNDRY, 0x2330465);   /* 0x2330465 */
                REG_WRITE(DISP_HD_VENC0_6562_VBLANK_START_BNDRY, 0x2300463);
                REG_WRITE(DISP_HD_VENC0_6562_VBLANK_END_BNDRY, 0x2470014);
                REG_WRITE(DISP_HD_VENC0_6562_CNTRL, 0xc000002a);
            }
            else if (mode->resolution == DSOR_720_480 )
            {
                REG_WRITE(DISP_HD_VENC0_6562_FIELD_BNDRY, 0x1090003);
                REG_WRITE(DISP_HD_VENC0_6562_VBLANK_START_BNDRY, 0x107020d);
                REG_WRITE(DISP_HD_VENC0_6562_VBLANK_END_BNDRY, 0x11a0013);
                REG_WRITE(DISP_HD_VENC0_6562_CNTRL, 0xc000002e);
            }
            else if (mode->resolution == DSOR_720_576 )
            {
                REG_WRITE(DISP_HD_VENC0_6562_FIELD_BNDRY, 0x1380271);   /* 0x1380271 */
                REG_WRITE(DISP_HD_VENC0_6562_VBLANK_START_BNDRY, 0x136026f);
                REG_WRITE(DISP_HD_VENC0_6562_VBLANK_END_BNDRY, 0x14f0016);
                REG_WRITE(DISP_HD_VENC0_6562_CNTRL, 0xc000002e);
            }
            else { 
                tmp = (mode->field1_vblank_start - 1) |
                    ((mode->field2_vblank_start - 1) << DISP_HD_VENC0_DVE_FIELD_BNDRY_DVE_FIELD_BNDRY_2_SHIFT);

                REG_WRITE( dve_656_reg_ptr->dve_field_bndry_reg, tmp);
                REG_WRITE( dve_656_reg_ptr->dve_vblank_start_bndry_reg, tmp);

                field1_vblank_length = (mode->v_total - mode->v_active) / 2;
                field2_vblank_length = field1_vblank_length + 1;
                field1_vblank_end = (mode->field1_vblank_start + field1_vblank_length) % mode->v_total - 1;
                field2_vblank_end = mode->field2_vblank_start + field2_vblank_length - 1;

                tmp = (field2_vblank_end << DISP_HD_VENC0_DVE_VBLANK_END_BNDRY_DVE_VBLANK_END_BNDRY_2_SHIFT) |
                    field1_vblank_end ;
                REG_WRITE( dve_656_reg_ptr->dve_vblank_end_bndry_reg, tmp);
            }
            
            //mdrv->output = 100;
      }
      else
      {
        
        if (dve_656_reg_ptr->dve_field_bndry_reg != DISP_SD_VENC0_DVE_FIELD_BNDRY)
        {
            if (mode->field1_vsync_start > 1)
            {
                field1_vsync_start = mode->field1_vsync_start - 1;
            }
            else
            {
                field1_vsync_start = mode->v_total;
            }
            
			field2_vsync_start = mode->field2_vsync_start - 1;
            //f2offlow = (mode->h_total / 2) & (DISP_HD_VENC0_DVE_VSYNC_START_F2OFFLOW_MASK >> DISP_HD_VENC0_DVE_VSYNC_START_F2OFFLOW_SHIFT);
			f2off = (mode->h_total/2) - (mode->h_total - mode->h_active) - 8;
			//f2off = (mode->h_total/2);
			
            if (mode->f2off != 0)
            {
                f2off   =   mode->f2off;  /* Set to 912 */
                /* 
                *   NOTE: This is a hack for now, so there is no need to do anything as of now ... 
                *   However, there will be a need to change the formula used to calculate f2off ... 
                */
            }
			f2offlow = f2off & 0xFF;
			
			f2offhigh = (f2off >> 8) & 0x1F;
			/* Seperate the f2off */
			
			tmp = field1_vsync_start |
			(field2_vsync_start << DISP_HD_VENC0_DVE_VSYNC_START_FRAME_2_SHIFT) |
			(f2offlow << DISP_HD_VENC0_DVE_VSYNC_START_F2OFFLOW_SHIFT);
            
            REG_WRITE( dve_656_reg_ptr->dve_vsync_start_reg, tmp);

            field1_vsync_end = mode->field1_vsync_start + mode->vsync_active_pulse_width - 1;
            field2_vsync_end = mode->field2_vsync_start + mode->vsync_active_pulse_width - 1;
            //f2offhigh = ((mode->h_total / 2) & 0x0f00) >> 8;
            tmp = field1_vsync_end |
                  (field2_vsync_end << DISP_HD_VENC0_DVE_VSYNC_END_FRAME_2_SHIFT) |
                  (f2offhigh << DISP_HD_VENC0_DVE_VSYNC_END_F2OFFHIGH_SHIFT);
            REG_WRITE( dve_656_reg_ptr->dve_vsync_end_reg, tmp);
			
			/*
			*	According to JimK
			*	f2off = (totalLineWidth/2) - hblank - 8
			*	For 1080i60 = (2200/2) - 280 - 8 = 812
			*/
			tmp = (mode->field1_vblank_start - 1) |
                    ((mode->field2_vblank_start - 1) << DISP_HD_VENC0_DVE_FIELD_BNDRY_DVE_FIELD_BNDRY_2_SHIFT);

                REG_WRITE( dve_656_reg_ptr->dve_field_bndry_reg, tmp);
                REG_WRITE( dve_656_reg_ptr->dve_vblank_start_bndry_reg, tmp);

                field1_vblank_length = (mode->v_total - mode->v_active) / 2;
                field2_vblank_length = field1_vblank_length + 1;
                field1_vblank_end = (mode->field1_vblank_start + field1_vblank_length) % mode->v_total - 1;
                field2_vblank_end = mode->field2_vblank_start + field2_vblank_length - 1;

                tmp = (field2_vblank_end << DISP_HD_VENC0_DVE_VBLANK_END_BNDRY_DVE_VBLANK_END_BNDRY_2_SHIFT) |
                    field1_vblank_end ;
                REG_WRITE( dve_656_reg_ptr->dve_vblank_end_bndry_reg, tmp);
			
			
            }
          }
        
    }
    else if (0)
    {
        /* 
        *   Interlaced frame packing case (it is actually progressive frame!!) 
        *   VBlank = <interlacedVblank, no F2off>
        *   Pixel freq = doubled
        *   Active video = (4 x (activeFieldHeight)) + (3 x Vblank size)
        *   
        */
#if 0        
        field1_vblank_length    =   (mode->v_total - mode->v_active) / 2;
        total_active_lines      =   (mode->v_active * 2) + (3 * field1_vblank_length);
        
        //field1_vblank_start     =   total_active_lines;
        field1_vblank_end       =   field1_vblank_start + field1_vblank_length;
        field1_vsync_start      =   field1_vblank_start + mode->vsync_active_edge_delay;
        field1_vsync_end        =   field1_vsync_start +  mode->vsync_active_pulse_width;

        tmp =   mode->field1_vsync_start |
                DISP_HD_VENC0_DVE_FIELD_BNDRY_DVE_FIELD_BNDRY_2_MASK;
                
        REG_WRITE( dve_656_reg_ptr->dve_field_bndry_reg, tmp);
        
        tmp = DISP_HD_VENC0_DVE_VBLANK_START_BNDRY_DVE_VBLANK_START_BNDRY_2_MASK | 
                   field1_vblank_start - 1;
        REG_WRITE( dve_656_reg_ptr->dve_vblank_start_bndry_reg, tmp);
        
        tmp = DISP_HD_VENC0_DVE_VBLANK_END_BNDRY_DVE_VBLANK_END_BNDRY_2_MASK | 
              ((field1_vblank_start + field1_vblank_length) % mode->v_total - 1);
        REG_WRITE( dve_656_reg_ptr->dve_vblank_end_bndry_reg, tmp);
        D_DEBUG("");
#endif        
    }
    else
    {
        /* Progressive case */
        tmp = mode->field1_vsync_start |
              DISP_HD_VENC0_DVE_FIELD_BNDRY_DVE_FIELD_BNDRY_2_MASK;

        REG_WRITE( dve_656_reg_ptr->dve_field_bndry_reg, tmp);

        tmp = DISP_HD_VENC0_DVE_VBLANK_START_BNDRY_DVE_VBLANK_START_BNDRY_2_MASK | 
                   mode->field1_vblank_start - 1;
        REG_WRITE( dve_656_reg_ptr->dve_vblank_start_bndry_reg, tmp);

        field1_vblank_length = mode->v_total - mode->v_active;
        tmp = DISP_HD_VENC0_DVE_VBLANK_END_BNDRY_DVE_VBLANK_END_BNDRY_2_MASK | 
              ((mode->field1_vblank_start + field1_vblank_length) % mode->v_total - 1);
        REG_WRITE( dve_656_reg_ptr->dve_vblank_end_bndry_reg, tmp);
		
		f2off = (mode->h_total/2) - (mode->h_total - mode->h_active) - 8;
		//f2off = (mode->h_total/2);
		f2offlow = f2off & 0xFF;
			
		f2offhigh = (f2off >> 8) & 0x1F;
		/* Seperate the f2off */

        if (dve_656_reg_ptr->dve_field_bndry_reg != DISP_SD_VENC0_DVE_FIELD_BNDRY)
        {
			/* 
			*
			*	The postive polarity "fix" was only necessary on A11 Viper
			*
			*/
			if (mode->pol_vsync == DISP_HD_VENC_POLARITY_NEG || mdrv->asic_flags & VIPER_ASIC_A21)
			{	

				/* Negative polarity VSYNC */
				if (mode->field1_vsync_start > 1) 
				{
					tmp = DISP_HD_VENC0_DVE_VSYNC_START_FRAME_2_MASK | 
					  ((f2offlow << DISP_HD_VENC0_DVE_VSYNC_START_F2OFFLOW_SHIFT) & 
					  DISP_HD_VENC0_DVE_VSYNC_START_F2OFFLOW_MASK) |
						  (mode->field1_vsync_start - 1);
				}
				else
				{
					tmp = DISP_HD_VENC0_DVE_VSYNC_START_FRAME_2_MASK | 
					((f2offlow << DISP_HD_VENC0_DVE_VSYNC_START_F2OFFLOW_SHIFT) & 
					  DISP_HD_VENC0_DVE_VSYNC_START_F2OFFLOW_MASK) |
						mode->v_total;
				}
				
				REG_WRITE( dve_656_reg_ptr->dve_vsync_start_reg, tmp);
				
				tmp = DISP_HD_VENC0_DVE_VSYNC_END_FRAME_2_MASK |
					((f2offhigh << DISP_HD_VENC0_DVE_VSYNC_END_F2OFFHIGH_SHIFT) & 
					  DISP_HD_VENC0_DVE_VSYNC_END_F2OFFHIGH_MASK) |
					(mode->field1_vsync_start + mode->vsync_active_pulse_width - 1);
				REG_WRITE( dve_656_reg_ptr->dve_vsync_end_reg, tmp);
			
			}
			else
			{
				/*
				*	Positive polarity VSYNC, swap sync values 
				*/
				if (mode->field1_vsync_start > 1) 
				{
					tmp = DISP_HD_VENC0_DVE_VSYNC_END_FRAME_2_MASK | 
						  DISP_HD_VENC0_DVE_VSYNC_END_F2OFFHIGH_MASK |
							(mode->field1_vsync_start - 1);
				}
				else
				{
					tmp = DISP_HD_VENC0_DVE_VSYNC_END_FRAME_2_MASK | 
						  DISP_HD_VENC0_DVE_VSYNC_END_F2OFFHIGH_MASK |
							mode->v_total;
				}
				REG_WRITE( dve_656_reg_ptr->dve_vsync_end_reg, tmp);
					
				tmp = DISP_HD_VENC0_DVE_VSYNC_START_FRAME_2_MASK |
					  DISP_HD_VENC0_DVE_VSYNC_START_F2OFFLOW_MASK |
					  (mode->field1_vsync_start + mode->vsync_active_pulse_width - 1);
				REG_WRITE( dve_656_reg_ptr->dve_vsync_start_reg, tmp);
			}
		}
	}
    
	if (dve_656_reg_ptr->dve_field_bndry_reg != DISP_SD_VENC0_DVE_FIELD_BNDRY)
    {
        //hsync_start = mode->hsync_active_edge_delay + 1;
        //hsync_end = hsync_start + mode->hsync_active_pulse_width - 1;
        /* 
		*	According to JimK this should be as follows:
		*	hsync_start = hsync_active_edge_delay + 4
		*	hsync_end = hsync_start + hsync_active_pulse_width
		*
		*/
		hsync_start = mode->hsync_active_edge_delay + 4;
		hsync_end	= hsync_start + mode->hsync_active_pulse_width;
		
		if (mode->pol_hsync == DISP_HD_VENC_POLARITY_NEG || mdrv->asic_flags & VIPER_ASIC_A21) 
		{
			/* Negative HSYNC polarity, leave calculation as is */
			REG_WRITE( dve_656_reg_ptr->dve_hsync_reg, hsync_start 
					| (hsync_end << DISP_HD_VENC0_DVE_HSYNC_END_SHIFT));
		}
		else
		{
			/* Positive HSYNC polarity, swap fields */
			REG_WRITE( dve_656_reg_ptr->dve_hsync_reg, hsync_end 
					| (hsync_start << DISP_HD_VENC0_DVE_HSYNC_END_SHIFT));
		}
    }

    //DVE output registers
    if (isfirst)
    {
        REG_WRITE( dve_656_reg_ptr->dve_cntrl_reg, DISP_DVE_OUTPUT_CONFIG);
        isfirst = 0;
    }else{
        tmp = REG_READ(dve_656_reg_ptr->dve_cntrl_reg);
        tmp &= (DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_VENC_MASK | DISP_HD_VENC0_DVE_CNTRL_VBI_ON_Y_MASK); 
        tmp |= DISP_DVE_OUTPUT_CONFIG;  
        REG_WRITE( dve_656_reg_ptr->dve_cntrl_reg, tmp); 
    }

    return;

}


void ViperHDDACPowerUp(pxc_dfb_driver_data mdrv)
{
	/* 
	*	Power up ONLY the HD analog DAC's 
	*	any other DAC's should remain untouched
	*	HD Y, Pb, Pr
	*/
	u32 tmp;
    
    
	/* Ignore this function */
	if (mdrv->asic_flags & VIPER_ASIC_A21)
	{
		
		REG_WRITE(DISP_VDAC_SET0_CNTL, 0x80000015);
		//REG_WRITE(DISP_VDAC_SET1_CNTL, 0x80000080);
		REG_WRITE(DISP_VDAC01_CNTL, 0xA600A600);
		
		tmp = REG_READ(DISP_VDAC23_CNTL);
		
		tmp|= 0xA600; /* Enable the HD Pr DAC*/
		
		REG_WRITE(DISP_VDAC23_CNTL, tmp);
		//REG_WRITE(DISP_VDAC45_CNTL, 0xA600A600);
		//REG_WRITE(DISP_VDAC_CABLE_CONNECT, 0xA6000000);
        REG_WRITE(DISP_VDAC_BIAS_CNTL, 0x10);
	}
    else
    {
        REG_WRITE(DISP_VDAC_SET0_CNTL,0x04440015);
        //REG_WRITE(DISP_VDAC_SET1_CNTL,0x04444080);
        REG_WRITE(DISP_VDAC01_CNTL,0xB600B600);
        
        tmp = REG_READ(DISP_VDAC23_CNTL);
        tmp |= 0xB600;
        REG_WRITE(DISP_VDAC23_CNTL, tmp);
        //REG_WRITE(DISP_VDAC23_CNTL,0xB600B600);
        
        
        //  REG_WRITE(DISP_VDAC45_CNTL,0xB600B600);
        //  REG_WRITE(DISP_VDAC_CABLE_CONNECT,0xB6000000);
        REG_WRITE(DISP_VDAC_BIAS_CNTL, 0x10);
    }
	
	D_DEBUG("Viper HD DAC Power Up...\n");
	return;
}


void ViperHDDACPowerDown(pxc_dfb_driver_data mdrv)
{
	/* 
	*	Power down ONLY the HD analog DAC's 
	*	any other DAC's should remain untouched
	*	HD Y, Pb, Pr
	*/
	u32 tmp;
	/* This shall function on any ASIC rev.  */
    //if (mdrv->asic_flags & VIPER_ASIC_A21)
	//{
		
		REG_WRITE(DISP_VDAC_SET0_CNTL, 0x0);
		//REG_WRITE(DISP_VDAC_SET1_CNTL, 0x0);
		REG_WRITE(DISP_VDAC01_CNTL, 0x0);
		
		tmp = REG_READ(DISP_VDAC23_CNTL);
		tmp&= ~0xFFFF;                       /* Disable the HD Pr DAC*/
		REG_WRITE(DISP_VDAC23_CNTL, tmp);
		//REG_WRITE(DISP_VDAC_BIAS_CNTL, 0x0);
	//}
    D_DEBUG("Viper HD DAC Power Down...\n");
	return;
}

/**************************************************************************************************/

#define DEFAULT_CBCRY 6 

void Xcode4_Set_HD_DVE_656_2_Output_Order(pxc_dfb_driver_data mdrv,
                               const DFBScreenOutputConfig *config)
{
    u32 tmp;
    u32 dual_output_order;
    u32 single_output_order;
    u32 bus_width;

    if (config->bus_type == DSBT_SINGLE)
    {
        
        if (config->output_order == DSSO_YCBCR)
        {
            single_output_order = DSSO_YCBCR;
        }
        else if (config->output_order == DSSO_CBYCR)
        {
            single_output_order = DSSO_CBYCR;
        }
        else if (config->output_order == DSSO_CBCRY)
        {
            single_output_order = DEFAULT_CBCRY;
        }
        
        
        if(config->out_signals == DSOS_656 )
        {
        	bus_width = 0;
        }
        else if (config->out_signals == DSOS_656_10)
        {
        	bus_width = 0x1;
        }
        else if (config->out_signals == DSOS_656_12)
        {
        	bus_width = 0x2;
        }
    	    
        RegMaskWrite(DISP_HD_VENC0_6562_CNTRL,
                     ((0x3 << DISP_HD_VENC0_6562_CNTRL_BUSTYPE_SHIFT) | (single_output_order << DISP_HD_VENC0_6562_CNTRL_OUTPUT_ORDER_SHIFT) | (bus_width << DISP_HD_VENC0_6562_CNTRL_BUSWIDTH_SHIFT)) ,
                     (DISP_HD_VENC0_6562_CNTRL_OUTPUT_ORDER_MASK | DISP_HD_VENC0_6562_CNTRL_BUSTYPE_MASK  | DISP_HD_VENC0_6562_CNTRL_BUSWIDTH_MASK) );
    }
    else if (config->bus_type == DSBT_DUAL)
    {
        if (config->output_order == DSSO_YCBCR)
        {
            dual_output_order = DSSO_YCBCR;
        }
        else if (config->output_order == DSSO_CBYCR)
        {
            dual_output_order = DSSO_CBYCR;
        }
        else if (config->output_order == DSSO_CBCRY)
        {
            dual_output_order = DEFAULT_CBCRY;
        }
        
        
        if(config->out_signals == DSOS_656 ||
           config->out_signals == DSOS_656E 
          )
        {
        	bus_width = 0;
        }
        else if (config->out_signals == DSOS_656_10 || 
                 config->out_signals == DSOS_656_10E 
                )
        {
        	bus_width = 0x1;
        }
        else if (config->out_signals == DSOS_656_12 ||
                 config->out_signals == DSOS_656_12E
                )
        {
        	bus_width = 0x2;
        }
        
        
        RegMaskWrite(DISP_HD_VENC0_6562_CNTRL,
                     ((config->bus_type << DISP_HD_VENC0_6562_CNTRL_BUSTYPE_SHIFT) | (dual_output_order << DISP_HD_VENC0_6562_CNTRL_OUTPUT_ORDER_SHIFT) | (bus_width << DISP_HD_VENC0_6562_CNTRL_BUSWIDTH_SHIFT)),
                     (DISP_HD_VENC0_6562_CNTRL_OUTPUT_ORDER_MASK | DISP_HD_VENC0_6562_CNTRL_BUSTYPE_MASK | DISP_HD_VENC0_6562_CNTRL_BUSWIDTH_MASK ));
    }
    else if (config->bus_type == DSBT_TRIPLE)
    {
        RegMaskWrite(DISP_HD_VENC0_6562_CNTRL,
                     ((config->bus_type << DISP_HD_VENC0_6562_CNTRL_BUSTYPE_SHIFT) | (config->output_order << DISP_HD_VENC0_6562_CNTRL_OUTPUT_ORDER_SHIFT)),
                     (DISP_HD_VENC0_6562_CNTRL_OUTPUT_ORDER_MASK | DISP_HD_VENC0_6562_CNTRL_BUSTYPE_MASK));
    }
    else if (config->bus_type == DSBT_SINGLE_BOTH)
    {
        RegMaskWrite(DISP_HD_VENC0_6562_CNTRL,
                     (config->bus_type << DISP_HD_VENC0_6562_CNTRL_BUSTYPE_SHIFT),
                     (DISP_HD_VENC0_6562_CNTRL_OUTPUT_ORDER_MASK | DISP_HD_VENC0_6562_CNTRL_BUSTYPE_MASK));
    }

    return;
}

/**************************************************************************************************/

void Set_HD_Component_Hard_Code(pxc_dfb_driver_data mdrv)
{
    u32 tmp;
#ifdef DISABLE_COMPONENT
	return;

#endif
	
    REG_WRITE(DISP_SFT_RESET, 0xef77);
    REG_WRITE(DISP_SFT_RESET, 0xef77);
    REG_WRITE(DISP_SFT_RESET, 0xffff);
    
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20138000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x30298000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x403f8000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x208a0000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0xfff8000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20138000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x8);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x30298000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x9);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x403f8000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xa);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x206b8000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xb);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x34498000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xc);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2fff8000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xd);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xe);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xf);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x11);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x12);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x13);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x14);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x15);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x16);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x17);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x18);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x19);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x20);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x21);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x22);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x23);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x24);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x25);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x26);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x27);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x28);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x29);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x30);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x31);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x32);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x33);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x34);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x35);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x36);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x37);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x38);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x39);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x40);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x41);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x42);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x43);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x44);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x45);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x46);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x47);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x48);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x49);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x50);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x51);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x52);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x53);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x54);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x55);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x56);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x57);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x58);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x59);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x60);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x61);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x63);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x64);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x65);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x66);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x67);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x68);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x69);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x70);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x71);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x72);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x73);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x74);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x75);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x76);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x77);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x78);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x79);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7f);
    REG_WRITE(DISP_HD_VENC0_CB_FREQ, 0x21f07c1f);
    REG_WRITE(DISP_HD_VENC0_CB_PHASE, 0x1040c0d0);
    REG_WRITE(DISP_HD_VENC0_CMPST_GAIN, 0x39487ff3);
    REG_WRITE(DISP_HD_VENC0_BLACK, 0x27);
    REG_WRITE(DISP_HD_VENC0_BLANK, 0x40013f);
    REG_WRITE(DISP_HD_VENC0_SYNC1, 0xb);
    REG_WRITE(DISP_HD_VENC0_SYNC2, 0x273);
    REG_WRITE(DISP_HD_VENC0_YCMPN, 0x3c000bf);
    REG_WRITE(DISP_HD_VENC0_CB2GAIN, 0x805a005a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0xa1);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x100);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x3ffe0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x101);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x142);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x102);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x248);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x103);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2e4);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x104);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20e0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x105);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2106);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x106);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2142);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x107);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2163);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x108);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x21a1);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x109);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x21c5);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2202);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2227);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2328);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x23a4);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x3ffe0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10f);
    REG_WRITE(DISP_HD_VENC0_FIELDBNDRY, 0x106);
    REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS0, 0x0);
    REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS1, 0x40a03fb);
    REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2);
    REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS3, 0x6b);
    REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS0, 0x0);
    REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS1, 0x2fef8f6);
    REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618);
    REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS3, 0x30);
    
    //REG_WRITE(DISP_HD_VENC0_DVE_VSYNC_START, 0x5a7ff002);
    //REG_WRITE(DISP_HD_VENC0_DVE_VSYNC_END, 0x37ff005);
    
    REG_WRITE(DISP_OUTPUT_MUX, 0x1d);
    
    REG_WRITE(DISP_HD_VENC0_DCS_CONFIG, 0x40);
    REG_WRITE(DISP_HD_VENC0_DCS_CONFIG, 0x801e0140);
    REG_WRITE(DISP_HD_VENC0_DCS_CMPNT, 0x4fc002c);
    REG_WRITE(DISP_HD_VENC0_DCS_CMPNT_ICLK227K, 0x450081);
    REG_WRITE(DISP_HD_VENC0_DCS_CMPNT_ICLK397K, 0x3f00e3);
    REG_WRITE(DISP_HD_VENC0_DCS_CONFIG_CMST, 0x2);
    REG_WRITE(DISP_HD_VENC0_DCS_CONFIG_CMST, 0x80210102);
    REG_WRITE(DISP_HD_VENC0_DCS_CMST, 0x3ad0028);
    REG_WRITE(DISP_HD_VENC0_DCS_CMST_ICLK227K, 0x450081);
    REG_WRITE(DISP_HD_VENC0_DCS_CMST_ICLK397K, 0x3f00e3);
    REG_WRITE(DISP_HD_VENC0_DCS_START_LINE, 0x1);
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0xb);

#if 0	
    REG_WRITE(DISP_VDAC_SET0_CNTL,0x04440015);
    REG_WRITE(DISP_VDAC_SET1_CNTL,0x04444080);
    REG_WRITE(DISP_VDAC01_CNTL,0xB600B600);
    REG_WRITE(DISP_VDAC23_CNTL,0xB600B600);
    REG_WRITE(DISP_VDAC45_CNTL,0xB600B600);
    REG_WRITE(DISP_VDAC_CABLE_CONNECT,0xB6000000);
#endif    
	
	RegMaskWrite(DISP_OUTPUT_MUX,0x1fc00,0x1fc00);
    RegMaskWrite(DISP_SFT_RESET,0x0,0x8);
    RegMaskWrite(DISP_SFT_RESET,0x8,0x8);

    
}
    
void Set_HD_Component_Hard_Code_720p(pxc_dfb_driver_data mdrv)
{
    u32 tmp;
#ifdef DISABLE_COMPONENT
	return;

#endif	

    REG_WRITE(DISP_SFT_RESET, 0xef77);
    REG_WRITE(DISP_SFT_RESET, 0xef77);
    REG_WRITE(DISP_SFT_RESET, 0xffff);
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20150001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x30550001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20588001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20600001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x206b8001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20820001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20960001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x4000008);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20150000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x8);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x31820000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x9);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x21c20000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xa);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x332f0000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xb);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x24000000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xc);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xd);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xe);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xf);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20150000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x30350000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x11);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x21c20000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x12);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x31e20000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x13);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x24000000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x14);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x15);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x16);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x17);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20150000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x18);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x30350000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x19);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x21c20000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x332d0000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x24000000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20150001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x20);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x30550001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x21);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20588001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x22);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20600001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x23);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x206b8001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x24);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20820001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x25);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20960001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x26);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x4000000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x27);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20150000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x28);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x31820000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x29);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x21c20000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x31e20000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x24000000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20150001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x30);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x30550001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x31);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20600001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x32);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20820001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x33);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20960001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x34);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x21c20001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x35);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x31e20001);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x36);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x24000000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x37);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20150000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x38);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x30350000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x39);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x24000000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20150002);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x40);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x30550000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x41);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20588000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x42);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20600006);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x43);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x206b8007);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x44);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x207e8003);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x45);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x74000000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x46);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x47);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x48);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x49);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x50);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x51);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x52);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x53);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x54);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x55);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x56);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x57);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x58);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x59);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x60);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x61);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x63);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x64);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x65);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x66);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x67);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x68);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x69);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x70);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x71);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x72);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x73);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x74);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x75);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x76);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x77);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x78);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x79);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x100);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x101);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x102);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x103);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x104);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x105);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x106);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x107);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x108);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x109);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x110);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x111);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x112);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x113);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x114);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x115);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x116);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x117);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x118);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x119);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x11a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x11b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x11c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x11d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x11e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x11f);
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0x3);
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0x3);
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0x3);
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0x3);
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0x3);
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0x3);
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0x0);
    
    REG_WRITE(DISP_SFT_RESET, 0xef77);
    REG_WRITE(DISP_SFT_RESET, 0xef77);
    REG_WRITE(DISP_SFT_RESET, 0xffff);
    
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20210000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x30350000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x40490000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20b70000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x7ff8000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20210000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x8);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x30350000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x9);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x40490000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xa);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20b70000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xb);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x33370000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xc);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x27ff8000);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xd);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xe);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0xf);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x11);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x12);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x13);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x14);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x15);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x16);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x17);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x18);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x19);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x20);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x21);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x22);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x23);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x24);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x25);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x26);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x27);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x28);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x29);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x30);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x31);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x32);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x33);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x34);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x35);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x36);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x37);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x38);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x39);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x40);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x41);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x42);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x43);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x44);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x45);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x46);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x47);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x48);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x49);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x50);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x51);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x52);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x53);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x54);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x55);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x56);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x57);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x58);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x59);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x60);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x61);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x62);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x63);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x64);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x65);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x66);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x67);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x68);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x69);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6f);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x70);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x71);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x72);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x73);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x74);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x75);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x76);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x77);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x78);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x79);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7f);
    REG_WRITE(DISP_HD_VENC0_CB_FREQ, 0x21f07c1f);
    REG_WRITE(DISP_HD_VENC0_CB_PHASE, 0x1040c0d0);
    REG_WRITE(DISP_HD_VENC0_CMPST_GAIN, 0x39487ff3);
    REG_WRITE(DISP_HD_VENC0_BLACK, 0x27);
    REG_WRITE(DISP_HD_VENC0_BLANK, 0x92013f);
    REG_WRITE(DISP_HD_VENC0_SYNC1, 0xb);
    REG_WRITE(DISP_HD_VENC0_SYNC2, 0x273);
    REG_WRITE(DISP_HD_VENC0_YCMPN, 0x3c000bf);
    REG_WRITE(DISP_HD_VENC0_CB2GAIN, 0x805a005a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0xa1);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x100);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x3ffe0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x101);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x142);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x102);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x248);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x103);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2e4);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x104);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20e0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x105);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2106);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x106);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2142);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x107);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2163);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x108);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x21a1);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x109);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x21c5);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10a);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2202);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10b);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2227);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10c);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2328);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10d);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x23a4);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10e);
    REG_WRITE(DISP_HD_VENC0_WRDATA, 0x3ffe0);
    REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10f);
    REG_WRITE(DISP_HD_VENC0_FIELDBNDRY, 0x106);
    REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS0, 0x0);
    REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS1, 0x40a03fb);
    REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2);
    REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS3, 0x6b);
    REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS0, 0x0);
    REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS1, 0x2fef8f6);
    REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618);
    REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS3, 0x30);
    REG_WRITE(DISP_HD_VENC0_DVE_VSYNC_START, 0x5a7ff002);
    REG_WRITE(DISP_HD_VENC0_DVE_VSYNC_END, 0x37ff005);
    REG_WRITE(DISP_OUTPUT_MUX, 0x1e);
    
    REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0xb);
    
    
#if 0    
    REG_WRITE(DISP_VDAC_SET0_CNTL,0x04440015);
    REG_WRITE(DISP_VDAC_SET1_CNTL,0x04444080);
    REG_WRITE(DISP_VDAC01_CNTL,0xB600B600);
    REG_WRITE(DISP_VDAC23_CNTL,0xB600B600);
    REG_WRITE(DISP_VDAC45_CNTL,0xB600B600);
    REG_WRITE(DISP_VDAC_CABLE_CONNECT,0xB6000000);
#endif

	
    RegMaskWrite(DISP_OUTPUT_MUX,0x1fc00,0x1fc00);
    RegMaskWrite(DISP_SFT_RESET,0x0,0x8);
    RegMaskWrite(DISP_SFT_RESET,0x8,0x8);
        
}
    
    
void Set_HD_Component_Hard_Code_1080i(pxc_dfb_driver_data mdrv)
{
    u32 tmp;
	
#ifdef DISABLE_COMPONENT
	return;

#endif	

	REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20140000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x302a0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x40400000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x208a8000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0xfff8000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20140000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x8);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x302a0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x9);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x40400000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0xa);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x206c0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0xb);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x32240000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0xc);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x223a0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0xd);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x32500000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0xe);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x42660000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0xf);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x22920000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x344a0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x11);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2fff8000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x12);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x13);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x14);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x15);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x16);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x17);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20140000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x18);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x302a0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x19);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x40400000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1a);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x223a0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1b);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x32500000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1c);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x42660000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1d);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x22920000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1e);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x344a0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x1f);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2fff8000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x20);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x21);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x22);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x23);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x24);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x25);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x26);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x27);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20140000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x28);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x302a0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x29);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x40400000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2a);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x206c0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2b);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x32240000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2c);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x223a0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2d);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x32500000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2e);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x42660000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x2f);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2fff8000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x30);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x31);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x32);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x33);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x34);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x35);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x36);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x37);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x20140000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x38);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x302a0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x39);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x40400000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3a);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x223a0000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3b);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x32500000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3c);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x42660000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3d);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2fff8000);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3e);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x3f);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x40);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x41);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x42);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x43);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x44);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x45);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x46);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x47);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x48);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x49);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4a);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4b);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4c);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4d);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4e);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x4f);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x50);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x51);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x52);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x53);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x54);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x55);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x56);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x57);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x58);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x59);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5a);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5b);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5c);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5d);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5e);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x5f);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x60);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x61);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x62);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x63);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x64);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x65);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x66);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x67);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x68);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x69);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6a);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6b);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6c);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6d);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6e);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x6f);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x70);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x71);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x72);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x73);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x74);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x75);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x76);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x77);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x78);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x79);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7a);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7b);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7c);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7d);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7e);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x7f);
	REG_WRITE(DISP_HD_VENC0_CB_FREQ, 0x21f07c1f);
	REG_WRITE(DISP_HD_VENC0_CB_PHASE, 0x1040c0d0);
	REG_WRITE(DISP_HD_VENC0_CMPST_GAIN, 0x39487ff3);
	REG_WRITE(DISP_HD_VENC0_BLACK, 0x27);
	REG_WRITE(DISP_HD_VENC0_BLANK, 0x92013f);
	REG_WRITE(DISP_HD_VENC0_SYNC1, 0xb);
	REG_WRITE(DISP_HD_VENC0_SYNC2, 0x273);
	REG_WRITE(DISP_HD_VENC0_YCMPN, 0x3c000bf);
	REG_WRITE(DISP_HD_VENC0_CB2GAIN, 0x805a005a);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0xa1);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x100);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0xc7);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x101);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x4640);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x102);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x4663);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x103);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x46e1);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x104);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x4705);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x105);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x8cc0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x106);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2142);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x107);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2163);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x108);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x21a1);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x109);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x21c5);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10a);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2202);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10b);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2227);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10c);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x2328);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10d);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x23a4);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10e);
	REG_WRITE(DISP_HD_VENC0_WRDATA, 0x3ffe0);
	REG_WRITE(DISP_HD_VENC0_WRADDR, 0x10f);
	REG_WRITE(DISP_HD_VENC0_FIELDBNDRY, 0x106);
	REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS0, 0x0);
	REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS1, 0x40a03fb);
	REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2);
	REG_WRITE(DISP_HD_VENC0_COEF_FYCMPS3, 0x6b);
	REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS0, 0x0);
	REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS1, 0x2fef8f6);
	REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618);
	REG_WRITE(DISP_HD_VENC0_COEF_FUVCMPS3, 0x30);

	REG_WRITE(DISP_HD_VENC0_DVE_VSYNC_START, 0x5a233002);
	REG_WRITE(DISP_HD_VENC0_DVE_VSYNC_END, 0x3238005);

	REG_WRITE(DISP_OUTPUT_MUX, 0x1e);
	REG_WRITE(DISP_HD_VENC0_CONT_STATUS, 0xb);
#if 0	
	REG_WRITE (DISP_VDAC01_CNTL,0x2000200);
	REG_WRITE (DISP_VDAC23_CNTL,0x2000200);
	REG_WRITE (DISP_VDAC45_CNTL,0x2000200);
	REG_WRITE (DISP_VDAC_CABLE_CONNECT,0x2000000);
	
	REG_WRITE(DISP_VDAC_SET0_CNTL,0x04440015);
	REG_WRITE(DISP_VDAC_SET1_CNTL,0x04444080);
	REG_WRITE(DISP_VDAC01_CNTL,0xB600B600);
	REG_WRITE(DISP_VDAC23_CNTL,0xB600B600);
	REG_WRITE(DISP_VDAC45_CNTL,0xB600B600);
	REG_WRITE(DISP_VDAC_CABLE_CONNECT,0xB6000000);
#endif
	
	RegMaskWrite(DISP_OUTPUT_MUX,0x1fc00,0x1fc00);
	RegMaskWrite(DISP_SFT_RESET,0x0,0x8);
	RegMaskWrite(DISP_SFT_RESET,0x8,0x8);
}

#define MSG(x)   fprintf(stderr, "edid: %s\n", x)

int
parse_edid(byte* edid, DFBScreenOutputEdidConfig *config);

int
parse_edid_extension(byte* edid, DFBScreenOutputEdidConfig *config);

int
parse_timing_description(byte* dtd, DFBScreenOutputEdidConfig *config);

int
parse_monitor_limits( byte* block );

int
block_type( byte* block );

char*
get_monitor_name( byte const*  block );

char*
get_vendor_sign( byte const* block );

int
parse_dpms_capabilities( byte flags );

void parse_established_timings_1(byte timing, DFBScreenOutputEdidConfig *config);
void parse_established_timings_2(byte timing, DFBScreenOutputEdidConfig *config);
void parse_standard_timings(byte* edid, DFBScreenOutputEdidConfig *config);

u8 isResolutionNotinConfig(DFBScreenOutputResolution resolution, DFBScreenOutputEdidConfig *config);
u8 isFrequencyNotinConfig(DFBScreenEncoderFrequency frequency, DFBScreenOutputEdidConfig *config);

int
parse_edid(byte* edid, DFBScreenOutputEdidConfig *config)
{
  unsigned i;
  byte* block;
  char* monitor_name = NULL;
  char monitor_alt_name[100];
  byte checksum = 0;
  char *vendor_sign;
  int ret = 0;
  unsigned extNum = edid[EXTENSION_FLAG];

  if (extNum != 0)
  {
//      DISP_EDID_DATA( "%d extension(s)\n", extNum);
//      parse_edid_extension(edid, config);
  }
  else
  {
      MSG( "No EDID extension" );
  }
  
  for( i = 0; i < EDID_LENGTH; i++ )
  {
    checksum += edid[ i ];
  }

  if (checksum != 0) 
  {
      MSG( "EDID checksum failed - data is corrupt. Continuing anyway." );
      ret = 1;
  } else 
      MSG( "EDID checksum passed." );  

  if ( strncmp( edid+EDID_HEADER, edid_v1_header, EDID_HEADER_END+1 ) )
  {
      MSG( "first bytes don't match EDID version 1 header" );
      MSG( "do not trust output (if any)." );
      ret = 1;
  }

  DISP_EDID_DATA( "\n\t# EDID version %d revision %d\n", (int)edid[EDID_STRUCT_VERSION],(int)edid[EDID_STRUCT_REVISION] );

  vendor_sign = get_vendor_sign( edid + ID_MANUFACTURER_NAME ); 

  DISP_EDID_DATA( "Section \"Monitor\"\n" );

  block = edid + DETAILED_TIMING_DESCRIPTIONS_START;

  for( i = 0; i < NO_DETAILED_TIMING_DESCRIPTIONS; i++,
	 block += DETAILED_TIMING_DESCRIPTION_SIZE )
    {

      if ( block_type( block ) == MONITOR_NAME )
	{
	  monitor_name = get_monitor_name( block );
	  break;
	}
    }

  if (!monitor_name) {
    /* Stupid djgpp hasn't snprintf so we have to hack something together */
    if(strlen(vendor_sign) + 10 > sizeof(monitor_alt_name))
      vendor_sign[3] = 0;
    
    sprintf(monitor_alt_name, "%s:%02x%02x",
	    vendor_sign, edid[ID_MODEL], edid[ID_MODEL+1]) ;
    monitor_name = monitor_alt_name;
  }

  DISP_EDID_DATA( "\tIdentifier \"%s\"\n", monitor_name );
  DISP_EDID_DATA( "\tVendorName \"%s\"\n", vendor_sign );
  DISP_EDID_DATA( "\tModelName \"%s\"\n", monitor_name );

  block = edid + DETAILED_TIMING_DESCRIPTIONS_START;

  for( i = 0; i < NO_DETAILED_TIMING_DESCRIPTIONS; i++,
	 block += DETAILED_TIMING_DESCRIPTION_SIZE )
    {

      if ( block_type( block ) == MONITOR_LIMITS )
	parse_monitor_limits( block );
    }

  parse_dpms_capabilities(edid[DPMS_FLAGS]);

  parse_established_timings_1(edid[ESTABLISHED_TIMING_1], config);

  parse_established_timings_2(edid[ESTABLISHED_TIMING_2], config);

  parse_standard_timings(edid, config);

  block = edid + DETAILED_TIMING_DESCRIPTIONS_START;

  for( i = 0; i < NO_DETAILED_TIMING_DESCRIPTIONS; i++,
	 block += DETAILED_TIMING_DESCRIPTION_SIZE )
  {
      if ( block_type( block ) == DETAILED_TIMING_BLOCK )
      {
        parse_timing_description( block, config);
      }
  }

  DISP_EDID_DATA( "EndSection\n" );

  return ret;
}

#define END_OF_FIRSTBLOCK 0xFF

#define BYTE_NUMBER_OFFSET 0x82
#define NO_DETAILED_TIMING_DESCRIPTIONS_EXT 2

#define MANUFACTURER_DEFINED_DESCRIPTOR 0xAD
#define MONITOR_SERIAL_NUMBER 0xBF

 
int
parse_edid_extension(byte* edid, DFBScreenOutputEdidConfig *config)
{
  int ret = 0;
  byte byte_number_offset = edid[BYTE_NUMBER_OFFSET];
  byte* block;
  char rev_num[8];
  char monitor_serial_number[10];
  unsigned i;
  byte checksum = 0;

  for( i = 0x80; i <= END_OF_FIRSTBLOCK; i++)
    checksum += edid[ i ];

  if (checksum != 0x0E7C) 
  {
      MSG( "EDID extension checksum failed - data is corrupt. Continuing anyway." );
      ret = 1;
  } 
  else 
  {
      MSG( "EDID extension checksum passed." );
  }

  if (byte_number_offset == 0)
  {
  	DISP_EDID_DATA( "no detailed timing descriptions\n" );
  }
  else
  {
	// byte_number_offset == 4
	// parse detailed timing description
	block = edid + byte_number_offset;

	for( i = 0; i < NO_DETAILED_TIMING_DESCRIPTIONS_EXT; i++,
	 block += DETAILED_TIMING_DESCRIPTION_SIZE )
	{
		if ( block_type( block ) == DETAILED_TIMING_BLOCK )
        {
		    parse_timing_description(block, config);
        }
	}

   }

  // Descriptor defined by manufacturer block
  for (i = MANUFACTURER_DEFINED_DESCRIPTOR;i<8;i++)
  {
	rev_num[i] = edid[i];
  }
  rev_num[8] = 0;
  DISP_EDID_DATA( "Descriptor Defined by Manufacturer: %s\n", rev_num);

  // monitor serial number
  for (i = MONITOR_SERIAL_NUMBER;i<10;i++)
  {
	monitor_serial_number[i] = edid[i];
  }
  monitor_serial_number[10] = 0;
  DISP_EDID_DATA( "monitor serial number: %s\n", monitor_serial_number);

  return ret;
}

u8 isResolutionNotinConfig(DFBScreenOutputResolution resolution, DFBScreenOutputEdidConfig *config)
{
    u8 retValue = 1;

    int i;
    for (i=0;i<config->numberOfModes;++i)
    {
        if (resolution == config->resolution[i])
        {
            // already exists
            retValue = 0;
            break;
        }
    }

    return retValue;
}

u8 isFrequencyNotinConfig(DFBScreenEncoderFrequency frequency, DFBScreenOutputEdidConfig *config)
{
    u8 retValue = 1;

    int i;
    for (i=0;i<config->numberOfModes;++i)
    {
        if (frequency == config->frequency[i])
        {
            // already exists
            retValue = 0;
            break;
        }
    }

    return retValue;
}

int
parse_timing_description( byte* dtd, DFBScreenOutputEdidConfig *config)
{
  int htotal = H_ACTIVE + H_BLANKING;
  int vtotal = V_ACTIVE + V_BLANKING;
  unsigned long vfreq_remain = PIXEL_CLOCK % (vtotal*htotal);
  unsigned long vfreq = PIXEL_CLOCK / (vtotal*htotal);
  DFBScreenEncoderFrequency frequency;
  DFBScreenOutputResolution resolution;
  
  DISP_EDID_DATA( "\tMode \t\"%dx%d\"", H_ACTIVE, V_ACTIVE );
  DISP_EDID_DATA( "\t# vfreq %3.3fHz, hfreq %6.3fkHz\n",
	  (float)PIXEL_CLOCK/((float)vtotal*(float)htotal),
	  (float)PIXEL_CLOCK/(float)(htotal*1000));

  DISP_EDID_DATA( "\t\tDotClock\t%f\n", (float)PIXEL_CLOCK/1000000.0f );

  DISP_EDID_DATA( "\t\tHTimings\t%u %u %u %u\n", H_ACTIVE,
	  H_ACTIVE+H_SYNC_OFFSET, 
	  H_ACTIVE+H_SYNC_OFFSET+H_SYNC_WIDTH,
	  htotal );

  DISP_EDID_DATA( "\t\tVTimings\t%u %u %u %u\n", V_ACTIVE,
	  V_ACTIVE+V_SYNC_OFFSET,
	  V_ACTIVE+V_SYNC_OFFSET+V_SYNC_WIDTH,
	  vtotal );

  if ( INTERLACED || (SYNC_TYPE == SYNC_SEPARATE)) {
    DISP_EDID_DATA( "\t\tFlags\t%s\"%sHSync\" \"%sVSync\"\n",
	INTERLACED ? "\"Interlace\" ": "",
	HSYNC_POSITIVE ? "+": "-",
	VSYNC_POSITIVE ? "+": "-");
  }

  DISP_EDID_DATA( "\tEndMode\n" );

  /* set configuration value */
  if ((640 == H_ACTIVE) && (480 == V_ACTIVE))
  {
      resolution = DSOR_640_480;
  }
  else if ((720 == H_ACTIVE) && (480 == V_ACTIVE))
  {
      resolution = DSOR_720_480;
  }
  else if ((720 == H_ACTIVE) && (576 == V_ACTIVE))
  {
      resolution = DSOR_720_576;
  }
  else if ((800 == H_ACTIVE) && (600 == V_ACTIVE))
  {
      resolution = DSOR_800_600;
  }
  else if ((1024 == H_ACTIVE) && (768 == V_ACTIVE))
  {
      resolution = DSOR_1024_768;
  }
  else if ((1152 == H_ACTIVE) && (864 == V_ACTIVE))
  {
      resolution = DSOR_1152_864;
  }
  else if ((1280 == H_ACTIVE) && (720 == V_ACTIVE))
  {
      resolution = DSOR_1280_720;
  }
  else if ((1280 == H_ACTIVE) && (768 == V_ACTIVE))
  {
      resolution = DSOR_1280_768;
  }
  else if ((1280 == H_ACTIVE) && (960 == V_ACTIVE))
  {
      resolution = DSOR_1280_960;
  }
  else if ((1280 == H_ACTIVE) && (1024 == V_ACTIVE))
  {
      resolution = DSOR_1280_1024;
  }
  else if ((1400 == H_ACTIVE) && (1050 == V_ACTIVE))
  {
      resolution = DSOR_1400_1050;
  }
  else if ((1600 == H_ACTIVE) && (1200 == V_ACTIVE))
  {
      resolution = DSOR_1600_1200;
  }
  else if ((1920 == H_ACTIVE) && (1080 == V_ACTIVE))
  {
      resolution = DSOR_1920_1080;
  }
  else
  {
      resolution = DSOR_UNKNOWN;
  }

  /* frequency */
  if (0 == vfreq_remain)
  {
      if (25 == vfreq)
      {
        frequency = DSEF_25HZ;
      }
      else if (50 == vfreq)
      {
        frequency = DSEF_50HZ;
      }
      else if (60 == vfreq)
      {
        frequency = DSEF_60HZ;
      }
      else if (75 == vfreq)
      {
        frequency = DSEF_75HZ;
      }
      else if (30 == vfreq)
      {
        frequency = DSEF_30HZ;
      }
      else if (24 == vfreq)
      {
        frequency = DSEF_24HZ;
      }
      else
      {
        frequency = DSEF_UNKNOWN;
      }
  }
  else /* there is remainder */
  {
      if (23 == vfreq)
      {
        frequency = DSEF_23_976HZ;
      }
      else if (29 == vfreq)
      {
        frequency = DSEF_29_97HZ;
      }
      else if (59 == vfreq)
      {
        frequency = DSEF_59_94HZ;
      }
      else
      {
        frequency = DSEF_UNKNOWN;
      }
  }

//  if ((isResolutionNotinConfig(resolution, config) != 0) &&
//      (isFrequencyNotinConfig(frequency, config) != 0))
  if ((resolution != DSOR_UNKNOWN) &&
      (frequency != DSEF_UNKNOWN))

  {
    config->resolution[config->numberOfModes] = resolution;
    config->frequency[config->numberOfModes] = frequency;
    ++(config->numberOfModes);
  }

  return 0;
}

void parse_established_timings_1(byte timing, DFBScreenOutputEdidConfig *config)
{
    if (timing & 0x20)
    {
        config->resolution[config->numberOfModes] = DSOR_640_480;
        config->frequency[config->numberOfModes] = DSEF_60HZ;
        ++(config->numberOfModes);
    }
    else if (timing & 0x40)
    {
        config->resolution[config->numberOfModes] = DSOR_640_480;
        config->frequency[config->numberOfModes] = DSEF_75HZ;
        ++(config->numberOfModes);
    }
    else if (timing & 0x1)
    {
        config->resolution[config->numberOfModes] = DSOR_800_600;
        config->frequency[config->numberOfModes] = DSEF_60HZ;
        ++(config->numberOfModes);
    }
}

void parse_established_timings_2(byte timing, DFBScreenOutputEdidConfig *config)
{
    if (timing & 0x40)
    {
        config->resolution[config->numberOfModes] = DSOR_800_600;
        config->frequency[config->numberOfModes] = DSEF_75HZ;
        ++(config->numberOfModes);
    }
    else if (timing & 0x8)
    {
        config->resolution[config->numberOfModes] = DSOR_1024_768;
        config->frequency[config->numberOfModes] = DSEF_60HZ;
        ++(config->numberOfModes);
    }
    else if (timing & 0x2)
    {
        config->resolution[config->numberOfModes] = DSOR_1024_768;
        config->frequency[config->numberOfModes] = DSEF_75HZ;
        ++(config->numberOfModes);
    }
    else if (timing & 0x1)
    {
        config->resolution[config->numberOfModes] = DSOR_1280_1024;
        config->frequency[config->numberOfModes] = DSEF_75HZ;
        ++(config->numberOfModes);
    }
}

void parse_standard_timings(byte* edid, DFBScreenOutputEdidConfig *config)
{
unsigned i;
uint16_t HorizontalPixel = 0;
uint16_t VerticalPixel = 0;
unsigned char refreshRate;
DFBScreenEncoderFrequency frequency;
DFBScreenOutputResolution resolution;

	for ( i = STANDARD_TIMING_ID_START; i < STANDARD_TIMING_ID_END; i += 2)
	{
		if ((HorizontalPixel = (uint16_t) (edid[i] + 31) * 8) > 256)
		{

			/* Check bits 7 & 6 */
			switch ((edid[i + 1] & 192) >> 6)
			{
				case 0:
					VerticalPixel = HorizontalPixel * 10 / 16;
					break;

				case 1:
					VerticalPixel = HorizontalPixel * 3 / 4;
					break;

				case 2:
					VerticalPixel = HorizontalPixel * 4 / 5;
					break;

				case 3:
					VerticalPixel = HorizontalPixel * 9 / 16;
					break;

				default:
					MSG("Error parsing Standard Timing Identifications");
					break;
			}

			refreshRate = (unsigned char)((edid[i + 1] & 63) + 60);
			DISP_EDID_DATA("\t%u x %u\t@ %uHz\n", (unsigned int)HorizontalPixel, (unsigned int)VerticalPixel, 
					refreshRate);

		  /* set configuration value */
		  if ((640 == HorizontalPixel) && (480 == VerticalPixel))
		  {
		      resolution = DSOR_640_480;
		  }
		  else if ((720 == HorizontalPixel) && (480 == VerticalPixel))
		  {
		      resolution = DSOR_720_480;
		  }
		  else if ((720 == HorizontalPixel) && (576 == VerticalPixel))
		  {
		      resolution = DSOR_720_576;
		  }
		  else if ((800 == HorizontalPixel) && (600 == VerticalPixel))
		  {
		      resolution = DSOR_800_600;
		  }
		  else if ((1024 == HorizontalPixel) && (768 == VerticalPixel))
		  {
		      resolution = DSOR_1024_768;
		  }
		  else if ((1152 == HorizontalPixel) && (864 == VerticalPixel))
		  {
		      resolution = DSOR_1152_864;
		  }
		  else if ((1280 == HorizontalPixel) && (720 == VerticalPixel))
		  {
		      resolution = DSOR_1280_720;
		  }
		  else if ((1280 == HorizontalPixel) && (768 == VerticalPixel))
		  {
		      resolution = DSOR_1280_768;
		  }
		  else if ((1280 == HorizontalPixel) && (960 == VerticalPixel))
		  {
		      resolution = DSOR_1280_960;
		  }
		  else if ((1280 == HorizontalPixel) && (1024 == VerticalPixel))
		  {
		      resolution = DSOR_1280_1024;
		  }
		  else if ((1400 == HorizontalPixel) && (1050 == VerticalPixel))
		  {
		      resolution = DSOR_1400_1050;
		  }
		  else if ((1600 == HorizontalPixel) && (1200 == VerticalPixel))
		  {
		      resolution = DSOR_1600_1200;
		  }
		  else if ((1920 == HorizontalPixel) && (1080 == VerticalPixel))
		  {
		      resolution = DSOR_1920_1080;
		  }
		  else
		  {
		      resolution = DSOR_UNKNOWN;
		  }

		  // refresh rate
		  if (60 == refreshRate)
		  {
			frequency = DSEF_60HZ;
		  }
		  else if (75 == refreshRate)
		  {
			frequency = DSEF_75HZ;
		  }

		  if ((resolution != DSOR_UNKNOWN) &&
		      (frequency != DSEF_UNKNOWN))

		  {
		    config->resolution[config->numberOfModes] = resolution;
		    config->frequency[config->numberOfModes] = frequency;
		    ++(config->numberOfModes);
		  }

		}
	}
}

int
block_type( byte* block )
{
  if ( !strncmp( edid_v1_descriptor_flag, block, 2 ) )
    {
      DISP_EDID_DATA("\t# Block type: 2:%x 3:%x\n", block[2], block[3]);

      /* descriptor */

      if ( block[ 2 ] != 0 )
	return UNKNOWN_DESCRIPTOR;


      return block[ 3 ];
    } else {

      /* detailed timing block */

      return DETAILED_TIMING_BLOCK;
    }
}

char*
get_monitor_name( byte const* block )
{
  static char name[ 13 ];
  unsigned i;
  byte const* ptr = block + DESCRIPTOR_DATA;


  for( i = 0; i < 13; i++, ptr++ )
    {

      if ( *ptr == 0xa )
	{
	  name[ i ] = 0;
	  return name;
	}

      name[ i ] = *ptr;
    }


  return name;
}


char* get_vendor_sign( byte const* block )
{
  static char sign[4];
  unsigned short h;

  /*
     08h	WORD	big-endian manufacturer ID (see #00136)
		    bits 14-10: first letter (01h='A', 02h='B', etc.)
		    bits 9-5: second letter
		    bits 4-0: third letter
  */
  h = COMBINE_HI_8LO(block[0], block[1]);
  sign[0] = ((h>>10) & 0x1f) + 'A' - 1;
  sign[1] = ((h>>5) & 0x1f) + 'A' - 1;
  sign[2] = (h & 0x1f) + 'A' - 1;
  sign[3] = 0;
  return sign;
}

int
parse_monitor_limits( byte* block )
{
  DISP_EDID_DATA( "\tHorizSync %u-%u\n", H_MIN_RATE, H_MAX_RATE );
  DISP_EDID_DATA( "\tVertRefresh %u-%u\n", V_MIN_RATE, V_MAX_RATE );
  if ( MAX_PIXEL_CLOCK == 10*0xff )
    DISP_EDID_DATA( "\t# Max dot clock not given\n" );
  else
    DISP_EDID_DATA( "\t# Max dot clock (video bandwidth) %u MHz\n", (int)MAX_PIXEL_CLOCK );

  if ( GTF_SUPPORT )
    {
      DISP_EDID_DATA( "\t# EDID version 3 GTF given: contact author\n" );
    }
  
  return 0;
}

int
parse_dpms_capabilities(byte flags)
{
  DISP_EDID_DATA("\t# DPMS capabilities: Active off:%s  Suspend:%s  Standby:%s\n\n",
    (flags & DPMS_ACTIVE_OFF) ? "yes" : "no",
    (flags & DPMS_SUSPEND)    ? "yes" : "no",
    (flags & DPMS_STANDBY)    ? "yes" : "no");

  return 0;
}

/*
** return value <=0: error, otherwise pass
*/
int get_edid_data(pxc_dfb_driver_data mdrv, DFBScreenOutputEdidConfig *config)
{
    app_ioctl_command dfb_ioctl_cmd;
    u8 edid_buffer[128*3]; // could be 254 blocks
    int retValue = 1;

//    printf( "get_edid_data: enter \n");

    if (mdrv->fb0_handle != -1)
    {
        //
        // Inform FB driver to get edid data
        // 
        dfb_ioctl_cmd.uSize = sizeof(app_ioctl_command);
        dfb_ioctl_cmd.uIOCTLCommand = CMD_APP_IOCTL_GET_HDMI_EDID;
	dfb_ioctl_cmd.hdmi_edid.data = edid_buffer;
        
        if  (ioctl( mdrv->fb0_handle, CMD_APPLICATION_IOCTL , &dfb_ioctl_cmd ) < 0)
        {
             retValue = -1;
             D_ERROR("failed to inform FB driver of gettign edid data\n");
        }
        else
        {
            if (dfb_ioctl_cmd.hdmi_edid.size == 0)
            {
		printf("get_edid_data: no data\n");
                retValue = 0;
            }
            else
            {
                // parse data
//		printf("get_edid_data: parse data\n");
                parse_edid(edid_buffer, config);
            }
        }
    }

//    printf( "get_edid_data: exit \n");
}

static DFBResult 
Xcode4GetEdidOutputConfig ( CoreScreen                  *screen,
                    void                        *driver_data,
                    void                        *screen_data,
                    int                          output,
                    DFBScreenOutputEdidConfig *config )
{
    DFBResult result = DFB_UNSUPPORTED;

    D_DEBUG( "Xcode4GetEdidOutputConfig enter \n");

    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    if (get_edid_data(mdrv, config) > 0)
    {
        result = DFB_OK;
    }

    D_DEBUG( "Xcode4GetEdidOutputConfig exit \n");
    return result;
}


static DFBResult Xcode4SetBackgroundColor ( CoreScreen                  *screen,
                                            void                        *driver_data,
                                            void                        *screen_data,
									   u8                        red,
									   u8                        green,
                                       u8                        blue)
{
    u32 bg_color_y, bg_color_cb, bg_color_cr;
    u32 tmp;
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;

    D_DEBUG( "Xcode4SetBackgroundColor enter \n");

    //set screen background color
    tmp = REG_READ(DISP_HD_COMP_ALPHA_MODE);

    RGB_TO_YCBCR(red, 
                 green,
                 blue,
                 bg_color_y,
                 bg_color_cb,
                 bg_color_cr);


    bg_color_y = bg_color_y & 0x0ff;
    bg_color_cb = bg_color_cb & 0x0ff;
    bg_color_cr = bg_color_cr & 0x0ff;

	tmp = tmp & (~DISP_HD_COMP_ALPHA_MODE_BACKGROUND_Y_MASK);
	tmp = tmp & (~DISP_HD_COMP_ALPHA_MODE_BACKGROUND_CB_MASK);
	tmp = tmp & (~DISP_HD_COMP_ALPHA_MODE_BACKGROUND_CR_MASK);

    tmp = tmp | 
          (bg_color_y << DISP_HD_COMP_ALPHA_MODE_BACKGROUND_Y_SHIFT) |
          (bg_color_cb << DISP_HD_COMP_ALPHA_MODE_BACKGROUND_CB_SHIFT) |
          (bg_color_cr << DISP_HD_COMP_ALPHA_MODE_BACKGROUND_CR_SHIFT);

    REG_WRITE(DISP_HD_COMP_ALPHA_MODE, tmp);

    //make the sd has the same background as hd
    tmp = REG_READ(DISP_SD_COMP_ALPHA_MODE);

	tmp = tmp & (~DISP_SD_COMP_ALPHA_MODE_BACKGROUND_Y_MASK);
	tmp = tmp & (~DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CB_MASK);
	tmp = tmp & (~DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CR_MASK);

    tmp = tmp | 
          (bg_color_y << DISP_SD_COMP_ALPHA_MODE_BACKGROUND_Y_SHIFT) |
          (bg_color_cb << DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CB_SHIFT) |
          (bg_color_cr << DISP_SD_COMP_ALPHA_MODE_BACKGROUND_CR_SHIFT);

    REG_WRITE(DISP_SD_COMP_ALPHA_MODE, tmp);

    D_DEBUG( "Xcode4SetBackgroundColor exit \n");

    return DFB_OK;
}

static DFBResult Xcode4WaitVSync      ( CoreScreen             *screen,
                                   void                   *driver_data,
                                   void                   *screen_data )
{
    pxc_dfb_driver_data mdrv = (pxc_dfb_driver_data) driver_data;
    u32 base_frm_cnt;
    WAIT_FOP_NEXT_HD_FRAME;
    return DFB_OK;
}


const ScreenFuncs XcodeScreenFuncs = {
     InitScreen:    XcodeInitScreen,
     TestEncoderConfig:    Xcode4TestEncoderConfig,
     TestOutputConfig:    Xcode4TestOutputConfig,
     InitEncoder:   Xcode4InitEncoder,
     InitOutput:    Xcode4InitOutput,
     SetEncoderConfig:   Xcode4SetEncoderConfig,
     SetOutputConfig:    Xcode4SetOutputConfig,
     GetScreenSize: Xcode4GetScreenSize,
     SetPowerMode:  Xcode4SetPowerMode,
     GetEdidOutputConfig: Xcode4GetEdidOutputConfig,
     SetBackgroundColor: Xcode4SetBackgroundColor,
     WaitVSync: Xcode4WaitVSync
};

